/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef GFX_CTRL_H_
#define GFX_CTRL_H_


#include <nw/gfx.h>
#include "../include/SmBase.h"
#include "../include/SmModel.h"
#include "../include/SmCamera.h"
#include "../include/SmSceneCtrl.h"
#include "../include/SmParticle.h"


#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
//------------------------------------------------------------------------------
//
class GfxCtrl : public SmBase
{
public:
    //
    enum
    {
        GFX_CTRL_VRAM_A = NN_GX_MEM_VRAMA,
        GFX_CTRL_VRAM_B = NN_GX_MEM_VRAMB,
        GFX_CTRL_VRAM_NONE
    };
    typedef u32 GfxVramType;

    //
    struct GfxResourceSet
    {
        nw::ut::MoveArray<u8>       buffer;
        nw::gfx::ResGraphicsFile    resource;
    };
    #define GFX_RES_ARRAY_COUNT (32)

    //
    typedef nw::ut::FixedSizeArray<GfxResourceSet, GFX_RES_ARRAY_COUNT>         GfxResourceArray;

    //
    typedef nw::ut::FixedSizeArray<SmModel*, GFX_RES_ARRAY_COUNT>               SmModelInstanceArray;

    //
    typedef nw::ut::FixedSizeArray<nw::gfx::AnimObject*, GFX_RES_ARRAY_COUNT>   GfxAnimObjectArray;

public:
    //
    GfxCtrl( SmSceneCtrl* sceneCtrl );

    //
    ~GfxCtrl();

    //
    SmModel* CreateModel( SmModel* parentModel, const wchar_t* resourcePath, GfxVramType vramType );

    //
    SmParticle* CreateParticle( const wchar_t* resourcePath, GfxVramType vramType );

    //
    void CreateSceneEnvironment( const wchar_t* resourcePath );

    //
    void CreateCamera( const wchar_t* resourcePath );

    //
    void InitializeScene();

    //
    void TraverseScene();

    //
    void Update();


    //
    nw::gfx::SceneNode* GetSceneRoot() { return m_SceneRoot; }

    //
    nw::gfx::SceneEnvironmentSetting* GetSceneEnvSetting() { return m_SceneEnvSetting; }

    //
    nw::gfx::Camera* GetCamera()
    {
        return m_SmCamera->GetGxCamera();
    }

    //
    SmCamera* GetSmCamera()
    {
        return m_SmCamera;
    }

    //
    bool AddModel( SmModel* model );

    //
    nw::gfx::AnimEvaluator* GetLightAnimEvaluator()
    {
        return m_LightAnimation;
    }

    // Output SceneCtrl log.
    void OutputLog()
    {
        m_GfxSceneCtrl->OutputLog();
    }

private:
    SmSceneCtrl*                        m_GfxSceneCtrl;

    nw::gfx::SceneNode*                 m_SceneRoot;
    nw::gfx::Camera*                    m_Camera;
    nw::gfx::Camera*                    m_ResCamera;
    SmCamera*                           m_SmCamera;
    SmParticle*                         m_SmParticle;

    nw::gfx::ParticleContext*           m_ParticleContext;
    nw::gfx::ParticleSceneUpdater*      m_ParticleSceneUpdater;

    GfxResourceArray                    m_ResourcesArray;
    GfxAnimObjectArray                  m_AnimationArray;
    SmModelInstanceArray                m_SmModelInstArray;

    nw::gfx::SceneEnvironmentSetting*   m_SceneEnvSetting;

    nw::gfx::AnimEvaluator*             m_LightAnimation;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif


#endif  // GFX_CTRL_H_
