/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef LYT_CTRL_H_
#define LYT_CTRL_H_

#include <nw/lyt.h>
#include "../include/SmBase.h"
#include "../include/SmFile.h"
#include "../include/SmLayout.h"



#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
//------------------------------------------------------------------------------
//
typedef nw::ut::FixedSizeArray<std::string, 8> StringArray;
struct LytResourceFile
{
    std::string         clyt;       // Layout Files
    StringArray         clan;       // Animation files (up to 8)
};

typedef nw::ut::FixedSizeArray<std::string, 1> StringArray2;

struct testtest
{
    std::string         clyt;
    StringArray         clan;
};

//------------------------------------------------------------------------------
//
class LytCtrl : public SmBase
{
public:
    struct LytResourceSet
    {
        nw::lyt::ArcResourceAccessor*   resAccessor;
        SmFile                          lytFile;
    };
    #define LYT_RESOURCES_COUNT (32)
    typedef nw::ut::FixedSizeArray<LytResourceSet, LYT_RESOURCES_COUNT>     LytResourceArray;
    typedef nw::ut::FixedSizeArray<SmLayoutResource*, LYT_RESOURCES_COUNT>  SmLytResourceArray;
    typedef nw::ut::FixedSizeArray<SmLayout*, LYT_RESOURCES_COUNT>          SmLytInstanceArray;

public:
    LytCtrl();
    ~LytCtrl();

    //
    SmLayoutResource* CreateSmLayoutResource( const wchar_t* archivePath, LytResourceFile& lytResFile );

    //
    SmLayout* CreateSmLayout( SmLayoutResource* layoutResource );

    //
    void Render();

private:
    // Initialize m_DrawInfo.
    void setupCamera( nw::ut::Rect rect );

private:
    nw::lyt::Drawer                 m_Drawer;
    nw::lyt::GraphicsResource       m_GraphicsResource;
    nw::lyt::DrawInfo               m_DrawInfo;

    LytResourceArray                m_ResourcesArray;
    SmLytResourceArray              m_SmLytResourceArray;
    SmLytInstanceArray              m_SmLytInstanceArray;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif  // LYT_CTRL_H_
