/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef SM_COMMAND_REUSER_H_
#define SM_COMMAND_REUSER_H_

#include <nn/gx.h>

//---------------------------------------------------------------------------
// Class for creating and re-using commands.
class SmCommandReuser
{
public:
    SmCommandReuser()
    : m_StoreCommandListId(0),
      m_CommandListId(0),
      m_BufferOffset(0),
      m_BufferSize(0),
      m_RequestId(0),
      m_RequestSize(0)
    {}

    // Start creating cache.
    void StartCaching(GLuint cachingCommandListId)
    {
        // Store currently bound command list.
        nngxGetCmdlistParameteri( NN_GX_CMDLIST_BINDING, &m_StoreCommandListId );

        // Switch to cache command list.
        m_CommandListId = cachingCommandListId;
        nngxBindCmdlist(cachingCommandListId);
        nngxClearCmdlist();
        nngxStartCmdlistSave();
        NW_GL_ASSERT();
    }

    // Stop creating cache.
    void EndCaching()
    {
        nngxSplitDrawCmdlist();
        NW_GL_ASSERT();

        nngxStopCmdlistSave(
            &this->m_BufferOffset,
            &this->m_BufferSize,
            &this->m_RequestId,
            &this->m_RequestSize);
        NW_GL_ASSERT();

        nngxBindCmdlist( m_StoreCommandListId );
    }

    // Use cached commands.
    void UseCommand( bool isCopyBuffer )
    {
        GLboolean copyBuffer = isCopyBuffer ? GL_TRUE : GL_FALSE;
        nngxUseSavedCmdlist(
            this->m_CommandListId,
            this->m_BufferOffset,
            this->m_BufferSize,
            this->m_RequestId,
            this->m_RequestSize,
            0,
            copyBuffer);
        NW_GL_ASSERT();
    }

    // Is it cached?
    bool IsCach()
    {
        if( m_BufferSize > 0 )
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    GLsizei GetBufferSize() const
    {
        return m_BufferSize;
    }

private:
    GLint       m_StoreCommandListId;
    GLuint      m_CommandListId;
    GLuint      m_BufferOffset;
    GLsizei     m_BufferSize;
    GLuint      m_RequestId;
    GLsizei     m_RequestSize;
};



//---------------------------------------------------------------------------
// Allocate static command cache and register commands.
class SmStaticCommandCach
{
public:

    // Constructor
    SmStaticCommandCach()
    {
        nngxGenCmdlists(1, &(m_CachingCommandList));
        nngxBindCmdlist(m_CachingCommandList);
        nngxCmdlistStorage(description.cachingCommandBufferSize, description.cachingRequestCount);
        nngxSetCmdlistParameteri(NN_GX_CMDLIST_RUN_MODE, NN_GX_CMDLIST_SERIAL_RUN);
    }


};

#endif // SM_COMMAND_REUSER_H_
