/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_FILE_H_
#define SM_FILE_H_

#include <nn/fs.h>
#include "../include/SmBase.h"


#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
//------------------------------------------------------------------------------
// Class for loading files.
class SmFile
{
public:
    // Constructor
    SmFile( bool autoFree = false )
    : m_Buffer(NULL),
      m_Size(0),
      m_AutoFree( autoFree ),
      m_Allocator( NULL ){}

    // Destructor
    ~SmFile()
    {
         if ( m_AutoFree )
        {
            Release();
        }
    }

    // Load specified file.
    bool Read( const wchar_t* fileName, nw::os::IAllocator* allocator )
    {
        NW_ASSERT(m_Buffer == NULL);
        NW_NULL_ASSERT( allocator );

        m_Allocator = allocator;

        nn::fs::FileReader fileReader(fileName);
        if (fileReader.GetSize() <= 0)
        {
            return false;
        }

        this->m_Size = static_cast<u32>(fileReader.GetSize());
        this->m_Buffer = static_cast<u8*>(m_Allocator->Alloc(this->m_Size, 128));
        if (this->m_Buffer == NULL)
        {
            this->m_Size = 0;
            return false;
        }

        fileReader.Read(this->m_Buffer, this->m_Size);
        fileReader.Finalize();

        return true;
    }

    // Get buffer containing loaded file.
    u8* Buffer()
    {
        return this->m_Buffer;
    }

    // Get the size of the loaded file.
    u32 Size()
    {
        return this->m_Size;
    }

    // Free allocated memory.
    void Release()
    {
         if ( m_Buffer )
        {
            m_Allocator->Free( m_Buffer );
            m_Buffer = NULL;
        }
    }

private:
    u8*                     m_Buffer;
    u32                     m_Size;
    bool                    m_AutoFree;
    nw::os::IAllocator*     m_Allocator;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif  // SM_FILE_H_
