/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_LYT_SLIDERBAR_H_
#define SM_LYT_SLIDERBAR_H_

#include "../include/SmLayout.h"
#include "../include/SmMessage.h"
#include "../include/SmRectCollision.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
//------------------------------------------------------------------------------
// Class for displaying a layout slider bar.
class SmLytSliderBar : public SmMessage
{
public:
    // Constructor
    SmLytSliderBar( f32 x,
                    f32 y,
                    SmLayout* smLayout);

    // Destructor
    ~SmLytSliderBar();

    // Receive message
    virtual bool ReceveMessage( SmMessageType type, void* object, uint targetId );


    // Toggle visibility.
    virtual void SetVisible( bool visible )
    {
        // Toggle layout visibility.
        m_Layout->SetVisible( visible );
    }

    // Rendering
    virtual void Render();

    // Sets parameter values.
    void SetParam( f32 param );

    // Get parameter values.
    f32 GetParam() const { return m_Param; }

private:
    SmLayout*           m_Layout;
    nw::lyt::Pane*      m_HandlePane;
    SmRectCollision     m_Collision;
    f32                 m_Param;
    bool                m_IsGrab;
    s32                 m_RangeMinY;
    s32                 m_RangeMaxY;
    f32                 m_BaseY;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif  // SM_LYT_SLIDERBAR_H_
