/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_MODEL_H_
#define SM_MODEL_H_

#include <nw/gfx.h>
#include "../include/SmBase.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
//------------------------------------------------------------------------------
// Class for accessing various NW4C model features.
class SmModel : public SmBase
{
    friend class GfxCtrl;

    // Animation array.
    #define ANIM_COUNT (32)
    typedef nw::ut::FixedSizeArray<nw::gfx::TransformAnimEvaluator*, ANIM_COUNT> AnimEvaluatorArray;


private:
    // Constructor
    SmModel();

    // Constructor
    ~SmModel();

public:
    // Set the node.
    void SetSceneNode( nw::gfx::SceneNode* pParentNode, nw::gfx::SceneNode* pSceneNode );

    // Get the node.
    nw::gfx::SceneNode* GetSceneNode() const { return m_pSceneNode; }

    // Set the model.
    void SetModel( nw::gfx::Model* pModel, nw::gfx::SkeletalModel* pSkelModel );

    // Update
    void Update();

    /*
        Positioning features
    */
    // position
    void SetPosition( f32 x, f32 y, f32 z );

    // Enlarge, reduce
    void SetScale( f32 x, f32 y, f32 z );

    // Rotation
    void SetRotate( f32 x, f32 y, f32 z );

    // Matrix
    void SetMatrix( const nw::math::MTX34* matrix );
    void SetWorldMatrix( const nw::math::MTX34* matrix );


    // Control visibility.
    void SetVisible( bool visible );
    void SwapVisible();
    bool IsVisible();

    // Copy.
    SmModel* Duplicate();

    /*
        Animation features.
    */
    // Add animations.
    void AddAnimEvaluator( nw::gfx::TransformAnimEvaluator* animEvaluator );

    // Get the number of animations.
    uint GetAnimEvaluatorNum() const;

    // Configure animations.
    bool SetAnimEvaluatorNo( uint animEvaluatorNo );

    // Configure the animation frame.
    void SetAnimationFrame( f32 setFrame );

    // Advance the animation frame.
    bool AddAnimationFrame( f32 addFrame, bool loop = true );

    // Get the animation frame.
    f32 GetAnimationFrame() const;

    // Get the largest frame of the animation being configured.
    f32 GetAnimationMaxFrame() const;


    /*
        Material animation features.
    */
    // Configure material animation.
    bool SetMaterialAnim( nw::gfx::AnimEvaluator* animEvaluator );


    /*
        Feature for resolving parent-child inheritance.
    */
    void SetConstrainId( s32 nodeId );

    /*
        Testing features.
    */
    void SetVisibleZone( f32 term );
    void SetLayerId( u8 id )
    {
         if ( m_SkeletalModel )
        {
            m_SkeletalModel->SetLayerId( id );
        }
    }
    u8 GetLayerId() const
    {
         if ( m_SkeletalModel )
        {
            return m_SkeletalModel->GetLayerId();
        }

        return 0;
    }

private:
    // Resolve parent-child inheritance.
    void updateConstrain();

private:
    nw::gfx::SceneNode*         m_pParentNode;
    nw::gfx::SceneNode*         m_pSceneNode;

    nw::gfx::Model*             m_Model;
    nw::gfx::SkeletalModel*     m_SkeletalModel;

    AnimEvaluatorArray          m_AnimEvaluatorArray;
    f32                         m_AnimationFrame;
    s32                         m_CurrentAnimEvNo;

    s32                         m_CostrainId;

    bool                        m_Duplicated;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif  // SM_MODEL_H_
