/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_PARTICLE_H_
#define SM_PARTICLE_H_

#include <nw/gfx.h>
#include "../include/SmBase.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
//------------------------------------------------------------------------------
// Class for accessing various NW4C particle features.
class SmParticle : public SmBase
{
    friend class GfxCtrl;

    #define ARRAY_COUNT (32)

    //
    typedef nw::ut::FixedSizeArray<nw::gfx::ParticleModel*, ARRAY_COUNT> ParticleModelInstanceArray;

    //
    typedef nw::ut::FixedSizeArray<nw::gfx::ParticleEmitter*, ARRAY_COUNT> ParticleEmitterInstanceArray;

    //
    typedef nw::ut::FixedSizeArray<bool, ARRAY_COUNT> ParticleInstAttachedArray;

private:
    // Constructor
    SmParticle( nw::gfx::SceneNode* parentNode );

    // Destructor
    ~SmParticle();

public:
    // Add particle model.
    void AddParticleModel( nw::gfx::ParticleModel* particleModel );

    // Add particle emitter.
    void AddParticleEmitter( nw::gfx::ParticleEmitter* particleEmitter );

    // Configure the animation frame.
    void SetAnimationFrame( f32 setFrame );

    // Advance the animation frame.
    bool AddAnimationFrame( f32 addFrame, bool loop = true );

    // Resets
    void Reset();

    // Control visibility.
    void SetVisible( bool visible )
    {
        /*
            At present, this code only works if the number of particle models and emitters are the same.
        */
        if( m_ParticleModelInstArray.size() != m_ParticleEmitterInstArray.size() )
        {
            return;
        }

        for ( int i=0; i<m_ParticleModelInstArray.size(); i++ )
        {
            if ( visible )
            {
                // If attaching, look at the flag and restore.
                if( m_ParticleInstAttachedArray[i] )
                {
                    attach( i );
                }
            }
            else
            {
                // If detaching, detach all.
                detach( i );
            }
        }
    }
    void SwapVisible(){}
    bool IsVisible()
    {
        return true;
    }


    /*
        Layer operations.
    */
    void SetLayerId( u8 id )
    {
        for ( int i=0; i<m_ParticleModelInstArray.size(); i++ )
        {
            m_ParticleModelInstArray[i]->SetLayerId( id );
        }
    }


    /*
        Emitter operations.
    */
    nw::gfx::ParticleEmitter* GetEmitter( uint idx = 0 )
    {
        return m_ParticleEmitterInstArray[idx];
    }

    void ResetEmitter( int idx = -1 )
    {
        if ( idx == -1 )
        {
            for ( int i=0; i<m_ParticleEmitterInstArray.size(); i++ )
            {
                m_ParticleEmitterInstArray[i]->Reset();
            }
        }
        else
        {
            if ( idx < m_ParticleEmitterInstArray.size() )
            {
                m_ParticleEmitterInstArray[idx]->Reset();
            }
        }
    }

    //
    void SetVisibleZone( f32 term );


private:
    void detach( int index )
    {
        if ( m_ParticleModelInstArray[index]->GetParent() )
        {
            m_pParentNode->DetachChild( m_ParticleModelInstArray[index] );
        }
        if ( m_ParticleEmitterInstArray[index]->GetParent() )
        {
            m_pParentNode->DetachChild( m_ParticleEmitterInstArray[index] );
        }
    }

    void attach( int index )
    {
        if ( !m_ParticleModelInstArray[index]->GetParent() )
        {
            m_pParentNode->AttachChild( m_ParticleModelInstArray[index] );
        }
        if ( !m_ParticleEmitterInstArray[index]->GetParent() )
        {
            m_pParentNode->AttachChild( m_ParticleEmitterInstArray[index] );
        }
    }

private:
    nw::gfx::SceneNode*                 m_pParentNode;
    ParticleModelInstanceArray          m_ParticleModelInstArray;
    ParticleEmitterInstanceArray        m_ParticleEmitterInstArray;

    bool                                m_ParticleInstAttachedArray[ARRAY_COUNT];

    f32                                 m_AnimationFrame;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif  // SM_PARTICLE_H_
