/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_PRIMITIVE_H_
#define SM_PRIMITIVE_H_

#include "../include/SmBase.h"
#include "../include/SmFile.h"
#include "../include/SmCommandUtility.h"



#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
//------------------------------------------------------------------------------
// 1 vertex information
struct SmVertexPC
{
    nw::math::VEC2      position;
    nw::ut::Color8      color;
};


//------------------------------------------------------------------------------
// Manage state and shaders.
class SmOgl : public SmBase
{
public:
    // todo: Borrow from demo_GraphicsDrawing
    enum
    {
        SMP_ATTR_POSITION  = 0,
        SMP_ATTR_COLOR,
        SMP_ATTR_OFFSET,
        SMP_ATTR_TEXCOORD0
    };

private:
    // Constructor
    SmOgl( const wchar_t* shaderName );

    // Destructor
    virtual ~SmOgl();

public:

    // Initialize SmOgl.
    static SmOgl* InitializeSmOgl( const wchar_t* shaderName );

    // End SmOgl.
    static void TerminateSmOgl();

    // Start 2D entry.
    void SetBegin2D();

    void SetupMaterial();
    void SetupTexEnv();

private:
    GLuint              m_ProgramID;
    GLuint              m_ShaderID;
    SmFile              m_ShaderFile;

    nw::math::MTX44     m_ProjectionMatrix;
    nw::math::MTX44     m_ViewMatrix;

    SmCommandReuser     m_SetBeginCmd;
    SmStaticCommandCache::CopiedCmdCache  m_CopiedCmdCache;
};


//------------------------------------------------------------------------------
// Class for drawing 2D polygons.
class Sm2DPrimPC : public SmBase
{
    // todo: Borrow from demo_GraphicsDrawing
    enum
    {
        VTXATTR_POSITION  = 0,
        VTXATTR_COLOR,
        VTXATTR_TEXCOORD0
    };

public:
    // Constructor
    Sm2DPrimPC( uint vertexCnt );

    // Destructor
    ~Sm2DPrimPC();

    // Update using currently accumulated vertices.
    void Update();

    // Render
    void Render( GLenum entryMode, f32 posX = 0.f, f32 posY = 0.f );

    // Register vertex.
    void SetVertex( uint vertexIdx, f32 x, f32 y, nw::ut::Color8 color );

    // Register vertex color.
    void SetVertexColor( nw::ut::Color8 color );

private:
    void RenderSub( GLenum entryMode );


private:
    SmVertexPC*             m_pVertex;
    uint                    m_VertexCnt;

    bool                    m_VertexUpdate;
    SmCommandReuser         m_CmdCache;

    SmStaticCommandCache::CopiedCmdCache  m_CopiedCmdCache;
};


//------------------------------------------------------------------------------
// Class for drawing static 2D polygons.
class Sm2DDynamicPrimPC : public SmBase
{
    // todo: Borrow from demo_GraphicsDrawing
    enum
    {
        VTXATTR_POSITION  = 0,
        VTXATTR_COLOR,
        VTXATTR_TEXCOORD0
    };

public:
    // Constructor
    Sm2DDynamicPrimPC();

    // Destructor
    ~Sm2DDynamicPrimPC(){}

    // Start entry.
    bool Begin( uint                vertexCnt,      // Number of vertices
                GLenum              entryMode,      // Entry mode.
                f32                 posX = 0.f,
                f32                 posY = 0.f );

    // Register vertex.
    void SetVertex( f32 x, f32 y, nw::ut::Color8 color );

    // End entry.
    void End();

private:
    Sm2DPrimPC*             m_pPrimitive;
    uint                    m_VertexCnt;
    uint                    m_EntryCnt;
    GLenum                  m_EntryMode;
    f32                     m_OffsetX;
    f32                     m_OffsetY;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif  // SM_PRIMITIVE_H_
