/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef SM_SCENE_CTRL_H_
#define SM_SCENE_CTRL_H_


#include <nw/gfx.h>
#include "../include/SmBase.h"

//------------------------------------------------------------------------------
// Class containing various scene features.
class SmSceneCtrl : public SmBase
{
public:
    //----------------------------------------
    // Constructor
    SmSceneCtrl( nw::gfx::RenderContext* renderContext );

    //----------------------------------------
    // Destructor
    ~SmSceneCtrl();

    //----------------------------------------
    // Initializes the scene.
    void InitializeScene(nw::gfx::SceneNode* sceneRoot);

    //----------------------------------------
    void TraverseScene(nw::gfx::SceneNode* sceneRoot);

    //----------------------------------------
    void SetSceneEnvironmentSetting( nw::gfx::SceneEnvironmentSetting* sceneEnvironmentSetting );

    //----------------------------------------
    // Updates the scene.
    void UpdateScene();

    //----------------------------------------
    // Updates a scene based on the camera's visual range, and builds the render queue.
    void SubmitView( nw::gfx::Camera* currentCamera );
#if 0
    //----------------------------------------
    // Set the drawing environment.
    void SetEnvironment( s32 cameraIndex );
#endif
    //----------------------------------------
    // Performs rendering.
    void Render();

    //----------------------------------------
    // Gets a scene context.
    nw::gfx::SceneContext*   GetSceneContext() { return m_SceneContext; }

    //----------------------------------------
    // Output log.
    void OutputLog();

private:
    nw::gfx::SceneContext*              m_SceneContext;
    nw::gfx::SceneTraverser*            m_SceneTraverser;
    nw::gfx::SceneUpdater*              m_SceneUpdater;
    nw::gfx::SceneInitializer*          m_SceneInitializer;

    nw::gfx::RenderContext*             m_RenderContext;
    nw::gfx::RenderKeyFactory*          m_PriorMaterialRenderKeyFactory;
    nw::gfx::RenderKeyFactory*          m_PriorDepthRenderKeyFactory;
    nw::gfx::RenderQueue*               m_RenderQueue;
    nw::gfx::MeshRenderer*              m_MeshRenderer;
};


#endif // SM_SCENE_CTRL_H_
