/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "../include/SmParticle.h"

namespace
{

//----------------------------------------
// Constructor
SmParticle::SmParticle( nw::gfx::SceneNode* parentNode )
{
    m_pParentNode = parentNode;
    m_ParticleModelInstArray.clear();
    m_ParticleEmitterInstArray.clear();

    for ( int i=0; i<ARRAY_COUNT; i++ )
    {
        m_ParticleInstAttachedArray[i] = true;
    }
}


//----------------------------------------
// Destructor
SmParticle::~SmParticle()
{

}


//----------------------------------------
// Add particle model.
void SmParticle::AddParticleModel( nw::gfx::ParticleModel* particleModel )
{
    bool isPushed = m_ParticleModelInstArray.push_back( particleModel );
    NW_ASSERT(isPushed);
}


//----------------------------------------
// Add particle emitter.
void SmParticle::AddParticleEmitter( nw::gfx::ParticleEmitter* particleEmitter )
{
    bool isPushed = m_ParticleEmitterInstArray.push_back( particleEmitter );
    NW_ASSERT(isPushed);
}


//----------------------------------------
// Resets
void SmParticle::Reset()
{
    for ( int i=0; i<m_ParticleModelInstArray.size(); i++ )
    {
        for (u32 j = 0; j < m_ParticleModelInstArray[i]->GetParticleSetsCount(); ++j)
        {
            m_ParticleModelInstArray[i]->GetParticleSets(j)->GetParticleCollection()->KillParticles();
        }
        m_ParticleModelInstArray[i]->ParticleAnimFrameController().SetFrame(0);
    }

    for ( int i=0; i<m_ParticleEmitterInstArray.size(); i++ )
    {
        m_ParticleEmitterInstArray[i]->Reset();
    }
}


//----------------------------------------
// Configure the animation frame.
void SmParticle::SetAnimationFrame( f32 setFrame )
{
    f32 endFrame = 0.0f;
    for ( int i=0; i<m_ParticleModelInstArray.size(); i++ )
    {
        if (endFrame < m_ParticleModelInstArray[i]->ParticleAnimFrameController().GetEndFrame())
        {
            endFrame = m_ParticleModelInstArray[i]->ParticleAnimFrameController().GetEndFrame();
        }
    }

    if (setFrame > endFrame)
    {
        setFrame = endFrame;
    }

    // Do not update animation if there is no frame update.
    if (m_AnimationFrame != setFrame) return;

    // Set frame count.
    for (int i=0; i<m_ParticleModelInstArray.size(); i++)
    {
        m_ParticleModelInstArray[i]->ParticleAnimFrameController().SetFrame( setFrame );
    }

    m_AnimationFrame = setFrame;
}


//----------------------------------------
// Adjust the display level of particles from 0 to 1.
void
SmParticle::SetVisibleZone( f32 term )
{
    /*
        At present, this code only works if the number of particle models and emitters are the same.
    */
    if( m_ParticleModelInstArray.size() != m_ParticleEmitterInstArray.size() )
    {
        return;
    }

    s32 visibleNum = m_ParticleModelInstArray.size() * term;

    for ( int i = 0; i < m_ParticleModelInstArray.size(); i++ )
    {
        if ( i < visibleNum )
        {
            attach( i );
            m_ParticleInstAttachedArray[i] = true;
        }
        else
        {
            detach( i );
            m_ParticleInstAttachedArray[i] = false;
        }
    }
}


#if 0
//----------------------------------------
// Advance the animation frame.
bool SmParticle::AddAnimationFrame( f32 addFrame, bool loop = true )
{
    NW_UNUSED_VARIABLE(addFrame);
    NW_UNUSED_VARIABLE(loop);

    return false;
}
#endif

} // Namespace


