/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_CAMERA_H_
#define NW_DEMO_CAMERA_H_

#include <nw/gfx.h>

#include <nw/ut/ut_MoveArray.h>
#include <nw/ut/ut_Preprocessor.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
// Implemented while referencing the NW4R demo camera (camera.cpp/h from within demos\g3d\demolib).

namespace nw {
namespace os
{
class IAllocator;
} // namespace os

namespace demo {

// 
//         The analog stick alone controls rotation, whereas the analog stick plus the A Button controls forward and backward movement, and plus the B Button controls up/down right/left movement.
// 
//         Pressing the X Button restores the camera to its initial position.

class CameraController
{
private:
    NW_DISALLOW_COPY_AND_ASSIGN(CameraController);

public:
    //
    struct Description
    {
        size_t maxCameraCount;  //

        //
        Description()
        : maxCameraCount(MAX_CAMERA_COUNT)
        {}
    };

    //
    class Builder
    {
    public:
        //
        Builder& CameraDescription(const Description& description)
        {
            m_Description = description;
            return *this;
        }

        //
        Builder& MaxCameraCount(size_t count)
        {
            m_Description.maxCameraCount = count;
            return *this;
        }

        //---------------------------------------------------------------------------
        //
        //
        //
        //
        //
        //---------------------------------------------------------------------------
        CameraController* Create(os::IAllocator* allocator);

    private:
        Description m_Description;
    };

    //
    void Destroy();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void Register(nw::gfx::Camera* camera, unsigned int cameraIndex = 0);

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void Update(unsigned int cameraIndex = 0);

private:
    //
    static const size_t MAX_CAMERA_COUNT = 10;

    //
    CameraController(
        os::IAllocator* allocator,
        const Description& description);

    ~CameraController();

    //
    class CameraEntry
    {
    public:
        CameraEntry(nw::gfx::Camera* camera);

        ~CameraEntry() {};

        //
        void Update();

    private:
        //
        void UpdateCamera();

        nw::math::VEC3    m_Rotate;
        nw::math::VEC3    m_TargetPos;
        f32               m_TargetDistance;
        nw::gfx::Camera*  m_Camera;

        nw::math::VEC3    m_InitialRotate;
        nw::math::VEC3    m_InitialTargetPos;
        f32               m_InitialTargetDistance;

        //
        static const f32 DOLLY_SPEED_BASE;
        //
        static const f32 TUMBLE_SPEED;
        //
        static const f32 MOVE_SPEED_BASE;
        //
        static const f32 ROTATE_X_LIMIT;

        //
        //
        static const f32 DEFAULT_DISTANCE;
    };

    nw::os::IAllocator* m_Allocator;
    Description m_Description;

    nw::ut::MoveArray<CameraEntry*> m_CameraEntries;

    bool m_UsagePrinted;
};

} // namespace nw::demo
} // namespace nw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif

#endif /* NW_DEMO_CAMERA_H_ */

