/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_GRAPHICSDRAWING_H_
#define NW_DEMO_GRAPHICSDRAWING_H_

#include <GLES2/gl2.h>

#include <nw/types.h>
#include <nw/ut/ut_Color.h>
#include <nw/math/math_Types.h>
#include <nw/font/font_TextWriter.h>
#include <nw/font/font_ResFont.h>
#include <nw/font/font_RectDrawer.h>

#include <nn/gr.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw {

namespace os {
class IAllocator;
}

namespace font {
class Font;
}

namespace demo {

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class GraphicsDrawing
{
public:
    //----------------------------------------
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetScreenSize( s32 width, s32 height )
    {
        m_ScreenSize = math::VEC2( static_cast<f32>(width), static_cast<f32>(height) );
    }

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    GraphicsDrawing();

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    ~GraphicsDrawing();

    //

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void Finalize();

    //========================================================================================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool InitializeShape( os::IAllocator* allocator, const wchar_t* shaderPath );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool InitializeShape( void* pShaderBinary, size_t binarySize );

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void DestroyShape();

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void SetupShape();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetMaxShapeVertexCount( s32 maxShapeVertexCount )
    {
        m_MaxShapeVertexCount = maxShapeVertexCount;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetLineWidth( f32 width ) { m_LineWidth = width; }

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void BeginDrawingShape();

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void EndDrawingShape();

    // Shape Drawing

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawLine( s32 x1, s32 y1, s32 x2, s32 y2, ut::Color8 color )
        {
            this->DrawLine(
                math::VEC2(static_cast<f32>(x1), static_cast<f32>(y1)),
                math::VEC2(static_cast<f32>(x2), static_cast<f32>(y2)),
                color
            );
        }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawLine( const math::VEC2& p1, const math::VEC2& p2, ut::Color8 color );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawRectangle( s32 posh, s32 posv, s32 sizeh, s32 sizev, ut::Color8 color )
        {
            this->DrawRectangle(
                math::VEC2(static_cast<f32>(posh), static_cast<f32>(posv)),
                math::VEC2(static_cast<f32>(sizeh), static_cast<f32>(sizev)),
                color
            );
        }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawRectangle( const math::VEC2& pos, const math::VEC2& size, ut::Color8 color );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void FillRectangle( s32 posh, s32 posv, s32 sizeh, s32 sizev, ut::Color8 color )
        {
            this->FillRectangle(
                math::VEC2(static_cast<f32>(posh), static_cast<f32>(posv)),
                math::VEC2(static_cast<f32>(sizeh), static_cast<f32>(sizev)),
                color
            );
        }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void FillRectangle( const math::VEC2& pos, const math::VEC2& size, ut::Color8 color );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void FillTriangle( const math::VEC2& pt1, const math::VEC2& pt2, const math::VEC2& pt3, ut::Color8 color );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void FillTriangles( const math::VEC2* pPointArray, s32 pointCount, ut::Color8 color );

    //

    //========================================================================================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetMaxTextCount( s32 maxTextCount )
    {
        m_MaxTextCount = maxTextCount;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool InitializeFont( os::IAllocator* allocator, const wchar_t* fontShaderPath, const wchar_t* fontPath );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool InitializeFont( os::IAllocator* allocator, const wchar_t* fontShaderPath, void* fontBinary, size_t fontSize );

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void DestroyFont();

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void BeginDrawingString();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    f32 DrawString( const char* format, ... );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    f32 DrawString( s32 posh, s32 posv, const char* format, ... );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    f32 DrawStringArgs( s32 posh, s32 posv, const char* format, std::va_list args );


    //
    font::TextWriter& GetWriter() { return m_TextWriter; }
    const font::TextWriter& GetWriter() const { return m_TextWriter; }

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void EndDrawingString();

    //

private:
    //
    void InitializeShapeGraphicsState();

    //
    void SetupShapeMaterial();

    //
    void SetupShapeTexEnv();

    //
    void SendShapeMatrix();

    //
    //
    //
    //
    //
    //
    //
    //
    s32 BuildTriangle( const math::VEC2& pt1, const math::VEC2& pt2, const math::VEC2& pt3, math::VEC2 *pVecArray );

    //
    //
    //
    //
    //
    //
    //
    s32 BuildRectangle( const math::VEC2& pos, const math::VEC2& size, math::VEC2 *pVecArray );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    s32 BuildRectangle(
        const math::VEC2& pt1,
        const math::VEC2& pt2,
        const math::VEC2& pt3,
        const math::VEC2& pt4,
        math::VEC2 *pVecArray );

    //
    //
    //
    //
    //
    //
    //
    //
    s32 BuildLine( const math::VEC2& pt1, const math::VEC2& pt2, f32 lineWidth, math::VEC2 *pVecArray );

    //
    //
    //
    //
    //
    void BuildColor( const s32 vertexCount, const ut::Color8 color,
                     f32* pColorArray );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool InitializeFontFromBinary(
        void* shaderBinary,
        size_t shaderSize,
        void* fontBinary,
        size_t fontSize,
        void* drawBuffer,
        void* stringBuffer
    );

    //
    //
    //
    //
    //
    //
    bool InitializeFontResource( void* fontData, size_t fontSize );

    //
    //
    //
    //
    //
    //
    bool InitializeFontShader( void* shaderData, size_t shaderSize );

    //
    void SendFontMatrix();

    static const s32               DEFAULT_SCREEN_WIDTH           = 320;
    static const s32               DEFAULT_SCREEN_HEIGHT          = 240;
    static const s32               DEFAULT_MAX_TEXT_COUNT         = 512;
    static const s32               DEFAULT_MAX_SHAPE_VERTEX_COUNT = 128;

    static const s32               SHAPE_DRAW_LINE_VERTEX_COUNT        = 6;
    static const s32               SHAPE_DRAW_RECTANGLE_VERTEX_COUNT   = 4 * SHAPE_DRAW_LINE_VERTEX_COUNT;
    static const s32               SHAPE_FILL_TRIANGLE_VERTEX_COUNT    = 3;
    static const s32               SHAPE_FILL_RECTANGLE_VERTEX_COUNT   = 2 * SHAPE_FILL_TRIANGLE_VERTEX_COUNT;

    static const f32               FONT_SIZE;
    static const f32               FONT_FIXED_WIDTH;

    math::VEC2                     m_ScreenSize;

    // Shape rendering member variables.
    f32                            m_LineWidth;

    s32                            m_MaxShapeVertexCount;
    s32                            m_ShapeVertexCount;

    void*                          m_pShapeShaderBinary;
    nn::gr::Shader                 m_ShapeShader;
    nn::gr::BindSymbolVSFloat      m_ShapeBindSymbolProjectionMatrix;
    nn::gr::BindSymbolVSFloat      m_ShapeBindSymbolModelViewMatrix;

    nn::gr::CTR::BindSymbolVSInput m_ShapeBindSymbolPos;
    nn::gr::CTR::BindSymbolVSInput m_ShapeBindSymbolColor;

    nn::gr::Viewport               m_ShapeViewport;
    nn::gr::RenderState            m_ShapeRenderState;
    nn::gr::Combiner               m_ShapeCombiner;
    nn::gr::Vertex                 m_ShapeVertex;
    nn::gr::Vertex::IndexStream    m_ShapeIndexStream;

    math::VEC2*                    m_ShapePositionStreamArray;
    f32*                           m_ShapeColorStreamArray;
    u16*                           m_ShapeIndexStreamArray;

    nw::os::IAllocator*            m_ShapeAllocator;

    // Members for fonts.
    nw::font::ResFont              m_Font;
    nw::font::TextWriter           m_TextWriter;
    nw::font::RectDrawer           m_Drawer;
    s32                            m_MaxTextCount;
    void*                          m_FontCommandBuffer;
    bool                           m_IsFontOwner;

    nw::os::IAllocator*            m_FontAllocator;
};

} /* namespace demo */
} /* namespace nw */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

/* NW_DEMO_GRAPHICSDRAWING_H_ */
#endif
