/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_PARTICLE_H_
#define NW_DEMO_PARTICLE_H_

#include <nw/gfx.h>
#include <nw/ut.h>
#include <nw/demo/demo_GraphicsSystem.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw
{

namespace demo
{

//
//
class ParticleHandle : public gfx::TransformNode
{
private:
    NW_DISALLOW_COPY_AND_ASSIGN(ParticleHandle);

public:
    NW_UT_RUNTIME_TYPEINFO;

    //----------------------------------------
    //
    //

    //
    //
    //
    class DynamicBuilder
    {
    public:
        //
        DynamicBuilder() {}
        //
        ~DynamicBuilder() {}

        //
        //
        //
        //
        //
        DynamicBuilder& IsFixedSizeMemory(bool isFixedSizeMemory)
        {
            m_Description.isFixedSizeMemory = isFixedSizeMemory;
            return *this;
        }

        //
        DynamicBuilder& MaxChildren(int maxChildren)
        {
            m_Description.maxChildren = maxChildren;
            return *this;
        }

        //
        DynamicBuilder& MaxCallbacks(int maxCallbacks)
        {
            m_Description.maxCallbacks = maxCallbacks;
            return *this;
        }

        //
        //
        //
        //
        //
        //
        ParticleHandle* Create(os::IAllocator* allocator);

    private:
        gfx::TransformNode::Description m_Description;
    };

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    int GetID() const
    {
        return m_Id;
    }

    //
    //
    //
    void SetID(int id)
    {
        m_Id = id;
    }

    //
    //
    //
    //
    //
    void SetTranslate(f32 x, f32 y, f32 z)
    {
        this->Transform().SetTranslate(x, y, z);
    }

    //
    //
    //
    void SetTranslate(const nw::math::VEC3& translate)
    {
        this->Transform().SetTranslate(translate);
    }

    //
    void GetTranslate(nw::math::VEC3* translate) const
    {
        this->Transform().GetTranslate(translate);
    }

    //
    //
    //
    //
    //
    void SetRotate(f32 radX, f32 radY, f32 radZ)
    {
        this->Transform().SetRotateXYZ(radX, radY, radZ );
    }

    //
    //
    //
    //
    //
    void SetScale(f32 x, f32 y, f32 z)
    {
        this->Transform().SetScale(x, y, z);
    }

    //
    //
    //
    void SetStepFrame(f32 stepFrame)
    {
        u32 emitterNum = this->GetParticleEmitterSize();
        for (u32 i = 0; i < emitterNum; ++i)
        {
            nw::gfx::ParticleEmitter* emitter = this->GetParticleEmitter(i);
            emitter->ParticleAnimFrameController().SetStepFrame(stepFrame);
        }

        u32 modelNum = this->GetParticleModelSize();

        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);
            model->ParticleAnimFrameController().SetStepFrame(stepFrame);
        }
    }

    //
    //
    //
    void SetFrame(f32 frame)
    {
        u32 emitterNum = this->GetParticleEmitterSize();
        for (u32 i = 0; i < emitterNum; ++i)
        {
            nw::gfx::ParticleEmitter* emitter = this->GetParticleEmitter(i);
            emitter->ParticleAnimFrameController().SetFrame(frame);
        }

        u32 modelNum = this->GetParticleModelSize();

        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);
            model->ParticleAnimFrameController().SetFrame(frame);
        }
    }

    //
    //
    //
    //
    //
    void AddTimes(uint times, gfx::ParticleContext* particleContext)
    {
        for (u32 k = 0; k < times; ++k)
        {
            for (u32 i = 0; i < this->GetParticleModelSize(); ++i)
            {
                this->GetParticleModel(i)->UpdateParticleFrame();
            }

            for (u32 i = 0; i < this->GetParticleEmitterSize(); ++i)
            {
                this->GetParticleEmitter(i)->UpdateParticleFrame();
                this->GetParticleEmitter(i)->Emission(particleContext);
            }

            for (u32 i = 0; i < this->GetParticleModelSize(); ++i)
            {
                for (u32 j = 0; j < this->GetParticleModel(i)->GetParticleSetsCount(); ++j)
                {
                    this->GetParticleModel(i)->GetParticleSets(j)->UpdateParticles(particleContext);
                }
            }
        }
    }

    //
    //
    //
    //
    void Reset()
    {
        u32 emitterNum = this->GetParticleEmitterSize();
        for (u32 i = 0; i < emitterNum; ++i)
        {
            nw::gfx::ParticleEmitter* emitter = this->GetParticleEmitter(i);
            emitter->Reset();
        }

        u32 modelNum = this->GetParticleModelSize();

        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);
            model->ForeachParticleSet(nw::gfx::ParticleSetsClear());
            model->ParticleAnimFrameController().SetFrame(0.f);
            // Reset the hint information for inspections for processing sequences
            model->ResetDebugHint();
        }
    }

    //
    //
    //
    void SetEmissionRatio(f32 ratio)
    {
        u32 emitterNum = this->GetParticleEmitterSize();
        for (u32 i = 0; i < emitterNum; ++i)
        {
            nw::gfx::ParticleEmitter* emitter = this->GetParticleEmitter(i);
            nw::gfx::ResParticleEmitterParameter resParameter = emitter->GetResParticleEmitterParameterCopy();
            NW_ASSERT(resParameter.IsValid());

            resParameter.SetEmissionRatio(ratio);
        }
    }

    //
    //
    //
    u32 GetParticleSize()
    {
        u32 particleNum = 0;

        u32 modelNum = this->GetParticleModelSize();

        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);
            for (u32 j = 0; j < model->GetParticleSetsCount(); ++j )
            {
                nw::gfx::ParticleSet* particleSet = model->GetParticleSets(j);
                particleNum += particleSet->GetParticleCollection()->GetCount();
            }
        }

        return particleNum;
    }

    //
    //
    //
    bool IsDone()
    {
        u32 modelNum = this->GetParticleModelSize();
        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);
            if (model->HasParticle())
            {
                return false;
            }
        }

        u32 emitterNum = this->GetParticleEmitterSize();
        for (u32 i = 0; i < emitterNum; ++i)
        {
            nw::gfx::ParticleEmitter* emitter = this->GetParticleEmitter(i);
            if (emitter->IsAlive())
            {
                return false;
            }
        }

        return true;
    }

    //
    //
    //
    bool IsInfinity()
    {
        bool isInfinity = false;

        u32 emitterNum = this->GetParticleEmitterSize();
        for (u32 i = 0; i < emitterNum; ++i)
        {
            nw::gfx::ParticleEmitter* emitter = this->GetParticleEmitter(i);
            const nw::gfx::ResParticleEmitterParameter resource = emitter->GetResParticleEmitterParameterCopy(false);
            NW_ASSERT(resource.IsValid());

            if (resource.GetEmissionSpanInfinity())
            {
                isInfinity = true;
            }
        }

        return isInfinity;
    }

    //
    //
    //
    void SetParticleScaleOffset( f32 particleOffset )
    {
        u32 modelNum = this->GetParticleModelSize();

        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);

            for (int j = 0; j < (int)model->GetParticleSetsCount(); ++j)
            {
                nw::gfx::ParticleSet* particleSet = model->GetParticleSets(j);
                particleSet->SetScaleOffset(nw::math::VEC3(particleOffset, particleOffset, particleOffset));
            }
        }
    }

    //
    //
    //
    //
    //
    void SetParticleScaleOffset( f32 particleOffsetX, f32 particleOffsetY, f32 particleOffsetZ = 1.f )
    {
        u32 modelNum = this->GetParticleModelSize();

        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);

            for (int j = 0; j < (int)model->GetParticleSetsCount(); ++j)
            {
                nw::gfx::ParticleSet* particleSet = model->GetParticleSets(j);
                particleSet->SetScaleOffset(nw::math::VEC3(particleOffsetX, particleOffsetY, particleOffsetZ));
            }
        }
    }

    //
    //
    //
    void Emission(gfx::ParticleContext* particleContext)
    {
        for (u32 i = 0; i < this->GetParticleEmitterSize(); ++i)
        {
            nw::gfx::ParticleEmitter* emitter = this->GetParticleEmitter(i);
            emitter->UpdateParticleFrame();
            emitter->Emission(particleContext);
            emitter->Reset();
        }
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    void RegisterParticleModel( nw::gfx::ParticleModel* model )
    {
        m_ModelInstances.PushBack(model);
    }

    //
    //
    //
    void RegisterParticleEmitter( nw::gfx::ParticleEmitter* emitter )
    {
        m_EmitterInstances.PushBack(emitter);
    }

    //
    //
    //
    u32 GetParticleModelSize() const
    {
        return m_ModelInstances.size();
    }

    //
    //
    //
    u32 GetParticleEmitterSize() const
    {
        return m_EmitterInstances.size();
    }

    //
    //
    //
    nw::gfx::ParticleModel* GetParticleModel(u32 index)
    {
        if (this->GetParticleModelSize() > index)
        {
            return m_ModelInstances[index];
        }
        return NULL;
    }

    //
    //
    //
    nw::gfx::ParticleEmitter* GetParticleEmitter(u32 index)
    {
        if (this->GetParticleEmitterSize() > index)
        {
            return m_EmitterInstances[index];
        }
        return NULL;
    }

    //
    //
    //
    //
    nw::gfx::ParticleSet* GetParticleSet(const char* name)
    {
        u32 modelNum = this->GetParticleModelSize();

        for (u32 i = 0; i < modelNum; ++i)
        {
            nw::gfx::ParticleModel* model = this->GetParticleModel(i);

            for (int j = 0; j < (int)model->GetParticleSetsCount(); ++j)
            {
                nw::gfx::ParticleSet* particleSet = model->GetParticleSets(j);
                const char* particleSetName = particleSet->GetResParticleSet().GetName();

                if (std::strcmp(particleSetName, name) == 0)
                {
                    return particleSet;
                }
            }
        }

        return NULL;
    }

    //

protected:
    //----------------------------------------
    //
    //

    //
    ParticleHandle(
        os::IAllocator* allocator,
        gfx::ResTransformNode resObj,
        const gfx::TransformNode::Description& description);

    //
    virtual ~ParticleHandle()
    {

    }

    //

private:
    int m_Id;
    ut::MoveArray<nw::gfx::ParticleModel*>      m_ModelInstances;
    ut::MoveArray<nw::gfx::ParticleEmitter*>    m_EmitterInstances;
};


// To maintain compatibility with NW4C 1.2.0.
typedef ParticleHandle ParticleNode;



//
// The feature to manage particles is to be implemented at the application level.
// This class is an implementation example for reference.
// This implementation stresses simplicity over performance.
class ParticleEffect
{
public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    static ParticleEffect* Create(os::IAllocator* mainAllocator,
                                  os::IAllocator* deviceAllocator,
                                  bool autoAlocate,
                                  gfx::ParticleContext* particleContext);

    //
    //
    void Destroy();

private:
    //
    //
    //
    //
    //
    //
    //
    ParticleEffect(
        os::IAllocator* mainAllocator,
        os::IAllocator* deviceAllocator,
        bool autoAlocate,
        gfx::ParticleContext* particleContext
        )
        : m_NextId(0),
        m_MainAllocator(mainAllocator),
        m_DeviceAllocator(deviceAllocator),
        m_AutoAlocate(autoAlocate),
        m_ResModels(mainAllocator),
        m_ResEmitters(mainAllocator),
        m_FreeInstances(mainAllocator),
        m_ActiveInstances(mainAllocator),
        m_ParticleContext(particleContext)
    {
    }

    //
    ~ParticleEffect()
    {
        NW_ASSERT(this->GetActiveSize() == 0);
        this->FreePool();
    }
public:
    //
    //
    //
    void Setup(gfx::ResGraphicsFile resource, bool useParticleMaterial = false);

    //
    //
    //
    //
    void Register(gfx::ResGraphicsFile resource, const char** nodeNames);

    //
    //
    //
    void Register(gfx::ResGraphicsFile resource);

    //

    //----------------------------------------
    //
    //

    //
    //
    ParticleHandle* CreateMultiEmitterParticleHandle(u32 particleEmitterNum);

    //
    //
    ParticleHandle* LeaseInstance()
    {
        if (this->GetFreeSize() == 0)
        {
            if (this->m_AutoAlocate)
            {
                this->AddPool(1);
            }
            else
            {
                return NULL;
            }
        }

        ParticleHandle* node = this->m_FreeInstances.Back();
        this->m_FreeInstances.PopBack();
        this->m_ActiveInstances.PushBack(node);

        this->ResetParticle(node);

        return node;
    }

    //
    //
    void ReleaseInstance(gfx::SceneNode* node)
    {
        ParticleHandle* topNode = ut::DynamicCast<ParticleHandle*>(node);

        if (topNode != NULL)
        {
            int activeCount = this->GetActiveSize();
            this->m_ActiveInstances.EraseFind(topNode);
            NW_ASSERT(this->GetActiveSize() == activeCount - 1);

            this->m_FreeInstances.PushBack(topNode);
        }
    }

    //
    void DestroyParticleHandle(ParticleHandle* node);
    void DestroyParticleNode(ParticleHandle* node)
    {
        return DestroyParticleHandle(node);
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    ParticleHandle* GetActiveEffect(int index)
    {
        if (index < 0 || index >= this->GetActiveSize())
        {
            return NULL;
        }

        return this->m_ActiveInstances[index];
    }

    //
    //
    void ResetParticle(nw::gfx::SceneNode* topNode)
    {
        if (nw::ut::IsTypeOf<nw::gfx::ParticleModel>(topNode))
        {
            nw::gfx::ParticleModel* particleModel =
                nw::ut::DynamicCast<nw::gfx::ParticleModel*>(topNode);

            particleModel->ForeachParticleSet(nw::gfx::ParticleSetsClear());
            particleModel->ParticleAnimFrameController().SetFrame(0);
        }
        else if (nw::ut::IsTypeOf<nw::gfx::ParticleEmitter>(topNode))
        {
            nw::gfx::ParticleEmitter* emitter =
                nw::ut::DynamicCast<nw::gfx::ParticleEmitter*>(topNode);

            emitter->Reset();
        }

        nw::gfx::SceneNodeChildren::iterator end = topNode->GetChildEnd();
        for (nw::gfx::SceneNodeChildren::iterator child = topNode->GetChildBegin();
            child != end;
            ++child)
        {
            ResetParticle(*child);
        }
    }

    //

    //----------------------------------------
    //
    //

    //
    int GetPoolSize() const
    {
        return this->GetFreeSize() + this->GetActiveSize();
    }

    //
    int GetFreeSize() const
    {
        return this->m_FreeInstances.Size();
    }

    //
    int GetActiveSize() const
    {
        return this->m_ActiveInstances.Size();
    }

    //
    void AddPool(int num)
    {
        for (int i = 0 ; i < num ; ++i)
        {
            ParticleHandle* node = this->Allocate();
            this->m_FreeInstances.push_back(node);
        }
    }

    //
    void FreePool()
    {
        while (this->m_FreeInstances.Size() > 0)
        {
            ParticleHandle* node = this->m_FreeInstances.Back();
            this->m_FreeInstances.PopBack();
            DestroyParticleHandle(node);
        }
    }

    //


    //----------------------------------------
    //
    //

    //
    static void InitializeShaderBinary(const wchar_t* shaderFilePath, os::IAllocator* deviceAllocator);

    //
    static void FinalizeShaderBinary();

    //

private:
    //
    ParticleHandle* Allocate();

    int m_NextId;

    os::IAllocator* m_MainAllocator;
    os::IAllocator* m_DeviceAllocator;


    bool m_AutoAlocate;

    ut::MoveArray<gfx::ResParticleModel> m_ResModels;
    ut::MoveArray<gfx::ResParticleEmitter> m_ResEmitters;

    ut::MoveArray<ParticleHandle*> m_FreeInstances;
    ut::MoveArray<ParticleHandle*> m_ActiveInstances;

    gfx::ParticleContext* m_ParticleContext;

    static nw::demo::ResourceSet* m_ShaderResource;
    static os::IAllocator* m_ShaderAllocator;
};


} // namespace demo

} // namespace nw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif // NW_DEMO_PARTICLE_H_
