/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_UTILITY_H_
#define NW_DEMO_UTILITY_H_

#include <nw/types.h>
#include <nw/ut/ut_MoveArray.h>
#include <nw/demo/demo_GraphicsDrawing.h>
#include <nw/demo/demo_GraphicsSystem.h>
#include <nw/demo/demo_Pad.h>

// A filename definition utility macro.
#define NW_DEMO_FILE_PATH(name) (L"rom:/" name)


namespace nw
{

namespace demo
{

namespace internal
{
    bool IsTerminatingImpl();
} // namespace internal

//============================================================================
//
//

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class Utility
{
public:
    static const nw::math::VEC3 CAMERA_POSITION; //
    static const nw::math::VEC3 TARGET_POSITION; //
    static const f32 NEAR_CLIP; //
    static const f32 FAR_CLIP; //
    static const f32 FOVY_RADIAN; //
    static const nw::math::VEC2 PROJECTION_CENTER; //
    static const f32 PROJECTION_HEIGHT; //

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    enum AnimationType {
        SKELETAL_ANIMATION,       //
        MATERIAL_ANIMATION,       //
        VISIBILITY_ANIMATION,     //
        CAMERA_ANIMATION,         //
        LIGHT_ANIMATION,          //
        FOG_ANIMATION             //
    };

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void InitializeGraphicsDrawing(
        os::IAllocator* allocator,
        nw::demo::GraphicsDrawing&  graphicsDrawing
    );

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static ut::MoveArray<u8> LoadFile(
        os::IAllocator* allocator,
        const wchar_t* filePath,
        u32 align = 32
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static ResourceSet*
    LoadResources(ResourceArray& resourceArray, const wchar_t* resourcePath, os::IAllocator* allocator);

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static nw::gfx::Camera* CreateCamera(
        os::IAllocator* allocator,
        const nw::math::VEC3& cameraPosition = CAMERA_POSITION,
        const nw::math::VEC3& targetPosition = TARGET_POSITION,
        f32 nearClip = NEAR_CLIP,
        f32 farClip = FAR_CLIP,
        f32 fovyRadian = FOVY_RADIAN,
        nw::math::PivotDirection pivotDirection = nw::math::PIVOT_UPSIDE_TO_TOP,
        f32 wScale = 0.0f
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static nw::gfx::Camera* CreateAimCamera(
        os::IAllocator* allocator,
        const nw::math::VEC3& cameraPosition = CAMERA_POSITION,
        const nw::math::VEC3& targetPosition = TARGET_POSITION,
        f32 nearClip = NEAR_CLIP,
        f32 farClip = FAR_CLIP,
        f32 fovyRadian = FOVY_RADIAN,
        nw::math::PivotDirection pivotDirection = nw::math::PIVOT_UPSIDE_TO_TOP,
        f32 wScale = 0.0f
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static nw::gfx::Camera* CreateFrustumCamera(
        os::IAllocator* allocator,
        const nw::math::VEC3& cameraPosition = CAMERA_POSITION,
        const nw::math::VEC3& targetPosition = TARGET_POSITION,
        f32 nearClip = NEAR_CLIP,
        f32 farClip = FAR_CLIP,
        const nw::math::VEC2& projectionCenter = PROJECTION_CENTER,
        f32 projectionHeight = PROJECTION_HEIGHT,
        nw::math::PivotDirection pivotDirection = nw::math::PIVOT_UPSIDE_TO_TOP,
        f32 wScale = 0.0f
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static nw::gfx::Camera* CreateOrthoCamera(
        os::IAllocator* allocator,
        const nw::math::VEC3& cameraPosition = CAMERA_POSITION,
        const nw::math::VEC3& targetPosition = TARGET_POSITION,
        f32 nearClip = NEAR_CLIP,
        f32 farClip = FAR_CLIP,
        const nw::math::VEC2& projectionCenter = PROJECTION_CENTER,
        f32 projectionHeight = PROJECTION_HEIGHT,
        nw::math::PivotDirection pivotDirection = nw::math::PIVOT_UPSIDE_TO_TOP,
        f32 wScale = 0.0f
    );



    // TODO: Confirm whether the expression, 'base camera', is appropriate.
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void CreateStereoCameras(
        nw::gfx::Camera** ppBaseCamera,
        nw::gfx::Camera** ppLeftCamera,
        nw::gfx::Camera** ppRightCamera,
        os::IAllocator* allocator,
        const nw::math::VEC3& cameraPosition = CAMERA_POSITION,
        const nw::math::VEC3& targetPosition = TARGET_POSITION,
        f32 nearClip = NEAR_CLIP,
        f32 farClip = FAR_CLIP,
        f32 fovyRadian = FOVY_RADIAN,
        f32 wScale = 0.0f
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void SetCameraAspectRatio(
        nw::gfx::Camera* camera,
        const nw::gfx::IRenderTarget* renderTarget,
        bool isAntiAlias2x1 = false
    )
    {
        NW_POINTER_ASSERT(renderTarget);
        // Since the vertical and horizontal are reversed for the on-screen buffer, set the width and height in reverse and find the aspect ratio.
        // 
        SetCameraAspectRatio(
            camera,
            static_cast<f32>(renderTarget->GetDescription().height) /
                static_cast<f32>(renderTarget->GetDescription().width) *
                    (isAntiAlias2x1 ? 2.0f : 1.0f)
        );
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void SetCameraAspectRatio(
        nw::gfx::Camera* camera,
        f32 aspectRatio
    );

    //

    //=====================
    //
    //


    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------

    static nw::gfx::IRenderTarget* CreateUpperScreenBuffer(
        os::IAllocator* allocator,
        nw::demo::RenderSystem::Description& renderDescription
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------

    static nw::gfx::IRenderTarget* CreateLowerScreenBuffer(
        os::IAllocator* allocator,
        nw::demo::RenderSystem::Description& renderDescription
    );

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static nw::gfx::SceneNode* CreateSceneNode(
        os::IAllocator* deviceAllocator,
        nw::gfx::ResSceneObject resource,
        bool isAnimationEnabled = true,
        nw::gfx::Model::BufferOption bufferOption = nw::gfx::Model::FLAG_BUFFER_NOT_USE,
        s32 maxAnimObjects = 1
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    template <class TObjectArray>
    static nw::gfx::SceneObject* FindObjectByName(
        TObjectArray& objects,
        const char* objectName
    )
    {
        // Find the scene object with the specified name.
        nw::gfx::SceneObject* targetObject = NULL;

        typename TObjectArray::iterator objEnd = objects.end();
        for (typename TObjectArray::iterator obj = objects.begin(); obj != objEnd; ++obj)
        {
            if (std::strcmp(objectName, (*obj)->GetName()) == 0)
            {
                targetObject = *obj;
                break;
            }
        }

        return targetObject;
    }

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    template <class TResArray>
    static nw::anim::ResAnim FindResAnimByName(
        TResArray& resources,
        const char* animationName,
        AnimationType animationType
    )
    {
        nw::anim::ResAnim resAnim;

        typename TResArray::iterator resEnd = resources.end();
        for (typename TResArray::iterator res = resources.begin(); res != resEnd; ++res)
        {
            switch (animationType)
            {
            case SKELETAL_ANIMATION:
                resAnim = res->GetSkeletalAnims(animationName);
                break;
            case MATERIAL_ANIMATION:
                resAnim = res->GetMaterialAnims(animationName);
                break;
            case VISIBILITY_ANIMATION:
                resAnim = res->GetVisibilityAnims(animationName);
                break;
            case CAMERA_ANIMATION:
                resAnim = res->GetCameraAnims(animationName);
                break;
            case LIGHT_ANIMATION:
                resAnim = res->GetLightAnims(animationName);
                break;
            default:
                NW_FATAL_ERROR("Invalid animation type");
            }

            if (resAnim.IsValid())
            {
                break;
            }
        }

        return resAnim;
    }


    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static nw::gfx::BaseAnimEvaluator* BindAnimation(
        os::IAllocator* allocator,
        nw::gfx::SceneObject* object,
        nw::anim::ResAnim resAnim,
        AnimationType animationType,
        bool allocCache
    )
    {
        // Get the animation group.
        nw::gfx::AnimGroup* animGroup = GetAnimGroup(object, animationType);
        if (!animGroup)
        {
            NW_LOG("BindAnimation() : Specified type animation group not found\n");
            return NULL;
        }

        // Generate the animation object.
        nw::gfx::BaseAnimEvaluator* animObject;
        if (animationType == SKELETAL_ANIMATION)
        {
            animObject = nw::gfx::TransformAnimEvaluator::Builder()
                .AnimData(resAnim)
                .MaxMembers(animGroup->GetMemberCount())
                .MaxAnimMembers(resAnim.GetMemberAnimSetCount())
                .AllocCache(allocCache)
                .Create(allocator);
        }
        else
        {
            animObject = nw::gfx::AnimEvaluator::Builder()
                .AnimData(resAnim)
                .MaxMembers(animGroup->GetMemberCount())
                .MaxAnimMembers(resAnim.GetMemberAnimSetCount())
                .AllocCache(allocCache)
                .Create(allocator);
        }
        NW_NULL_ASSERT(animObject);

        // If it's a skeletal animation, set the invalidation flag for movement animation.
        // 
        if (animationType == SKELETAL_ANIMATION)
        {
            nw::gfx::SkeletalModel* skeletalModel =
                nw::ut::DynamicCast<nw::gfx::SkeletalModel*>(object);
            nw::gfx::ResSkeleton resSkeleton = skeletalModel->GetSkeleton()->GetResSkeleton();
            const bool translateAnimEnabled = nw::ut::CheckFlag(
                resSkeleton.GetFlags(),
                nw::gfx::ResSkeletonData::FLAG_TRANSLATE_ANIMATION_ENABLED
            );
            reinterpret_cast<nw::gfx::TransformAnimEvaluator*>(animObject)
                ->SetIsTranslateDisabled(!translateAnimEnabled);
        }

        // Bind animation objects and scene objects.
        if (!BindAnimationObject(object, animObject, animationType))
        {

            NW_LOG("BindAnimation() : Failed to bind animation object\n");
            nw::ut::SafeDestroy(animObject);
            return NULL;
        }

        return animObject;
    }


    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    template <typename TAnimObject>
    static TAnimObject* BindAnimationBlender(
        os::IAllocator* allocator,
        nw::gfx::SceneObject* object,
        int maxAnimObjects,
        AnimationType animationType
    )
    {
        // Get the animation group.
        nw::gfx::AnimGroup* animGroup = GetAnimGroup(object, animationType);
        if (!animGroup)
        {
            NW_LOG("BindAnimationBlender() : Specified type animation group not found\n");
            return NULL;
        }

        // Generate the animation object.
        TAnimObject* animObject = TAnimObject::Builder()
            .MaxAnimObjects(maxAnimObjects)
            .Create(allocator);
        NW_NULL_ASSERT(animObject);


        // Bind animation objects and scene objects.
        if (!BindAnimationObject(object, animObject, animationType))
        {
            NW_LOG("BindAnimationBlender() : Failed to bind animation object\n");
            nw::ut::SafeDestroy(animObject);
            return NULL;
        }

        return animObject;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    template <class TObjectArray, class TResArray>
    static nw::gfx::BaseAnimEvaluator* BindAnimationByName(
        os::IAllocator* allocator,
        TObjectArray& objects,
        TResArray& resources,
        const char* objectName,
        const char* animationName,
        AnimationType animationType,
        bool allocCache
    )
    {
        nw::gfx::SceneObject* targetObject = FindObjectByName(objects, objectName);

        if (!targetObject)
        {
            // When the object with the specified name was not found, return NULL.
            NW_LOG("BindAnimationByName() : Specified object not found [%s]\n", objectName);
            return NULL;
        }


        // Find the animation data with the specified name.
        nw::anim::ResAnim resAnim = FindResAnimByName(resources, animationName, animationType);

        if (!resAnim.IsValid())
        {
            // When the animation data with the specified type and name was not found, return NULL.
            NW_LOG("BindAnimationByName() : Specified animation resource not found [%s]\n", animationName);
            return NULL;
        }

        // Bind the animation.
        nw::gfx::BaseAnimEvaluator* animObject = BindAnimation(
            allocator,
            targetObject,
            resAnim,
            animationType,
            allocCache);

        return animObject;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static nw::gfx::AnimGroup* GetAnimGroup(
        nw::gfx::SceneObject* object,
        AnimationType animationType
    );

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static bool InitializeSharedFont(
        os::IAllocator* allocator,
        nw::demo::GraphicsDrawing&  graphicsDrawing
    );

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static bool
    IsTerminating()
    {
        #if defined(NW_DEBUG_CHECK_MEMORY_LEAK)
            nw::demo::Pad* pad = nw::demo::PadFactory::GetPad();
            if (pad->IsButtonDown(nw::demo::Pad::BUTTON_START))
            {
                return true;
            }
        #endif //defined(NW_DEBUG_CHECK_MEMORY_LEAK)

        return internal::IsTerminatingImpl();
    }

    //

private:

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static bool BindAnimationObject(
        nw::gfx::SceneObject* object,
        nw::gfx::AnimObject* animObject,
        AnimationType animationType
    );

private:
    static const wchar_t* FONT_SHADER_FILE_NAME;
    static const wchar_t* SHAPE_2D_SHADER_FILE_NAME;
    static bool s_SharedFontAlreadyInitialized;
};

//

} // namespace demo
} // namespace nw

#endif // NW_DEMO_UTILITY_H_
