/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_FONT_RESFONTBASE_H_
#define NW_FONT_RESFONTBASE_H_

#include <nn/types.h>
#include <nw/font/font_Font.h>
#include <nw/font/font_ResourceFormat.h>


namespace nw {
namespace font {

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ResFontBase : public Font
{
public:
    /* ------------------------------------------------------------------------
            Functions
       ------------------------------------------------------------------------ */

    //
    //

    //
                            ResFontBase();

    //
    virtual                 ~ResFontBase();

    //

    //
    //

    virtual int             GetWidth() const;

    virtual int             GetHeight() const;

    virtual int             GetAscent() const;

    virtual int             GetDescent() const;

    virtual int             GetMaxCharWidth() const;

    virtual Type            GetType() const;

    virtual TexFmt          GetTextureFormat() const;

    virtual int             GetLineFeed() const;

    virtual const CharWidths
                            GetDefaultCharWidths() const;

    //
    //

    virtual void            SetLineFeed(int linefeed);

    virtual void            SetDefaultCharWidths(const CharWidths& widths);

    virtual bool            SetAlternateChar(CharCode c);

    //

    //
    //

    virtual int             GetCharWidth(CharCode c) const;

    virtual const CharWidths
                            GetCharWidths(CharCode c) const;

    virtual void            GetGlyph(
                                Glyph*      pGlyph,
                                CharCode    c
                            ) const;

    virtual bool            HasGlyph(CharCode c) const;

    //

    //
    //

    virtual CharacterCode   GetCharacterCode() const;

    //

    //
    //

    virtual int             GetBaselinePos() const;
    virtual int             GetCellHeight() const;
    virtual int             GetCellWidth() const;

    //


    //
    //

    virtual void            EnableLinearFilter(
                                bool    atSmall,
                                bool    atLarge);

    virtual bool            IsLinearFilterEnableAtSmall() const;

    virtual bool            IsLinearFilterEnableAtLarge() const;

    virtual u32             GetTextureWrapFilterValue() const;

    //

protected:
    /* ------------------------------------------------------------------------
            Types
       ------------------------------------------------------------------------ */
    typedef u16             GlyphIndex;

    /* ------------------------------------------------------------------------
            Constants
       ------------------------------------------------------------------------ */
    static const GlyphIndex GLYPH_INDEX_NOT_FOUND = INVALID_GLYPH_INDEX;


    /* ------------------------------------------------------------------------
            Functions
       ------------------------------------------------------------------------ */
    //---- Member access

    FontInformation*        GetFINF()       { return m_pFontInfo; }

    const FontInformation*  GetFINF() const { return m_pFontInfo; }

    //
    //
    //
    //
    //
    //
    //
    bool                    IsManaging(const void* ptr) const { return m_pResource == ptr; }

    //
    //
    //
    //
    //
    //
    void                    SetResourceBuffer(
                                void*               pUserBuffer,
                                FontInformation*    pFontInfo);

    //
    //
    //
    //
    //
    void*                   RemoveResourceBuffer();


    //---- glyph index


    //
    //
    //
    //
    //
    //
    //
    //
    GlyphIndex              GetGlyphIndex(CharCode c) const;

    //
    //
    //
    //
    //
    //
    //
    //
    GlyphIndex              FindGlyphIndex(CharCode c) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    GlyphIndex              FindGlyphIndex(
                                const FontCodeMap*  pMap,
                                CharCode            c
                            ) const;

    //
    //
    //
    //
    //
    //
    const CharWidths&       GetCharWidthsFromIndex(GlyphIndex index) const;

    //
    //
    //
    //
    //
    //
    //
    //
    const CharWidths&       GetCharWidthsFromIndex(
                                const FontWidth*    pWidth,
                                GlyphIndex          index
                            ) const;

    //
    //
    //
    //
    //
    void                    GetGlyphFromIndex(
                                Glyph*      glyph,
                                GlyphIndex  index
                            ) const;


    //
    //
    //
    //
    //
    //
    static void             SetGlyphMember(
                                Glyph*                  glyph,
                                GlyphIndex              index,
                                const FontTextureGlyph& tg);


    //
    //

    //
    //
    //
    //
    virtual int             GetActiveSheetNum() const;

    //
    //
    //
    //
    internal::TextureObject*
                            GetTextureObjectsBufferPtr()
    {
        return m_pTexObjs;
    }

    //
    //
    //
    //
    const internal::TextureObject*
                            GetTextureObjectsBufferPtr() const
    {
        return m_pTexObjs;
    }

    //
    //
    //
    //
    void                    SetTextureObjectsBufferPtr(void* buffer)
    {
        m_pTexObjs = static_cast<internal::TextureObject*>(buffer);
    }

    //
    //
    //
    //
    //
    //
    const internal::TextureObject*
                            GetTextureObject(int index) const
    {
        NN_POINTER_ASSERT(GetTextureObjectsBufferPtr());
        return &GetTextureObjectsBufferPtr()[index];
    }

    //
    void                    GenTextureNames();

    // Deletes texture names.
    void                    DeleteTextureNames();

    //

private:
    /* ------------------------------------------------------------------------
            Variables
       ------------------------------------------------------------------------ */

    void*                   m_pResource;    //

    //
    FontInformation*        m_pFontInfo;

    internal::TextureObject*
                            m_pTexObjs;     //
    u32                     m_WrapFilter;   //

    //
    mutable CharCode        m_LastCharCode;

    //
    mutable GlyphIndex      m_LastGlyphIndex;
};

namespace internal {

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
inline
u32
GetCellsInASheet(const FontTextureGlyph& tg)
{
    return static_cast<u32>(tg.sheetRow * tg.sheetLine);
}


}   // namespace internal

}   // namespace font
}   // namespace nw

#endif //  NW_FONT_RESFONTBASE_H_
