/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_FONT_TYPES_H_
#define NW_FONT_TYPES_H_

#if defined(NW_FONT2_TYPES_H_)
#error Both font1 and font2 header included. You can't use them together.
#endif

#if defined(_MSC_VER) && _MSC_VER >= 1500
#pragma once
#endif

#include <nn/types.h>
#include <nw/ut/ut_Color.h>
#include <nw/font/font_DrawerTypes.h>

namespace nw {
namespace font {
namespace internal {

enum
{
    VERTEX_ATTR_POS,
    VERTEX_ATTR_POS_Z,
    VERTEX_ATTR_COLOR,
    VERTEX_ATTR_TEXCOORD,

    VERTEX_ATTR_NUM
};

enum
{
    POS_X,
    POS_Y,

    POS_NUM
};

enum
{
    COLOR_R,
    COLOR_G,
    COLOR_B,
    COLOR_A,

    COLOR_NUM
};

enum
{
    TEXCOORD_X,
    TEXCOORD_Y,

    TEXCOORD_NUM
};

enum
{
    TRIFAN_VTX_RT,
    TRIFAN_VTX_LT,
    TRIFAN_VTX_LB,
    TRIFAN_VTX_RB,

    TRIFAN_VTX_MAX
};

enum
{
    LOC_PROJECTION,
    LOC_MODELVIEW,

    LOC_FRAGMENTLIGHTING_ENABLED,
    LOC_TEXTURE0_SAMPLERTYPE,
    LOC_FOG_MODE,
    LOC_FRAGOPERATION_ENABLEALPHATEST,
    LOC_FRAGOPERATION_ALPHAREFVALUE,
    LOC_FRAGOPERATION_ALPHATESTFUNC,

    LOC_MAX
};

enum
{
    TCLOC_SRCRGB,
    TCLOC_SRCALPHA,
    TCLOC_OPERANDRGB,
    TCLOC_OPERANDALPHA,
    TCLOC_COMBINERGB,
    TCLOC_COMBINEALPHA,
    TCLOC_SCALERGB,
    TCLOC_SCALEALPHA,
    TCLOC_CONSTRGBA,

    TCLOC_MAX
};

enum
{
    TEXENV_3,
    TEXENV_4,
    TEXENV_5,

    TEXENV_MAX
};

enum TextColor
{
    TEXTCOLOR_START,        //
    TEXTCOLOR_END,          //

    TEXTCOLOR_MAX           //
};

//
const int               VERTEX_INDEX_NUM = 6;

struct VertexAttribute
{
    f32                 pos[POS_NUM];
    ut::Color8          color;
    f32                 tex[TEXCOORD_NUM];
};

}   // namespace internal

#if defined(NW_FONT_ENABLE_EDGE_FONT)
//---------------------------------------------------------------------------
//
//
//
//---------------------------------------------------------------------------
enum FontCombinerType
{
    //
    FONT_COMBINER_TYPE_DEFAULT = 0,

    //
    FONT_COMBINER_TYPE_EDGE,

    //
    FONT_COMBINER_TYPE_NO_EDGE,

    //
    FONT_COMBINER_TYPE_MAX
};
#endif

}   // namespace font
}   // namespace nw

#endif //  NW_FONT_TYPES_H_
