﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nn/types.h>
#include <nn/assert.h>

#include <nw/config.h>
#include <nw/font/font_DispStringBuffer.h>

namespace nw {
namespace font {

DispStringBuffer::DispStringBuffer(u32 charNum)
:   charCountMax(charNum),
    charCount(0),
    commandBufferMode(CBM_DYNAMIC),
    subCommandBuffer(NULL),
    commandBufferSize(0),
    textColorCommandOffset(0)
{
    ResetDrawCharCount();
    ClearCommand();

    u8* ptr = reinterpret_cast<u8*>(this->GetCharAttrs())
               + sizeof(internal::CharAttribute) * this->charCountMax;

    this->drawFlags = ptr;
    const u32 drawFlagBytes = math::RoundUp(this->charCountMax, 8) / 8;
    ptr += drawFlagBytes;

    const int COMMANDBUFF_ALIGNMENT = 16;
    ptr = static_cast<u8*>(ut::RoundUp(ptr, COMMANDBUFF_ALIGNMENT));
    this->initialCommandBuffer = reinterpret_cast<u32*>(ptr);
    this->commandBufferCapacity = CalcCommandBufferCapacity(charNum);
    ptr += sizeof(u32) * this->commandBufferCapacity;

    this->commandBuffer = this->initialCommandBuffer;
}

}   // namespace font
}   // namespace nw
