/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_GFX_SHADERUNIFORMS_H_
#define NW_GFX_SHADERUNIFORMS_H_

#include <nw/types.h>
#include <nw/gfx/gfx_GfxObject.h>
#include <nw/ut/ut_MoveArray.h>

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>

namespace nw
{
namespace gfx
{

#if defined(NW_GFX_PROGRAM_OBJECT_ENABLED)

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
enum ShaderUniform
{
    SHADER_UNIFORM_PROJMTX, //
    SHADER_UNIFORM_VIEWMTX, //
    SHADER_UNIFORM_WRLDMTX, //
    SHADER_UNIFORM_NORMMTX, //
    SHADER_UNIFORM_POSOFFS, //
    SHADER_UNIFORM_IRSCALE, //
    SHADER_UNIFORM_TEXCMAP, //
    SHADER_UNIFORM_TEXMTX0, //
    SHADER_UNIFORM_TEXMTX1, //
    SHADER_UNIFORM_TEXMTX2, //
    SHADER_UNIFORM_TEXTRAN, //
    SHADER_UNIFORM_MATAMBI, //
    SHADER_UNIFORM_MATDIFF, //
    SHADER_UNIFORM_HSLGCOL, //
    SHADER_UNIFORM_HSLSCOL, //
    SHADER_UNIFORM_HSLSDIR, //
    SHADER_UNIFORM_UNIVREG, //
    SHADER_UNIFORM_ISSMOSK, //
    SHADER_UNIFORM_ISRGDSK, //
    SHADER_UNIFORM_ISHEMIL, //
    SHADER_UNIFORM_ISHEMIO, //
    SHADER_UNIFORM_ISVERTA, //
    SHADER_UNIFORM_ISBONEW, //
    SHADER_UNIFORM_UVMAP0, //
    SHADER_UNIFORM_UVMAP1, //
    SHADER_UNIFORM_UVMAP2, //
    SHADER_UNIFORM_ISVERTL, //
    SHADER_UNIFORM_ISTEX1, //
    SHADER_UNIFORM_ISTEX2, //
    SHADER_UNIFORM_ISQUATE, //
    SHADER_UNIFORM_LIGHTCT, //
    SHADER_UNIFORM_DMP_TEXENV0_COMBINERGB, //
    SHADER_UNIFORM_DMP_TEXENV1_COMBINERGB, //
    SHADER_UNIFORM_DMP_TEXENV2_COMBINERGB, //
    SHADER_UNIFORM_DMP_TEXENV3_COMBINERGB, //
    SHADER_UNIFORM_DMP_TEXENV4_COMBINERGB, //
    SHADER_UNIFORM_DMP_TEXENV5_COMBINERGB, //
    SHADER_UNIFORM_DMP_TEXENV0_COMBINEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV1_COMBINEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV2_COMBINEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV3_COMBINEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV4_COMBINEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV5_COMBINEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV0_SRCRGB, //
    SHADER_UNIFORM_DMP_TEXENV1_SRCRGB, //
    SHADER_UNIFORM_DMP_TEXENV2_SRCRGB, //
    SHADER_UNIFORM_DMP_TEXENV3_SRCRGB, //
    SHADER_UNIFORM_DMP_TEXENV4_SRCRGB, //
    SHADER_UNIFORM_DMP_TEXENV5_SRCRGB, //
    SHADER_UNIFORM_DMP_TEXENV0_SRCALPHA, //
    SHADER_UNIFORM_DMP_TEXENV1_SRCALPHA, //
    SHADER_UNIFORM_DMP_TEXENV2_SRCALPHA, //
    SHADER_UNIFORM_DMP_TEXENV3_SRCALPHA, //
    SHADER_UNIFORM_DMP_TEXENV4_SRCALPHA, //
    SHADER_UNIFORM_DMP_TEXENV5_SRCALPHA, //
    SHADER_UNIFORM_DMP_TEXENV0_OPERANDRGB, //
    SHADER_UNIFORM_DMP_TEXENV1_OPERANDRGB, //
    SHADER_UNIFORM_DMP_TEXENV2_OPERANDRGB, //
    SHADER_UNIFORM_DMP_TEXENV3_OPERANDRGB, //
    SHADER_UNIFORM_DMP_TEXENV4_OPERANDRGB, //
    SHADER_UNIFORM_DMP_TEXENV5_OPERANDRGB, //
    SHADER_UNIFORM_DMP_TEXENV0_OPERANDALPHA, //
    SHADER_UNIFORM_DMP_TEXENV1_OPERANDALPHA, //
    SHADER_UNIFORM_DMP_TEXENV2_OPERANDALPHA, //
    SHADER_UNIFORM_DMP_TEXENV3_OPERANDALPHA, //
    SHADER_UNIFORM_DMP_TEXENV4_OPERANDALPHA, //
    SHADER_UNIFORM_DMP_TEXENV5_OPERANDALPHA, //
    SHADER_UNIFORM_DMP_TEXENV1_BUFFERINPUT, //
    SHADER_UNIFORM_DMP_TEXENV2_BUFFERINPUT, //
    SHADER_UNIFORM_DMP_TEXENV3_BUFFERINPUT, //
    SHADER_UNIFORM_DMP_TEXENV4_BUFFERINPUT, //
    SHADER_UNIFORM_DMP_TEXENV0_SCALERGB, //
    SHADER_UNIFORM_DMP_TEXENV1_SCALERGB, //
    SHADER_UNIFORM_DMP_TEXENV2_SCALERGB, //
    SHADER_UNIFORM_DMP_TEXENV3_SCALERGB, //
    SHADER_UNIFORM_DMP_TEXENV4_SCALERGB, //
    SHADER_UNIFORM_DMP_TEXENV5_SCALERGB, //
    SHADER_UNIFORM_DMP_TEXENV0_SCALEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV1_SCALEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV2_SCALEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV3_SCALEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV4_SCALEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV5_SCALEALPHA, //
    SHADER_UNIFORM_DMP_TEXENV0_CONSTRGBA, //
    SHADER_UNIFORM_DMP_TEXENV1_CONSTRGBA, //
    SHADER_UNIFORM_DMP_TEXENV2_CONSTRGBA, //
    SHADER_UNIFORM_DMP_TEXENV3_CONSTRGBA, //
    SHADER_UNIFORM_DMP_TEXENV4_CONSTRGBA, //
    SHADER_UNIFORM_DMP_TEXENV5_CONSTRGBA, //
    SHADER_UNIFORM_DMP_TEXENV0_BUFFERCOLOR, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTING_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTING_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SAMPLERD0, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SAMPLERD1, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SAMPLERRR, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SAMPLERRG, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SAMPLERRB, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SAMPLERFR, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_EMISSION, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTMATERIAL_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_ENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_AMBIENT, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_DIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_SPECULAR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_SPECULAR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_POSITION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_SPOTDIRECTION, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_SHADOWED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_GEOMFACTOR0, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_GEOMFACTOR1, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_TWOSIDEDIFFUSE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_SPOTENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_DISTANCEATTENUATIONENABLED, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_DISTANCEATTENUATIONBIAS, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_DISTANCEATTENUATIONSCALE, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_SAMPLERSP, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE0_SAMPLERDA, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE1_SAMPLERDA, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE2_SAMPLERDA, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE3_SAMPLERDA, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE4_SAMPLERDA, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE5_SAMPLERDA, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE6_SAMPLERDA, //
    SHADER_UNIFORM_DMP_FRAGMENTLIGHTSOURCE7_SAMPLERDA, //
    SHADER_UNIFORM_DMP_LIGHTENV_ABSLUTINPUTD0, //
    SHADER_UNIFORM_DMP_LIGHTENV_ABSLUTINPUTD1, //
    SHADER_UNIFORM_DMP_LIGHTENV_ABSLUTINPUTRR, //
    SHADER_UNIFORM_DMP_LIGHTENV_ABSLUTINPUTRG, //
    SHADER_UNIFORM_DMP_LIGHTENV_ABSLUTINPUTRB, //
    SHADER_UNIFORM_DMP_LIGHTENV_ABSLUTINPUTSP, //
    SHADER_UNIFORM_DMP_LIGHTENV_ABSLUTINPUTFR, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTINPUTD0, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTINPUTD1, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTINPUTRR, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTINPUTRG, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTINPUTRB, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTINPUTSP, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTINPUTFR, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTSCALED0, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTSCALED1, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTSCALERR, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTSCALERG, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTSCALERB, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTSCALESP, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTSCALEFR, //
    SHADER_UNIFORM_DMP_LIGHTENV_SHADOWSELECTOR, //
    SHADER_UNIFORM_DMP_LIGHTENV_BUMPSELECTOR, //
    SHADER_UNIFORM_DMP_LIGHTENV_BUMPMODE, //
    SHADER_UNIFORM_DMP_LIGHTENV_BUMPRENORM, //
    SHADER_UNIFORM_DMP_LIGHTENV_CONFIG, //
    SHADER_UNIFORM_DMP_LIGHTENV_INVERTSHADOW, //
    SHADER_UNIFORM_DMP_LIGHTENV_SHADOWPRIMARY, //
    SHADER_UNIFORM_DMP_LIGHTENV_SHADOWSECONDARY, //
    SHADER_UNIFORM_DMP_LIGHTENV_SHADOWALPHA, //
    SHADER_UNIFORM_DMP_LIGHTENV_FRESNELSELECTOR, //
    SHADER_UNIFORM_DMP_LIGHTENV_CLAMPHIGHLIGHTS, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTENABLEDSP, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTENABLEDD0, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTENABLEDD1, //
    SHADER_UNIFORM_DMP_LIGHTENV_LUTENABLEDREFL, //
    SHADER_UNIFORM_DMP_TEXTURE0_PERSPECTIVESHADOW, //
    SHADER_UNIFORM_DMP_TEXTURE0_SHADOWZSCALE, //
    SHADER_UNIFORM_DMP_TEXTURE0_SHADOWZBIAS, //
    SHADER_UNIFORM_DMP_TEXTURE0_SAMPLERTYPE, //
    SHADER_UNIFORM_DMP_TEXTURE1_SAMPLERTYPE, //
    SHADER_UNIFORM_DMP_TEXTURE2_SAMPLERTYPE, //
    SHADER_UNIFORM_DMP_TEXTURE3_SAMPLERTYPE, //
    SHADER_UNIFORM_DMP_TEXTURE2_TEXCOORD, //
    SHADER_UNIFORM_DMP_TEXTURE3_TEXCOORD, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTRGBMAP, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTALPHAMAP, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTALPHASEPARATE, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTCLAMPU, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTCLAMPV, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSHIFTU, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSHIFTV, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTMINFILTER, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTTEXWIDTH, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTTEXOFFSET, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTTEXBIAS, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTNOISEENABLE, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTNOISEU, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTNOISEV, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSAMPLERRGBMAP, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSAMPLERALPHAMAP, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSAMPLERNOISEMAP, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSAMPLERR, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSAMPLERG, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSAMPLERB, //
    SHADER_UNIFORM_DMP_TEXTURE3_PTSAMPLERA, //
    SHADER_UNIFORM_DMP_GAS_LIGHTXY, //
    SHADER_UNIFORM_DMP_GAS_LIGHTZ, //
    SHADER_UNIFORM_DMP_GAS_DELTAZ, //
    SHADER_UNIFORM_DMP_GAS_AUTOACC, //
    SHADER_UNIFORM_DMP_GAS_ACCMAX, //
    SHADER_UNIFORM_DMP_GAS_SHADINGDENSITYSRC, //
    SHADER_UNIFORM_DMP_GAS_COLORLUTINPUT, //
    SHADER_UNIFORM_DMP_GAS_SAMPLERTR, //
    SHADER_UNIFORM_DMP_GAS_SAMPLERTG, //
    SHADER_UNIFORM_DMP_GAS_SAMPLERTB, //
    SHADER_UNIFORM_DMP_GAS_ATTENUATION, //
    SHADER_UNIFORM_DMP_FOG_MODE, //
    SHADER_UNIFORM_DMP_FOG_COLOR, //
    SHADER_UNIFORM_DMP_FOG_ZFLIP, //
    SHADER_UNIFORM_DMP_FOG_SAMPLER, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_ENABLECLIPPINGPLANE, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_CLIPPINGPLANE, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_ENABLEALPHATEST, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_ALPHAREFVALUE, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_ALPHATESTFUNC, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_MODE, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_WSCALE, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_PENUMBRASCALE, //
    SHADER_UNIFORM_DMP_FRAGOPERATION_PENUMBRABIAS, //
    SHADER_UNIFORM_DMP_POINT_VIEWPORT, //
    SHADER_UNIFORM_DMP_POINT_DISTANCEATTENUATION, //
    SHADER_UNIFORM_DMP_LINE_WIDTH, //
    SHADER_UNIFORM_DMP_SILHOUETTE_WIDTH, //
    SHADER_UNIFORM_DMP_SILHOUETTE_SCALEBYW, //
    SHADER_UNIFORM_DMP_SILHOUETTE_COLOR, //
    SHADER_UNIFORM_DMP_SILHOUETTE_FRONTFACECCW, //
    SHADER_UNIFORM_DMP_SILHOUETTE_ACCEPTEMPTYTRIANGLES, //
    SHADER_UNIFORM_DMP_SILHOUETTE_OPENEDGECOLOR, //
    SHADER_UNIFORM_DMP_SILHOUETTE_OPENEDGEWIDTH, //
    SHADER_UNIFORM_DMP_SILHOUETTE_OPENEDGEDEPTHBIAS, //
    SHADER_UNIFORM_DMP_SILHOUETTE_OPENEDGEWIDTHSCALEBYW, //
    SHADER_UNIFORM_DMP_SILHOUETTE_OPENEDGEDEPTHBIASSCALEBYW, //
    SHADER_UNIFORM_DMP_SUBDIVISION_LEVEL, //
    SHADER_UNIFORM_DMP_SUBDIVISION_FRAGMENTLIGHTINGENABLED, //
    SHADER_UNIFORM_DMP_PARTSYS_COLOR, //
    SHADER_UNIFORM_DMP_PARTSYS_ASPECT, //
    SHADER_UNIFORM_DMP_PARTSYS_TIME, //
    SHADER_UNIFORM_DMP_PARTSYS_SPEED, //
    SHADER_UNIFORM_DMP_PARTSYS_COUNTMAX, //
    SHADER_UNIFORM_DMP_PARTSYS_RANDSEED, //
    SHADER_UNIFORM_DMP_PARTSYS_RANDOMCORE, //
    SHADER_UNIFORM_DMP_PARTSYS_DISTANCEATTENUATION, //
    SHADER_UNIFORM_DMP_PARTSYS_VIEWPORT, //
    SHADER_UNIFORM_DMP_PARTSYS_POINTSIZE, //
    SHADER_UNIFORM_COUNT
};

#endif

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
enum VertexShaderUniformIndex
{
    VERTEX_SHADER_UNIFORM_PROJMTX_INDEX = 86, //
    VERTEX_SHADER_UNIFORM_VIEWMTX_INDEX = 90, //
    VERTEX_SHADER_UNIFORM_WRLDMTX_INDEX = 0, //
    VERTEX_SHADER_UNIFORM_NORMMTX_INDEX = 3, //
    VERTEX_SHADER_UNIFORM_POSOFFS_INDEX = 6, //
    VERTEX_SHADER_UNIFORM_IRSCALE_INDEX = 7, //
    VERTEX_SHADER_UNIFORM_TEXCMAP_INDEX = 10, //
    VERTEX_SHADER_UNIFORM_TEXMTX0_INDEX = 11, //
    VERTEX_SHADER_UNIFORM_TEXMTX1_INDEX = 14, //
    VERTEX_SHADER_UNIFORM_TEXMTX2_INDEX = 17, //
    VERTEX_SHADER_UNIFORM_TEXTRAN_INDEX = 19, //
    VERTEX_SHADER_UNIFORM_MATAMBI_INDEX = 20, //
    VERTEX_SHADER_UNIFORM_MATDIFF_INDEX = 21, //
    VERTEX_SHADER_UNIFORM_HSLGCOL_INDEX = 22, //
    VERTEX_SHADER_UNIFORM_HSLSCOL_INDEX = 23, //
    VERTEX_SHADER_UNIFORM_HSLSDIR_INDEX = 24, //
    VERTEX_SHADER_UNIFORM_UNIVREG_INDEX = 25, //
    VERTEX_SHADER_UNIFORM_ISSMOSK_INDEX = 1, //
    VERTEX_SHADER_UNIFORM_ISRGDSK_INDEX = 2, //
    VERTEX_SHADER_UNIFORM_ISHEMIL_INDEX = 5, //
    VERTEX_SHADER_UNIFORM_ISHEMIO_INDEX = 6, //
    VERTEX_SHADER_UNIFORM_ISVERTA_INDEX = 7, //
    VERTEX_SHADER_UNIFORM_ISBONEW_INDEX = 8, //
    VERTEX_SHADER_UNIFORM_UVMAP0_INDEX = 9, //
    VERTEX_SHADER_UNIFORM_UVMAP1_INDEX = 10, //
    VERTEX_SHADER_UNIFORM_UVMAP2_INDEX = 11, //
    VERTEX_SHADER_UNIFORM_ISVERTL_INDEX = 12, //
    VERTEX_SHADER_UNIFORM_ISTEX1_INDEX = 13, //
    VERTEX_SHADER_UNIFORM_ISTEX2_INDEX = 14, //
    VERTEX_SHADER_UNIFORM_ISQUATE_INDEX = 15, //
    VERTEX_SHADER_UNIFORM_LIGHTCT_INDEX = 0, //
    VERTEX_SHADER_UNIFORM_INDEX_MAX = 96
};

#if defined(NW_GFX_PROGRAM_OBJECT_ENABLED)

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ShaderUniformLocation : public GfxObject
{
public:
    static ShaderUniformLocation* Create(os::IAllocator* allocator);

    void BuildUniformLocations(GLuint programObject);

    GLint GetUniformLocation(int location) const
    {
        return m_UniformLocations[location];
    }

private:
    NW_DISALLOW_COPY_AND_ASSIGN(ShaderUniformLocation);

    ShaderUniformLocation(os::IAllocator* allocator);
    ut::FixedSizeArray<GLint, SHADER_UNIFORM_COUNT> m_UniformLocations;
};

#endif

// A temporary macro for command cache use.
#define NW_GFX_VERTEX_UNIFORM(x)  VERTEX_SHADER_UNIFORM_##x##_INDEX

} // namespace gfx
} // namespace nw

#endif /* NW_GFX_SHADERUNIFORMS_H_ */

