/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_GFX_RESSKELETON_H_
#define NW_GFX_RESSKELETON_H_

#include <nw/ut/ut_ResUtil.h>
#include <nw/ut/ut_ResDictionary.h>
#include <nw/math/math_ResTypes.h>
#include <nw/gfx/res/gfx_ResSceneObject.h>
#include <nw/gfx/res/gfx_ResTypeInfo.h>

namespace nw {
namespace gfx {
namespace res {

//
//--------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
struct ResBoneData
{
    //
    //--------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    enum Flag
    {
        FLAG_IS_IDENTITY                    = 0x1 << 0,     //
        FLAG_IS_TRANSLATE_ZERO              = 0x1 << 1,     //
        FLAG_IS_ROTATE_ZERO                 = 0x1 << 2,     //
        FLAG_IS_SCALE_ONE                   = 0x1 << 3,     //
        FLAG_IS_UNIFORM_SCALE               = 0x1 << 4,     //
        FLAG_IS_SEGMENTSCALE_COMPENSATE     = 0x1 << 5,     //
        FLAG_IS_NEED_RENDERING              = 0x1 << 6,     //
        FLAG_IS_LOCAL_MATRIX_CALCULATE      = 0x1 << 7,     //
        FLAG_IS_WORLD_MATRIX_CALCULATE      = 0x1 << 8,     //
        FLAG_HAS_SKINNING_MATRIX            = 0x1 << 9      //
    };

    nw::ut::BinString toName;               //
    nw::ut::ResU32 m_Flags;                 //
    nw::ut::ResS32 m_Index;                 //
    nw::ut::ResS32 m_ParentBoneIndex;       //
    nw::ut::Offset toParentBone;            //
    nw::ut::Offset toChildBone;             //
    nw::ut::Offset toPrevSibling;           //
    nw::ut::Offset toNextSibling;           //
    nw::ut::ResTransform3 m_Transform;      //
    nw::ut::ResMtx34 m_LocalMatrix;         //
    nw::ut::ResMtx34 m_WorldMatrix;         //
    nw::ut::ResMtx34 m_InverseBaseMatrix;   //
    nw::ut::ResS32 m_BillboardMode;         //
    nw::ut::ResS32 m_UserDataDicCount;      //
    nw::ut::Offset toUserDataDic;           //
};

//
//--------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
struct ResSkeletonData : public ResSceneObjectData
{
    //
    //--------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    enum Flag
    {
        FLAG_MODEL_COORDINATE                       = 0x1 << 0,     //
        FLAG_TRANSLATE_ANIMATION_ENABLED            = 0x1 << 1      //
    };

    nw::ut::ResS32 m_BonesDicCount;     //
    nw::ut::Offset toBonesDic;          //
    nw::ut::Offset toRootBone;          //
    nw::ut::ResS32 m_ScalingRule;       //
    nw::ut::ResU32 m_Flags;             //
};

//--------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ResBone : public nw::ut::ResCommon< ResBoneData >
{
public:

    //
    enum BillboardMode
    {
        BILLBOARD_MODE_OFF,                 //
        BILLBOARD_MODE_WORLD,               //
        BILLBOARD_MODE_WORLD_VIEWPOINT,     //
        BILLBOARD_MODE_SCREEN,              //
        BILLBOARD_MODE_SCREEN_VIEWPOINT,    //
        BILLBOARD_MODE_Y_AXIAL,             //
        BILLBOARD_MODE_Y_AXIAL_VIEWPOINT    //
    };

    //--------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    enum Flag
    {
        FLAG_IS_IDENTITY                    = 0x1 << 0,     //
        FLAG_IS_TRANSLATE_ZERO              = 0x1 << 1,     //
        FLAG_IS_ROTATE_ZERO                 = 0x1 << 2,     //
        FLAG_IS_SCALE_ONE                   = 0x1 << 3,     //
        FLAG_IS_UNIFORM_SCALE               = 0x1 << 4,     //
        FLAG_IS_SEGMENTSCALE_COMPENSATE     = 0x1 << 5,     //
        FLAG_IS_NEED_RENDERING              = 0x1 << 6,     //
        FLAG_IS_LOCAL_MATRIX_CALCULATE      = 0x1 << 7,     //
        FLAG_IS_WORLD_MATRIX_CALCULATE      = 0x1 << 8,     //
        FLAG_HAS_SKINNING_MATRIX            = 0x1 << 9      //
    };

    NW_RES_CTOR( ResBone )

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_STRING_DECL( Name )                    // GetName()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_FLAGS_DECL( u32, Flags )               // GetFlags(), SetFlags(), EnableFlags(), DisableFlags()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_PRIMITIVE_DECL( s32, Index )           // GetIndex(), SetIndex()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_PRIMITIVE_DECL( s32, ParentBoneIndex ) // GetParentBoneIndex(), SetParentBoneIndex()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_CLASS_DECL( ResBone, ParentBone )      // GetParentBone()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_CLASS_DECL( ResBone, ChildBone )       // GetChildBone()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_CLASS_DECL( ResBone, PrevSibling )     // GetPrevSibling()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_CLASS_DECL( ResBone, NextSibling )     // GetNextSibling()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_STRUCT_DECL( nw::math::Transform3, Transform )    // GetTransform()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_STRUCT_DECL( nw::math::MTX34, LocalMatrix )       // GetLocalMatrix()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_STRUCT_DECL( nw::math::MTX34, WorldMatrix )       // After official support for buffer for matrix calculation, plan to delete it.

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_STRUCT_DECL( nw::math::MTX34, InverseBaseMatrix ) // GetInverseBaseMatrix()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_PRIMITIVE_DECL( BillboardMode, BillboardMode )   // GetBillboardMode(), SetBillboardMode()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_CLASS_DIC_DECL( nw::ut::ResMetaData, UserData, nw::ut::ResDicPatricia ) // GetMetaData(int idx), GetMetaData(const char*), GetMetaDataIndex(const char*), GetMetaDataCount()
};

//
typedef nw::ut::ResArrayPatricia<ResBone>::type  ResBoneArray;


//--------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ResSkeleton : public ResSceneObject
{
public:
    //--------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    enum Flag
    {
        FLAG_MODEL_COORDINATE                       = 0x1 << 0,     //
        FLAG_TRANSLATE_ANIMATION_ENABLED            = 0x1 << 1      //
    };

    enum { TYPE_INFO = NW_GFX_RES_TYPE_INFO(ResSkeleton) };
    enum { SIGNATURE = NW_RES_SIGNATURE32('CSKL') };

    NW_RES_CTOR_INHERIT( ResSkeleton, ResSceneObject )

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_CLASS_DIC_DECL( ResBone, Bones, nw::ut::ResDicPatricia ) // GetBones(int idx), GetBones(const char*), GetBonesIndex(const char*), GetBonesCount()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_CLASS_DECL( ResBone, RootBone )    // GetRootBone()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_PRIMITIVE_DECL( s32, ScalingRule ) // GetScalingRule(), SetScalingRule()

    //---------------------------------------------------------------------------
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    NW_RES_FIELD_FLAGS_DECL( u32, Flags )           // GetFlags(), SetFlags(), EnableFlags(), DisableFlags()
};

} // namespace res
} // namespace gfx
} // namespace nw

#endif // NW_GFX_RESSKELETON_H_
