/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/anim/anim_AnimBlend.h>
#include <nw/gfx/gfx_AnimGroup.h>
#include <nw/gfx/gfx_AnimOverrider.h>

namespace nw
{
namespace gfx
{

NW_UT_RUNTIME_TYPEINFO_DEFINITION(AnimOverrider    , AnimBlender);

//----------------------------------------------------------
const anim::AnimResult*
AnimOverrider::GetResult(
    void* target,
    int memberIdx
) const
{
    //----------------------------------------------------------
    // When there is no blend operation or when there is no blending for a blend operation, overwrite
    // 
    const anim::AnimBlendOp* blendOp = m_AnimGroup->GetBlendOperation(memberIdx);
    if (blendOp == NULL || !blendOp->HasBlend())
    {
        for (int animIdx = m_AnimObjects.Size() - 1; animIdx >= 0; --animIdx)
        {
            if (m_AnimObjects[animIdx] == NULL)
            {
                continue;
            }
            const anim::AnimResult* childResult =
                m_AnimObjects[animIdx]->GetResult(target, memberIdx);
            if (childResult != NULL)
            {
                return childResult;
            }
        }
        return NULL;
    }

    //----------------------------------------------------------
    // Loop for all child animations
    anim::AnimResult* result = reinterpret_cast<anim::AnimResult*>(target);
    anim::AnimResult workResult;
    bool written = false;
    for (int animIdx = m_AnimObjects.Size() - 1; animIdx >= 0; --animIdx)
    {
        if (m_AnimObjects[animIdx] == NULL)
        {
            continue;
        }
        const anim::AnimResult* childResult =
            m_AnimObjects[animIdx]->GetResult(&workResult, memberIdx);
        if (childResult != NULL)
        {
            written = true;
            if (blendOp->Override(result, childResult)) // TRUE is returned is all components are overwritten
            {
                return result;
            }
        }
    }

    if (!written) // No valid child animation
    {
        return NULL;
    }

    return result;
}

} // namespace gfx
} // namespace nw
