/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/gfx/gfx_BaseAnimEvaluator.h>
#include <nw/gfx/res/gfx_ResMaterial.h>

namespace nw
{
namespace gfx
{

namespace internal
{
    using namespace anim;
    // HACK
    void ClearMaterialHash(ResAnimGroupMember member)
    {
        ResMaterial material(reinterpret_cast<void*>(member.GetResMaterialPtr()));

        switch(member.GetObjectType())
        {
        case ResAnimGroupMember::OBJECT_TYPE_MATERIAL_COLOR:
            material.SetMaterialColorHash(0x0);
            break;

        case ResAnimGroupMember::OBJECT_TYPE_TEXTURE_SAMPLER:
            material.SetTextureMappersHash(0x0);
            material.SetTextureSamplersHash(0x0);
            break;

        case ResAnimGroupMember::OBJECT_TYPE_TEXTURE_MAPPER:
            // TODO: When the sampler does not change, it is possible to recalculate the hash value
            material.SetTextureMappersHash(0x0);
            break;

        case ResAnimGroupMember::OBJECT_TYPE_BLEND_OPERATION:
            material.SetFragmentOperationHash(0x0);
            break;

        case ResAnimGroupMember::OBJECT_TYPE_TEXTURE_COORDINATOR:
            material.SetTextureCoordinatorsHash(0x0);
            break;

        default:
            break;
        }
    }
}

NW_UT_RUNTIME_TYPEINFO_DEFINITION(BaseAnimEvaluator, AnimObject);

//
const int BaseAnimEvaluator::NotFoundIndex = -1;

//----------------------------------------------------------
Result
BaseAnimEvaluator::TryBind(AnimGroup* animGroup)
{
    return TryBindTemplate(animGroup, BasicIndexGetterFunctor());
}

//----------------------------------------------------------
void
BaseAnimEvaluator::ResetMember(int memberIdx)
{
    NW_NULL_ASSERT(m_AnimGroup);

    if (!m_AnimGroup->HasOriginalValue())
    {
        return;
    }

    anim::ResGraphicsAnimGroup data = m_AnimGroup->GetResGraphicsAnimGroup();
    NW_ASSERT(data.IsValid());

    const anim::ResAnimGroupMember resAnimGroupMember = m_AnimGroup->GetResAnimGroupMember(memberIdx);
    resAnimGroupMember.SetValueForType(
        m_AnimGroup->GetTargetObject(memberIdx),
        m_AnimGroup->GetOriginalValue(memberIdx));

    m_IsCacheDirty = true;
}

} // namespace gfx
} // namespace nw
