﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/gfx/gfx_CommandUtil.h>

namespace nw
{
namespace gfx
{

namespace internal
{

// #define NW_GFX_DISABLE_ASM

#if !defined(NW_GFX_DISABLE_ASM)

#include <nn/hw/ARM/code32.h>

//--------------------------------------------------------------------------------
asm void
NWCopyVec4Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s3}
    VSTR.F32    s0,[r0,#12]
    VSTR.F32    s1,[r0,#8]
    VSTR.F32    s2,[r0,#4]
    VSTR.F32    s3,[r0,#0]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyVec4WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s3}
    STR         r2,[r0,#4]
    VSTR.F32    s0,[r0,#16]
    VSTR.F32    s1,[r0,#12]
    VSTR.F32    s2,[r0,#8]
    VSTR.F32    s3,[r0,#0]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx24Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s7}
    VSTR.F32    s0,[r0,#12]
    VSTR.F32    s1,[r0,#8]
    VSTR.F32    s2,[r0,#4]
    VSTR.F32    s3,[r0,#0]
    VSTR.F32    s4,[r0,#28]
    VSTR.F32    s5,[r0,#24]
    VSTR.F32    s6,[r0,#20]
    VSTR.F32    s7,[r0,#16]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx24WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s7}
    STR         r2,[r0,#4]
    VSTR.F32    s0,[r0,#16]
    VSTR.F32    s1,[r0,#12]
    VSTR.F32    s2,[r0,#8]
    VSTR.F32    s3,[r0,#0]
    VSTR.F32    s4,[r0,#32]
    VSTR.F32    s5,[r0,#28]
    VSTR.F32    s6,[r0,#24]
    VSTR.F32    s7,[r0,#20]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx34Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s11}
    VSTR.F32    s0,[r0,#12]
    VSTR.F32    s1,[r0,#8]
    VSTR.F32    s2,[r0,#4]
    VSTR.F32    s3,[r0,#0]
    VSTR.F32    s4,[r0,#28]
    VSTR.F32    s5,[r0,#24]
    VSTR.F32    s6,[r0,#20]
    VSTR.F32    s7,[r0,#16]
    VSTR.F32    s8,[r0,#44]
    VSTR.F32    s9,[r0,#40]
    VSTR.F32    s10,[r0,#36]
    VSTR.F32    s11,[r0,#32]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx34WithHeader(
    f32*  /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s11}
    STR         r2,[r0,#4]
    VSTR.F32    s0,[r0,#16]
    VSTR.F32    s1,[r0,#12]
    VSTR.F32    s2,[r0,#8]
    VSTR.F32    s3,[r0,#0]
    VSTR.F32    s4,[r0,#32]
    VSTR.F32    s5,[r0,#28]
    VSTR.F32    s6,[r0,#24]
    VSTR.F32    s7,[r0,#20]
    VSTR.F32    s8,[r0,#48]
    VSTR.F32    s9,[r0,#44]
    VSTR.F32    s10,[r0,#40]
    VSTR.F32    s11,[r0,#36]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx44Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s15}
    VSTR.F32    s0,[r0,#12]
    VSTR.F32    s1,[r0,#8]
    VSTR.F32    s2,[r0,#4]
    VSTR.F32    s3,[r0,#0]
    VSTR.F32    s4,[r0,#28]
    VSTR.F32    s5,[r0,#24]
    VSTR.F32    s6,[r0,#20]
    VSTR.F32    s7,[r0,#16]
    VSTR.F32    s8,[r0,#44]
    VSTR.F32    s9,[r0,#40]
    VSTR.F32    s10,[r0,#36]
    VSTR.F32    s11,[r0,#32]
    VSTR.F32    s12,[r0,#60]
    VSTR.F32    s13,[r0,#56]
    VSTR.F32    s14,[r0,#52]
    VSTR.F32    s15,[r0,#48]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void NWCopyMtx44WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32         /* header */ )
{
    VLDMIA      r1!,{s0-s15}
    STR         r2,[r0,#4]
    VSTR.F32    s0,[r0,#16]
    VSTR.F32    s1,[r0,#12]
    VSTR.F32    s2,[r0,#8]
    VSTR.F32    s3,[r0,#0]
    VSTR.F32    s4,[r0,#32]
    VSTR.F32    s5,[r0,#28]
    VSTR.F32    s6,[r0,#24]
    VSTR.F32    s7,[r0,#20]
    VSTR.F32    s8,[r0,#48]
    VSTR.F32    s9,[r0,#44]
    VSTR.F32    s10,[r0,#40]
    VSTR.F32    s11,[r0,#36]
    VSTR.F32    s12,[r0,#64]
    VSTR.F32    s13,[r0,#60]
    VSTR.F32    s14,[r0,#56]
    VSTR.F32    s15,[r0,#52]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyVec3Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s2}
    MOV         r3, #0
    STR         r3,[r0,#0]

    VSTR.F32    s0,[r0,#12]
    VSTR.F32    s1,[r0,#8]
    VSTR.F32    s2,[r0,#4]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyVec3WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s2}
    MOV         r3, #0
    STR         r2,[r0,#4]
    STR         r3,[r0,#0]

    VSTR.F32    s0,[r0,#16]
    VSTR.F32    s1,[r0,#12]
    VSTR.F32    s2,[r0,#8]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx23Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s5}
    MOV         r3, #0
    STR         r3,[r0,#0]
    STR         r3,[r0,#16]

    VSTR.F32    s0,[r0,#12]
    VSTR.F32    s1,[r0,#8]
    VSTR.F32    s2,[r0,#4]
    VSTR.F32    s3,[r0,#28]
    VSTR.F32    s4,[r0,#24]
    VSTR.F32    s5,[r0,#20]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx23WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s5}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#20]

    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#12]
    VSTR.F32    s2, [r0,#8]
    VSTR.F32    s3, [r0,#32]
    VSTR.F32    s4, [r0,#28]
    VSTR.F32    s5, [r0,#24]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx33Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s8}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#16]
    STR         r3, [r0,#32]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#8]
    VSTR.F32    s2, [r0,#4]
    VSTR.F32    s3, [r0,#28]
    VSTR.F32    s4, [r0,#24]
    VSTR.F32    s5, [r0,#20]
    VSTR.F32    s6, [r0,#44]
    VSTR.F32    s7, [r0,#40]
    VSTR.F32    s8, [r0,#36]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx33WithHeader(
    f32*  /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s8}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#20]
    STR         r3, [r0,#36]

    VSTR.F32    s0,[r0,#16]
    VSTR.F32    s1,[r0,#12]
    VSTR.F32    s2,[r0,#8]
    VSTR.F32    s3,[r0,#32]
    VSTR.F32    s4,[r0,#28]
    VSTR.F32    s5,[r0,#24]
    VSTR.F32    s6,[r0,#48]
    VSTR.F32    s7,[r0,#44]
    VSTR.F32    s8,[r0,#40]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx43Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s11}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#16]
    STR         r3, [r0,#32]
    STR         r3, [r0,#48]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#8]
    VSTR.F32    s2, [r0,#4]
    VSTR.F32    s3, [r0,#28]
    VSTR.F32    s4, [r0,#24]
    VSTR.F32    s5, [r0,#20]
    VSTR.F32    s6, [r0,#44]
    VSTR.F32    s7, [r0,#40]
    VSTR.F32    s8, [r0,#36]
    VSTR.F32    s9, [r0,#60]
    VSTR.F32    s10, [r0,#56]
    VSTR.F32    s11, [r0,#52]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx43WithHeader(
    f32*  /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s11}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#20]
    STR         r3, [r0,#36]
    STR         r3, [r0,#52]

    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#12]
    VSTR.F32    s2, [r0,#8]
    VSTR.F32    s3, [r0,#32]
    VSTR.F32    s4, [r0,#28]
    VSTR.F32    s5, [r0,#24]
    VSTR.F32    s6, [r0,#48]
    VSTR.F32    s7, [r0,#44]
    VSTR.F32    s8, [r0,#40]
    VSTR.F32    s9, [r0,#64]
    VSTR.F32    s10, [r0,#60]
    VSTR.F32    s11, [r0,#56]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyVec2Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s1}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#8]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyVec2WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s1}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]

    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#12]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx22Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s3}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]
    STR         r3, [r0,#16]
    STR         r3, [r0,#20]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#8]
    VSTR.F32    s2, [r0,#28]
    VSTR.F32    s3, [r0,#24]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx22WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s3}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]


    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#12]
    VSTR.F32    s2, [r0,#32]
    VSTR.F32    s3, [r0,#28]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx32Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s5}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]
    STR         r3, [r0,#16]
    STR         r3, [r0,#20]
    STR         r3, [r0,#32]
    STR         r3, [r0,#36]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#8]
    VSTR.F32    s2, [r0,#28]
    VSTR.F32    s3, [r0,#24]
    VSTR.F32    s4, [r0,#44]
    VSTR.F32    s5, [r0,#40]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx32WithHeader(
    f32*  /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s5}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]
    STR         r3, [r0,#36]
    STR         r3, [r0,#40]

    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#12]
    VSTR.F32    s2, [r0,#32]
    VSTR.F32    s3, [r0,#28]
    VSTR.F32    s4, [r0,#48]
    VSTR.F32    s5, [r0,#44]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx42Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s7}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]
    STR         r3, [r0,#16]
    STR         r3, [r0,#20]
    STR         r3, [r0,#32]
    STR         r3, [r0,#36]
    STR         r3, [r0,#48]
    STR         r3, [r0,#52]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#8]
    VSTR.F32    s2, [r0,#28]
    VSTR.F32    s3, [r0,#24]
    VSTR.F32    s4, [r0,#44]
    VSTR.F32    s5, [r0,#40]
    VSTR.F32    s6, [r0,#60]
    VSTR.F32    s7, [r0,#56]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx42WithHeader(
    f32*  /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s7}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]
    STR         r3, [r0,#36]
    STR         r3, [r0,#40]
    STR         r3, [r0,#52]
    STR         r3, [r0,#56]

    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#12]
    VSTR.F32    s2, [r0,#32]
    VSTR.F32    s3, [r0,#28]
    VSTR.F32    s4, [r0,#48]
    VSTR.F32    s5, [r0,#44]
    VSTR.F32    s6, [r0,#64]
    VSTR.F32    s7, [r0,#60]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyVec1Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDR.F32    s0, [r1]
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]
    STR         r3, [r0,#8]

    VSTR.F32    s0, [r0,#12]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyVec1WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDR.F32    s0, [r1]
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]
    STR         r3, [r0,#12]

    VSTR.F32    s0, [r0,#16]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx21Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s1}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]
    STR         r3, [r0,#8]
    STR         r3, [r0,#16]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#28]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx21WithHeader(
    f32*        /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s1}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]
    STR         r3, [r0,#12]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]
    STR         r3, [r0,#28]

    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#32]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx31Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s2}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]
    STR         r3, [r0,#8]
    STR         r3, [r0,#16]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]
    STR         r3, [r0,#32]
    STR         r3, [r0,#36]
    STR         r3, [r0,#40]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#28]
    VSTR.F32    s2, [r0,#44]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx31WithHeader(
    f32*  /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s2}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]
    STR         r3, [r0,#12]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]
    STR         r3, [r0,#28]
    STR         r3, [r0,#36]
    STR         r3, [r0,#40]
    STR         r3, [r0,#44]

    VSTR.F32    s0, [r0,#16]
    VSTR.F32    s1, [r0,#32]
    VSTR.F32    s2, [r0,#48]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx41Reverse(
    f32*        /* dst */,
    const f32*  /* src */)
{
    VLDMIA      r1!,{s0-s3}
    MOV         r3, #0
    STR         r3, [r0,#0]
    STR         r3, [r0,#4]
    STR         r3, [r0,#8]
    STR         r3, [r0,#16]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]
    STR         r3, [r0,#32]
    STR         r3, [r0,#36]
    STR         r3, [r0,#40]
    STR         r3, [r0,#48]
    STR         r3, [r0,#52]
    STR         r3, [r0,#56]

    VSTR.F32    s0,[r0,#12]
    VSTR.F32    s1,[r0,#28]
    VSTR.F32    s2,[r0,#44]
    VSTR.F32    s3,[r0,#60]
    BX      lr
}

//--------------------------------------------------------------------------------
asm void
NWCopyMtx41WithHeader(
    f32*  /* dst */,
    const f32*  /* src */,
    u32   /* header */ )
{
    VLDMIA      r1!,{s0-s3}
    MOV         r3, #0
    STR         r2, [r0,#4]
    STR         r3, [r0,#0]
    STR         r3, [r0,#8]
    STR         r3, [r0,#12]
    STR         r3, [r0,#20]
    STR         r3, [r0,#24]
    STR         r3, [r0,#28]
    STR         r3, [r0,#36]
    STR         r3, [r0,#40]
    STR         r3, [r0,#44]
    STR         r3, [r0,#52]
    STR         r3, [r0,#56]
    STR         r3, [r0,#60]

    VSTR.F32    s0, [r0,#12]
    VSTR.F32    s1, [r0,#28]
    VSTR.F32    s2, [r0,#44]
    VSTR.F32    s3, [r0,#60]
    BX      lr
}

#include <nn/hw/ARM/codereset.h>

#else // defined(NW_GFX_DISABLE_ASM)

//--------------------------------------------------------------------------------
void
NWCopyVec4Reverse(
    f32*        dst,
    const f32*  src)
{
    dst[0] = src[3];
    dst[1] = src[2];
    dst[2] = src[1];
    dst[3] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyVec4WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    dst[0] = src[3];
    *(u32*)(&dst[1]) = header;
    dst[2] = src[2];
    dst[3] = src[1];
    dst[4] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyMtx24Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec4Reverse(&dst[0], &src[0]);
    NWCopyVec4Reverse(&dst[4], &src[4]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx24WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec4WithHeader(&dst[0], &src[0], header);
    NWCopyVec4Reverse(&dst[5], &src[4]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx34Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec4Reverse(&dst[0], &src[0]);
    NWCopyVec4Reverse(&dst[4], &src[4]);
    NWCopyVec4Reverse(&dst[8], &src[8]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx34WithHeader(
    f32*  dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec4WithHeader(&dst[0], &src[0], header);
    NWCopyVec4Reverse(&dst[5], &src[4]);
    NWCopyVec4Reverse(&dst[9], &src[8]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx44Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec4Reverse(&dst[0], &src[0]);
    NWCopyVec4Reverse(&dst[4], &src[4]);
    NWCopyVec4Reverse(&dst[8], &src[8]);
    NWCopyVec4Reverse(&dst[12], &src[12]);
}

//--------------------------------------------------------------------------------
void NWCopyMtx44WithHeader(
    f32*        dst,
    const f32*  src,
    u32         header )
{
    NWCopyVec4WithHeader(&dst[0], &src[0], header);
    NWCopyVec4Reverse(&dst[5], &src[4]);
    NWCopyVec4Reverse(&dst[9], &src[8]);
    NWCopyVec4Reverse(&dst[13], &src[12]);
}

//--------------------------------------------------------------------------------
void
NWCopyVec3Reverse(
    f32*        dst,
    const f32*  src)
{
    dst[0] = 0.0f;
    dst[1] = src[2];
    dst[2] = src[1];
    dst[3] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyVec3WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    dst[0] = 0.0f;
    *(u32*)(&dst[1]) = header;
    dst[2] = src[2];
    dst[3] = src[1];
    dst[4] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyMtx23Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec3Reverse(&dst[0], &src[0]);
    NWCopyVec3Reverse(&dst[4], &src[3]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx23WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec3WithHeader(&dst[0], &src[0], header);
    NWCopyVec3Reverse(&dst[5], &src[3]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx33Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec3Reverse(&dst[0], &src[0]);
    NWCopyVec3Reverse(&dst[4], &src[3]);
    NWCopyVec3Reverse(&dst[8], &src[6]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx33WithHeader(
    f32*  dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec3WithHeader(&dst[0], &src[0], header);
    NWCopyVec3Reverse(&dst[5], &src[3]);
    NWCopyVec3Reverse(&dst[9], &src[6]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx43Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec3Reverse(&dst[0], &src[0]);
    NWCopyVec3Reverse(&dst[4], &src[3]);
    NWCopyVec3Reverse(&dst[8], &src[6]);
    NWCopyVec3Reverse(&dst[12], &src[9]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx43WithHeader(
    f32*  dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec3WithHeader(&dst[0], &src[0], header);
    NWCopyVec3Reverse(&dst[5], &src[3]);
    NWCopyVec3Reverse(&dst[9], &src[6]);
    NWCopyVec3Reverse(&dst[13], &src[9]);
}

//--------------------------------------------------------------------------------
void
NWCopyVec2Reverse(
    f32*        dst,
    const f32*  src)
{
    dst[0] = 0.0f;
    dst[1] = 0.0f;
    dst[2] = src[1];
    dst[3] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyVec2WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    dst[0] = 0.0f;
    *(u32*)(&dst[1]) = header;
    dst[2] = 0.0f;
    dst[3] = src[1];
    dst[4] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyMtx22Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec2Reverse(&dst[0], &src[0]);
    NWCopyVec2Reverse(&dst[4], &src[2]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx22WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec2WithHeader(&dst[0], &src[0], header);
    NWCopyVec2Reverse(&dst[5], &src[2]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx32Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec2Reverse(&dst[0], &src[0]);
    NWCopyVec2Reverse(&dst[4], &src[2]);
    NWCopyVec2Reverse(&dst[8], &src[4]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx32WithHeader(
    f32*  dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec2WithHeader(&dst[0], &src[0], header);
    NWCopyVec2Reverse(&dst[5], &src[2]);
    NWCopyVec2Reverse(&dst[9], &src[4]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx42Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec2Reverse(&dst[0], &src[0]);
    NWCopyVec2Reverse(&dst[4], &src[2]);
    NWCopyVec2Reverse(&dst[8], &src[4]);
    NWCopyVec2Reverse(&dst[12], &src[6]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx42WithHeader(
    f32*  dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec2WithHeader(&dst[0], &src[0], header);
    NWCopyVec2Reverse(&dst[5], &src[2]);
    NWCopyVec2Reverse(&dst[9], &src[4]);
    NWCopyVec2Reverse(&dst[13], &src[6]);
}

//--------------------------------------------------------------------------------
void
NWCopyVec1Reverse(
    f32*        dst,
    const f32*  src)
{
    dst[0] = 0.0f;
    dst[1] = 0.0f;
    dst[2] = 0.0f;
    dst[3] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyVec1WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    dst[0] = 0.0f;
    *(u32*)(&dst[1]) = header;
    dst[2] = 0.0f;
    dst[3] = 0.0f;
    dst[4] = src[0];
}

//--------------------------------------------------------------------------------
void
NWCopyMtx21Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec1Reverse(&dst[0], &src[0]);
    NWCopyVec1Reverse(&dst[4], &src[1]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx21WithHeader(
    f32*        dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec1WithHeader(&dst[0], &src[0], header);
    NWCopyVec1Reverse(&dst[5], &src[1]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx31Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec1Reverse(&dst[0], &src[0]);
    NWCopyVec1Reverse(&dst[4], &src[1]);
    NWCopyVec1Reverse(&dst[8], &src[2]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx31WithHeader(
    f32*  dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec1WithHeader(&dst[0], &src[0], header);
    NWCopyVec1Reverse(&dst[5], &src[1]);
    NWCopyVec1Reverse(&dst[9], &src[2]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx41Reverse(
    f32*        dst,
    const f32*  src)
{
    NWCopyVec1Reverse(&dst[0], &src[0]);
    NWCopyVec1Reverse(&dst[4], &src[1]);
    NWCopyVec1Reverse(&dst[8], &src[2]);
    NWCopyVec1Reverse(&dst[12], &src[3]);
}

//--------------------------------------------------------------------------------
void
NWCopyMtx41WithHeader(
    f32*  dst,
    const f32*  src,
    u32   header )
{
    NWCopyVec1WithHeader(&dst[0], &src[0], header);
    NWCopyVec1Reverse(&dst[5], &src[1]);
    NWCopyVec1Reverse(&dst[9], &src[2]);
    NWCopyVec1Reverse(&dst[13], &src[3]);
}

#endif


} // namespace internal
} // namespace gfx
} // namespace nw
