﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"
#include <nw/gfx/gfx_PerspectiveProjectionUpdater.h>
#include <nw/os/os_Memory.h>

namespace nw
{
namespace gfx
{

NW_UT_RUNTIME_TYPEINFO_DEFINITION(PerspectiveProjectionUpdater,CameraProjectionUpdater);

//----------------------------------------
PerspectiveProjectionUpdater*
PerspectiveProjectionUpdater::Create(os::IAllocator* allocator)
{
    NW_NULL_ASSERT(allocator);

    void* updaterMemory = allocator->Alloc(sizeof(PerspectiveProjectionUpdater));
    NW_NULL_ASSERT(updaterMemory);

    void* dataMemory
        = AllocateAndFill<ResPerspectiveProjectionUpdaterData>(allocator, 0);

    ResPerspectiveProjectionUpdaterData* buffer =
            new(dataMemory) ResPerspectiveProjectionUpdaterData();

    buffer->typeInfo = ResPerspectiveProjectionUpdater::TYPE_INFO;
    buffer->m_Near = PROJECTION_NEAR_CLIP;
    buffer->m_Far = PROJECTION_FAR_CLIP;
    buffer->m_Fovy = PROJECTION_FOVY_RADIAN;
    buffer->m_AspectRatio = PROJECTION_ASPECT_RATIO;

    ResPerspectiveProjectionUpdater resUpdater = ResPerspectiveProjectionUpdater(buffer);

    return new(updaterMemory) PerspectiveProjectionUpdater(allocator, true, resUpdater);
}

//----------------------------------------
PerspectiveProjectionUpdater*
PerspectiveProjectionUpdater::Create(
    os::IAllocator* allocator,
    ResPerspectiveProjectionUpdater resUpdater)
{
    NW_NULL_ASSERT(allocator);

    void* updaterMemory = allocator->Alloc(sizeof(PerspectiveProjectionUpdater));
    NW_NULL_ASSERT(updaterMemory);

    return new(updaterMemory) PerspectiveProjectionUpdater(allocator, false, resUpdater);
}

//----------------------------------------
PerspectiveProjectionUpdater::PerspectiveProjectionUpdater(
    os::IAllocator* allocator,
    bool isDynamic,
    ResPerspectiveProjectionUpdater resUpdater
)
: CameraProjectionUpdater(allocator, isDynamic),
  m_Resource(resUpdater)
{
}

//----------------------------------------
PerspectiveProjectionUpdater::~PerspectiveProjectionUpdater()
{
    if (this->IsDynamic() && this->m_Resource.IsValid())
    {
        this->GetAllocator().Free(m_Resource.ptr());
    }
}

//----------------------------------------
void
PerspectiveProjectionUpdater::Update(math::MTX44* projectionMatrix, math::MTX34* textureProjectionMatrix)
{
    NW_ASSERT(m_Resource.IsValid());

    float fovy = this->m_Resource.GetFovy();
    NW_ASSERT(0 < fovy && fovy < nw::math::F_PI);

    float aspect = this->m_Resource.GetAspectRatio();
    NW_ASSERT(aspect != 0);

    float near = this->m_Resource.GetNear();
    float far = this->m_Resource.GetFar();
    NW_ASSERT(near != far);

    math::MTX44PerspectivePivotRad(
        projectionMatrix,
        fovy,
        aspect,
        near,
        far,
        this->GetPivotDirection());

    math::MTX34TextureProjectionPerspective(
        textureProjectionMatrix,
        fovy,
        aspect,
        this->TextureScale().x,
        this->TextureScale().y,
        this->TextureTranslate().x,
        this->TextureTranslate().y);
}

} // namespace gfx
} // namespace nw
