﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#include "precompiled.h"

#include <nw/gfx/gfx_ShaderProgram.h>
#include <nw/gfx/gfx_ShaderUniforms.h>

namespace nw
{
namespace gfx
{

#if defined(NW_GFX_PROGRAM_OBJECT_ENABLED)

void
ShaderUniformLocation::BuildUniformLocations(GLuint programObject)
{
    m_UniformLocations.reserve(SHADER_UNIFORM_COUNT);
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "ProjMtx"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "ViewMtx"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "WrldMtx"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "NormMtx"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "PosOffs"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IrScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "TexcMap"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "TexMtx0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "TexMtx1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "TexMtx2"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "TexTran"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "MatAmbi"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "MatDiff"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "HslGCol"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "HslSCol"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "HslSDir"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "UnivReg"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsSmoSk"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsRgdSk"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsHemiL"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsHemiO"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsVertA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsBoneW"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "UvMap0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "UvMap1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "UvMap2"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsVertL"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsTex1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsTex2"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "IsQuate"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "LightCt"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].combineRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].combineRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].combineRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].combineRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].combineRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].combineRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].combineAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].combineAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].combineAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].combineAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].combineAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].combineAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].srcRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].srcRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].srcRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].srcRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].srcRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].srcRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].srcAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].srcAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].srcAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].srcAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].srcAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].srcAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].operandRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].operandRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].operandRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].operandRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].operandRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].operandRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].operandAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].operandAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].operandAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].operandAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].operandAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].operandAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].bufferInput"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].bufferInput"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].bufferInput"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].bufferInput"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].scaleRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].scaleRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].scaleRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].scaleRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].scaleRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].scaleRgb"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].scaleAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].scaleAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].scaleAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].scaleAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].scaleAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].scaleAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].constRgba"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[1].constRgba"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[2].constRgba"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[3].constRgba"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[4].constRgba"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[5].constRgba"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_TexEnv[0].bufferColor"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLighting.enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLighting.ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.samplerD0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.samplerD1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.samplerRR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.samplerRG"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.samplerRB"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.samplerFR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.emission"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentMaterial.specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].enabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].ambient"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].diffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].specular0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].specular1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].position"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].spotDirection"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].shadowed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].geomFactor0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].geomFactor1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].twoSideDiffuse"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].spotEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].distanceAttenuationEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].distanceAttenuationBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].distanceAttenuationScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].samplerSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[0].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[1].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[2].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[3].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[4].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[5].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[6].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragmentLightSource[7].samplerDA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.absLutInputD0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.absLutInputD1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.absLutInputRR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.absLutInputRG"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.absLutInputRB"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.absLutInputSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.absLutInputFR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutInputD0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutInputD1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutInputRR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutInputRG"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutInputRB"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutInputSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutInputFR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutScaleD0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutScaleD1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutScaleRR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutScaleRG"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutScaleRB"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutScaleSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutScaleFR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.shadowSelector"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.bumpSelector"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.bumpMode"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.bumpRenorm"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.config"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.invertShadow"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.shadowPrimary"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.shadowSecondary"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.shadowAlpha"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.fresnelSelector"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.clampHighlights"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutEnabledSP"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutEnabledD0"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutEnabledD1"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_LightEnv.lutEnabledRefl"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[0].perspectiveShadow"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[0].shadowZScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[0].shadowZBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[0].samplerType"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[1].samplerType"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[2].samplerType"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].samplerType"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[2].texcoord"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].texcoord"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptRgbMap"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptAlphaMap"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptAlphaSeparate"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptClampU"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptClampV"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptShiftU"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptShiftV"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptMinFilter"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptTexWidth"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptTexOffset"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptTexBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptNoiseEnable"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptNoiseU"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptNoiseV"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptSamplerRgbMap"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptSamplerAlphaMap"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptSamplerNoiseMap"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptSamplerR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptSamplerG"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptSamplerB"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Texture[3].ptSamplerA"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.lightXY"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.lightZ"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.deltaZ"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.autoAcc"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.accMax"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.shadingDensitySrc"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.colorLutInput"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.samplerTR"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.samplerTG"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.samplerTB"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Gas.attenuation"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Fog.mode"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Fog.color"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Fog.zFlip"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Fog.sampler"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.enableClippingPlane"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.clippingPlane"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.enableAlphaTest"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.alphaRefValue"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.alphaTestFunc"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.mode"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.wScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.penumbraScale"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_FragOperation.penumbraBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Point.viewport"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Point.distanceAttenuation"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Line.width"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.width"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.scaleByW"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.color"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.frontFaceCCW"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.acceptEmptyTriangles"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.openEdgeColor"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.openEdgeWidth"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.openEdgeDepthBias"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.openEdgeWidthScaleByW"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Silhouette.openEdgeDepthBiasScaleByW"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Subdivision.level"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_Subdivision.fragmentLightingEnabled"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.color"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.aspect"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.time"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.speed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.countMax"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.randSeed"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.randomCore"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.distanceAttenuation"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.viewport"));
    m_UniformLocations.push_back(glGetUniformLocation(programObject, "dmp_PartSys.pointSize"));
}

//----------------------------------------
ShaderUniformLocation::ShaderUniformLocation(os::IAllocator* allocator)
 : GfxObject(allocator)
{
}

//----------------------------------------
ShaderUniformLocation*
ShaderUniformLocation::Create(os::IAllocator* allocator)
{
    void* memory = allocator->Alloc(sizeof(ShaderUniformLocation));
    NW_NULL_ASSERT(memory);

    return new(memory) ShaderUniformLocation(allocator);
}

#endif

} // namespace gfx
} // namespace nw
