﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/gfx/gfx_Skeleton.h>

#include <nw/ut/ut_ResUtil.h>
#include <nw/ut/ut_ResDictionary.h>

namespace nw
{
namespace gfx
{

NW_UT_RUNTIME_TYPEINFO_DEFINITION(Skeleton, SceneObject);

//----------------------------------------
void
Skeleton::CreateCallbacks(os::IAllocator* allocator, int maxCallbacks, bool isFixedSizeMemory)
{
    if (isFixedSizeMemory)
    {
        if (maxCallbacks == 0)
        {
            this->m_PreCalculateMatrixSignal =
                CalculateMatrixSignal::CreateInvalidateSignal(allocator);
            this->m_PostCalculateMatrixSignal =
                CalculateMatrixSignal::CreateInvalidateSignal(allocator);
        }
        else
        {
            this->m_PreCalculateMatrixSignal =
                CalculateMatrixSignal::CreateFixedSizedSignal(maxCallbacks, allocator);
            this->m_PostCalculateMatrixSignal =
                CalculateMatrixSignal::CreateFixedSizedSignal(maxCallbacks, allocator);
        }
    }
    else
    {
        this->m_PreCalculateMatrixSignal =
            CalculateMatrixSignal::CreateVariableSizeSignal(allocator);
        this->m_PostCalculateMatrixSignal =
            CalculateMatrixSignal::CreateVariableSizeSignal(allocator);
    }
}

} // namespace gfx
} // namespace nw
