/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/anim/anim_AnimBlend.h>
#include <nw/gfx/gfx_AnimGroup.h>
#include <nw/gfx/gfx_CalculatedTransform.h>
#include <nw/gfx/gfx_TransformAnimBlendOp.h>
#include <nw/gfx/gfx_TransformAnimEvaluator.h>
#include <nw/gfx/gfx_TransformAnimOverrider.h>

namespace nw
{
namespace gfx
{

NW_UT_RUNTIME_TYPEINFO_DEFINITION(TransformAnimOverrider   , AnimOverrider);

//----------------------------------------------------------
const anim::AnimResult*
TransformAnimOverrider::GetResult(
    void* target,
    int memberIdx
) const
{
    //----------------------------------------------------------
    // Get the blend operation
    const anim::AnimBlendOp* blendOp = m_AnimGroup->GetBlendOperation(memberIdx);

    //----------------------------------------------------------
    // Raise the target's IGNORE flag, and set to unwritten state
    CalculatedTransform* transform =
        reinterpret_cast<CalculatedTransform*>(target);
    const bit32 flagsBak = transform->GetFlags();
    transform->EnableFlags(CalculatedTransform::FLAG_IS_IGNORE_ALL);

    //----------------------------------------------------------
    // Loop for all child animations
    CalculatedTransform workResult;
    bool written = false;
    for (int animIdx = m_AnimObjects.Size() - 1; animIdx >= 0; --animIdx)
    {
        const AnimObject* animObj = m_AnimObjects[animIdx];
        if (animObj == NULL)
        {
            continue;
        }

        float srcWeights[3] = { 1.0f, 1.0f, 1.0f };
        TransformAnimEvaluator::DisableSRTWeightsIfNeeded(srcWeights, animObj);

        if (!TransformAnimEvaluator::CheckWeightsNearlyZero(srcWeights))
        {
            // Evaluations
            const anim::AnimResult* childResult =
                animObj->GetResult(&workResult, memberIdx);
            if (childResult != NULL)
            {
                written = true;
                if (blendOp->Override(reinterpret_cast<anim::AnimResult*>(transform),
                    childResult)) // TRUE is returned is all components are overwritten
                {
                    break;
                }
            }
        }
    }

    if (!written) // No valid child animation
    {
        transform->RestoreFlags(CalculatedTransform::FLAG_IS_IGNORE_ALL, flagsBak);
        return NULL;
    }

    transform->UpdateScaleFlags();
    transform->UpdateRotateFlagsStrictly();
    transform->UpdateTranslateFlags();
    transform->UpdateCompositeFlags();

    // Cast AnimResult to the target and return (use with the parent's blender)
    return reinterpret_cast<anim::AnimResult*>(target);
}

} // namespace gfx
} // namespace nw
