﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "../precompiled.h"

#include <nw/gfx/res/gfx_ResUtil.h>
#include <nw/gfx/res/gfx_ResLight.h>
#include <nw/gfx/res/gfx_ResGraphicsFile.h>

namespace nw {
namespace gfx {
namespace res {

//----------------------------------------
Result
ResLight::Setup(os::IAllocator* allocator, ResGraphicsFile graphicsFile)
{
    NW_UNUSED_VARIABLE(allocator);

    Result result = RESOURCE_RESULT_OK;

    ResFragmentLight fragment = ResDynamicCast<ResFragmentLight>(*this);

    if (fragment.IsValid())
    {
        ResReferenceLookupTable referenceDistanceSampler =
            ResDynamicCast<ResReferenceLookupTable>(fragment.GetDistanceSampler());

        Result distanceResult = SetupReferenceLut(referenceDistanceSampler, graphicsFile);

        if ( fragment.GetDistanceSampler().IsValid() )
        {
            if (distanceResult.IsSuccess())
            {
                result |= fragment.GetDistanceSampler().Setup();
            }
        }

        result |= distanceResult;

        if (fragment.GetAngleSampler().IsValid())
        {
            ResLookupTable resLut = fragment.GetAngleSampler().GetSampler();

            if (resLut.IsValid())
            {
                ResReferenceLookupTable referenceAngleSampler =
                    ResDynamicCast<ResReferenceLookupTable>(resLut);

                Result angleResult = SetupReferenceLut(referenceAngleSampler, graphicsFile);

                if (angleResult.IsSuccess())
                {
                    result |= resLut.Setup();
                }

                result |= angleResult;
            }
            else
            {
                result |= RESOURCE_RESULT_IRRELEVANT_LOCATION_LUT;
            }
        }
    }

    return result;
}

//----------------------------------------
void
ResLight::Cleanup()
{
    ResFragmentLight fragment = ResDynamicCast<ResFragmentLight>(*this);

    if (fragment.IsValid())
    {
        ut::SafeCleanup(fragment.GetDistanceSampler());
        ut::SafeCleanup(fragment.GetAngleSampler());
    }
}

} /* namespace res */
} /* namespace gfx */
} /* namespace nw */

