/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_ARC_H_
#define NW_LYT_ARC_H_

#if defined(_MSC_VER) && _MSC_VER >= 1500
#pragma once
#endif

#include <nn/types.h>
#include <nw/types.h>
#include <nw/lyt/lyt_Types.h>

namespace nw
{
namespace lyt
{

//
//
//
//
typedef struct
{
    u32             signature;              //
    u16             byteOrder;              //
    u16             headerSize;             //
    u32             version;                //
    u32             fileSize;               //
    s32             fstStart;				//
    s32             fstSize;				//
    s32             fileStart;				//

} ARCHeader;

//
const u32       DARCH_SIGNATURE         = 0x63726164;   // 'darc'

//
const u16       DARCH_BYTE_ORDER_MARK   = 0xFEFF;

//
const u32       DARCH_VERSION           = 0x01000000;

//
//
//
//
typedef struct
{
    void*       archiveStartAddr;
    void*       FSTStart;
    void*       fileStart;
    u32         entryNum;
    wchar_t*    FSTStringStart;
    u32         FSTLength;
    u32         currDir;

} ARCHandle;

//
//
//
//
typedef struct
{
    ARCHandle*  handle;
    u32         startOffset;
    u32         length;

} ARCFileInfo;

//
//
//
//
typedef struct
{
    ARCHandle*  handle;
    u32         entryNum;
    u32         location;
    u32         next;
} ARCDir;

//
//
//
//
typedef struct
{
    ARCHandle*  handle;
    u32         entryNum;
    wchar_t*    name;
    u8          isDir;
    u8          padding_[3];
} ARCDirEntry;

//
//

//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCInitHandle(void* arcStart, ARCHandle* handle);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCOpen(ARCHandle* handle, const wchar_t* fileName, ARCFileInfo* af);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCFastOpen(ARCHandle* handle, s32 entrynum, ARCFileInfo* af);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
NW_INLINE bool ARCOpen(ARCHandle* handle, ARCDirEntry* dirent, ARCFileInfo* af)
{
    NW_NULL_ASSERT(dirent);
    return ARCFastOpen(handle, static_cast<s32>(dirent->entryNum), af);
}

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
s32   ARCConvertPathToEntrynum(ARCHandle* handle, const wchar_t* pathPtr);

//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCEntrynumIsDir(const ARCHandle * handle, s32 entrynum);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
void* ARCGetStartAddrInMem(ARCFileInfo* af);

//
//
//
//
//
//
//
//
//
//
u32   ARCGetStartOffset(ARCFileInfo* af);

//
//
//
//
//
//
//
//
//
//
//
u32   ARCGetLength(ARCFileInfo* af);

//
//
//
//
//
//
//
//
//
bool  ARCClose(ARCFileInfo* af);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCChangeDir(ARCHandle* handle, const wchar_t* dirName);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCChangeDir(ARCHandle* handle, s32 entrynum);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
NW_INLINE bool ARCChangeDir(ARCHandle* handle, ARCDir* dir)
{
    NW_NULL_ASSERT(dir);
    return ARCChangeDir(handle, static_cast<s32>(dir->entryNum));
}

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
NW_INLINE bool ARCChangeDir(ARCHandle* handle, ARCDirEntry* dirent)
{
    NW_NULL_ASSERT(dirent);
    return ARCChangeDir(handle, static_cast<s32>(dirent->entryNum));
}

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCGetCurrentDir(ARCHandle* handle, wchar_t* path, u32 maxlen);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCOpenDir(ARCHandle* handle, const wchar_t* dirName, ARCDir* dir);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool ARCOpenDir(ARCHandle* handle, s32 entrynum, ARCDir* dir);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
NW_INLINE bool ARCOpenDir(ARCHandle* handle, ARCDirEntry* dirent, ARCDir* dir)
{
    NW_NULL_ASSERT(dirent);
    return ARCOpenDir(handle, static_cast<s32>(dirent->entryNum), dir);
}

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
bool  ARCReadDir(ARCDir* dir, ARCDirEntry* dirent);

//
//
//
//
//
//
//
//
//
bool  ARCCloseDir(ARCDir* dir);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
NW_INLINE u32 ARCTellDir(ARCDir* dir)
{
    return dir->location;
}

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
NW_INLINE void ARCSeekDir(ARCDir* dir, u32 loc)
{
    dir->location = loc;
}

//
//
//
//
//
//
NW_INLINE void ARCRewindDir(ARCDir* dir)
{
    dir->location = dir->entryNum + 1;
}

//
//
//
//
//
//
//
//
//
//
//
NW_INLINE const wchar_t* ARCGetDirEntryName(ARCDirEntry* dirent)
{
    return dirent->name;
}

//
//
//
//
//
//
//
//
NW_INLINE s32 ARCGetDirEntryEntrynum(ARCDirEntry* dirent)
{
    return static_cast<s32>(dirent->entryNum);
}

//
//
//
//
//
//
//
//
//
NW_INLINE bool ARCDirEntryIsDir(ARCDirEntry* dirent)
{
    return dirent->isDir != 0;
}

// @}

} // namespace lyt
} // namespace nw

#endif //  NW_LYT_ARC_H_
