/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_ARCRESOURCEACCESSOR_H_
#define NW_LYT_ARCRESOURCEACCESSOR_H_

#include <nw/types.h>

#include <nw/ut/ut_LinkList.h>
#include <nw/lyt/lyt_FontContainer.h>
#include <nw/lyt/lyt_ResourceAccessor.h>
#include <nw/lyt/lyt_TextureContainer.h>
#include <nw/lyt/lyt_Arc.h>
#include <cstddef>

namespace nw
{
namespace lyt
{

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class ArcResourceAccessor : public ResourceAccessor
{
public:
    //
    static const int ROOTPATH_MAX = 64;

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    ArcResourceAccessor();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool Attach(
        void* archiveStart,
        const char* resourceRootDirectory
    );

    //
    //
    //
    //
    //
    //
    void* Detach();

    virtual void* GetResource(
        ResType resType,
        const char* name,
        u32* pSize = 0
    );

    //
    //
    //
    //
    //
    //
    bool IsAttached() const
    {
        return m_ArcBuf != 0;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    const void* GetArchiveDataStart() const
    {
        return m_ArcBuf;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    FontKey RegistFont(const char* name, font::Font* pFont);

    //
    //
    //
    //
    //
    //
    //
    //
    void UnregistFont(FontKey key);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    TextureKey RegistTexture(const char* name, const TextureInfo& textureInfo);

    //
    //
    //
    //
    //
    //
    //
    //
    void UnregistTexture(TextureKey key);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual const TextureInfo GetTexture(const char* name);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual font::Font* GetFont(const char* name);

    //
    //
    //
    //
    //
    //
    //
    TextureContainer& GetTextureContainer()
    {
        return m_TextureList;
    }

    //
    //
    //
    //
    //
    //
    //
    const TextureContainer& GetTextureContainer() const
    {
        return m_TextureList;
    }

    //
    //
    //
    //
    //
    //
    //
    FontContainer& GetFontContainer()
    {
        return m_FontList;
    }

    //
    //
    //
    //
    //
    //
    //
    const FontContainer& GetFontContainer() const
    {
        return m_FontList;
    }

    //

protected:
    //
    ARCHandle m_ArcHandle;

    //
    void* m_ArcBuf;

    //
    FontContainer m_FontList;

    //
    TextureContainer m_TextureList;

    //
    wchar_t m_ResRootDir[ROOTPATH_MAX];

    //
    wchar_t m_ResNameWork[ROOTPATH_MAX];
};

class MultiArcResourceAccessor;

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class ArcResourceLink
{
    friend class MultiArcResourceAccessor;

public:
    //
    static const int ROOTPATH_MAX = ArcResourceAccessor::ROOTPATH_MAX;

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    ArcResourceLink()
    {
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool Set(
        void* archiveStart,
        const char* resourceRootDirectory
    );

    //
    //
    //
    //
    //
    //
    const void* GetArchiveDataStart() const;

    //
    //
    //
    //
    //
    //
    const wchar_t* GetResRootDir() const
    {
        return m_ResRootDir;
    }

    //

    //
    //
    ut::LinkListNode m_Link;

protected:
    //
    ARCHandle* GetArcHandle()
    {
        return &m_ArcHandle;
    }

    //
    ARCHandle m_ArcHandle;

    //
    wchar_t m_ResRootDir[ROOTPATH_MAX];
};

namespace internal
{

typedef ut::LinkList<ArcResourceLink, offsetof(ArcResourceLink, m_Link)> ArcResourceList;

}   // namespace internal

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class MultiArcResourceAccessor : public ResourceAccessor
{
public:
    //
    static const int ROOTPATH_MAX = ArcResourceAccessor::ROOTPATH_MAX;

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    MultiArcResourceAccessor();

    //
    //
    //
    //
    virtual ~MultiArcResourceAccessor();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Attach(ArcResourceLink* pLink);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    ArcResourceLink* Detach(const void* archiveStart);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Detach(ArcResourceLink* pLink);

    //
    //
    //
    //
    //
    //
    void DetachAll();

    virtual void* GetResource(
        ResType resType,
        const char* name,
        u32* pSize = 0
    );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    FontKey RegistFont(const char* name, font::Font* pFont);

    //
    //
    //
    //
    //
    //
    //
    //
    void UnregistFont(FontKey key);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    TextureKey RegistTexture(const char* name, const TextureInfo& textureInfo);

    //
    //
    //
    //
    //
    //
    //
    //
    void UnregistTexture(TextureKey key);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual const TextureInfo GetTexture(const char* name);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual font::Font* GetFont(const char* name);

    //
    //
    //
    //
    //
    //
    //
    TextureContainer& GetTextureContainer()
    {
        return m_TextureList;
    }

    //
    //
    //
    //
    //
    //
    //
    const TextureContainer& GetTextureContainer() const
    {
        return m_TextureList;
    }

    //
    //
    //
    //
    //
    //
    //
    FontContainer& GetFontContainer()
    {
        return m_FontList;
    }

    //
    //
    //
    //
    //
    //
    //
    const FontContainer& GetFontContainer() const
    {
        return m_FontList;
    }

    //

protected:
    //
    internal::ArcResourceList m_ArcList;

    //
    FontContainer m_FontList;

    //
    TextureContainer m_TextureList;

    //
    wchar_t m_ResNameWork[ROOTPATH_MAX];
};

}   // namespace lyt
}   // namespace nw

/* NW_LYT_ARCRESOURCEACCESSOR_H_ */
#endif
