/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_DRAWINFO_H_
#define NW_LYT_DRAWINFO_H_

#include <nw/math/math_Types.h>
#include <nw/font/font_WideTextWriter.h>
#include <nw/ut/ut_Rect.h>
#include <nw/lyt/lyt_Types.h>

namespace nw
{
namespace lyt
{

class GraphicsResource;
class Layout;

//---------------------------------------------------------------------------
//
//
//
//
//
//
//
//
//
//
//
//
//
//---------------------------------------------------------------------------
class DrawInfo
{
public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    DrawInfo();

    //
    //
    //
    //
    virtual ~DrawInfo();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    const math::MTX34&  GetViewMtx() const
    {
        return m_ViewMtx;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetViewMtx(const math::MTX34& value)
    {
        m_ViewMtx = value;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    bool IsInfluencedAlpha() const
    {
        return m_Flag.influencedAlpha;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetInfluencedAlpha(bool bEnable)
    {
        m_Flag.influencedAlpha = bEnable;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    bool IsLocationAdjust() const
    {
        return m_Flag.locationAdjust;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetLocationAdjust(bool bEnable)
    {
        m_Flag.locationAdjust = bEnable;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    const math::VEC2& GetLocationAdjustScale() const
    {
        return m_LocationAdjustScale;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetLocationAdjustScale(const math::VEC2& scale)
    {
        m_LocationAdjustScale = scale;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    f32 GetGlobalAlpha() const
    {
        return m_GlobalAlpha;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetGlobalAlpha(f32 alpha)
    {
        m_GlobalAlpha = alpha;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    bool IsInvisiblePaneCalculateMtx() const
    {
        return m_Flag.invisiblePaneCalculateMtx;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetInvisiblePaneCalculateMtx(bool bEnable)
    {
        m_Flag.invisiblePaneCalculateMtx = bEnable;
    }

#if ! defined(NW_RELEASE)
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool IsDebugDrawMode() const
    {
        return m_Flag.debugDrawMode;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetDebugDrawMode(bool bEnable)
    {
        m_Flag.debugDrawMode = bEnable;
    }
#endif

    //
    //
    //
    //
    //
    //
    //
    //
    GraphicsResource* GetGraphicsResource() const
    {
        return m_pGraphicsResource;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetGraphicsResource(GraphicsResource* pGraphicsResource)
    {
        m_pGraphicsResource = pGraphicsResource;
    }

    //
    //
    //
    //
    //
    //
    void SetProjectionMtx(const nw::math::MTX44& mtx);

    //

    //
    //
    //
    //
    //
    //
    //
    //
    Layout* GetLayout() const
    {
        return m_pLayout;
    }

    //
    //
    void SetLayout(Layout* pLayout) const
    {
        m_pLayout = pLayout;
    }

    //
    //
    const nw::math::MTX44& GetProjectionMtx() const
    {
        return m_ProjMtx;
    }

protected:
    //
    math::MTX44 m_ProjMtx;

    //
    math::MTX34 m_ViewMtx;

    //
    math::VEC2 m_LocationAdjustScale;

    //
    f32 m_GlobalAlpha;

    //
    GraphicsResource* m_pGraphicsResource;

    //
    mutable Layout* m_pLayout;

    //
    //
    struct Flag
    {
        u8 influencedAlpha : 1;
        u8 locationAdjust : 1;
        u8 invisiblePaneCalculateMtx : 1;
#if ! defined(NW_RELEASE)
        u8 debugDrawMode : 1;
#endif

        u8 padding_[3];
    };

    //
    Flag m_Flag;
};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_DRAWINFO_H_

