/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_FONTCONTAINER_H_
#define NW_LYT_FONTCONTAINER_H_

#include <nw/types.h>

#include <nw/ut/ut_LinkList.h>
#include <nw/lyt/lyt_ResourceAccessor.h>
#include <cstddef>

namespace nw
{
namespace lyt
{

//---------------------------------------------------------------------------
//
//
//
//
//
//
//
//---------------------------------------------------------------------------
class FontRefLink
{
public:
    //
    static const int FONTNAMEBUF_MAX = 128;

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    FontRefLink();

    //
    //
    //
    //
    //
    //
    //
    //
    ~FontRefLink();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Set(
        const char* name,
        font::Font* pFont,
        bool own
    );

    //
    //
    //
    //
    //
    //
    const char* GetFontName() const
    {
        return m_FontName;
    }

    //
    //
    //
    //
    //
    //
    font::Font* GetFont() const
    {
        return m_pFont;
    }

    //

    //
    //
    ut::LinkListNode m_Link;

protected:
    //
    char m_FontName[FONTNAMEBUF_MAX];

    //
    font::Font* m_pFont;

    //
    u8 m_Own;

    //
    u8 padding[3];
};

//---------------------------------------------------------------------------
//
//
//
//
//
//
//
//
//
//
//---------------------------------------------------------------------------
class FontContainer : public ut::LinkList<FontRefLink, offsetof(FontRefLink, m_Link)>
{
public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    ~FontContainer();

    // @}

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    void Finalize();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    FontKey RegistFont(const char* name, font::Font* pFont, bool own);

    //
    //
    //
    //
    //
    //
    void UnregistFont(FontKey key);

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    nw::font::Font* FindFontByName(
        const char* name
    );

    //
    //
    //
    //
    //
    //
    //
    //
    nw::font::Font* FindFontByKey(
        FontKey key
    );

    //
};

}   // namespace lyt
}   // namespace nw

/* NW_LYT_FONTCONTAINER_H_ */
#endif
