/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_MATERIAL_H_
#define NW_LYT_MATERIAL_H_

//#include <revolution/tpl.h>

#include <nw/ut/ut_Color.h>
#include <nw/math/math_Types.h>

#include <nw/lyt/lyt_Config.h>
#include <nw/lyt/lyt_Drawer.h>
#include <nw/lyt/lyt_TexMap.h>
#include <nw/lyt/lyt_Types.h>

namespace nw
{
namespace lyt
{
namespace res
{

struct Material;

} // namespace nw::lyt::res

namespace internal
{

// Matrix Member Numbers
struct MatMemNums
{
    u32 texMap: 2;
    u32 texSRT: 2;
    u32 texCoordGen: 2;
    u32 tevStage: 3;
    u32 alpComp: 1;
    u32 blendMode: 1;
    u32 padding_: 21;
};

} // namespace nw::lyt::internal

class AnimTransform;
class AnimResource;
class DrawInfo;
class Layout;
class TexMap;
struct ResBlockSet;

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class Material
{
public:
    enum
    {
        //
        FLAG_USER_ALLOCATED,
        //
        FLAG_TEXTURE_ONLY,
        //
        FLAG_TEXTURE_CLEAN,
        //
        FLAG_MAX
    };

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    Material();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Material(
        const res::Material* pRes,
        const ResBlockSet& resBlockSet);

    //
    //
    //
    //
    virtual ~Material();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void ReserveMem(
        u8 texMapNum,
        u8 texSRTNum,
        u8 texCoordGenNum,
        u8 tevStageNum = 0,
        bool allocAlpComp = false,
        bool allocBlendMode = false);

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexMapCap() const
    {
        return u8(m_MemCap.texMap);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexSRTCap() const
    {
        return u8(m_MemCap.texSRT);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexCoordGenCap() const
    {
        return u8(m_MemCap.texCoordGen);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTevStageCap() const
    {
        return u8(m_MemCap.tevStage);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    bool IsAlphaCompareCap() const
    {
        return m_MemCap.alpComp != 0;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    bool IsBlendModeCap() const
    {
        return m_MemCap.blendMode != 0;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    const char* GetName() const
    {
        return m_Name;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetName(const char* name);

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexMapNum() const
    {
        return u8(m_MemNum.texMap);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexMapNum(u8 num);

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexCoordGenNum() const
    {
        return u8(m_MemNum.texCoordGen);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexCoordGenNum(u8 num);

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexSRTNum() const
    {
        return u8(m_MemNum.texSRT);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexSRTNum(u8 num);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const TexMap& GetTexMap(u32 idx) const
    {
        NW_ASSERT(idx < m_MemNum.texMap);
        return GetTexMapAry()[idx];
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexMap(u32 idx, const TexMap& value)
    {
        NW_ASSERT(idx < m_MemNum.texMap);
        GetTexMapAry()[idx] = value;
        this->SetTextureDirty();
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexMap(u32 idx, const TextureInfo& value)
    {
        NW_ASSERT(idx < m_MemNum.texMap);
        GetTexMapAry()[idx].Set(value);
        this->SetTextureDirty();
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const TexSRT& GetTexSRT(u32 idx) const
    {
        NW_ASSERT(idx < m_MemNum.texSRT);
        return GetTexSRTAry()[idx];
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexSRT(u32 idx, const TexSRT& value)
    {
        NW_ASSERT(idx < m_MemNum.texSRT);
        GetTexSRTAry()[idx] = value;
        this->SetTextureDirty();
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const TexCoordGen& GetTexCoordGen(u32 idx) const
    {
        NW_ASSERT(idx < m_MemNum.texCoordGen);
        return GetTexCoordGenAry()[idx];
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexCoordGen(u32 idx, const TexCoordGen& value)
    {
        NW_ASSERT(idx < m_MemNum.texCoordGen);
        GetTexCoordGenAry()[idx] = value;
        this->SetTextureDirty();
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    f32 GetTexSRTElement(u32 idx, u32 eleIdx) const
    {
        NW_ASSERT(idx < m_MemNum.texSRT);

        const f32 *const srtAry = &GetTexSRTAry()[idx].translate.x;
        return srtAry[eleIdx];
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexSRTElement(u32 idx, u32 eleIdx, f32 value)
    {
        NW_ASSERT(idx < m_MemNum.texSRT);

        f32 *const srtAry = &GetTexSRTAry()[idx].translate.x;
        srtAry[eleIdx] = value;
        this->SetTextureDirty();
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTevStageNum() const
    {
        return u8(m_MemNum.tevStage);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetTevStageNum(u8 num);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const ut::Color8& GetColor(u32 idx) const
    {
        NW_ASSERT(idx < MatColorMax);
        return m_Colors[idx];
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetColor(u32 idx, ut::Color8 value)
    {
        NW_ASSERT(idx < MatColorMax);
        m_Colors[idx] = value;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetColorElement(u32 colorType) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetColorElement(u32 colorType, u8 value);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const TevStage& GetTevStage(u32 idx) const
    {
        NW_ASSERT(idx < m_MemNum.tevStage);
        return GetTevStageAry()[idx];
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTevStage(u32 idx, const TevStage& value)
    {
        NW_ASSERT(idx < m_MemNum.tevStage);
        GetTevStageAry()[idx] = value;
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    const AlphaCompare& GetAlphaCompare() const
    {
        NW_ASSERT(IsAlphaCompareCap());
        return *GetAlphaComparePtr();
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetAlphaCompare(AlphaCompare value)
    {
        NW_ASSERT(IsAlphaCompareCap());
        *GetAlphaComparePtr() = value;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    const BlendMode& GetBlendMode() const
    {
        NW_ASSERT(IsBlendModeCap());
        return *GetBlendModePtr();
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetBlendMode(BlendMode value)
    {
        NW_ASSERT(IsBlendModeCap());
        *GetBlendModePtr() = value;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    bool GetTextureOnly() const
    {
        return internal::TestBit(m_Flag, FLAG_TEXTURE_ONLY);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTextureOnly(bool value)
    {
        m_Flag = internal::SetBit(m_Flag, FLAG_TEXTURE_ONLY, value);
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void BindAnimation(AnimTransform* pAnimTrans);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void UnbindAnimation(AnimTransform*  pAnimTrans);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void UnbindAllAnimation();

    //
    //
    //
    //
    virtual void Animate();

    //
    //
    //
    //
    //
    //
    void AddAnimationLink(AnimationLink* pAnimationLink);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual AnimationLink* FindAnimationLink(AnimTransform* pAnimTrans);

    //
    //
    //
    //
    //
	//
	//
	//
	//
	//
	//
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual AnimationLink* FindAnimationLink(const AnimResource& animRes);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetAnimationEnable(
        AnimTransform* pAnimTrans,
        bool bEnable);

    //
    //
    //
    //
    //
	//
	//
	//
	//
	//
	//
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetAnimationEnable(
        const AnimResource& animRes,
        bool bEnable);

    //
    //
    //
    //
    //
    //
    //
    //
    const AnimationList& GetAnimationList() const
    {
        return m_AnimList;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    AnimationList& GetAnimationList()
    {
        return m_AnimList;
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    bool IsUserAllocated() const
    {
        return internal::TestBit(m_Flag, FLAG_USER_ALLOCATED);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetUserAllocated()
    {
        internal::SetBit(&m_Flag, FLAG_USER_ALLOCATED, true);
    }

    //

#ifdef NW_LYT_DMPGL_ENABLED
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetupGraphics(
        const DrawInfo& drawInfo,
        u8 alpha,
        bool bInitFrameTransform = true);
#endif

    //
    bool IsTextureDirty() const
    {
        return ! internal::TestBit(m_Flag, FLAG_TEXTURE_CLEAN);
    }

    //
    void SetTextureDirty(bool bDirty = true)
    {
        internal::SetBit(&m_Flag, FLAG_TEXTURE_CLEAN, ! bDirty);
    }

protected:
    //
    //
    //
    //
    //
    //
    const TexMap* GetTexMapAry() const;

    //
    //
    //
    //
    //
    //
    TexMap* GetTexMapAry();

    //
    //
    //
    //
    //
    //
    const TexSRT* GetTexSRTAry() const;

    //
    //
    //
    //
    //
    //
    TexSRT* GetTexSRTAry();

    //
    //
    //
    //
    //
    //
    const TexCoordGen* GetTexCoordGenAry() const;

    //
    //
    //
    //
    //
    //
    TexCoordGen* GetTexCoordGenAry();

    //
    //
    //
    //
    //
    //
    const AlphaCompare* GetAlphaComparePtr() const;

    //
    //
    //
    //
    //
    //
    AlphaCompare* GetAlphaComparePtr();

    //
    //
    //
    //
    //
    //
    const BlendMode* GetBlendModePtr() const;

    //
    //
    //
    //
    //
    //
    BlendMode* GetBlendModePtr();

    //
    //
    //
    //
    //
    //
    const TevStage* GetTevStageAry() const;

    //
    //
    //
    //
    //
    //
    TevStage* GetTevStageAry();

protected:
    //
    void Init();

    //
    void InitMatMemNums(internal::MatMemNums* ptr);

    //
    AnimationList m_AnimList;

    //
    ut::Color8 m_Colors[MatColorMax];

    //
    internal::MatMemNums m_MemCap;

    //
    internal::MatMemNums m_MemNum;

    //
    void* m_pMem;

    //
    char m_Name[MaterialNameStrMax + 1];

    //
    u8 m_Flag;

    //
    u8 m_Padding[2];

private:
    // Prohibits calling the copy constructor and assignment operator. This must not be implemented.
    Material(const Material& other);
    Material& operator = (const Material& other);

};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_MATERIAL_H_
