/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_PICTURE_H_
#define NW_LYT_PICTURE_H_

#include <nw/ut/ut_Color.h>
#include <nw/ut/ut_RuntimeTypeInfo.h>

#include <nw/lyt/lyt_Config.h>
#include <nw/lyt/lyt_Material.h>
#include <nw/lyt/lyt_Pane.h>

namespace nw
{
namespace lyt
{

class DrawInfo;

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class Picture : public Pane
{
    typedef Pane Base;

public:
    //
    NW_UT_RUNTIME_TYPEINFO;

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    explicit Picture(u8 texNum);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    explicit Picture(const TexMap& texMap);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    explicit Picture(const TextureInfo& texInfo);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Picture(
        const res::Picture* pBlock,
        const ResBlockSet& resBlockSet);

    //
    //
    //
    //
    virtual ~Picture();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    virtual void Append(const TexMap& texMap);

    //
    //
    //
    //
    //
    //
    virtual void Append(const TextureInfo& texInfo)
    {
        this->Append(TexMap(texInfo));
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual const ut::Color8 GetVtxColor(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetVtxColor(u32 idx, ut::Color8 value);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u8 GetVtxColorElement(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetVtxColorElement(u32 idx, u8 value);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void ReserveTexCoord(u8 num);

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexCoordNum() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexCoordNum(u8 num);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void GetTexCoord(u32 idx, TexCoordQuad coords) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexCoord(u32 idx, const TexCoordQuad coords);

    using Base::GetMaterial;

    //
    //
    //
    //
    //
    //
    virtual u8 GetMaterialNum() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual Material* GetMaterial(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetMaterial(Material* pMaterial);

    //

#ifdef NW_LYT_DRAWER_ENABLE
public:

    // Virtual implementation is written in lyt_Drawer.cpp.
    virtual u32* MakeUniformDataSelf( u32* pCmdStream, DrawInfo* pDrawInfo, Drawer* pDrawer ) const;

protected: // Drawer is used to reduce processing.

    //
    //
    mutable u8 m_IsTexCoordInited;

    //
    u8 padding[3];

    //
    //
    mutable u32 m_UniformTexCoordNum;

    //
    //
    mutable nw::math::VEC4 m_UniformTexCoords[ TexMapMax * 2 ];

#endif

protected:
#ifdef NW_LYT_DMPGL_ENABLED
    virtual void DrawSelf(const DrawInfo& drawInfo);
#endif

    //
    void Init(u8 texNum);

protected:
    //
    Material* m_pMaterial;

    //
    ut::Color8 m_VtxColors[VERTEXCOLOR_MAX];

    //
    internal::TexCoordAry m_TexCoordAry;

private:
    // Prohibits calling the copy constructor and assignment operator. This must not be implemented.
    Picture(const Picture& other);
    Picture& operator = (const Picture& other);
};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_PICTURE_H_

