/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_TEXMAP_H_
#define NW_LYT_TEXMAP_H_

#include <nw/types.h>
#include <nw/lyt/lyt_Types.h>

#include <nw/lyt/lyt_Drawer.h>

namespace nw
{
namespace lyt
{

namespace res
{
    struct Image;
}

class TextureInfo;

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class TexMap
{
public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    TexMap();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    explicit TexMap(const TextureInfo& textureInfo);

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    u32 GetTextureObject() const
    {
        return m_TexObject;
    }

    //
    //
    //
    //
    //
    //
    void SetTextureObject(u32 texObject)
    {
        m_TexObject = texObject;

#ifdef NW_LYT_DRAWER_ENABLE
        ResetU32Info();
#endif
    }

    //
    //
    //
    //
    //
    //
    TexWrap GetWrapModeS() const
    {
        return TexWrap(m_Bits.wrapS);
    }

    //
    //
    //
    //
    //
    //
    TexWrap GetWrapModeT() const
    {
        return TexWrap(m_Bits.wrapT);
    }

    //
    //
    //
    //
    //
    //
    //
    void SetWrapMode(TexWrap wrapS, TexWrap wrapT)
    {
        NW_ASSERT(wrapS < TEXWRAP_MAX);
        NW_ASSERT(wrapT < TEXWRAP_MAX);

        m_Bits.wrapS = wrapS;
        m_Bits.wrapT = wrapT;

#ifdef NW_LYT_DRAWER_ENABLE
        ResetU32Info();
#endif
    }

    //
    //
    //
    //
    //
    //
    TexFilter GetMinFilter() const
    {
        return TexFilter(m_Bits.minFilter);
    }

    //
    //
    //
    //
    //
    //
    TexFilter GetMagFilter() const
    {
        return TexFilter(m_Bits.magFilter);
    }

    //
    //
    //
    //
    //
    //
    //
    void SetFilter(TexFilter minFlt, TexFilter magFlt)
    {
        NW_ASSERT(minFlt <= TEXFILTER_MAX);
        NW_ASSERT(magFlt <= TEXFILTER_MAX);

        m_Bits.minFilter = minFlt;
        m_Bits.magFilter = magFlt;

#ifdef NW_LYT_DRAWER_ENABLE
        ResetU32Info();
#endif
    }

    //
    //
    //
    //
    //
    //
    u16 GetWidth() const
    {
        return m_Width;
    }

    //
    //
    //
    //
    //
    //
    u16 GetHeight() const
    {
        return m_Height;
    }

    //
    //
    //
    //
    //
    //
    const TexSize GetSize() const
    {
        return TexSize(m_Width, m_Height);
    }

    //
    //
    //
    //
    //
    //
    //
    void SetSize(u16 width, u16 height)
    {
        m_Width = width;
        m_Height = height;

#ifdef NW_LYT_DRAWER_ENABLE
        ResetU32Info();
#endif
    }

    //
    //
    //
    void SetSize(const TexSize& size)
    {
        SetSize(size.width, size.height);
    }

    //
    //
    //
    //
    //
    //
    u16 GetRealWidth() const
    {
        return m_RealWidth;
    }

    //
    //
    //
    //
    //
    //
    u16 GetRealHeight() const
    {
        return m_RealHeight;
    }

    //
    //
    //
    //
    //
    //
    const TexSize GetRealSize() const
    {
        return TexSize(m_RealWidth, m_RealHeight);
    }

    //
    //
    //
    //
    //
    //
    //
    void SetRealSize(u16 width, u16 height)
    {
        m_RealWidth = width;
        m_RealHeight = height;

#ifdef NW_LYT_DRAWER_ENABLE
        ResetU32Info();
#endif
    }

    //
    //
    //
    //
    //
    //
    void SetRealSize(const TexSize& size)
    {
        SetRealSize(size.width, size.height);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetPhysicalAddress(uptr physicalAddress)
    {
        m_PhysicalAddress = physicalAddress;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    uptr GetPhysicalAddress() const
    {
        return m_PhysicalAddress;
    }

    //
    //
    //
    //
    //
    //
    TexFormat GetFormat() const
    {
        return TexFormat(m_Bits.format);
    }

    //
    //
    //
    //
    //
    //
    void SetFormat(TexFormat format)
    {
        m_Bits.format = format;

#ifdef NW_LYT_DRAWER_ENABLE
        ResetU32Info();
#endif
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Set(const TextureInfo& src)
    {
        m_TexObject = src.GetTextureObject();
        m_PhysicalAddress = src.GetPhysicalAddress();
        m_Width = src.GetSize().width;
        m_Height = src.GetSize().height;
        m_RealWidth = src.GetRealSize().width;
        m_RealHeight = src.GetRealSize().height;
        m_Bits.format = src.GetFormat();

#ifdef NW_LYT_DRAWER_ENABLE
        ResetU32Info();
#endif
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Set(const TexMap& texMap)
    {
        *this = texMap;
    }

    //

#ifdef NW_LYT_DRAWER_ENABLE
public:
    //
    inline u32 GetU32WrapFilter() const
    {
        return m_u32WrapFilter;
    }

    //
    inline u32 GetU32WidthHeight() const
    {
        return m_u32WidthHeight;
    }

    //
    inline u32 GetU32Format() const
    {
        return m_u32Format;
    }

protected:
    //
    void ResetU32Info();
#endif

protected:
    //
    u32 m_TexObject;

    //
    uptr m_PhysicalAddress;

    //
    u16 m_Width;

    //
    u16 m_Height;

    //
    u16 m_RealWidth;

    //
    u16 m_RealHeight;

    //
    struct Bits
    {
        u32 wrapS: 2;
        u32 wrapT: 2;
        u32 minFilter: 3;
        u32 magFilter: 1;
        u32 format: 4;
        u32 padding_: 20;
    };

    //
    Bits m_Bits;

#ifdef NW_LYT_DRAWER_ENABLE
    //
    u32 m_u32WrapFilter;

    //
    u32 m_u32WidthHeight;

    //
    u32 m_u32Format;
#endif
};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_TEXMAP_H_
