/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/lyt/lyt_Common.h>
#include <nw/lyt/lyt_Group.h>
#include <nw/lyt/lyt_Layout.h>

namespace nw
{
namespace lyt
{

Group::Group()
{
    Init();

    std::memset(m_Name, 0, sizeof(m_Name));
}

Group::Group(
    const res::Group* pResGroup,
    Pane* pRootPane
)
{
    Init();

    ut::strcpy(m_Name, sizeof(m_Name), pResGroup->name);

    const char *const paneNameBase = internal::ConvertOffsToPtr<char>(pResGroup, sizeof(res::Group));

    for (int i = 0; i < pResGroup->paneNum; ++i)
    {
        if (Pane* pFindPane = pRootPane->FindPaneByName(paneNameBase + i * ResourceNameStrMax, true))
        {
            AppendPane(pFindPane);
        }
    }
}

void
Group::Init()
{
    m_UserAllocated = false;
}

Group::~Group()
{
    // OSReport("Group::~Group()\n");

    for (PaneLinkList::Iterator it = m_PaneLinkList.GetBeginIter(); it != m_PaneLinkList.GetEndIter();)
    {
        PaneLinkList::Iterator currIt = it++;
        m_PaneLinkList.Erase(currIt);
        Layout::DeleteObj(&(*currIt));
    }
}

void
Group::AppendPane(Pane* pPane)
{
    if (PaneLink* pPaneLink = Layout::NewObj<PaneLink>())
    {
        pPaneLink->target = pPane;
        m_PaneLinkList.PushBack(pPaneLink);
    }
}

GroupContainer::~GroupContainer()
{
    // OSReport("GroupContainer::~GroupContainer()\n");

    // Group's post-processing
    for (GroupList::Iterator it = m_GroupList.GetBeginIter(); it != m_GroupList.GetEndIter();)
    {
        GroupList::Iterator currIt = it++;
        m_GroupList.Erase(currIt);
        if (! currIt->IsUserAllocated())
        {
            Layout::DeleteObj(&(*currIt));
        }
    }
}

void
GroupContainer::AppendGroup(Group* pGroup)
{
    m_GroupList.PushBack(pGroup);
}

Group*
GroupContainer::FindGroupByName(const char* findName)
{
    // Search whether there are matches among the children
    for (GroupList::Iterator it = m_GroupList.GetBeginIter(); it != m_GroupList.GetEndIter(); ++it)
    {
        if (internal::EqualsResName(it->GetName(), findName))
        {
            return &(*it);
        }
    }

    return 0;
}

} // namespace nw::lyt
} // namespace nw
