﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/lyt/lyt_Layout.h>
#include <nw/lyt/lyt_ResourceAccessor.h>
#include <nw/lyt/lyt_Resources.h>
#include <nw/lyt/lyt_Util.h>

namespace nw
{
namespace lyt
{

ResourceAccessor::~ResourceAccessor()
{
}

ResourceAccessor::ResourceAccessor()
{
#if defined(NW_FONT_ENABLE_EDGE_FONT)
    m_CombinerType = font::FONT_COMBINER_TYPE_DEFAULT;
#endif
}

const TextureInfo
ResourceAccessor::LoadTexture(const char *name)
{
    u32 size = 0;
    void* pTexRes = this->GetResource(res::RESOURCETYPE_TEXTURE, name, &size);
    if (!pTexRes || size == 0)
    {
        NW_WARNING(false, "texture resource not found. - %s", name);
        return TextureInfo();
    }

    TextureInfo texInfo = lyt::LoadTexture(pTexRes, size);
    if (!texInfo.IsValid())
    {
        NW_WARNING(false, "texture object unavailable. - %s", name);
        return TextureInfo();
    }

    return texInfo;
}

font::Font*
ResourceAccessor::LoadFont(const char *name)
{
    u32 size = 0;
    void* pFontRes = this->GetResource(res::RESOURCETYPE_FONT, name, &size);
    if (!pFontRes || size == 0)
    {
        NW_WARNING(false, "font resource not found. - %s", name);
        return NULL;
    }

    font::ResFont* pResFont = Layout::NewObj<font::ResFont>();
    if (pResFont == NULL)
    {
        NW_WARNING(false, "font object creation failed.");
        return NULL;
    }

    bool bSuccess = pResFont->SetResource(pFontRes);
    if (!bSuccess)
    {
        NW_WARNING(false, "Fail to load ResFont.");
        Layout::DeleteObj(pResFont);
        return NULL;
    }

#if !defined(NW_FONT_TEXTURENAMES_BINARY_ALLOCATED)
    {
        const u32 drawBufferSize = font::ResFont::GetDrawBufferSize(pFontRes);
        void* drawBuffer = Layout::AllocMemory(drawBufferSize, 4);
        pResFont->SetDrawBuffer(drawBuffer);
    }
#endif

    return pResFont;
}

} // namespace nw::lyt
} // namespace nw
