/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nn/applet.h>
#include <nw/viewer.h>

/*
  
 */
extern "C" void nninitStartUp()
{
    // Gets the total maximum number of bytes of memory that can be used with the application.
    const size_t assingment   = nn::os::GetAppMemorySize();

    // Gets the total number of bytes of memory that have currently been allocated to the program.
    const size_t currentUsing = nn::os::GetUsingMemorySize();

    // Defines the device memory size.
    const size_t DEVICE_MEMORY_SIZE = 32 * 1024 * 1024;

    // Calculates the heap size.
    const size_t available = assingment - currentUsing;
    const size_t heapSize  = available - DEVICE_MEMORY_SIZE;

    // Sets the heap size.
    NN_PANIC_IF_FAILED( nn::os::SetHeapSize( heapSize ) );

    // Sets the device memory size.
    NN_PANIC_IF_FAILED( nn::os::SetDeviceMemorySize( DEVICE_MEMORY_SIZE ) );
}

/*
  
 */
void
nnMain()
{
    nn::applet::Enable();

    nw::viewer::sys::App& app = nw::viewer::sys::App::GetInstance();
    app.SystemInit(NULL);
    app.Init();                     // Performs initialization.

    bool isContinuing = true;
    while ( isContinuing )
    {
        app.PollStream();           // MCS communication processing

        app.BeginDraw();            // Makes preparations for rendering.
        app.Main();                 // Performs the main processing.

        app.UpdateFrame();          // Updates the animation frames.
        app.CalcAndDraw();          // Processes updates and rendering.

        app.DrawMenu();             // Displays menu and information systems.
        app.EndDraw();              // Ends rendering.

        app.WaitNextFrame();        // Waits for the next frame
    }

    app.Finalize();                 // Performs shutdown.

    nn::applet::CloseApplication();
}
