/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_EF_MODULEEF_PUB_H_
#define NW4C_VIEWER_EF_MODULEEF_PUB_H_

#include <nw/gfx.h>
#include <nw/viewer/types.h>
#include <nw/viewer/sys/Module.h>
#include <nw/viewer/g3d/Module3D.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw     {
namespace viewer {

namespace sys    {
class App;
} /* Namespace */

namespace ef     {

class ParticleModelIter;
class ParticleEmitterIter;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class IResourceListener
{
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnLoadResource( nw::gfx::res::ResGraphicsFile /* file */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddParticleModel( nw::gfx::ParticleModel* /* Model */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddParticleEmitter( nw::gfx::ParticleEmitter* /* Emitter */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateParticleModel( nw::gfx::ParticleModel* /* Model */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateParticleEmitter( nw::gfx::ParticleEmitter* /* Emitter */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateParticleSet( nw::gfx::ParticleModel* /* Model */, nw::gfx::ParticleSet* /* Set */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateMaterial( nw::gfx::ParticleModel* /* Model */, nw::gfx::Material* /* Material */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveParticleModel( nw::gfx::ParticleModel* /* Model */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveParticleEmitter( nw::gfx::ParticleEmitter* /* Emitter */ ) {}

    //
    //
    //
    virtual /* dtor */ ~IResourceListener() {}
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ModuleEF : public sys::Module
{
//---- Public members -----
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleModelIter GetParticleModel( u32 idx = 0 ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetParticleModelNum() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleEmitterIter GetParticleEmitter( u32 idx = 0 ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetParticleEmitterNum() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    IResourceListener* GetResourceListener();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetResourceListener( IResourceListener* listener );

//---- Private members -----
private:
    //
    //
    //
    ModuleEF();

    NW4C_VIEWER_FORBIDDEN_FUNC( ModuleEF );

    //
    //
    //
    IResourceListener* mResListener;

    friend class sys::App;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ParticleModelIter : public nw::viewer::g3d::NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    ParticleModelIter() : NodeIter(NULL) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::ParticleModel* GetParticleModel() const
    {
        return nw::ut::DynamicCast<nw::gfx::ParticleModel*>(GetSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResParticleModel GetResParticleModel() const
    {
        return nw::ut::ResDynamicCast<nw::gfx::res::ResParticleModel>(GetResSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleModelIter GetNext() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleModelIter &operator++()
    {
        *this = this->GetNext(); return *this;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleModelIter operator++(int)
    {
        ParticleModelIter it = *this; *this = this->GetNext(); return it;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator==( const ParticleModelIter& it ) const
    {
        return (this->m_pEntity == it.m_pEntity);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator!=( const ParticleModelIter& it ) const
    {
        return !(*this == it);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::ParticleEmitter* GetParticleEmitter(u32 idx) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetParticleEmitteriterNum() const;

protected:
    //
    //
    //
    ParticleModelIter(g3d::NodeEntity *pParticleModel): NodeIter(pParticleModel){}
    friend class ModuleEF;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ParticleEmitterIter : public nw::viewer::g3d::NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    ParticleEmitterIter() : NodeIter(NULL) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::ParticleEmitter* GetParticleEmitter() const
    {
        return nw::ut::DynamicCast<nw::gfx::ParticleEmitter*>(GetSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResParticleEmitter GetResParticleEmitter() const
    {
        return nw::ut::ResDynamicCast<nw::gfx::res::ResParticleEmitter>(GetResSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleEmitterIter GetNext() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleEmitterIter &operator++()
    {
        *this = this->GetNext(); return *this;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ParticleEmitterIter operator++(int)
    {
        ParticleEmitterIter it = *this; *this = this->GetNext(); return it;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator==( const ParticleEmitterIter& it ) const
    {
        return (this->m_pEntity == it.m_pEntity);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator!=( const ParticleEmitterIter& it ) const
    {
        return !(*this == it);
    }
protected:
    //
    //
    //
    ParticleEmitterIter(g3d::NodeEntity *pParticleEmitter) : NodeIter(pParticleEmitter) {}
    friend class ModuleEF;
    friend class ParticleModelIter;
};


} /* namespace ef    */
} /* namespace viewer */
} /* namespace nw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif /* NW4C_VIEWER_EF_MODULEEF_PUB_H_ */
