/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_LYT_MODULELYT_PUB_H_
#define NW4C_VIEWER_LYT_MODULELYT_PUB_H_

#include <nw/types.h>
#include <nw/viewer/types.h>
#include <nw/viewer/sys/Module.h>

#ifdef NW_PLATFORM_CTR
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
#endif
namespace nw     {
namespace lyt    {

class AnimResource;
class GraphicsResource;
class Layout;

} // namespace nw::lyt

namespace viewer {

namespace sys    {

class App;

} // namespace nw::viewer::sys

//
//
namespace lyt    {

class FileResourceAccessor;
class LayoutInfo;

//---------------------------------------------------------------------------
//
//
//
//
//---------------------------------------------------------------------------
class IResourceListener
{
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual bool OnLoadResource( FileResourceAccessor* accessor )
    {
        NW_UNUSED_VARIABLE( accessor );
        return false;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual nw::lyt::Layout* OnBuildLayout(
        const void* res,
        const char* name,
        FileResourceAccessor* accessor )
    {
        NW_UNUSED_VARIABLE( res );
        NW_UNUSED_VARIABLE( name );
        NW_UNUSED_VARIABLE( accessor );
        return NULL;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual bool OnBindAnimation(
        nw::lyt::Layout* layout,
        const nw::lyt::AnimResource& res,
        FileResourceAccessor* accessor )
    {
        NW_UNUSED_VARIABLE( layout );
        NW_UNUSED_VARIABLE( res );
        NW_UNUSED_VARIABLE( accessor );
        return false;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnLoadResourceDone(bool result)
    {
        NW_UNUSED_VARIABLE( result );
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnLoadToUpper()
    {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnLoadToLower()
    {}

    //
    //
    //
    virtual /* dtor */ ~IResourceListener() {}
};

//---------------------------------------------------------------------------
//
//
//---------------------------------------------------------------------------
class ModuleLYT : public sys::Module
{
//---- Public members -----
public:
    //-------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //-------------------------------------------------------------------------
    void SetModuleEnable(bool enable = true)
    {
        mbModuleEnable = enable;
    }

    //-------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //-------------------------------------------------------------------------
    bool GetModuleEnable() const
    {
        return mbModuleEnable;
    }

    //-------------------------------------------------------------------------
    //
    //
    //
    //-------------------------------------------------------------------------
    IResourceListener* GetResourceListener();

    //-------------------------------------------------------------------------
    //
    //
    //
    //-------------------------------------------------------------------------
    void SetResourceListener( IResourceListener* listener );

    //-------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //-------------------------------------------------------------------------
    void Reset();

    //-------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    void SetGraphicsResource(nw::lyt::GraphicsResource* pGraphicsResource);

    //-------------------------------------------------------------------------
    //
    //
    //
    //
    //
    nw::lyt::GraphicsResource* GetGraphicsResource() const;

    //-------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    LayoutInfo GetFirstLayout();

//---- Private members -----
private:
    //
    //
    ModuleLYT();

    NW4C_VIEWER_FORBIDDEN_FUNC( ModuleLYT );

    //
    //
    //
    IResourceListener* mResListener;

    //
    //
    //
    bool mbModuleEnable;

#if defined(NW_PLATFORM_CTR)
    u8 padding[3];
#endif

    friend class sys::App;
};

} // namespace nw::viewer::lyt
} // namespace nw::viewer
} // namespace nw
#ifdef NW_PLATFORM_CTR
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif
#endif

#endif /* NW4C_VIEWER_LYT_MODULELYT_PUB_H_ */
