/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_APPLICATION_PUB_H_
#define NW4C_VIEWER_SYSTEM_APPLICATION_PUB_H_

#include <nw/viewer/sys/Heap.h>
#include <nw/viewer/sys/VideoSetting.h>
#include <nw/viewer/sys/McsManager.h>
#include <nw/viewer/g3d/Module3D.h>
#include <nw/viewer/ef/ModuleEF.h>
#include <nw/viewer/lyt/ModuleLYT.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw     {
namespace viewer {
namespace sys    {

class App;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class App
{
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    static App& GetInstance();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void SystemInit( nw::os::IAllocator* pDeviceAllocator );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void Init();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void Init( nw::os::IAllocator* pAllocator, nw::os::IAllocator* pDeviceAllocator );

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void ViewerMenuInit();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void ViewerGraphicsInit();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void Finalize();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void PollStream();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void Main();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void UpdateFrame();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void CalcAndDraw( bool sync = true );

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void Calc();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void Draw( bool sync = true );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawMenu( bool sync = true );

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void WaitNextFrame();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void BeginDraw();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void EndDraw();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    VideoSetting&           GetVideoSetting()       { return mVideoSetting; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    const VideoSetting&     GetVideoSetting() const { return mVideoSetting; }


    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    Heap&                   GetHeap()               { return mHeap;         }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------

    const McsManager&       GetMcsManager() const   { return mMcsManager;   }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    McsManager&             GetMcsManager()         { return mMcsManager;   }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::viewer::g3d::Module3D&        GetModule3D()       { return mModule3D; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    const nw::viewer::g3d::Module3D&  GetModule3D() const { return mModule3D; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::viewer::ef::ModuleEF&       GetModuleEF()       { return mModuleEF; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    const nw::viewer::ef::ModuleEF& GetModuleEF() const { return mModuleEF; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::viewer::lyt::ModuleLYT&       GetModuleLYT()       { return mModuleLYT; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    const nw::viewer::lyt::ModuleLYT& GetModuleLYT() const { return mModuleLYT; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool GetConfigString( const char* param, const char** pValue );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool GetConfigInt( const char* param, s32* pValue );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool GetConfigUInt( const char* param, u32* pValue );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool GetConfigFloat( const char* param, f32* pValue );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetConfigFilePath( const char* filePath );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    const char* GetConfigFilePath();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void ReloadConfig();

    //nw::viewer::snd::ModuleSND&       GetModuleSND()       { return mModuleSND; }
    //const nw::viewer::snd::ModuleSND& GetModuleSND() const { return mModuleSND; }
    //
    //
    //const nw::ut::Font&               GetRomFont() const;
    //
    //void                    SetTitle( const char* fmt, ... );
    //
    //void                    SetInfoVisibility( bool visibility );
    //bool                    GetInfoVisibility() const;
    //

private:
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    App();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ~App() {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    VideoSetting    mVideoSetting;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    Heap            mHeap;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    McsManager      mMcsManager;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    g3d::Module3D   mModule3D;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ef::ModuleEF    mModuleEF;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    lyt::ModuleLYT    mModuleLYT;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    NW4C_VIEWER_FORBIDDEN_FUNC( App );
};


} // namespace sys
} // namespace viewer
} // namespace nw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif /* NW4C_VIEWER_SYSTEM_APPLICATION_PUB_H_ */

