/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_ProtocolSound_H_
#define NW4R_ProtocolSound_H_

//#pragma warn_padding on

/*
    ==========================================================================
    Note: This protocol header is automatically generated from$(NW4R_ROOT)/ToolUtil/ProtocolUtil/Protocol/ProtocolSound/ProtocolSound.xml.
    
    
    When editing this protocol, you must not directly edit the .h or .cs files but rather edit ProtocolSound.xml and then re-convert.
    
    ==========================================================================
    Viewer - SoundMaker Communication Protocol
    Communication message type
        Viewer Message  Message sent from the viewer to the editor
        Editor Message  Message sent from the editor to the viewer
        Error Message    Message sent when an error occurs
    When communicating, be sure to send the MessageHeader and then send the MessageData.
    MessageHeader is composed of a 2-byte MessageType and a 2-byte MessageSize.
    +---------------------------+--------------------+
    |      MessageHeader        |    MessageData     |
    +-------------+-------------+                    |
    | MessageType | MessageSize | (MessageSize byte) |
    | (2byte)     | (2byte)     |                    |
    +-------------+-------------+--------------------+
    MessageType comprises a 1-bit ErrorFlag, a 1-bit ReplyFlag, a 6-bit Category, and an 8-bit Command.
    
    15             8 7             0
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    |E|R| Category  |    Command    |
    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
    When Error is TRUE, Reply is always FALSE.
 






*/
namespace NW4R{
namespace ProtocolSound{

	//-----------------------------------------------------------------------
	/*
            Version Information Definition
            Please match usage rules for using the version number with those for each module and tool
         */
	/* Protocol version. */
	const unsigned long	PROTOCOL_VERSION =	0x00010002;

	//-----------------------------------------------------------------------
	/* Message flag and mask definition */
	/* Flag for errors         */
	const unsigned short	HEADER_ERROR_FLAG =	0x8000;
	/* Require response message flag */
	const unsigned short	HEADER_REPLY_FLAG =	0x4000;
	/* Mask for categories       */
	const unsigned short	HEADER_CATEGORY_MASK =	0x3F00;
	/* Mask for commands       */
	const unsigned short	HEADER_COMMAND_MASK =	0x00FF;

	//-----------------------------------------------------------------------
	/*
            Category definitions
         */
	/* System-related categories  */
	const unsigned short	CATEGORY_SYSTEM =	0x0000;
	/* 3D model-related categories  */
	const unsigned short	CATEGORY_3D =	0x0100;
	/* Sound-related category  */
	const unsigned short	CATEGORY_SOUND =	0x0300;
	/* MCS communication test categories */
	const unsigned short	CATEGORY_TEST =	0x3F00;

#if 0
	//-----------------------------------------------------------------------
	/* Sound protocol message shared header */
	typedef struct {
		/* Message types.    */
		unsigned short	type;
		/* Message size    */
		unsigned short	size;
	} Header;
#endif

	//-----------------------------------------------------------------------
	/* System-related category message */
	/* Version request command */
	const unsigned short	TOOL_SYSTEM_VERSION =	0x00 + CATEGORY_SYSTEM + HEADER_REPLY_FLAG;
	/* Command for synchronization */
	const unsigned short	TOOL_SYSTEM_SYNC =	0x01 + CATEGORY_SYSTEM + HEADER_REPLY_FLAG;
	/* Module reset command */
	const unsigned short	TOOL_SYSTEM_RESET =	0x02 + CATEGORY_SYSTEM;

	//-----------------------------------------------------------------------
	/* 3D model-related category message */
	/* Command to load model data */
	const unsigned short	TOOL_3D_LOAD_MODEL =	0x00 + CATEGORY_3D + HEADER_REPLY_FLAG;
	/* Command to get loaded model list */
	const unsigned short	TOOL_3D_GET_MODEL_LIST =	0x01 + CATEGORY_3D + HEADER_REPLY_FLAG;
	/* Command to get model node list */
	const unsigned short	TOOL_3D_GET_NODE_LIST =	0x02 + CATEGORY_3D + HEADER_REPLY_FLAG;

	//-----------------------------------------------------------------------
	/* Sound-related category message */
	/* Sound system-related message range */
	const unsigned short	TOOL_SOUND_SYSTEMBEGIN =	0x00 + CATEGORY_SOUND;
	/* Sound system-related message range */
	const unsigned short	TOOL_SOUND_SYSTEMEND =	0x39 + CATEGORY_SOUND;
	/* Command to forcibly switch to sound mode */
	const unsigned short	TOOL_SOUND_FOCUS =	0x01 + CATEGORY_SOUND;
	/* Echo command */
	const unsigned short	TOOL_SOUND_ECHO =	0x02 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Stop All Channels */
	const unsigned short	TOOL_SOUND_ALLSTOP =	0x03 + CATEGORY_SOUND;
	/* Sound system settings */
	const unsigned short	TOOL_SOUND_SET_SYSTEMPARAMETERS =	0x04 + CATEGORY_SOUND;
	/* Sound-related message range */
	const unsigned short	TOOL_SOUND_SOUNDBEGIN =	0x40 + CATEGORY_SOUND;
	/* Sound-related message range */
	const unsigned short	TOOL_SOUND_SOUNDEND =	0x79 + CATEGORY_SOUND;
	/* Command to load sound archive */
	const unsigned short	TOOL_SOUND_LOADSAR =	0x41 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Sound label setting */
	const unsigned short	TOOL_SOUND_SET_SOUNDLABELS =	0x42 + CATEGORY_SOUND;
	/* Get the sound state */
	const unsigned short	TOOL_SOUND_GET_SOUNDINFORMATIONS =	0x43 + CATEGORY_SOUND;
	/* Display sound information */
	const unsigned short	TOOL_SOUND_SHOWSOUNDINFORMATION =	0x44 + CATEGORY_SOUND;
	/* Playback command */
	const unsigned short	TOOL_SOUND_PLAY =	0x45 + CATEGORY_SOUND;
	/* Pause command */
	const unsigned short	TOOL_SOUND_PAUSE =	0x46 + CATEGORY_SOUND;
	/* Stop command */
	const unsigned short	TOOL_SOUND_STOP =	0x47 + CATEGORY_SOUND;
	/* Transfer effect parameters PC -> Viewer */
	const unsigned short	TOOL_SOUND_SET_EFFECTPARAMETERS =	0x48 + CATEGORY_SOUND;
	/* Transfer effect parameters PC <- Viewer */
	const unsigned short	TOOL_SOUND_GET_EFFECTPARAMETERS =	0x49 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Transfer sequence variables PC -> Viewer */
	const unsigned short	TOOL_SOUND_SET_SEQVARIABLES =	0x50 + CATEGORY_SOUND;
	/* Transfer sequence variables PC <- Viewer */
	const unsigned short	TOOL_SOUND_GET_SEQVARIABLES =	0x51 + CATEGORY_SOUND;
	/* Transfer preview player parameters PC -> Viewer */
	const unsigned short	TOOL_SOUND_SET_PREVIEWPLAYERPARAMETER =	0x52 + CATEGORY_SOUND;
	/* Animation sound-related message range */
	const unsigned short	TOOL_SOUND_ANIM_SOUNDBEGIN =	0x80 + CATEGORY_SOUND;
	/* Animation sound-related message range */
	const unsigned short	TOOL_SOUND_ANIM_SOUNDEND =	0xB9 + CATEGORY_SOUND;
	/* Command to load 3D model data */
	const unsigned short	TOOL_SOUND_ANIM_LOAD3DMODEL =	0x81 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Command to load animation data */
	const unsigned short	TOOL_SOUND_ANIM_LOADANIMATION =	0x82 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Command to load animation sound data */
	const unsigned short	TOOL_SOUND_ANIM_LOADANIMSOUND =	0x83 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Command to destroy animation cache */
	const unsigned short	TOOL_SOUND_ANIM_DISPOSECACHE =	0x84 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Switch active animation */
	const unsigned short	TOOL_SOUND_ANIM_ACTIVATEANIMATION =	0x85 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Command to get animation frame */
	const unsigned short	TOOL_SOUND_ANIM_QUERYFRAME =	0x86 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Command to set animation frame */
	const unsigned short	TOOL_SOUND_ANIM_SETFRAME =	0x87 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Animation Playback */
	const unsigned short	TOOL_SOUND_ANIM_PLAY =	0x88 + CATEGORY_SOUND;
	/* Stop animation */
	const unsigned short	TOOL_SOUND_ANIM_STOP =	0x89 + CATEGORY_SOUND;
	/* Command to set animation playback rate */
	const unsigned short	TOOL_SOUND_ANIM_SETRATE =	0x90 + CATEGORY_SOUND;
	/* Command to set loop state */
	const unsigned short	TOOL_SOUND_ANIM_SETLOOP =	0x91 + CATEGORY_SOUND;
	/* Command to set animation playback range */
	const unsigned short	TOOL_SOUND_ANIM_SETRANGE =	0x92 + CATEGORY_SOUND + HEADER_REPLY_FLAG;
	/* Command to update animation sound event */
	const unsigned short	TOOL_SOUND_ANIM_MODIFIEREVENT =	0x93 + CATEGORY_SOUND;

	//-----------------------------------------------------------------------
	/* Test category message */
	/* EFB color change command */
	const unsigned short	TOOL_TEST_EFB_COLOR =	0x00 + CATEGORY_TEST;

	//-----------------------------------------------------------------------
	/*
         Version request response message
            Receive tool protocol version, and return viewer protocol version.
         */
	typedef struct {
		/* 4B: Version number */
		unsigned long	version;
	} ToolSystemVersion;

	//-----------------------------------------------------------------------
	/*
        Synchronization message
              When receiving this synchronization command, the Viewer returns to the application the response command that echoes the specified tagName name.
              
              The tagName can be freely set on the tool-side.
         

*/
	typedef struct {
		/* 4B: Tag name. */
		unsigned long	tagName;
	} ToolSystemSync;

	//-----------------------------------------------------------------------
	/*
            Load model data message
            When fileSize is 0, the data that has that model ID is considered to be destroyed.
            When loading succeeds, Tool3DLoadModelReply is returned as a successful response message.
            If unsuccessful, return the ER_DATA_LOAD command.
         

*/
	typedef struct {
		/*   4B: File size.        */
		unsigned long	fileSize;
		/* 260B: Path name for the data on the PC  */
		unsigned char	pathName[260];
	} Tool3DLoadModel;

	//-----------------------------------------------------------------------
	/*
            Model data load response message
         */
	typedef struct {
		/*   4B: Management ID of the loaded model   */
		unsigned long	id;
	} Tool3DLoadModelReply;

	//-----------------------------------------------------------------------
	/*
            Loaded model list response message
            A variable length response message with a size number of model IDs in succession after this.
         */
	typedef struct {
		/*                 4B: Number of models loaded to the Viewer            */
		unsigned long	modelCount;
	} Tool3DGetModelListReply;

	//-----------------------------------------------------------------------
	/*
            Model list element structure
         */
	typedef struct {
		/*           4B: Model management ID               */
		unsigned long	id;
		/*           4B: Model name string length (including NULL)  */
		unsigned long	nameSize;
	} Tool3DGetModelListReplyElement;

	//-----------------------------------------------------------------------
	/*
            Model node list get message
         */
	typedef struct {
		/*   4B: Model management ID            */
		unsigned long	id;
	} Tool3DGetNodeList;

	//-----------------------------------------------------------------------
	/*
            Model node list response message
            When no model exists, nameLength is returned as 0
         */
	typedef struct {
		/*                4B: Number of nodes in the model     */
		long	nodeCount;
	} Tool3DGetNodeListReply;

	//-----------------------------------------------------------------------
	/*
            Node list element structure
         */
	typedef struct {
		/*           4B: Node index                                  */
		long	idx;
		/*           4B: Parent node index (negative value for a root node)  */
		long	parentIdx;
		/*           4B: Node name string length (including NULL)                           */
		unsigned long	nameSize;
	} Tool3DGetNodeListReplyElement;

	//-----------------------------------------------------------------------
	/* Item container for batch sending */
	typedef struct {
		/* Offset to data */
		unsigned long	offsetForData;
		/* Number of items */
		unsigned long	itemCount;
		/* Size of 1 item */
		unsigned long	itemSize;
	} ToolSoundItemContainer;

	//-----------------------------------------------------------------------
	/* File information */
	typedef struct {
		/* 260B: Path name for the data on the PC */
		unsigned char	fileName[260];
	} ToolSoundFileInfo;

	//-----------------------------------------------------------------------
	/* Sound state */
	enum ToolSoundOutputMode {
		/* Invalid value */
		OutputMode_Invalid =	0,
		/* Mono */
		OutputMode_Monoral =	1,
		/* Stereo */
		OutputMode_Stereo =	2,
		/* Surround */
		OutputMode_Surround =	3,
		/* DPL2 */
		OutputMode_Dpl2 =	4,
		/* Define enum as 4-byte type */
		ToolSoundOutputMode_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Sound archive load message  */
	typedef struct {
		/*  4B: Output mode */
		ToolSoundOutputMode	outputMode;
	} ToolSoundSystemParameters;

	//-----------------------------------------------------------------------
	/* Sound archive load message  */
	typedef struct {
		/* 260B: Path name for the data on the PC */
		unsigned char	pathName[260];
	} ToolSoundLoadSar;

	//-----------------------------------------------------------------------
	/* Sound archive load response message */
	typedef struct {
		/* 4B: Unused */
		unsigned long	id;
	} ToolSoundLoadSarReply;

	//-----------------------------------------------------------------------
	/* Sound state */
	enum ToolSoundPreviewSoundState {
		/* No sound is specified */
		SoundState_Disabled =	0,
		/* Stopped */
		SoundState_Stopped =	1,
		/* Preparing for playback */
		SoundState_Preparing =	2,
		/* Playing */
		SoundState_Playing =	3,
		/* Paused */
		SoundState_Paused =	4,
		/* Define enum as 4-byte type */
		ToolSoundPreviewSoundState_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Sound label */
	typedef struct {
		/*   4B: Parameter enable flag (1: enabled; 0: disabled) */
		unsigned long	index;
		/* 260B: Sound name (Not a setting target when disabled) */
		unsigned char	label[260];
	} ToolSoundPreviewSoundLabel;

	//-----------------------------------------------------------------------
	/* Sound information */
	typedef struct {
		/* 4B: Sound state */
		ToolSoundPreviewSoundState	state;
	} ToolSoundPreviewSoundInfomation;

	//-----------------------------------------------------------------------
	/* Preview player identifier */
	typedef struct {
		/* 4B: Preview player index */
		unsigned long	index;
	} ToolSoundPreviewPlayerID;

	//-----------------------------------------------------------------------
	/* Bus type */
	enum ToolSoundAuxBusType {
		/* Aux A  */
		AuxBusType_AuxA =	0,
		/* Aux B  */
		AuxBusType_AuxB =	1,
		/* Aux C  */
		AuxBusType_AuxC =	2,
		/* Define enum as 4-byte type */
		ToolSoundAuxBusType_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Effect type. */
	enum ToolSoundEffectType {
		/* NO EFFECT   */
		EffectType_NoEffect =	0,
		/* REVERB HI   */
		EffectType_ReverbHi =	1,
		/* REVERB STD  */
		EffectType_ReverbStd =	2,
		/* CHORUS      */
		EffectType_Chorus =	3,
		/* DELAY       */
		EffectType_Delay =	4,
		/* Define enum as 4-byte type */
		ToolSoundEffectType_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Post-position reverberation sound type */
	enum ToolSoundFusedMode {
		/* old axfx     */
		FusedMode_OldAxfx =	0,
		/* metal tank   */
		FusedMode_MetalTank =	1,
		/* small room   */
		FusedMode_SmallRoom =	2,
		/* large room   */
		FusedMode_LargeRoom =	3,
		/* hall         */
		FusedMode_Hall =	4,
		/* cavernous    */
		FusedMode_Cavernous =	5,
		/* Define enum as 4-byte type */
		ToolSoundFusedMode_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Shared portion for all effect parameters */
	typedef struct {
		/*  4B: Bus type */
		ToolSoundAuxBusType	busType;
		/*  4B: Index */
		unsigned long	effectIndex;
		/*  4B: Effect type */
		ToolSoundEffectType	effectType;
		/* 44B: Reserved */
		unsigned char	reserved[44];
	} ToolSoundEffectCommonParameter;

	//-----------------------------------------------------------------------
	/* NO EFFECT parameter */
	typedef struct {
		/*  4B: Bus type */
		ToolSoundAuxBusType	busType;
		/*  4B: Index */
		unsigned long	effectIndex;
		/*  4B: Effect type */
		ToolSoundEffectType	effectType;
		/* 44B: Reserved */
		unsigned char	reserved[44];
	} ToolSoundNoEffectParameter;

	//-----------------------------------------------------------------------
	/* REVERB HI parameter */
	typedef struct {
		/*  4B: Bus type */
		ToolSoundAuxBusType	busType;
		/*  4B: Index */
		unsigned long	effectIndex;
		/*  4B: Effect type. */
		ToolSoundEffectType	effectType;
		/*  4B: early mode */
		long	earlyMode;
		/*  4B: pre delay time max */
		float	preDelayTimeMax;
		/*  4B: pre delay time */
		float	preDelayTime;
		/*  4B: fused mode */
		ToolSoundFusedMode	fusedMode;
		/*  4B: fused time */
		float	fusedTime;
		/*  4B: coloration */
		float	coloration;
		/*  4B: damping */
		float	damping;
		/*  4B: crosstalk */
		float	crosstalk;
		/*  4B: early gain */
		float	earlyGain;
		/*  4B: fused gain */
		float	fusedGain;
		/*  4B: out gain */
		float	outGain;
	} ToolSoundReverbHiParameter;

	//-----------------------------------------------------------------------
	/* REVERB STD parameter */
	typedef struct {
		/*  4B: Bus type */
		ToolSoundAuxBusType	busType;
		/*  4B: Index */
		unsigned long	effectIndex;
		/*  4B: Effect type */
		ToolSoundEffectType	effectType;
		/*  4B: early mode */
		long	earlyMode;
		/*  4B: pre delay time max */
		float	preDelayTimeMax;
		/*  4B: pre delay time */
		float	preDelayTime;
		/*  4B: fused mode */
		ToolSoundFusedMode	fusedMode;
		/*  4B: fused time */
		float	fusedTime;
		/*  4B: coloration */
		float	coloration;
		/*  4B: damping */
		float	damping;
		/*  4B: early gain */
		float	earlyGain;
		/*  4B: fused gain */
		float	fusedGain;
		/*  4B: out gain */
		float	outGain;
		/*  4B: Reserved */
		unsigned char	reserved[4];
	} ToolSoundReverbStdParameter;

	//-----------------------------------------------------------------------
	/* CHORUS parameter */
	typedef struct {
		/*  4B: Bus type */
		ToolSoundAuxBusType	busType;
		/*  4B: Index */
		unsigned long	effectIndex;
		/*  4B: Effect type. */
		ToolSoundEffectType	effectType;
		/*  4B: delay time */
		float	delayTime;
		/*  4B: depth */
		float	depth;
		/*  4B: rate */
		float	rate;
		/*  4B: feedback */
		float	feedback;
		/*  4B: out gain */
		float	outGain;
		/* 24B: Reserved */
		unsigned char	reserved[24];
	} ToolSoundChorusParameter;

	//-----------------------------------------------------------------------
	/* DELAY parameter */
	typedef struct {
		/*  4B: Bus type */
		ToolSoundAuxBusType	busType;
		/*  4B: Index */
		unsigned long	effectIndex;
		/*  4B: Effect type. */
		ToolSoundEffectType	effectType;
		/*  4B: max delay time */
		float	maxDelayTime;
		/*  4B: delay time */
		float	delayTime;
		/*  4B: feedback */
		float	feedback;
		/*  4B: out gain */
		float	outGain;
		/*  4B: LPF */
		float	lpf;
		/* 24B: Reserved */
		unsigned char	reserved[24];
	} ToolSoundDelayParameter;

	//-----------------------------------------------------------------------
	/* Sequence variable type */
	enum ToolSoundSeqVariableType {
		/* Global Variables */
		SeqVariableType_Global =	1,
		/* Local Variables */
		SeqVariableType_Local =	2,
		/* Track Variables */
		SeqVariableType_Track =	3,
		/* Define enum as 4-byte type */
		ToolSoundSeqVariableType_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Sequence variable container header */
	typedef struct {
		/*  4B: Sequence variable type */
		ToolSoundSeqVariableType	variableType;
		/*  4B: Preview player index */
		unsigned long	previewPlayerIndex;
		/*  4B: Track number */
		unsigned long	trackNo;
	} ToolSoundSeqVariableContainerHeader;

	//-----------------------------------------------------------------------
	/* Sequence variable container */
	typedef struct {
		/* 12B: Container header */
		ToolSoundSeqVariableContainerHeader	header;
		/* 64B: Value of variable (Invalid value if ShortMax+1) */
		long	variables[16];
	} ToolSoundSeqVariableContainer;

	//-----------------------------------------------------------------------
	/* Bus type */
	enum ToolSoundBiquadType {
		/* OFF  */
		BiquadType_Off =	0,
		/* LPF  */
		BiquadType_Lpf =	1,
		/* HPF  */
		BiquadType_Hpf =	2,
		/* BPF512  */
		BiquadType_Bpf512 =	3,
		/* BPF1024  */
		BiquadType_Bpf1024 =	4,
		/* BPF2048  */
		BiquadType_Bpf2048 =	5,
		/* Define enum as 4-byte type */
		ToolSoundBiquadType_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Preview Player Parameters */
	typedef struct {
		/*  4B: Target preview player index */
		unsigned long	previewPlayerIndex;
		/*  4B: Volume */
		float	volume;
		/*  4B: Pitch */
		float	pitch;
		/*  4B: Pan */
		float	pan;
		/*  4B: surround pan */
		float	surroundPan;
		/*  4B: LPF frequency */
		float	lpfFrequency;
		/*  4B: BiquadType */
		ToolSoundBiquadType	biquadType;
		/*  4B: BiquadValue */
		float	biquadValue;
		/*  4B: Playback start position [ms] */
		unsigned long	startOffsetTime;
		/*  4B: Main output switch */
		long	mainOutEnabled;
		/*  4B: Wii Remote Control 1 output switch */
		long	remote0OutEnabled;
		/*  4B: Wii Remote Control 2 output switch */
		long	remote1OutEnabled;
		/*  4B: Wii Remote Control 3 output switch */
		long	remote2OutEnabled;
		/*  4B: Wii Remote Control 4 output switch */
		long	remote3OutEnabled;
		/*  4B: Main output */
		float	mainOutVolume;
		/*  4B: Wii Remote Control 1 output */
		float	remote0OutVolume;
		/*  4B: Wii Remote Control 2 output */
		float	remote1OutVolume;
		/*  4B: Wii Remote Control 3 output */
		float	remote2OutVolume;
		/*  4B: Wii Remote Control 4 output */
		float	remote3OutVolume;
		/*  4B: Main send */
		float	mainSend;
		/*  4B: Effect Aux A send */
		float	effectAuxASend;
		/*  4B: Effect Aux B send */
		float	effectAuxBSend;
		/*  4B: Effect Aux C send */
		float	effectAuxCSend;
		/*  4B: Fade in frame */
		long	fadeInFrames;
		/*  4B: Fade out frame */
		long	fadeOutFrames;
		/*  4B: Pose fade frame */
		long	pauseFadeFrames;
	} ToolSoundPreviewPlayerParameter;

	//-----------------------------------------------------------------------
	/* Animation Component Type */
	enum ToolSoundAnimComponentType {
		/* 3D model */
		AnimComponentType_3DModel =	1,
		/* Action */
		AnimComponentType_Action =	2,
		/* Animation Sound */
		AnimComponentType_AnimSound =	3,
		/* Define enum as 4-byte type */
		ToolSoundAnimComponentType_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/* Animation component identifier */
	typedef struct {
		/*  4B: Component type */
		ToolSoundAnimComponentType	type;
		/*  4B: Model ID */
		unsigned long	modelID;
		/*  4B: Action ID (-1 if not specified) */
		unsigned long	actionID;
		/*  4B: Animation sound ID (-1 if not specified) */
		unsigned long	animSoundID;
	} ToolSoundAnimComponentID;

	//-----------------------------------------------------------------------
	/* Animation component add-on data */
	typedef struct {
		/*  16B: Target ID */
		ToolSoundAnimComponentID	id;
		/* 260B: Filename */
		unsigned char	fileName[260];
		/*   4B: 1 when cache is not used; 0 when cache is used */
		long	reload;
	} ToolSoundAnimComponentAddInfo;

	//-----------------------------------------------------------------------
	/*
            Animation sound load message
         */
	typedef struct {
		/* 260B: Path name for the data on the PC */
		unsigned char	pathName[260];
	} ToolSoundLoadAsd;

	//-----------------------------------------------------------------------
	/*
            Animation sound load response message
         */
	typedef struct {
		/* 4B: Unused */
		unsigned long	id;
	} ToolSoundLoadAsdReply;

	//-----------------------------------------------------------------------
	/*
            Animation frame setting message
         */
	typedef struct {
		/* 4B: Animation frame */
		float	frame;
		/* 4B: Whether to process the event */
		long	doEvent;
		/* 4B: Whether to synchronize */
		long	sync;
	} ToolSoundAnimFrame;

	//-----------------------------------------------------------------------
	/*
            Animation playback rate setting message
         */
	typedef struct {
		/* 4B: Animation playback rate */
		float	rate;
	} ToolSoundAnimRate;

	//-----------------------------------------------------------------------
	/*
            Animation frame get response message
         */
	typedef struct {
		/* 4B: Animation frame */
		float	frame;
	} ToolSoundQueryAnimFrameReply;

	//-----------------------------------------------------------------------
	/*
            Loop state setting message
         */
	typedef struct {
		/* 4B: Loop state */
		long	loopFlag;
	} ToolSoundSetLoop;

	//-----------------------------------------------------------------------
	/*
            Animation playback range setting message
         */
	typedef struct {
		/* 4B: Loop start position */
		float	start;
		/* 4B: Loop end position */
		float	end;
	} ToolSoundAnimRange;

	//-----------------------------------------------------------------------
	/*
            Animation playback range setting response message
         */
	typedef struct {
		/* 4B: Current frame after setting change */
		float	frame;
	} ToolSoundAnimRangeReply;

	//-----------------------------------------------------------------------
	/*
            Command to update animation sound event
         */
	typedef struct {
		/* 4B: 0 for update; 1 for deletion */
		long	isDelete;
		/* 4B: Index */
		long	index;
		/* 4B: Sound ID */
		unsigned long	soundId;
		/* 260B: Name */
		unsigned char	soundName[260];
		/* 1B: Volume */
		unsigned char	volume;
		/* 1B: Padding. */
		unsigned char	padding1;
		/* 1B: Padding. */
		unsigned char	padding2;
		/* 1B: Padding. */
		unsigned char	padding3;
		/* 4B: Pitch */
		float	pitch;
		/* 4B: Start frame */
		long	startFrame;
		/* 4B: End frame */
		long	endFrame;
		/* 1B: frame processing option flag */
		unsigned char	frameFlag;
		/* 1B: number of playback loops */
		char	loopOffset;
		/* 1B: playback loop interval after loopCount */
		unsigned char	loopInterval;
		/* 1B: Padding. */
		unsigned char	padding4;
	} ToolSoundModifierEvent;

	//-----------------------------------------------------------------------
	/*
         EFB color change message
         */
	typedef struct {
		/* 1B: R Component */
		unsigned char	r;
		/* 1B: G Component */
		unsigned char	g;
		/* 1B: B Component */
		unsigned char	b;
		/* 1B: A Component */
		unsigned char	a;
	} ToolTestEfbColor;

	//-----------------------------------------------------------------------
	/* Viewer-initiated system-related category message */
	/* Viewer-initiated test command */
	const unsigned short	VIEWER_TEST_NOP =	0x00 + CATEGORY_TEST;

	//-----------------------------------------------------------------------
	/* System error category message */
	/* Undefined command error */
	const unsigned short	ERR_SYSTEM_UNKNOWN_COMMAND =	0x0 + CATEGORY_SYSTEM + HEADER_ERROR_FLAG;

	//-----------------------------------------------------------------------
	/* Data error category message */
	/* Data load error         */
	const unsigned short	ERR_3D_LOAD =	0x0 + CATEGORY_3D + HEADER_ERROR_FLAG;

	//-----------------------------------------------------------------------
	/* Sound category message */
	/* Sound error */
	const unsigned short	ERR_SOUND =	0x0 + CATEGORY_SOUND + HEADER_ERROR_FLAG;

	//-----------------------------------------------------------------------
	/*
            Undefined command error message
            Returns the received undefined command with the command parameter.
         
*/
	typedef struct {
		/*   2B: Received undefined command   */
		unsigned short	command;
	} ErrSystemUnknownCommand;

	//-----------------------------------------------------------------------
	/* Load data error type */
	enum Err3DLoadCode {
		/* Failed to load file.         */
		ERR_3D_LOAD_FILE_NOT_FOUND =	0,
		/* Failed to allocate memory.   */
		ERR_3D_LOAD_MEMORY_ALLOCATE =	1,
		/* No valid data exists.       */
		ERR_3D_LOAD_INVALID_DATA =	2,
		/* An error occurred during MCS communication.            */
		ERR_3D_LOAD_MCS_STREAM_FAIL =	3,
		/* Define enum as 4-byte type */
		Err3DLoadCode_4BYTE_ENUM =	0xFFFFFFFF
	};

	//-----------------------------------------------------------------------
	/*
            Data load error.
         */
	typedef struct {
		/*   4B: Error Code           */
		Err3DLoadCode	code;
	} Err3DLoad;

	//-----------------------------------------------------------------------
	/* Error code flag definition */
	/* Error flag */
	const unsigned long	MCSSOUND_ERRORFLAG =	0x80000000;

	//-----------------------------------------------------------------------
	/* Error code category definition */
	/* General */
	const unsigned long	MCSSOUNDCATEGORY_GENERAL =	0x00000000;
	/* Sound */
	const unsigned long	MCSSOUNDCATEGORY_SOUND =	0x00010000;
	/* Animation Sound */
	const unsigned long	MCSSOUNDCATEGORY_ANIMSOUND =	0x00020000;

	//-----------------------------------------------------------------------
	/* Sound error code */
	/* Success. */
	const unsigned long	MCSSOUND_S_OK =	0x0000;
	/* An error occurred.         */
	const unsigned long	MCSSOUND_E_FAILED =	0x0000 + MCSSOUNDCATEGORY_GENERAL + MCSSOUND_ERRORFLAG;
	/* Cannot find file.     */
	const unsigned long	MCSSOUND_E_FILENOTFOUND =	0x0001 + MCSSOUNDCATEGORY_GENERAL + MCSSOUND_ERRORFLAG;
	/* Insufficient memory.       */
	const unsigned long	MCSSOUND_E_OUTOFMEMORY =	0x0002 + MCSSOUNDCATEGORY_GENERAL + MCSSOUND_ERRORFLAG;
	/* Invalid data.             */
	const unsigned long	MCSSOUND_E_INVALIDDATA =	0x0003 + MCSSOUNDCATEGORY_GENERAL + MCSSOUND_ERRORFLAG;
	/* An error occurred during MCS communication.  */
	const unsigned long	MCSSOUND_E_TRANSFERFAILED =	0x0004 + MCSSOUNDCATEGORY_GENERAL + MCSSOUND_ERRORFLAG;
	/* Invalid animation ID.       */
	const unsigned long	MCSSOUND_E_ANIM_INVALIDCOMPONENTID =	0x0001 + MCSSOUNDCATEGORY_ANIMSOUND + MCSSOUND_ERRORFLAG;
	/* Cannot find model cache. */
	const unsigned long	MCSSOUND_E_ANIM_MODELCACHENOTFOUND =	0x0002 + MCSSOUNDCATEGORY_ANIMSOUND + MCSSOUND_ERRORFLAG;
	/* Cannot find action cache. */
	const unsigned long	MCSSOUND_E_ANIM_ACTIONCACHENOTFOUND =	0x0003 + MCSSOUNDCATEGORY_ANIMSOUND + MCSSOUND_ERRORFLAG;

	//-----------------------------------------------------------------------
	/* Sound error */
	typedef struct {
		/*  4B: Error Code */
		unsigned long	errorCode;
	} SoundError;

} /* namespace ProtocolSound */
} /* namespace NW4R */

//#pragma warn_padding reset

#endif /* NW4R_ProtocolSound_H_ */
