// ---------------------------------------------------------------------------
//  Project:  NintendoWare
//
//  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.
//
//  These coded instructions, statements, and computer programs contain proprietary
//  information of Nintendo and/or its licensed developers and are protected by
//  national and international copyright laws. They may not be disclosed to third
//  parties or copied or duplicated in any form, in whole or in part, without the
//  prior written consent of Nintendo.
//
//  The content herein is highly confidential and should be handled accordingly.
// ---------------------------------------------------------------------------

/*
 * NOTE:
 *    Dummy protocol used for communication test
 */


const unsigned short NW4R_VWS_CATEGORY_MASK = 0xFF00;
const unsigned short NW4R_VWS_MESG_MASK     = 0x00FF;

enum : unsigned short
{
    NW4R_VWS_CATEGORY_CTRL          = 0x0000,   // Control messages
    NW4R_VWS_CATEGORY_VIDEO         = 0x0100    // Video configuration messages
};

enum : unsigned short
{
    NW4R_VWS_MESG_CTRL_SYNC         = NW4R_VWS_CATEGORY_CTRL | 0x00,
    NW4R_VWS_MESG_CTRL_RESET        = NW4R_VWS_CATEGORY_CTRL | 0x01
};

enum : unsigned short
{
    NW4R_VWS_MESG_VIDEO_FB_SIZE     = NW4R_VWS_CATEGORY_VIDEO | 0x00,
    NW4R_VWS_MESG_VIDEO_EFB_CLR     = NW4R_VWS_CATEGORY_VIDEO | 0x01,
    NW4R_VWS_MESG_VIDEO_MODE        = NW4R_VWS_CATEGORY_VIDEO | 0x02
};


[StructLayout(LayoutKind.Sequential)]
struct NW4RVwsHeader
{
    public unsigned short      messageID;          // Message ID
    public unsigned short      messageSize;        // Message size
}



[StructLayout(LayoutKind.Sequential)]
struct NW4RVwsCtrlSync
{
    public NW4RVwsHeader       header;
    public int                 tag;                // Tags
}


[StructLayout(LayoutKind.Sequential)]
struct NW4RVwsCtrlReset
{
    public NW4RVwsHeader       header;
}


[StructLayout(LayoutKind.Sequential)]
struct NW4RVwsVideoFbSize
{
    public NW4RVwsHeader       header;
    public unsigned short      width;              // Width
    public unsigned short      height;             // Height
}



[StructLayout(LayoutKind.Sequential)]
struct NW4RVwsVideoEfbClr
{
    public NW4RVwsHeader       header;
    public unsigned long       color;              // Clear color
    public unsigned long       z;                  // Clear Z
}



enum : unsigned char
{
    NW4R_VWS_VIDEO_MODE_NON_INTERLACE,      // Non-interlaced
    NW4R_VWS_VIDEO_MODE_INTERLACE,          // Interlaced
    NW4R_VWS_VIDEO_MODE_PROGRESSIVE         // Progressive
};


[StructLayout(LayoutKind.Sequential)]
struct NW4RVwsVideoMode
{
    NW4RVwsHeader       header;
    bool                antiAlias;          // Enable/disable anti-aliasing
    bool                vfilter;            // Enable/disable V filter
    bool                fieldRender;        // Flag indicating use of field rendering
    unsigned char       scanMode;           // Scan mode
}




