/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_PROTOCOL_SYSTEM_H_
#define NW4R_VIEWER_PROTOCOL_SYSTEM_H_

#ifdef __cplusplus
extern "C" {
#endif

/*
 * NOTE:
 *    Dummy protocol used for communication test
 */



#define NW4R_VWS_CATEGORY_MASK      0xFF00
#define NW4R_VWS_MESG_MASK          0x00FF

enum
{
    NW4R_VWS_CATEGORY_CTRL          = 0x0000,   // Control messages
    NW4R_VWS_CATEGORY_VIDEO         = 0x0100    // Video configuration messages
};

enum
{
    NW4R_VWS_MESG_CTRL_SYNC         = NW4R_VWS_CATEGORY_CTRL | 0x00,
    NW4R_VWS_MESG_CTRL_RESET        = NW4R_VWS_CATEGORY_CTRL | 0x01
};

enum
{
    NW4R_VWS_MESG_VIDEO_FB_SIZE     = NW4R_VWS_CATEGORY_VIDEO | 0x00,
    NW4R_VWS_MESG_VIDEO_EFB_CLR     = NW4R_VWS_CATEGORY_VIDEO | 0x01,
    NW4R_VWS_MESG_VIDEO_MODE        = NW4R_VWS_CATEGORY_VIDEO | 0x02
};


typedef struct
{
    unsigned short      messageID;          // Message ID
    unsigned short      messageSize;        // Message size
}
NW4RVwsHeader;


typedef struct
{
    NW4RVwsHeader       header;
    int                 tag;                // Tags
}
NW4RVwsCtrlSync;

typedef struct
{
    NW4RVwsHeader       header;
}
NW4RVwsCtrlReset;



typedef struct
{
    NW4RVwsHeader       header;
    unsigned short      fbWidth;              // Width
    unsigned short      fbHeight;             // Height
    unsigned short      viWidth;              // Width
    unsigned short      viHeight;             // Height
}
NW4RVwsVideoFbSize;

typedef struct
{
    NW4RVwsHeader       header;
    unsigned long       color;              // Clear color
    unsigned long       z;                  // Clear Z
}
NW4RVwsVideoEfbClr;


enum
{
    NW4R_VWS_VIDEO_MODE_INTERLACE,          // Interlaced
    NW4R_VWS_VIDEO_MODE_NON_INTERLACE,      // Non-interlaced
    NW4R_VWS_VIDEO_MODE_PROGRESSIVE         // Progressive
};


typedef struct
{
    NW4RVwsHeader       header;
    bool                antiAlias;          // Enable/disable anti-aliasing
    bool                vfilter;            // Enable/disable V filter
    bool                fieldRender;        // Flag indicating use of field rendering
    unsigned char       scanMode;           // Scan mode
}
NW4RVwsVideoMode;



#ifdef __cplusplus
}/* extern "C" */
#endif




/* NW4R_VIEWER_PROTOCOL_SYSTEM_H_ */
#endif

