/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_CAMERA_H_
#define NW4C_VIEWER_G3D_CAMERA_H_

#include "DirectionalObj.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw  {
namespace g3d {

/*-------------------------------------------------------
  Name:         Camera

  Description:  Camera management class
  -------------------------------------------------------*/
class Camera
{
//--public members-----------------------------------------
public:

    static const float  VIRTUAL_TARGET_DISTANCE = 10.f;    // The distance from the virtual gaze point used by RotateCamera. - TODO: Adjustable

    Camera(nw::gfx::Camera* camera, const char *name);
    nw::gfx::Camera*    GetCamera();
    void                Reset();                           // In the end, returns the position to the pre-set position.
    void                Preset();                          // Stores the current settings so that the status can return if there is a reset.
    void                MoveCamera3D(vw::system::HidStatus& hidStatus);
    void                Focus(nw::math::VEC3 pos);
    void                Focus(nw::math::AABB aabb);
    const char *        GetName(){ return mName; }
    static nw::math::MTX33 GetOrientation(nw::gfx::Camera* camera);
    void                SetPreset(const nw::math::VEC3& cameraPos,
                                  const nw::math::VEC3& cameraTarget,
                                  const nw::math::VEC3& cameraUp);

private:
    void                Rotate(DirectionalObj& directionalObj);
    void                Dolly(DirectionalObj& directionalObj);
    void                Tilt(DirectionalObj& directionalObj);
    void                Slide(DirectionalObj& direcitonalObj);
    void                Track(DirectionalObj& directionalObj);
    void                Zoom(DirectionalObj& directionalObj);
    void                Reset(DirectionalObj& directionalObj);
    void                InitializeVirtualTarget();

private:
    Camera();
    nw::gfx::Camera*    m_Camera;

    nw::math::VEC3      m_CameraPos;
    nw::math::VEC3      m_CameraTarget;
    nw::math::VEC3      m_CameraUp;
    nw::math::VEC3      m_Rotate;
    nw::math::VEC3      m_VirtualTarget;    // Virtual gaze point used by RotateCamera.
    f32                 m_Twist;
    nw::ut::Rect        m_OrthoRect;

    f32                 m_RotateSpeed;
    f32                 m_TransSpeed;
    f32                 m_TrackSpeed;

    s32                 m_TouchPanelX;
    s32                 m_TouchPanelY;
    bool                m_TouchLeftMode;

    f32                 m_TiltXSpeed;
    f32                 m_TiltYSpeed;
    f32                 m_SlideXSpeed;
    f32                 m_SlideYSpeed;
    char                mName[512];
};

}   /* namespace g3d */
}   /* namespace vw  */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif


/* NW4C_VIEWER_G3D_CAMERA_H_ */
#endif
