/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_LIGHT_MGR_H_
#define NW4C_VIEWER_G3D_LIGHT_MGR_H_

#include "Light.h"
#include "../../../src/g3d/include/SceneElemSelector.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw  {
namespace g3d {

class LightMgr
{
public:
    // <CODE>GetInstance</CODE>
    static LightMgr*    GetInstance();
    // Initialize
    static void         Initialize();
    // Finalize
    static void         Finalize();
    // Setup
    void                Setup( SceneElemSelector::LightList::iterator lightsBegin,
                               SceneElemSelector::LightList::iterator lightsEnd);
    // MoveLight3D
    void                MoveLight3D(vw::system::HidStatus& hidStatus);
    // GetActiveTaretLightIdx
    int                 GetActiveTargetLightIdx(){ return m_TargetIdx; }
    // GetTargetLight
    nw::gfx::Light*     GetTargetLight(){ return m_LightArray[m_TargetIdx].GetLight(); }
    // GetTargetLight
    g3d::Light*         GetTargetLightG3D(){ return &m_LightArray[m_TargetIdx]; }
    // GetTargetLightName
    const char*         GetTargetLightName(){ return m_LightArray[m_TargetIdx].GetName(); }
    // GetLightNum
    u32                 GetLightNum(){ return m_LightArray.size(); }
    // IsFirstTargetLight()
    bool                IsFirstTargetLight();
    // IsLastTargetLight()
    bool                IsLastTargetLight();
    // SwitchToFirstTargetLight()
    void                SwitchToFirstTargetLight();
    // SwitchToLastTargetLight()
    void                SwitchToLastTargetLight();
    // SwitchToPrevTargetLight()
    void                SwitchToPrevTargetLight();
    // SwitchToNextTargetLight()
    void                SwitchToNextTargetLight();
    // GetLightArray
    nw::ut::MoveArray<g3d::Light>* GetLightArray() { return &m_LightArray; }
    // Reset Default Light
    void                ReloadDefaultLight( nw::gfx::Light* light );

private:
    /* ctor */
    LightMgr();
    /* dtor */
    ~LightMgr();
    // Operation target light index
    s16                             m_TargetIdx;
    // Lights
    nw::ut::MoveArray<g3d::Light>   m_LightArray;
    // Name of operation target light used immediately before
    char                            m_PrevTargetLightName[512];
    char                            m_PrevTargetCameraName[512];
    // Static instance
    static LightMgr*                m_Instance;
};

}
}
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

/* NW4C_VIEWER_G3D_LIGHT_MGR_H_ */
#endif
