/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef PROTOCOL4C_DEF
#define PROTOCOL4C_DEF

#include <cstring>
#include <cassert>

namespace Protocol4C
{

    struct Constants
    {
        static const unsigned int VERSION = 0x02000000;          // Protocol version.
        static const unsigned int VERSION_MAJOR_MASK = 0xFF000000;          // Major protocol version mask.
        static const unsigned int VERSION_MINOR_MASK = 0x00FF0000;          // Minor protocol version mask.
        static const unsigned int VERSION_REVISION_MASK = 0x0000FF00;          // Protocol revision mask.
        static const unsigned int CHANNEL_ACTIVE = 0x0000000C;          // Communication channel number with 3DEditor - To Viewer
        static const unsigned int CHANNEL_PASSIVE = 0x0000000D;          // Communication channel number with 3DEditor - From Viewer
        static const unsigned int CHANNEL_SURVIVAL = 0x0000000E;          // Communication channel number with 3DEditor - For existence confirmation
        static const unsigned int MESSAGE_CATEGORY_MASK = 0x0000FF00;          // Message category mask.
    };

    enum EditTargetType // Edit target types.
    {
        EditTargetType_Node = 0x01,           // Node Target
        EditTargetType_Material = 0x02,           // Material Target
        EditTargetType_Mesh = 0x03,           // Mesh target
        EditTargetType_Model = 0x04,           // Model Target
        EditTargetType_Texture = 0x05,           // Texture target
        EditTargetType_Bone = 0x06,           // Bone target
        EditTargetType_Camera = 0x07,           // Camera Target
        EditTargetType_Fog = 0x08,           // Fog Target
        EditTargetType_Light = 0x09,           // Light Target
        EditTargetType_Mask = 0xFF,           // Mask
        //
        EditTargetType_FORCE32    = 0xFFFFFFFF
    };

    enum BinaryDataType // Binary data type
    {
        BinaryDataType_Bcmdl = 0,           // model
        BinaryDataType_Bctex = 1,           // Texture
        BinaryDataType_Bcskla = 2,           // Skeletal animations
        BinaryDataType_Bclts = 3,           // Lookup table
        BinaryDataType_Bcmata = 4,           // Material Animations
        BinaryDataType_Bcscn = 5,           // Scene
        BinaryDataType_Bccam = 6,           // Camera
        BinaryDataType_Bclgt = 7,           // Lights
        BinaryDataType_Bcfog = 8,           // Fog
        BinaryDataType_Bcmdla = 9,           // Model animation (visibility animation)
        BinaryDataType_Bcptl = 16,           // Particle
        //
        BinaryDataType_FORCE32    = 0xFFFFFFFF
    };

    enum MaterialCullingMode // Material culling mode
    {
        MaterialCullingMode_FrontFace = 0,           // Mode to perform culling on the front face polygon
        MaterialCullingMode_BackFace = 1,           // Mode to perform culling on the back face polygon
        MaterialCullingMode_Always = 2,           // Mode to perform culling on both the front face and back face polygons and to make sure it does not display
        MaterialCullingMode_Never = 3,           // Mode that does not perform culling and always displays
        //
        MaterialCullingMode_FORCE32    = 0xFFFFFFFF
    };

    enum TranslucencyKindCtr // Definitions that represent the transparency type.
    {
        TranslucencyKindCtr_Layer0 = 0,           // Identifier that represents the 0th layer
        TranslucencyKindCtr_Layer1 = 1,           // Identifier that represents the 1st layer
        TranslucencyKindCtr_Layer2 = 2,           // Identifier that represents the 2nd layer
        TranslucencyKindCtr_Layer3 = 3,           // Identifier that represents the 3rd layer
        TranslucencyKindCtr_Opaque = 4,           // Identifier for opaque substances
        TranslucencyKindCtr_Translucent = 5,           // Identifier for translucent substances
        TranslucencyKindCtr_Subtractive = 6,           // Identifier for substances that perform subtraction composition
        TranslucencyKindCtr_Additive = 7,           // Identifier for substances that perform addition composition
        //
        TranslucencyKindCtr_FORCE32    = 0xFFFFFFFF
    };

    enum Boolean // Definitions that represent true and false.
    {
        Boolean_FALSE = 0,           // false
        Boolean_TRUE = 1,           // true
        //
        Boolean_FORCE32    = 0xFFFFFFFF
    };

    enum FragmentShaderConfig // Configuration type of the fragment shader
    {
        FragmentShaderConfig_ConfigurationType0 = 0,           // Configuration type ０
        FragmentShaderConfig_ConfigurationType1 = 1,           // Configuration type １
        FragmentShaderConfig_ConfigurationType2 = 2,           // Configuration type ２
        FragmentShaderConfig_ConfigurationType3 = 3,           // Configuration type ３
        FragmentShaderConfig_ConfigurationType4 = 4,           // Configuration type ４
        FragmentShaderConfig_ConfigurationType5 = 5,           // Configuration type ５
        FragmentShaderConfig_ConfigurationType6 = 6,           // Configuration type ６
        FragmentShaderConfig_ConfigurationType7 = 7,           // Configuration type ７
        FragmentShaderConfig_ConfigurationType8 = 8,           // Configuration type ８
        FragmentShaderConfig_ConfigurationType9 = 9,           // Configuration type ９
        FragmentShaderConfig_ConfigurationType10 = 10,           // Configuration type 10
        //
        FragmentShaderConfig_FORCE32    = 0xFFFFFFFF
    };

    enum FresnelConfigCtr // Fresnel factor definition
    {
        FresnelConfigCtr_No = 0,           // Not applied to either the primary color or the secondary color
        FresnelConfigCtr_Pri = 1,           // Applied only to the alpha component of the primary color
        FresnelConfigCtr_Sec = 2,           // Applied only to the alpha component of the secondary color
        FresnelConfigCtr_PriSec = 3,           // Applied to the alpha component of both the primary color and secondary color
        //
        FresnelConfigCtr_FORCE32    = 0xFFFFFFFF
    };

    enum TestFunctionCtr // Definition of types of function to determine test passage
    {
        TestFunctionCtr_Never = 0,           // No values pass
        TestFunctionCtr_Always = 1,           // All values pass
        TestFunctionCtr_Less = 2,           // Passes if smaller than the reference value
        TestFunctionCtr_Lequal = 3,           // Passes if less than or equal to the reference value
        TestFunctionCtr_Equal = 4,           // Passes if equal to the reference value
        TestFunctionCtr_Gequal = 5,           // Passes if greater than or equal to the reference value
        TestFunctionCtr_Greater = 6,           // Passes if larger than the reference value
        TestFunctionCtr_Notequal = 7,           // Passes if not equal to the reference value
        //
        TestFunctionCtr_FORCE32    = 0xFFFFFFFF
    };

    enum StencilTestOperationCtr // Definition of the type of process performed on fragments that pass the stencil test
    {
        StencilTestOperationCtr_Keep = 0,           // Maintains the current value
        StencilTestOperationCtr_Zero = 1,           // Replace with 0
        StencilTestOperationCtr_Replace = 2,           // Replace with the reference value
        StencilTestOperationCtr_Increase = 3,           // Adds. Clamping takes place if the result of the increase is exceeding the extreme value.
        StencilTestOperationCtr_IncreaseWrap = 4,           // Adds. Wrapping takes place if the result of the increase is exceeding the extreme value.
        StencilTestOperationCtr_Decrease = 5,           // Decrease. Clamping takes place if the result of the decrease is exceeding the extreme value.
        StencilTestOperationCtr_DecreaseWrap = 6,           // Decrease. Wrapping takes place if the result of the decrease is exceeding the extreme value.
        StencilTestOperationCtr_Invert = 7,           // Performs reversal in bit units
        //
        StencilTestOperationCtr_FORCE32    = 0xFFFFFFFF
    };

    enum BlendModeCtr // Definition of processes used for blending
    {
        BlendModeCtr_NotUsed = 0,           // Do not use blend processes or logical operations
        BlendModeCtr_Blend = 1,           // Use blend processing
        BlendModeCtr_SeparateBlend = 2,           // Use separate mode blend processing
        BlendModeCtr_Logic = 3,           // Use logical operations
        //
        BlendModeCtr_FORCE32    = 0xFFFFFFFF
    };

    enum LogicOperationCtr // Method for logical operations that use blending
    {
        LogicOperationCtr_Clear = 0,           // Set all bits values to 0
        LogicOperationCtr_Copy = 1,           // Takes source color bit values
        LogicOperationCtr_NoOp = 2,           // Takes destination color bit values
        LogicOperationCtr_Set = 3,           // Set all bits values to 1
        LogicOperationCtr_CopyInverted = 4,           // Takes source reverse bit values
        LogicOperationCtr_Invert = 5,           // Takes destination reverse bit values
        LogicOperationCtr_AndReverse = 6,           // Takes the logical product of the source color and destination color reverse bit values
        LogicOperationCtr_OrReverse = 7,           // Takes the logical sum of the source color and destination color reverse bit values
        LogicOperationCtr_And = 8,           // Takes the logical product of the source color and destination color
        LogicOperationCtr_Or = 9,           // Takes the logical sum of the source color and destination color
        LogicOperationCtr_Nand = 10,           // Takes the reverse bit value of the logical product of the source color and destination color
        LogicOperationCtr_Nor = 11,           // Takes the reverse bit value of the logical sum of the source color and destination color
        LogicOperationCtr_Xor = 12,           // Takes the exclusive OR of the source color and destination color
        LogicOperationCtr_Equiv = 13,           // Takes the reverse bit value of the exclusive OR of the source color and destination color
        LogicOperationCtr_AndInverted = 14,           // Takes the logical product of the reverse bit value of the source color and the destination color
        LogicOperationCtr_OrInverted = 15,           // Takes the logical sum of the reverse bit value of the source color and the destination color
        //
        LogicOperationCtr_FORCE32    = 0xFFFFFFFF
    };

    enum BlendFactorCtr // Blending coefficient
    {
        BlendFactorCtr_Zero = 0,           // Use ０ for the specified component
        BlendFactorCtr_One = 1,           // Use 1 for the specified component
        BlendFactorCtr_SourceColor = 2,           // Use the source color for the specified component
        BlendFactorCtr_OneMinusSourceColor = 3,           // Use 1-SourceColor for the specified component
        BlendFactorCtr_DestinationColor = 4,           // Use the destination color for the specified component
        BlendFactorCtr_OneMinusDestinationColor = 5,           // Uses 1-DestinationColor for the specified component.
        BlendFactorCtr_SourceAlpha = 6,           // Use the alpha value of the source color for the specified component
        BlendFactorCtr_OneMinusSourceAlpha = 7,           // Use 1-SourceAlpha for the specified component
        BlendFactorCtr_DestinationAlpha = 8,           // Use the alpha value of the destination color for the specified component
        BlendFactorCtr_OneMinusDestinationAlpha = 9,           // Use 1-Destination for the specified component
        BlendFactorCtr_ConstantColor = 10,           // Use the constant color for the specified component
        BlendFactorCtr_OneMinusConstantColor = 11,           // Use 1-ConstantColor for the specified component
        BlendFactorCtr_ConstantAlpha = 12,           // Use the alpha value of the constant color for the specified component
        BlendFactorCtr_OneMinusConstantAlpha = 13,           // Use 1-ConstantAlpha for the specified component
        BlendFactorCtr_SourceAlphaSaturate = 14,           // Use the saturation calculations for the specified component
        //
        BlendFactorCtr_FORCE32    = 0xFFFFFFFF
    };

    enum BlendEquationCtr // Blending formula
    {
        BlendEquationCtr_FuncAdd = 0,           // Add source and destination
        BlendEquationCtr_FuncSubtract = 1,           // Subtract destination from source
        BlendEquationCtr_FuncReverseSubtract = 2,           // Subtract source from destination
        BlendEquationCtr_Min = 3,           // Select the smallest of the source and destination
        BlendEquationCtr_Max = 4,           // Select the largest of the source and destination
        //
        BlendEquationCtr_FORCE32    = 0xFFFFFFFF
    };

    enum TextureSamplerMinFilter // Interpolation method for pixels rendered during texture compression
    {
        TextureSamplerMinFilter_Nearest = 0,           // Nearest
        TextureSamplerMinFilter_Linear = 1,           // Linear
        TextureSamplerMinFilter_NearestMipmapNearest = 2,           // NearestMipmapNearest
        TextureSamplerMinFilter_NearestMipmapLinear = 3,           // NearestMipmapLinear
        TextureSamplerMinFilter_LinearMipmapNearest = 4,           // LinearMipmapNearest
        TextureSamplerMinFilter_LinearMipmapLinear = 5,           // LinearMipmapLinear
        //
        TextureSamplerMinFilter_FORCE32    = 0xFFFFFFFF
    };

    enum TextureSamplerMagFilter // Interpolation method for pixels rendered during texture expansion
    {
        TextureSamplerMagFilter_Nearest = 0,           // Nearest
        TextureSamplerMagFilter_Linear = 1,           // Linear
        //
        TextureSamplerMagFilter_FORCE32    = 0xFFFFFFFF
    };

    enum TextureSamplerWrap // Definition for display methods after the texture edge
    {
        TextureSamplerWrap_Repeat = 0,           // Repeat
        TextureSamplerWrap_MirroredRepeat = 1,           // MirroredRepeat
        TextureSamplerWrap_ClampToEdge = 2,           // ClampToEdge
        TextureSamplerWrap_ClampToBorder = 3,           // ClampToBorder
        //
        TextureSamplerWrap_FORCE32    = 0xFFFFFFFF
    };

    enum TextureCoordinateConfigCtr // Definitions for texture coordinate unit assignment settings (the three coordinates 0, 1, and 2 are assigned in numerical order of the last digit of the definition name to four units)
    {
        TextureCoordinateConfigCtr_Config0120 = 0,           // Assigns coordinate numbers 0, 1, 2, and 0 to units.
        TextureCoordinateConfigCtr_Config0110 = 1,           // Assigns coordinate numbers 0, 1, 1, and 0 to units.
        TextureCoordinateConfigCtr_Config0111 = 2,           // Assigns coordinate numbers 0, 1, 1, and 1 to units.
        TextureCoordinateConfigCtr_Config0112 = 3,           // Assigns coordinate numbers 0, 1, 1, and 2 to units.
        TextureCoordinateConfigCtr_Config0121 = 4,           // Assigns coordinate numbers 0, 1, 2, and 1 to units.
        TextureCoordinateConfigCtr_Config0122 = 5,           // Assigns coordinate numbers 0, 1, 2, and 2 to units.
        //
        TextureCoordinateConfigCtr_FORCE32    = 0xFFFFFFFF
    };

    enum TextureMappingMethodCtr // Texture mapping method
    {
        TextureMappingMethodCtr_UvCoordinateMap = 0,           // UvCoordinateMap
        TextureMappingMethodCtr_CameraCubeEnvMap = 1,           // CameraCubeEnvMap
        TextureMappingMethodCtr_CameraSphereEnvMap = 2,           // CameraSphereEnvMap
        TextureMappingMethodCtr_ProjectionMap = 3,           // ProjectionMap
        //
        TextureMappingMethodCtr_FORCE32    = 0xFFFFFFFF
    };

    enum TextureMatrixModeCtr // Texcoord calculation method
    {
        TextureMatrixModeCtr_DccMaya = 0,           // Maya mode
        TextureMatrixModeCtr_DccSoftimage = 1,           // SoftImage mode
        TextureMatrixModeCtr_Dcc3dsMax = 2,           // 3dsMax mode
        //
        TextureMatrixModeCtr_FORCE32    = 0xFFFFFFFF
    };

    enum TextureCombineMode // Definition of the combiner function for the texture combiner
    {
        TextureCombineMode_Replace = 0,           // Use only operation 0
        TextureCombineMode_Modulate = 1,           // Multiply operation 0 and operation 1
        TextureCombineMode_Add = 2,           // Add operation 0 and operation 1
        TextureCombineMode_AddSigned = 3,           // Subtract 0.5 from the sum of operation 0 and operation 1
        TextureCombineMode_Interpolate = 4,           // Interpolate operation 0 and operation 1 with operation 2
        TextureCombineMode_Subtract = 5,           // Subtract operation 1 from operation 0
        TextureCombineMode_DotProduct3Rgb = 6,           // Replace each RGB with the inner product of operation 0 and operation 1
        TextureCombineMode_DotProduct3Rgba = 7,           // Replace each RGBA with the inner product of operation 0 and operation 1
        TextureCombineMode_AddMult = 8,           // Multiply the sum of operation 0 and operation 1 with operation 2
        TextureCombineMode_MultAdd = 9,           // Add the product of operation 0 and operation 1 with operation 2
        //
        TextureCombineMode_FORCE32    = 0xFFFFFFFF
    };

    enum TextureCombineScale // Scale for the output of the texture combiner
    {
        TextureCombineScale_One = 0,           // Same scale mode
        TextureCombineScale_Two = 1,           // Double scale mode
        TextureCombineScale_Four = 2,           // Quadruple scale mode
        //
        TextureCombineScale_FORCE32    = 0xFFFFFFFF
    };

    enum ConstantColorCtr // Defintion of the color used for constant color
    {
        ConstantColorCtr_Constant0 = 0,           // Constant color
        ConstantColorCtr_Constant1 = 1,           // Second constant color
        ConstantColorCtr_Constant2 = 2,           // Third constant color
        ConstantColorCtr_Constant3 = 3,           // Fourth constant color
        ConstantColorCtr_Constant4 = 4,           // Fifth constant color
        ConstantColorCtr_Constant5 = 5,           // Sixth constant color
        ConstantColorCtr_Emission = 6,           // Emission color
        ConstantColorCtr_Ambient = 7,           // Ambient color
        ConstantColorCtr_Diffuse = 8,           // Diffuse color
        ConstantColorCtr_Specular0 = 9,           // Specular color
        ConstantColorCtr_Specular1 = 10,           // Second specular color
        //
        ConstantColorCtr_FORCE32    = 0xFFFFFFFF
    };

    enum TextureCombineBufferInput // Definition of texture combiner buffer input
    {
        TextureCombineBufferInput_Previous = 0,           // Enters the output of the blender for the previous texture combiner to the buffer
        TextureCombineBufferInput_PreviousBuffer = 1,           // Enters the previous buffer into the buffer as is
        //
        TextureCombineBufferInput_FORCE32    = 0xFFFFFFFF
    };

    enum TextureCombinerSourceKind // Definition of texture combiner source
    {
        TextureCombinerSourceKind_Texture0 = 0,           // Texture ０
        TextureCombinerSourceKind_Texture1 = 1,           // Texture １
        TextureCombinerSourceKind_Texture2 = 2,           // Texture ２
        TextureCombinerSourceKind_Texture3 = 3,           // Texture 3
        TextureCombinerSourceKind_Constant = 4,           // Constant color
        TextureCombinerSourceKind_PrimaryColor = 5,           // Fragment color input in texture combiner 0 before texture processing
        TextureCombinerSourceKind_Previous = 6,           // Fragment color input from the previous texture combiner
        TextureCombinerSourceKind_PreviousBuffer = 7,           // Color input from the previous texture combiner buffer
        TextureCombinerSourceKind_FragmentPrimaryColor = 8,           // Fragment light primary color (represents diffuse, ambient,and emissive)
        TextureCombinerSourceKind_FragmentSecondaryColor = 9,           // Fragment light secondary color (represents specular)
        //
        TextureCombinerSourceKind_FORCE32    = 0xFFFFFFFF
    };

    enum TextureCombinerOperandKind // Definition of texture combiner operand
    {
        TextureCombinerOperandKind_Color = 0,           // Source color
        TextureCombinerOperandKind_OneMinusColor = 1,           // 1-Color
        TextureCombinerOperandKind_Alpha = 2,           // Source alpha value
        TextureCombinerOperandKind_OneMinusAlpha = 3,           // 1-Alpha
        TextureCombinerOperandKind_Red = 4,           // Source Red value
        TextureCombinerOperandKind_Green = 5,           // Source Green value
        TextureCombinerOperandKind_Blue = 6,           // Source Blue value
        TextureCombinerOperandKind_OneMinusRed = 7,           // 1-Red
        TextureCombinerOperandKind_OneMinusGreen = 8,           // 1-Green
        TextureCombinerOperandKind_OneMinusBlue = 9,           // 1-Blue
        //
        TextureCombinerOperandKind_FORCE32    = 0xFFFFFFFF
    };

    enum LightingLookupTableInput // Defines types of input values for accessing the reference table
    {
        LightingLookupTableInput_CosNormalHalf = 0,           // Cosine of the angle formed by the normal and half vector
        LightingLookupTableInput_CosViewHalf = 1,           // Cosine of the angle formed by the visual vector and half vector
        LightingLookupTableInput_CosNormalView = 2,           // Cosine of the angle formed by the normal and visual vector
        LightingLookupTableInput_CosLightNormal = 3,           // Cosine of the angle formed by the light vector and the normal
        LightingLookupTableInput_CosLightSpot = 4,           // Cosine of the angle formed by the light vector and spot directional vector
        LightingLookupTableInput_CosPhi = 5,           // Cosine of the angle formed by the projection angle on the half vector's tangential plane and the tangent vector
        //
        LightingLookupTableInput_FORCE32    = 0xFFFFFFFF
    };

    enum LightingLookupTableScale // Defines the type of scale values to apply to the output value taken from the reference table
    {
        LightingLookupTableScale_Quarter = 0,           // 0.25 times
        LightingLookupTableScale_Half = 1,           // 0.5 times
        LightingLookupTableScale_One = 2,           // 1 times
        LightingLookupTableScale_Two = 3,           // 2 times
        LightingLookupTableScale_Four = 4,           // 4 times
        LightingLookupTableScale_Eight = 5,           // 8 times
        //
        LightingLookupTableScale_FORCE32    = 0xFFFFFFFF
    };

    enum BumpTextureIndex // Definition of the texture unit that apply the bump mapping
    {
        BumpTextureIndex_Texture0 = 0,           // Texture ０
        BumpTextureIndex_Texture1 = 1,           // Texture １
        BumpTextureIndex_Texture2 = 2,           // Texture ２
        BumpTextureIndex_Texture3 = 3,           // Texture 3
        //
        BumpTextureIndex_FORCE32    = 0xFFFFFFFF
    };

    enum BumpMode // Bump mapping mode definition
    {
        BumpMode_NotUsed = 0,           // Do not use bump mapping or tangent mapping
        BumpMode_AsBump = 1,           // Bump mapping is used.
        BumpMode_AsTangent = 2,           // Use tangent mapping
        //
        BumpMode_FORCE32    = 0xFFFFFFFF
    };

    enum BoneBillboardMode // Defines the mode for billboards that is supported by bones
    {
        BoneBillboardMode_Off = 0,           // Do not perform billboard processing
        BoneBillboardMode_World = 1,           // Rotate so that the Z-axis is parallel to the point of view axis
        BoneBillboardMode_WorldViewpoint = 2,           // Rotate so that the Z-axis is the direction of the point of view
        BoneBillboardMode_Screen = 3,           // Rotate so that the Z-axis is parallel to the point of view axis and the Y-axis is toward the top of the screen
        BoneBillboardMode_ScreenViewpoint = 4,           // Rotate so that the Z-axis faces in the direction of the point of view and the Y-axis is toward the top of the screen
        BoneBillboardMode_Yaxial = 5,           // Rotate only the Y-axis so that the Z-axis is parallel to the point of view axis
        BoneBillboardMode_YAxialViewpoint = 6,           // Rotate only the Y-axis so that the Z-axis faces in the direction of the point of view
        //
        BoneBillboardMode_FORCE32    = 0xFFFFFFFF
    };

    enum FogKindCtr // Fog type
    {
        FogKindCtr_Linear = 0,           // Linear
        FogKindCtr_Exponent = 1,           // Exponent
        FogKindCtr_ExponentSquare = 2,           // ExponentSquare
        FogKindCtr_None = 3,           // None
        //
        FogKindCtr_FORCE32    = 0xFFFFFFFF
    };

    enum LightKindCtr // Light source type
    {
        LightKindCtr_DirectionalLight = 0,           // Parallel light sources
        LightKindCtr_PointLight = 1,           // Point lights
        LightKindCtr_SpotLight = 2,           // Spot light source
        LightKindCtr_Unused = 3,           // Does not use the light source type of Ambient and HemiSphere.
        //
        LightKindCtr_FORCE32    = 0xFFFFFFFF
    };

    enum SceneAnimationType // Scene animation type
    {
        SceneAnimationType_Camera = 0,           // Camera animation
        SceneAnimationType_Light = 1,           // Light animation
        SceneAnimationType_Fog = 2,           // Fog animation
        //
        SceneAnimationType_FORCE32    = 0xFFFFFFFF
    };

    enum ViewMode // Camera view matrix type
    {
        ViewMode_AimTarget = 0,           // View using the camera target and the camera's rotation angle.
        ViewMode_LookAtTarget = 1,           // View using the camera target's coordinates and up vector.
        ViewMode_Rotate = 2,           // View using the camera's rotation angle.
        //
        ViewMode_FORCE32    = 0xFFFFFFFF
    };

    enum StoreTargetObjectType // Type of object to evacuate
    {
        StoreTargetObjectType_Model = 0,           // The model.
        StoreTargetObjectType_Camera = 1,           // Camera.
        StoreTargetObjectType_Light = 2,           // The light.
        StoreTargetObjectType_Fog = 3,           // Fog.
        //
        StoreTargetObjectType_FORCE32    = 0xFFFFFFFF
    };

    enum UserDataTargetType // User data edit object type
    {
        UserDataTargetType_Model = 0x01,           // model
        UserDataTargetType_Material = 0x02,           // Material
        UserDataTargetType_Mesh = 0x03,           // Mesh
        UserDataTargetType_Texture = 0x04,           // Texture
        UserDataTargetType_Bone = 0x05,           // Bones
        UserDataTargetType_Camera = 0x06,           // Camera
        UserDataTargetType_Fog = 0x07,           // Fog
        UserDataTargetType_Light = 0x08,           // Lights
        UserDataTargetType_Scene = 0x09,           // Scene
        UserDataTargetType_LUTS = 0x0A,           // Lookup table set
        UserDataTargetType_SkeletalAnim = 0x0B,           // Skeletal Animations
        UserDataTargetType_MaterialAnim = 0x0C,           // Material Animations
        UserDataTargetType_ModelAnim = 0x0D,           // Model animation (visibility animation)
        UserDataTargetType_CameraAnim = 0x0E,           // Camera animation
        UserDataTargetType_FogAnim = 0x0F,           // Fog animation.
        UserDataTargetType_LightAnim = 0x10,           // Light animation
        UserDataTargetType_Mask = 0xFF,           // Mask
        //
        UserDataTargetType_FORCE32    = 0xFFFFFFFF
    };

    enum MessageCategory  // Message categories.
    {
        MessageCategory_File = 0x0000,      // File
        MessageCategory_Object = 0x0100,      // Object
        MessageCategory_Material = 0x0200,      // Material
        MessageCategory_Model = 0x0300,      // model
        MessageCategory_Mesh = 0x0400,      // Mesh
        MessageCategory_Bone = 0x0500,      // Bones
        MessageCategory_LUT = 0x0600,      // Lookup table
        MessageCategory_Operation = 0x0700,      // Controls
        MessageCategory_Camera = 0x0800,      // Camera
        MessageCategory_Fog = 0x0900,      // Fog
        MessageCategory_Light = 0x0A00,      // Lights
        MessageCategory_Scene = 0x0B00,      // Scene
        MessageCategory_Animation = 0x0C00,      // Animation
        MessageCategory_UserData = 0x0D00,      // User data
        //
        MessageCategory_FORCE32    = 0xFFFFFFFF
    };

    enum MessageType   // Message types.
    {
        MessageType_File_CloseAll = 0x00 | MessageCategory_File,    // Closes all files.
        MessageType_File_Close = 0x01 | MessageCategory_File,    // Close selected models.
        MessageType_File_LoadBinaryData = 0x02 | MessageCategory_File,    // Load binary data.
        MessageType_Object_Select = 0x00 | MessageCategory_Object,    // Selects.
        MessageType_Object_Forcus = 0x01 | MessageCategory_Object,    // Move focus to selected object.
        MessageType_Material_SetColor = 0x00 | MessageCategory_Material,    // Sets the material color.
        MessageType_Material_SetCullingMode = 0x01 | MessageCategory_Material,    // Sets the culling mode.
        MessageType_Material_SetTranslucencyKind = 0x02 | MessageCategory_Material,    // Sets the transparency.
        MessageType_Material_SetTexture = 0x03 | MessageCategory_Material,    // Sets the texture.
        MessageType_Material_SetFog = 0x04 | MessageCategory_Material,    // Configures fog.
        MessageType_Material_SetLight = 0x05 | MessageCategory_Material,    // Sets the lights.
        MessageType_Material_SetFShaderConfig = 0x06 | MessageCategory_Material,    // Sets the configuration type for the fragment shader.
        MessageType_Material_SetFragmentLighting = 0x07 | MessageCategory_Material,    // Sets the fragment lighting.
        MessageType_Material_SetFragmentLightingTable = 0x08 | MessageCategory_Material,    // Sets the fragment lighting table.
        MessageType_Material_SetAlphaTest = 0x09 | MessageCategory_Material,    // Sets the alpha test.
        MessageType_Material_SetDepthTest = 0x0A | MessageCategory_Material,    // Sets the depth test.
        MessageType_Material_SetStencilTest = 0x0B | MessageCategory_Material,    // Sets the stencil test.
        MessageType_Material_SetBlend = 0x0C | MessageCategory_Material,    // Sets blend.
        MessageType_Material_SetFShaderBufferColor = 0x0D | MessageCategory_Material,    // Sets the buffer color for the fragment shader.
        MessageType_Material_SetTextureCoordinateConfig = 0x0E | MessageCategory_Material,    // Sets the unit assignment for the texture coordinates.
        MessageType_Material_SetTextureMinFilter = 0x0F | MessageCategory_Material,    // Sets MinFilter.
        MessageType_Material_SetTextureMagFilter = 0x10 | MessageCategory_Material,    // Sets MagFilter.
        MessageType_Material_SetTextureWrapS = 0x11 | MessageCategory_Material,    // Sets WrapS.
        MessageType_Material_SetTextureWrapT = 0x12 | MessageCategory_Material,    // Sets WrapT.
        MessageType_Material_SetTextureMinLod = 0x13 | MessageCategory_Material,    // Sets MinLod.
        MessageType_Material_SetTextureLodBias = 0x14 | MessageCategory_Material,    // Sets LodBias.
        MessageType_Material_SetTextureBorderColor = 0x15 | MessageCategory_Material,    // Sets BorderColor.
        MessageType_Material_SetTexCoordSourceCoordinate = 0x16 | MessageCategory_Material,    // Sets SourceCoordinate.
        MessageType_Material_SetTexCoordMappingMethod = 0x17 | MessageCategory_Material,    // Sets MappingMethod.
        MessageType_Material_SetTexCoordReferenceCamera = 0x18 | MessageCategory_Material,    // Sets ReferenceCamera.
        MessageType_Material_SetTexCoordMatrixMode = 0x19 | MessageCategory_Material,    // Sets MatrixMode.
        MessageType_Material_SetTexCoordScaleS = 0x1A | MessageCategory_Material,    // Sets ScaleS.
        MessageType_Material_SetTexCoordScaleT = 0x1B | MessageCategory_Material,    // Sets ScaleT.
        MessageType_Material_SetTexCoordScaleST = 0x1C | MessageCategory_Material,    // Sets ScaleS and ScaleT.
        MessageType_Material_SetTexCoordRotate = 0x1D | MessageCategory_Material,    // Sets Rotate.
        MessageType_Material_SetTexCoordTranslateS = 0x1E | MessageCategory_Material,    // Sets TranslateS.
        MessageType_Material_SetTexCoordTranslateT = 0x1F | MessageCategory_Material,    // Sets TranslateT.
        MessageType_Material_SetTextureCombiner = 0x20 | MessageCategory_Material,    // Sets the texture combiner.
        MessageType_Material_SetBumpTextureIndex = 0x21 | MessageCategory_Material,    // Sets the index for the texture used as a bump map.
        MessageType_Material_SetBumpMode = 0x22 | MessageCategory_Material,    // Sets the bump mapping mode.
        MessageType_Material_SetIsBumpRenormalize = 0x23 | MessageCategory_Material,    // Sets the flag to enable recalculation of the z component of the bump vector.
        MessageType_Material_SetTextureCombinerList = 0x24 | MessageCategory_Material,    // Sets the entire texture combiner.
        MessageType_Material_SetTextureAttributeList = 0x25 | MessageCategory_Material,    // Sets the entire texture configuration.
        MessageType_Material_SetIsPolygonOffsetEnabled = 0x26 | MessageCategory_Material,    // Sets the enable flag for the polygon offset.
        MessageType_Material_SetPolygonOffsetUnit = 0x27 | MessageCategory_Material,    // Sets the unit element of the polygon offset.
        MessageType_Material_SetShaderProgramDescriptionIndex = 0x28 | MessageCategory_Material,    // Sets the vertex shader program index.
        MessageType_Model_SetModelVisibility = 0x00 | MessageCategory_Model,    // Set the hide/show for the model.
        MessageType_Model_SetModelTransformScaleX = 0x01 | MessageCategory_Model,    // Sets the X value for the model TransformScale.
        MessageType_Model_SetModelTransformScaleY = 0x02 | MessageCategory_Model,    // Sets the Y value for the model TransformScale.
        MessageType_Model_SetModelTransformScaleZ = 0x03 | MessageCategory_Model,    // Sets the Z value for the model TransformScale.
        MessageType_Model_SetModelTransformRotateX = 0x04 | MessageCategory_Model,    // Sets the X value for the model TransformRotate.
        MessageType_Model_SetModelTransformRotateY = 0x05 | MessageCategory_Model,    // Sets the Y value for the model TransformRotate.
        MessageType_Model_SetModelTransformRotateZ = 0x06 | MessageCategory_Model,    // Sets the Z value for the model TransformRotate.
        MessageType_Model_SetModelTransformTranslateX = 0x07 | MessageCategory_Model,    // Sets the X value for the model TransformTranslate.
        MessageType_Model_SetModelTransformTranslateY = 0x08 | MessageCategory_Model,    // Sets the Y value for the model TransformTranslate.
        MessageType_Model_SetModelTransformTranslateZ = 0x09 | MessageCategory_Model,    // Sets the Z value for the model TransformTranslate.
        MessageType_Model_SetModelPreviewVisibility = 0x0A | MessageCategory_Model,    // Set the hide/show for the model on the viewer.
        MessageType_Model_SetModelPreviewTransformScaleX = 0x0B | MessageCategory_Model,    // Sets the X value for the model TransformScale on the viewer.
        MessageType_Model_SetModelPreviewTransformScaleY = 0x0C | MessageCategory_Model,    // Sets the Y value for the model TransformScale on the viewer.
        MessageType_Model_SetModelPreviewTransformScaleZ = 0x0D | MessageCategory_Model,    // Sets the Z value for the model TransformScale on the viewer.
        MessageType_Model_SetModelPreviewTransformRotateX = 0x0E | MessageCategory_Model,    // Sets the X value for the model TransformRotate on the viewer.
        MessageType_Model_SetModelPreviewTransformRotateY = 0x0F | MessageCategory_Model,    // Sets the Y value for the model TransformRotate on the viewer.
        MessageType_Model_SetModelPreviewTransformRotateZ = 0x10 | MessageCategory_Model,    // Sets the Z value for the model TransformRotate on the viewer.
        MessageType_Model_SetModelPreviewTransformTranslateX = 0x11 | MessageCategory_Model,    // Sets the X value for the model TransformTranslate on the viewer.
        MessageType_Model_SetModelPreviewTransformTranslateY = 0x12 | MessageCategory_Model,    // Sets the Y value for the model TransformTranslate on the viewer.
        MessageType_Model_SetModelPreviewTransformTranslateZ = 0x13 | MessageCategory_Model,    // Sets the Z value for the model TransformTranslate on the viewer.
        MessageType_Model_SetModelTransformValue = 0x14 | MessageCategory_Model,    // Sets the value for model transformation.
        MessageType_Model_SetModelPreviewTransformValue = 0x15 | MessageCategory_Model,    // Sets the value for model transformation on the viewer.
        MessageType_Model_SetModelPreviewBoneLink = 0x16 | MessageCategory_Model,    // Set the model constraint on the viewer.
        MessageType_Model_SetModelPreviewBoneUnLink = 0x17 | MessageCategory_Model,    // Remove the model constraint on the viewer.
        MessageType_Mesh_SetMeshVisibility = 0x00 | MessageCategory_Mesh,    // Set the hide/show for the mesh.
        MessageType_Mesh_SetMeshNodeVisibility = 0x01 | MessageCategory_Mesh,    // Set the hide/show for the mesh node.
        MessageType_Mesh_SetMeshRenderPriority = 0x02 | MessageCategory_Mesh,    // Sets the mesh render priority.
        MessageType_Bone_SetBoneBillboard = 0x00 | MessageCategory_Bone,    // Sets the bone billboard mode.
        MessageType_LUT_ModifyLookUpTable = 0x00 | MessageCategory_LUT,    // Sets the changed LUT.
        MessageType_Operation_EnableDrawing = 0x00 | MessageCategory_Operation,    // Restarts the Viewer drawing.
        MessageType_Operation_DisableDrawing = 0x01 | MessageCategory_Operation,    // Stops the Viewer drawing.
        MessageType_Operation_CaptureScreen = 0x02 | MessageCategory_Operation,    // Captures the Viewer top screen.
        MessageType_Camera_SetTargetViewAim = 0x00 | MessageCategory_Camera,    // Sets the camera view (Aim).
        MessageType_Camera_SetTargetViewLookAt = 0x01 | MessageCategory_Camera,    // Sets the camera view (LookAt).
        MessageType_Camera_SetTargetViewRotate = 0x02 | MessageCategory_Camera,    // Sets the camera view (Rotate).
        MessageType_Camera_SetOrthoProjection = 0x03 | MessageCategory_Camera,    // Sets the project matrix with orthogonal projection.
        MessageType_Camera_SetPerspectiveProjection = 0x04 | MessageCategory_Camera,    // Sets the project matrix using the angle of view.
        MessageType_Camera_SetCameraTransform = 0x05 | MessageCategory_Camera,    // Sets Transform for the camera.
        MessageType_Camera_SetCameraIsWBuffer = 0x06 | MessageCategory_Camera,    // Sets the camera's W buffer.
        MessageType_Camera_GetCameraInfo = 0x07 | MessageCategory_Camera,    // Gets the camera information.
        MessageType_Fog_SetFog = 0x00 | MessageCategory_Fog,    // Configures fog.
        MessageType_Fog_SetFogIsZFlip = 0x01 | MessageCategory_Fog,    // Sets the fog Z-flip enable flag.
        MessageType_Fog_SetFogColor = 0x02 | MessageCategory_Fog,    // This function sets the fog color.
        MessageType_Fog_SetMinFogDepth = 0x03 | MessageCategory_Fog,    // SGets the depth value in the view coordinate system where fog begins.
        MessageType_Fog_SetMaxFogDepth = 0x04 | MessageCategory_Fog,    // Sets the depth value in the view coordinate system where fog density is maximum.
        MessageType_Fog_SetFogDensity = 0x05 | MessageCategory_Fog,    // Sets parameters for adjusting fog density.
        MessageType_Light_SetLightIsLightEnabled = 0x00 | MessageCategory_Light,    // Sets the light enable flag.
        MessageType_Light_SetLightTranslate = 0x01 | MessageCategory_Light,    // Sets Transform (Translate) for lights.
        MessageType_Light_SetAmbientLightColor = 0x02 | MessageCategory_Light,    // Sets the ambient light color.
        MessageType_Light_SetHemiSphereGroundColor = 0x03 | MessageCategory_Light,    // Sets the ground color.
        MessageType_Light_SetHemiSphereSkyColor = 0x04 | MessageCategory_Light,    // Sets the sky color.
        MessageType_Light_SetHemiSphereDirection = 0x05 | MessageCategory_Light,    // Sets the light direction.
        MessageType_Light_SetHemiSphereLerpFactor = 0x06 | MessageCategory_Light,    // Sets the linear interpolation factor.
        MessageType_Light_SetHemiSphereIsInheritingDirectionRotate = 0x07 | MessageCategory_Light,    // Sets the parent node rotation inherit enable flag.
        MessageType_Light_SetVLLightKind = 0x08 | MessageCategory_Light,    // Sets the light type.
        MessageType_Light_SetVLAmbient = 0x09 | MessageCategory_Light,    // Configures ambient light color.
        MessageType_Light_SetVLDiffuse = 0x0A | MessageCategory_Light,    // Sets the diffuse color.
        MessageType_Light_SetVLDirection = 0x0B | MessageCategory_Light,    // Sets the vertex light direction.
        MessageType_Light_SetVLIsInheritingDirectionRotate = 0x0C | MessageCategory_Light,    // Sets the parent node rotation inherit enable flag.
        MessageType_Light_SetVLDistanceAttenuationConstant = 0x0D | MessageCategory_Light,    // Set the distance attenuation constant attenuation coefficient.
        MessageType_Light_SetVLDistanceAttenuationLinear = 0x0E | MessageCategory_Light,    // Set the distance attenuation linear attenuation coefficient.
        MessageType_Light_SetVLDistanceAttenuationQuadratic = 0x0F | MessageCategory_Light,    // Set the distance attenuation secondary attenuation coefficient.
        MessageType_Light_SetVLIsDistanceAttenuationEnabled = 0x10 | MessageCategory_Light,    // Sets the enable flag for the distance attenuation.
        MessageType_Light_SetVLSpotExponent = 0x11 | MessageCategory_Light,    // Sets a spotlight's angular attenuation coefficients.
        MessageType_Light_SetVLSpotCutoffAngle = 0x12 | MessageCategory_Light,    // Sets the cut-off angle for the spotlight.
        MessageType_Light_SetFLKind = 0x13 | MessageCategory_Light,    // Sets the light type.
        MessageType_Light_SetFLAmbient = 0x14 | MessageCategory_Light,    // Configures ambient light color.
        MessageType_Light_SetFLDiffuse = 0x15 | MessageCategory_Light,    // Sets the diffuse color.
        MessageType_Light_SetFLIsTwoSideDiffuse = 0x16 | MessageCategory_Light,    // Sets the enable flag for both face lighting for diffuse color.
        MessageType_Light_SetFLSpecular0 = 0x17 | MessageCategory_Light,    // Sets specular color.
        MessageType_Light_SetFLSpecular1 = 0x18 | MessageCategory_Light,    // Sets the second specular color.
        MessageType_Light_SetFLIsInheritingDirectionRotate = 0x19 | MessageCategory_Light,    // Sets the parent node rotation inherit enable flag.
        MessageType_Light_SetFLDirection = 0x1A | MessageCategory_Light,    // Sets the light direction.
        MessageType_Light_SetFLSamplerLUTS = 0x1B | MessageCategory_Light,    // Sets the reference table set in AngleSampler and DistanceSampler.
        MessageType_Light_SetFLIsDistanceAttenuationEnabled = 0x1C | MessageCategory_Light,    // Sets the enable flag for the distance attenuation.
        MessageType_Light_SetFLIsDistanceAttenuationStart = 0x1D | MessageCategory_Light,    // Sets the distance where the light brightness begins to be attenuated.
        MessageType_Light_SetFLIsDistanceAttenuationEnd = 0x1E | MessageCategory_Light,    // Sets the distance where the light brightness stops being attenuated.
        MessageType_Light_SetFLDistanceSamplerLUT = 0x1F | MessageCategory_Light,    // Sets reference tables for distance samplers.
        MessageType_Light_SetFLAngleSamplerLUT = 0x20 | MessageCategory_Light,    // Sets reference tables for angle samplers.
        MessageType_Light_SetFLAngleSamplerInput = 0x21 | MessageCategory_Light,    // Sets input values for accessing reference tables for angle samplers.
        MessageType_Light_SetFLAngleSamplerScale = 0x22 | MessageCategory_Light,    // Sets the scale value type applied to the output value taken from the angle sampler reference table.
        MessageType_Light_GetFragmentLightInfo = 0x23 | MessageCategory_Light,    // Gets the fragment light information.
        MessageType_Scene_SetSceneActive = 0x00 | MessageCategory_Scene,    // Sets a scene to active.
        MessageType_Animation_AddAnimationSet = 0x00 | MessageCategory_Animation,    // Configures the association of an animation set and a model.
        MessageType_Animation_AddStrayAnimations = 0x01 | MessageCategory_Animation,    // Configures the association of an animation not in an animation set and a model.
        MessageType_Animation_ActivateAnimationSet = 0x02 | MessageCategory_Animation,    // Sets the animation set binding.
        MessageType_Animation_DeActivateAnimationSet = 0x03 | MessageCategory_Animation,    // Sets the animation set unbinding.
        MessageType_Animation_ClearAnimationSet = 0x04 | MessageCategory_Animation,    // Clears the animation set.
        MessageType_Animation_AddLightAnimation = 0x05 | MessageCategory_Animation,    // Adds a light animation set.
        MessageType_Animation_AddCameraAnimation = 0x06 | MessageCategory_Animation,    // Adds a camera animation set.
        MessageType_Animation_AddFogAnimation = 0x07 | MessageCategory_Animation,    // Adds a fog animation set.
        MessageType_Animation_RenameAnimationSet = 0x08 | MessageCategory_Animation,    // Renames an animation set.
        MessageType_Animation_BindSceneAnimation = 0x09 | MessageCategory_Animation,    // Sets the bind for the camera, light, and fog animation.
        MessageType_Animation_UnBindSceneAnimation = 0x0A | MessageCategory_Animation,    // Releases the bind for the camera, light, and fog animation.
        MessageType_Animation_StoreCurrentFrame = 0x0B | MessageCategory_Animation,    // Evacuates the current frame.
        MessageType_Animation_RestoreCurrentFrame = 0x0C | MessageCategory_Animation,    // Restores the current frame.
        MessageType_UserData_SendUserData = 0x00 | MessageCategory_UserData,    // Sends edited UserData.
        //
        MessageType_FORCE32    = 0xFFFFFFFF
    };

    struct Header
    {
        static const int StructSize = 12;

        MessageType type;
        int size;
        int arraySize;
    };

    typedef int Int;
    typedef unsigned int Uint;
    typedef float Float;

    struct String
    {
        wchar_t Value[260];

        const wchar_t *ToString()
        {
            return Value;
        }

    };

    struct Rgb
    {
        float R;
        float G;
        float B;
    };

    struct Rgba
    {
        float R;
        float G;
        float B;
        float A;
    };

    struct Vector2
    {
        float X;
        float Y;
    };

    struct Vector3
    {
        float X;
        float Y;
        float Z;
    };

    struct Vector4
    {
        float X;
        float Y;
        float Z;
        float W;
    };

    // Closes all files.
    struct MESSAGEPACKET_File_CloseAll
    {
        static const MessageType MessagePacketType = MessageType_File_CloseAll;    // Message types.
        static const int StructSize = 0 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

    };

    // Close selected models.
    struct MESSAGEPACKET_File_Close
    {
        static const MessageType MessagePacketType = MessageType_File_Close;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Type = 1<<0,
            PropertyFlag_Id = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        BinaryDataType    Type;       // Type.
        Uint    Id;       // Returns the ID.
    };

    // Load binary data.
    struct MESSAGEPACKET_File_LoadBinaryData
    {
        static const MessageType MessagePacketType = MessageType_File_LoadBinaryData;    // Message types.
        static const int StructSize = 532 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Type = 1<<0,
            PropertyFlag_Id = 1<<1,
            PropertyFlag_FileSize = 1<<2,
            PropertyFlag_FilePath = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        BinaryDataType    Type;       // Type.
        Uint    Id;       // Returns the ID.
        Uint    FileSize;       // The size of the file.
        String    FilePath;       // The file path.
    };

    // Selects.
    struct MESSAGEPACKET_Object_Select
    {
        static const MessageType MessagePacketType = MessageType_Object_Select;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ObjectType = 1<<0,
            PropertyFlag_ObjectCount = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        EditTargetType    ObjectType;       // Object type.
        Int    ObjectCount;       // Number of objects.
    };

    // Move focus to selected object.
    struct MESSAGEPACKET_Object_Forcus
    {
        static const MessageType MessagePacketType = MessageType_Object_Forcus;    // Message types.
        static const int StructSize = 0 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

    };

    // Sets the material color.
    struct MESSAGEPACKET_Material_SetColor
    {
        static const MessageType MessagePacketType = MessageType_Material_SetColor;    // Message types.
        static const int StructSize = 164 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_VertexColorScale = 1<<0,
            PropertyFlag_Emission = 1<<1,
            PropertyFlag_Ambient = 1<<2,
            PropertyFlag_Diffuse = 1<<3,
            PropertyFlag_Specular0 = 1<<4,
            PropertyFlag_Specular1 = 1<<5,
            PropertyFlag_Constant0 = 1<<6,
            PropertyFlag_Constant1 = 1<<7,
            PropertyFlag_Constant2 = 1<<8,
            PropertyFlag_Constant3 = 1<<9,
            PropertyFlag_Constant4 = 1<<10,
            PropertyFlag_Constant5 = 1<<11,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    VertexColorScale;       // Vertex color scaling
        Rgb    Emission;       // Emission color
        Rgb    Ambient;       // Ambient color
        Rgba    Diffuse;       // Diffuse color
        Rgb    Specular0;       // Specular color
        Rgb    Specular1;       // Specular color
        Rgba    Constant0;       // Constant
        Rgba    Constant1;       // Constant
        Rgba    Constant2;       // Constant
        Rgba    Constant3;       // Constant
        Rgba    Constant4;       // Constant
        Rgba    Constant5;       // Constant
    };

    // Sets the culling mode.
    struct MESSAGEPACKET_Material_SetCullingMode
    {
        static const MessageType MessagePacketType = MessageType_Material_SetCullingMode;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Mode = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        MaterialCullingMode    Mode;       // Culling mode
    };

    // Sets the transparency.
    struct MESSAGEPACKET_Material_SetTranslucencyKind
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTranslucencyKind;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Kind = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        TranslucencyKindCtr    Kind;       // Translucency
    };

    // Sets the texture.
    struct MESSAGEPACKET_Material_SetTexture
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexture;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Id = 1<<0,
            PropertyFlag_Index = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Id;       // ID (hash code for the texture name).
        Uint    Index;       // Index (index in the texture map).
    };

    // Configures fog.
    struct MESSAGEPACKET_Material_SetFog
    {
        static const MessageType MessagePacketType = MessageType_Material_SetFog;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Enable = 1<<0,
            PropertyFlag_FogIndex = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    Enable;       // Fog calculation enable flag
        Int    FogIndex;       // Fog number
    };

    // Sets the lights.
    struct MESSAGEPACKET_Material_SetLight
    {
        static const MessageType MessagePacketType = MessageType_Material_SetLight;    // Message types.
        static const int StructSize = 20 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsFragmentLightEnabled = 1<<0,
            PropertyFlag_IsVertexLightEnabled = 1<<1,
            PropertyFlag_IsHemiSphereLightEnabled = 1<<2,
            PropertyFlag_IsHemiSphereOcclusionEnabled = 1<<3,
            PropertyFlag_LightSetIndex = 1<<4,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsFragmentLightEnabled;       // Fragment light flag
        Boolean    IsVertexLightEnabled;       // Vertex light flag
        Boolean    IsHemiSphereLightEnabled;       // Hemisphere light flag
        Boolean    IsHemiSphereOcclusionEnabled;       // Hemisphere light making item flag
        Int    LightSetIndex;       // Light set index
    };

    // Sets the configuration type for the fragment shader.
    struct MESSAGEPACKET_Material_SetFShaderConfig
    {
        static const MessageType MessagePacketType = MessageType_Material_SetFShaderConfig;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Config = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        FragmentShaderConfig    Config;       // Configuration type of the fragment shader
    };

    // Sets the fragment lighting.
    struct MESSAGEPACKET_Material_SetFragmentLighting
    {
        static const MessageType MessagePacketType = MessageType_Material_SetFragmentLighting;    // Message types.
        static const int StructSize = 28 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_FresnelConfig = 1<<0,
            PropertyFlag_IsClampHighLight = 1<<1,
            PropertyFlag_IsDistribution0Enabled = 1<<2,
            PropertyFlag_IsDistribution1Enabled = 1<<3,
            PropertyFlag_IsGeometricFactor0Enabled = 1<<4,
            PropertyFlag_IsGeometricFactor1Enabled = 1<<5,
            PropertyFlag_IsReflectionEnabled = 1<<6,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        FresnelConfigCtr    FresnelConfig;       // Fresnel factors
        Boolean    IsClampHighLight;       // Specular color clamp enable flag
        Boolean    IsDistribution0Enabled;       // Distribution factor ０ enable flag
        Boolean    IsDistribution1Enabled;       // Distribution factor 1 enable flag
        Boolean    IsGeometricFactor0Enabled;       // Geometric factor ０ enable flag
        Boolean    IsGeometricFactor1Enabled;       // Geometric factor 1 enable flag
        Boolean    IsReflectionEnabled;       // Refraction factor enable flag
    };

    // Sets the fragment lighting table.
    struct MESSAGEPACKET_Material_SetFragmentLightingTable
    {
        static const MessageType MessagePacketType = MessageType_Material_SetFragmentLightingTable;    // Message types.
        static const int StructSize = 3712 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LookupTableSetName = 1<<0,
            PropertyFlag_RefSamplerRIsAbs = 1<<1,
            PropertyFlag_RefSamplerRInput = 1<<2,
            PropertyFlag_RefSamplerRScale = 1<<3,
            PropertyFlag_RefSamplerRLutName = 1<<4,
            PropertyFlag_RefSamplerGIsAbs = 1<<5,
            PropertyFlag_RefSamplerGInput = 1<<6,
            PropertyFlag_RefSamplerGScale = 1<<7,
            PropertyFlag_RefSamplerGLutName = 1<<8,
            PropertyFlag_RefSamplerBIsAbs = 1<<9,
            PropertyFlag_RefSamplerBInput = 1<<10,
            PropertyFlag_RefSamplerBScale = 1<<11,
            PropertyFlag_RefSamplerBLutName = 1<<12,
            PropertyFlag_Dis0SamplerIsAbs = 1<<13,
            PropertyFlag_Dis0SamplerInput = 1<<14,
            PropertyFlag_Dis0SamplerScale = 1<<15,
            PropertyFlag_Dis0SamplerLutName = 1<<16,
            PropertyFlag_Dis1SamplerIsAbs = 1<<17,
            PropertyFlag_Dis1SamplerInput = 1<<18,
            PropertyFlag_Dis1SamplerScale = 1<<19,
            PropertyFlag_Dis1SamplerLutName = 1<<20,
            PropertyFlag_FresnelSamplerIsAbs = 1<<21,
            PropertyFlag_FresnelSamplerInput = 1<<22,
            PropertyFlag_FresnelSamplerScale = 1<<23,
            PropertyFlag_FresnelSamplerLutName = 1<<24,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        String    LookupTableSetName;       // Name of LookupTableSet to reference.
        Boolean    RefSamplerRIsAbs;       // Flag to make input value an absolute value to access the reference table
        LightingLookupTableInput    RefSamplerRInput;       // Input value to access the reference table
        LightingLookupTableScale    RefSamplerRScale;       // Scale value to apply to the output value taken from the reference table
        String    RefSamplerRLutName;       // Name of ImageLookupTableCtr to reference.
        Boolean    RefSamplerGIsAbs;       // Flag to make input value an absolute value to access the reference table
        LightingLookupTableInput    RefSamplerGInput;       // Input value to access the reference table
        LightingLookupTableScale    RefSamplerGScale;       // Scale value to apply to the output value taken from the reference table
        String    RefSamplerGLutName;       // Name of ImageLookupTableCtr to reference.
        Boolean    RefSamplerBIsAbs;       // Flag to make input value an absolute value to access the reference table
        LightingLookupTableInput    RefSamplerBInput;       // Input value to access the reference table
        LightingLookupTableScale    RefSamplerBScale;       // Scale value to apply to the output value taken from the reference table
        String    RefSamplerBLutName;       // Name of ImageLookupTableCtr to reference.
        Boolean    Dis0SamplerIsAbs;       // Flag to make input value an absolute value to access the reference table
        LightingLookupTableInput    Dis0SamplerInput;       // Input value to access the reference table
        LightingLookupTableScale    Dis0SamplerScale;       // Scale value to apply to the output value taken from the reference table
        String    Dis0SamplerLutName;       // Name of ImageLookupTableCtr to reference.
        Boolean    Dis1SamplerIsAbs;       // Flag to make input value an absolute value to access the reference table
        LightingLookupTableInput    Dis1SamplerInput;       // Input value to access the reference table
        LightingLookupTableScale    Dis1SamplerScale;       // Scale value to apply to the output value taken from the reference table
        String    Dis1SamplerLutName;       // Name of ImageLookupTableCtr to reference.
        Boolean    FresnelSamplerIsAbs;       // Flag to make input value an absolute value to access the reference table
        LightingLookupTableInput    FresnelSamplerInput;       // Input value to access the reference table
        LightingLookupTableScale    FresnelSamplerScale;       // Scale value to apply to the output value taken from the reference table
        String    FresnelSamplerLutName;       // Name of ImageLookupTableCtr to reference.
    };

    // Sets the alpha test.
    struct MESSAGEPACKET_Material_SetAlphaTest
    {
        static const MessageType MessagePacketType = MessageType_Material_SetAlphaTest;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsTestEnabled = 1<<0,
            PropertyFlag_TestFunction = 1<<1,
            PropertyFlag_TestReference = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsTestEnabled;       // Flag to enable alpha test
        TestFunctionCtr    TestFunction;       // Comparison function for alpha test
        Float    TestReference;       // Reference value for alpha test
    };

    // Sets the depth test.
    struct MESSAGEPACKET_Material_SetDepthTest
    {
        static const MessageType MessagePacketType = MessageType_Material_SetDepthTest;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsTestEnabled = 1<<0,
            PropertyFlag_TestFunction = 1<<1,
            PropertyFlag_IsMaskEnabled = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsTestEnabled;       // Depth test enable flag
        TestFunctionCtr    TestFunction;       // Comparison function used in the depth test
        Boolean    IsMaskEnabled;       // Flag to write the depth value to the frame buffer
    };

    // Sets the stencil test.
    struct MESSAGEPACKET_Material_SetStencilTest
    {
        static const MessageType MessagePacketType = MessageType_Material_SetStencilTest;    // Message types.
        static const int StructSize = 28 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsTestEnabled = 1<<0,
            PropertyFlag_TestFunction = 1<<1,
            PropertyFlag_TestReference = 1<<2,
            PropertyFlag_TestMask = 1<<3,
            PropertyFlag_FailOperation = 1<<4,
            PropertyFlag_ZFailOperation = 1<<5,
            PropertyFlag_PassOperation = 1<<6,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsTestEnabled;       // Flag to enable stencil test
        TestFunctionCtr    TestFunction;       // Stencil test pass function
        Int    TestReference;       // Stencil test pass reference value
        Uint    TestMask;       // Stencil test pass mask
        StencilTestOperationCtr    FailOperation;       // Operation on fragments that fail stencil test
        StencilTestOperationCtr    ZFailOperation;       // Operation on fragments that pass stencil test but fail depth test
        StencilTestOperationCtr    PassOperation;       // Operation on fragments that pass both stencil and depth tests, or pass stencil test when depth test is disabled
    };

    // Sets blend.
    struct MESSAGEPACKET_Material_SetBlend
    {
        static const MessageType MessagePacketType = MessageType_Material_SetBlend;    // Message types.
        static const int StructSize = 48 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Mode = 1<<0,
            PropertyFlag_LogicOperation = 1<<1,
            PropertyFlag_RgbFuncSource = 1<<2,
            PropertyFlag_RgbFuncDestination = 1<<3,
            PropertyFlag_RgbEquation = 1<<4,
            PropertyFlag_AlphaFuncSource = 1<<5,
            PropertyFlag_AlphaFuncDestination = 1<<6,
            PropertyFlag_AlphaEquation = 1<<7,
            PropertyFlag_BlendColor = 1<<8,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        BlendModeCtr    Mode;       // Blend mode
        LogicOperationCtr    LogicOperation;       // Logical operation
        BlendFactorCtr    RgbFuncSource;       // Component used as the source color during blending
        BlendFactorCtr    RgbFuncDestination;       // Component used as the destination color during blending
        BlendEquationCtr    RgbEquation;       // Calculation method used during blending
        BlendFactorCtr    AlphaFuncSource;       // Component used as the source color during blending
        BlendFactorCtr    AlphaFuncDestination;       // Component used as the destination color during blending
        BlendEquationCtr    AlphaEquation;       // Calculation method used during blending
        Rgba    BlendColor;       // Blending source color
    };

    // Sets the buffer color for the fragment shader.
    struct MESSAGEPACKET_Material_SetFShaderBufferColor
    {
        static const MessageType MessagePacketType = MessageType_Material_SetFShaderBufferColor;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_BufferColor = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    BufferColor;       // Buffer color
    };

    // Sets the unit assignment for the texture coordinates.
    struct MESSAGEPACKET_Material_SetTextureCoordinateConfig
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureCoordinateConfig;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_TextureCoordinateConfig = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        TextureCoordinateConfigCtr    TextureCoordinateConfig;       // TextureCoordinater settings
    };

    // Sets MinFilter.
    struct MESSAGEPACKET_Material_SetTextureMinFilter
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureMinFilter;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_MinFilter = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexMap in Material.
        TextureSamplerMinFilter    MinFilter;       // MinFilter settings
    };

    // Sets MagFilter.
    struct MESSAGEPACKET_Material_SetTextureMagFilter
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureMagFilter;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_MagFilter = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexMap in Material.
        TextureSamplerMagFilter    MagFilter;       // MagFilter settings
    };

    // Sets WrapS.
    struct MESSAGEPACKET_Material_SetTextureWrapS
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureWrapS;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_WrapS = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexMap in Material.
        TextureSamplerWrap    WrapS;       // WrapS settings
    };

    // Sets WrapT.
    struct MESSAGEPACKET_Material_SetTextureWrapT
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureWrapT;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_WrapT = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexMap in Material.
        TextureSamplerWrap    WrapT;       // WrapT settings
    };

    // Sets MinLod.
    struct MESSAGEPACKET_Material_SetTextureMinLod
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureMinLod;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_MinLod = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexMap in Material.
        Uint    MinLod;       // MinLod settings
    };

    // Sets LodBias.
    struct MESSAGEPACKET_Material_SetTextureLodBias
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureLodBias;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_LodBias = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexMap in Material.
        Float    LodBias;       // LodBias settings
    };

    // Sets BorderColor.
    struct MESSAGEPACKET_Material_SetTextureBorderColor
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureBorderColor;    // Message types.
        static const int StructSize = 20 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_BorderColor = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexMap in Material.
        Rgba    BorderColor;       // BorderColor settings
    };

    // Sets SourceCoordinate.
    struct MESSAGEPACKET_Material_SetTexCoordSourceCoordinate
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordSourceCoordinate;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_SourceCoordinate = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Int    SourceCoordinate;       // SourceCoordinate settings
    };

    // Sets MappingMethod.
    struct MESSAGEPACKET_Material_SetTexCoordMappingMethod
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordMappingMethod;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_MappingMethod = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        TextureMappingMethodCtr    MappingMethod;       // MappingMethod settings
    };

    // Sets ReferenceCamera.
    struct MESSAGEPACKET_Material_SetTexCoordReferenceCamera
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordReferenceCamera;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_ReferenceCamera = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Int    ReferenceCamera;       // ReferenceCamera settings
    };

    // Sets MatrixMode.
    struct MESSAGEPACKET_Material_SetTexCoordMatrixMode
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordMatrixMode;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_MatrixMode = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        TextureMatrixModeCtr    MatrixMode;       // MatrixMode settings
    };

    // Sets ScaleS.
    struct MESSAGEPACKET_Material_SetTexCoordScaleS
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordScaleS;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_ScaleS = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Float    ScaleS;       // ScaleS settings
    };

    // Sets ScaleT.
    struct MESSAGEPACKET_Material_SetTexCoordScaleT
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordScaleT;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_ScaleT = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Float    ScaleT;       // ScaleT settings
    };

    // Sets ScaleS and ScaleT.
    struct MESSAGEPACKET_Material_SetTexCoordScaleST
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordScaleST;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_ScaleS = 1<<1,
            PropertyFlag_ScaleT = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Float    ScaleS;       // ScaleS settings
        Float    ScaleT;       // ScaleT settings
    };

    // Sets Rotate.
    struct MESSAGEPACKET_Material_SetTexCoordRotate
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordRotate;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_Rotate = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Float    Rotate;       // Rotate settings
    };

    // Sets TranslateS.
    struct MESSAGEPACKET_Material_SetTexCoordTranslateS
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordTranslateS;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_TranslateS = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Float    TranslateS;       // TranslateS settings
    };

    // Sets TranslateT.
    struct MESSAGEPACKET_Material_SetTexCoordTranslateT
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTexCoordTranslateT;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_TranslateT = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TexCoordinator in Material.
        Float    TranslateT;       // TranslateT settings
    };

    // Sets the texture combiner.
    struct MESSAGEPACKET_Material_SetTextureCombiner
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureCombiner;    // Message types.
        static const int StructSize = 88 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_CombineRgb = 1<<1,
            PropertyFlag_CombineAlpha = 1<<2,
            PropertyFlag_ScaleRgb = 1<<3,
            PropertyFlag_ScaleAlpha = 1<<4,
            PropertyFlag_Constant = 1<<5,
            PropertyFlag_BufferInputRgb = 1<<6,
            PropertyFlag_OutputBufferRgb = 1<<7,
            PropertyFlag_BufferInputAlpha = 1<<8,
            PropertyFlag_OutputBufferAlpha = 1<<9,
            PropertyFlag_SourceRgb0 = 1<<10,
            PropertyFlag_SourceRgb1 = 1<<11,
            PropertyFlag_SourceRgb2 = 1<<12,
            PropertyFlag_OperandRgb0 = 1<<13,
            PropertyFlag_OperandRgb1 = 1<<14,
            PropertyFlag_OperandRgb2 = 1<<15,
            PropertyFlag_SourceAlpha0 = 1<<16,
            PropertyFlag_SourceAlpha1 = 1<<17,
            PropertyFlag_SourceAlpha2 = 1<<18,
            PropertyFlag_OperandAlpha0 = 1<<19,
            PropertyFlag_OperandAlpha1 = 1<<20,
            PropertyFlag_OperandAlpha2 = 1<<21,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index in the TextureCombiner list.
        TextureCombineMode    CombineRgb;       // Color combiner function
        TextureCombineMode    CombineAlpha;       // Alpha combiner function
        TextureCombineScale    ScaleRgb;       // Scale for the output color
        TextureCombineScale    ScaleAlpha;       // Scale for the output alpha
        ConstantColorCtr    Constant;       // Constant color index
        TextureCombineBufferInput    BufferInputRgb;       // Color buffer input
        Boolean    OutputBufferRgb;       // Color next step buffer output
        TextureCombineBufferInput    BufferInputAlpha;       // Alpha buffer input
        Boolean    OutputBufferAlpha;       // Alpha next step buffer output
        TextureCombinerSourceKind    SourceRgb0;       // Color value source 0
        TextureCombinerSourceKind    SourceRgb1;       // Color value source 1
        TextureCombinerSourceKind    SourceRgb2;       // Color value source 2
        TextureCombinerOperandKind    OperandRgb0;       // Color value operand 0
        TextureCombinerOperandKind    OperandRgb1;       // Color value operand 1
        TextureCombinerOperandKind    OperandRgb2;       // Color value operand 2
        TextureCombinerSourceKind    SourceAlpha0;       // Alpha value source 0
        TextureCombinerSourceKind    SourceAlpha1;       // Alpha value source 1
        TextureCombinerSourceKind    SourceAlpha2;       // Alpha value source 2
        TextureCombinerOperandKind    OperandAlpha0;       // Alpha value operand 0
        TextureCombinerOperandKind    OperandAlpha1;       // Alpha value operand 1
        TextureCombinerOperandKind    OperandAlpha2;       // Alpha value operand 2
    };

    // Sets the index for the texture used as a bump map.
    struct MESSAGEPACKET_Material_SetBumpTextureIndex
    {
        static const MessageType MessagePacketType = MessageType_Material_SetBumpTextureIndex;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_TextureIndex = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        BumpTextureIndex    TextureIndex;       // Index of the Texture to Bump map.
    };

    // Sets the bump mapping mode.
    struct MESSAGEPACKET_Material_SetBumpMode
    {
        static const MessageType MessagePacketType = MessageType_Material_SetBumpMode;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Mode = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        BumpMode    Mode;       // Bump mapping mode.
    };

    // Sets the flag to enable recalculation of the z component of the bump vector.
    struct MESSAGEPACKET_Material_SetIsBumpRenormalize
    {
        static const MessageType MessagePacketType = MessageType_Material_SetIsBumpRenormalize;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsBumpRenormalize = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsBumpRenormalize;       // Enable flag for recalculation of the bump vector z-component.
    };

    // Sets the entire texture combiner.
    struct MESSAGEPACKET_Material_SetTextureCombinerList
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureCombinerList;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ListCount = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ListCount;       // Number of TextureCombiners that were sent. Probably will fix to 6.
    };

    // Sets the entire texture configuration.
    struct MESSAGEPACKET_Material_SetTextureAttributeList
    {
        static const MessageType MessagePacketType = MessageType_Material_SetTextureAttributeList;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Index = 1<<0,
            PropertyFlag_ListCount = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    Index;       // Index of TextureAttribute that had changes.
        Uint    ListCount;       // Number of TextureAttributes that were sent. Maximum of 3.
    };

    // Sets the enable flag for the polygon offset.
    struct MESSAGEPACKET_Material_SetIsPolygonOffsetEnabled
    {
        static const MessageType MessagePacketType = MessageType_Material_SetIsPolygonOffsetEnabled;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsPolygonOffsetEnabled = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsPolygonOffsetEnabled;       // Polygon offset enable flag
    };

    // Sets the unit element of the polygon offset.
    struct MESSAGEPACKET_Material_SetPolygonOffsetUnit
    {
        static const MessageType MessagePacketType = MessageType_Material_SetPolygonOffsetUnit;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_PolygonOffsetUnit = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    PolygonOffsetUnit;       // Polygon offset unit element
    };

    // Sets the vertex shader program index.
    struct MESSAGEPACKET_Material_SetShaderProgramDescriptionIndex
    {
        static const MessageType MessagePacketType = MessageType_Material_SetShaderProgramDescriptionIndex;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ShaderProgramDescriptionIndex = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Int    ShaderProgramDescriptionIndex;       // Vertex shader program index
    };

    // Set the hide/show for the model.
    struct MESSAGEPACKET_Model_SetModelVisibility
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelVisibility;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_IsVisible = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Boolean    IsVisible;       // Flag that determines visibility.
    };

    // Sets the X value for the model TransformScale.
    struct MESSAGEPACKET_Model_SetModelTransformScaleX
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformScaleX;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformScaleX
    };

    // Sets the Y value for the model TransformScale.
    struct MESSAGEPACKET_Model_SetModelTransformScaleY
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformScaleY;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformScaleY
    };

    // Sets the Z value for the model TransformScale.
    struct MESSAGEPACKET_Model_SetModelTransformScaleZ
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformScaleZ;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformScaleZ
    };

    // Sets the X value for the model TransformRotate.
    struct MESSAGEPACKET_Model_SetModelTransformRotateX
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformRotateX;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformRotateX
    };

    // Sets the Y value for the model TransformRotate.
    struct MESSAGEPACKET_Model_SetModelTransformRotateY
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformRotateY;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformRotateY
    };

    // Sets the Z value for the model TransformRotate.
    struct MESSAGEPACKET_Model_SetModelTransformRotateZ
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformRotateZ;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformRotateZ
    };

    // Sets the X value for the model TransformTranslate.
    struct MESSAGEPACKET_Model_SetModelTransformTranslateX
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformTranslateX;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformTranslateX
    };

    // Sets the Y value for the model TransformTranslate.
    struct MESSAGEPACKET_Model_SetModelTransformTranslateY
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformTranslateY;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformTranslateY
    };

    // Sets the Z value for the model TransformTranslate.
    struct MESSAGEPACKET_Model_SetModelTransformTranslateZ
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformTranslateZ;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformTranslateZ
    };

    // Set the hide/show for the model on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewVisibility
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewVisibility;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_IsVisible = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Boolean    IsVisible;       // Flag that determines visibility.
    };

    // Sets the X value for the model TransformScale on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformScaleX
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformScaleX;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformScaleX
    };

    // Sets the Y value for the model TransformScale on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformScaleY
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformScaleY;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformScaleY
    };

    // Sets the Z value for the model TransformScale on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformScaleZ
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformScaleZ;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformScaleZ
    };

    // Sets the X value for the model TransformRotate on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformRotateX
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformRotateX;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformRotateX
    };

    // Sets the Y value for the model TransformRotate on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformRotateY
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformRotateY;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformRotateY
    };

    // Sets the Z value for the model TransformRotate on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformRotateZ
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformRotateZ;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformRotateZ
    };

    // Sets the X value for the model TransformTranslate on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformTranslateX
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformTranslateX;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformTranslateX
    };

    // Sets the Y value for the model TransformTranslate on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformTranslateY
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformTranslateY;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformTranslateY
    };

    // Sets the Z value for the model TransformTranslate on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformTranslateZ
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformTranslateZ;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_Value = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Float    Value;       // TranceformTranslateZ
    };

    // Sets the value for model transformation.
    struct MESSAGEPACKET_Model_SetModelTransformValue
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelTransformValue;    // Message types.
        static const int StructSize = 44 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_IsVisible = 1<<1,
            PropertyFlag_TranceformScaleX = 1<<2,
            PropertyFlag_TranceformScaleY = 1<<3,
            PropertyFlag_TranceformScaleZ = 1<<4,
            PropertyFlag_TranceformRotateX = 1<<5,
            PropertyFlag_TranceformRotateY = 1<<6,
            PropertyFlag_TranceformRotateZ = 1<<7,
            PropertyFlag_TranceformTranslateX = 1<<8,
            PropertyFlag_TranceformTranslateY = 1<<9,
            PropertyFlag_TranceformTranslateZ = 1<<10,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Boolean    IsVisible;       // Flag that determines visibility.
        Float    TranceformScaleX;       // TranceformScaleX
        Float    TranceformScaleY;       // TranceformScaleY
        Float    TranceformScaleZ;       // TranceformScaleZ
        Float    TranceformRotateX;       // TranceformRotateX
        Float    TranceformRotateY;       // TranceformRotateY
        Float    TranceformRotateZ;       // TranceformRotateZ
        Float    TranceformTranslateX;       // TranceformTranslateX
        Float    TranceformTranslateY;       // TranceformTranslateY
        Float    TranceformTranslateZ;       // TranceformTranslateZ
    };

    // Sets the value for model transformation on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewTransformValue
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewTransformValue;    // Message types.
        static const int StructSize = 44 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_IsVisible = 1<<1,
            PropertyFlag_TranceformScaleX = 1<<2,
            PropertyFlag_TranceformScaleY = 1<<3,
            PropertyFlag_TranceformScaleZ = 1<<4,
            PropertyFlag_TranceformRotateX = 1<<5,
            PropertyFlag_TranceformRotateY = 1<<6,
            PropertyFlag_TranceformRotateZ = 1<<7,
            PropertyFlag_TranceformTranslateX = 1<<8,
            PropertyFlag_TranceformTranslateY = 1<<9,
            PropertyFlag_TranceformTranslateZ = 1<<10,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Boolean    IsVisible;       // Flag that determines visibility.
        Float    TranceformScaleX;       // TranceformScaleX
        Float    TranceformScaleY;       // TranceformScaleY
        Float    TranceformScaleZ;       // TranceformScaleZ
        Float    TranceformRotateX;       // TranceformRotateX
        Float    TranceformRotateY;       // TranceformRotateY
        Float    TranceformRotateZ;       // TranceformRotateZ
        Float    TranceformTranslateX;       // TranceformTranslateX
        Float    TranceformTranslateY;       // TranceformTranslateY
        Float    TranceformTranslateZ;       // TranceformTranslateZ
    };

    // Set the model constraint on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewBoneLink
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewBoneLink;    // Message types.
        static const int StructSize = 528 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_ModelNameHashID = 1<<1,
            PropertyFlag_BoneName = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Uint    ModelNameHashID;       // Hash ID for the specified model name.
        String    BoneName;       // Name of bone to constrain to.
    };

    // Remove the model constraint on the viewer.
    struct MESSAGEPACKET_Model_SetModelPreviewBoneUnLink
    {
        static const MessageType MessagePacketType = MessageType_Model_SetModelPreviewBoneUnLink;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
    };

    // Set the hide/show for the mesh.
    struct MESSAGEPACKET_Mesh_SetMeshVisibility
    {
        static const MessageType MessagePacketType = MessageType_Mesh_SetMeshVisibility;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_MeshIndex = 1<<1,
            PropertyFlag_IsVisible = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Uint    MeshIndex;       // Index of the mesh that will be the target.
        Boolean    IsVisible;       // Mesh display flag.
    };

    // Set the hide/show for the mesh node.
    struct MESSAGEPACKET_Mesh_SetMeshNodeVisibility
    {
        static const MessageType MessagePacketType = MessageType_Mesh_SetMeshNodeVisibility;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_MeshIndex = 1<<1,
            PropertyFlag_IsVisible = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Uint    MeshIndex;       // Index of the mesh that will be the target.
        Boolean    IsVisible;       // Mesh display flag.
    };

    // Sets the mesh render priority.
    struct MESSAGEPACKET_Mesh_SetMeshRenderPriority
    {
        static const MessageType MessagePacketType = MessageType_Mesh_SetMeshRenderPriority;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_MeshIndex = 1<<1,
            PropertyFlag_RenderPriority = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Uint    MeshIndex;       // Index of the mesh that will be the target.
        Uint    RenderPriority;       // Render priority of the mesh.
    };

    // Sets the bone billboard mode.
    struct MESSAGEPACKET_Bone_SetBoneBillboard
    {
        static const MessageType MessagePacketType = MessageType_Bone_SetBoneBillboard;    // Message types.
        static const int StructSize = 528 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_BoneName = 1<<1,
            PropertyFlag_BillboardMode = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        String    BoneName;       // Name of the bone that will be the target.
        BoneBillboardMode    BillboardMode;       // Billboard mode.
    };

    // Sets the changed LUT.
    struct MESSAGEPACKET_LUT_ModifyLookUpTable
    {
        static const MessageType MessagePacketType = MessageType_LUT_ModifyLookUpTable;    // Message types.
        static const int StructSize = 1568 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_LutSetName = 1<<1,
            PropertyFlag_LutName = 1<<2,
            PropertyFlag_FileSize = 1<<3,
            PropertyFlag_FilePath = 1<<4,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        String    LutSetName;       // Name of the LUTSet that will be the target.
        String    LutName;       // Name of the LUT that will be the target.
        Uint    FileSize;       // Size of the file that made the LutSet binary.
        String    FilePath;       // Path of the file that made the LutSet binary.
    };

    // Restarts the Viewer drawing.
    struct MESSAGEPACKET_Operation_EnableDrawing
    {
        static const MessageType MessagePacketType = MessageType_Operation_EnableDrawing;    // Message types.
        static const int StructSize = 0 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

    };

    // Stops the Viewer drawing.
    struct MESSAGEPACKET_Operation_DisableDrawing
    {
        static const MessageType MessagePacketType = MessageType_Operation_DisableDrawing;    // Message types.
        static const int StructSize = 0 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

    };

    // Captures the Viewer top screen.
    struct MESSAGEPACKET_Operation_CaptureScreen
    {
        static const MessageType MessagePacketType = MessageType_Operation_CaptureScreen;    // Message types.
        static const int StructSize = 520 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_CapturePath = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        String    CapturePath;       // Placement path of the captured image file.
    };

    // Sets the camera view (Aim).
    struct MESSAGEPACKET_Camera_SetTargetViewAim
    {
        static const MessageType MessagePacketType = MessageType_Camera_SetTargetViewAim;    // Message types.
        static const int StructSize = 24 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_TargetPosition = 1<<0,
            PropertyFlag_Twist = 1<<1,
            PropertyFlag_IsInheritingTargetRotate = 1<<2,
            PropertyFlag_IsInheritingTargetTranslate = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    TargetPosition;       // Camera target coordinates.
        Float    Twist;       // The rotational angle with the camera visual vector as the axis.
        Boolean    IsInheritingTargetRotate;       // Flag indicating that the target position inherits the camera's rotation.
        Boolean    IsInheritingTargetTranslate;       // Flag indicating the target position inherits the camera's translation.
    };

    // Sets the camera view (LookAt).
    struct MESSAGEPACKET_Camera_SetTargetViewLookAt
    {
        static const MessageType MessagePacketType = MessageType_Camera_SetTargetViewLookAt;    // Message types.
        static const int StructSize = 36 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_TargetPosition = 1<<0,
            PropertyFlag_UpwardVector = 1<<1,
            PropertyFlag_IsInheritingTargetRotate = 1<<2,
            PropertyFlag_IsInheritingTargetTranslate = 1<<3,
            PropertyFlag_IsInheritingUpRotate = 1<<4,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    TargetPosition;       // Camera target coordinates.
        Vector3    UpwardVector;       // The camera Up vector.
        Boolean    IsInheritingTargetRotate;       // Flag indicating that the target position inherits the camera's rotation.
        Boolean    IsInheritingTargetTranslate;       // Flag indicating the target position inherits the camera's translation.
        Boolean    IsInheritingUpRotate;       // Flag indicating that the up vector inherits the camera's rotation.
    };

    // Sets the camera view (Rotate).
    struct MESSAGEPACKET_Camera_SetTargetViewRotate
    {
        static const MessageType MessagePacketType = MessageType_Camera_SetTargetViewRotate;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ViewRotate = 1<<0,
            PropertyFlag_IsInheritingRotate = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    ViewRotate;       // Amount to rotate the camera.
        Boolean    IsInheritingRotate;       // Flag indicating that the camera's rotation is to be inherited.
    };

    // Sets the project matrix with orthogonal projection.
    struct MESSAGEPACKET_Camera_SetOrthoProjection
    {
        static const MessageType MessagePacketType = MessageType_Camera_SetOrthoProjection;    // Message types.
        static const int StructSize = 24 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Near = 1<<0,
            PropertyFlag_Far = 1<<1,
            PropertyFlag_AspectRatio = 1<<2,
            PropertyFlag_Height = 1<<3,
            PropertyFlag_Center = 1<<4,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    Near;       // Distance to the near clipping plane.
        Float    Far;       // Distance to far clip.
        Float    AspectRatio;       // Aspect ratio.
        Float    Height;       // The rectangle's height.
        Vector2    Center;       // Center of the rectangle.
    };

    // Sets the project matrix using the angle of view.
    struct MESSAGEPACKET_Camera_SetPerspectiveProjection
    {
        static const MessageType MessagePacketType = MessageType_Camera_SetPerspectiveProjection;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Near = 1<<0,
            PropertyFlag_Far = 1<<1,
            PropertyFlag_Fovy = 1<<2,
            PropertyFlag_AspectRatio = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    Near;       // Distance to the near clipping plane.
        Float    Far;       // Distance to far clip.
        Float    Fovy;       // Angle of view.
        Float    AspectRatio;       // Aspect ratio.
    };

    // Sets Transform for the camera.
    struct MESSAGEPACKET_Camera_SetCameraTransform
    {
        static const MessageType MessagePacketType = MessageType_Camera_SetCameraTransform;    // Message types.
        static const int StructSize = 36 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Scale = 1<<0,
            PropertyFlag_Rotate = 1<<1,
            PropertyFlag_Translate = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    Scale;       // Scale component.
        Vector3    Rotate;       // Rotate component.
        Vector3    Translate;       // Translate component.
    };

    // Sets the camera's W buffer.
    struct MESSAGEPACKET_Camera_SetCameraIsWBuffer
    {
        static const MessageType MessagePacketType = MessageType_Camera_SetCameraIsWBuffer;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsWBuffer = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsWBuffer;       // W buffer scale factor.
    };

    // Gets the camera information.
    struct MESSAGEPACKET_Camera_GetCameraInfo
    {
        static const MessageType MessagePacketType = MessageType_Camera_GetCameraInfo;    // Message types.
        static const int StructSize = 64 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_CameraID = 1<<0,
            PropertyFlag_CameraIndex = 1<<1,
            PropertyFlag_CameraType = 1<<2,
            PropertyFlag_Position = 1<<3,
            PropertyFlag_TargetPosition = 1<<4,
            PropertyFlag_Twist = 1<<5,
            PropertyFlag_UpwardVector = 1<<6,
            PropertyFlag_ViewRotate = 1<<7,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    CameraID;       // Camera ID.
        Uint    CameraIndex;       // Camera index.
        ViewMode    CameraType;       // Camera view matrix type.
        Vector3    Position;       // Camera position.
        Vector3    TargetPosition;       // Camera focal point.
        Float    Twist;       // The rotational angle with the camera visual vector as the axis.
        Vector3    UpwardVector;       // The camera Up vector.
        Vector3    ViewRotate;       // Amount to rotate the camera.
    };

    // Configures fog.
    struct MESSAGEPACKET_Fog_SetFog
    {
        static const MessageType MessagePacketType = MessageType_Fog_SetFog;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_FogKind = 1<<0,
            PropertyFlag_MinFogDepth = 1<<1,
            PropertyFlag_MaxFogDepth = 1<<2,
            PropertyFlag_Density = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        FogKindCtr    FogKind;       // Fog Types
        Float    MinFogDepth;       // Depth value in the point of view coordinate system where the fog starts.
        Float    MaxFogDepth;       // Depth value in the point of view coordinate system where the fog density is at a maximum.
        Float    Density;       // Parameter to adjust fog density.
    };

    // Sets the fog Z-flip enable flag.
    struct MESSAGEPACKET_Fog_SetFogIsZFlip
    {
        static const MessageType MessagePacketType = MessageType_Fog_SetFogIsZFlip;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsZFlip = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsZFlip;       // Flag enabling fog z-flip.
    };

    // This function sets the fog color.
    struct MESSAGEPACKET_Fog_SetFogColor
    {
        static const MessageType MessagePacketType = MessageType_Fog_SetFogColor;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Color = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Color;       // Fog color.
    };

    // SGets the depth value in the view coordinate system where fog begins.
    struct MESSAGEPACKET_Fog_SetMinFogDepth
    {
        static const MessageType MessagePacketType = MessageType_Fog_SetMinFogDepth;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_MinFogDepth = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    MinFogDepth;       // Depth value in the point of view coordinate system where the fog starts.
    };

    // Sets the depth value in the view coordinate system where fog density is maximum.
    struct MESSAGEPACKET_Fog_SetMaxFogDepth
    {
        static const MessageType MessagePacketType = MessageType_Fog_SetMaxFogDepth;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_MaxFogDepth = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    MaxFogDepth;       // Depth value in the point of view coordinate system where the fog density is at a maximum.
    };

    // Sets parameters for adjusting fog density.
    struct MESSAGEPACKET_Fog_SetFogDensity
    {
        static const MessageType MessagePacketType = MessageType_Fog_SetFogDensity;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Density = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    Density;       // Parameter to adjust fog density.
    };

    // Sets the light enable flag.
    struct MESSAGEPACKET_Light_SetLightIsLightEnabled
    {
        static const MessageType MessagePacketType = MessageType_Light_SetLightIsLightEnabled;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsLightEnabled = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsLightEnabled;       // Light enable flag.
    };

    // Sets Transform (Translate) for lights.
    struct MESSAGEPACKET_Light_SetLightTranslate
    {
        static const MessageType MessagePacketType = MessageType_Light_SetLightTranslate;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Translate = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    Translate;       // Translate component.
    };

    // Sets the ambient light color.
    struct MESSAGEPACKET_Light_SetAmbientLightColor
    {
        static const MessageType MessagePacketType = MessageType_Light_SetAmbientLightColor;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Color = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Color;       // Ambient light color.
    };

    // Sets the ground color.
    struct MESSAGEPACKET_Light_SetHemiSphereGroundColor
    {
        static const MessageType MessagePacketType = MessageType_Light_SetHemiSphereGroundColor;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_GroundColor = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    GroundColor;       // Ground color.
    };

    // Sets the sky color.
    struct MESSAGEPACKET_Light_SetHemiSphereSkyColor
    {
        static const MessageType MessagePacketType = MessageType_Light_SetHemiSphereSkyColor;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_SkyColor = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    SkyColor;       // Sky color.
    };

    // Sets the light direction.
    struct MESSAGEPACKET_Light_SetHemiSphereDirection
    {
        static const MessageType MessagePacketType = MessageType_Light_SetHemiSphereDirection;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Direction = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    Direction;       // The light direction.
    };

    // Sets the linear interpolation factor.
    struct MESSAGEPACKET_Light_SetHemiSphereLerpFactor
    {
        static const MessageType MessagePacketType = MessageType_Light_SetHemiSphereLerpFactor;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LerpFactor = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    LerpFactor;       // Linear interpolation coefficient.
    };

    // Sets the parent node rotation inherit enable flag.
    struct MESSAGEPACKET_Light_SetHemiSphereIsInheritingDirectionRotate
    {
        static const MessageType MessagePacketType = MessageType_Light_SetHemiSphereIsInheritingDirectionRotate;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsInheritingDirectionRotate = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsInheritingDirectionRotate;       // Flag enabling the inheritance of the parent node's rotation.
    };

    // Sets the light type.
    struct MESSAGEPACKET_Light_SetVLLightKind
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLLightKind;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LightKind = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        LightKindCtr    LightKind;       // Light source type.
    };

    // Configures ambient light color.
    struct MESSAGEPACKET_Light_SetVLAmbient
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLAmbient;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Ambient = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Ambient;       // Ambient color.
    };

    // Sets the diffuse color.
    struct MESSAGEPACKET_Light_SetVLDiffuse
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLDiffuse;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Diffuse = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Diffuse;       // Diffuse color.
    };

    // Sets the vertex light direction.
    struct MESSAGEPACKET_Light_SetVLDirection
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLDirection;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Direction = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    Direction;       // Vertex light direction.
    };

    // Sets the parent node rotation inherit enable flag.
    struct MESSAGEPACKET_Light_SetVLIsInheritingDirectionRotate
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLIsInheritingDirectionRotate;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsInheritingDirectionRotate = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsInheritingDirectionRotate;       // Flag enabling the inheritance of the parent node's rotation.
    };

    // Set the distance attenuation constant attenuation coefficient.
    struct MESSAGEPACKET_Light_SetVLDistanceAttenuationConstant
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLDistanceAttenuationConstant;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_DistanceAttenuationConstant = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    DistanceAttenuationConstant;       // Constant attenuation coefficient for distance attenuation.
    };

    // Set the distance attenuation linear attenuation coefficient.
    struct MESSAGEPACKET_Light_SetVLDistanceAttenuationLinear
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLDistanceAttenuationLinear;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_DistanceAttenuationLinear = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    DistanceAttenuationLinear;       // Linear attenuation coefficient for distance attenuation.
    };

    // Set the distance attenuation secondary attenuation coefficient.
    struct MESSAGEPACKET_Light_SetVLDistanceAttenuationQuadratic
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLDistanceAttenuationQuadratic;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_DistanceAttenuationQuadratic = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    DistanceAttenuationQuadratic;       // Secondary attenuation coefficient for distance attenuation.
    };

    // Sets the enable flag for the distance attenuation.
    struct MESSAGEPACKET_Light_SetVLIsDistanceAttenuationEnabled
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLIsDistanceAttenuationEnabled;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsDistanceAttenuationEnabled = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsDistanceAttenuationEnabled;       // Flag enabling distance attenuation.
    };

    // Sets a spotlight's angular attenuation coefficients.
    struct MESSAGEPACKET_Light_SetVLSpotExponent
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLSpotExponent;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_SpotExponent = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    SpotExponent;       // Spot light angular attenuation coefficient.
    };

    // Sets the cut-off angle for the spotlight.
    struct MESSAGEPACKET_Light_SetVLSpotCutoffAngle
    {
        static const MessageType MessagePacketType = MessageType_Light_SetVLSpotCutoffAngle;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_SpotCutoffAngle = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    SpotCutoffAngle;       // Spot light cut-off angle.
    };

    // Sets the light type.
    struct MESSAGEPACKET_Light_SetFLKind
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLKind;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LightKind = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        LightKindCtr    LightKind;       // Light source type.
    };

    // Configures ambient light color.
    struct MESSAGEPACKET_Light_SetFLAmbient
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLAmbient;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Ambient = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Ambient;       // Ambient color.
    };

    // Sets the diffuse color.
    struct MESSAGEPACKET_Light_SetFLDiffuse
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLDiffuse;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Diffuse = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Diffuse;       // Diffuse color.
    };

    // Sets the enable flag for both face lighting for diffuse color.
    struct MESSAGEPACKET_Light_SetFLIsTwoSideDiffuse
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLIsTwoSideDiffuse;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsTwoSideDiffuse = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsTwoSideDiffuse;       // Enable flag for both face lighting for diffuse color.
    };

    // Sets specular color.
    struct MESSAGEPACKET_Light_SetFLSpecular0
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLSpecular0;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Specular0 = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Specular0;       // Specular color.
    };

    // Sets the second specular color.
    struct MESSAGEPACKET_Light_SetFLSpecular1
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLSpecular1;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Specular1 = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Rgba    Specular1;       // 2nd specular color.
    };

    // Sets the parent node rotation inherit enable flag.
    struct MESSAGEPACKET_Light_SetFLIsInheritingDirectionRotate
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLIsInheritingDirectionRotate;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsInheritingDirectionRotate = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsInheritingDirectionRotate;       // Flag enabling the inheritance of the parent node's rotation.
    };

    // Sets the light direction.
    struct MESSAGEPACKET_Light_SetFLDirection
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLDirection;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Direction = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Vector3    Direction;       // The light direction.
    };

    // Sets the reference table set in AngleSampler and DistanceSampler.
    struct MESSAGEPACKET_Light_SetFLSamplerLUTS
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLSamplerLUTS;    // Message types.
        static const int StructSize = 520 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LookupTableSetName = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        String    LookupTableSetName;       // Reference table set name.
    };

    // Sets the enable flag for the distance attenuation.
    struct MESSAGEPACKET_Light_SetFLIsDistanceAttenuationEnabled
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLIsDistanceAttenuationEnabled;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_IsDistanceAttenuationEnabled = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Boolean    IsDistanceAttenuationEnabled;       // Flag enabling distance attenuation.
    };

    // Sets the distance where the light brightness begins to be attenuated.
    struct MESSAGEPACKET_Light_SetFLIsDistanceAttenuationStart
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLIsDistanceAttenuationStart;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_DistanceAttenuationStart = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    DistanceAttenuationStart;       // Distance where the light brightness begins to be attenuated.
    };

    // Sets the distance where the light brightness stops being attenuated.
    struct MESSAGEPACKET_Light_SetFLIsDistanceAttenuationEnd
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLIsDistanceAttenuationEnd;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_DistanceAttenuationEnd = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Float    DistanceAttenuationEnd;       // Distance where the light brightness stops being attenuated.
    };

    // Sets reference tables for distance samplers.
    struct MESSAGEPACKET_Light_SetFLDistanceSamplerLUT
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLDistanceSamplerLUT;    // Message types.
        static const int StructSize = 1040 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LookupTableSetName = 1<<0,
            PropertyFlag_LookupTableName = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        String    LookupTableSetName;       // Reference table set name.
        String    LookupTableName;       // Reference table name for distance sampler
    };

    // Sets reference tables for angle samplers.
    struct MESSAGEPACKET_Light_SetFLAngleSamplerLUT
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLAngleSamplerLUT;    // Message types.
        static const int StructSize = 1040 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LookupTableSetName = 1<<0,
            PropertyFlag_LookupTableName = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        String    LookupTableSetName;       // Reference table set name.
        String    LookupTableName;       // Reference table name for angle sampler
    };

    // Sets input values for accessing reference tables for angle samplers.
    struct MESSAGEPACKET_Light_SetFLAngleSamplerInput
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLAngleSamplerInput;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Input = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        LightingLookupTableInput    Input;       // Input values for accessing reference tables for angle samplers.
    };

    // Sets the scale value type applied to the output value taken from the angle sampler reference table.
    struct MESSAGEPACKET_Light_SetFLAngleSamplerScale
    {
        static const MessageType MessagePacketType = MessageType_Light_SetFLAngleSamplerScale;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_Scale = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        LightingLookupTableScale    Scale;       // Scale value type applied to the output value taken from the angle sampler reference table.
    };

    // Gets the fragment light information.
    struct MESSAGEPACKET_Light_GetFragmentLightInfo
    {
        static const MessageType MessagePacketType = MessageType_Light_GetFragmentLightInfo;    // Message types.
        static const int StructSize = 36 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LightID = 1<<0,
            PropertyFlag_LightIndex = 1<<1,
            PropertyFlag_LightKind = 1<<2,
            PropertyFlag_Position = 1<<3,
            PropertyFlag_Direction = 1<<4,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    LightID;       // Light ID.
        Uint    LightIndex;       // The light index.
        LightKindCtr    LightKind;       // Light source type.
        Vector3    Position;       // The light position.
        Vector3    Direction;       // The light direction.
    };

    // Sets a scene to active.
    struct MESSAGEPACKET_Scene_SetSceneActive
    {
        static const MessageType MessagePacketType = MessageType_Scene_SetSceneActive;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_SceneId = 1<<0,
            PropertyFlag_Index = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    SceneId;       // Target scene file hash ID.
        Uint    Index;       // Target scene index.
    };

    // Configures the association of an animation set and a model.
    struct MESSAGEPACKET_Animation_AddAnimationSet
    {
        static const MessageType MessagePacketType = MessageType_Animation_AddAnimationSet;    // Message types.
        static const int StructSize = 528 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_AnimationSetName = 1<<1,
            PropertyFlag_AnimationCount = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        String    AnimationSetName;       // Animation Set name.
        Uint    AnimationCount;       // Number of animation IDs.
    };

    // Configures the association of an animation not in an animation set and a model.
    struct MESSAGEPACKET_Animation_AddStrayAnimations
    {
        static const MessageType MessagePacketType = MessageType_Animation_AddStrayAnimations;    // Message types.
        static const int StructSize = 8 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_AnimationCount = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        Uint    AnimationCount;       // Number of animation IDs.
    };

    // Sets the animation set binding.
    struct MESSAGEPACKET_Animation_ActivateAnimationSet
    {
        static const MessageType MessagePacketType = MessageType_Animation_ActivateAnimationSet;    // Message types.
        static const int StructSize = 524 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_AnimationSetName = 1<<1,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        String    AnimationSetName;       // Animation Set name.
    };

    // Sets the animation set unbinding.
    struct MESSAGEPACKET_Animation_DeActivateAnimationSet
    {
        static const MessageType MessagePacketType = MessageType_Animation_DeActivateAnimationSet;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
    };

    // Clears the animation set.
    struct MESSAGEPACKET_Animation_ClearAnimationSet
    {
        static const MessageType MessagePacketType = MessageType_Animation_ClearAnimationSet;    // Message types.
        static const int StructSize = 4 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
    };

    // Adds a light animation set.
    struct MESSAGEPACKET_Animation_AddLightAnimation
    {
        static const MessageType MessagePacketType = MessageType_Animation_AddLightAnimation;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_LightID = 1<<0,
            PropertyFlag_LightIndex = 1<<1,
            PropertyFlag_AnimationID = 1<<2,
            PropertyFlag_AnimationIndex = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    LightID;       // Light ID.
        Uint    LightIndex;       // The light index.
        Uint    AnimationID;       // Light animation ID.
        Uint    AnimationIndex;       // Light animation index.
    };

    // Adds a camera animation set.
    struct MESSAGEPACKET_Animation_AddCameraAnimation
    {
        static const MessageType MessagePacketType = MessageType_Animation_AddCameraAnimation;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_CameraID = 1<<0,
            PropertyFlag_CameraIndex = 1<<1,
            PropertyFlag_AnimationID = 1<<2,
            PropertyFlag_AnimationIndex = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    CameraID;       // Camera ID.
        Uint    CameraIndex;       // Camera index.
        Uint    AnimationID;       // Camera animation ID.
        Uint    AnimationIndex;       // Camera animation index.
    };

    // Adds a fog animation set.
    struct MESSAGEPACKET_Animation_AddFogAnimation
    {
        static const MessageType MessagePacketType = MessageType_Animation_AddFogAnimation;    // Message types.
        static const int StructSize = 16 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_FogID = 1<<0,
            PropertyFlag_FogIndex = 1<<1,
            PropertyFlag_AnimationID = 1<<2,
            PropertyFlag_AnimationIndex = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    FogID;       // Fog ID.
        Uint    FogIndex;       // Fog index.
        Uint    AnimationID;       // Fog animation ID.
        Uint    AnimationIndex;       // Fog animation index.
    };

    // Renames an animation set.
    struct MESSAGEPACKET_Animation_RenameAnimationSet
    {
        static const MessageType MessagePacketType = MessageType_Animation_RenameAnimationSet;    // Message types.
        static const int StructSize = 1044 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ModelId = 1<<0,
            PropertyFlag_OldAnimationSetName = 1<<1,
            PropertyFlag_NewAnimationSetName = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Uint    ModelId;       // Target model ID.
        String    OldAnimationSetName;       // Old animation Set name.
        String    NewAnimationSetName;       // New animation Set name.
    };

    // Sets the bind for the camera, light, and fog animation.
    struct MESSAGEPACKET_Animation_BindSceneAnimation
    {
        static const MessageType MessagePacketType = MessageType_Animation_BindSceneAnimation;    // Message types.
        static const int StructSize = 532 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_AnimType = 1<<0,
            PropertyFlag_TargetID = 1<<1,
            PropertyFlag_TargetIndex = 1<<2,
            PropertyFlag_AnimationName = 1<<3,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        SceneAnimationType    AnimType;       // Type (camera, light, or fog).
        Uint    TargetID;       // Target ID (filename hash).
        Uint    TargetIndex;       // Target index.
        String    AnimationName;       // Name of the animation.
    };

    // Releases the bind for the camera, light, and fog animation.
    struct MESSAGEPACKET_Animation_UnBindSceneAnimation
    {
        static const MessageType MessagePacketType = MessageType_Animation_UnBindSceneAnimation;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_AnimType = 1<<0,
            PropertyFlag_TargetID = 1<<1,
            PropertyFlag_TargetIndex = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        SceneAnimationType    AnimType;       // Type (camera, light, or fog).
        Uint    TargetID;       // Target ID (filename hash).
        Uint    TargetIndex;       // Target index.
    };

    // Evacuates the current frame.
    struct MESSAGEPACKET_Animation_StoreCurrentFrame
    {
        static const MessageType MessagePacketType = MessageType_Animation_StoreCurrentFrame;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_TargetType = 1<<0,
            PropertyFlag_TargetID = 1<<1,
            PropertyFlag_TargetIndex = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        StoreTargetObjectType    TargetType;       // Type (camera, light, or fog).
        Uint    TargetID;       // Target ID (filename hash).
        Uint    TargetIndex;       // Target index.
    };

    // Restores the current frame.
    struct MESSAGEPACKET_Animation_RestoreCurrentFrame
    {
        static const MessageType MessagePacketType = MessageType_Animation_RestoreCurrentFrame;    // Message types.
        static const int StructSize = 12 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_TargetType = 1<<0,
            PropertyFlag_TargetID = 1<<1,
            PropertyFlag_TargetIndex = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        StoreTargetObjectType    TargetType;       // Type (camera, light, or fog).
        Uint    TargetID;       // Target ID (filename hash).
        Uint    TargetIndex;       // Target index.
    };

    // Sends edited UserData.
    struct MESSAGEPACKET_UserData_SendUserData
    {
        static const MessageType MessagePacketType = MessageType_UserData_SendUserData;    // Message types.
        static const int StructSize = 528 + 4 + 4;    // Size as a structure.  The +4 is for ModifiedProperties and ModifiedPropertyElements.

        enum PropertyFlag
        {
            PropertyFlag_ObjectCount = 1<<0,
            PropertyFlag_Id = 1<<1,
            PropertyFlag_FilePath = 1<<2,
            PropertyFlag_FORCE32    = 0xFFFFFFFF
        };

        Uint    ModifiedProperties;              // Modified properties.
        Uint    ModifiedPropertyElements;        // Modified property elements.

        Int    ObjectCount;       // Number of selected objects.
        Uint    Id;       // Returns the ID.
        String    FilePath;       // The file path.
    };

    // Specifies the object to edit.
    struct EditSelectedObject
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Id;    // Returns the ID.
        Uint    Index;    // The index.
    };

    // Support for object from material
    struct MaterialBindObject
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Id;    // Returns the ID.
        Uint    Index;    // The index.
    };

    // MinFilter
    struct SetTexMinFilter
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        TextureSamplerMinFilter    MinFilter;    // MinFilter.
    };

    // MinFilter
    struct SetTexMagFilter
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        TextureSamplerMagFilter    MagFilter;    // MagFilter.
    };

    // WrapS
    struct SetTexWrapS
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        TextureSamplerWrap    WrapS;    // WrapS.
    };

    // WrapT
    struct SetTexWrapT
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        TextureSamplerWrap    WrapT;    // WrapT.
    };

    // MinLod
    struct SetTexMinLod
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Uint    MinLod;    // MinLod.
    };

    // LodBias
    struct SetTexLodBias
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Float    LodBias;    // LodBias.
    };

    // BorderColor
    struct SetTexBorderColor
    {
        static const int StructSize = 20;    // Size as a structure.

        Uint    Index;    // The index.
        Rgba    BorderColor;    // BorderColor.
    };

    // SourceCoordinate
    struct SetTCSourceCoordinate
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Int    SourceCoordinate;    // SourceCoordinate.
    };

    // MappingMethod
    struct SetTCMappingMethod
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        TextureMappingMethodCtr    MappingMethod;    // MappingMethod.
    };

    // ReferenceCamera
    struct SetTCReferenceCamera
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Int    ReferenceCamera;    // ReferenceCamera.
    };

    // MatrixMode
    struct SetTCMatrixMode
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        TextureMatrixModeCtr    MatrixMode;    // MatrixMode.
    };

    // ScaleS
    struct SetTCScaleS
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Float    ScaleS;    // ScaleS.
    };

    // ScaleT
    struct SetTCScaleT
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Float    ScaleT;    // ScaleT.
    };

    // ScaleS and ScaleT
    struct SetTCScaleST
    {
        static const int StructSize = 12;    // Size as a structure.

        Uint    Index;    // The index.
        Float    ScaleS;    // ScaleS.
        Float    ScaleT;    // ScaleT.
    };

    // Rotate
    struct SetTCRotate
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Float    Rotate;    // Rotate.
    };

    // TranslateS
    struct SetTCTranslateS
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Float    TranslateS;    // TranslateS.
    };

    // TranslateT
    struct SetTCTranslateT
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // The index.
        Float    TranslateT;    // TranslateT.
    };

    // TextureCombiner
    struct TextureCombiner
    {
        static const int StructSize = 76;    // Size as a structure.

        TextureCombineMode    CombineRgb;    // Color combiner function
        TextureCombineMode    CombineAlpha;    // Alpha combiner function
        TextureCombineScale    ScaleRgb;    // Scale for the output color
        TextureCombineScale    ScaleAlpha;    // Scale for the output alpha
        ConstantColorCtr    Constant;    // Constant color index
        TextureCombineBufferInput    BufferInputRgb;    // Color buffer input
        TextureCombineBufferInput    BufferInputAlpha;    // Alpha buffer input
        TextureCombinerSourceKind    SourceRgb0;    // Color value source 0
        TextureCombinerSourceKind    SourceRgb1;    // Color value source 1
        TextureCombinerSourceKind    SourceRgb2;    // Color value source 2
        TextureCombinerOperandKind    OperandRgb0;    // Color value operand 0
        TextureCombinerOperandKind    OperandRgb1;    // Color value operand 1
        TextureCombinerOperandKind    OperandRgb2;    // Color value operand 2
        TextureCombinerSourceKind    SourceAlpha0;    // Alpha value source 0
        TextureCombinerSourceKind    SourceAlpha1;    // Alpha value source 1
        TextureCombinerSourceKind    SourceAlpha2;    // Alpha value source 2
        TextureCombinerOperandKind    OperandAlpha0;    // Alpha value operand 0
        TextureCombinerOperandKind    OperandAlpha1;    // Alpha value operand 1
        TextureCombinerOperandKind    OperandAlpha2;    // Alpha value operand 2
    };

    // When send all TextureCombiners
    struct TextureCombinerList
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // Index of TextureCombiner that had changes.
        Uint    ListCount;    // Number of TextureCombiners that were sent. Probably will fix to 6.
    };

    // Texture information.
    struct TextureAttribute
    {
        static const int StructSize = 44;    // Size as a structure.

        Uint    Id;    // ID (hash code for the texture name).
        TextureSamplerMinFilter    MinFilter;    // MinFilter settings
        TextureSamplerMagFilter    MagFilter;    // MagFilter settings
        TextureSamplerWrap    WrapS;    // WrapS settings
        TextureSamplerWrap    WrapT;    // WrapT settings
        Float    MinLod;    // MinLod settings
        Float    LodBias;    // LodBias settings
        Rgba    BorderColor;    // BorderColor settings
    };

    // When send all TextureAttributes
    struct TextureAttributeList
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    Index;    // Index of TextureAttribute that had changes.
        Uint    ListCount;    // Number of TextureAttributes that were sent. Maximum of 3.
    };

    // FragmentLightingTableCtr
    struct FragmentLightingTableCtr
    {
        static const int StructSize = 3712;    // Size as a structure.

        String    LookupTableSetName;    // Name of LookupTableSet to reference.
        Boolean    RefSamplerRIsAbs;    // Flag to make input value an absolute value to access the reference table (not currently sent)
        LightingLookupTableInput    RefSamplerRInput;    // Input value to access the reference table
        LightingLookupTableScale    RefSamplerRScale;    // Scale value to apply to the output value taken from the reference table
        String    RefSamplerRLutName;    // Name of ImageLookupTableCtr to reference.
        Boolean    RefSamplerGIsAbs;    // Flag to make input value an absolute value to access the reference table (not currently sent)
        LightingLookupTableInput    RefSamplerGInput;    // Input value to access the reference table
        LightingLookupTableScale    RefSamplerGScale;    // Scale value to apply to the output value taken from the reference table
        String    RefSamplerGLutName;    // Name of ImageLookupTableCtr to reference.
        Boolean    RefSamplerBIsAbs;    // Flag to make input value an absolute value to access the reference table (not currently sent)
        LightingLookupTableInput    RefSamplerBInput;    // Input value to access the reference table
        LightingLookupTableScale    RefSamplerBScale;    // Scale value to apply to the output value taken from the reference table
        String    RefSamplerBLutName;    // Name of ImageLookupTableCtr to reference.
        Boolean    Dis0SamplerIsAbs;    // Flag to make input value an absolute value to access the reference table (not currently sent)
        LightingLookupTableInput    Dis0SamplerInput;    // Input value to access the reference table
        LightingLookupTableScale    Dis0SamplerScale;    // Scale value to apply to the output value taken from the reference table
        String    Dis0SamplerLutName;    // Name of ImageLookupTableCtr to reference.
        Boolean    Dis1SamplerIsAbs;    // Flag to make input value an absolute value to access the reference table (not currently sent)
        LightingLookupTableInput    Dis1SamplerInput;    // Input value to access the reference table
        LightingLookupTableScale    Dis1SamplerScale;    // Scale value to apply to the output value taken from the reference table
        String    Dis1SamplerLutName;    // Name of ImageLookupTableCtr to reference.
        Boolean    FresnelSamplerIsAbs;    // Flag to make input value an absolute value to access the reference table (not currently sent)
        LightingLookupTableInput    FresnelSamplerInput;    // Input value to access the reference table
        LightingLookupTableScale    FresnelSamplerScale;    // Scale value to apply to the output value taken from the reference table
        String    FresnelSamplerLutName;    // Name of ImageLookupTableCtr to reference.
    };

    // For show/hide of model
    struct ModelVisibility
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        Boolean    IsVisible;    // Flag that determines visibility.
    };

    // One component of model position matrix
    struct ModelMatrixOneValue
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        Float    Value;    // One value of matrix element
    };

    // Value for model transform
    struct ModelTransformValue
    {
        static const int StructSize = 44;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        Boolean    IsVisible;    // Flag that determines visibility.
        Float    TranceformScaleX;    // TranceformScaleX
        Float    TranceformScaleY;    // TranceformScaleY
        Float    TranceformScaleZ;    // TranceformScaleZ
        Float    TranceformRotateX;    // TranceformRotateX
        Float    TranceformRotateY;    // TranceformRotateY
        Float    TranceformRotateZ;    // TranceformRotateZ
        Float    TranceformTranslateX;    // TranceformTranslateX
        Float    TranceformTranslateY;    // TranceformTranslateY
        Float    TranceformTranslateZ;    // TranceformTranslateZ
    };

    // For Mesh visibility setting
    struct MeshVisibility
    {
        static const int StructSize = 12;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        Uint    MeshIndex;    // Index of the mesh that will be the target.
        Boolean    IsVisible;    // Mesh display flag.
    };

    // For MeshNode visibility setting
    struct MeshNodeVisibility
    {
        static const int StructSize = 12;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        Uint    MeshIndex;    // Index of the mesh that will be the target.
        Boolean    IsVisible;    // Mesh display flag.
    };

    // For Mesh render priority setting
    struct MeshRenderPriority
    {
        static const int StructSize = 12;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        Uint    MeshIndex;    // Index of the mesh that will be the target.
        Uint    RenderPriority;    // Render priority of the mesh.
    };

    // For Bone billboard mode setting
    struct BoneBillboard
    {
        static const int StructSize = 528;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        String    BoneName;    // Name of the bone that will be the target.
        BoneBillboardMode    BillboardMode;    // Billboard mode.
    };

    // Sets the changed LUT.
    struct ModLut
    {
        static const int StructSize = 1568;    // Size as a structure.

        Uint    ModelId;    // Target model ID.
        String    LutSetName;    // Name of the LUTSet that will be the target.
        String    LutName;    // Name of the LUT that will be the target.
        Uint    FileSize;    // Size of the file that made the LutSet binary.
        String    FilePath;    // Path of the file that made the LutSet binary.
    };

    // Set the camera view (Aim).
    struct CameraViewAim
    {
        static const int StructSize = 24;    // Size as a structure.

        Vector3    TargetPosition;    // Camera target coordinates.
        Float    Twist;    // The rotational angle with the camera visual vector as the axis.
        Boolean    IsInheritingTargetRotate;    // Flag indicating that the target position inherits the camera's rotation.
        Boolean    IsInheritingTargetTranslate;    // Flag indicating the target position inherits the camera's translation.
    };

    // Set the camera view (LookAt).
    struct CameraViewLookAt
    {
        static const int StructSize = 36;    // Size as a structure.

        Vector3    TargetPosition;    // Camera target coordinates.
        Vector3    UpwardVector;    // The camera Up vector.
        Boolean    IsInheritingTargetRotate;    // Flag indicating that the target position inherits the camera's rotation.
        Boolean    IsInheritingTargetTranslate;    // Flag indicating the target position inherits the camera's translation.
        Boolean    IsInheritingUpRotate;    // Flag indicating that the up vector inherits the camera's rotation.
    };

    // Set the camera view (Rotate).
    struct CameraViewRotate
    {
        static const int StructSize = 16;    // Size as a structure.

        Vector3    ViewRotate;    // Amount to rotate the camera.
        Boolean    IsInheritingRotate;    // Flag indicating that the camera's rotation is to be inherited.
    };

    // Sets the project matrix with orthogonal projection.
    struct CameraOrthoProjection
    {
        static const int StructSize = 24;    // Size as a structure.

        Float    Near;    // Distance to the near clipping plane.
        Float    Far;    // Distance to far clip.
        Float    AspectRatio;    // Aspect ratio.
        Float    Height;    // The rectangle's height.
        Vector2    Center;    // Center of the rectangle.
    };

    // Sets the project matrix using the angle of view.
    struct CameraPerspectiveProjection
    {
        static const int StructSize = 16;    // Size as a structure.

        Float    Near;    // Distance to the near clipping plane.
        Float    Far;    // Distance to far clip.
        Float    Fovy;    // Angle of view.
        Float    AspectRatio;    // Aspect ratio.
    };

    // Sets Transform.
    struct Transform
    {
        static const int StructSize = 36;    // Size as a structure.

        Vector3    Scale;    // Scale component.
        Vector3    Rotate;    // Rotate component.
        Vector3    Translate;    // Translate component.
    };

    // Configures fog.
    struct FogUpdater
    {
        static const int StructSize = 16;    // Size as a structure.

        FogKindCtr    FogKind;    // Fog Types
        Float    MinFogDepth;    // Depth value in the point of view coordinate system where the fog starts.
        Float    MaxFogDepth;    // Depth value in the point of view coordinate system where the fog density is at a maximum.
        Float    Density;    // Parameter to adjust fog density.
    };

    // Sets LUTset name and LUT name.
    struct LutNameSet
    {
        static const int StructSize = 1040;    // Size as a structure.

        String    LookupTableSetName;    // Reference table set name.
        String    LookupTableName;    // Reference table name
    };

    // Sets the scene environment to be used for preview.
    struct ActiveScene
    {
        static const int StructSize = 8;    // Size as a structure.

        Uint    SceneId;    // Target scene file hash ID.
        Uint    Index;    // Target scene index.
    };

    // For creating animation ID arrays.
    struct AnimationId
    {
        static const int StructSize = 4;    // Size as a structure.

        Uint    Id;    // Target animation file hash ID.
    };

    // For getting the fragment light information.
    struct FragmentLightInfo
    {
        static const int StructSize = 36;    // Size as a structure.

        Uint    LightID;    // Light ID.
        Uint    LightIndex;    // The light index.
        LightKindCtr    LightKind;    // Light source type.
        Vector3    Position;    // The light position.
        Vector3    Direction;    // The light direction.
    };

    // Gets the camera information.
    struct CameraInfo
    {
        static const int StructSize = 64;    // Size as a structure.

        Uint    CameraID;    // Camera ID.
        Uint    CameraIndex;    // Camera index.
        ViewMode    CameraType;    // Camera view matrix type.
        Vector3    Position;    // Camera position.
        Vector3    TargetPosition;    // Camera focal point.
        Float    Twist;    // The rotational angle with the camera visual vector as the axis.
        Vector3    UpwardVector;    // The camera Up vector.
        Vector3    ViewRotate;    // Amount to rotate the camera.
    };

    // Specifies the object to edit.
    struct EditUserDataObject
    {
        static const int StructSize = 12;    // Size as a structure.

        UserDataTargetType    ObjectType;    // Object type.
        Uint    Id;    // Returns the ID.
        Uint    Index;    // The index.
    };

}
#endif
