/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SOUND_PROTOCOL_SOUND_H_
#define NW4C_VIEWER_SOUND_PROTOCOL_SOUND_H_

#include <cstring>
#include <cassert>

namespace ProtocolSound
{
    // TODO: * All content of this file is a virtual implementation.
    //          Revise to match the sound HIO communication protocol.

    struct Constants
    {
#if 1
        static const u16          CHANNEL_ACTIVE        = 0x00000008;   // Transmission communication channel
        static const u16          CHANNEL_PASSIVE       = 0x00000009;   // Reception communication channel
        static const u16          CHANNEL_PING          = 0x0000000A;   // Ping channel
#endif

        static const u16          CHANNEL               = 0x00000008;   // Sound communication channel
        static const unsigned int VERSION               = 0x00010000;   // Protocol version.
        static const unsigned int VERSION_MAJOR_MASK    = 0xFF000000;   // Major protocol version mask.
        static const unsigned int VERSION_MINOR_MASK    = 0x00FF0000;   // Minor protocol version mask.
        static const unsigned int VERSION_REVISION_MASK = 0x0000FF00;   // Protocol revision mask.
        static const unsigned int MESSAGE_CATEGORY_MASK = 0x0000FF00;   // Message category mask.
    };

    enum MessageHeader
        {
            MessageHeader_Mask = 0x0000FFFF,

            MessageHeader_Reply_Flag = 0x00004000,
            MessageHeader_Category_Mask = 0x00003F00,
            MessageHeader_Command_Mask = 0x000000FF,

            MessageHeader_Force32 = 0xFFFFFFFF
        };

    enum MessageCategory  // Message categories.
        {
            MessageCategory_General = 0x0000,     // General
            MessageCategory_Sound = 0x0300,       // Sound

            MessageCategory_Force32 = 0xFFFFFFFF
        };

    // Message types.
    enum MessageType
        {
            MessageType_General_Reset = 0x00 | MessageCategory_General,   // Resets.

            // Sound
            MessageType_Sound_Focus                     = 0x01 | MessageCategory_Sound,
            MessageType_Sound_Echo                      = 0x02 | MessageCategory_Sound,
            MessageType_Sound_AllStop                   = 0x03 | MessageCategory_Sound,
            MessageType_Sound_SetSystemParameters       = 0x04 | MessageCategory_Sound,

            MessageType_Sound_SoundBegin                = 0x40 | MessageCategory_Sound,

            MessageType_Sound_LoadSar                   = 0x41 | MessageCategory_Sound,
            MessageType_Sound_SetSoundLabels            = 0x42 | MessageCategory_Sound,
            MessageType_Sound_GetSoundInformations      = 0x43 | MessageCategory_Sound,
            MessageType_Sound_ShowSoundInformation      = 0x44 | MessageCategory_Sound,
            MessageType_Sound_Play                      = 0x45 | MessageCategory_Sound,
            MessageType_Sound_Pause                     = 0x46 | MessageCategory_Sound,
            MessageType_Sound_Stop                      = 0x47 | MessageCategory_Sound,
            MessageType_Sound_SetEffectParameters       = 0x48 | MessageCategory_Sound,
            MessageType_Sound_GetEffectParameters       = 0x49 | MessageCategory_Sound,

            MessageType_Sound_SetSeqVariables           = 0x50 | MessageCategory_Sound,
            MessageType_Sound_GetSeqVariables           = 0x51 | MessageCategory_Sound,
            MessageType_Sound_SetPreviewPlayerParameter = 0x52 | MessageCategory_Sound,

            MessageType_Sound_SoundEnd                  = 0x79 | MessageCategory_Sound,

            MessageType_AnimSound_Load3dModel           = 0x81 | MessageCategory_Sound,
            MessageType_AnimSound_LoadAnimation         = 0x82 | MessageCategory_Sound,
            MessageType_AnimSound_LoadAnimSound         = 0x83 | MessageCategory_Sound,
            MessageType_AnimSound_Get3dAnimationFrame   = 0x86 | MessageCategory_Sound,
            MessageType_AnimSound_Set3dAnimationFrame   = 0x87 | MessageCategory_Sound,
            MessageType_AnimSound_PlayAnimation         = 0x88 | MessageCategory_Sound,
            MessageType_AnimSound_StopAnimation         = 0x89 | MessageCategory_Sound,
            MessageType_AnimSound_SetAnimationRate      = 0x90 | MessageCategory_Sound,
            MessageType_AnimSound_SetAnimationLoop      = 0x91 | MessageCategory_Sound,
            MessageType_AnimSound_Set3dAnimationRange   = 0x92 | MessageCategory_Sound,
            MessageType_AnimSound_EditAnimSoundEvent    = 0x93 | MessageCategory_Sound,
            MessageType_AnimSound_UpdateAllEventID      = 0x94 | MessageCategory_Sound,

            MessageType_Force32 = 0xFFFFFFFF
        };

    struct Header
    {
        static const int StructSize = 8;

        MessageType type;
        int size;
    };

    //-----------------------------------------------------------------------
    /* Sound state */
    enum ToolSoundPreviewSoundState {
        /* No sound is specified */
        SoundState_Disabled =	0,
        /* Stopped */
        SoundState_Stopped =	1,
        /* Preparing for playback */
        SoundState_Preparing =	2,
        /* Playing */
        SoundState_Playing =	3,
        /* Paused */
        SoundState_Paused =	4,
        /* Define enum as 4-byte type */
        ToolSoundPreviewSoundState_4BYTE_ENUM =	0xFFFFFFFF
    };

    //-----------------------------------------------------------------------
    /* Sound label */
    typedef struct {
        /*   4B: Parameter enable flag (1: enabled; 0: disabled) */
        unsigned long	index;
        /* 260B: Sound name (Not a setting target when disabled) */
        unsigned char	label[260];
    } ToolSoundPreviewSoundLabel;

    //-----------------------------------------------------------------------
    /* Sound information */
    typedef struct {
        /* 4B: Sound state */
        ToolSoundPreviewSoundState	state;
    } ToolSoundPreviewSoundInfomation;

    //-----------------------------------------------------------------------
    /* Item container for batch sending */
    typedef struct {
        /* Offset to data */
        unsigned long	offsetForData;
        /* Number of items */
        unsigned long	itemCount;
        /* Size of 1 item */
        unsigned long	itemSize;
    } ToolSoundItemContainer;

    //-----------------------------------------------------------------------
    /* Bus type */
    enum ToolSoundBiquadType {
        /* OFF  */
        BiquadType_Off =	0,
        /* LPF  */
        BiquadType_Lpf =	1,
        /* HPF  */
        BiquadType_Hpf =	2,
        /* BPF512  */
        BiquadType_Bpf512 =	3,
        /* BPF1024  */
        BiquadType_Bpf1024 =	4,
        /* BPF2048  */
        BiquadType_Bpf2048 =	5,
        /* Define enum as 4-byte type */
        ToolSoundBiquadType_4BYTE_ENUM =	0xFFFFFFFF
    };

    //-----------------------------------------------------------------------
    /* Preview Player Parameters */
    typedef struct {
        /*  4B: Target preview player index */
        unsigned long	previewPlayerIndex;
        /*  4B: Volume */
        float	volume;
        /*  4B: Pitch */
        float	pitch;
        /*  4B: Pan */
        float	pan;
        /*  4B: surround pan */
        float	surroundPan;
        /*  4B: LPF frequency */
        float	lpfFrequency;
        /*  4B: BiquadType */
        ToolSoundBiquadType	biquadType;
        /*  4B: BiquadValue */
        float	biquadValue;
        /*  4B: Playback start position [ms] */
        unsigned long	startOffsetTime;
        /*  4B: Main output switch */
        long	mainOutEnabled;
        /*  4B: Wii Remote Control 1 output switch */
        long	remote0OutEnabled;
        /*  4B: Wii Remote Control 2 output switch */
        long	remote1OutEnabled;
        /*  4B: Wii Remote Control 3 output switch */
        long	remote2OutEnabled;
        /*  4B: Wii Remote Control 4 output switch */
        long	remote3OutEnabled;
        /*  4B: Main output */
        float	mainOutVolume;
        /*  4B: Wii Remote Control 1 output */
        float	remote0OutVolume;
        /*  4B: Wii Remote Control 2 output */
        float	remote1OutVolume;
        /*  4B: Wii Remote Control 3 output */
        float	remote2OutVolume;
        /*  4B: Wii Remote Control 4 output */
        float	remote3OutVolume;
        /*  4B: Main send */
        float	mainSend;
        /*  4B: Effect Aux A send */
        float	effectAuxASend;
        /*  4B: Effect Aux B send */
        float	effectAuxBSend;
        /*  4B: Effect Aux C send */
        float	effectAuxCSend;
        /*  4B: Fade in frame */
        long	fadeInFrames;
        /*  4B: Fade out frame */
        long	fadeOutFrames;
        /*  4B: Pose fade frame */
        long	pauseFadeFrames;
    } ToolSoundPreviewPlayerParameter;

} // namespace ProtocolSound

#endif // NW4C_VIEWER_SOUND_PROTOCOL_SOUND_H_
