/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

/*

  

  */

#ifndef NW4C_VIEWER_SYSTEM_POSSIZERECT_H_
#define NW4C_VIEWER_SYSTEM_POSSIZERECT_H_

namespace vw     {
namespace system {

//---------------------------------------------------------------------
//pos, size(nw::math::VEC2) <-> nw::ut::Rect
inline nw::ut::Rect PosSizeToRect( const nw::math::VEC2& pos, const nw::math::VEC2& size )
{
	return nw::ut::Rect( pos.x, pos.y, pos.x + size.x, pos.y + size.y );
}
inline void RectToPosSize( const nw::ut::Rect& rect, nw::math::VEC2* pos, nw::math::VEC2* size )
{
	(*pos)  = nw::math::VEC2( rect.GetX(), rect.GetY() );
	(*size) = nw::math::VEC2( rect.GetWidth(), rect.GetHeight() );
}

//---------------------------------------------------------------------
//In a rectangular region?
inline bool IsPosInRect( f32 x, f32 y,
			const nw::math::VEC2& leftTop, const nw::math::VEC2& size )
{
	return leftTop.x <= x && x <= leftTop.x + size.x
		&& leftTop.y <= y && y <= leftTop.y + size.y;
}
inline bool IsPosInRect( const nw::math::VEC2& pos,
			const nw::math::VEC2& leftTop, const nw::math::VEC2& size )
{
	return leftTop.x <= pos.x && pos.x <= leftTop.x + size.x
		&& leftTop.y <= pos.y && pos.y <= leftTop.y + size.y;
}
inline bool IsPosInRect( f32 x, f32 y, const nw::ut::Rect& rect )
{
	return rect.left <= x && x <= rect.right
		&& rect.top  <= y && y <= rect.bottom;
}
inline bool IsPosInRect( const nw::math::VEC2& pos,  const nw::ut::Rect& rect )
{
	return rect.left <= pos.x && pos.x <= rect.right
		&& rect.top  <= pos.y && pos.y <= rect.bottom;
}

}   /* namespace system */
}   /* namespace vw     */

#endif // POS_SIZE_RECT_H_
