/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_STREAM_H_
#define NW4C_VIEWER_SYSTEM_STREAM_H_

namespace vw     {
namespace system {

// MCS channel allocation
// System
//  'SY'(0x5359) | 0x8000 - 'SY'+0x10 (0x5369) | 0x8000
// 3D
//  '3D'(0x3344) | 0x8000 - '3D'+0x10 (0x3354) | 0x8000
// 2D
//  '2D'(0x3244) | 0x8000 - '2D'+0x10 (0x3254) | 0x8000
// Effect
//  'EF'(0x4546) | 0x8000 - 'EF'+0x10 (0x4546) | 0x8000
// Sound
//  'SD'(0x5344) | 0x8000 - 'SD'+0x10 (0x5354) | 0x8000
const u16    SYSTEM_CHANNEL   = 'SY' | 0x8000;
const u16    G3D_CHANNEL      = '3D' | 0x8000;
const u16    G2D_CHANNEL      = '2D' | 0x8000;
const u16    EFFECT_CHANNEL   = 'EF' | 0x8000;
const u16    SOUND_CHANNEL    = 'SD' | 0x8000;

#if defined(USE_SAMPLE)
// Sample
//  'sm'(0x736D) | 0x8000 - 'sm'+0x10 (0x737D) | 0x8000
const u16    SAMPLE_CHANNEL     = 'sm' | 0x8000;
#endif

// MCS channel definition for the system

// MCS channel definition for 3D
const u16    G3D_CHANNEL_PASSIVE  = G3D_CHANNEL + 0x0;
const u16    G3D_CHANNEL_ACTIVE   = G3D_CHANNEL + 0x1;
const u16    G3D_CHANNEL_SURVIVAL = G3D_CHANNEL + 0xF;

// MCS channel definition for 2D
const u16    LYT_CHANNEL_PASSIVE  = G2D_CHANNEL + 0x0;
const u16    LYT_CHANNEL_ACTIVE   = G2D_CHANNEL + 0x1;
const u16    LYT_CHANNEL_SURVIVAL = G2D_CHANNEL + 0xF;

// MCS channel definition for effects

// MCS channel definition for sound
const u16    SOUND_CHANNEL_PASSIVE  = SOUND_CHANNEL + 0x0;
const u16    SOUND_CHANNEL_ACTIVE   = SOUND_CHANNEL + 0x1;
const u16    ANIMSOUND_CHANNEL_PASSIVE  = SOUND_CHANNEL + 0x2;
const u16    ANIMSOUND_CHANNEL_ACTIVE   = SOUND_CHANNEL + 0x3;
const u16    ANIMSOUND_CHANNEL_SURVIVAL = SOUND_CHANNEL + 0xE;
const u16    SOUND_CHANNEL_SURVIVAL = SOUND_CHANNEL + 0xF;

#if defined(USE_SAMPLE)
// MCS channel definition for samples
const u16    SAMPLE_CHANNEL_PASSIVE  = SAMPLE_CHANNEL + 0x0;
const u16    SAMPLE_CHANNEL_ACTIVE   = SAMPLE_CHANNEL + 0x1;
const u16    SAMPLE_CHANNEL_SURVIVAL = SAMPLE_CHANNEL + 0xF;
#endif


// Default buffer size.
const u32    STREAM_DEFAULT_BUF_SIZE  = 0x20000;



void        Stream_Init         ( void );
void        Stream_UnInit       ( void );
bool        Stream_Polling      ( u16 channel );
bool        Stream_IsConnected  ( u16 channel );

u32         Stream_GetFileSize  ( const char *fileName );
u32         Stream_ReadFile     ( const char *fileName, void* buf );
u32         Stream_WriteFile    ( const char *fileName, const void* buf, u32 size );

void        Stream_CreateChannel( u16 channel, u32 bufSize = STREAM_DEFAULT_BUF_SIZE );
void*       Stream_TryCreateChannel( u16 channel, u32 bufSize = STREAM_DEFAULT_BUF_SIZE );

u32         Stream_GetReadableBytes( u16 channel );
u32         Stream_Read            ( u16 channel, void* data, u32 size );
u32         Stream_Peek            ( u16 channel, void* data, u32 size );
u32         Stream_Skip            ( u16 channel, u32 size );
u32         Stream_GetWritableBytes( u16 channel, u32* pWritableBytes );
u32         Stream_Write           ( u16 channel, const void* data, u32 size );


}   /* namespace system */
}   /* namespace vw     */



/* NW4C_VIEWER_SYSTEM_STREAM_H_ */
#endif
