/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_VTASK_H_
#define NW4C_VIEWER_SYSTEM_VTASK_H_

#include <viewer.h>

namespace vw     {
namespace system {


typedef void (*VTaskFunc)( void* arg );


struct VTaskObj
{
    VTaskFunc       func;   // Task main function
    void*           arg;    // Parameter arguments

    nw::ut::LinkListNode m_Link;
};


void    VTask_Init( void );
void    VTask_Start( void );
void    VTask_Finalize( void );

void    VTask_AddObj( VTaskObj* task );
void    VTask_RemoveObj( VTaskObj* task );


/*---------------------------------------------------------------------------*
  Name:         VTask_InitObj

  Description:  Initializes VTask objects

  Arguments:    task    VTask object to initialize
                func    VTask function
                arg     Parameter arguments

  Returns:      None.
 *---------------------------------------------------------------------------*/
inline void
VTask_InitObj( VTaskObj* task, VTaskFunc func, void* arg )
{
    NW_NULL_ASSERT( task );

    task->func = func;
    task->arg  = arg;
}




}   /* namespace system */
}   /* namespace vw     */



/* NW4C_VIEWER_SYSTEM_VTASK_H_ */
#endif
