/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_VIDEO_H_
#define NW4C_VIEWER_SYSTEM_VIDEO_H_

#ifndef VW_USER_MULTIPATH_RENDERING
#define VW_USER_MULTIPATH_RENDERING
#endif

#include <nw/types.h>
#include <nw/ut.h>
#include <nw/io.h>
#include <nw/os.h>
#include <nw/math.h>
#include <nw/font.h>
#include <nw/lyt.h>
#include <nw/gfx.h>
#include <nw/anim.h>
#include <nw/snd.h>

namespace vw     {
namespace system {

enum ScreenKind
{
    UPPER_SCREEN        = 0x1 << 0,                 //
    LOWER_SCREEN        = 0x1 << 1,                 //
    EXTENSION_SCREEN    = 0x1 << 2,                 //
    BOTH_SCREENS        = UPPER_SCREEN | LOWER_SCREEN | EXTENSION_SCREEN
};

struct VideoSetting
{
    nw::ut::FloatColor          clearColor;         //
    f32                         clearZ;             //
    u32                         frameRate;          //
    u16                         upperWidth;         //
    u16                         upperHeight;        //
    u16                         lowerWidth;         //
    u16                         lowerHeight;        //
    nw::gfx::RenderColorFormat  colorFormat;        //
    nw::gfx::RenderDepthFormat  depthFormat;        //
};

void                    Video_Init( const VideoSetting* setting = NULL );
void                    Video_InitMenu();
void                    Video_InitGraphics();
void                    Video_Finalize();
void                    Video_ChangeSetting( const VideoSetting* setting );
void                    Video_GetSetting( VideoSetting* setting );
void                    Video_SetFrameRate( u32 frameRate );
u32                     Video_GetFrameRate();

void                    Video_TransferBuffer();
void                    Video_WaitCommandList();
void                    Video_PresentBuffer();
void                    Video_SwapBuffer();
void                    Video_WaitVSync();

void                    Video_ClearBuffer();
void                    Video_ClearDepth();
void                    Video_SetRenderTarget(ScreenKind screenKind);
nw::gfx::IRenderTarget* Video_GetRenderTarget(ScreenKind screenKind);
nw::gfx::RenderContext* Video_GetRenderContext();

void                    Video_ClearBuffers();

f32                     Video_GetFPS();
void                    Video_CalcFPS();

u32                     Video_GetScreenWidth();
u32                     Video_GetScreenHeight();

void                    Video_SetScreenSize(u32 width, u32 height);
void                    Video_ChangeFormat();

bool                    Video_CanAllocateA(u32 size);
bool                    Video_CanAllocateB(u32 size);

}   /* namespace system */
}   /* namespace vw     */



/* NW4R_VIEWER_SYSTEM_VIDEO_H_ */
#endif
