/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

/*

  

  */

#ifndef NW4C_VIEWER_SYSTEM_WIN_H_
#define NW4C_VIEWER_SYSTEM_WIN_H_

#ifndef VW_USER_MULTIPATH_RENDERING
#define VW_USER_MULTIPATH_RENDERING
#endif

namespace vw     {
namespace system {

//========================================================================//
//							Window
//========================================================================//
struct Window {
	nw::math::VEC2	pos;  //Upper left coordinate
	nw::math::VEC2  size;
	nw::math::VEC2	shadowRelPos;
	nw::ut::Color8	shadowCol;		//If alpha = 0, no shadow
	nw::ut::Color8	backCol;
	f32 			wakuLineWidth;
	nw::ut::Color8	wakuCol;
	nw::ut::Color8	titleBakCol;
	f32 			titleMarginDx;
	f32 			titleMarginDy;
	f32				titleFontSize;
	nw::ut::Color8	titleFontCol;
	const char *    titleStr; //If NULL is set, no title

	Window()
	: pos( 100, 100 ),
	  size( 100, 100 ),
	  shadowRelPos( 6, 6 ),
	  shadowCol( 0, 0, 0, 255 * 3 / 8 ),
	  backCol( 0, 0, 0, 255 * 11 / 16 ),
	  wakuLineWidth( 2.0f ),
	  wakuCol( nw::ut::Color8::WHITE ),
	  titleBakCol( nw::ut::Color8::BLUE ),
	  titleMarginDx( 4.0f ),
	  titleMarginDy( 1.0f ),
	  titleFontSize( 8.0f ),
	  titleFontCol( nw::ut::Color8::WHITE ),
	  titleStr( NULL )
	{}
	void Disp()
	{
		DrawAllParam paramBak;
		paramBak.Push();
		{
			//Shadow
			if ( shadowCol.a ) {
				FillRectangle( pos + shadowRelPos, size, shadowCol );
			}
			//Background
			FillRectangle( pos, size, backCol );
			SetLineWidth( wakuLineWidth );
			if ( titleStr ) {
				//Title background
				nw::math::VEC2 lineWidthSize( wakuLineWidth, wakuLineWidth );
				nw::math::VEC2 titleBakSize( size.x, GetTitleHeight() );
				FillRectangle( pos, titleBakSize, titleBakCol );
				//Frame
				DrawRectangle( pos, size, wakuCol );
				DrawLine( pos + nw::math::VEC2( 0, titleBakSize.y ),
						  pos + titleBakSize, wakuCol );
				//Title text
				SetFontSize( titleFontSize );
				SetTextColor( titleFontCol );
				SetTextPos( pos.x + titleMarginDx, pos.y + titleMarginDy + 1.0f );
				Print( titleStr );
			} else {
				DrawRectangle( pos, size, wakuCol );
			}
		}

		paramBak.Pop();
	}
	nw::ut::Rect GetRect() const {
		nw::math::VEC2 pos2 = pos + size;
		return nw::ut::Rect( pos.x, pos.y, pos2.x, pos2.y );
	}
	f32 GetTitleHeight()  const { return FontSizeToDy( titleFontSize ) + titleMarginDy * 2; }
	nw::ut::Rect GetTitleRect() const { return nw::ut::Rect( pos.x, pos.y, pos.x + size.x, pos.y + GetTitleHeight() ); }
	nw::math::VEC2 GetClientPos()  const { return nw::math::VEC2( pos.x, pos.y + GetTitleHeight() ); }
	nw::math::VEC2 GetClientSize() const { return nw::math::VEC2( size.x, size.y - GetTitleHeight() ); }
	void SetClientSize( const nw::math::VEC2 & csize ) {
		size = nw::math::VEC2( csize.x, csize.y + GetTitleHeight() );
	}
	nw::math::VEC2 GetMinimumSize() const //Get minimum size (only the title text is displayed)
	{
		const int len = std::strlen( titleStr );
		const nw::math::VEC2 fsize = FontSizeToPixelSize( titleFontSize );
		const f32 wMin  = fsize.x * len + titleMarginDx * 2;
		const f32 hMin = fsize.y + titleMarginDy * 2;
		return nw::math::VEC2( wMin, hMin );
	}
	void Resize( const nw::math::VEC2& size ) //Resize (set a size that is not smaller than the minimum size)
	{
		const nw::math::VEC2 minSize = GetMinimumSize();
		this->size = nw::math::VEC2( nw::ut::Max(size.x, minSize.x), nw::ut::Max(size.y, minSize.y) );
	}
};

}   /* namespace system */
}   /* namespace vw     */

#endif //NW4C_VIEWER_SYSTEM_WIN_H_
