﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

/*

  
         
         
         
         

 */

#ifndef NW4C_VIEWER_SYSTEM_MAIN_H_
#define NW4C_VIEWER_SYSTEM_MAIN_H_

#include <viewer.h>

namespace vw     {
namespace system {

/*---------------------------------------------------------------------------*
  Name:         Startup
  Description:  Initialization of the system executed before the static initializer
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void Startup( void );

/*---------------------------------------------------------------------------*
  Name:         Init
  Description:  Initialize system
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void Init( void );

/*---------------------------------------------------------------------------*
  Name:         Finalize
  Description:  System finalization
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void Finalize( void );

/*---------------------------------------------------------------------------*
  Name:         Start
  Description:  Starts system processing.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void Start( void );


/*---------------------------------------------------------------------------*
  Name:         PollStream
  Description:  MCS communication processing
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void PollStream( void );


/*---------------------------------------------------------------------------*
  Name:         PreMain
  Description:  System pre-main routine
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void PreMain( void );


/*---------------------------------------------------------------------------*
  Name:         Main
  Description:  System main routine
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void Main( void );



/*---------------------------------------------------------------------------*
  Name:         Reset
  Description:  System reset routine
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void Reset( void );


/*---------------------------------------------------------------------------*
  Name:         DrawInfo
  Description:  System rendering
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void DrawInfo( void );


/*---------------------------------------------------------------------------*
  Name:         DrawBG
  Description:  Background rendering
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void DrawBG( void );



/*---------------------------------------------------------------------------*
  Name:         WriteBackConfigFile
  Description:  Writes back the configuration file
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void WriteBackConfigFile();


/*---------------------------------------------------------------------------*
  Name:         RestoreChangeMode
  Description:  Sets the mode change to enabled/disabled
  Arguments:    None.
  Returns:      Returns the value before setting.
 *---------------------------------------------------------------------------*/
bool RestoreChangeMode( bool enabled );


/*---------------------------------------------------------------------------*
  Name:         SetPointerType
  Description:  Switches the pointer display
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/

void SetPointerType( u32 icon_type );

/*---------------------------------------------------------------------------*
  Name:         EnableChangeMode
  Description:  Enables mode change
  Arguments:    None.
  Returns:      Returns the value before setting.
 *---------------------------------------------------------------------------*/
inline bool
EnableChangeMode( void )
{
    return RestoreChangeMode( true );
}


/*---------------------------------------------------------------------------*
  Name:         DisableChangeMode
  Description:  Disables mode change
  Arguments:    None.
  Returns:      Returns the value before setting.
 *---------------------------------------------------------------------------*/
inline bool
DisableChangeMode( void )
{
    return RestoreChangeMode( false );
}


}   /* namespace system */
}   /* namespace vw     */


/* NW4C_VIEWER_SYSTEM_MAIN_H_ */
#endif
