/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_EFFECT_MODULE_EFFECT_IMPL_H_
#define NW4C_VIEWER_EFFECT_MODULE_EFFECT_IMPL_H_

#include <viewer/system/Hid.h>
#include <viewer/effect/ModuleEffect.h>

namespace vw {
namespace effect {

class ModuleEffectImpl : public ModuleEffect
{
public:
    // Ctor
    ModuleEffectImpl();

    // Dtor
    ~ModuleEffectImpl();

    // Init
    virtual void    Init( void );

    // PollStream
    virtual void    PollStream( void );

    // Main
    virtual void    Main(vw::system::HidStatus& hidStatus);

    // Calc
    virtual void    Calc( void );

    // Draw
    virtual void    Draw( void );

    // DrawInfo
    virtual void    DrawInfo( void );

    // Destroy
    virtual void    Destroy( void );

    // OnModuleFocused
    virtual void    OnModuleFocused( void );

    // Command for playback all is executed
    virtual void        OnAnmCmd( u32   /* command    */,     // Command (RESET/PLAY/REV_PLAY/STOP/STEP/REV_STEP/LOOP/SET_FRAME)
                                  u32   /* target     */,     // Target (ALL/3D_MDL/2D_LYT)
                                  u32   /* targetID   */,     // Target ID (do not use when target is ANM_TGT_ALL)
                                  void* /* arg        */,     // Option parameters
                                  u32   /* trigModule */ );   // Module ID that is the trigger

    // <CODE>Available</CODE>
    virtual bool    Available( void ) { return true; }

    // HidControl
    void            SetMaxFrame(f32 maxFrame);
    void            SetLoopFlag(bool flag);

    // ResetMaxParticle
    void            ResetMaxParticle();

    // SetHidden
    static void     SetHidden(bool hide) { sHiddenFlag = hide; }

    // GetModuleG3dImpl
    static ModuleEffectImpl* GetInstance();

protected:

    // instance
    static ModuleEffectImpl* m_pInstance;

    // Hidden Flag
    static bool sHiddenFlag /* = true */;

private:
    s32 GetParticleCount();
    void ResetEffect();

    s32 mParticleCount;
    s32 mMaxParticle;

    friend class ModuleEffect;
};

}   /* namespace effect */
}   /* namespace vw     */

#endif
