/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/ModuleEffectImpl.h"
#include "../include/ConfigEffect.h"
#include "../include/SubModeManager.h"
#include "../include/AnimManager.h"

using namespace vw::common;
using namespace vw::system;

namespace vw  {
namespace effect {

static void     ConfigEffect_Memory_();
static void     ConfigEffect_WriteBack_Memory_();

/*---------------------------------------------------------------------------*
  Name:         ConfigEffect_Execute

  Description:  Effect-related configuration settings

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ConfigEffect_Execute()
{
    ConfigEffect_Memory_();
}

/*---------------------------------------------------------------------------*
  Name:         ConfigEffect_WriteBack

  Description:  Effect-related configuration write back

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ConfigEffect_WriteBack()
{
    ConfigEffect_WriteBack_Memory_();
}


/*---------------------------------------------------------------------------*
  Name:         ConfigEffect_Memory_
  Description:  Memory settings
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
static void
ConfigEffect_Memory_()
{
    f32 fValue;
    s32 iValue;

    // Default mode setting
    {
        const char* cmode = GetConfigFile()->GetString("/effect/config/sub_mode");
        if ( cmode )
        {
            if(0 == strcmp(cmode, "CAM"))
            {
                SubModeManager::GetInstance()->SetMode(vw::g3d::SubMode_Camera);
            }
            else if(0 == strcmp(cmode, "MNU"))
            {
                SubModeManager::GetInstance()->SetMode(vw::g3d::SubMode_Menu);
            }
            else if(0 == strcmp(cmode, "LOG"))
            {
                SubModeManager::GetInstance()->SetMode(vw::g3d::SubMode_Log);
            }
        }
    }

    // Maximum frame
    if ( GetConfigFile()->GetFloat("/effect/config/max_frame_count", &fValue ) )
    {
        if ( fValue >= 0.0f )
        {
            ModuleEffectImpl::GetInstance()->SetMaxFrame( fValue );
        }
    }

    // A loop playback flag
    if ( GetConfigFile()->GetInt("/effect/config/loop_flag", &iValue ) )
    {
        ModuleEffectImpl::GetInstance()->SetLoopFlag( iValue != 0 );
    }

    // Default playback rate setting
    if ( GetConfigFile()->GetFloat("/effect/animation/default_rate", &fValue ) )
    {
        AnimManager::GetInstance()->SetDefaultRate( fValue );
    }

    //u16 maxEffect = 100;
    //u16 maxEmitter = 1000;
    //u16 maxParticleManager = 1000;
    //u16 maxParticle = 10000;
    //u32 u32value;
    //f32 f32value;
    //s32 intvalue;
    //ModuleEffect::MemLocation breftMemLocation = ModuleEffect::MEMLOCATION_MEM1;
    //
    //if ( GetConfigFile() != NULL )
    //{
    //    if (GetConfigFile()->GetInt("/effect/config/xf_flush_safe", &intvalue))
    //    {
    //        bool b = (intvalue != 0) ? true : false;
    //        nw4r::ef::EffectSystem::GetInstance()->SetXFFlushSafe(b);
    //    }

    //    // Memory
    //    if (GetConfigFile()->GetUInt("/effect/memory/max_effect", &u32value))
    //    {
    //        if (u32value >= 0x10000)
    //        {
    //            u32value = 0xffff;
    //        }
    //        maxEffect = static_cast<u16>(u32value);
    //    }
    //    if (GetConfigFile()->GetUInt("/effect/memory/max_emitter", &u32value))
    //    {
    //        if (u32value >= 0x10000)
    //        {
    //            u32value = 0xffff;
    //        }
    //        maxEmitter = static_cast<u16>(u32value);
    //    }
    //    if (GetConfigFile()->GetUInt("/effect/memory/max_particlemanager", &u32value))
    //    {
    //        if (u32value >= 0x10000)
    //        {
    //            u32value = 0xffff;
    //        }
    //        maxParticleManager = static_cast<u16>(u32value);
    //    }
    //    if (GetConfigFile()->GetUInt("/effect/memory/max_particle", &u32value))
    //    {
    //        if (u32value >= 0x10000)
    //        {
    //            u32value = 0xffff;
    //        }
    //        maxParticle = static_cast<u16>(u32value);
    //    }
    //    const char* alloc_type;
    //    alloc_type = GetConfigFile()->GetString("/effect/mem_location/breft");
    //    if (alloc_type != NULL)
    //    {
    //        if (std::strncmp(alloc_type, "MEM2", 4) == 0)
    //        {
    //            breftMemLocation = ModuleEffect::MEMLOCATION_MEM2;
    //        }
    //        else if (std::strncmp(alloc_type, "MEM1", 4) == 0)
    //        {
    //            breftMemLocation = ModuleEffect::MEMLOCATION_MEM1;
    //        }
    //    }

    //    if (GetConfigFile()->GetInt("/effect/config/z_sort", &intvalue))
    //    {
    //        bool b = (intvalue != 0) ? true : false;
    //        ModuleEffect::GetEffectModule()->SetZSort(b);
    //    }

    //    if (GetConfigFile()->GetInt("/effect/config/skip_emitter", &intvalue))
    //    {
    //        bool b = (intvalue != 0) ? true : false;
    //        ModuleEffect::GetEffectModule()->SetSkipEmitter(b);
    //    }

    //    if (GetConfigFile()->GetInt("/effect/config/peakhold_enable", &intvalue))
    //    {
    //        bool b = (intvalue != 0) ? true : false;
    //        ModuleEffect::GetEffectModule()->SetPeakHoldEnable(b);
    //    }

    //    if (GetConfigFile()->GetUInt("/effect/config/peakhold_span", &u32value))
    //    {
    //        ModuleEffect::GetEffectModule()->SetPeakHoldSpan(u32value);
    //    }

    //    if (GetConfigFile()->GetFloat("/effect/config/guilde_scale", &f32value))
    //    {
    //        ModuleEffect::GetEffectModule()->SetGuideScale(f32value);
    //    }

    //    if (GetConfigFile()->GetFloat("/effect/config/emitlod_near", &f32value))
    //    {
    //        ModuleEffect::GetEffectModule()->SetEmitLODNear(f32value);
    //    }

    //    if (GetConfigFile()->GetFloat("/effect/config/emitlod_far", &f32value))
    //    {
    //        ModuleEffect::GetEffectModule()->SetEmitLODFar(f32value);
    //    }
    //}

    //ModuleEffect::GetEffectModule()->SetMemoryParams(maxEffect, maxEmitter, maxParticleManager, maxParticle);
    //ModuleEffect::GetEffectModule()->SetBreftLocation(breftMemLocation);
}


/*---------------------------------------------------------------------------*
  Name:         ConfigEffect_WriteBack_Memory_
  Description:  Memory settings
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
static void
ConfigEffect_WriteBack_Memory_()
{
    //if ( GetConfigFile() == NULL )
    //{
    //    return ;
    //}
    //
    //bool xfFlushSafe = nw4r::ef::EffectSystem::GetInstance()->GetXFFlushSafe();
    //GetConfigFile()->SetInt("/effect/config/xf_flush_safe", xfFlushSafe? 1 : 0 );
    //
    //u16 maxEffect;
    //u16 maxEmitter;
    //u16 maxParticleManager;
    //u16 maxParticle;
    //
    //ModuleEffect::GetEffectModule()->GetMemoryParams( &maxEffect, &maxEmitter, &maxParticleManager, &maxParticle );
    //
    //// Memory
    //GetConfigFile()->SetUInt("/effect/memory/max_effect",  maxEffect );
    //GetConfigFile()->SetUInt("/effect/memory/max_emitter", maxEmitter);
    //GetConfigFile()->SetUInt("/effect/memory/max_particlemanager", maxParticleManager);
    //GetConfigFile()->SetUInt("/effect/memory/max_particle", maxParticle);
    //
    //ModuleEffect::MemLocation breftMemLocation = ModuleEffect::GetEffectModule()->GetBreftLocation();
    //
    //GetConfigFile()->SetString("/effect/mem_location/breft", (breftMemLocation == ModuleEffect::MEMLOCATION_MEM1) ? "MEM1" : "MEM2" );
    //
    //bool z_sort = ModuleEffect::GetEffectModule()->GetZSort();
    //GetConfigFile()->SetInt("/effect/config/z_sort", z_sort ? 1 : 0);

    //bool skipEmitter = ModuleEffect::GetEffectModule()->GetSkipEmitter();
    //GetConfigFile()->SetInt("/effect/config/skip_emitter", skipEmitter ? 1 : 0);

}




}   /* namespace g3d */
}   /* namespace vw  */

