/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/MenuSubMode.h"

namespace vw  {
namespace effect {

#define ADD_MENU( type ) \
case vw::g3d::Menu_##type: \
{ \
    void* buf = pAllocator->Alloc(sizeof(vw::g3d::type##Menu)); \
    vw::g3d::type##Menu* menu = new(buf) vw::g3d::type##Menu(); \
    mMenu.push_back( menu ); \
} \
break;

#define DELETE_MENU( type ) \
case vw::g3d::Menu_##type: \
{ \
    vw::g3d::type##Menu* menu = (vw::g3d::type##Menu*)mMenu[i]; \
    menu->~type##Menu(); \
    pAllocator->Free( menu ); \
} \
break;

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuSubMode::MenuSubMode() : MenuSubModeBase()
{
     nw::os::IAllocator* pAllocator = Memory_GetAllocator();

     const vw::g3d::MenuType menus[] = {
        // [Video settings]
        /* NOTHING */

        // [System Basics]
        vw::g3d::Menu_CategorySystem,
        vw::g3d::Menu_LoadBinary,
        vw::g3d::Menu_ScreenCapture,
        vw::g3d::Menu_FrameRate,
        vw::g3d::Menu_ScreenPreview,

        // [3D Settings/General]
        /* NOTHING */

        // [3D Settings/Various Information]
        vw::g3d::Menu_Category3D_Information,
        vw::g3d::Menu_GridScale,
        vw::g3d::Menu_GridDivision,
        vw::g3d::Menu_TargetCameraAxis,

        // [3D Settings/Camera]
        vw::g3d::Menu_Category3D_Camera,
        vw::g3d::Menu_ForceUseCameraWScale,

        // [3D Settings/Lights]
        /* NOTHING */

        // [3D Settings/Stereoscopic view]
        vw::g3d::Menu_Category3D_Stereovision,
        vw::g3d::Menu_DepthLevel,
        vw::g3d::Menu_Factor,
        vw::g3d::Menu_RealSwitch,

        // [Effect settings]
        vw::g3d::Menu_CategoryEffect,
        vw::g3d::Menu_ParticleEmitterAxisForm,
        vw::g3d::Menu_ConstraintTargetBoneAxis

        // [2D setting]
        /* NOTHING */

        // [2D Settings/Camera Settings]
        /* NOTHING */

        // [2D Settings/Stereoscopic view]
        /* NOTHING */
    };
    const int menuNum = sizeof( menus ) / sizeof( vw::g3d::MenuType );

    mMenu = nw::ut::MoveArray<vw::g3d::IMenu*>( menuNum, pAllocator );
    for ( int i = 0; i < menuNum; ++i )
    {
        switch ( menus[i] )
        {
        ADD_MENU( ScreenCapture )
        ADD_MENU( LoadBinary )
        ADD_MENU( Sort )
        ADD_MENU( FrameRate )
        ADD_MENU( DepthLevel )
        ADD_MENU( Factor )
        ADD_MENU( RealSwitch )
        ADD_MENU( GridScale )
        ADD_MENU( GridDivision )
        ADD_MENU( ScreenPreview )
        ADD_MENU( TargetCameraAxis )
        ADD_MENU( ParticleEmitterAxisForm )
        ADD_MENU( ConstraintTargetBoneAxis )
        ADD_MENU( ForceUseCameraWScale )
        ADD_MENU( CategorySystem )
        ADD_MENU( Category3D_Information )
        ADD_MENU( Category3D_Camera )
        ADD_MENU( Category3D_Stereovision )
        ADD_MENU( CategoryEffect )
        default:
            break;
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuSubMode::~MenuSubMode()
{
    nw::os::IAllocator* pAllocator = Memory_GetAllocator();

    for ( int i = 0; i < mMenu.size(); ++i )
    {
        switch ( mMenu[i]->GetMenuType() )
        {
        DELETE_MENU( ScreenCapture )
        DELETE_MENU( LoadBinary )
        DELETE_MENU( Sort )
        DELETE_MENU( FrameRate )
        DELETE_MENU( DepthLevel )
        DELETE_MENU( Factor )
        DELETE_MENU( RealSwitch )
        DELETE_MENU( GridScale )
        DELETE_MENU( GridDivision )
        DELETE_MENU( ScreenPreview )
        DELETE_MENU( TargetCameraAxis )
        DELETE_MENU( ParticleEmitterAxisForm )
        DELETE_MENU( ConstraintTargetBoneAxis )
        DELETE_MENU( ForceUseCameraWScale )
        DELETE_MENU( CategorySystem )
        DELETE_MENU( Category3D_Information )
        DELETE_MENU( Category3D_Camera )
        DELETE_MENU( Category3D_Stereovision )
        DELETE_MENU( CategoryEffect )
        default:
            break;
        }
    }
    mMenu.clear();
}


}   /* namespace effect */
}   /* namespace vw  */
