/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include <viewer/effect/ProtocolEM4C.h>
#include "../include/ProtocolExecEM4C.h"
#include "../include/ModuleEffectImpl.h"

#include <viewer/g3d/CameraMgr.h>
#include "../../g3d/include/InfoDrawer.h"
#include "../../system/include/HidManager.h"
#include "../../g3d/include/ResourceManager.h"
#include "../../g3d/include/SceneManager.h"
#include "../../g3d/include/AnimManager.h"
#include "../../g3d/include/SceneRenderer.h"
#include "../include/AnimManager.h"
#include "../include/InfoDrawer.h"
#include "../include/ConfigEffect.h"
#include "../include/SubModeManager.h"
#include "../include/SurvivalStreamTask.h"
#include "../include/ParamSetter.h"


namespace vw {
namespace effect {


// An instance.
ModuleEffectImpl* ModuleEffectImpl::m_pInstance = NULL;


static ModuleEffectImpl sInstance;

// When changing the target mode, whether it is permitted to move the focus to SAMPLE mode is determined by the IsHidden() method.
//  Variable to dynamically set with these conditions.
bool ModuleEffectImpl::sHiddenFlag = true;



// Function to get instance
ModuleEffect* ModuleEffect::GetInstance()
{
    //NW_ASSERT( ModuleEffectImpl::m_pInstance != NULL );
    //return ModuleEffectImpl::m_pInstance;
    return &sInstance;
}
ModuleEffectImpl* ModuleEffectImpl::GetInstance()
{
    //NW_ASSERT( ModuleEffectImpl::m_pInstance != NULL );
    //return ModuleEffectImpl::m_pInstance;
    return &sInstance;
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffect::Initialize
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffect::Initialize()
{/*
    if (ModuleEffectImpl::m_pInstance != NULL) return;
    void *buf = Memory_Alloc(sizeof(ModuleEffectImpl));
    if (buf == NULL) return;
    ModuleEffectImpl::m_pInstance = new(buf) ModuleEffectImpl();
*/

}
/*---------------------------------------------------------------------------*
  Name:         ModuleEffect::Finalize
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffect::Finalize()
{/*
    if (ModuleEffectImpl::m_pInstance == NULL) return;
    ModuleEffectImpl::m_pInstance->~ModuleEffectImpl();
    Memory_Free( (void*)ModuleEffectImpl::m_pInstance );
    ModuleEffectImpl::m_pInstance = NULL;

*/
}


/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::ModuleEffectImpl
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
ModuleEffectImpl::ModuleEffectImpl() : ModuleEffect()
{
    mParticleCount = 0;
    mMaxParticle = 0;
}
/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::~ModuleEffectImpl
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
ModuleEffectImpl::~ModuleEffectImpl()
{
}

// Command for playback all is executed
void
ModuleEffectImpl::OnAnmCmd( u32   command    ,     // Command (RESET/PLAY/REV_PLAY/STOP/STEP/REV_STEP/LOOP/SET_FRAME)
                         u32   target     ,     // Target (ALL/3D_MDL/2D_LYT)
                         u32   targetID   ,     // Target ID (do not use when target is ANM_TGT_ALL)
                         void* arg        ,     // Option parameters
                         u32   trigModule  )    // Module ID that is the trigger
{
    switch (target)
    {
    case system::ANM_TGT_ALL:
        AnimManager::GetInstance()->OnAnmCmd( command, arg );
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::Init
  Description:  Effect module initialization function
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::Init()
{
    // Scene, resource initialization
    AnimManager::Initialize();
    InfoDrawer::Initialize();
    SubModeManager::Initialize();
    ParamSetter::Initialize();

    // Establish serial communication connection
    const u32 BODY_BUFFER_SIZE = 32 * 1024;
    vw::system::Stream_CreateChannel( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_ACTIVE, BODY_BUFFER_SIZE );
    vw::system::Stream_CreateChannel( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_PASSIVE, BODY_BUFFER_SIZE );

    // Initialize existence confirmation communication
    vw::effect::SurvivalStream_Init();

    // Config settings
    ConfigEffect_Execute();
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::PoolStream
  Description:  Effect module communication function
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::PollStream( void )
{
    // Serial communications check
    ProtocolExec_Main();
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::Main
  Description:  The main function of the effect module
  Arguments:    hidStatus  Gets the status of the pad assigned to the effect module.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::Main(vw::system::HidStatus &hidStatus)
{
    // Config load process
    if (system::ConfigFile::IsReloaded()) ConfigEffect_Execute();

    // All types of control with HID
    if(EFFECT_ID == vw::system::App::GetInstance().GetFocusID())
    {
        // Sub-mode control
        SubModeManager::GetInstance()->Main(hidStatus);

        // Update of control managing the direct control of the module
        if(vw::g3d::SubMode_Camera == SubModeManager::GetInstance()->GetCurrentModeType())
        {
            vw::effect::AnimManager::GetInstance()->HidControl(hidStatus);

            switch (system::GetPadEvent( hidStatus, system::GetEffectPadBindings() ))
            {
            case system::EF_STOP_ALL_ANIMATIONS:
                ResetMaxParticle();
                break;
            case system::EF_SWITCH_VIEW_CAMERA:
                vw::g3d::CameraMgr::GetInstance()->SwitchToNextTargetCamera();
                vw::g3d::CameraMgr::GetInstance()->SwitchToNextViewCamera();
                break;
            case system::EF_CHANGE_INFO_RENDER_STATUS:
                vw::effect::InfoDrawer::GetInstance()->ChangeRenderInfoFlag();
            default:
                break;
            }
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::Calc
  Description:  The main function of the sample module
  Arguments:    pad  Gets the status of the pad assigned to the sample module.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::Calc()
{
    // Get the current number of displayed particles, and update the maximum display particles.
    mParticleCount = GetParticleCount();
    if (mMaxParticle < mParticleCount)
    {
        mMaxParticle = mParticleCount;
    }
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::Draw
  Description:  Function to display to the screen
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::Draw()
{
    // Scene rendering is waiting for all G3dModuleImpl
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffect::DrawInfo
  Description:  Function to display information on the screen
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::DrawInfo()
{
    // Check for skip processing
    bool IsUpdateScene = vw::system::App::GetInstance().IsUpdateScene();
    bool IsRenderScene = vw::system::App::GetInstance().IsRenderScene();
    if (!IsUpdateScene || !IsRenderScene) return;

    // Effect module processing
    if(vw::g3d::InfoDrawer::GetInstance()->IsRender2DInfo() ||
       vw::g3d::SubMode_Menu == SubModeManager::GetInstance()->GetCurrentModeType())
    {
        // information display
        InfoDrawer::DrawInfo info;
        {
            info.SubModeName   = SubModeManager::GetInstance()->GetCurrentModeName();
            info.ParticleCount = mParticleCount;
            info.MaxParticle   = mMaxParticle;
            info.StepFrame     = vw::effect::AnimManager::GetInstance()->GetStepFrame();
            info.LoopFlag      = vw::effect::AnimManager::GetInstance()->GetLoopFlag();
            info.FrameCount    = vw::effect::AnimManager::GetInstance()->GetFrame();
            info.MaxFrameCount = vw::effect::AnimManager::GetInstance()->GetMaxFrameCount();
        }
        vw::system::Draw2DHelper::GetInstance().SetScreenSize(vw::system::Video_GetScreenWidth(), vw::system::Video_GetScreenHeight());
        vw::system::FontDrawer::GetInstance().SetScreenSize(vw::system::Video_GetScreenWidth(), vw::system::Video_GetScreenHeight());
        InfoDrawer::GetInstance()->Draw(info);

        // Sub-mode information display
        SubModeManager::GetInstance()->Draw();
    }
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::Destroy
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::Destroy()
{
    ParamSetter::Finalize();
    InfoDrawer::Finalize();
    SubModeManager::Finalize();
    AnimManager::Finalize();
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::OnModuleFocused
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::OnModuleFocused()
{
    vw::effect::AnimManager::GetInstance()->Synchronize();
    vw::effect::AnimManager::GetInstance()->OnTargetModuleChanged(vw::system::EFFECT_ID);
    vw::g3d::AnimManager::GetInstance()->OnTargetModuleChanged(vw::system::EFFECT_ID);
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::GetId
  Description:  Sets the effect maximum frame
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::SetMaxFrame(f32 maxFrame)
{
    vw::effect::AnimManager::GetInstance()->SetMaxFrameCount( maxFrame );
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::SetLoopFlag
  Description:  Sets the effect loop flag
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::SetLoopFlag(bool flag)
{
    vw::effect::AnimManager::GetInstance()->SetLoopFlag( flag );
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::ResetMaxParticle
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ModuleEffectImpl::ResetMaxParticle()
{
    mMaxParticle = 0;
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEffectImpl::GetParticleCount
  Description:  Gets current number of displayed particles
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------*/
s32
ModuleEffectImpl::GetParticleCount()
{
    s32 count = 0;
    nw::gfx::SceneContext *sceneContext = vw::g3d::SceneManager::GetInstance()->GetSceneContext();
    nw::gfx::ParticleModelArray::iterator end = sceneContext->GetParticleModelEnd();
    for (nw::gfx::ParticleModelArray::iterator i = sceneContext->GetParticleModelBegin(); i != end;)
    {
        nw::gfx::ParticleModel* model = *i++;
        NW_NULL_ASSERT(model);

        for (u32 p=0; p<model->GetParticleSetsCount(); ++p)
        {
            const nw::gfx::ParticleSet* particleSet = model->GetParticleSets(p);
            const nw::gfx::ParticleCollection* collection = particleSet->GetParticleCollection();
            count += collection->GetCount();
        }
    }
    return count;
}

}   /* namespace effect */
}   /* namespace vw     */
