﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../../g3d/include/SubMode.h"
#include "../include/MenuSubMode.h"
#include "../../g3d/include/LogSubMode.h"

namespace vw  {
namespace effect {


#define ADD_SUBMODE( space, type ) \
{ \
    void* buf = pAllocator->Alloc(sizeof(space::type##SubMode)); \
    space::type##SubMode* mode = new(buf) space::type##SubMode(); \
    mMode.push_back( mode ); \
} \

#define DELETE_SUBMODE( space, type ) \
{ \
    space::type##SubMode* mode = (space::type##SubMode*)mMode[i]; \
    mode->~type##SubMode(); \
    pAllocator->Free( mode ); \
} \

SubModeManager* SubModeManager::m_pInstance = NULL;

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManager*
SubModeManager::GetInstance()
{
    NW_ASSERT( m_pInstance != NULL );
    return m_pInstance;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManager::SubModeManager() : vw::g3d::SubModeManagerBase()
{
     nw::os::IAllocator* pAllocator = Memory_GetAllocator();

     const vw::g3d::SubModeType modes[] = {
        vw::g3d::SubMode_Camera,
        vw::g3d::SubMode_Menu,
        vw::g3d::SubMode_Log
    };
    const int modeNum = sizeof( modes ) / sizeof( vw::g3d::SubModeType );

    mMode = nw::ut::MoveArray<vw::g3d::ISubMode*>( modeNum, pAllocator );
    for ( int i = 0; i < modeNum; ++i )
    {
        switch ( modes[i] )
        {
        case vw::g3d::SubMode_Camera:
            ADD_SUBMODE( vw::g3d, Camera )
            break;
        case vw::g3d::SubMode_Menu:
            ADD_SUBMODE( vw::effect, Menu )
            break;
        case vw::g3d::SubMode_Log:
            ADD_SUBMODE( vw::g3d, Log )
            break;
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManager::~SubModeManager()
{
    nw::os::IAllocator* pAllocator = Memory_GetAllocator();

    for ( int i = 0; i < mMode.size(); ++i )
    {
        switch ( mMode[i]->GetModeType() )
        {
        case vw::g3d::SubMode_Camera:
            DELETE_SUBMODE( vw::g3d, Camera )
            break;
        case vw::g3d::SubMode_Menu:
            DELETE_SUBMODE( vw::effect, Menu )
            break;
        case vw::g3d::SubMode_Log:
            DELETE_SUBMODE( vw::g3d, Log )
            break;
        }
    }
    mMode.clear();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SubModeManager::Initialize()
{
    if (m_pInstance != NULL) return;

    void *buf = Memory_GetAllocator()->Alloc(sizeof(SubModeManager));
    if (buf == NULL) return;

    m_pInstance = new(buf) SubModeManager();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SubModeManager::Finalize()
{
    if (m_pInstance == NULL) return;

    m_pInstance->~SubModeManager();
    Memory_GetAllocator()->Free(m_pInstance);

    m_pInstance = NULL;
}


}   /* namespace effect */
}   /* namespace vw  */
