/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nw/viewer/ef/ModuleEF.h>
#include <viewer/system.h>
#include <viewer/effect.h>

#include "../../common/include/if_common.h"

namespace nw     {
namespace viewer {
namespace ef     {

// Macro that defines the function to get the iterator.
#define DEFINE_NODE_ITER_GETTER(name)\
name##Iter \
ModuleEF::Get##name(u32 idx) const\
{\
    if(idx >= vw::g3d::SceneManager::GetInstance()->Get##name##sCount()) { \
        return name##Iter();\
    } \
\
    vw::g3d::SceneManager::name##List::iterator iter\
    = advance_iterator(vw::g3d::SceneManager::GetInstance()->Get##name##sBegin(), idx);\
\
    return name##Iter(iter == SCN_MGR->Get##name##sEnd() ? NULL : &*iter);\
}\
u32 \
ModuleEF::Get##name##Num() const\
{\
    return vw::g3d::SceneManager::GetInstance()->Get##name##sCount();\
}

/*---------------------------------------------------------------------------*
  Name:         ModuleEF::ModuleEF

  Description:  Constructor

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
/* ctor */
ModuleEF::ModuleEF()
 : sys::Module( vw::system::EFFECT_ID ),
   mResListener( NULL )
{
}

DEFINE_NODE_ITER_GETTER(ParticleModel)
DEFINE_NODE_ITER_GETTER(ParticleEmitter)

/*---------------------------------------------------------------------------*
  Name:         ModuleEF::GetResourceListener

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
IResourceListener*
ModuleEF::GetResourceListener()
{
    return mResListener;
}
/*---------------------------------------------------------------------------*
  Name:         ModuleEF::SetResourceListener

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
ModuleEF::SetResourceListener( IResourceListener* listener )
{
    mResListener = listener;
}
/*---------------------------------------------------------------------------*
  Name:         ParticleModelIter::GetNext

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
ParticleModelIter
ParticleModelIter::GetNext() const
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::SceneManager::ParticleModelList::iterator iter = ++PMODELELEMITER(m_pEntity);

    return iter == SCN_MGR->GetParticleModelsEnd() ? NULL : &*iter;
}
/*---------------------------------------------------------------------------*
  Name:         ParticleModelIter::GetParticleEmitterIter

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::ParticleEmitter *
ParticleModelIter::GetParticleEmitter(u32 idx) const
{
    if (!IsValid())
    {
        return NULL;
    }

    return static_cast<nw::gfx::ParticleEmitter*>
                (*advance_iterator(PMODELELEMITER(m_pEntity)->GetParticleEmittersBegin(), idx));
}
/*---------------------------------------------------------------------------*
  Name:         ParticleModelIter::GetParticleEmitteriterNum

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
u32
ParticleModelIter::GetParticleEmitteriterNum() const
{
    if (!IsValid())
    {
        return 0;
    }

    vw::g3d::SceneManager::ParticleModelList::iterator iter = ++PMODELELEMITER(m_pEntity);

    return iter == SCN_MGR->GetParticleModelsEnd() ? 0 : iter->GetParticleEmittersCount();
}
/*---------------------------------------------------------------------------*
  Name:         ParticleEmitterIter::GetNext

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
ParticleEmitterIter
ParticleEmitterIter::GetNext() const
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::SceneManager::ParticleEmitterList::iterator iter = ++PEMITTERELEMITER(m_pEntity);

    return iter == SCN_MGR->GetParticleEmittersEnd() ? NULL : &*iter;
}
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::GetEffectSystem
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//nw4r::ef::EffectSystem*
//ModuleEF::GetEffectSystem() const
//{
//    return nw4r::ef::EffectSystem::GetInstance();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::GetResource
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//nw4r::ef::Resource*
//ModuleEF::GetResource() const
//{
//    return nw4r::ef::Resource::GetInstance();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::GetFrame
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//ModuleEF::GetFrame() const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    if (module == NULL)
//    {
//        return 0;
//    }
//
//    if (module->GetPreviewManager().HasPreview())
//    {
//        return module->GetPreviewManager().GetCurrentFrame();
//    }
//    else
//    {
//        return module->effectCounter;
//    }
//}
//
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::IsAnimStop
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//bool
//ModuleEF::IsAnimStop() const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    if (module == NULL)
//    {
//        return false;
//    }
//    return module->GetPreviewManager().IsAnimStop();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::IsAnimStep
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//bool
//ModuleEF::IsAnimStep() const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    if (module == NULL)
//    {
//        return false;
//    }
//    return module->GetPreviewManager().IsAnimStep();
//}
//
//void
//ModuleEF::GetMemoryParams(u16* maxEffect, u16* maxEmitter, u16* maxParticleManager, u16* maxParticle, u32* maxGroup) const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    module->GetMemoryParams(maxEffect, maxEmitter, maxParticleManager, maxParticle, maxGroup);
//}
//
//nw4r::ut::List*
//ModuleEF::GetLeasedEffectList() const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    return module->GetLeasedEffectList();
//}
//
//nw4r::ut::List*
//ModuleEF::GetLeasedEmitterList() const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    return module->GetLeasedEmitterList();
//}
//
//nw4r::ut::List*
//ModuleEF::GetLeasedParticleManagerList() const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    return module->GetLeasedParticleManagerList();
//}
//
//nw4r::ut::List*
//ModuleEF::GetLeasedParticleList() const
//{
//    vw::effect::ModuleEffect* module = vw::effect::ModuleEffect::GetEffectModule();
//    return module->GetLeasedParticleList();
//}
//
//
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::Calc
//
//  Description:  Calc processing
//
//  Arguments:    None.
//
//  Returns:      None.
// *---------------------------------------------------------------------------*/
//void
//ModuleEF::Calc()
//{
//    vw::system::Module* module3D = vw::system::App::GetModuleInstance( vw::system::G3D_ID );
//    if ( module3D )
//    {
//        module3D->RisePostCalcCallback( vw::system::EFFECT_ID );
//    }
//
//    vw::system::Module* module2D = vw::system::App::GetModuleInstance( vw::system::G2D_ID );
//    if ( module2D )
//    {
//        module2D->RisePostCalcCallback( vw::system::EFFECT_ID );
//    }
//
//    sys::Module::Calc();
//}
//
//
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::Draw
//
//  Description:  Rendering
//
//  Arguments:    Sync
//
//  Returns:      None.
// *---------------------------------------------------------------------------*/
//void
//ModuleEF::Draw( bool sync /* = true */ )
//{
//    vw::system::Module* module3D = vw::system::App::GetModuleInstance( vw::system::G3D_ID );
//    if ( module3D )
//    {
//        module3D->RisePostDrawCallback( vw::system::EFFECT_ID );
//    }
//
//    vw::system::Module* module2D = vw::system::App::GetModuleInstance( vw::system::G2D_ID );
//    if ( module2D )
//    {
//        module2D->RisePostDrawCallback( vw::system::EFFECT_ID );
//    }
//
//    sys::Module::Draw( sync );
//}
//
////==================================
//// Resource listener
////==================================
//class ResManager : public vw::effect::ModuleEffect::ReloadListener
//{
//public:
//    /* ctor */          ResManager() : mListener( NULL ) {}
//    void                SetListener( IResourceListener* listener ) { mListener = listener; }
//
//    virtual void        InitializeMemory(u16 maxEffect, u16 maxEmitter, u16 maxParticleManager, u16 maxParticle, u32 maxGroup) { if (mListener) { NotifyInitializeMemory(maxEffect, maxEmitter, maxParticleManager, maxParticle, maxGroup); } }
//
//    virtual void UpdateResEmitter( nw4r::ef::ResEmitter& resEmitter ) { if (mListener) { NotifyUpdateResEmitter(resEmitter); } }
//    virtual void UpdateResTexture( nw4r::ef::ResTexture& resTexture ) { if (mListener) { NotifyUpdateResTexture(resTexture); } }
//
//    virtual void RemoveResEmitter( nw4r::ef::ResEmitter& resEmitter ) { if (mListener) { NotifyRemoveResEmitter(resEmitter); } }
//    virtual void RemoveResTexture( nw4r::ef::ResTexture& resTexture ) { if (mListener) { NotifyRemoveResTexture(resTexture); } }
//
//Private:
//    IResourceListener*  mListener;
//
//    void NotifyInitializeMemory(u16 maxEffect, u16 maxEmitter, u16 maxParticleManager, u16 maxParticle, u32 maxGroup);
//
//    void NotifyUpdateResEmitter( nw4r::ef::ResEmitter& resEmitter );
//    void NotifyUpdateResTexture( nw4r::ef::ResTexture& resTexture );
//
//    void NotifyRemoveResEmitter( nw4r::ef::ResEmitter& resEmitter );
//    void NotifyRemoveResTexture( nw4r::ef::ResTexture& resTexture );
//};
//
//void
//ResManager::NotifyInitializeMemory(u16 maxEffect, u16 maxEmitter, u16 maxParticleManager, u16 maxParticle, u32 maxGroup)
//{
//    mListener->OnInitializeMemory(maxEffect, maxEmitter, maxParticleManager, maxParticle, maxGroup);
//}
//
//void
//ResManager::NotifyUpdateResEmitter(nw4r::ef::ResEmitter& resEmitter)
//{
//    mListener->OnUpdateResEmitter(resEmitter);
//}
//
//void
//ResManager::NotifyUpdateResTexture(nw4r::ef::ResTexture& resTexture)
//{
//    mListener->OnUpdateResTexture(resTexture);
//}
//
//void
//ResManager::NotifyRemoveResEmitter(nw4r::ef::ResEmitter& resEmitter)
//{
//    mListener->OnRemoveResEmitter(resEmitter);
//}
//
//void
//ResManager::NotifyRemoveResTexture(nw4r::ef::ResTexture& resTexture)
//{
//    mListener->OnRemoveResTexture(resTexture);
//}
//
///*---------------------------------------------------------------------------*
//  Name:         ModuleEF::SetResourceListener
//
//  Description:
//
//  Arguments:    listener
//
//  Returns:      None.
// *---------------------------------------------------------------------------*/
//void
//ModuleEF::SetResourceListener( IResourceListener* listener )
//{
//    static ResManager sListener;
//
//    mResListener = listener;
//
//    sListener.SetListener( listener );
//
//    nw4r::ut::DynamicCast<vw::effect::ModuleEffect*>( vw::effect::ModuleEffect::GetInstance() )->ClearReloadListener();
//
//    nw4r::ut::DynamicCast<vw::effect::ModuleEffect*>( vw::effect::ModuleEffect::GetInstance() )->AddReloadListener( &sListener );
//}

#undef DEFINE_NODE_ITER_GETTER

} /* namespace ef     */
} /* namespace viewer */
} /* namespace nw4r   */
