/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G2D_ARCFILERESOURCEACCESSOR_H_
#define NW4C_VIEWER_G2D_ARCFILERESOURCEACCESSOR_H_

#if defined(__MWERKS__)
#pragma once
#endif

#include "FileResourceAccessor.h"
#include <nw/font/font_ArchiveFont.h>
#include <nw/lyt/lyt_ArcResourceAccessor.h>
#include <nw/lyt/lyt_Arc.h>

namespace vw
{
namespace g2d
{

class ArcFileResourceAccessor : public FileResourceAccessor
{
protected:
    class FindInfoImpl : public FileResourceAccessor::FindInfo
    {
    public:
        FindInfoImpl();
        virtual ~FindInfoImpl();
        virtual bool IsValid();
        void FindFirst(const nw::lyt::ARCDir& arcDir);
        virtual bool FindNext();
        virtual const char* GetName();

        nw::ut::MoveArray<char> m_Name;
        nw::lyt::ARCDir m_ArcDir;
        nw::lyt::ARCDirEntry m_ArcDirEntry;
    };

public:
    ArcFileResourceAccessor();
    virtual ~ArcFileResourceAccessor();

    // --- ResourceAccessor Interface Implementation ---

    virtual void* GetResource(
        nw::lyt::ResType resType,
        const char* name,
        u32* pSize);

    virtual nw::font::Font* GetFont(const char* name);

    virtual const nw::lyt::TextureInfo GetTexture(const char* name);

    //
    //
    //
    // @details
    // Destroy resource when the content of loaded resource is updated.
    // Destroy all resources when the path is changed.
    //
    // Destroy all resources when cannot determine if there have been changes.
    //
    //
    //
    virtual bool Attach(const char* path);

    // --- FileResourceAccessor Interface Implementation ---

    virtual nw::ut::MovePtr<FindInfo> FindResource(nw::lyt::ResType resType);

protected:
    void Finalize();
    bool OpenResourceDir(nw::lyt::ResType resType, nw::lyt::ARCDir* pArcDir);

    nw::lyt::ArcResourceAccessor* m_pArcResourceAccessor;
    nw::ut::MoveArray<u8> m_Archive;
    nw::lyt::ARCHandle m_ArcHandle;
    struct FontInfo
    {
        nw::lyt::FontKey key;
        nw::font::ArchiveFont* pFont;
        nw::ut::MoveArray<u8> buffer;
    };
    typedef nw::ut::MoveArray<FontInfo> FontInfoArray;
    FontInfoArray m_ArchiveFontInfoArray;
};

} // namespace g2d
} // namespace vw

#endif // NW4C_VIEWER_G2D_ARCFILERESOURCEACCESSOR_H_
