/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G2D_FLASHAPPEAL_H_
#define NW4C_VIEWER_G2D_FLASHAPPEAL_H_

#include <nw/lyt/lyt_Group.h>
#include <nw/lyt/lyt_Pane.h>
#include <nw/ut/ut_MoveArray.h>

namespace vw  {
namespace g2d {


//
// Selected object emphasized display control
//
class FlashAppeal
{
//-- public ---
public:
    /* ctor */          FlashAppeal();

    void                Init();

    void                Update();
    bool                IsTurnOff();

    void                Clear()                             { Stop(); }
    void                Start();
    void                Stop();

    void                SetPane(nw::lyt::Pane* pPane);

    void                SetGroup(nw::lyt::Group* pGroup);

    void                SetInterval( u16 interval )         { mInterval = interval; }
    void                SetNum     ( u16 num )              { mFlashNum = num;      }

//-- private ---
private:
    void                UpdateVisibility_(); // Update resource render settings

    u16                 mInterval;           // Blink interval
    u16                 mFlashNum;           // Number of times to flash

    u16                 mFrameCnt;           // Frame counter
    u16                 mFlashCnt;           // Blink counter
    nw::lyt::Pane*      mpPane;
    nw::lyt::Group*     mpGroup;
    typedef nw::ut::MoveArray<bool> GroupVisibleVec;
    GroupVisibleVec     mGroupVisibleVec;
    bool                mbPaneVisible;

#if defined(NW_PLATFORM_CTR)
    u8 padding[3];
#endif
};

}   /* namespace g2d */
}   /* namespace vw  */


/* NW4C_VIEWER_G2D_FLASHAPPEAL_H_ */
#endif
