/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G2D_MODULE_H_
#define NW4C_VIEWER_G2D_MODULE_H_

#include <nn.h>
#include <nw/lyt.h>
#include <viewer/g2d/g2d_main.h>
#include "CameraMgr.h"
#include "ResLayout.h"
#include "FlashAppeal.h"

// Test to make it possible to place in texture VRAM.
//#define NW_TEST_LYT_VRAM 1

namespace vw  {
namespace g2d {

//============================================
//
// 2D module implementation
//
//============================================
class ModuleG2dImpl : public ModuleG2d
{
//--------public members ------------------------
public:
    //NW4R_UT_RUNTIME_TYPEINFO;

    static ModuleG2dImpl& GetInstance();

    /* ctor */      ModuleG2dImpl();

    virtual void    Init     ( void );
    virtual void    PollStream( void );
    virtual void    Main     ( vw::system::HidStatus& hidStatus );
    virtual void    UpdateFrame( void );
    virtual void    Calc     ( void );
    virtual void    Draw     ( void );
    virtual void    DrawInfo ( void );
    virtual bool    Available( void );                      // When FALSE is returned, invalidated and Init, Main, and DrawInfo cannot be called.
    virtual bool    IsHidden ( void );                      // When TRUE is returned, skips when mode is switched.
    virtual void    Destroy  ( void );

    // Command for playback all is executed
    virtual void    OnAnmCmd( u32   /* command    */,     // Command (RESET/PLAY/REV_PLAY/STOP/STEP/REV_STEP/LOOP/SET_FRAME)
                              u32   /* target     */,     // Target (ALL/3D_MDL/2D_LYT)
                              u32   /* targetID   */,     // Target ID (do not use when target is ANM_TGT_ALL)
                              void* /* arg        */,     // Option parameters
                              u32   /* trigModule */ );   // Module ID that is the trigger
    virtual void    WriteBackConfigFile();

    CameraMgr&      GetCameraMgr() const    { return mCameraMgr; }

    bool            IsOrthoZCompare() const         { return mbOrthoZCompare; }
    void            SetOrthoZCompare(bool bEnable)  { mbOrthoZCompare = bEnable; }

    ResLayout       &GetResLayout() {   return mResLayout;  }
    bool            IsAnimStop()    {   return mbAnimStop;  }

    bool Load(
        const char* pathName,
        const char* layoutName,
        u32         type);

    void ToUpper();
    void ToLower();
    bool IsToUpper() const { return mbToUpper; }

    bool IsZCompareEnable() const { return mbOrthoZCompare; }
    void SetZCompareEnable(bool value) { mbOrthoZCompare = value; }

#ifdef NW_TEST_LYT_VRAM
    nw::lyt::MemArea GetTexMemArea() { return mTexMemArea; }
    void SetTexMemArea(nw::lyt::MemArea memArea) { mTexMemArea = memArea; }
#endif

    //
    //
    //
    //
    //
    //
    void Reset();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetGraphicsResource(nw::lyt::GraphicsResource* pGraphicsResource)
    {
        mpGraphicsResource = pGraphicsResource;
    }

    //
    //
    //
    //
    //
    nw::lyt::GraphicsResource* GetGraphicsResource() const
    {
        return mpGraphicsResource;
    }

    //
    void SetAvailable(bool bAvailable = true)
    {
        mbAvailable = bAvailable;
    }

    static const int    FovyMin = 1;
    static const int    FovyMax = 179;

//-------Private members ------------------------
private:

    void            InitUI_();
    bool            InitGraphicsResource_();
    void            CalcCamera( nw::lyt::DrawInfo* pDrawInfo, nw::lyt::Layout* pLayout );
    void            SetupCamera( nw::lyt::Layout* pLayout );
    void            InitGXSetting(int width, int height);
    void            LoadCameraConfig();
    void            DoMenuProcess( vw::system::HidStatus& hidStatus );
    void            DoDrawInfo();
    void            DoPad      ( vw::system::HidStatus& hidStatus );

    void            SetAnimStop(bool bStop);

    void            SetCursor( u32 cursor );
    void            ChangeCursorToPrev();
    void            ChangeCursorToNext();
    void SetFlashAppeal();

#if 0 // FIXME!
    system::MenuNode*           Res2DConstruct();
#endif

    nw::lyt::GraphicsResource*  mpGraphicsResource;

    ResLayout&                  mResLayout;
    CameraMgr&                  mCameraMgr;

    FlashAppeal                 mFlashAppeal;       // Emphasized display state

    bool                        mbAnimStop;

    bool                        mbModeEnable;

    bool                        mbOrthoZCompare;

    bool                        mbNeedResetFilter;

    bool mbToUpper;

    bool mbAvailable;

#if defined(NW_PLATFORM_CTR)
    u8 padding[2];
#endif

#ifdef NW_TEST_LYT_VRAM
    // Specify memory where texture is placed.
    nw::lyt::MemArea            mTexMemArea;
#endif

#if 0 // FIXME!
    system::MenuSystem*         mMenuSystem;
#endif


    enum
    {
        CURSOR_FRAME,
        CURSOR_ANIM,
        CURSOR_TARGETTYPE,
        CURSOR_TARGET,
        CURSOR_NULLPANEDISP,
        CURSOR_CAMTYPE,
        CURSOR_SCREEN,
        CURSOR_MAX
    };

    u32             mCursor;

#if 1 // FIXME! Simple measure until area around menus is aligned
    const char* GetCursorMark(u32 cursor) const;

    // Simple version to maintain the label display state only.
    class UILabel
    {
    public:
        UILabel()
        : mVisible(true)
        {}

        void SetVisible(bool visible)
        {
            mVisible = visible;
        }

        bool IsVisible() const
        {
            return mVisible;
        }

    private:
        bool mVisible;
#if defined(NW_PLATFORM_CTR)
        u8 padding[3];
#endif
    };
#endif

    UILabel         mLbFrame;       // Frame count
    UILabel         mLbAnimName;    // Animation name
    UILabel         mLbAnimIndex;   // Animation index
    UILabel         mLbAnimTargetType; // Animation Target Types
    UILabel         mLbAnimTargetName; // Animation target name
    UILabel         mLbAnimTargetIdx; // Animation target index
    UILabel         mLbNullPaneDisp;    // Whether to render the null pane and boundary pane
    UILabel         mLbCamType;     // Camera type
    UILabel         mLbScreenType;  // Wide mode
    UILabel         mLbSize;        // Screen size

    typedef nw::ut::MoveArray<UILabel*> UILabelVec;
    UILabelVec mCursorLabelVec;
};


}   /* namespace g2d */
}   /* namespace vw  */


/* NW4C_VIEWER_G2D_MODULE_H_ */
#endif
