/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "../include/CameraMgr.h"
#include <viewer/g2d/CameraAccess.h>
#include "../include/Resources.h"

namespace vw  {
namespace g2d {

/*---------------------------------------------------------------------------*
  Name:         CameraInfo::GetProjMtx
  Description:  Gets projection matrix for layout use.
  Arguments:    pMtx     Region to get the projection matrix
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
CameraInfo::GetProjMtx( nn::math::MTX44* pMtx ) const
{
    res::GetCameraMgr().GetProjMtx( pMtx );
}

/*---------------------------------------------------------------------------*
  Name:         CameraInfo::IsOrtho
  Description:  Determines whether it is an orthogonal projection
  Arguments:    None.
  Returns:      Returns TRUE if using an orthogonal projection; returns FALSE if using a perspective projection.

 *---------------------------------------------------------------------------


*/
bool
CameraInfo::IsOrtho() const
{
    return res::GetCameraMgr().IsOrtho();
}

/*---------------------------------------------------------------------------*
  Name:         CameraInfo::IsPerspective
  Description:  Determines whether it is an perspective projection
  Arguments:    None.
  Returns:      Returns TRUE if using a perspective projection; returns FALSE if using an orthogonal projection.

 *---------------------------------------------------------------------------


*/
bool
CameraInfo::IsPerspective() const
{
    return res::GetCameraMgr().IsPerspective();
}

/*---------------------------------------------------------------------------*
  Name:         CameraInfo::GetViewMtx
  Description:  Gets view matrix for layout use.
  Arguments:    pMtx     Region to get the view matrix
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
CameraInfo::GetViewMtx( nn::math::MTX34* pMtx ) const
{
    res::GetCameraMgr().GetViewMtx( pMtx );
}

/*---------------------------------------------------------------------------*
  Name:         CameraInfo::IsWide
  Description:  Determines whether widescreen settings are used.
  Arguments:    None.
  Returns:      Returns TRUE for widescreen settings; returns FALSE otherwise.

 *---------------------------------------------------------------------------


*/
bool
CameraInfo::IsWide() const
{
    return res::GetCameraMgr().IsWide();
}


}   /* namespace g2d */
}   /* namespace vw  */



