﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/InfoSubMode.h"
#include "../include/Module.h"
#include "../include/Resources.h"
#include "../../system/include/HidManager.h"
#include "../../system/include/VersionDraw.h"

namespace vw  {
namespace g2d {

InfoSubMode::InfoSubMode()
{
}

/* virtual */
InfoSubMode::~InfoSubMode()
{
}

/* virtual */
bool
InfoSubMode::Main(vw::system::HidStatus& /* hidStatus */)
{
    return false;
}

/* virtual */
void
InfoSubMode::Draw()
{
    vw::system::Draw2DHelper& draw2D = vw::system::Draw2DHelper::GetInstance();
    draw2D.BeginDraw();

    draw2D.Disable(
        vw::system::Draw2DHelper::ALPHABLEND |
        vw::system::Draw2DHelper::CULLFACE |
        vw::system::Draw2DHelper::DEPTHTEST);

    s32 aw = 10;
    s32 ah = 40;
    s32 w = 320 - aw * 2;
    s32 h = 240 - ah - aw;
    draw2D.FillRectangle(aw, ah, w, h, nw::ut::Color8(64, 64, 96, 220));
    draw2D.EndDraw();

    vw::system::FontDrawer& font = vw::system::FontDrawer::GetInstance();
    font.SetColor(nw::ut::Color8::WHITE);
    font.BeginDraw();

    font.DrawString(aw + 4, ah, "");

    font.SetFontSize(12, 14);
    font.SetFixedWidth(8.0f);

    CameraMgr& cameraMgr = res::GetCameraMgr();

    f32 dist = cameraMgr.GetDistance();
    f32 fovy = cameraMgr.GetFovy();
    f32 far = cameraMgr.GetFar();
    f32 near = cameraMgr.GetNear();
    f32 minNear = cameraMgr.GetMinNear();
    f32 basePlane = cameraMgr.GetStereoBasePlane();
    f32 parallax = cameraMgr.GetOrthoParallax();
    f32 depthLevel = nw::ut::Max(1.0f, dist - basePlane);

    font.DrawString("             FOVY = %f\n", fovy);
    font.DrawString("  CAMERA DISTANCE = %f\n", dist);
    font.DrawString("              FAR = %f\n", dist + far);
    font.DrawString("       ORTHO NEAR = %f\n", dist + near);
    font.DrawString("       PERSP NEAR = %f\n", nw::ut::Max(dist + near, minNear));
    font.DrawString("      DEPTH LEVEL = %f\n", depthLevel);
    font.DrawString("   ORTHO PARALLAX = %f\n", parallax);

    font.EndDraw();
}

/* virtual */
const char*
InfoSubMode::GetModeName() const
{
    return "Info";
}

/* virtual */
SubModeType
InfoSubMode::GetModeType() const
{
    return SubMode_Info;
}

}   /* namespace g2d */
}   /* namespace vw  */
