/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nn.h>
#include <viewer.h>

#include <nw/lyt.h>
#include <viewer/g2d.h>
#include <viewer/system/Capture.h>
#include "../include/Module.h"
#include "../include/ProtocolExecCTR.h"
#include "../include/SubModeManager.h"
#include "../../system/include/PerformDrawer.h"
#include <nw/viewer/sys/Application.h>

namespace vw
{
namespace g2d
{

namespace
{

ModuleG2dImpl sModuleG2d;
ResLayout sResLayout; // Layout resource
CameraMgr sCameraMgr; // Camera manager
nw::lyt::GraphicsResource sGraphicsResource;
nw::lyt::Drawer sDrawer;
nw::lyt::DrawInfo sDrawInfo; // Rendering Data


#if 0
/*---------------------------------------------------------------------------*
  Name:         ReadDvdFile
  Description:  Reads a file from the DVD.
  Arguments:    fileName: Filename
  Returns:      Pointer to the memory where the file was read
 *---------------------------------------------------------------------------


*/
void*
ReadDvdFile(
    const char*     fileName
)
{
    DVDFileInfo fileInfo;
    if (! DVDOpen(fileName, &fileInfo))
    {
        return NULL;
    }

    u32 fileLenUp32 = nw4r::ut::RoundUp(DVDGetLength(&fileInfo), 32);
    void* readBuf = Memory_Alloc(fileLenUp32, 32);
    s32 readBytes = DVDRead(&fileInfo, readBuf, s32(fileLenUp32), 0);
    NW_ASSERT(readBytes > 0);
    DVDClose(&fileInfo);

    return readBuf;
}

/*---------------------------------------------------------------------------*
  Name:         AfterInitGXSetting
  Description:  Sets GX settings that may affect other renders to the default setting state
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
AfterInitGXSetting()
{
    GXSetNumIndStages(0);
    for (int i = 0; i < GX_MAX_TEVSTAGE; ++i)
    {
        GXSetTevDirect(GXTevStageID(GX_TEVSTAGE0 + i));
    }
}
#endif

}   // Namespace

namespace res {

/*---------------------------------------------------------------------------*
  Name:         GetResLayout
  Description:  Gets a layout resource instance.
  Arguments:    None.
  Returns:      Layout resource instance
 *---------------------------------------------------------------------------


*/
ResLayout&
GetResLayout()
{
    return sResLayout;
}

/*---------------------------------------------------------------------------*
  Name:         GetCameraMgr
  Description:  Gets a camera manager instance.
  Arguments:    None.
  Returns:      Camera manager instance
 *---------------------------------------------------------------------------


*/
CameraMgr&
GetCameraMgr()
{
    return sCameraMgr;
}

/*---------------------------------------------------------------------------*
  Name:         GetDrawInfo
  Description:  Gets a render information instance.
  Arguments:    None.
  Returns:      Render information instance
 *---------------------------------------------------------------------------


*/
nw::lyt::DrawInfo&
GetDrawInfo()
{
    return sDrawInfo;
}

/*---------------------------------------------------------------------------*
  Name:         GetDrawer
  Description:  Gets a Drawer instance.
  Arguments:    None.
  Returns:      Drawer instance
 *---------------------------------------------------------------------------


*/
nw::lyt::Drawer&
GetDrawer()
{
    return sDrawer;
}

} /* namespace res */

ModuleG2d*
ModuleG2d::GetInstance()
{
    return &ModuleG2dImpl::GetInstance();
}

ModuleG2dImpl&
ModuleG2dImpl::GetInstance()
{
    return sModuleG2d;
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::ModuleG2dImpl
  Description:  Constructor
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
/* ctor */
ModuleG2dImpl::ModuleG2dImpl()
 : ModuleG2d()
 , mResLayout( sResLayout )
 , mCameraMgr( sCameraMgr )
 , mbAvailable( true )
{
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::InitUI
  Description:  UI initialization
                For the PC-Viewer Global constructor, the TextWriter is not used.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::InitUI_( void )
{
#if 0 // FIXME!
    const f32 LABEL_BASE_X = 24;
    const f32 LABEL_BASE_Y = 256;
    f32 lineHeight = system::TextWriter_GetInstance()->GetLineHeight();

    mLbFrame.SetPos         ( LABEL_BASE_X, LABEL_BASE_Y );
    mLbAnimName.SetPos      ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 1 );
    mLbAnimIndex.SetPos     ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 2 );
    mLbAnimPaneName.SetPos  ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 3 );
    mLbAnimPaneIdx.SetPos   ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 4 );
    mLbNullPaneDisp.SetPos  ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 5 );
    mLbCamType.SetPos       ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 6 );
    mLbFovy.SetPos          ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 7 );
    mLbScreenType.SetPos    ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 8 );
    mLbSize.SetPos          ( LABEL_BASE_X, LABEL_BASE_Y + lineHeight * 9 );

    // Level that does not set focus
    mLbAnimIndex.SetEnabled( false );
    mLbSize.SetEnabled( false );

    system::App::GetUIManager().AddObj( &mLbFrame       );
    system::App::GetUIManager().AddObj( &mLbAnimName    );
    system::App::GetUIManager().AddObj( &mLbAnimIndex   );
    system::App::GetUIManager().AddObj( &mLbAnimPaneName);
    system::App::GetUIManager().AddObj( &mLbAnimPaneIdx );
    system::App::GetUIManager().AddObj( &mLbNullPaneDisp);
    system::App::GetUIManager().AddObj( &mLbCamType     );
    system::App::GetUIManager().AddObj( &mLbFovy        );
    system::App::GetUIManager().AddObj( &mLbScreenType  );
    system::App::GetUIManager().AddObj( &mLbSize        );
#endif

    mCursorLabelVec = UILabelVec(CURSOR_MAX, Memory_GetAllocator());
    mCursorLabelVec.push_back(&mLbFrame);
    mCursorLabelVec.push_back(&mLbAnimName);
    mCursorLabelVec.push_back(&mLbAnimTargetType);
    mCursorLabelVec.push_back(&mLbAnimTargetName);
    mCursorLabelVec.push_back(&mLbNullPaneDisp);
    mCursorLabelVec.push_back(&mLbCamType);
    mCursorLabelVec.push_back(&mLbScreenType);
    mCursorLabelVec.push_back(&mLbSize);
    NW_ASSERT(mCursorLabelVec.size() == CURSOR_MAX);

#if 1 // FIXME!
    // Set to off because the NULL pane display does not function.
    mLbNullPaneDisp.SetVisible(false);
    // Set to off until the specifications are determined.
    mLbScreenType.SetVisible(false);
#endif

    this->SetCursor( CURSOR_ANIM );
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::InitGraphicsResource_
  Description:  Initializes GraphicsResource.
  Arguments:    Pointer to GraphicsResource.
  Returns:      Returns true on success.
 *---------------------------------------------------------------------------


*/
bool
ModuleG2dImpl::InitGraphicsResource_()
{
    sGraphicsResource.StartSetup();
    const wchar_t* resourcePath = 0;
    for (int i = 0;
         (resourcePath = sGraphicsResource.GetResourcePath(i)) != NULL;
         ++i)
    {
        nn::fs::FileReader fileReader;

        nn::Result result = fileReader.TryInitialize(resourcePath);
        if (result.IsFailure())
        {
            // Disable layout module if resource cannot be read.
            return false;
        }

        u32 size = fileReader.GetSize();
        char* fileBuf = static_cast<char*>(Memory_AllocDevice(size, 128 /* align */));
        fileReader.Read(fileBuf, s32(size));
        fileReader.Finalize();

        sGraphicsResource.SetResource(i, fileBuf, size, false);

        Memory_Free(fileBuf);
    }

    sGraphicsResource.FinishSetup();

    return true;
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::Init
  Description:  Initialization
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::Init( void )
{
    // Enable the layout module only if the user has explicitely enabled it to use an embedded implementation of Viewer.
    // 
    // 
    if (vw::system::App::GetInstance().IsEmbeddedViewer())
    {
        if (!nw::viewer::sys::App::GetInstance().GetModuleLYT().GetModuleEnable())
        {
            mbAvailable = false;
            return;
        }
    }

    this->InitUI_();

    mResLayout.Init();
    mCameraMgr.Init();
    mFlashAppeal.Init();

    // Use the Viewer allocator if an allocator has not been configured.
    {
        nw::os::IAllocator* pAllocator = nw::lyt::Layout::GetAllocator();
        nw::os::IAllocator* pDeviceAllocator = nw::lyt::Layout::GetDeviceMemoryAllocator();

        if (!pAllocator || !pDeviceAllocator)
        {
            if (!pAllocator)
            {
                pAllocator = Memory_GetAllocator();
            }

            if (!pDeviceAllocator)
            {
                pDeviceAllocator = Memory_GetAllocatorDevice();
            }

            nw::lyt::Initialize(pAllocator, pDeviceAllocator);
        }
    }

    // Loads the global resource file.
    // GraphicsResource is created inside Viewer the user has not configured it.
    if (!mpGraphicsResource)
    {
        if (this->InitGraphicsResource_())
        {
            mpGraphicsResource = &sGraphicsResource;
        }
        else
        {
            mbAvailable = false;
        }
    }

    // Code so far is to determine whether the module can be enabled.
    // Subsequent processing is halted if disabled.
    if (!mbAvailable)
    {
        return;
    }

    // Only set the channel to receive data if the module is enabled.
    ProtocolExecCTR::GetInstance().Initialize();

    sDrawInfo.SetGraphicsResource(mpGraphicsResource);
    sDrawer.Initialize(*mpGraphicsResource);

    mbModeEnable = true;
    mbAnimStop = false;
    mbOrthoZCompare = false;
    mbNeedResetFilter = true;
    mbToUpper = true;

#ifdef NW_TEST_LYT_VRAM
    mTexMemArea = nw::lyt::MEMAREA_FCRAM;
#endif

#if 0 // FIXME!
    mMenuSystem = NULL;
#endif

    // Initialize existence confirmation communication
    // vw::sample::SurvivalStream_Init();

#if 0 // FIXME!
    // Initialize MCS channel for SAMPLE communication
    system::Stream_CreateChannel( system::LYT_CHANNEL_ACTIVE, 256 * 1024 );  // Viewer-initiated communication channel
    system::Stream_CreateChannel( system::LYT_CHANNEL_PASSIVE, 256 * 1024  ); // Tool-initiated communication channel
#endif

    this->LoadCameraConfig();

#if 0 // FIXME!
    // Build menu resource.
    system::MenuNode *menu = Res2DConstruct();
    system::Menu::GetRootNode()->AddMenu( menu );
#endif
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::PoolStream
  Description:  Communication processing
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::PollStream( void )
{
    // Monitoring of communication commands
    ProtocolExecCTR::GetInstance().Main();
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::Main
  Description:  Main processing
  Arguments:    pad     Pad input
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::Main( vw::system::HidStatus& hidStatus )
{
    if (system::ConfigFile::IsReloaded())
    {
        this->LoadCameraConfig();
    }

    if (G2D_ID == vw::system::App::GetInstance().GetFocusID())
    {
        if (!SubModeManager::GetInstance().Main(hidStatus) &&
            SubMode_Main == SubModeManager::GetInstance().GetCurrentModeType())
        {
            this->DoPad(hidStatus);

            // Execute menu processing
            this->DoMenuProcess(hidStatus);
        }
    }
}

//
//
//
void ModuleG2dImpl::SetFlashAppeal()
{
    switch (mResLayout.GetAnimManager().GetTargetType())
    {
    case AnimManager::TargetType_Pane:
        mFlashAppeal.SetPane(mResLayout.GetAnimManager().GetTargetPane());
        break;

    case AnimManager::TargetType_Group:
        if (mResLayout.GetAnimManager().GetTargetGroup())
        {
            mFlashAppeal.SetGroup(mResLayout.GetAnimManager().GetTargetGroup());
        }
        else
        {
            mFlashAppeal.SetPane(mResLayout.GetLayout()->GetRootPane());
        }
        break;

    default:
        break;
    }
}

//
//
//
//
//
void
ModuleG2dImpl::DoPad( vw::system::HidStatus& /*hidStatus*/ )
{
    // TODO: All processes are listed without being divided.
    //       Must be cleaned up

    // Reconsider operation with DPD
#if 0
    // Automatic switching of the animation target pane with DPD
    if (pad.IsPressed(VWPAD_WBUTTON_B))
    {
        s16 posX, posY;
        if (pad.GetPointXY_WPAD(&posX, &posY))
        {
            // Make the center the origin.
            posX -= WPAD_CENTER_X;
            posY -= WPAD_CENTER_Y;
            posY = static_cast<s16>(-posY); // Make the up direction positive

            s32 animSel = -1;
            if (pad.IsPressed_WPAD(VWPAD_WBUTTON_UP))         { animSel = 0; }
            else if (pad.IsPressed_WPAD(VWPAD_WBUTTON_RIGHT)) { animSel = 1; }
            else if (pad.IsPressed_WPAD(VWPAD_WBUTTON_DOWN))  { animSel = 2; }
            else if (pad.IsPressed_WPAD(VWPAD_WBUTTON_LEFT))  { animSel = 3; }

            mResLayout.ChangeAnimeByDpd(nn::math::VEC2(posX, posY), animSel);
            return;
        }
    }
#endif

    if (Pad_IsPressed(PAD_BUTTON_START))
    {
        // For system use
        return;
    }

#if 0 // Use during document creation.
    if (Pad_IsTriggered(PAD_TRIGGER_L))
    {
        char root[1024] = "";
        nn::hio::CTR::GetEnvironmentVariable(root, sizeof(root), "NW4C_ROOT");
        std::string path;
        path = std::string(root) + "\\viewer1\\tool\\Viewer\\upper.bmp";
        vw::system::CaptureInterface::Capture(path.c_str(), NN_GX_DISPLAY0);
        path = std::string(root) + "\\viewer1\\tool\\Viewer\\lower.bmp";
        vw::system::CaptureInterface::Capture(path.c_str(), NN_GX_DISPLAY1);
    }
#endif

    if (Pad_IsRepeated(PAD_BUTTON_UP))
    {
        this->ChangeCursorToPrev();
    }
    else if (Pad_IsRepeated(PAD_BUTTON_DOWN))
    {
        this->ChangeCursorToNext();
    }

    AnimManager& animManager = mResLayout.GetAnimManager();

    if ( animManager.HasAnim() )
    {
        // Animation frame control

        if (Pad_IsTriggered(PAD_BUTTON_A))      // Toggle play/stop
        {
            if ( animManager.IsStopAtEnd() )
            {
                animManager.SetFrame( 0.0f );
                if (animManager.GetTargetIdx() == 0)
                {
                    SysAnmCtrl_ResetAll();
                    SysAnmCtrl_PlayAll();
                }
                if (! animManager.IsPlaying())                       // Put into playback state when not in playback state.
                {
                    SetAnimStop(false);
                }
            }
            else
            {
                SetAnimStop(animManager.IsPlaying());
                if (animManager.GetTargetIdx() == 0)
                {
                    if (! animManager.IsPlaying())
                    {
                        SysAnmCtrl_StopAll();
                    }
                    else
                    {
                        SysAnmCtrl_PlayAll();
                    }
                }
            }
        }
        else if (                                                   // Stop and return to start frame
             Pad_IsTriggered(PAD_BUTTON_B)
          || Pad_IsLongPressed(PAD_BUTTON_A)
        )
        {
            SetAnimStop( true );
            animManager.SetFrame( 0.0f );
            if (animManager.GetTargetIdx() == 0)
            {
                SysAnmCtrl_StopAll();
                SysAnmCtrl_ResetAll();
            }
        }
    }

    switch ( mCursor )
    {
    //------------
    // Frame Operations
    case CURSOR_FRAME:
        {
            if ( ! animManager.HasAnim() )
            {
                this->SetCursor( CURSOR_ANIM );
                return;
            }
            if (Pad_IsRepeated(PAD_BUTTON_RIGHT))     // Frame advance.
            {
                animManager.Step( 1.0f );
                if (animManager.GetTargetIdx() == 0)
                {
                    SysAnmCtrl_StepAll( 1.0f );
                }
                SetAnimStop(true);
            }
            if (Pad_IsRepeated(PAD_BUTTON_LEFT))      // Reverse frame advance
            {
                animManager.Step( -1.0f );
                if (animManager.GetTargetIdx() == 0)
                {
                    SysAnmCtrl_RevStepAll( 1.0f );
                }
                SetAnimStop(true);
            }
        }
        break;
    //------------
    // Animation Operations
    case CURSOR_ANIM:
        {
            bool bAnimChange = false;

            // Switch animation
            if ( animManager.GetAnimNum() > 1 )
            {
                if (Pad_IsTriggered(PAD_BUTTON_LEFT))
                {
                    animManager.ChangeAnimToPrev();
                    bAnimChange = true;
                }
                else if (Pad_IsTriggered(PAD_BUTTON_RIGHT))
                {
                    animManager.ChangeAnimToNext();
                    bAnimChange = true;
                }
            }

            // The animation is switched, and set frame to 0 if being played
            if ( bAnimChange && animManager.IsPlaying() )
            {
                animManager.SetFrame( 0.0f );
            }
        }
        break;
    //------------
    // Select animation target type
    case CURSOR_TARGETTYPE:
        {
            bool bChange = false;

            // Switch animation target type
            if (Pad_IsTriggered(PAD_BUTTON_LEFT))
            {
                animManager.ChangeTargetTypeToPrev();
                bChange = true;
            }
            else if (Pad_IsTriggered(PAD_BUTTON_RIGHT))
            {
                animManager.ChangeTargetTypeToNext();
                bChange = true;
            }

            if (bChange)
            {
                this->SetFlashAppeal();
            }
        }
        break;

    //------------
    // Animation Operations
    case CURSOR_TARGET:
        {
            bool bChange = false;

            // Switch animation
            if ( animManager.GetTargetNum() > 1 )
            {
                if (Pad_IsTriggered(PAD_BUTTON_LEFT))
                {
                    animManager.ChangeTargetToPrev();
                    bChange = true;
                }
                else if (Pad_IsTriggered(PAD_BUTTON_RIGHT))
                {
                    animManager.ChangeTargetToNext();
                    bChange = true;
                }
            }

            // Indicate select state when switching operation target pane.
            if (bChange)
            {
                this->SetFlashAppeal();
            }
        }
        break;

    //------------
    // Whether to render the null pane and boundary pane
    case CURSOR_NULLPANEDISP:
        {
            if (Pad_IsTriggered(PAD_BUTTON_LEFT | PAD_BUTTON_RIGHT))
            {
                #if ! defined(NW_RELEASE)
                    sDrawInfo.SetDebugDrawMode(! sDrawInfo.IsDebugDrawMode());
                #endif
            }
        }
        break;
    //------------
    // Camera type operation
    case CURSOR_CAMTYPE:
        {
            if (Pad_IsTriggered(PAD_BUTTON_LEFT | PAD_BUTTON_RIGHT))
            {
                mCameraMgr.SetPerspective(!mCameraMgr.IsPerspective());
            }
        }
        break;
    //------------
    // Wide mode operation
    case CURSOR_SCREEN:
        {
            if (Pad_IsTriggered(PAD_BUTTON_LEFT | PAD_BUTTON_RIGHT))
            {
                mCameraMgr.SetWide( ! mCameraMgr.IsWide() );
            }
        }
        break;
    }
}


/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::UpdateFrame
  Description:
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::UpdateFrame( void )
{
    if ( mResLayout.IsValid() )
    {
        if (vw::system::App::GetInstance().IsUpdateScene())
        {
            mResLayout.Update();
        }
    }
}


/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::Calc
  Description:  Calc processing
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::Calc( void )
{
    if ( mResLayout.IsValid() )
    {
        // Frame management for emphasized display
        mFlashAppeal.Update();

        vw::system::PerformMgr::GetInstance()->StartCalcMeasure();

        this->CalcCamera(&sDrawInfo, mResLayout.GetLayout());

        mResLayout.GetLayout()->Animate(nw::lyt::ANIMATEOPT_NOANIMATEINVISIBLE);
        mResLayout.GetLayout()->CalculateMtx(sDrawInfo);

        vw::system::PerformMgr::GetInstance()->StopCalcMeasure();
    }
}


/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::Draw
  Description:  Rendering
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::Draw( void )
{
    // Returns immediately when there is a scene update skip.
    if (!vw::system::App::GetInstance().IsUpdateScene()) return;

    // Returns immediately if there is a render skip when a frame drops.
    if (!vw::system::App::GetInstance().IsRenderScene()) return;

    if ( mResLayout.IsValid() )
    {
        nngxSplitDrawCmdlist();
        int profilerLeft = vw::system::CommandListSwapper::GetInstance()->AddGpuProfilingStartPoint(true);
        vw::system::PerformMgr::GetInstance()->StartDrawMeasure();

#if !defined( NN_PCGX )
        if (mbToUpper)
        {
            vw::system::Video_SetRenderTarget(vw::system::UPPER_SCREEN);

            mCameraMgr.SetupProjection(vw::system::UPPER_SCREEN);

            this->InitGXSetting(NN_GX_DISPLAY0_WIDTH, NN_GX_DISPLAY0_HEIGHT);

            sDrawer.DrawBegin(sDrawInfo);
            sDrawer.Draw(mResLayout.GetLayout(), sDrawInfo);
            sDrawer.DrawEnd(sDrawInfo);
        }
#endif

        if (mbToUpper)
        {
            vw::system::Video_SetRenderTarget(vw::system::EXTENSION_SCREEN);

            mCameraMgr.SetupProjection(vw::system::EXTENSION_SCREEN);

            sDrawer.DrawBegin(sDrawInfo);
            sDrawer.Draw(mResLayout.GetLayout(), sDrawInfo);
            sDrawer.DrawEnd(sDrawInfo);
        }

        if (!mbToUpper)
        {
            vw::system::Video_SetRenderTarget(vw::system::LOWER_SCREEN);

            mCameraMgr.SetupProjection(vw::system::LOWER_SCREEN);

            this->InitGXSetting(NN_GX_DISPLAY1_WIDTH, NN_GX_DISPLAY1_HEIGHT);

            sDrawer.DrawBegin(sDrawInfo);
            sDrawer.Draw(mResLayout.GetLayout(), sDrawInfo);
            sDrawer.DrawEnd(sDrawInfo);
        }

        vw::system::PerformMgr::GetInstance()->StopDrawMeasure();
        nngxSplitDrawCmdlist();
        vw::system::CommandListSwapper::GetInstance()->SetGpuProfilingEndPoint(profilerLeft);
    }
}


/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::DrawInfo
  Description:  Render process for information display
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::DrawInfo( void )
{
    // Check for skip processing
    bool IsUpdateScene = vw::system::App::GetInstance().IsUpdateScene();
    bool IsRenderScene = vw::system::App::GetInstance().IsRenderScene();
    if (!IsUpdateScene || !IsRenderScene) return;

    SubModeType type = SubModeManager::GetInstance().GetCurrentModeType();

    if (type == SubMode_Main || type == SubMode_Menu || type == SubMode_Info)
    {
        if (mbToUpper)
        {
            u32 width = NN_GX_DISPLAY1_HEIGHT;
            u32 height = NN_GX_DISPLAY1_WIDTH;
            vw::system::Draw2DHelper::GetInstance().SetScreenSize(width, height);
            vw::system::FontDrawer::GetInstance().SetScreenSize(width, height);

            vw::system::Video_SetRenderTarget(vw::system::LOWER_SCREEN);
            this->DoDrawInfo();
        }
        else
        {
            u32 width = NN_GX_DISPLAY0_HEIGHT;
            u32 height = NN_GX_DISPLAY0_WIDTH;
            vw::system::Draw2DHelper::GetInstance().SetScreenSize(width, height);
            vw::system::FontDrawer::GetInstance().SetScreenSize(width, height);

            vw::system::Video_SetRenderTarget(vw::system::UPPER_SCREEN);
            this->DoDrawInfo();

#if !defined( NN_PCGX )
            vw::system::Video_SetRenderTarget(vw::system::EXTENSION_SCREEN);
            this->DoDrawInfo();
#endif

            vw::system::FontDrawer::GetInstance().SetScreenSize(
                NN_GX_DISPLAY1_HEIGHT, NN_GX_DISPLAY1_WIDTH);
        }
    }

    // Sub-mode display
    if (mbToUpper)
    {
        u32 width = NN_GX_DISPLAY1_HEIGHT;
        u32 height = NN_GX_DISPLAY1_WIDTH;
        vw::system::Draw2DHelper::GetInstance().SetScreenSize(width, height);
        vw::system::FontDrawer::GetInstance().SetScreenSize(width, height);
        vw::system::Video_SetRenderTarget(vw::system::LOWER_SCREEN);

        SubModeManager::GetInstance().Draw();
    }
    else
    {
        u32 width = NN_GX_DISPLAY0_HEIGHT;
        u32 height = NN_GX_DISPLAY0_WIDTH;
        vw::system::Draw2DHelper::GetInstance().SetScreenSize(width, height);
        vw::system::FontDrawer::GetInstance().SetScreenSize(width, height);

        vw::system::Video_SetRenderTarget(vw::system::UPPER_SCREEN);
        SubModeManager::GetInstance().Draw();

#if !defined( NN_PCGX )
        vw::system::Video_SetRenderTarget(vw::system::EXTENSION_SCREEN);
        SubModeManager::GetInstance().Draw();
#endif

        vw::system::FontDrawer::GetInstance().SetScreenSize(
            NN_GX_DISPLAY1_HEIGHT, NN_GX_DISPLAY1_WIDTH);
    }
}

void
ModuleG2dImpl::DoDrawInfo()
{
    vw::system::PerformDrawer::GetInstance()->DrawGraph();
    vw::system::PerformDrawer::GetInstance()->DrawMeter();

    vw::system::FontDrawer& font = vw::system::FontDrawer::GetInstance();
    font.BeginDraw();

    const int INFO_LEFT = 2;
    const int INFO_TOP = 20;

    font.DrawString(INFO_LEFT, INFO_TOP, "Layout/%s\n\n", SubModeManager::GetInstance().GetCurrentModeName());

    AnimManager& animManager = mResLayout.GetAnimManager();

    if (animManager.HasAnim())
    {
        mLbFrame.SetVisible( true );
        font.DrawString( "%s%4.1f/%4.1f\n", this->GetCursorMark(CURSOR_FRAME), animManager.GetCurrentFrame(), animManager.GetMaxFrame() );

        font.DrawString( "%sANIM:%s\n", this->GetCursorMark(CURSOR_ANIM), animManager.GetAnimName());

        mLbAnimIndex.SetVisible( true );
        font.DrawString( "     %2d/%2d\n", animManager.GetAnimIdx() + 1, animManager.GetAnimNum() );
    }
    else
    {
        mLbFrame.SetVisible( false );
        mLbAnimIndex.SetVisible( false );
        font.DrawString("%sANIM:N/A\n", this->GetCursorMark(CURSOR_ANIM));
    }

    if (animManager.IsTargetSelectable())
    {
        mLbAnimTargetType.SetVisible( true );
        font.DrawString("%sTARGET:%s\n", this->GetCursorMark(CURSOR_TARGETTYPE), animManager.GetTargetTypeName());

        mLbAnimTargetName.SetVisible( true );
        font.DrawString("%s%s:", this->GetCursorMark(CURSOR_TARGET), animManager.GetTargetTypeName());
        nw::ut::Color8 prevColor = font.GetColor();
        if (!animManager.IsTargetAnimate())
        {
            font.SetColor(nw::ut::Color8::GRAY);
        }
        font.DrawString("%s", animManager.GetTargetName());
        font.SetColor(prevColor);
        font.DrawString("%s\n", animManager.IsTargetGroupInTag()? " *" : "");
        font.SetCursorX(INFO_LEFT);

        mLbAnimTargetIdx.SetVisible( true );
        font.DrawString( "     %2d/%2d\n", animManager.GetTargetIdx() + 1, animManager.GetTargetNum());
    }
    else
    {
        mLbAnimTargetType.SetVisible( false );
        mLbAnimTargetName.SetVisible( false );
        mLbAnimTargetIdx.SetVisible( false );
    }

#if 0 // FIXME! NULL pane display no longer functions
    {
        bool bNullPaneDisp = false;
        #if ! defined(NW4R_FINAL)
            bNullPaneDisp = sDrawInfo.IsDebugDrawMode();
        #endif

        mLbNullPaneDisp.Printf("NULL:%s", bNullPaneDisp ? "DISP": "OFF");
    }
#endif

    font.DrawString("%sCAM :%s\n", this->GetCursorMark(CURSOR_CAMTYPE), mCameraMgr.IsPerspective() ? "PERS": "ORTHO");

#if 0 // FIXME! Need to consider specifications
    font.DrawString("%sSCRN:%s\n", this->GetCursorMark(CURSOR_SCREEN), mpCameraMgr->IsWide() ? "WIDE": "NORM");
#endif

    if ( mResLayout.IsValid() )
    {
        const nw::ut::Rect layoutRect = mResLayout.GetLayout()->GetLayoutRect();
        mLbSize.SetVisible( true );
        font.DrawString("   SIZE:%dx%d\n", s32(layoutRect.GetWidth()), s32(-layoutRect.GetHeight()));
    }
    else
    {
        mLbSize.SetVisible( false );
    }

    font.EndDraw();
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::Available
  Description:  Gets whether the layout module is valid.
                In accordance with this return value, the system controls module switching.
  Arguments:    None.
  Returns:      Returns true if valid.
                Returns false if invalid.
 *---------------------------------------------------------------------------


*/
bool
ModuleG2dImpl::Available()
{
#if defined( NN_PCGX )
    return false;
#else
    return mbAvailable;
#endif
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::IsHidden
  Description:  Gets whether the layout mode is in hidden mode.
                In accordance with this return value, the system controls module switching.
  Arguments:    None.
  Returns:      Returns true if hidden.
                Returns false if not hidden.
 *---------------------------------------------------------------------------


*/
bool
ModuleG2dImpl::IsHidden()
{
    return ! mbModeEnable;
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::Destroy
  Description:  Finalize.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::Destroy( void )
{
    ProtocolExecCTR::GetInstance().Finalize();

#if defined( NN_PCGX )
    //  Free resources here because an error results on PCs during finalization due to differences in when global variables and allocator variables are freed.
    //  
    mCursorLabelVec.Reset(NULL, 0);
    if (mpGraphicsResource == &sGraphicsResource)
    {
        sGraphicsResource.Finalize();
    }
#endif

    // Enable module in preparation for re-initialization.
    // nw::viewer::lyt::ModuleLYT::GetModuleEnable() is re-evaluated on the next call to Init.
    // 
    mbAvailable = true;
}
/*
Finalize()
{
    Delete( mpLayout );
    Delete( mpResAccessor );
    MEMFreeToExpHeap(sMemHandle, archiveBuf);
}


*/

/*---------------------------------------------------------------------------*
  Name:         LoadCameraConfig
  Description:  Camera initial setting
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::LoadCameraConfig()
{
    s32 iValue;
    f32 value;
    const char* sValue;
    f32 near;
    f32 far;
    f32 minNear;

    if ((sValue = system::GetConfigFile()->GetString("/g2d/projection")) != NULL)
    {
        if (0 == nw::ut::stricmp(sValue, "ORTHO"))
        {
            mCameraMgr.SetPerspective(false);
        }
        else if (0 == nw::ut::stricmp(sValue, "PERS"))
        {
            mCameraMgr.SetPerspective(true);
        }
        else
        {
            NW_WARNING(false, "must /g2d/projection ORTHO or PERS. - %d\n", iValue);
        }
    }

    if (system::GetConfigFile()->GetFloat("/g2d/stereo/base_plane", &value))
    {
        mCameraMgr.SetStereoBasePlane(value);
    }

    if (system::GetConfigFile()->GetFloat("/g2d/stereo/factor", &value))
    {
        if (0.0f <= value && value <= 1.0f)
        {
            mCameraMgr.SetStereoFactor(value);
        }
        else
        {
            NW_WARNING(false, "must /g2d/stereo/factor range 0 - 1. - %f\n", value);
        }
    }

    if ((sValue = system::GetConfigFile()->GetString("/g2d/stereo/real_switch")) != NULL)
    {
        if (0 == nw::ut::stricmp(sValue, "ON"))
        {
            mCameraMgr.SetRealSwitch(true);
        }
        else if (0 == nw::ut::stricmp(sValue, "OFF"))
        {
            mCameraMgr.SetRealSwitch(false);
        }
        else
        {
            NW_WARNING(false, "must /g2d/stereo/real_switch ON or OFF. - %d\n", iValue);
        }
    }

    if ((sValue = system::GetConfigFile()->GetString("/g2d/z_compare")) != NULL)
    {
        if (0 == nw::ut::stricmp(sValue, "ON"))
        {
            this->SetZCompareEnable(true);
        }
        else if (0 == nw::ut::stricmp(sValue, "OFF"))
        {
            this->SetZCompareEnable(false);
        }
        else
        {
            NW_WARNING(false, "must /g2d/z_compare ON or OFF. - %d\n", iValue);
        }
    }

    if (system::GetConfigFile()->GetFloat("/g2d/camera/fovy", &value))
    {
        if (FovyMin <= value && value <= FovyMax)
        {
            mCameraMgr.SetFovy( value );
        }
        else
        {
            NW_WARNING(false, "must /g2d/camera/fovy range %d - %d. - %f\n",
                FovyMin, FovyMax, value);
        }
    }

    far = mCameraMgr.GetFar();
    near = mCameraMgr.GetNear();
    minNear = mCameraMgr.GetMinNear();

    if (system::GetConfigFile()->GetFloat("/g2d/camera/near", &value))
    {
        near = value;
    }
    if (system::GetConfigFile()->GetFloat("/g2d/camera/far", &value))
    {
        far = value;
    }
    if (system::GetConfigFile()->GetFloat("/g2d/camera/min_near", &value))
    {
        minNear = value;
    }

    if (0 >= far)
    {
        NW_WARNING(false, "must 0 < /g2d/camera/far, far %f\n", far);
    }
    if (near >= far)
    {
        NW_WARNING(false, "must /g2d/camera/near < /g2d/camera/far, near %f, far %f\n", near, far);
    }
    if (minNear >= far)
    {
        NW_WARNING(false, "must /g2d/camera/min_near < /g2d/camera/far, min_near %f, far %f\n", minNear, far);
    }
    if (0 < far && near < far && minNear < far)
    {
        mCameraMgr.SetFar( far );
        mCameraMgr.SetNear( near );
        mCameraMgr.SetMinNear( minNear );
    }

#if 0 // FIXME! Specification consideration
    if (system::GetConfigFile()->GetFloat("/g2d/wide_location/scale", &value))
    {
        if (value > 0)
        {
            mpCameraMgr->SetWideScale( value );
        }
        else
        {
            NW_WARNING(false, "must /g2d/wide_location/scale > 0. - %f\n", value);
        }
    }

    if (system::GetConfigFile()->GetFloat("/g2d/wide_location_test/scale", &value))
    {
        if (value > 0)
        {
            mpCameraMgr->SetTestScale( value );
        }
        else
        {
            NW_WARNING(false, "must /g2d/wide_location_test/scale > 0. - %f\n", value);
        }
    }
#endif
}


/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::CalcCamera
  Description:  Sets the model view matrix and projection matrix used for displaying text.
  Arguments:    pDrawInfo
                pLayout
                videoSetting
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::CalcCamera(nw::lyt::DrawInfo* pDrawInfo, nw::lyt::Layout* pLayout)
{
    const f32 widthScale = mCameraMgr.IsWide() ? mCameraMgr.GetWideScale() : 1.f;

    // When mbWideLocation is TRUE, the horizontal direction scale for the root pane is expanded
    pLayout->GetRootPane()->SetScale(nn::math::VEC2(widthScale, 1.f));

    // Settings for WideLocation
    pDrawInfo->SetLocationAdjust( mCameraMgr.IsWide() );
    pDrawInfo->SetLocationAdjustScale( nn::math::VEC2(1.f / widthScale, 1.f) );

    // Get the layout Rect.
    // When mbWideLocation is TRUE, Rect also reflects that effect.
    nw::ut::Rect layoutRect = pLayout->GetLayoutRect();
    mCameraMgr.SetLayoutRect( layoutRect );

    nn::math::MTX34 viewMtx;

    mCameraMgr.GetViewMtx( &viewMtx );
    mCameraMgr.CalcProjMtx();

    pDrawInfo->SetViewMtx(viewMtx);
}

/*---------------------------------------------------------------------------*
  Name:         SetupCamera
  Description:  Sets the model view matrix and projection matrix used for displaying text.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::SetupCamera(nw::lyt::Layout* pLayout)
{
    NW_UNUSED_VARIABLE( pLayout );

    //mCameraMgr.SetupProjection();
}

/*---------------------------------------------------------------------------*
  Name:         InitGXSetting
  Description:  Initializes GX rendering settings.
  Returns:      None.
 *---------------------------------------------------------------------------

*/
void
ModuleG2dImpl::InitGXSetting(int width, int height)
{
    // Color buffer information
    // Switches the width and height to match the LCD direction.
    const nw::font::ColorBufferInfo colBufInfo =
    {
        width, height, PICA_DATA_DEPTH24_STENCIL8_EXT
    };

    const u32 commands[] =
    {
        // Viewport settings
        NW_FONT_CMD_SET_VIEWPORT(0, 0, colBufInfo.width, colBufInfo.height),

        // Disable scissoring
        NW_FONT_CMD_SET_DISABLE_SCISSOR(colBufInfo),

        // Disable W buffer
        // Set the depth range
        // Disable polygon offset
        NW_FONT_CMD_SET_WBUFFER_DEPTHRANGE_POLYGONOFFSET(
            0.0f,           // wScale : W buffer is disabled at 0.0
            0.0f,           // depth range near
            1.0f,           // depth range far
            0,              // polygon offset units : polygon offset is disabled with 0.0
            colBufInfo),

        // Depth test settings
        // Make all elements of the color buffer writable
        NW_FONT_CMD_SET_DEPTH_FUNC_COLOR_MASK(
            mbOrthoZCompare,  // isDepthTestEnabled
            PICA_DATA_DEPTH_TEST_LEQUAL,      // depthFunc
            true,   // depthMask
            true,   // red
            true,   // Green
            true,   // Blue
            true),  // Alpha

        // Depth test settings
        NW_FONT_CMD_SET_DEPTH_FUNC2( PICA_DATA_DEPTH_TEST_LEQUAL ),

        // Framebuffer access control
        NW_FONT_CMD_SET_FBACCESS(
            true,   // colorRead
            true,   // colorWrite
            mbOrthoZCompare,  // depthRead
            mbOrthoZCompare,  // depthWrite
            false,  // stencilRead
            false), // stencilWrite
    };

    nngxAdd3DCommand(commands, sizeof(commands), true);

    const u32 constCommands[] =
    {
        // Disable culling
        NW_FONT_CMD_SET_CULL_FACE(NW_FONT_CMD_CULL_FACE_DISABLE),

        // Disable stencil test
        NW_FONT_CMD_SET_DISABLE_STENCIL_TEST(),

        // Disable early depth test
        NW_FONT_CMD_SET_ENABLE_EARLY_DEPTH_TEST(false),
    };

    nngxAdd3DCommand(constCommands, sizeof(constCommands), true);
}

/*---------------------------------------------------------------------------*
  Name:         DoMenuProcess_
  Description:  Performs menu system processing.
  Arguments:    pad   Pad information
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::DoMenuProcess(vw::system::HidStatus& hidStatus)
{
#if 0 // FIXME!
    NW_UNUSED_VARIABLE( pad );

    system::MenuOpenFileAccess  openFileMenu( G2D_ID );

    if ( mbNeedResetFilter )
    {
        mbNeedResetFilter = false;

        openFileMenu.ClearFilter();
        openFileMenu.AddFilter("brlyt");
    }

    system::FileInfo *const fileInfo = openFileMenu.GetResultFileInfo();
    if (fileInfo)
    {
        if ( fileInfo->GetType() == system::FileInfo::TYPE_DVD
          || fileInfo->GetType() == system::FileInfo::TYPE_MCS
        )
        {
            const char sepCha = fileInfo->GetType() == system::FileInfo::TYPE_DVD ? '/': '\\';

            NW4R_LOG("filename %s\n", fileInfo->GetFileName());
            // Loads a file

            const size_t pathStrLen = std::strlen(fileInfo->GetFileName());
            char *const pathBuf = static_cast<char*>( Memory_Alloc( pathStrLen + 1 ) );
            std::strcpy(pathBuf, fileInfo->GetFileName());

            char* layoutFileNameStart = std::strrchr(pathBuf, sepCha);
            char* layoutFileNameEnd = std::strrchr(pathBuf, '.');
            if (layoutFileNameStart && layoutFileNameEnd)
            {
                *layoutFileNameStart = '\0';    // Fill \ with 0
                layoutFileNameStart++;          // Move so that the layout name is indicated

                *layoutFileNameEnd ='\0';      // Set "\0" so that the extension portion is not included

                // Search the folder above
                if (char* resDirStr = std::strrchr(pathBuf, sepCha))
                {
                    *resDirStr = '\0';
                    if (mResLayout.Load(pathBuf, layoutFileNameStart, fileInfo->GetType()))
                    {
                        // Maintain playback mode
                        if (! mbAnimStop)
                        {
                            mResLayout.Play();
                        }
                    }

                }

            }
            Memory_Free( pathBuf );
        }

        delete fileInfo;
    }
#endif
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::WriteBackConfigFile
  Description:  Writes back the layout-related settings information to the configuration file instance.

                It only writes back to the instance and does not actually output to a file.

  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::WriteBackConfigFile()
{
    system::GetConfigFile()->SetFloat("/g2d/camera/fovy", mCameraMgr.GetFovy());
    system::GetConfigFile()->SetFloat("/g2d/camera/far", mCameraMgr.GetFar());
    system::GetConfigFile()->SetFloat("/g2d/camera/near", mCameraMgr.GetNear());
    system::GetConfigFile()->SetFloat("/g2d/camera/min_near", mCameraMgr.GetMinNear());

    system::GetConfigFile()->SetString("/g2d/projection", mCameraMgr.IsPerspective()? "PERS" : "ORTHO");
    system::GetConfigFile()->SetString("/g2d/z_compare", this->IsZCompareEnable()? "ON" : "OFF");

    system::GetConfigFile()->SetFloat("/g2d/location_adjust/scale", mCameraMgr.GetWideScale());
    system::GetConfigFile()->SetFloat("/g2d/location_adjust_test/scale", mCameraMgr.GetTestScale());

    system::GetConfigFile()->SetFloat("/g2d/stereo/factor", mCameraMgr.GetStereoFactor());
    system::GetConfigFile()->SetString("/g2d/stereo/real_switch", mCameraMgr.GetRealSwitch()? "ON" : "OFF");
}



/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::OnAnmCmd
  Description:  Command for playback all is executed
  Arguments:    command     Command (RESET/PLAY/REV_PLAY/STOP/STEP/REV_STEP/LOOP/SET_FRAME)
                target      Target (ALL/3D_MDL/2D_LYT)
                targetID   Target ID (do not use when target is ANM_TGT_ALL)
                arg         Option parameter
                trigModule Module ID that is the trigger
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::OnAnmCmd( u32   command,
                     u32   target,
                     u32   targetID,
                     void* arg,
                     u32   trigModule  )
{
    NW_UNUSED_VARIABLE( targetID );

    AnimManager& animManager = mResLayout.GetAnimManager();

    if ( trigModule == G2D_ID || animManager.GetTargetIdx() != 0)
    {
        return;
    }

    switch ( command )
    {
    //-------------------------
    // Resetting Animations
    case system::ANM_CMD_RESET:
        if ( target == system::ANM_TGT_ALL )
        {
            animManager.ResetAnim();
            animManager.SetWaitFrame();

            if (system::App::GetInstance().HasFocus(G2D_ID) && !mbAnimStop)
            {
                animManager.Play();
                SysAnmCtrl_PlayAll();
            }
        }
        else if ( target == system::ANM_TGT_2D_LYT )
        {
            if ( mResLayout.GetID() == targetID )
            {
                animManager.ResetAnim();
                animManager.SetWaitFrame();

                if (system::App::GetInstance().HasFocus(G2D_ID) && !mbAnimStop)
                {
                    animManager.Play();
                    SysAnmCtrl_PlayAll();
                }
            }
        }
        break;
    //-------------------------
    // Animation playback start
    case system::ANM_CMD_PLAY:
        {
            system::AnmPlayArg *param = static_cast<system::AnmPlayArg*>( arg );
            if ( target == system::ANM_TGT_ALL )
            {
                SetAnimStop(false);
            }
            else if ( target == system::ANM_TGT_2D_LYT )
            {
                if ( mResLayout.GetID() == targetID )
                {
                    SetAnimStop(false);
                }
            }
        }
        break;
    //-------------------------
    // Animation pause
    case system::ANM_CMD_STOP:
        if ( target == system::ANM_TGT_ALL )
        {
            SetAnimStop(true);
        }
        else if ( target == system::ANM_TGT_2D_LYT )
        {
            if ( mResLayout.GetID() == targetID )
            {
                SetAnimStop(true);
            }
        }
        break;
    //-------------------------
    // Animation reverse playback start
    case system::ANM_CMD_REV_PLAY:
        {
            system::AnmPlayArg *param = static_cast<system::AnmPlayArg*>( arg );
            if ( target == system::ANM_TGT_ALL )
            {
                // GetResource3D().RevPlayAnimAll( param->speed, param->loopArg );
            }
            else if ( target == system::ANM_TGT_2D_LYT )
            {
                // GetResource3D().RevPlayAnim( targetID, param->speed, param->loopArg );
            }
        }
        break;
    //-------------------------
    // Animation frame advance
    case system::ANM_CMD_STEP:
        if ( target == system::ANM_TGT_ALL )
        {
            animManager.Step( vw::system::PtrToF32(arg) );
        }
        else if ( target == system::ANM_TGT_2D_LYT )
        {
            if ( mResLayout.GetID() == targetID )
            {
                animManager.Step( vw::system::PtrToF32(arg) );
            }
        }
        break;
    //-------------------------
    // Animation reverse frame advance
    case system::ANM_CMD_REV_STEP:
        if ( target == system::ANM_TGT_ALL )
        {
            animManager.Step( -vw::system::PtrToF32(arg) );
        }
        else
        {
            if ( mResLayout.GetID() == targetID )
            {
                animManager.Step( -vw::system::PtrToF32(arg) );
            }
        }
        break;
    //-------------------------
    // Animation loop
    case system::ANM_CMD_LOOP:
        if ( target == system::ANM_TGT_ALL )
        {
            animManager.SetFrame( 0.0f );
            animManager.SetWaitFrame();
        }
        else if ( target == system::ANM_TGT_2D_LYT )
        {
            if ( mResLayout.GetID() == targetID )
            {
                animManager.SetFrame( 0.0f );
                animManager.SetWaitFrame();
            }
        }
        break;
    //-------------------------
    // Set animation frame to specific value
    case system::ANM_CMD_SET_FRAME:
        if ( target == system::ANM_TGT_ALL )
        {
            animManager.SetFrame( vw::system::PtrToF32(arg) );
        }
        else if ( target == system::ANM_TGT_2D_LYT )
        {
            if ( mResLayout.GetID() == targetID )
            {
                animManager.SetFrame( vw::system::PtrToF32(arg) );
            }
        }
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::SetAnimStop
  Description:  Changes the animation playback state.
  Arguments:    bStop   Specifies true while stopped, false during played back.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::SetAnimStop( bool bStop )
{
    AnimManager& animManager = mResLayout.GetAnimManager();

    if (animManager.GetTargetIdx() == 0)
    {
        mbAnimStop = bStop;
    }

    if (bStop)
    {
        animManager.Stop();
    }
    else
    {
        animManager.Play();
    }
}


/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::SetCursor
  Description:  Sets the cursor position on the screen.
  Arguments:    cursor  Specifies cursor position
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::SetCursor( u32 cursor )
{
    mCursor = cursor;
#if 0 // FIXME!
    system::App::GetInstance().GetUIManager().SetFocus( G2D_ID, mCursorLabelVec[ mCursor ] );
#endif
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::ChangeCursorToPrev
  Description:  Moves the cursor position for operation to the previous one.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::ChangeCursorToPrev()
{
    for (u32 cursor = (mCursor + CURSOR_MAX - 1) % CURSOR_MAX;
         cursor != mCursor;
         cursor = (cursor + CURSOR_MAX - 1) % CURSOR_MAX)
    {
        if (mCursorLabelVec[cursor]->IsVisible())
        {
            this->SetCursor(cursor);
            break;
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:         ModuleG2dImpl::ChangeCursorToNext
  Description:  Moves the cursor position for operation to the next one.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
ModuleG2dImpl::ChangeCursorToNext()
{
    for (u32 cursor = (mCursor + 1) % CURSOR_MAX;
         cursor != mCursor;
         cursor = (cursor + 1) % CURSOR_MAX)
    {
        if (mCursorLabelVec[cursor]->IsVisible())
        {
            this->SetCursor(cursor);
            break;
        }
    }
}

const char*
ModuleG2dImpl::GetCursorMark(u32 cursor) const
{
    if (cursor == mCursor)
    {
        return "[*]";
    }
    else
    {
        return "[ ]";
    }
}

bool
ModuleG2dImpl::Load(
    const char* pathName,
    const char* layoutName,
    u32 type)
{
    mFlashAppeal.Clear();
    bool res = mResLayout.Load(pathName, layoutName, type);
    nw::viewer::lyt::IResourceListener* pResListener = nw::viewer::sys::App::GetInstance().GetModuleLYT().GetResourceListener();
    if (pResListener)
    {
        pResListener->OnLoadResourceDone(res);
    }
    return res;
}

void
ModuleG2dImpl::ToUpper()
{
    mbToUpper = true;

    nw::viewer::lyt::IResourceListener* pResListener = nw::viewer::sys::App::GetInstance().GetModuleLYT().GetResourceListener();
    if (pResListener)
    {
        pResListener->OnLoadToUpper();
    }
}

void
ModuleG2dImpl::ToLower()
{
    mbToUpper = false;

    nw::viewer::lyt::IResourceListener* pResListener = nw::viewer::sys::App::GetInstance().GetModuleLYT().GetResourceListener();
    if (pResListener)
    {
        pResListener->OnLoadToLower();
    }
}

void
ModuleG2dImpl::Reset()
{
    mFlashAppeal.Clear();
    return mResLayout.Destroy();
}

}   /* namespace g2d */
}   /* namespace vw  */
