/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_ANIM_MANAGER_H_
#define NW4C_VIEWER_G3D_ANIM_MANAGER_H_

#include <viewer/system/Hid.h>
#include "ResourceManager.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw  {
namespace g3d {

struct AnimFrameRecord {
    ResourceSet::BinaryFileType fileType;
    unsigned int hashID;
    unsigned int index;
    float frame;
    int animSetIndex;
    bool isActive;
};

#define ANIM_RECORD_NUM 5

class IAnimController;
class AnimManager
{
public:
    static AnimManager* GetInstance();
    static void Initialize();
    static void Finalize();

    // Ctor
    AnimManager();
    // Dtor
    ~AnimManager();
    // Button operation
    void HidControl(s32 padEvent);
    void UpdateAll();
    void Reset();
    void StopAll();
    void PlayAll();
    void PauseAll();
    void SpeedUpAll(f32 diff);
    void StepForwardAll();
    void StepBackwardAll();
    void OnTargetModuleChanged(u16 id);
    void OnAnmCmd(u32 command, void* arg);

    bool ActiveAnimationExists();

    void SetFrame(f32 frame);
    void SetStepFrameAll(f32 step);
    void SetDefaultRate(f32 rate){ m_DefaultRate = rate; }
    f32  GetDefaultRate() const{ return m_DefaultRate; }
    void SetLoopFlag( u32 flag ){ m_LoopFlag = (bool)flag; }
    bool GetLoopFlag(){ return m_LoopFlag; }

    bool StoreCurrentAnimFrame( ResourceSet::BinaryFileType fileType, unsigned int hashID, unsigned int index );
    bool RestoreCurrentAnimFrame( ResourceSet::BinaryFileType fileType, unsigned int hashID, unsigned int index );

private:
    // Default play rate.
    f32     m_DefaultRate;
    // Loop management flag
    bool    m_LoopFlag;
    // Animation frame evacuation data
    AnimFrameRecord m_AnimFrameRecord[ ANIM_RECORD_NUM ];
    // Static instance
    static AnimManager* m_pInstance;

    void HidControl_(s32 padEvent, IAnimController *controller);
};


}   /* namespace g3d */
}   /* namespace vw  */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif


/* NW4R_VIEWER_G3D_ANIM_MANAGER_H_ */
#endif
