/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef NW4R_VIEWER_G3D_NODE_CONSTRAINT_H_
#define NW4R_VIEWER_G3D_NODE_CONSTRAINT_H_

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw {
namespace g3d {

struct Constraint {
    nw::gfx::TransformNode* m_ConstrainedObject;
    f32 m_OffsetScale[ 3 ];
    f32 m_OffsetRotate[ 3 ];
    f32 m_OffsetTranslate[ 3 ];
    bool m_UseOffset[ 3 ];          // Scale/Totate/Translate
    bool m_InheritTransform[ 3 ];   // Scale/Totate/Translate
};

class NodeConstraint
{
public:
    NodeConstraint(nw::gfx::Model* model, nw::os::IAllocator* allocator);
    ~NodeConstraint();

    // Add objects to constrain and target bones.
    void AddConstraint(const Constraint& constraint, const char* boneName);

    // Delete the specified object constraints from all bones.
    void DeleteConstraint(nw::gfx::TransformNode* constrainedObject);

    // Delete all constraints linked to the specified bones.
    void DeleteConstraint(const char* boneName);

    // Delete all constraints.
    void ClearConstraints();

    // Get the position conversion information for the constraint target bones.
    const nw::gfx::CalculatedTransform* GetBoneTransform(nw::gfx::TransformNode* constrainedObject);

private:
    class CallbackConstraint : public nw::gfx::Skeleton::CalculateMatrixSlot
    {
    public:
        CallbackConstraint(nw::gfx::Skeleton* skeleton, int boneIdx, nw::os::IAllocator* allocator);
        virtual ~CallbackConstraint();

        // Add objects to be constrained.
        void AddConstraint(const Constraint& constraint);

        // Delete objects to be constrained.
        void DeleteConstraint(nw::gfx::TransformNode* constrainedObject);

        // Delete all objects to be constrained.
        void ClearConstraints();

        // Get the number of objects to be constrained.
        int GetConstraintCount();

        // Gets the bone index of the constraint target.
        int GetTargetBoneIndex(void);

        // The callback function.
        virtual void Invoke(nw::gfx::Skeleton* skeleton, int boneIdx);

        typedef nw::ut::MoveArray<Constraint> ConstraintArray;
        ConstraintArray m_Constraints;
        nw::gfx::Skeleton* m_Skeleton;
        int m_BoneIdx;
    };

    typedef nw::ut::MoveArray<CallbackConstraint*> ConstraintArray;
    ConstraintArray m_Constraints;
    nw::gfx::Skeleton* m_Skeleton;
    nw::os::IAllocator* m_Allocator;
};

}
}
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif

#endif
