/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_PARTICLE_SCENE_ELEM_H_
#define NW4C_VIEWER_G3D_PARTICLE_SCENE_ELEM_H_

#include "SceneElem.h"
#include "ParticleAnimController.h"
#include "../../effect/include/AnimManager.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw {
namespace g3d {

class ResourceSet;
class AnimSetLinker;

typedef SceneElem<nw::gfx::ParticleEmitter, nw::gfx::res::ResParticleEmitter> ParticleEmitterElem;
typedef SceneElem<nw::gfx::ParticleSet, nw::gfx::res::ResParticleSet> ParticleSetElem;
typedef nw::ut::LinkList<ParticleSetElem, offsetof(ParticleSetElem, m_Link)> ParticleSetList;

class ParticleModelElem : public SceneElem<nw::gfx::ParticleModel, nw::gfx::res::ResParticleModel>
{
public:
    typedef nw::ut::MoveArray<nw::gfx::ParticleEmitter*> ParticleEmitterArray;

    class Builder
    {
    public:
        Builder() : m_ResSource( NULL ) {}
        ~Builder() {}

        ParticleModelElem*
        Create(nw::os::IAllocator *pAllocator, nw::os::IAllocator *pDeviceAllocator)
        {
            void *buf = pAllocator->Alloc(sizeof(ParticleModelElem));
            if (buf == NULL) return NULL;

            ParticleModelElem* elem = new(buf) ParticleModelElem(
                m_ResSource, m_ResObject,
                pAllocator, pDeviceAllocator
            );

            return elem;
        }

        Builder&
        ResSource(ResourceSet* resSource)
        {
            m_ResSource = resSource;
            return *this;
        }

        Builder&
        ResObject(nw::gfx::res::ResParticleModel resObject)
        {
            m_ResObject = resObject;
            return *this;
        }

    private:
        ResourceSet* m_ResSource;
        nw::gfx::res::ResParticleModel m_ResObject;
    };

    virtual void Destroy()
    {
        this->~ParticleModelElem();
        this->m_pAllocator->Free( this );
    }

    void AddParticleEmitter( nw::gfx::ParticleEmitter* emitter)
    {
        m_ParticleEmitters.push_back( emitter );
        emitter->ParticleAnimFrameController().SetStepFrame( vw::effect::AnimManager::GetInstance()->GetStepFrame() );
    }

    ParticleEmitterArray::iterator GetParticleEmittersBegin(){ return m_ParticleEmitters.begin(); }
    ParticleEmitterArray::iterator GetParticleEmittersEnd(){ return m_ParticleEmitters.end(); }
    u32                            GetParticleEmittersCount(){ return m_ParticleEmitters.size(); }

    virtual AnimController *GetAnimController(){ return m_pAnimController; }
    ParticleAnimController *GetParticleAnimController(){ return m_pParticleAnimController; }

    void SetEmissionStartFrame(s32 frame)
    {
        m_pParticleAnimController->SetEmissionStartFrame(frame);
        m_pAnimController->SetStartFrame(frame);
    }
    void SetEmissionLength(s32 length){ m_pParticleAnimController->SetEmissionLength(length); }

protected:
    ParticleEmitterArray m_ParticleEmitters;
    ParticleAnimController *m_pParticleAnimController;
    AnimController *m_pAnimController;

    ParticleModelElem(
        ResourceSet* resSource,
        nw::gfx::res::ResParticleModel resObject,
        nw::os::IAllocator* pAllocator,
        nw::os::IAllocator* pDeviceAllocator
    ) : SceneElem<nw::gfx::ParticleModel, nw::gfx::res::ResParticleModel>
            ( resSource, resObject, NULL, true, pAllocator, pDeviceAllocator )
    {
        m_ParticleEmitters = ParticleEmitterArray( m_pAllocator );
        m_pParticleAnimController = ParticleAnimController::Builder()
            .ParticleElem( this ).Create( pAllocator );
        m_pAnimController = AnimController::Builder()
            .Node( this->m_pNode ).Create( pAllocator );
    }

    virtual ~ParticleModelElem()
    {
        m_pParticleAnimController->Destroy();
        m_pAnimController->Destroy();
        // For ParticleModel only, the entire scene tree must be deleted
        if (m_DestroyNode) {
            nw::gfx::SafeDestroyBranch( m_pNode );
            m_DestroyNode = false;
        }
    }
};

}
}
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif
