﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_RESOURCE_LINKER_H_
#define NW4C_VIEWER_G3D_RESOURCE_LINKER_H_

#include <nw/gfx.h>

namespace vw {
namespace g3d {

#define DECL_RES_LINK_INFO( type, target ) \
    typedef nw::ut::MoveArray<nw::gfx::res::ResReference##type> type##LinkArray; \
    private: type##LinkArray m_##type##Array; \
    public: type##LinkArray::iterator \
    Get##type##sBegin() { return m_##type##Array.begin(); } \
    public: type##LinkArray::iterator \
    Get##type##sEnd() { return m_##type##Array.end(); } \
    public: u32 \
    Get##type##sCount() { return m_##type##Array.size(); } \
    public: void \
    Link( nw::gfx::res::Res##target target ); \
    public: void \
    Unlink( nw::gfx::res::Res##target target ); \
    public: void \
    Unlink( nw::gfx::res::ResReference##type ref ); \
    private: void \
    AddReference( nw::gfx::res::ResReference##type ref );

class ResourceLinker
{
public:
    class Builder
    {
    public:
        Builder() {}
        ~Builder() {}
        ResourceLinker* Create( nw::os::IAllocator *pAllocator );
    };
    void Destroy();

    void ParseLink( nw::gfx::SceneNode* node );
    void CheckError();

    DECL_RES_LINK_INFO( LookupTable, LookupTableSet )
    DECL_RES_LINK_INFO( Texture, Texture )
    //DECL_RES_LINK_INFO( Shader, Shader )

private:
    ResourceLinker( nw::os::IAllocator* pAllocator );
    ~ResourceLinker();

    void ParseLink( nw::gfx::res::ResMaterial material );
    void ParseLink( nw::gfx::Model* model );
    void ParseLink( nw::gfx::Light* light );

    nw::os::IAllocator* m_pAllocator;
};

}
}

#endif
