/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_SCENE_SELECTOR_H_
#define NW4C_VIEWER_G3D_SCENE_SELECTOR_H_

#include "SceneElem.h"
#include "ParticleSceneElem.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw {
namespace g3d {

#define DECL_SELECT_LINK_LIST( type, name ) \
    public: typedef nw::ut::MoveArray<type*> name##List; \
    private: name##List m_##name##Array; \
    private: name##List::iterator m_target##name; \
    public: name##List::iterator \
    Get##name##sBegin() { return m_##name##Array.begin(); } \
    public: name##List::iterator \
    Get##name##sEnd() { return m_##name##Array.end(); } \
    public: u32 Get##name##sCount() { return m_##name##Array.size(); } \
    public: type* GetTarget##name(); \
    public: void SetTarget##name( type* target ); \
    public: bool IsFirst##name(); \
    public: bool IsLast##name(); \
    public: void ChangeToFirst##name(); \
    public: void ChangeToLast##name(); \
    public: void ChangeToPrev##name(); \
    public: void ChangeToNext##name(); \
    private: void Activate##name( type* target );

class SceneElemSelector
{
public:
    enum SelectElemType {
        ELEM_MODEL = 0,
        ELEM_CAMERA,
        ELEM_LIGHT,
        ELEM_FOG,
        ELEM_PARTICLEMODEL,
        ELEM_SCENE_ENVIRONMENT_SETTING,
        ELEM_NUM
    };

    struct CameraInfo {
        nw::gfx::Camera* m_camera;
        int m_cameraIndex;
    };

    // Function to get instance
    static SceneElemSelector* GetInstance();

    // Instance initialization, finalization
    static void Initialize();
    static void Finalize();

    // Update selection information
    void Setup();

    // Checks if the selected item of the operation target of the specified type is at the beginning
    bool IsFirstElem( const SelectElemType type );

    // Checks if the selected item of the operation target of the specified type is at the end
    bool IsLastElem( const SelectElemType type );

    // Restore operation target of specified type to the beginning
    void ChangeToFirstElem( const SelectElemType type );

    // Advance the operation target of specified type to the end
    void ChangeToLastElem( const SelectElemType type );

    // Restore operation target of specified type to the previous
    void ChangeToPrevElem( const SelectElemType type );

    // Advance the operation target of specified type to the next
    void ChangeToNextElem( const SelectElemType type );

    // Restore animation of specified type to the previous
    void ChangeToPrevAnim( const SelectElemType type );

    // Advance the animation of specified type to the next
    void ChangeToNextAnim( const SelectElemType type );

    // Function to get the camera index
    int GetCameraIndex( nw::gfx::Camera* camera );

    // Get operation target resource of the specified type
    nw::gfx::SceneObject* GetTargetElem( const SelectElemType type );

    // Get anim linker for the operation target resource of the specified type
    AnimSetLinker* GetTargetElemLinker( const SelectElemType type );

    // Get anim controller for the operation target resource of the specified type
    IAnimController* GetTargetElemController( const SelectElemType type );

    // Get anim controller for the operation target resource of the specified type
    const char* GetTargetElemName( const SelectElemType type );

    // Declarations for each resource array
    DECL_SELECT_LINK_LIST( ModelElem, Model )
    DECL_SELECT_LINK_LIST( CameraElem, Camera )
    DECL_SELECT_LINK_LIST( LightElem, Light )
    DECL_SELECT_LINK_LIST( FogElem, Fog )
    DECL_SELECT_LINK_LIST( SceneEnvironmentSettingElem, SceneEnvironmentSetting )
    private: nw::ut::MoveArray<int> m_CameraIndexArray;

private:
    SceneElemSelector();
    ~SceneElemSelector();

    void ApplyCurrentSceneEnvironmentSetting();

    bool m_UseDefaultLight;

    static SceneElemSelector* m_pInstance;
};

}
}
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif

#endif
