﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_USERDATA_MANAGER_H_
#define NW4C_VIEWER_G3D_USERDATA_MANAGER_H_

#include <nw/gfx.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw {
namespace g3d {

enum UserDataTargetType {
    TARGET_SCENE_OBJECT = 0,
    TARGET_BONE,
    TARGET_ANIMATION
};

struct UserDataTarget{
    void* m_target;
    UserDataTargetType m_targetType;
    ResourceSet* m_resourceSet;
    nw::ut::ResS32 m_originalDictCount;
    void* m_originalDict;
};

struct UserDataLink {
    typedef nw::ut::MoveArray<UserDataTarget> UserDataTargetList;

    UserDataTargetList m_targets;
    nw::gfx::res::ResSceneObject m_replace;
    unsigned int m_hashID;
};

class UserDataManager
{
public:
    typedef nw::ut::MoveArray<UserDataLink> UserDataLinkList;
    struct LinkData {
        void* m_target;
        UserDataTargetType m_targetType;
        ResourceSet* m_resourceSet;
    };

    static UserDataManager* GetInstance();

    static void Initialize();
    static void Finalize();

    void Link(
        nw::ut::MoveArray<LinkData>& targets,
        nw::gfx::res::ResSceneObject replace,
        unsigned int hashID
    );
    void Unlink( void* object );
    void Unlink( ResourceSet* resourceSet );

    void Clear();

private:
    UserDataManager();
    ~UserDataManager();

    template<class CheckFunction> void Unlink(CheckFunction checkFunction);

    UserDataLinkList m_userDataArray;
    nw::os::IAllocator* m_pAllocator;
    nw::os::IAllocator* m_pDeviceAllocator;

    static UserDataManager* m_pInstance;
};

}
}
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif
