/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include <viewer/g3d/CameraMgr.h>
#include "../../system/include/HidManager.h"
#include "../include/MenuSubMode.h"
#include "../include/ResourceManager.h"
#include "../include/AnimManager.h"
#include "../include/SceneManager.h"
#include "../include/SceneRenderer.h"
#include "../include/UserDataManager.h"
#include "../include/InfoDrawer.h"
#include "../include/Grid.h"
#include "../../effect/include/AnimManager.h"
#include "../../effect/include/ModuleEffectImpl.h"
#include "../../system/include/LogSystem.h"
#include "../../system/include/VersionDraw.h"

namespace vw  {
namespace g3d {

#define DECL_MENU_BASE_FUNC( type ) \
type##Menu::type##Menu() {} \
type##Menu::~type##Menu() {} \
MenuType type##Menu::GetMenuType() const { return Menu_##type; }

#define DECL_MENU_FUNC( type ) \
DECL_MENU_BASE_FUNC( type ) \
void type##Menu::Prev() {} \
void type##Menu::Next() {} \
const char* type##Menu::GetMenuName() const { return #type; }

#define DECL_RENDER_FLAG_MENU_FUNC( type ) \
DECL_MENU_BASE_FUNC( type ) \
void type##Menu::Prev() \
{ \
    bool* renderFlag = InfoDrawer::GetInstance()->GetRenderFlag(); \
    if ( renderFlag[SceneRenderer::Render##type] ) { \
        renderFlag[SceneRenderer::Render##type] = false; \
    } else { \
        renderFlag[SceneRenderer::Render##type] = true; \
    } \
} \
void type##Menu::Next() { Prev(); } \
void type##Menu::Apply() {} \
const char* type##Menu::GetMenuName() const \
{ \
    bool* renderFlag = InfoDrawer::GetInstance()->GetRenderFlag(); \
    if ( renderFlag[SceneRenderer::Render##type] ) { \
        return #type ": On"; \
    } else { \
        return #type ": Off"; \
    } \
}

#define ADD_MENU( type ) \
case Menu_##type: \
{ \
    void* buf = pAllocator->Alloc(sizeof(type##Menu)); \
    type##Menu* menu = new(buf) type##Menu(); \
    mMenu.push_back( menu ); \
} \
break;

#define DELETE_MENU( type ) \
case Menu_##type: \
{ \
    type##Menu* menu = (type##Menu*)mMenu[i]; \
    menu->~type##Menu(); \
    pAllocator->Free( menu ); \
} \
break;

#define DECL_MENU_CATEGORY_FUNC( type, retMenuName ) \
DECL_MENU_BASE_FUNC( type ) \
void type##Menu::Prev() {} \
void type##Menu::Next() {} \
void type##Menu::Apply() {} \
const char* type##Menu::GetMenuName() const { return retMenuName; }


DECL_MENU_FUNC( LoadBinary )
DECL_MENU_BASE_FUNC( FrameRate )
DECL_MENU_BASE_FUNC( DepthLevel )
DECL_MENU_BASE_FUNC( Factor )
DECL_MENU_BASE_FUNC( RealSwitch )
DECL_MENU_BASE_FUNC( GridScale )
DECL_MENU_BASE_FUNC( GridDivision )
DECL_MENU_BASE_FUNC( ScreenPreview )
DECL_RENDER_FLAG_MENU_FUNC( TargetCameraAxis )
DECL_MENU_BASE_FUNC( ParticleEmitterAxisForm )
DECL_RENDER_FLAG_MENU_FUNC( ConstraintTargetBoneAxis )
DECL_RENDER_FLAG_MENU_FUNC( BoundingBox )
DECL_MENU_BASE_FUNC( ForceUseCameraWScale )
DECL_MENU_BASE_FUNC( CameraNear )
DECL_MENU_BASE_FUNC( CameraFar )
DECL_MENU_BASE_FUNC( FrameSkip )
DECL_MENU_BASE_FUNC( FocusScale )
DECL_MENU_BASE_FUNC( SettingReload )
DECL_MENU_BASE_FUNC( GridLineWidth )
DECL_MENU_BASE_FUNC( ManipulatorScale )
DECL_MENU_BASE_FUNC( DepthLevelBias )
DECL_MENU_BASE_FUNC( DollyThreshold )
DECL_MENU_BASE_FUNC( CameraManipulatorScale )
DECL_RENDER_FLAG_MENU_FUNC( DepthLevelSurface )
DECL_MENU_BASE_FUNC( DepthLevelSurfaceAlpha )
DECL_MENU_CATEGORY_FUNC( CategorySystem, "---[System]---" )
DECL_MENU_CATEGORY_FUNC( Category3D_General, "---[3D/General]---" )
DECL_MENU_CATEGORY_FUNC( Category3D_Information, "---[3D/Information]---" )
DECL_MENU_CATEGORY_FUNC( Category3D_Camera, "---[3D/Camera]---" )
DECL_MENU_CATEGORY_FUNC( Category3D_Stereovision, "---[3D/Stereovision]---" )
DECL_MENU_CATEGORY_FUNC( CategoryEffect, "---[Effect]---" )


/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
ScreenCaptureMenu::ScreenCaptureMenu()
{
}


/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
ScreenCaptureMenu::~ScreenCaptureMenu()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuType ScreenCaptureMenu::GetMenuType() const
{
    return Menu_ScreenCapture;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ScreenCaptureMenu::Prev()
{
    f32 magnify = vw::system::GetCapture().GetMagnify();

    magnify -= 1.0f;

    if( magnify < 1.0f )
    {
        magnify = 0.5f;
    }

    vw::system::GetCapture().SetMagnify( magnify );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ScreenCaptureMenu::Next()
{
    f32 magnify = vw::system::GetCapture().GetMagnify();

    if( magnify < 1.0f)
    {
        magnify = 1.0f;
    }
    else if( magnify < 16.0f )
    {
        magnify += 1.0f;
    }

    vw::system::GetCapture().SetMagnify( magnify );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void ScreenCaptureMenu::Apply()
{
    vw::system::GetCapture().Capture();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
ScreenCaptureMenu::GetMenuName() const
{
    u32 magnify = vw::system::GetCapture().GetMagnify();

    static char str[128];
    if( magnify < 1.0f)
    {
        sprintf( str, "ScreenCapture: x 0.5" );
    }
    else
    {
        sprintf( str, "ScreenCapture: x %u", magnify );
    }

    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void LoadBinaryMenu::Apply()
{
    bool IsRunning = false;
    nn::Result ret = nn::hio::GetHioDaemonStatus( &IsRunning );
    if(ret.IsSuccess() && IsRunning == false)
    {
        return;
    }

    vw::effect::ModuleEffectImpl::GetInstance()->ResetMaxParticle();

    vw::system::SystemSetting setting;
    vw::system::System_GetSetting(&setting);
    vw::common::CharArray filepath( vw::system::Memory_GetSysAllocator() );
    const char* filename = NULL;

    // Get the reload path
    SceneRenderer::GetInstance()->ResetBlinkAnimation();
    if ( *( setting.reloadTargetPath.Get() ) == '\0' ) {
        char path[1024];
        nn::hio::CTR::GetEnvironmentVariable(path, 1024 , "NW4C_ROOT");
        filepath  = path;
        filepath += "\\temp\\model.bcmdl";
        filename = filepath.Get();
    } else {
        filename = setting.reloadTargetPath.Get();
    }

    // Execute reload
    ResourceManager::GetInstance()->LoadBinaryData(
        ResourceSet::Type_Packed,
        0,
        0,
        filename,
        false
    );

    vw::effect::AnimManager::GetInstance()
        ->OnTargetModuleChanged( vw::system::App::GetInstance().GetFocusID() );
    vw::g3d::AnimManager::GetInstance()
        ->OnTargetModuleChanged( vw::system::App::GetInstance().GetFocusID() );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SortMenu::SortMenu()
: mSortMode(nw::gfx::ISceneUpdater::ALL_MESH_BASE_SORT)
{}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SortMenu::~SortMenu()
{}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SortMenu::Prev()
{
    if ( mSortMode == nw::gfx::ISceneUpdater::ALL_MESH_BASE_SORT) {
        mSortMode = nw::gfx::ISceneUpdater::OPAQUE_MESH_BASE_AND_TRANSLUCENT_MODEL_BASE_SORT;
    } else {
        mSortMode = nw::gfx::ISceneUpdater::ALL_MESH_BASE_SORT;
    }
    SceneRenderer::GetInstance()->SetRenderSortMode( mSortMode );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SortMenu::Next()
{
    Prev();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SortMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
SortMenu::GetMenuName() const
{
    switch( mSortMode )
    {
    case nw::gfx::ISceneUpdater::ALL_MESH_BASE_SORT:
        return "SortMode: All Mesh Base";
    case nw::gfx::ISceneUpdater::OPAQUE_MESH_BASE_AND_TRANSLUCENT_MODEL_BASE_SORT:
        return "SortMode: Opaque Mesh + Translucent Model Base";
    }
    return 0;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuType
SortMenu::GetMenuType() const
{
    return Menu_Sort;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FrameRateMenu::Prev()
{
    if ( system::System_GetSetting()->updateRate < 12 )
    {
        system::SystemSetting setting;
        system::System_GetSetting( &setting );

        // Animation frame rate
        setting.updateRate++;
        setting.frameRate =  static_cast<u32>( 60 / setting.updateRate );

        // Set measurement count
        system::PerformMgr::GetInstance()->SetMeasureCount(setting.frameRate);

        system::System_ChangeSetting( &setting );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FrameRateMenu::Next()
{
    if ( system::System_GetSetting()->updateRate > 1 )
    {
        system::SystemSetting setting;
        system::System_GetSetting( &setting );

        // Animation frame rate
        setting.updateRate--;
        setting.frameRate =  static_cast<u32>( 60 / setting.updateRate );

        // Set measurement count
        system::PerformMgr::GetInstance()->SetMeasureCount(setting.frameRate);

        system::System_ChangeSetting( &setting );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FrameRateMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
FrameRateMenu::GetMenuName() const
{
    static char str[32];
    sprintf( str, "FrameRate: %.1f", 60.0f / (float)vw::system::System_GetSetting()->updateRate );
    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelMenu::Prev()
{
    DirectionalObj::GetDirectionalInfo().depthLevel -= 0.1f;
    if ( abs( DirectionalObj::GetDirectionalInfo().depthLevel ) < 0.02f ) {
        DirectionalObj::GetDirectionalInfo().depthLevel = 0.0f;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelMenu::Next()
{
    DirectionalObj::GetDirectionalInfo().depthLevel += 0.1f;
    if ( abs( DirectionalObj::GetDirectionalInfo().depthLevel ) < 0.02f ) {
        DirectionalObj::GetDirectionalInfo().depthLevel = 0.0f;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
DepthLevelMenu::GetMenuName() const
{
    static char str[32];
    sprintf( str, "DepthLevel: %.1f", DirectionalObj::GetDirectionalInfo().depthLevel );
    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FactorMenu::Prev()
{
    DirectionalObj::GetDirectionalInfo().factor -= 0.01f;
    if ( DirectionalObj::GetDirectionalInfo().factor < 0.01f ) {
        DirectionalObj::GetDirectionalInfo().factor = 0.01f;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FactorMenu::Next()
{
    DirectionalObj::GetDirectionalInfo().factor += 0.01f;
    if ( DirectionalObj::GetDirectionalInfo().factor > 1.0f ) {
        DirectionalObj::GetDirectionalInfo().factor = 1.0f;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FactorMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
FactorMenu::GetMenuName() const
{
    static char str[32];
    sprintf( str, "Factor: %.2f", DirectionalObj::GetDirectionalInfo().factor );
    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
RealSwitchMenu::Prev()
{
    if ( DirectionalObj::GetDirectionalInfo().realSwitch ) {
        DirectionalObj::GetDirectionalInfo().realSwitch = false;
    } else {
        DirectionalObj::GetDirectionalInfo().realSwitch = true;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
RealSwitchMenu::Next()
{
    Prev();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
RealSwitchMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
RealSwitchMenu::GetMenuName() const
{
    if ( DirectionalObj::GetDirectionalInfo().realSwitch ) {
        return "RealSwitch: On";
    } else {
        return "RealSwitch: Off";
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridScaleMenu::Prev()
{
    f32 scale = SceneRenderer::GetInstance()->GetGrid()->GetScale();
    scale -= 0.5f;
    if ( scale < 0.001f ) scale = 0.001f;

    if ( scale != SceneRenderer::GetInstance()->GetGrid()->GetScale() ) {
        SceneRenderer::GetInstance()->GetGrid()->CreateBuffer(
            scale,
            SceneRenderer::GetInstance()->GetGrid()->GetDivision()
        );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridScaleMenu::Next()
{
    f32 scale = SceneRenderer::GetInstance()->GetGrid()->GetScale();
    scale += 0.5f;
    if ( scale > 1000.0f ) scale = 1000.0f;

    if ( scale != SceneRenderer::GetInstance()->GetGrid()->GetScale() ) {
        SceneRenderer::GetInstance()->GetGrid()->CreateBuffer(
            scale,
            SceneRenderer::GetInstance()->GetGrid()->GetDivision()
        );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridScaleMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
GridScaleMenu::GetMenuName() const
{
    static char str[32];
    sprintf( str, "GridScale: %.1f", SceneRenderer::GetInstance()->GetGrid()->GetScale() );
    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridDivisionMenu::Prev()
{
    u32 division = SceneRenderer::GetInstance()->GetGrid()->GetDivision();
    if ( division > 1 ) {
        --division;
        SceneRenderer::GetInstance()->GetGrid()->CreateBuffer(
            SceneRenderer::GetInstance()->GetGrid()->GetScale(),
            division
        );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridDivisionMenu::Next()
{
    u32 division = SceneRenderer::GetInstance()->GetGrid()->GetDivision();
    if ( division < 20 ) {
        ++division;
        SceneRenderer::GetInstance()->GetGrid()->CreateBuffer(
            SceneRenderer::GetInstance()->GetGrid()->GetScale(),
            division
        );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridDivisionMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
GridDivisionMenu::GetMenuName() const
{
    static char str[32];
    sprintf( str, "GridDivision: %u", SceneRenderer::GetInstance()->GetGrid()->GetDivision() );
    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ScreenPreviewMenu::Prev()
{
    vw::system::SystemSetting setting;
    vw::system::System_GetSetting( &setting );
    setting.lowerPreview = !setting.lowerPreview;
    vw::system::System_ChangeSetting( &setting );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ScreenPreviewMenu::Next()
{
    Prev();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ScreenPreviewMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
ScreenPreviewMenu::GetMenuName() const
{
    bool lowerPreview = vw::system::System_GetSetting()->lowerPreview;
    if ( lowerPreview ) {
        return "LowerScreenPreview: On";
    } else {
        return "LowerScreenPreview: Off";
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ParticleEmitterAxisFormMenu::Prev()
{
    bool* renderFlag = InfoDrawer::GetInstance()->GetRenderFlag();
    if ( renderFlag[SceneRenderer::RenderParticleEmitterAxis] ) {
        renderFlag[SceneRenderer::RenderParticleEmitterAxis] = false;
        renderFlag[SceneRenderer::RenderParticleEmitterForm] = false;
    } else {
        renderFlag[SceneRenderer::RenderParticleEmitterAxis] = true;
        renderFlag[SceneRenderer::RenderParticleEmitterForm] = true;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ParticleEmitterAxisFormMenu::Next()
{
    Prev();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ParticleEmitterAxisFormMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
ParticleEmitterAxisFormMenu::GetMenuName() const
{
    bool* renderFlag = InfoDrawer::GetInstance()->GetRenderFlag();
    if ( renderFlag[SceneRenderer::RenderParticleEmitterAxis] ) {
        return "ParticleEmitterAxis&Form: On";
    } else {
        return "ParticleEmitterAxis&Form: Off";
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ForceUseCameraWScaleMenu::Prev()
{
    if ( SceneManager::GetInstance()->GetForceUseCameraWScale() ) {
        SceneManager::GetInstance()->SetForceUseCameraWScale( false );
    } else {
        SceneManager::GetInstance()->SetForceUseCameraWScale( true );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ForceUseCameraWScaleMenu::Next()
{
    Prev();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ForceUseCameraWScaleMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
ForceUseCameraWScaleMenu::GetMenuName() const
{
    if ( SceneManager::GetInstance()->GetForceUseCameraWScale() ) {
        return "ForceUseCameraWScale: On";
    } else {
        return "ForceUseCameraWScale: Off";
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraNearMenu::Prev()
{
    vw::g3d::CameraElem* g3d_camera = vw::g3d::SceneManager::GetInstance()->GetDefaultCamera();
    nw::gfx::ResCameraProjectionUpdater resProjUpdater = g3d_camera->GetSceneNode()->GetProjectionUpdater()->GetResource();

    f32 near = resProjUpdater.GetNear();
    near -= near * 0.01f;

    const f32 MIN_NEAR = 0.01f;
    if( near < MIN_NEAR)
        near = MIN_NEAR;

    resProjUpdater.SetNear(near);
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraNearMenu::Next()
{
    vw::g3d::CameraElem* g3d_camera = vw::g3d::SceneManager::GetInstance()->GetDefaultCamera();
    nw::gfx::ResCameraProjectionUpdater resProjUpdater = g3d_camera->GetSceneNode()->GetProjectionUpdater()->GetResource();

    f32 near = resProjUpdater.GetNear();
    near += near * 0.01f;

    const f32 MAX_NEAR = resProjUpdater.GetFar() - 0.01f;
    if( near > MAX_NEAR)
        near = MAX_NEAR;

    resProjUpdater.SetNear(near);
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraNearMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
CameraNearMenu::GetMenuName() const
{
    vw::g3d::CameraElem* g3d_camera = vw::g3d::SceneManager::GetInstance()->GetDefaultCamera();
    nw::gfx::ResCameraProjectionUpdater resProjUpdater = g3d_camera->GetSceneNode()->GetProjectionUpdater()->GetResource();
    f32 nearValue = resProjUpdater.GetNear();

    static char str[64];
    sprintf( str, "Default Camera Near: %f", nearValue );
    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraFarMenu::Prev()
{
    vw::g3d::CameraElem* g3d_camera = vw::g3d::SceneManager::GetInstance()->GetDefaultCamera();
    nw::gfx::ResCameraProjectionUpdater resProjUpdater = g3d_camera->GetSceneNode()->GetProjectionUpdater()->GetResource();

    f32 far = resProjUpdater.GetFar();
    far -= far * 0.01f;

    const f32 MIN_FAR = resProjUpdater.GetNear() + 0.01f;
    if( far < MIN_FAR)
        far = MIN_FAR;

    resProjUpdater.SetFar(far);

    if ( SceneManager::GetInstance()->GetForceUseCameraWScale() )
    {
        SceneManager::GetInstance()->SetCameraWScale( g3d_camera->GetSceneNode() );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraFarMenu::Next()
{
    vw::g3d::CameraElem* g3d_camera = vw::g3d::SceneManager::GetInstance()->GetDefaultCamera();
    nw::gfx::ResCameraProjectionUpdater resProjUpdater = g3d_camera->GetSceneNode()->GetProjectionUpdater()->GetResource();

    f32 far = resProjUpdater.GetFar();
    far += far * 0.01f;

    const f32 MAX_FAR = 100000.0f;
    if( far > MAX_FAR)
        far = MAX_FAR;

    resProjUpdater.SetFar(far);

    if ( SceneManager::GetInstance()->GetForceUseCameraWScale() )
    {
        SceneManager::GetInstance()->SetCameraWScale( g3d_camera->GetSceneNode() );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraFarMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
CameraFarMenu::GetMenuName() const
{
    vw::g3d::CameraElem* g3d_camera = vw::g3d::SceneManager::GetInstance()->GetDefaultCamera();
    nw::gfx::ResCameraProjectionUpdater resProjUpdater  = g3d_camera->GetSceneNode()->GetProjectionUpdater()->GetResource();
    f32 farValue = resProjUpdater.GetFar();

    static char str[64];
    sprintf( str, "Default Camera Far: %f", farValue );
    return str;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FrameSkipMenu::Prev()
{
    vw::system::SystemSetting setting;
    vw::system::System_GetSetting( &setting );

    setting.frameSkip = !setting.frameSkip;

    vw::system::System_ChangeSetting( &setting );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FrameSkipMenu::Next()
{
    FrameSkipMenu::Prev();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FrameSkipMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
FrameSkipMenu::GetMenuName() const
{
    vw::system::SystemSetting setting;
    vw::system::System_GetSetting( &setting );

    if ( setting.frameSkip ) {
        return "FrameSkip: On";
    } else {
        return "FrameSkip: Off";
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FocusScaleMenu::Prev()
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    info.FocusScale -= 0.01f;

    const f32 MIN_FOCUS_SCALE = 0.01f;
    if( info.FocusScale < MIN_FOCUS_SCALE)
        info.FocusScale = MIN_FOCUS_SCALE;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FocusScaleMenu::Next()
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    info.FocusScale += 0.01f;

    const f32 MAX_FOCUS_SCALE = 10.0f;
    if( info.FocusScale > MAX_FOCUS_SCALE)
        info.FocusScale = MAX_FOCUS_SCALE;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
FocusScaleMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
FocusScaleMenu::GetMenuName() const
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    static char str[32];
    sprintf( str, "Focus Scale: %.2f", info.FocusScale );
    return str;
}




/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SettingReloadMenu::Prev()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SettingReloadMenu::Next()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SettingReloadMenu::Apply()
{
    vw::system::ConfigFile* config = vw::system::ConfigFile::GetInstance();

    const char* pSettingPath = config->GetConfigFilePath();
    config->Reload( pSettingPath );

    const char* default_mode = vw::system::GetConfigFile()->GetString("/system/config/default_mode");
    if ( default_mode )
    {
        if(0 == strcmp(default_mode, "3D"))
        {
            vw::system::App::GetInstance().SetFocusID( G3D_ID );
        }
        else if(0 == strcmp(default_mode, "EFF"))
        {
            vw::system::App::GetInstance().SetFocusID( EFFECT_ID );
        }
        else if(0 == strcmp(default_mode, "LYT"))
        {
            vw::system::App::GetInstance().SetFocusID( G2D_ID );
        }
        else if(0 == strcmp(default_mode, "SND"))
        {
            vw::system::App::GetInstance().SetFocusID( SOUND_ID );
        }
#if 0
        else if(0 == strcmp(default_mode, "TMP"))
        {
            vw::system::App::GetInstance().SetFocusID( TEMPLATE_ID );
        }
#endif
        else
        {
            vw::system::App::GetInstance().SetFocusID( G3D_ID );
        }
    }

    s32 iValue = 0;
    u32 flag = 0;
    if ( vw::system::GetConfigFile()->GetInt( "/g3d/information/grid_visible", &iValue ) )
    {
        flag = ( iValue == 0 ) ? 1 : 0;
    }
    InfoDrawer::GetInstance()->SetRenderInfoFlag( flag );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
SettingReloadMenu::GetMenuName() const
{
    return "Setting Reload";
}




/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridLineWidthMenu::Prev()
{
    f32 lineWidth = SceneRenderer::GetInstance()->GetGrid()->GetLineWidth();

    lineWidth -= 0.01f;
    const f32 MIN_LINE_WIDTH = 0.1f;
    if ( lineWidth < MIN_LINE_WIDTH ) { lineWidth = MIN_LINE_WIDTH; }
    if ( lineWidth != SceneRenderer::GetInstance()->GetGrid()->GetLineWidth() )
    {
        SceneRenderer::GetInstance()->GetGrid()->SetLineWidth( lineWidth );
        SceneRenderer::GetInstance()->GetGrid()->CreateBuffer(
            SceneRenderer::GetInstance()->GetGrid()->GetScale(),
            SceneRenderer::GetInstance()->GetGrid()->GetDivision()
        );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridLineWidthMenu::Next()
{
    f32 lineWidth = SceneRenderer::GetInstance()->GetGrid()->GetLineWidth();

    lineWidth += 0.01f;
    const f32 MAX_LINE_WIDTH = 2.0f;
    if ( lineWidth > MAX_LINE_WIDTH ) { lineWidth = MAX_LINE_WIDTH; }
    if ( lineWidth != SceneRenderer::GetInstance()->GetGrid()->GetLineWidth() )
    {
        SceneRenderer::GetInstance()->GetGrid()->SetLineWidth( lineWidth );
        SceneRenderer::GetInstance()->GetGrid()->CreateBuffer(
            SceneRenderer::GetInstance()->GetGrid()->GetScale(),
            SceneRenderer::GetInstance()->GetGrid()->GetDivision()
        );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
GridLineWidthMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
GridLineWidthMenu::GetMenuName() const
{
    f32 lineWidth = SceneRenderer::GetInstance()->GetGrid()->GetLineWidth();

    static char str[32];
    sprintf( str, "Grid Line Width: %.2f", lineWidth );
    return str;
}




/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ManipulatorScaleMenu::Prev()
{
    f32 maniScale = SceneRenderer::GetInstance()->GetManipulatorScale();

    maniScale -= 0.05f;
    const f32 MIN_MANIPULATOR_SCALE = 0.05f;
    if ( maniScale < MIN_MANIPULATOR_SCALE ) { maniScale = MIN_MANIPULATOR_SCALE; }
    SceneRenderer::GetInstance()->SetManipulatorScale( maniScale );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ManipulatorScaleMenu::Next()
{
    f32 maniScale = SceneRenderer::GetInstance()->GetManipulatorScale();

    maniScale += 0.05f;
    const f32 MAX_MANIPULATOR_SCALE = 10.0f;
    if ( maniScale > MAX_MANIPULATOR_SCALE ) { maniScale = MAX_MANIPULATOR_SCALE; }
    SceneRenderer::GetInstance()->SetManipulatorScale( maniScale );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ManipulatorScaleMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
ManipulatorScaleMenu::GetMenuName() const
{
    f32 maniScale = SceneRenderer::GetInstance()->GetManipulatorScale();

    static char str[32];
    sprintf( str, "Manipulator Scale: %.2f", maniScale );
    return str;
}




/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelBiasMenu::Prev()
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    info.depthLevelBias -= 0.01f;
    const f32 MIN_DEPTH_LEVEL_BIAS = -10.0f;
    if ( info.depthLevelBias < MIN_DEPTH_LEVEL_BIAS ) { info.depthLevelBias = MIN_DEPTH_LEVEL_BIAS; }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelBiasMenu::Next()
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    info.depthLevelBias += 0.01f;
    const f32 MAX_DEPTH_LEVEL_BIAS = 10.0f;
    if ( info.depthLevelBias > MAX_DEPTH_LEVEL_BIAS ) { info.depthLevelBias = MAX_DEPTH_LEVEL_BIAS; }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelBiasMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
DepthLevelBiasMenu::GetMenuName() const
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    f32 depthLevelBias = info.depthLevelBias;

    static char str[32];
    sprintf( str, "Depth Level Bias: %.2f", depthLevelBias );
    return str;
}




/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DollyThresholdMenu::Prev()
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    info.DollyThreshold -= 0.01f;
    const f32 MIN_DOLLY_THRESHOLD = 0.01f;
    if ( info.DollyThreshold < MIN_DOLLY_THRESHOLD ) { info.DollyThreshold = MIN_DOLLY_THRESHOLD; }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DollyThresholdMenu::Next()
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

   info.DollyThreshold += 0.01f;
    const f32 MAX_DOLLY_THRESHOLD = 2.00f;
    if ( info.DollyThreshold > MAX_DOLLY_THRESHOLD ) { info.DollyThreshold = MAX_DOLLY_THRESHOLD; }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DollyThresholdMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
DollyThresholdMenu::GetMenuName() const
{
    vw::g3d::DirectionalInfo& info = vw::g3d::DirectionalObj::GetDirectionalInfo();

    f32 dollyThreshold = info.DollyThreshold;

    static char str[32];
    sprintf( str, "Dolly Threshold: %.2f", dollyThreshold );
    return str;
}




/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraManipulatorScaleMenu::Prev()
{
    f32 maniScale = SceneRenderer::GetInstance()->GetCameraManipulatorScale();

    maniScale -= 0.05f;
    const f32 MIN_MANIPULATOR_SCALE = 0.05f;
    if ( maniScale < MIN_MANIPULATOR_SCALE ) { maniScale = MIN_MANIPULATOR_SCALE; }
    SceneRenderer::GetInstance()->SetCameraManipulatorScale( maniScale );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraManipulatorScaleMenu::Next()
{
    f32 maniScale = SceneRenderer::GetInstance()->GetCameraManipulatorScale();

    maniScale += 0.05f;
    const f32 MAX_MANIPULATOR_SCALE = 10.0f;
    if ( maniScale > MAX_MANIPULATOR_SCALE ) { maniScale = MAX_MANIPULATOR_SCALE; }
    SceneRenderer::GetInstance()->SetCameraManipulatorScale( maniScale );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
CameraManipulatorScaleMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
CameraManipulatorScaleMenu::GetMenuName() const
{
    f32 maniScale = SceneRenderer::GetInstance()->GetCameraManipulatorScale();

    static char str[32];
    sprintf( str, "Camera Manipulator Scale: %.2f", maniScale );
    return str;
}



/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelSurfaceAlphaMenu::Prev()
{
    f32 alpha = SceneRenderer::GetInstance()->GetDepthLevelSurfaceAlpha();

    alpha -= 0.01f;
    if ( alpha < SceneRenderer::GetDepthLevelSurfaceMinAlpha() ) { alpha = SceneRenderer::GetDepthLevelSurfaceMinAlpha(); }
    SceneRenderer::GetInstance()->SetDepthLevelSurfaceAlpha( alpha );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelSurfaceAlphaMenu::Next()
{
    f32 alpha = SceneRenderer::GetInstance()->GetDepthLevelSurfaceAlpha();

    alpha += 0.01f;
    if ( alpha > SceneRenderer::GetDepthLevelSurfaceMaxAlpha() ) { alpha = SceneRenderer::GetDepthLevelSurfaceMaxAlpha(); }
    SceneRenderer::GetInstance()->SetDepthLevelSurfaceAlpha( alpha );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
DepthLevelSurfaceAlphaMenu::Apply()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
DepthLevelSurfaceAlphaMenu::GetMenuName() const
{
    f32 alpha = SceneRenderer::GetInstance()->GetDepthLevelSurfaceAlpha();

    static char str[32];
    sprintf( str, "DepthLevel Surface Alpha: %.2f", alpha );
    return str;
}



/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuSubModeBase::MenuSubModeBase() : mCurrentMenuIndex( 0 )
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuSubModeBase::~MenuSubModeBase()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
MenuSubModeBase::Main(vw::system::HidStatus& hidStatus)
{
    // Get current pad status
    switch( system::GetPadEvent( hidStatus, system::GetMenuSubModePadBindings() ) )
    {
    case system::MNU_MENU_APPLY:
        mMenu[mCurrentMenuIndex]->Apply();
        break;
    case system::MNU_SWITCH_PREV_MENU:
        mCurrentMenuIndex--;
        if( mCurrentMenuIndex < 0 ) mCurrentMenuIndex = mMenu.size() - 1;
        break;
    case system::MNU_SWITCH_NEXT_MENU:
        mCurrentMenuIndex++;
        if( mCurrentMenuIndex > mMenu.size() - 1 ) mCurrentMenuIndex = 0;
        break;
    case system::MNU_SWITCH_PREV_MENU_ITEM:
        mMenu[mCurrentMenuIndex]->Prev();
        break;
    case system::MNU_SWITCH_NEXT_MENU_ITEM:
        mMenu[mCurrentMenuIndex]->Next();
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
MenuSubModeBase::Draw()
{
    const char* name = mMenu[mCurrentMenuIndex]->GetMenuName();
    const char* thirdPrevName  = mCurrentMenuIndex > 2                ? mMenu[mCurrentMenuIndex - 3]->GetMenuName() : "";
    const char* secondPrevName = mCurrentMenuIndex > 1                ? mMenu[mCurrentMenuIndex - 2]->GetMenuName() : "";
    const char* prevName       = mCurrentMenuIndex > 0                ? mMenu[mCurrentMenuIndex - 1]->GetMenuName() : "";
    const char* nextName       = mCurrentMenuIndex < mMenu.size() - 1 ? mMenu[mCurrentMenuIndex + 1]->GetMenuName() : "";
    const char* secondNextName = mCurrentMenuIndex < mMenu.size() - 2 ? mMenu[mCurrentMenuIndex + 2]->GetMenuName() : "";
    const char* thirdNextName  = mCurrentMenuIndex < mMenu.size() - 3 ? mMenu[mCurrentMenuIndex + 3]->GetMenuName() : "";
    if( name == 0 )           name           = "N/A";
    if( thirdPrevName  == 0 ) thirdPrevName  = "N/A";
    if( secondPrevName == 0 ) secondPrevName = "N/A";
    if( prevName == 0 )       prevName       = "N/A";
    if( nextName == 0 )       nextName       = "N/A";
    if( secondNextName == 0 ) secondNextName = "N/A";
    if( thirdNextName == 0 )  thirdNextName  = "N/A";

    vw::system::Draw2DHelper& draw2D = vw::system::Draw2DHelper::GetInstance();
    draw2D.BeginDraw();
    s32 aw = 5;
    s32 ah = 58;
    s32 w = 320-aw*2;
    s32 h = 240-ah*2;
    draw2D.FillRectangle(aw,ah,w,h, nw::ut::Color8(64,64,96,255));
    draw2D.EndDraw();

    // draw sort mode
    vw::system::FontDrawer& font = vw::system::FontDrawer::GetInstance();
    font.BeginDraw();
    font.DrawString(aw + 8, 110, name);
    font.SetColor(nw::ut::Color8::GRAY);
    font.DrawString(aw + 8, 62,  thirdPrevName);
    font.DrawString(aw + 8, 78,  secondPrevName);
    font.DrawString(aw + 8, 94,  prevName);
    font.DrawString(aw + 8, 126, nextName);
    font.DrawString(aw + 8, 142, secondNextName);
    font.DrawString(aw + 8, 158, thirdNextName);
    font.SetColor(nw::ut::Color8::WHITE);
    font.EndDraw();

    // draw version
    vw::system::Version_Draw();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
MenuSubModeBase::GetModeName() const
{
    return "Menu";
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeType
MenuSubModeBase::GetModeType() const
{
    return SubMode_Menu;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuSubMode::MenuSubMode() : MenuSubModeBase()
{
     nw::os::IAllocator* pAllocator = Memory_GetAllocator();

    const MenuType menus[] = {
        // [Video settings]
        /* NOTHING */

        // [System Basics]
        Menu_CategorySystem,
        Menu_LoadBinary,
        Menu_SettingReload,
        Menu_ScreenCapture,
        Menu_FrameRate,
        Menu_FrameSkip,
        Menu_ScreenPreview,

        // [3D Settings/General]
        Menu_Category3D_General,
        Menu_Sort,

        // [3D Settings/Various Information]
        Menu_Category3D_Information,
        Menu_GridScale,
        Menu_GridDivision,
        Menu_GridLineWidth,
        Menu_ManipulatorScale,
        Menu_CameraManipulatorScale,
        Menu_BoundingBox,
        Menu_TargetCameraAxis,

        // [3D Settings/Camera]
        Menu_Category3D_Camera,
        Menu_DollyThreshold,
        Menu_CameraNear,
        Menu_CameraFar,
        Menu_FocusScale,
        Menu_ForceUseCameraWScale,

        // [3D Settings/Lights]
        /* NOTHING */

        // [3D Settings/Stereoscopic view]
        Menu_Category3D_Stereovision,
        Menu_DepthLevel,
        Menu_DepthLevelBias,
        Menu_Factor,
        Menu_RealSwitch,
        Menu_DepthLevelSurface,
        Menu_DepthLevelSurfaceAlpha,

        // [Effect settings]
        Menu_CategoryEffect,
        Menu_ParticleEmitterAxisForm,
        Menu_ConstraintTargetBoneAxis

        // [2D setting]
        /* NOTHING */

        // [2D Settings/Camera Settings]
        /* NOTHING */

        // [2D Settings/Stereoscopic view]
        /* NOTHING */
    };
    const int menuNum = sizeof( menus ) / sizeof( MenuType );

    mMenu = nw::ut::MoveArray<IMenu*>( menuNum, pAllocator );
    for ( int i = 0; i < menuNum; ++i )
    {
        switch ( menus[i] )
        {
        ADD_MENU( ScreenCapture )
        ADD_MENU( LoadBinary )
        ADD_MENU( Sort )
        ADD_MENU( FrameRate )
        ADD_MENU( DepthLevel )
        ADD_MENU( Factor )
        ADD_MENU( RealSwitch )
        ADD_MENU( GridScale )
        ADD_MENU( GridDivision )
        ADD_MENU( ScreenPreview )
        ADD_MENU( TargetCameraAxis )
        ADD_MENU( ParticleEmitterAxisForm )
        ADD_MENU( ConstraintTargetBoneAxis )
        ADD_MENU( BoundingBox )
        ADD_MENU( ForceUseCameraWScale )
        ADD_MENU( CameraNear )
        ADD_MENU( CameraFar )
        ADD_MENU( FrameSkip )
        ADD_MENU( FocusScale )
        ADD_MENU( SettingReload )
        ADD_MENU( GridLineWidth )
        ADD_MENU( ManipulatorScale )
        ADD_MENU( DepthLevelBias )
        ADD_MENU( DollyThreshold )
        ADD_MENU( CameraManipulatorScale )
        ADD_MENU( DepthLevelSurface )
        ADD_MENU( DepthLevelSurfaceAlpha )
        ADD_MENU( CategorySystem )
        ADD_MENU( Category3D_General )
        ADD_MENU( Category3D_Information )
        ADD_MENU( Category3D_Camera )
        ADD_MENU( Category3D_Stereovision )
        ADD_MENU( CategoryEffect )
        default:
            break;
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
MenuSubMode::~MenuSubMode()
{
    nw::os::IAllocator* pAllocator = Memory_GetAllocator();

    for ( int i = 0; i < mMenu.size(); ++i )
    {
        switch ( mMenu[i]->GetMenuType() )
        {
        DELETE_MENU( ScreenCapture )
        DELETE_MENU( LoadBinary )
        DELETE_MENU( Sort )
        DELETE_MENU( FrameRate )
        DELETE_MENU( DepthLevel )
        DELETE_MENU( Factor )
        DELETE_MENU( RealSwitch )
        DELETE_MENU( GridScale )
        DELETE_MENU( GridDivision )
        DELETE_MENU( ScreenPreview )
        DELETE_MENU( TargetCameraAxis )
        DELETE_MENU( ParticleEmitterAxisForm )
        DELETE_MENU( ConstraintTargetBoneAxis )
        DELETE_MENU( BoundingBox )
        DELETE_MENU( ForceUseCameraWScale )
        DELETE_MENU( CameraNear )
        DELETE_MENU( CameraFar )
        DELETE_MENU( FrameSkip )
        DELETE_MENU( FocusScale )
        DELETE_MENU( SettingReload )
        DELETE_MENU( GridLineWidth )
        DELETE_MENU( ManipulatorScale )
        DELETE_MENU( DepthLevelBias )
        DELETE_MENU( DollyThreshold )
        DELETE_MENU( CameraManipulatorScale )
        DELETE_MENU( DepthLevelSurface )
        DELETE_MENU( DepthLevelSurfaceAlpha )
        DELETE_MENU( CategorySystem )
        DELETE_MENU( Category3D_General )
        DELETE_MENU( Category3D_Information )
        DELETE_MENU( Category3D_Camera )
        DELETE_MENU( Category3D_Stereovision )
        DELETE_MENU( CategoryEffect )
        default:
            break;
        }
    }
    mMenu.clear();
}

}   /* namespace g3d */
}   /* namespace vw  */
