/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/ResourceLinker.h"
#include "../include/ResourceManager.h"
#include "../include/SceneManager.h"

namespace vw {
namespace g3d {

#define DECL_RES_LINK_FUNC( type ) \
    void ResourceLinker::Link( nw::gfx::res::Res##type target ) \
    { \
        if ( !target.IsValid() ) return; \
        void* pDefault##type = \
            ResourceManager::GetInstance()->GetDefault##type().ptr(); \
        type##LinkArray::iterator it = Get##type##sBegin(); \
        type##LinkArray::iterator end = Get##type##sEnd(); \
        for ( ; it != end; ++it ) { \
            if ( it->ref().toTarget##type.to_ptr() != pDefault##type ) continue; \
            if ( it->GetPath() != NULL && !strcmp( it->GetPath(), target.GetName() ) ) { \
                it->ref().toTarget##type.set_ptr( target.ptr() ); \
            } \
        } \
    } \
    void ResourceLinker::Unlink( nw::gfx::res::Res##type target ) \
    { \
        if ( !target.IsValid() ) return; \
        void* pDefault##type = \
            ResourceManager::GetInstance()->GetDefault##type().ptr(); \
        type##LinkArray::iterator it = Get##type##sBegin(); \
        type##LinkArray::iterator end = Get##type##sEnd(); \
        for ( ; it != end; ++it ) { \
            if ( it->ref().toTarget##type.to_ptr() == target.ptr() ) { \
                it->ref().toTarget##type.set_ptr( pDefault##type ); \
            } \
        } \
    } \
    void ResourceLinker::Unlink( nw::gfx::res::ResReference##type ref ) \
    { \
        if ( !ref.IsValid() ) return; \
        void* pDefault##type = \
            ResourceManager::GetInstance()->GetDefault##type().ptr(); \
        type##LinkArray::iterator it = Get##type##sBegin(); \
        type##LinkArray::iterator end = Get##type##sEnd(); \
        for ( ; it != end; ++it ) { \
            if ( it->ptr() == ref.ptr() ) { \
                it->ref().toTarget##type.set_ptr( pDefault##type ); \
            } \
        } \
    } \
    void ResourceLinker::AddReference( nw::gfx::res::ResReference##type ref ) \
    { \
        if ( !ref.IsValid() ) return; \
        type##LinkArray::iterator it_ = m_##type##Array.begin(); \
        type##LinkArray::iterator end_ = m_##type##Array.end(); \
        for ( ; it_ != end_; ++it_ ) { \
            if ( it_->ptr() == ref.ptr() ) break; \
        } \
        if ( it_ != end_ ) return; \
        void* pDefault##type = \
            ResourceManager::GetInstance()->GetDefault##type().ptr(); \
        if ( ref.ref().toTarget##type.to_ptr() == NULL ) { \
            SceneManager::type##List::iterator it = \
                SceneManager::GetInstance()->Get##type##sBegin(); \
            SceneManager::type##List::iterator end = \
                SceneManager::GetInstance()->Get##type##sEnd(); \
            for ( ; it != end; ++it ) Link( it->GetResObject() ); \
            if ( ref.ref().toTarget##type.to_ptr() == NULL ) { \
                ref.ref().toTarget##type.set_ptr( pDefault##type ); \
            } \
        } \
        m_##type##Array.push_back( ref ); \
    }

DECL_RES_LINK_FUNC( Texture )
//DECL_RES_LINK_FUNC( Shader )

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::Link( nw::gfx::res::ResLookupTableSet target )
{
    if ( !target.IsValid() ) return;

    void* pDefaultLookupTable =
        ResourceManager::GetInstance()->GetDefaultLookupTable().ptr();
    LookupTableLinkArray::iterator it = GetLookupTablesBegin();
    LookupTableLinkArray::iterator end = GetLookupTablesEnd();
    for ( ; it != end; ++it ) {
        if ( it->ref().toTargetLut.to_ptr() != pDefaultLookupTable ) continue;
        if ( it->GetPath() != NULL && !strcmp( it->GetPath(), target.GetName() ) ) {
            nw::gfx::res::ResImageLookupTable resLut = target.GetSamplers( it->GetTableName() );
            if ( resLut.IsValid() ) {
                it->ref().toTargetLut.set_ptr( resLut.ptr() );
            }
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::Unlink( nw::gfx::res::ResLookupTableSet target )
{
    if ( !target.IsValid() ) return;

    void* pDefaultLookupTable =
        ResourceManager::GetInstance()->GetDefaultLookupTable().ptr();
    LookupTableLinkArray::iterator it = GetLookupTablesBegin();
    LookupTableLinkArray::iterator end = GetLookupTablesEnd();
    for ( ; it != end; ++it ) {
        for ( int i = 0; i < target.GetSamplersCount(); ++i ) {
            nw::gfx::res::ResImageLookupTable resLut = target.GetSamplers( i );
            if ( it->ref().toTargetLut.to_ptr() == resLut.ptr() ) {
                it->ref().toTargetLut.set_ptr( pDefaultLookupTable );
            }
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::Unlink( nw::gfx::res::ResReferenceLookupTable ref )
{
    if ( !ref.IsValid() ) return;

    void* pDefaultLookupTable =
        ResourceManager::GetInstance()->GetDefaultLookupTable().ptr();
    LookupTableLinkArray::iterator it = GetLookupTablesBegin();
    LookupTableLinkArray::iterator end = GetLookupTablesEnd();
    for ( ; it != end; ++it ) {
        if ( it->ptr() == ref.ptr() ) {
            it->ref().toTargetLut.set_ptr( pDefaultLookupTable );
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::AddReference( nw::gfx::res::ResReferenceLookupTable ref )
{
    if ( !ref.IsValid() ) return;

    LookupTableLinkArray::iterator lut_it = m_LookupTableArray.begin();
    LookupTableLinkArray::iterator lut_end = m_LookupTableArray.end();
    for ( ; lut_it != lut_end; ++lut_it ) {
        if ( lut_it->ptr() == ref.ptr() ) break;
    }
    if ( lut_it != lut_end ) return;

    void* pDefaultLookupTable =
        ResourceManager::GetInstance()->GetDefaultLookupTable().ptr();
    if ( ref.ref().toTargetLut.to_ptr() == NULL ) {
        SceneManager::LookupTableSetList::iterator it =
            SceneManager::GetInstance()->GetLookupTableSetsBegin();
        SceneManager::LookupTableSetList::iterator end =
            SceneManager::GetInstance()->GetLookupTableSetsEnd();
        for ( ; it != end; ++it ) Link( it->GetResObject() );
        if ( ref.ref().toTargetLut.to_ptr() == NULL ) {
            ref.ref().toTargetLut.set_ptr( pDefaultLookupTable );
        }
    }
    m_LookupTableArray.push_back( ref );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
ResourceLinker*
ResourceLinker::Builder::Create( nw::os::IAllocator *pAllocator )
{
    void *buf = pAllocator->Alloc(sizeof(ResourceLinker));
    if (buf == NULL) return NULL;

    ResourceLinker* linker = new(buf) ResourceLinker( pAllocator );

    return linker;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::Destroy()
{
    this->~ResourceLinker();
    m_pAllocator->Free( this );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::ParseLink( nw::gfx::SceneNode* node )
{
    nw::gfx::Model* model = nw::ut::DynamicCast<nw::gfx::Model*>( node );
    if ( model ) ParseLink( model );

    nw::gfx::Light* light = nw::ut::DynamicCast<nw::gfx::Light*>( node );
    if ( light ) ParseLink( light );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
ResourceLinker::ResourceLinker( nw::os::IAllocator* pAllocator )
 : m_LookupTableArray( pAllocator ), m_TextureArray( pAllocator ),
   m_pAllocator( pAllocator )
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
ResourceLinker::~ResourceLinker()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::ParseLink( nw::gfx::res::ResMaterial material )
{
    if ( !material.IsValid() ) return;

    // Check texture reference
    {
        // Check external reference of image texture
        for ( int j = 0; j < material.GetTextureMappersCount(); ++j )
        {
            // Get texture mapper data
            nw::gfx::res::ResPixelBasedTextureMapper res_mapper = material.GetTextureMappers( j );
            if ( res_mapper.IsValid() )
            {
                // Examine external reference of texture mapper
                nw::gfx::res::ResReferenceTexture res_ref;
                {
                    res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceTexture>(
                        res_mapper.GetTexture()
                    );
                    AddReference( res_ref );
                }
            }
        }

        // Check external reference of procedural texture
        {
            // Get texture mapper data
            nw::gfx::res::ResProceduralTextureMapper res_mapper = material.GetProceduralTextureMapper();
            if ( res_mapper.IsValid() )
            {
                // Examine external reference of texture mapper
                nw::gfx::res::ResReferenceTexture res_ref;
                {
                    res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceTexture>(
                        res_mapper.GetTexture()
                    );
                    AddReference( res_ref );
                }
            }
        }
    }

    // Check external reference of lookup table
    {
        // Get fragment shader data
        nw::gfx::res::ResFragmentShader res_shader = material.GetFragmentShader();
        if ( !res_shader.IsValid() ) return;

        // Get lookup table data
        nw::gfx::res::ResFragmentLightingTable res_table = res_shader.GetFragmentLightingTable();
        if ( !res_table.IsValid() ) return;

        // Examine external reference of lookup table
        nw::gfx::res::ResReferenceLookupTable res_ref;
        {
            // Add ReflectanceRSampler
            if ( res_table.GetReflectanceRSampler().IsValid() )
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_table.GetReflectanceRSampler().GetSampler()
                );
                AddReference( res_ref );
            }

            // Add ReflectanceGSampler
            if ( res_table.GetReflectanceGSampler().IsValid() )
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_table.GetReflectanceGSampler().GetSampler()
                );
                AddReference( res_ref );
            }

            // Add ReflectanceBSampler
            if ( res_table.GetReflectanceBSampler().IsValid() )
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_table.GetReflectanceBSampler().GetSampler()
                );
                AddReference( res_ref );
            }

            // Add Distribution0Sampler
            if ( res_table.GetDistribution0Sampler().IsValid() )
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_table.GetDistribution0Sampler().GetSampler()
                );
                AddReference( res_ref );
            }

            // Add Distribution1Sampler
            if ( res_table.GetDistribution1Sampler().IsValid() )
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_table.GetDistribution1Sampler().GetSampler()
                );
                AddReference( res_ref );
            }

            // Add FresnelSampler
            if ( res_table.GetFresnelSampler().IsValid() )
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_table.GetFresnelSampler().GetSampler()
                );
                AddReference( res_ref );
            }
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::ParseLink( nw::gfx::Model* model )
{
    for ( int i = 0; i < model->GetMaterialCount(); ++i )
    {
        // Check material data
        ParseLink( model->GetMaterial( i )->GetOriginal() );
        ParseLink( model->GetMaterial( i )->GetBuffer() );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResourceLinker::ParseLink( nw::gfx::Light* light )
{
    // Get fragment light data
    nw::gfx::res::ResFragmentLight res_light =
        nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(light->GetResSceneObject());
    if ( res_light.IsValid() )
    {
        // Examine external reference of lookup table
        nw::gfx::res::ResReferenceLookupTable res_ref;
        {
            // Add DistanceSampler
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_light.GetDistanceSampler()
                );
                AddReference( res_ref );
            }

            // Add AngleSampler
            if ( res_light.GetAngleSampler().IsValid() )
            {
                res_ref = nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(
                    res_light.GetAngleSampler().GetSampler()
                );
                AddReference( res_ref );
            }
        }
    }
}

}
}
