/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <version.h>
#include <viewer/system.h>
#include "../include/SceneElemSelector.h"
#include "../include/SceneManager.h"
#include "../include/AnimController.h"
#include "../include/ErrorCheck.h"
#include <viewer/g3d/CameraMgr.h>
#include <viewer/g3d/LightMgr.h>

namespace vw {
namespace g3d {

#define INIT_ARRAY( type, name ) \
    m_##name##Array( nw::ut::MoveArray<type*>( Memory_GetAllocator() ) ), \
    m_target##name( m_##name##Array.begin() )

#define SETUP_ARRAY( type, name ) \
    type* name##_ptr = GetTarget##name(); \
    { \
        m_##name##Array.clear(); \
        SceneManager::name##List::iterator it = SceneManager::GetInstance()->Get##name##sBegin(); \
        SceneManager::name##List::iterator end = SceneManager::GetInstance()->Get##name##sEnd(); \
        for ( ; it != end; ++it ) { \
            m_##name##Array.push_back(&(*it)); \
        } \
    } \
    SetTarget##name( name##_ptr );

/*---------------------------------------------------------------------------*/
#define DECL_SELECT_ACCESSORS( type, name ) \
    type* \
    SceneElemSelector::GetTarget##name() \
    { \
        if ( m_target##name == m_##name##Array.end() ) return NULL; \
        return *m_target##name; \
    } \
    void \
    SceneElemSelector::SetTarget##name( type* target ) \
    { \
        name##List::iterator it = m_##name##Array.begin(); \
        name##List::iterator end = m_##name##Array.end(); \
        for ( ; it != end; ++it ) { \
            if ( *it == target ) { m_target##name = it; break; } \
        } \
        if ( it == end ) { \
            m_target##name = m_##name##Array.begin(); \
        } \
        if ( m_target##name != m_##name##Array.end() ) { \
            Activate##name( *m_target##name ); \
        } else { \
            Activate##name( NULL ); \
        } \
    }

#define DECL_SELECT_FUNC( type, name ) \
    DECL_SELECT_ACCESSORS( type, name ) \
    bool \
    SceneElemSelector::IsFirst##name() \
    { \
        return ( m_target##name == m_##name##Array.begin() ); \
    } \
    bool \
    SceneElemSelector::IsLast##name() \
    { \
        name##List::iterator end = m_##name##Array.end(); \
        if ( m_target##name == end ) return true; \
        return ( m_target##name == --end ); \
    } \
    void \
    SceneElemSelector::ChangeToFirst##name() \
    { \
        m_target##name = m_##name##Array.begin(); \
        Activate##name( GetTarget##name() ); \
    } \
    void \
    SceneElemSelector::ChangeToLast##name() \
    { \
        m_target##name = m_##name##Array.end(); \
        if ( m_target##name != m_##name##Array.begin() ) --m_target##name; \
        Activate##name( GetTarget##name() ); \
    } \
    void \
    SceneElemSelector::ChangeToPrev##name() \
    { \
        if ( m_target##name != m_##name##Array.end() ) { \
            if ( m_target##name == m_##name##Array.begin() ) { \
                m_target##name = m_##name##Array.end(); \
            } \
            --m_target##name; \
            NW_ASSERT( m_target##name != m_##name##Array.end() ); \
            Activate##name( *m_target##name ); \
        } else { \
            Activate##name( NULL ); \
        } \
    } \
    void \
    SceneElemSelector::ChangeToNext##name() \
    { \
        if ( m_target##name != m_##name##Array.end() ) { \
            ++m_target##name; \
            if ( m_target##name == m_##name##Array.end() ) { \
                m_target##name = m_##name##Array.begin(); \
            } \
            NW_ASSERT( m_target##name != m_##name##Array.end() ); \
            Activate##name( *m_target##name ); \
        } else { \
            Activate##name( NULL ); \
        } \
    }

DECL_SELECT_FUNC( ModelElem, Model )
DECL_SELECT_FUNC( CameraElem, Camera )
DECL_SELECT_FUNC( LightElem, Light )
DECL_SELECT_FUNC( FogElem, Fog )
DECL_SELECT_ACCESSORS( SceneEnvironmentSettingElem, SceneEnvironmentSetting )

#define DECL_EXISTS( Capital, LowerCase ) \
bool Exists( \
    SceneElemSelector::SceneEnvironmentSettingList &sceneEnvironmentSettings, \
    nw::gfx::Capital *LowerCase \
) { \
    SceneElemSelector::SceneEnvironmentSettingList::iterator env_it \
        = sceneEnvironmentSettings.begin(); \
    SceneElemSelector::SceneEnvironmentSettingList::iterator \
        env_end = sceneEnvironmentSettings.end(); \
    for (; env_it != env_end; ++env_it) \
    { \
        nw::gfx::SceneEnvironmentSetting::Capital##BinderArray::iterator LowerCase##_it \
            = (*env_it)->GetSceneNode()->Get##Capital##Begin(); \
        nw::gfx::SceneEnvironmentSetting::Capital##BinderArray::iterator LowerCase##_end \
            = (*env_it)->GetSceneNode()->Get##Capital##End(); \
        for (; LowerCase##_it!=LowerCase##_end; ++LowerCase##_it) { \
            if ( (*LowerCase##_it).LowerCase == LowerCase ) return true; \
        } \
    } \
    return false; \
}

DECL_EXISTS( Camera, camera )
DECL_EXISTS( Fog, fog )

SceneElemSelector* SceneElemSelector::m_pInstance = NULL;

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SceneElemSelector*
SceneElemSelector::GetInstance()
{
    NW_ASSERT( m_pInstance != NULL );
    return m_pInstance;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::Setup()
{
    SETUP_ARRAY( ModelElem, Model )
    SETUP_ARRAY( SceneEnvironmentSettingElem, SceneEnvironmentSetting )
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ActivateModel(ModelElem* target)
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ActivateCamera(CameraElem* target)
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ActivateLight(LightElem* target)
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ActivateFog(FogElem* target)
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
bool Exists(
    SceneElemSelector::SceneEnvironmentSettingList &sceneEnvironmentSettings,
    nw::gfx::Light *light
) {
    SceneElemSelector::SceneEnvironmentSettingList::iterator env_it
        = sceneEnvironmentSettings.begin();
    SceneElemSelector::SceneEnvironmentSettingList::iterator
        env_end = sceneEnvironmentSettings.end();
    for (; env_it != env_end; ++env_it)
    {
        nw::gfx::SceneEnvironmentSetting::LightSetBinderArray::iterator set_it = (*env_it)->GetSceneNode()->GetLightSetBegin();
        nw::gfx::SceneEnvironmentSetting::LightSetBinderArray::iterator set_end = (*env_it)->GetSceneNode()->GetLightSetEnd();
        for ( ; set_it != set_end; ++set_it )
        {
            nw::gfx::LightSet* lightSet = set_it->lightSet.Get();
            if ( !lightSet ) continue;

            // Ambient light check
            if ( lightSet->GetAmbientLight() == light ) return true;

            // Hemisphere light check
            if ( lightSet->GetHemiSphereLight() == light ) return true;

            // Vertex light check
        #if defined(NW_GFX_VERTEX_LIGHT_ENABLED)
            {
                nw::gfx::VertexLightArray::iterator it = lightSet->GetVertexLightBegin();
                nw::gfx::VertexLightArray::iterator end = lightSet->GetVertexLightEnd();
                for ( ; it != end; ++it ) {
                    if ( (*it) == light ) return true;
                }
            }
        #endif

            // Fragment light check
            {
                nw::gfx::FixedFragmentLightArray::iterator it = lightSet->GetFragmentLightBegin();
                nw::gfx::FixedFragmentLightArray::iterator end = lightSet->GetFragmentLightEnd();
                for ( ; it != end; ++it ) {
                    if ( (*it) == light ) return true;
                }
            }
        }
    }
    return false;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
bool
SceneElemSelector::IsFirstSceneEnvironmentSetting()
{
    return ( m_targetSceneEnvironmentSetting == m_SceneEnvironmentSettingArray.begin() );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
bool
SceneElemSelector::IsLastSceneEnvironmentSetting()
{
    return ( m_targetSceneEnvironmentSetting == m_SceneEnvironmentSettingArray.end() );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToFirstSceneEnvironmentSetting()
{
    m_targetSceneEnvironmentSetting = m_SceneEnvironmentSettingArray.begin();
    ActivateSceneEnvironmentSetting( GetTargetSceneEnvironmentSetting() );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToLastSceneEnvironmentSetting()
{
    m_targetSceneEnvironmentSetting = m_SceneEnvironmentSettingArray.end();
    ActivateSceneEnvironmentSetting( GetTargetSceneEnvironmentSetting() );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToPrevSceneEnvironmentSetting()
{
    if ( m_targetSceneEnvironmentSetting == m_SceneEnvironmentSettingArray.begin() )
    {
        m_targetSceneEnvironmentSetting = m_SceneEnvironmentSettingArray.end();
    }
    else
    {
        --m_targetSceneEnvironmentSetting;
    }

    ActivateSceneEnvironmentSetting( GetTargetSceneEnvironmentSetting() );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToNextSceneEnvironmentSetting()
{
    if ( m_targetSceneEnvironmentSetting == m_SceneEnvironmentSettingArray.end() )
    {
        m_targetSceneEnvironmentSetting = m_SceneEnvironmentSettingArray.begin();
    }
    else
    {
        ++m_targetSceneEnvironmentSetting;
    }

    ActivateSceneEnvironmentSetting( GetTargetSceneEnvironmentSetting() );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ActivateSceneEnvironmentSetting(SceneEnvironmentSettingElem* target)
{
    SceneManager *scnMgr = SceneManager::GetInstance();
    // Rebuild camera/light selected object array
    {
        CameraElem* camera_ptr = GetTargetCamera();
        LightElem* light_ptr = GetTargetLight();
        FogElem* fog_ptr = GetTargetFog();

        m_CameraArray.clear();
        m_CameraIndexArray.clear();
        m_LightArray.clear();
        m_FogArray.clear();

        if ( target != NULL )
        {
            // Add camera
            {
                nw::gfx::SceneEnvironmentSetting::CameraBinderArray::iterator it = target->GetSceneNode()->GetCameraBegin();
                nw::gfx::SceneEnvironmentSetting::CameraBinderArray::iterator end = target->GetSceneNode()->GetCameraEnd();
                for ( ; it != end; ++it )
                {
                    CameraElem *camera = &*SceneManager::GetInstance()->FindCamera( it->camera );
                    m_CameraArray.push_back( camera );
                    m_CameraIndexArray.push_back( it->index );
                }
            }

            // Add light
            {
                nw::gfx::SceneEnvironmentSetting::LightSetBinderArray::iterator set_it = target->GetSceneNode()->GetLightSetBegin();
                nw::gfx::SceneEnvironmentSetting::LightSetBinderArray::iterator set_end = target->GetSceneNode()->GetLightSetEnd();
                for ( ; set_it != set_end; ++set_it )
                {
                    nw::gfx::LightSet* lightSet = set_it->lightSet.Get();
                    if ( !lightSet ) continue;

                    // Add ambient light
                    {
                        if ( lightSet->GetAmbientLight() )
                        {
                            m_LightArray.push_back( &*scnMgr->FindLight( lightSet->GetAmbientLight() ) );
                        }
                    }

                    // Add hemisphere light
                    {
                        if ( lightSet->GetHemiSphereLight() )
                        {
                            m_LightArray.push_back( &*scnMgr->FindLight( lightSet->GetHemiSphereLight() ) );
                        }
                    }

                    // Add vertex light
                #if defined(NW_GFX_VERTEX_LIGHT_ENABLED)
                    {
                        nw::gfx::VertexLightArray::iterator it = lightSet->GetVertexLightBegin();
                        nw::gfx::VertexLightArray::iterator end = lightSet->GetVertexLightEnd();
                        for ( ; it != end; ++it ) {
                            if ( *it ) m_LightArray.push_back( &*scnMgr->FindLight( *it ) );
                        }
                    }
                #endif

                    // Add fragment light
                    {
                        nw::gfx::FixedFragmentLightArray::iterator it = lightSet->GetFragmentLightBegin();
                        nw::gfx::FixedFragmentLightArray::iterator end = lightSet->GetFragmentLightEnd();
                        for ( ; it != end; ++it ) {
                            if ( *it ) m_LightArray.push_back( &*scnMgr->FindLight( *it ) );
                        }
                    }
                }
            }

            // Add fog
            {
                nw::gfx::SceneEnvironmentSetting::FogBinderArray::iterator it = target->GetSceneNode()->GetFogBegin();
                nw::gfx::SceneEnvironmentSetting::FogBinderArray::iterator end = target->GetSceneNode()->GetFogEnd();
                for ( ; it != end; ++it )
                {
                    FogElem *fog = &*SceneManager::GetInstance()->FindFog( it->fog );
                    m_FogArray.push_back( fog );
                }
            }
        }
        else
        {
            // Add camera
            {
                int idx = 0;
                SceneManager::CameraList::iterator it = SceneManager::GetInstance()->GetCamerasBegin();
                SceneManager::CameraList::iterator end = SceneManager::GetInstance()->GetCamerasEnd();
                for ( ; it != end; ++it ) {
                    if ( !Exists( m_SceneEnvironmentSettingArray, it->GetSceneNode() ) )
                    {
                        m_CameraArray.push_back( &*it );
                        m_CameraIndexArray.push_back( idx++ );
                    }
                }
            }

            // Add light
            {
                SceneManager::LightList::iterator it = SceneManager::GetInstance()->GetLightsBegin();
                SceneManager::LightList::iterator end = SceneManager::GetInstance()->GetLightsEnd();
                for ( ; it != end; ++it ) {
                    if ( !Exists( m_SceneEnvironmentSettingArray, it->GetSceneNode() ) )
                    {
                        m_LightArray.push_back( &*it );
                    }
                }
            }

            // Add fog
            {
                SceneManager::FogList::iterator it = SceneManager::GetInstance()->GetFogsBegin();
                SceneManager::FogList::iterator end = SceneManager::GetInstance()->GetFogsEnd();
                for ( ; it != end; ++it ) {
                    if ( !Exists( m_SceneEnvironmentSettingArray, it->GetSceneNode() ) )
                    {
                       m_FogArray.push_back( &*it );
                    }
                }
            }
        }

        // Add default camera
        //if ( m_CameraArray.size() == 0 )
        {
            // Search for unused index for default camera
            int defaultCameraIndex = 0;
            nw::ut::MoveArray<int>::iterator end = m_CameraIndexArray.end();
            while ( true ) {
                nw::ut::MoveArray<int>::iterator it = m_CameraIndexArray.begin();
                for ( ; it != end; ++it ) {
                    if ( *it == defaultCameraIndex ) {
                        ++defaultCameraIndex;
                        break;
                    }
                }
                if ( it == end ) break;
            }
            m_CameraArray.push_back( SceneManager::GetInstance()->GetDefaultCamera() );
            m_CameraIndexArray.push_back( defaultCameraIndex );
        }

        // Check default light addition
        m_UseDefaultLight = false;
        if ( m_LightArray.size() == 0 ) {
            m_LightArray.push_back( SceneManager::GetInstance()->GetDefaultAmbientLight() );
            m_LightArray.push_back( SceneManager::GetInstance()->GetDefaultFragmentLight() );
            m_UseDefaultLight = true;
        }

        // Camera selected array initialization
        {
            nw::ut::MoveArray<CameraElem*>& array = SceneManager::GetInstance()->GetExternalCameraArray();
            nw::ut::MoveArray<CameraElem*> copyArray( Memory_GetAllocator() );
            nw::ut::MoveArray<CameraElem*>::iterator it = m_CameraArray.begin();
            nw::ut::MoveArray<CameraElem*>::iterator end = m_CameraArray.end();
            for ( ; it != end; ++it ) copyArray.push_back( *it );
            it = array.begin();
            end = array.end();
            for ( ; it != end; ++it ) copyArray.push_back( *it );
            CameraMgr::GetInstance()->Setup( copyArray.begin(), copyArray.end() );
        }

        // Light selected array initialization
        {
            nw::ut::MoveArray<LightElem*>& array = SceneManager::GetInstance()->GetExternalLightArray();
            nw::ut::MoveArray<LightElem*> copyArray( Memory_GetAllocator() );
            nw::ut::MoveArray<LightElem*>::iterator it = m_LightArray.begin();
            nw::ut::MoveArray<LightElem*>::iterator end = m_LightArray.end();
            for ( ; it != end; ++it ) copyArray.push_back( *it );
            it = array.begin();
            end = array.end();
            for ( ; it != end; ++it ) copyArray.push_back( *it );
            LightMgr::GetInstance()->Setup( copyArray.begin(), copyArray.end() );
        }

        // Target setting.
        SetTargetCamera( camera_ptr );
        SetTargetLight( light_ptr );
        SetTargetFog( fog_ptr );
    }

    // Set scene environment
    ApplyCurrentSceneEnvironmentSetting();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ApplyCurrentSceneEnvironmentSetting()
{
    // Clear environment that is already applied
    nw::gfx::SceneEnvironment& sceneEnvironment =
        vw::system::Video_GetRenderContext()->GetSceneEnvironment();
    sceneEnvironment.ClearSettings();

    // Apply scene environment setting
    if ( m_targetSceneEnvironmentSetting != m_SceneEnvironmentSettingArray.end() )
    {
        // Get current setting information
        nw::gfx::SceneEnvironmentSetting* setting = (*m_targetSceneEnvironmentSetting)->GetSceneNode();

        // Generate scene environment from current information
        sceneEnvironment.ApplyFrom(*setting);

        // Add default camera
        // if ( setting->GetCameraBegin() == setting->GetCameraEnd() )
        {
            sceneEnvironment.SetCamera(
                m_CameraIndexArray.back(),
                m_CameraArray.back()->GetSceneNode()
            );
        }

        // Check default light addition
        if ( m_UseDefaultLight )
        {
            // Get light set
            nw::gfx::LightSet* lightSet = SceneManager::GetInstance()->GetDefaultLightSet();
            lightSet->ClearAll();

            // Add default light
            {
                lightSet->SetAmbientLight(
                    nw::ut::DynamicCast<nw::gfx::AmbientLight*>(
                    SceneManager::GetInstance()->GetDefaultAmbientLight()->GetSceneNode())
                );
#if NW4C_VIEWER_VERSION >= NW4C_VIEWER_VERSION_NUMBER(2,1,0,0)
                lightSet->PushFragmentLight(
                    nw::ut::DynamicCast<nw::gfx::FragmentLight*>(
                    SceneManager::GetInstance()->GetDefaultFragmentLight()->GetSceneNode())
                );
#else
                lightSet->SetFragmentLight(
                    nw::ut::DynamicCast<nw::gfx::FragmentLight*>(
                    SceneManager::GetInstance()->GetDefaultFragmentLight()->GetSceneNode())
                );
#endif
            }

            // Light set settings
            sceneEnvironment.SetLightSet( 0, lightSet );
        }
    }
    else
    {
        // Camera configuration
        // TODO: Check sceneEnvironment camera setting limit number
        {
            CameraList::iterator it = m_CameraArray.begin();
            CameraList::iterator end = m_CameraArray.end();
            nw::ut::MoveArray<int>::iterator idx = m_CameraIndexArray.begin();
            for ( ; it != end; ++idx, ++it ) {
                sceneEnvironment.SetCamera(*idx, (*it)->GetSceneNode());
            }
        }

        //　 Light set settings
        {
            // Get light set
            nw::gfx::LightSet* lightSet = SceneManager::GetInstance()->GetDefaultLightSet();
            lightSet->ClearAll();

            // Patrol light
            LightList::iterator it = m_LightArray.begin();
            LightList::iterator end = m_LightArray.end();
            bool firstAmibientLight = true;
            bool firstHemiSphereLight = true;
            for ( ; it != end; ++it )
            {
                switch( (*it)->GetSceneNode()->GetResSceneObject().GetTypeInfo() )
                {
                case nw::gfx::res::ResAmbientLight::TYPE_INFO:
                    if ( firstAmibientLight ) {
                        lightSet->SetAmbientLight(nw::ut::DynamicCast<nw::gfx::AmbientLight*>((*it)->GetSceneNode()));
                        firstAmibientLight = false;
                    }
                    break;
                case nw::gfx::res::ResHemiSphereLight::TYPE_INFO:
                    if ( firstHemiSphereLight ) {
                        lightSet->SetHemiSphereLight(nw::ut::DynamicCast<nw::gfx::HemiSphereLight*>((*it)->GetSceneNode()));
                        firstHemiSphereLight = false;
                    }
                    break;
            #if defined(NW_GFX_VERTEX_LIGHT_ENABLED)
                case nw::gfx::res::ResVertexLight::TYPE_INFO:
#if NW4C_VIEWER_VERSION >= NW4C_VIEWER_VERSION_NUMBER(2,1,0,0)
                    lightSet->PushVertexLight(nw::ut::DynamicCast<nw::gfx::VertexLight*>((*it)->GetSceneNode()));
#else
                    lightSet->SetVertexLight(nw::ut::DynamicCast<nw::gfx::VertexLight*>((*it)->GetSceneNode()));
#endif
                    break;
#endif
                case nw::gfx::res::ResFragmentLight::TYPE_INFO:
#if NW4C_VIEWER_VERSION >= NW4C_VIEWER_VERSION_NUMBER(2,1,0,0)
                    lightSet->PushFragmentLight(nw::ut::DynamicCast<nw::gfx::FragmentLight*>((*it)->GetSceneNode()));
#else
                    lightSet->SetFragmentLight(nw::ut::DynamicCast<nw::gfx::FragmentLight*>((*it)->GetSceneNode()));
#endif
                    break;
                default:
                    break;
                }
            }

            // Light set settings
            sceneEnvironment.SetLightSet( 0, lightSet );
        }

        // Fog settings
        // TODO: Check sceneEnvironment fog setting limit number
        {
            SceneManager::FogList::iterator it = SceneManager::GetInstance()->GetFogsBegin();
            SceneManager::FogList::iterator end = SceneManager::GetInstance()->GetFogsEnd();
            for ( int idx = 0; it != end; ++idx, ++it ) {
                sceneEnvironment.SetFog(idx, it->GetSceneNode());
            }
        }
    }

    // Check resource reference cut
    {
        SceneManager::ModelList::iterator it = SceneManager::GetInstance()->GetModelsBegin();
        SceneManager::ModelList::iterator end = SceneManager::GetInstance()->GetModelsEnd();
        for ( ; it != end; ++it )
        {
            // Check light set reference cut
            ErrorCheck::CheckModelLightSetIndexReference( it->GetSceneNode() );

            // Check projection texture camera reference cut
            ErrorCheck::CheckModelCameraReferenceIndexReference( it->GetSceneNode() );

            // Check fog reference cut
            ErrorCheck::CheckModelFogIndexReference( it->GetSceneNode() );
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
bool
SceneElemSelector::IsFirstElem( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        return IsFirstModel();
    case ELEM_CAMERA:
        return IsFirstCamera();
    case ELEM_LIGHT:
        return IsFirstLight();
    case ELEM_FOG:
        return IsFirstFog();
    case ELEM_SCENE_ENVIRONMENT_SETTING:
        return IsFirstSceneEnvironmentSetting();
    default:
        break;
    }

    return false;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
bool
SceneElemSelector::IsLastElem( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        return IsLastModel();
    case ELEM_CAMERA:
        return IsLastCamera();
    case ELEM_LIGHT:
        return IsLastLight();
    case ELEM_FOG:
        return IsLastFog();
    case ELEM_SCENE_ENVIRONMENT_SETTING:
        return IsLastSceneEnvironmentSetting();
    default:
        break;
    }

    return false;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToFirstElem( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        ChangeToFirstModel();
        break;
    case ELEM_CAMERA:
        ChangeToFirstCamera();
        break;
    case ELEM_LIGHT:
        ChangeToFirstLight();
        break;
    case ELEM_FOG:
        ChangeToFirstFog();
        break;
    case ELEM_SCENE_ENVIRONMENT_SETTING:
        ChangeToFirstSceneEnvironmentSetting();
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToLastElem( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        ChangeToLastModel();
        break;
    case ELEM_CAMERA:
        ChangeToLastCamera();
        break;
    case ELEM_LIGHT:
        ChangeToLastLight();
        break;
    case ELEM_FOG:
        ChangeToLastFog();
        break;
    case ELEM_SCENE_ENVIRONMENT_SETTING:
        ChangeToLastSceneEnvironmentSetting();
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToPrevElem( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        ChangeToPrevModel();
        break;
    case ELEM_CAMERA:
        ChangeToPrevCamera();
        break;
    case ELEM_LIGHT:
        ChangeToPrevLight();
        break;
    case ELEM_FOG:
        ChangeToPrevFog();
        break;
    case ELEM_SCENE_ENVIRONMENT_SETTING:
        ChangeToPrevSceneEnvironmentSetting();
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToNextElem( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        ChangeToNextModel();
        break;
    case ELEM_CAMERA:
        ChangeToNextCamera();
        break;
    case ELEM_LIGHT:
        ChangeToNextLight();
        break;
    case ELEM_FOG:
        ChangeToNextFog();
        break;
    case ELEM_SCENE_ENVIRONMENT_SETTING:
        ChangeToNextSceneEnvironmentSetting();
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToPrevAnim( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        {
            ModelElem *pTargetModel = this->GetTargetModel();
            if(pTargetModel) {
                pTargetModel->GetAnimLinker()->ChangeToPrevAnimation();
            }
        }
        break;
    case ELEM_CAMERA:
        {
            CameraElem *pTargetCamera = this->GetTargetCamera();
            if(pTargetCamera) {
                pTargetCamera->GetAnimLinker()->ChangeToPrevAnimation();;
            }
        }
        break;
    case ELEM_LIGHT:
        {
            LightElem *pTargetLight = this->GetTargetLight();
            if(pTargetLight) {
                pTargetLight->GetAnimLinker()->ChangeToPrevAnimation();;
            }
        }
        break;
    case ELEM_FOG:
        {
            FogElem *pTargetFog = this->GetTargetFog();
            if(pTargetFog) {
                pTargetFog->GetAnimLinker()->ChangeToPrevAnimation();;
            }
        }
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::ChangeToNextAnim( const SelectElemType type )
{
    switch ( type )
    {
    case ELEM_MODEL:
        {
            ModelElem *pTargetModel = this->GetTargetModel();
            if(pTargetModel) {
                pTargetModel->GetAnimLinker()->ChangeToNextAnimation();
            }
        }
        break;
    case ELEM_CAMERA:
        {
            CameraElem *pTargetCamera = this->GetTargetCamera();
            if(pTargetCamera) {
                pTargetCamera->GetAnimLinker()->ChangeToNextAnimation();;
            }
        }
        break;
    case ELEM_LIGHT:
        {
            LightElem *pTargetLight = this->GetTargetLight();
            if(pTargetLight) {
                pTargetLight->GetAnimLinker()->ChangeToNextAnimation();;
            }
        }
        break;
    case ELEM_FOG:
        {
            FogElem *pTargetFog = this->GetTargetFog();
            if(pTargetFog) {
                pTargetFog->GetAnimLinker()->ChangeToNextAnimation();;
            }
        }
        break;
    default:
        break;
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
int
SceneElemSelector::GetCameraIndex( nw::gfx::Camera* camera )
{
    CameraList::iterator it = m_CameraArray.begin();
    CameraList::iterator end = m_CameraArray.end();
    nw::ut::MoveArray<int>::iterator idx = m_CameraIndexArray.begin();
    for ( ; it != end; ++idx, ++it ) {
        if ( (*it)->GetSceneNode() == camera ) return *idx;
    }
    NW_ASSERT( it != end );
    return 0;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
nw::gfx::SceneObject*
SceneElemSelector::GetTargetElem( const SelectElemType type )
{
    nw::gfx::SceneObject* object = NULL;
    switch ( type )
    {
    case ELEM_MODEL:
        {
            ModelElem *pTargetModel = this->GetTargetModel();
            if(pTargetModel) {
                object = pTargetModel->GetSceneNode();
            }
        }
        break;
    case ELEM_CAMERA:
        {
            CameraElem *pTargetCamera = this->GetTargetCamera();
            if(pTargetCamera) {
                object = pTargetCamera->GetSceneNode();
            }
        }
        break;
    case ELEM_LIGHT:
        {
            LightElem *pTargetLight = this->GetTargetLight();
            if(pTargetLight) {
                object = pTargetLight->GetSceneNode();
            }
        }
        break;
    case ELEM_FOG:
        {
            FogElem *pTargetFog = this->GetTargetFog();
            if(pTargetFog) {
                object = pTargetFog->GetSceneNode();
            }
        }
        break;
    case ELEM_SCENE_ENVIRONMENT_SETTING:
        {
            SceneEnvironmentSettingElem *pTargetScene = this->GetTargetSceneEnvironmentSetting();
            if(pTargetScene) {
                object = pTargetScene->GetSceneNode();
            }
        }
        break;
    default:
        break;
    }

    return object;

}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
AnimSetLinker*
SceneElemSelector::GetTargetElemLinker( const SelectElemType type )
{
    AnimSetLinker* linker = NULL;
    switch ( type )
    {
    case ELEM_MODEL:
        {
            ModelElem *pTargetModel = this->GetTargetModel();
            if(pTargetModel) {
                linker = pTargetModel->GetAnimLinker();
            }
        }
        break;
    case ELEM_CAMERA:
        {
            CameraElem *pTargetCamera = this->GetTargetCamera();
            if(pTargetCamera) {
                linker = pTargetCamera->GetAnimLinker();
            }
        }
        break;
    case ELEM_LIGHT:
        {
            LightElem *pTargetLight = this->GetTargetLight();
            if(pTargetLight) {
                linker = pTargetLight->GetAnimLinker();
            }
        }
        break;
    case ELEM_FOG:
        {
            FogElem *pTargetFog = this->GetTargetFog();
            if(pTargetFog) {
                linker = pTargetFog->GetAnimLinker();
            }
        }
        break;
    default:
        break;
    }

    return linker;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
IAnimController*
SceneElemSelector::GetTargetElemController( const SelectElemType type )
{
    IAnimController* controller = NULL;
    switch ( type )
    {
    case ELEM_MODEL:
        {
            ModelElem *pTargetModel = this->GetTargetModel();
            if(pTargetModel) {
                controller = pTargetModel->GetAnimController();
            }
        }
        break;
    case ELEM_CAMERA:
        {
            CameraElem *pTargetCamera = this->GetTargetCamera();
            if(pTargetCamera) {
                controller = pTargetCamera->GetAnimController();
            }
        }
        break;
    case ELEM_LIGHT:
        {
            LightElem *pTargetLight = this->GetTargetLight();
            if(pTargetLight) {
                controller = pTargetLight->GetAnimController();
            }
        }
        break;
    case ELEM_FOG:
        {
            FogElem *pTargetFog = this->GetTargetFog();
            if(pTargetFog) {
                controller = pTargetFog->GetAnimController();
            }
        }
        break;
    case ELEM_PARTICLEMODEL:
        {
            // TODO: Add ParticleModel
        }
    default:
        break;
    }

    return controller;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
SceneElemSelector::GetTargetElemName( const SelectElemType type )
{
    const char* pName = NULL;
    switch ( type )
    {
    case ELEM_MODEL:
        {
            ModelElem *pTargetModel = this->GetTargetModel();
            if(pTargetModel) {
                pName = pTargetModel->GetName();
            }
        }
        break;
    case ELEM_CAMERA:
        {
            CameraElem *pTargetCamera = this->GetTargetCamera();
            if(pTargetCamera) {
                pName = pTargetCamera->GetName();
            }
        }
        break;
    case ELEM_LIGHT:
        {
            LightElem *pTargetLight = this->GetTargetLight();
            if(pTargetLight) {
                pName = pTargetLight->GetName();
            }
        }
        break;
    case ELEM_FOG:
        {
            FogElem *pTargetFog = this->GetTargetFog();
            if(pTargetFog) {
                pName = pTargetFog->GetName();
            }
        }
        break;
    case ELEM_PARTICLEMODEL:
        {
            // TODO: Add ParticleModel
        }
    default:
        break;
    }

    return pName;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SceneElemSelector::SceneElemSelector()
 : INIT_ARRAY( ModelElem, Model ),
   INIT_ARRAY( CameraElem, Camera ),
   INIT_ARRAY( LightElem, Light ),
   INIT_ARRAY( FogElem, Fog ),
   INIT_ARRAY( SceneEnvironmentSettingElem, SceneEnvironmentSetting ),
   m_CameraIndexArray( nw::ut::MoveArray<int>( Memory_GetAllocator() ) )
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SceneElemSelector::~SceneElemSelector()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::Initialize()
{
    if (m_pInstance != NULL) return;

    void *buf = Memory_GetAllocator()->Alloc(sizeof(SceneElemSelector));
    if (buf == NULL) return;

    m_pInstance = new(buf) SceneElemSelector();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SceneElemSelector::Finalize()
{
    if (m_pInstance == NULL) return;

    m_pInstance->~SceneElemSelector();
    Memory_GetAllocator()->Free(m_pInstance);

    m_pInstance = NULL;
}

}
}
