/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/



#include "../include/Setting3DMisc.h"

namespace vw  {
namespace g3d {



/*---------------------------------------------------------------------------*
  Name:         Setting::GetInstance

  Description:  Gets the 3D settings instance.

  Arguments:    None.

  Returns:
 *---------------------------------------------------------------------------*/
/* static */ Setting&
Setting::GetInstance()
{
    static Setting sInstance;
    return sInstance;
}


/*---------------------------------------------------------------------------*
  Name:         Setting::Setting

  Description:  Constructor

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
/* ctor */
Setting::Setting()
{
    m_TexAllocType = ALLOCTYPE_FCRAM;
    m_ModelAllocType = ALLOCTYPE_FCRAM;
}


/*---------------------------------------------------------------------------*
  Name:         Setting::SetTexAllocType

  Description:  Sets the texture resource priority placement location.

  Arguments:    alloc_type  Priority placement location (MEM1/MEM2)

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
Setting::SetTexAllocType( u32 alloc_type )
{
    m_TexAllocType = u8(alloc_type);
}


/*---------------------------------------------------------------------------*
  Name:         Setting::SetModelAllocType

  Description:  Sets the model resource priority placement location.

  Arguments:    alloc_type  Priority placement location (MEM1/MEM2)

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
Setting::SetModelAllocType( u32 alloc_type )
{
    m_ModelAllocType = u8(alloc_type);
}


}   /* namespace g3d */
}   /* namespace vw  */

