/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/g3d/Protocol4C.h>
#include "../include/SurvivalStreamTask.h"
#include "../include/ModuleG3dImpl.h"

namespace vw  {
namespace g3d {

static void SurvivalStream_Main_( void* arg );

// Communication task for existence confirmation
static system::VTaskObj sSurvivalStreamTask;

/*---------------------------------------------------------------------------*
  Name:         SurvivalStream_Main_

  Description:  Communication process for existence confirmation

  Arguments:    retraceCount    VIRetrace count value
                arg     Parameter arguments

  Returns:      None.
 *---------------------------------------------------------------------------*/
static void
SurvivalStream_Main_(void* arg )
{
    u32 size;
    const u32 survival = ModuleG3dImpl::GetInstance()->Get3DEditorSurvivalChannel();

    while ( ( size = system::Stream_GetReadableBytes( survival ) ) >= sizeof(Protocol4C::Header) )
    {
        Protocol4C::Header header;
        system::Stream_Peek( survival, &header, sizeof(Protocol4C::Header) );
        if ( size < (header.size + sizeof(Protocol4C::Header)) )
        {
            break;
        }
        system::Stream_Skip( survival, sizeof(Protocol4C::Header) );
        system::Stream_Skip( survival, header.size );
        header.size = 0;
        system::Stream_Write( survival, &header, sizeof(Protocol4C::Header) );
    }
}


/*---------------------------------------------------------------------------*
  Name:         SurvivalStream_Init

  Description:  Registers the existence confirmation process to VTask

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SurvivalStream_Init( void )
{
    static const u32 SUVIVAL_STREAM_BUFFER_SIZE = 128;
    const u32 survival = ModuleG3dImpl::GetInstance()->Get3DEditorSurvivalChannel();

    system::Stream_CreateChannel( survival, SUVIVAL_STREAM_BUFFER_SIZE );

    VTask_InitObj( &sSurvivalStreamTask, SurvivalStream_Main_, NULL );
    VTask_AddObj( &sSurvivalStreamTask );
}


}   /* namespace g3d */
}   /* namespace vw  */

