/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nw/viewer/g3d/Module3D.h>
#include <viewer/system.h>
#include <viewer/g3d/CameraMgr.h>
#include "../../common/include/if_common.h"

// Macro that defines the function to get the iterator.
#define DEFINE_NODE_ITER_GETTER(name)\
name##Iter \
Module3D::Get##name(u32 idx) const\
{\
    if(idx >= vw::g3d::SceneManager::GetInstance()->Get##name##sCount()) { \
        return name##Iter();\
    } \
\
    vw::g3d::SceneManager::name##List::iterator iter\
    = advance_iterator(vw::g3d::SceneManager::GetInstance()->Get##name##sBegin(), idx);\
\
    return name##Iter(iter == SCN_MGR->Get##name##sEnd() ? NULL : &*iter);\
}\
u32 \
Module3D::Get##name##Num() const\
{\
    return vw::g3d::SceneManager::GetInstance()->Get##name##sCount();\
}

namespace nw     {
namespace viewer {
namespace g3d    {

/*---------------------------------------------------------------------------*
  Name:         Module3D::Module3D

  Description:  Constructor

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
/* ctor */
Module3D::Module3D()
 : sys::Module( vw::system::G3D_ID ),
   mResListener( NULL )
{
}
#if 0
/*---------------------------------------------------------------------------*
  Name:         Module3D::RenderScene

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::RenderScene(nw::gfx::IRenderTarget* renderTarget, nw::gfx::Camera* camera)
{
    nw::gfx::Camera* left = vw::g3d::CameraMgr::GetInstance()->GetLeftViewCamera();
    left->ViewMatrix() = camera->ViewMatrix();
    left->ProjectionMatrix() = camera->ProjectionMatrix();

    vw::g3d::SceneRenderer::GetInstance()->RenderScene( renderTarget );
}
#endif

/*---------------------------------------------------------------------------*
  Name:         Module3D::DrawScene

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::DrawScene(nw::gfx::IRenderTarget* renderTarget, nw::viewer::g3d::ScreenKind screen)
{
    nw::gfx::Camera* camera = NULL;
    switch(screen)
    {
    case nw::viewer::g3d::LEFT_SCREEN:
        camera = vw::g3d::CameraMgr::GetInstance()->GetLeftViewCamera();
        break;
    case nw::viewer::g3d::RIGHT_SCREEN:
        camera = vw::g3d::CameraMgr::GetInstance()->GetRightViewCamera();
        break;
    }

    if( renderTarget != NULL && camera != NULL )
    {
        vw::g3d::SceneRenderer::GetInstance()->RenderScene( renderTarget, camera );
    }
}

/*---------------------------------------------------------------------------*
  Name:         Module3D::Draw3DInfo

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::Draw3DInfo(nw::gfx::Camera* viewCamera, nw::gfx::Camera* baseCamera, u32 flag)
{
    bool isRender[vw::g3d::SceneRenderer::RenderNum];
    for(u32 i=0; i<vw::g3d::SceneRenderer::RenderNum; ++i)
    {
        isRender[i] = (flag & 1<<i) > 0;
    }

    vw::g3d::SceneRenderer::GetInstance()->DrawInfo(viewCamera, baseCamera, isRender);
}

///
/// Definition of the function to get the iterator
///
DEFINE_NODE_ITER_GETTER(Model)
DEFINE_NODE_ITER_GETTER(Light)
DEFINE_NODE_ITER_GETTER(Camera)
DEFINE_NODE_ITER_GETTER(Fog)
DEFINE_NODE_ITER_GETTER(SceneEnvironmentSetting)
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetResourceListener

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
IResourceListener*
Module3D::GetResourceListener()
{
    return mResListener;
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::SetResourceListener

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::SetResourceListener( IResourceListener* listener )
{
    mResListener = listener;
}
/*---------------------------------------------------------------------------*
  Name:         NodeIter::GetCurrentAnimEvaluatorsBegin

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::ut::MoveArray<nw::gfx::BaseAnimEvaluator*>::iterator
NodeIter::GetCurrentAnimEvaluatorsBegin()
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::AnimSetLinker *linker = m_pEntity->GetAnimLinker();

    if (NULL == linker)
    {
        return NULL;
    }

    return linker->GetEvaluatorsBegin();
}
/*---------------------------------------------------------------------------*
  Name:         NodeIter::GetCurrentAnimEvaluatorsBegin

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::ut::MoveArray<nw::gfx::BaseAnimEvaluator*>::iterator
NodeIter::GetCurrentAnimEvaluatorsEnd()
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::AnimSetLinker *linker = m_pEntity->GetAnimLinker();

    if (NULL == linker)
    {
        return NULL;
    }

    return linker->GetEvaluatorsEnd();
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetCurrentSceneEnvironmentSetting

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
SceneEnvironmentSettingIter
Module3D::GetCurrentSceneEnvironmentSetting()
{
    vw::g3d::SceneEnvironmentSettingElem* elem =
        ELM_SEL->GetTargetSceneEnvironmentSetting();

    return SceneEnvironmentSettingIter(elem);
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetDefaultCamera

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::Camera*
Module3D::GetDefaultCamera()
{
    return vw::g3d::SceneManager::GetInstance()->GetDefaultCamera()->GetSceneNode();
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetDefaultCamera

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::Camera*
Module3D::GetViewCamera()
{
    return vw::g3d::CameraMgr::GetInstance()->GetViewCamera();
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetDefaultCamera

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::Camera*
Module3D::GetLeftViewCamera()
{
    return vw::g3d::CameraMgr::GetInstance()->GetLeftViewCamera();
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetDefaultCamera

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::Camera*
Module3D::GetRightViewCamera()
{
    return vw::g3d::CameraMgr::GetInstance()->GetRightViewCamera();
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetCurrentSceneEnvironmentSetting

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::SceneEnvironment*
Module3D::GetSceneEnvironment()
{
    return &( vw::system::Video_GetRenderContext()->GetSceneEnvironment() );
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetSceneEnvironmentArraySize

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::GetSceneEnvironmentArraySize( int& cameraSize, int& lightSetSize, int& fogSize )
{
    cameraSize   = nw::gfx::RenderContext::DEFAULT_MAX_CAMERAS;
    lightSetSize = nw::gfx::RenderContext::DEFAULT_MAX_LIGHT_SETS;
    fogSize      = nw::gfx::RenderContext::DEFAULT_MAX_FOGS;
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetCameraIndex

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
int
Module3D::GetCameraIndex( nw::gfx::Camera* camera )
{
    return vw::g3d::SceneElemSelector::GetInstance()->GetCameraIndex( camera );
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::AddCamera

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::AddCamera( nw::gfx::Camera* camera )
{
    vw::g3d::SceneManager::GetInstance()->AddExternalCamera(camera);
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::AddLight

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::RemoveCamera( nw::gfx::Camera* camera )
{
    vw::g3d::SceneManager::GetInstance()->RemoveExternalCamera(camera);
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::AddLight

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::AddLight( nw::gfx::Light* light )
{
    vw::g3d::SceneManager::GetInstance()->AddExternalLight(light);
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::RemoveLight

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::RemoveLight( nw::gfx::Light* light )
{
    vw::g3d::SceneManager::GetInstance()->RemoveExternalLight(light);
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::RemoveLight

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::SetRootTransform( const nw::math::Transform3& transform )
{
    nw::gfx::CalculatedTransform& root =
        vw::g3d::SceneManager::GetInstance()->GetSceneRoot()->Transform();
    root.SetTransform( transform );
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::RemoveLight

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
Module3D::SetRootTransform( const math::VEC3& scale, const math::MTX34& rotateTranslate )
{
    nw::gfx::CalculatedTransform& root =
        vw::g3d::SceneManager::GetInstance()->GetSceneRoot()->Transform();
    root.SetScale( scale );
    root.SetTransformMatrix( rotateTranslate );
}
/*---------------------------------------------------------------------------*
  Name:         Module3D::GetRenderFlag

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
u32
Module3D::GetRenderFlag()
{
    u32 result = 0;
    s32 count = 0;
    bool renderFlag[ vw::g3d::SceneRenderer::RenderNum ];
    vw::g3d::SceneRenderer::GetInstance()->GetCorrectedRenderFlag( renderFlag );

    for( u32 i=1; i<vw::g3d::SceneRenderer::RenderNum; ++i )
    {
        if ( renderFlag[ i ] )
        {
            result |= ( 0x01 << i );
            count++;
        }
    }

    if ( count == ( vw::g3d::SceneRenderer::RenderNum - 1 ) )
    { result = nw::viewer::g3d::RenderAll; }

    return result;
}
/*---------------------------------------------------------------------------*
  Name:         NodeIter::NodeIter

  Description:  Constructor.

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
NodeIter::NodeIter(NodeEntity *pNode)
{
    m_pEntity = pNode;
}
/*---------------------------------------------------------------------------*
  Name:         NodeIter::SceneObject

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::SceneObject*
NodeIter::GetSceneObject() const
{
    return SCENEOBJECT(m_pEntity);
}
/*---------------------------------------------------------------------------*
  Name:         NodeIter::GetResSceneObject

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
nw::gfx::res::ResSceneObject
NodeIter::GetResSceneObject() const
{
    if (!IsValid())
    {
        return nw::gfx::res::ResSceneObject(NULL);
    }
    return SCENEOBJECT(m_pEntity)->GetResSceneObject();
}
/*---------------------------------------------------------------------------*
  Name:         ModelIter::GetNext

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
ModelIter
ModelIter::GetNext() const
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::SceneManager::ModelList::iterator iter = ++MODELELEMITER(m_pEntity);

    return iter == SCN_MGR->GetModelsEnd() ? NULL : &*iter;
}
/*---------------------------------------------------------------------------*
  Name:         LightIter::GetNext

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
LightIter
LightIter::GetNext() const
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::SceneManager::LightList::iterator iter = ++LIGHTELEMITER(m_pEntity);

    return iter == SCN_MGR->GetLightsEnd() ? NULL : &*iter;
}
/*---------------------------------------------------------------------------*
  Name:         CameraIter::GetNext

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
CameraIter
CameraIter::GetNext() const
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::SceneManager::CameraList::iterator iter = ++CAMERAELEMITER(m_pEntity);

    return iter == SCN_MGR->GetCamerasEnd() ? NULL : &*iter;
}
/*---------------------------------------------------------------------------*
  Name:         FogIter::GetNext

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
FogIter
FogIter::GetNext() const
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::SceneManager::FogList::iterator iter = ++FOGELELEMITER(m_pEntity);

    return iter == SCN_MGR->GetFogsEnd() ? NULL : &*iter;
}
/*---------------------------------------------------------------------------*
  Name:         SceneEnvironmentSettingIter::GetNext

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
SceneEnvironmentSettingIter
SceneEnvironmentSettingIter::GetNext() const
{
    if (!IsValid())
    {
        return NULL;
    }

    vw::g3d::SceneManager::SceneEnvironmentSettingList::iterator iter = ++SCENESETTINGELELEMITER(m_pEntity);

    return iter == SCN_MGR->GetSceneEnvironmentSettingsEnd() ? NULL : &*iter;
}
/*---------------------------------------------------------------------------*
  Name:         SceneEnvironmentSettingIter::Activate

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
SceneEnvironmentSettingIter::Activate() const
{
    if(!IsValid()) {
        ELM_SEL->ChangeToLastSceneEnvironmentSetting();
    } else {
        ELM_SEL->SetTargetSceneEnvironmentSetting( SCENESETTINGELEM(m_pEntity) );
    }
}
/*---------------------------------------------------------------------------*
  Name:         SceneEnvironmentSettingIter::Deactivate

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
void
SceneEnvironmentSettingIter::Deactivate()
{
    ELM_SEL->ChangeToLastSceneEnvironmentSetting();
}

#undef DEFINE_NODE_ITER_GETTER

} /* namespace g3d    */
} /* namespace viewer */
} /* namespace nw     */
