/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_G3D_IF_LISTENER_H_
#define NW4R_VIEWER_G3D_IF_LISTENER_H_

#include <dolphin.h>
#include <nw4r/g3d.h>
#include <nw4r/viewer/g3d/Module3D.h>
#include <viewer/system/STL.h>
#include "g3d_private.h"

namespace nw4r   {
namespace viewer {
namespace g3d    {

//==================================
// Resource listener
//==================================
class ResManager : public vw::g3d::ModuleG3dImpl::ReloadListener
{
public:
    /* ctor */          ResManager() : mListener( NULL ) {}
    void                SetListener( IResourceListener* listener ) { mListener = listener; }

    virtual void        ReloadMdl ( u32 modelID ) { if ( mListener ) { NotifyReloadMdl( modelID ); } }
    virtual void        ReloadScn ( u32 sceneID ) { if ( mListener ) { NotifyReloadScn( sceneID ); } }
    virtual void        ReloadTex ( u32 /* texID  */ ) {}
    virtual void        ReloadPltt( u32 /* plttID */ ) {}
    virtual void        ReloadAnm ( u32 modelID, u32 anmID, u8 type ) { if ( mListener ) { NotifyReloadAnm( modelID, anmID, type ); } }

    virtual void        RemoveMdl ( u32 modelID ) { if ( mListener ) { NotifyRemoveMdl( modelID ); } }
    virtual void        RemoveScn ( u32 sceneID ) { if ( mListener ) { NotifyRemoveScn( sceneID ); } }
    virtual void        RemoveTex ( u32 /* texID  */ ) {}
    virtual void        RemovePltt( u32 /* plttID */ ) {}
    virtual void        RemoveAnm ( u32 modelID, u32 anmID, u8 type ) { if ( mListener ) { NotifyRemoveAnm( modelID, anmID, type ); } }

    virtual void        UpdateMat( u32 modelID, u32 matID ) { if ( mListener ) { AppendUpdateMat( modelID, matID ); } }
    virtual void        FlushUpdateMat() { if ( mListener ) { NotifyUpdateMat(); } }

private:
    IResourceListener*   mListener;

    void                NotifyReloadMdl( u32 modelID );
    void                NotifyReloadScn( u32 sceneID );
    void                NotifyReloadAnm( u32 modelID, u32 anmID, u8 type );
    void                NotifyRemoveMdl( u32 modelID );
    void                NotifyRemoveScn( u32 sceneID );
    void                NotifyRemoveAnm( u32 modelID, u32 anmID, u8 type );

    void                AppendUpdateMat( u32 modelID, u32 matID );
    void                NotifyUpdateMat();

    typedef struct MatID
    {
        /* ctor */ MatID( u32 model, u32 mat ) : mdlID( model ), matID( mat ) {}
        u32 mdlID;
        u32 matID;
    } MatID;
    typedef vw::vector<MatID>::type MatIDVec;
    MatIDVec   mMatIDs;
};


} /* namespace g3d    */
} /* namespace viewer */
} /* namespace nw4r   */


/* NW4R_VIEWER_G3D_IF_LISTENER_H_ */
#endif
