/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

//----------------------------------------------------------
// Input registers map

#pragma     bind_symbol(    vVtxData,       v0,     v0 )
#define     vVtxData                        v0

// Input map
// { bufferIndex, vtxIndex, X, Y }
//----------------------------------------------------------

//----------------------------------------------------------
// Constant values

#define     cVal                            c5
def         cVal,           0.0,    1.0,    2.0,    3.0
//----------------------------------------------------------

//----------------------------------------------------------
// Uniform

#define                     uProjection     c0
#define                     uStereoCamera   c4
#define                     uUniformAddr    c6
#define                     uUniformMtx     c32
#define                     uUniformData    c64
#define                     uTexCoordMode   i0

//----------------------------------------------------------


//----------------------------------------------------------
// Output registers map

#pragma output_map(position,      o0)
#pragma output_map(color,         o1)
#pragma output_map(texture0,      o2.xy)
#pragma output_map(texture1,      o3.xy)
#pragma output_map(texture2,      o4.xy)
#define oPosition                 o0
#define oColor                    o1
#define oTexCoord0                o2
#define oTexCoord1                o3
#define oTexCoord2                o4
//----------------------------------------------------------

//----------------------------------------------------------
// Temporary registers

#define     rQuad       r0
#define     rPosition   r1
#define     rTransform  r2
#define     rTempView   r3
#define     rTempPos    r4
#define     rTexCoord0  r5
#define     rTexCoord1  r6
#define     rTexCoord2  r7
#define     rCounter    r8

//----------------------------------------------------------

main:
    // Get the square of what order
    mova    a0.x,           vVtxData

    // Save address information
    mov     rQuad,          uUniformAddr[ a0.x ].wzyx

    // Set the basic square's coordinates
    mov     rPosition.xy,   vVtxData.zwzw
    mov     rPosition.zw,   cVal.xyxy

    // Get the matrix data's address and the rectangle information's address
    mova    a0.xy,          rQuad.xy

    // Transform the square (local coordinates system)
    mov     rTransform.xw,  uUniformData[ a0.y ].wwyy
    mov     rTransform.yz,  cVal.xxxx
    dp4     rTempPos.x,     rPosition, rTransform
    mov     rTransform.yw,  uUniformData[ a0.y ].zzxx
    mov     rTransform.xz,  cVal.xxxx
    dp4     rTempPos.y,     rPosition,      rTransform
    mov     rTempPos.zw,    rPosition.zwzw

    // Convert the square to screen coordinates system
    dp4     rTempView.x,    rTempPos,       uUniformMtx[ a0.x + 0 ].wzyx
    dp4     rTempView.y,    rTempPos,       uUniformMtx[ a0.x + 1 ].wzyx
    dp4     rTempView.z,    rTempPos,       uUniformMtx[ a0.x + 2 ].wzyx
    mov     rTempView.w,    rPosition.w

    mov     rTempPos.z,     uUniformMtx[ a0.x + 2 ].x           // temp z = z
    abs     rTempPos.z,     rTempPos.z                          // temp z = abs(temp z)
    add     rTempPos.z,     rTempPos.z,     uStereoCamera.y     // temp z += add dist

    // Determine whether to apply a parallax
    mov     rTempPos.x,     uStereoCamera.w
    cmp     1, 1, rTempPos.xz, cVal.xx          // 1: !=     half parallax != 0, z != 0 ?
    ifc     1, 1, 1                             // 1: AND

    mov     rTempPos.x,     uStereoCamera.w                             // temp x = half parallax
    sub     rTempPos.y,     rTempPos.z,         uStereoCamera.z         // temp y = temp z - Dlevel
    rcp     rTempPos.z,     rTempPos.z                                  // temp z = 1 / temp z
    mul     rTempPos.z,     rTempPos.y,         rTempPos.z              // temp z *= temp y
    mad     rTempView.x,    rTempPos.x,         rTempPos.z,         rTempView.x     // x = temp x * temp z + x

    endif

    dp4     oPosition.x,    rTempView,      uProjection[ 0 ].wzyx
    dp4     oPosition.y,    rTempView,      uProjection[ 1 ].wzyx
    dp4     oPosition.z,    rTempView,      uProjection[ 2 ].wzyx
    dp4     oPosition.w,    rTempView,      uProjection[ 3 ].wzyx

    // Get whether there is vertex color information
    cmp   3, 3,     rQuad.w, cVal.y

    ifc   1, 1, 1
    // No vertex color setting.

    // Set the vertex color to the default
    mov   oColor.xyz, cVal.y

    // Set the alpha
    mov   oColor.w,   rQuad.w

    else
    // Vertex color setting available.

    // Get the vertex color information address
    mova    a0.y,           rQuad.w

    // Determine up/down/left/right for the currently processing vertex
    cmp     1,  1,          cVal.xx,    rPosition.xy

    ifc     0,  0,  1       // upper left
    mov     oColor,         uUniformMtx[ a0.y + 0 ].wzyx
    endif
    ifc     1,  0,  1       // upper right
    mov     oColor,         uUniformMtx[ a0.y + 1 ].wzyx
    endif
    ifc     0,  1,  1       // lower left
    mov     oColor,         uUniformMtx[ a0.y + 2 ].wzyx
    endif
    ifc     1,  1,  1       // lower right
    mov     oColor,         uUniformMtx[ a0.y + 3 ].wzyx
    endif
    nop
    endif

    // Texture coordinates initialization
    cmp     1,  1,  cVal.xx,    rPosition.xy
    ifc     0,  0,  1           // upper left
    mov     rTexCoord0,         cVal.xyyy
    mov     rTexCoord1,         cVal.xyyy
    mov     rTexCoord2,         cVal.xyyy
    endif
    ifc     1,  0,  1           // upper right
    mov     rTexCoord0,         cVal.yyyy
    mov     rTexCoord1,         cVal.yyyy
    mov     rTexCoord2,         cVal.yyyy
    endif
    ifc     0,  1,  1           // lower left
    mov     rTexCoord0,         cVal.xxyy
    mov     rTexCoord1,         cVal.xxyy
    mov     rTexCoord2,         cVal.xxyy
    endif
    ifc     1,  1,  1           // lower right
    mov     rTexCoord0,         cVal.yxyy
    mov     rTexCoord1,         cVal.yxyy
    mov     rTexCoord2,         cVal.yxyy
    endif

    // Texture coordinate settings
    mov     rCounter,           cVal.x
    loop    uTexCoordMode

    cmp     0,  0,  cVal.y,     rCounter
    ifc     1,  1,  1

    // Get the texture coordinates' address
    mova    a0.y,               rQuad.z
    cmp     1,  1,  cVal.xx,    rPosition.xy
    ifc     0,  0,  1           // upper left
    mov     rTexCoord0.xy,      uUniformData[ a0.y + 0 ].wz
    mov     rTexCoord1.xy,      uUniformData[ a0.y + 1 ].wz
    mov     rTexCoord2.xy,      uUniformData[ a0.y + 2 ].wz
    endif
    ifc     1,  0,  1           // upper right
    mov     rTexCoord0.xy,      uUniformData[ a0.y + 0 ].yz
    mov     rTexCoord1.xy,      uUniformData[ a0.y + 1 ].yz
    mov     rTexCoord2.xy,      uUniformData[ a0.y + 2 ].yz
    endif
    ifc     0,  1,  1           // lower left
    mov     rTexCoord0.xy,      uUniformData[ a0.y + 0 ].wx
    mov     rTexCoord1.xy,      uUniformData[ a0.y + 1 ].wx
    mov     rTexCoord2.xy,      uUniformData[ a0.y + 2 ].wx
    endif
    ifc     1,  1,  1           // lower right
    mov     rTexCoord0.xy,      uUniformData[ a0.y + 0 ].yx
    mov     rTexCoord1.xy,      uUniformData[ a0.y + 1 ].yx
    mov     rTexCoord2.xy,      uUniformData[ a0.y + 2 ].yx
    endif
    nop
    endif

    // The upper right and lower left coordinates come in only when the texture matrix is rotating
    cmp     2,  2,  cVal.w,     rCounter
    ifc     1,  1,  1

    cmp     1,  1,  cVal.xx,    rPosition.xy
    ifc     1,  0,  1           // Overwrite upper right
    mov     rTexCoord0.xy,      uUniformData[ a0.y + 3 ].yx
    mov     rTexCoord1.xy,      uUniformData[ a0.y + 4 ].yx
    mov     rTexCoord2.xy,      uUniformData[ a0.y + 5 ].yx
    endif
    ifc     0,  1,  1           // Overwrite lower left
    mov     rTexCoord0.xy,      uUniformData[ a0.y + 3 ].wz
    mov     rTexCoord1.xy,      uUniformData[ a0.y + 4 ].wz
    mov     rTexCoord2.xy,      uUniformData[ a0.y + 5 ].wz
    endif
    nop
    endif

    add     rCounter,   rCounter,   cVal.y
    endloop

    mov     oTexCoord0,         rTexCoord0
    mov     oTexCoord1,         rTexCoord1
    mov     oTexCoord2,         rTexCoord2

    end
endmain:
