/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_SYSTEM_CLEAR_IMAGE_H_
#define NW4R_VIEWER_SYSTEM_CLEAR_IMAGE_H_

#include <viewer.h>

namespace vw     {
namespace system {


//==========================================
//
// Class for rendering clear images
//
//==========================================
const int FILE_PATH_MAX = 260;

class ClearImage
{
public:
    static ClearImage&  GetInstance();

    static void Initialize();
    static void Finalize();

    bool                IsValid() const { return ( m_TexImg != NULL ); }
    void                ReadImg( void* data, u32 dataSize = 0 );
    void                Destroy();

    void                Draw(u32 width, u32 height);

    enum
    {
        POS_STRETCH,
        POS_TILE,
        POS_CENTER
    };

    static u32          GetPosition()          { return sPosition; }
    static void         SetPosition( u32 pos ) { sPosition = pos;  }
    static void         SetLatestPath( const char* path );
    static const char*  GetLatestPath();

private:

    /* ctor */          ClearImage();
    /* dtor */          ~ClearImage( void );
    void DrawImage(u32 width, u32 height);

    static ClearImage*  sInstance; /* = NULL */
    static u32          sPosition; /* = POS_STRETCH */
    static char         sLatestPath[ FILE_PATH_MAX ];

    u8*         m_TexImg;      // Pointer to the texture image

    u16         mImgWidth;    // Image width
    u16         mImgHeight;   // Image size
    u16         mTexWidth;    // Texture width
    u16         mTexHeight;   // Texture height

    vw::system::Texture* m_Texture;
};


/*---------------------------------------------------------------------------*
  Name:         GetClearImage

  Description:  Gets instance of the clear image class

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
inline ClearImage&
GetClearImage()
{
    return ClearImage::GetInstance();
}


}   /* namespace system */
}   /* namespace vw     */



/* NW4C_VIEWER_SYSTEM_IMAGE_FMT_H_ */
#endif
