/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_HID_MANAGER_H_
#define NW4C_VIEWER_G3D_HID_MANAGER_H_

#include <viewer/system/Setting.h>
#include <viewer/system/hid.h>

namespace vw {
namespace system {

// Association of pad input and processing
struct PadBinding
{
    s32 enumurate;
    u32 bit;
};

struct PadBindings
{
    PadBinding *binding;
    s32 size;
};

enum SystemPadOperations
{
    SYS_SWITCH_MODE_PREV = 0,
    SYS_SWITCH_MODE_NEXT,
    SYS_NONE = -1
};

enum G3dPadOperations
{
    G3D_RELOAD = 0,
    G3D_SLOW_DOWN_ANIMATION,
    G3D_SLOW_DOWN_ALL_ANIMATIONS,
    G3D_SPEED_UP_ANIMATION,
    G3D_SPEED_UP_ALL_ANIMATIONS,
    G3D_INCREMENT_ANIMATION_FRAME,
    G3D_INCREMENT_ALL_ANIMATION_FRAMES,
    G3D_DECREMENT_ANIMATION_FRAME,
    G3D_DECREMENT_ALL_ANIMATION_FRAMES,
    G3D_STOP_ANIMATION,
    G3D_STOP_ALL_ANIMATIONS,
    G3D_START_ANIMATION,
    G3D_START_ALL_ANIMATIONS,
    G3D_SWITCH_PREV_VIEW_CAMERA,
    G3D_SWITCH_NEXT_VIEW_CAMERA,
    G3D_SWITCH_PREV_CONTROL_TARGET_TYPE,
    G3D_SWITCH_NEXT_CONTROL_TARGET_TYPE,
    G3D_SWITCH_PREV_CONTROL_TARGET,
    G3D_SWITCH_NEXT_CONTROL_TARGET,
    G3D_SWITCH_PREV_CONTROL_TARGET_SEQUENTIALLY,
    G3D_SWITCH_NEXT_CONTROL_TARGET_SEQUENTIALLY,
    G3D_SWITCH_PREV_RESOURCE_TYPE,
    G3D_SWITCH_NEXT_RESOURCE_TYPE,
    G3D_SWITCH_PREV_RESOURCE,
    G3D_SWITCH_NEXT_RESOURCE,
    G3D_SWITCH_PREV_RESOURCE_SEQUENTIALLY,
    G3D_SWITCH_NEXT_RESOURCE_SEQUENTIALLY,
    G3D_SWITCH_PREV_ANIMATION,
    G3D_SWITCH_NEXT_ANIMATION,
    G3D_SWITCH_PREV_SCENE,
    G3D_SWITCH_NEXT_SCENE,
    G3D_SWITCH_PREV_SELECTION,
    G3D_SWITCH_NEXT_SELECTION,
    G3D_SWITCH_PREV_SELECTION_ITEM_MAIN,
    G3D_SWITCH_NEXT_SELECTION_ITEM_MAIN,
    G3D_SWITCH_PREV_SELECTION_ITEM_SUB,
    G3D_SWITCH_NEXT_SELECTION_ITEM_SUB,
    G3D_CHANGE_INFO_RENDER_STATUS,
    G3D_TOGGLE_ONETIME_LOOP,
    G3D_NONE = -1
};

enum EffectPadOperations
{
    EF_SWITCH_VIEW_CAMERA = 0,
    EF_SLOW_DOWN_ALL_ANIMATIONS,
    EF_SPEED_UP_ALL_ANIMATIONS,
    EF_INCREMENT_ALL_ANIMATION_FRAMES,
    EF_STOP_ALL_ANIMATIONS,
    EF_START_ALL_ANIMATIONS,
    EF_INCREMENT_MAX_FRAME,
    EF_DECREMENT_MAX_FRAME,
    EF_CHANGE_ANIMATION_LOOP_STATUS,
    EF_CHANGE_INFO_RENDER_STATUS,
    EF_NONE = -1
};

enum SubModePadOperations
{
    SUB_SWITCH_PREV_SUBMODE = 0,
    SUB_SWITCH_NEXT_SUBMODE,
    SUB_NONE = -1
};

enum MenuSubModePadOperations
{
    MNU_MENU_APPLY = 0,
    MNU_SWITCH_PREV_MENU,
    MNU_SWITCH_NEXT_MENU,
    MNU_SWITCH_PREV_MENU_ITEM,
    MNU_SWITCH_NEXT_MENU_ITEM,
    MNU_NONE = -1
};

enum LogSubModePadOperations
{
    LOG_EXPORT = 0,
    LOG_CLEAR,
    LOG_SWITCH_PREV_LOG,
    LOG_SWITCH_NEXT_LOG,
    LOG_SWITCH_PREV_LOG_ITEM,
    LOG_SWITCH_NEXT_LOG_ITEM,
    LOG_NONE = -1
};

// Returns array with correspondence between pad input and processing
const PadBindings* GetSystemPadBindings();
const PadBindings* GetG3dPadBindings();
const PadBindings* GetEffectPadBindings();
const PadBindings* GetSubModePadBindings();
const PadBindings* GetMenuSubModePadBindings();
const PadBindings* GetLogSubModePadBindings();

// Checks the received PadBinding, and returns the one that matches the Pad input
s32 GetPadEvent( vw::system::HidStatus &hidStatus, const PadBindings *bindings );

}
}

#endif
