/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_IMAGE_READER_H_
#define NW4C_VIEWER_SYSTEM_IMAGE_READER_H_

#include <viewer.h>

namespace vw     {
namespace system {

class ImageReaderImpl
{
public:
    /* ctor */          ImageReaderImpl() {}
    /* dtor */ virtual  ~ImageReaderImpl() {}

    virtual bool        IsValid()   = 0;    // Check whether valid data is stored
    virtual u32         GetWidth()  = 0;    // Get the image width
    virtual u32         GetHeight() = 0;    // Get the image height
    virtual bool        Expand( void* /* expBuf */, u32 /* bufWidth */, u32 /* bufHeight */ ) = 0; // Expand to texture format
    virtual u32         GetType()   = 0;    // Get the image type
    virtual void        Dispose()   = 0;    // Free allocated memory for internal processing.

    enum
    {
        TYPE_UNKNOWN  =  0x000000,
        TYPE_BMP      =  0x010000,
        TYPE_TGA      =  0x020000,
        TYPE_JPEG     =  0x030000,
        TYPE_OPT_MASK =  0x00FFFF,
        TYPE_MASK     = ~0x00FFFF
    };
};


//-----------------------------
// Image read class
//
//-----------------------------
class ImageReader
{
//--- Public members --------------
public:
    /* ctor */          ImageReader( void* data, u32 dataSize = 0 );
    /* dtor */          ~ImageReader( void );

    u32                 GetWidth()  const { return mWidth;  }
    u32                 GetHeight() const { return mHeight; }
    bool                IsValid()   const { return (mReaderImpl)? mReaderImpl->IsValid() : false; }
    bool                Expand( void* expBuf, u32 bufWidth, u32 bufHeight );

//--- Private members--------------
private:

    ImageReaderImpl*    mReaderImpl;
    void*               mBuf;       // Pointer to the data buffer
    u32                 mType;      // Data type.
    u32                 mWidth;     // Image width
    u32                 mHeight;    // Image height
};



}   /* namespace system */
}   /* namespace vw     */



/* NW4C_VIEWER_SYSTEM_IMAGE_READER_H_ */
#endif
