/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_SYSTEM_JPEG_DATA_H_
#define NW4R_VIEWER_SYSTEM_JPEG_DATA_H_

#include "ImageFmt.h"

namespace vw     {
namespace system {


class JpegData : public ImageFmt
{
//--- Public members --------------
public:
    static const u32 DEFAULT_QUALITY = 250;
    /* ctor */          JpegData( u32 width, u32 height, u32 quality = DEFAULT_QUALITY );
    /* dtor */ virtual  ~JpegData();

    virtual u32         GetBodySize() const { return mBodySize; }

    virtual u8*         GetHeader() { return NULL; }
    virtual u32         GetHeaderSize() const { return 0; }

    virtual void        ConvertData( void );    // Convert RGBA data in buffer to match format.
    virtual void        ConvertData( void* src_image, u32 src_width, u32 src_height, u32 offset_x, u32 offset_y );
    virtual void        ConvertData( void* src_image,
                                     u32 src_width, u32 src_height,
                                     u32 dst_ofs_x, u32 dst_ofs_y,
                                     u32 src_ofs_x, u32 src_ofs_y,
                                     u32 width,     u32 height );
    virtual void        Finalize( void );

//--- Private members -------------
private:
    void        ConvertDataImpl_();    // Convert RGBA data in buffer to match format.
    void        ConvertDataImpl_( void* src_image,
                                  u32 src_width, u32 src_height,
                                  u32 dst_ofs_x, u32 dst_ofs_y,
                                  u32 src_ofs_x, u32 src_ofs_y,
                                  u32 width, u32 height );
    void        CompressDataImpl_();      // Data compression

    u32 mQuality;
    u32 mBodySize;  // Data size
};


}   /* namespace system */
}   /* namespace vw     */



/* NW4R_VIEWER_SYSTEM_JPEG_DATA_H_ */
#endif
