/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_TEXUTIL_H_
#define NW4C_VIEWER_SYSTEM_TEXUTIL_H_

namespace vw     {
namespace system {

class UtilRGBA8
{
public:
    static nw::ut::Color8 GetColor( const u8* buf, u32 x, u32 y, u32 width );
    static nw::ut::Color8 GetRGBA( const u8* buf, u32 x, u32 y, u32 width );
    static void    SetColor( nw::ut::Color8 col, u8* buf, u32 x, u32 y, u32 width, u32 height );

private:
    /* ctor */ UtilRGBA8() {}

    static inline u32
    GetAddress( u32 x, u32 y, u32 width )
    {
        u32 blockX = x * 4;
        u32 blockY = y * 4;
        u32 addr = blockX + blockY * width;
        return addr;
    }
    static inline u32
    GetAddress( u32 x, u32 y, u32 width, u32 height )
    {
        y = height - y - 1;
        u32 blockX = x * 4;
        u32 blockY = y * 4;
        u32 addr = blockX + blockY * width;
        return addr;
    }
};

/*---------------------------------------------------------------------------*
  Name:         UtilRGBA8::GetColor

  Description:  Gets color information from the RGBA8 texture

  Arguments:    buf     Texture buffer
                x       X-coordinate
                y       Y-coordinate
                width   Texture width

  Returns:      Texture color
 *---------------------------------------------------------------------------*/
/* static */ inline nw::ut::Color8
UtilRGBA8::GetColor( const u8* buf, u32 x, u32 y, u32 width )
{
    u32 addr = UtilRGBA8::GetAddress( x, y, width );
    nw::ut::Color8 result;

    result.a = buf[ addr     ];
    result.r = buf[ addr + 1 ];
    result.g = buf[ addr + 2 ];
    result.b = buf[ addr + 3 ];
    return result;
}

/*---------------------------------------------------------------------------*
  Name:         UtilRGBA8::GetRGBA

  Description:  Gets color information from the RGBA8 texture

  Arguments:    buf     Texture buffer
                x       X-coordinate
                y       Y-coordinate
                width   Texture width

  Returns:      Texture color
 *---------------------------------------------------------------------------*/
/* static */ inline nw::ut::Color8
UtilRGBA8::GetRGBA( const u8* buf, u32 x, u32 y, u32 width )
{
    u32 addr = UtilRGBA8::GetAddress( x, y, width );
    nw::ut::Color8 result;

    result.r = buf[ addr + 0 ];
    result.g = buf[ addr + 1 ];
    result.b = buf[ addr + 2 ];
    result.a = buf[ addr + 3 ];
    return result;
}

/*---------------------------------------------------------------------------*
  Name:         UtilRGBA8::SetColor

  Description:  Sets color to the RGBA8 texture

  Arguments:    col     Color to set
                buf     Texture buffer
                x       X-coordinate
                y       Y-coordinate
                width   Texture width

  Returns:      None.
 *---------------------------------------------------------------------------*/
/* static */ inline void
UtilRGBA8::SetColor( nw::ut::Color8 col, u8* buf, u32 x, u32 y, u32 width, u32 height )
{
    u32 addr = UtilRGBA8::GetAddress( x, y, width, height );

    buf[ addr     ] = col.r;
    buf[ addr + 1 ] = col.g;
    buf[ addr + 2 ] = col.b;
    buf[ addr + 3 ] = col.a;
}

}   /* namespace system */
}   /* namespace vw  */


/* NW4C_VIEWER_SYSTEM_TEXUTIL_H_ */
#endif
