/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>



namespace vw     {
namespace system {

/*---------------------------------------------------------------------------*
  Name:         LoadFile_FromDVD
  Description:  Loads file from DVD
  Arguments:    path Path name to model data
  Returns:      Pointer to the loaded data
 *---------------------------------------------------------------------------*/
//void*
//LoadFile_FromDVD( u16 moduleID, const char* path, s32* pSize, nw::os::IAllocator* allocator /* = NULL */ )
//{
//    // Exit as-is if cannot access DVD
//    if ( ! system::App::GetInstance().IsEnableDVD() )
//    {
//        *pSize = LOAD_FILE_RESULT_NOT_FOUND;
//        return NULL;
//    }
//    DVDFileInfo fileInfo;
//
//    if ( ! DVDOpen( path, &fileInfo ) )
//    // End if file does not exist
//    {
//        if ( pSize ) { *pSize = LOAD_FILE_RESULT_NOT_FOUND; }
//        return NULL;
//    }
//
//    u32   length = DVDGetLength( &fileInfo );
//    void* buf;
//    if ( allocator )
//    {
//        buf = ::MEMAllocFromAllocator( allocator, OSRoundUp32B( length ) );
//    }
//    else
//    {
//        buf = system::Memory_Alloc( moduleID, OSRoundUp32B( length ) );
//    }
//
//    if ( buf == NULL )
//    // End if memory cannot be allocated
//    {
//        if ( pSize ) { *pSize = LOAD_FILE_RESULT_MEM_ALLOC_ERROR; }
//        return NULL;
//    }
//
//    s32 result = DVDRead( &fileInfo, buf, (s32)OSRoundUp32B( length ), 0 );
//    (void)DVDClose( &fileInfo );
//
//    if ( result <= 0 )
//    // When there was a DVD read error
//    {
//        if ( pSize ) { *pSize = LOAD_FILE_RESULT_TRANSFER_ERROR; }
//        system::Memory_Free( buf );
//        return NULL;
//    }
//
//    if ( pSize ) { *pSize = s32(length); }
//    return buf;
//}


/*---------------------------------------------------------------------------*
  Name:         LoadFile_FromMCS
  Description:  Loads file from MCS
  Arguments:    path Path name to model data
  Returns:      Pointer to the loaded data
 *---------------------------------------------------------------------------*/
void*
LoadFile_FromHIO( u16 moduleID, const char* path, s32* pSize, nw::os::IAllocator* allocator /* = NULL */ )
{
    const int FILE_PATH_MAX = 260;
    if ( std::strlen( path ) > FILE_PATH_MAX )
    // Cannot load when the filename exceeds the max
    {
        if ( pSize ) { *pSize = LOAD_FILE_RESULT_NOT_FOUND; }
        return NULL;
    }

    bool IsRunning = false;
    nn::Result ret = nn::hio::GetHioDaemonStatus( &IsRunning );
    if(ret.IsSuccess() && IsRunning == false)
    {
        return NULL;
    }

    // Opens files
    nn::hio::CTR::HostFile hostFile;
    nn::Result result = hostFile.Open(
        path,
        nn::hio::CTR::HostFile::ACCESS_MODE_READ,
        nn::hio::CTR::HostFile::OPEN_DISP_OPEN_EXISTING);
    if (result.IsFailure())
    {
        if ( pSize ) { *pSize = LOAD_FILE_RESULT_NOT_FOUND; }
        return NULL;
    }
    // Get the file size
    s64 filesize;
    hostFile.Seek(&filesize,0,nn::hio::CTR::HostFile::SEEK_TYPE_END);
    hostFile.Seek(0,nn::hio::CTR::HostFile::SEEK_TYPE_SET);
    if ( filesize == 0 )
    {
        if ( pSize ) { *pSize = LOAD_FILE_RESULT_NOT_FOUND; }
        return NULL;
    }

    // Allocate a buffer for file loading
    void* buf;
    if ( allocator )
    {
        buf = allocator->Alloc( filesize );
    }
    else
    {
        buf = system::Memory_Alloc( moduleID, filesize );
    }
    if ( ! buf )
    {
        if ( pSize ) { *pSize = LOAD_FILE_RESULT_MEM_ALLOC_ERROR; }
        return NULL;
    }

    // Load from file
    size_t rSize = 0;
    result = hostFile.Read(&rSize,buf,(s32)filesize);
    if( result.IsFailure() )
    {
        system::Memory_Free( buf );
        if ( pSize ) { *pSize = LOAD_FILE_RESULT_TRANSFER_ERROR; }
        return NULL;
    }
    hostFile.Close();

    if ( pSize ) { *pSize = rSize; }
    return buf;
}



}   /* namespace system */
}   /* namespace vw     */


