/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "viewer.h"
#include "../include/PerformDrawer.h"



namespace vw     {
namespace system {



PerformDrawer* PerformDrawer::GetInstance()
{
    static PerformDrawer Instance;
    return &Instance;
}

PerformDrawer::PerformDrawer()
{
}

void
PerformDrawer::DrawGraph()
{
    vw::system::Draw2DHelper& draw2D = vw::system::Draw2DHelper::GetInstance();
    draw2D.BeginDraw();
    draw2D.Disable( vw::system::Draw2DHelper::ALPHABLEND | vw::system::Draw2DHelper::DEPTHTEST );

    // Processing load flag display
    const f32 MILLI_TO_RATE = 1.0f / (1000.0f / 60.0f);
    f32 CpuTime  = PerformMgr::GetInstance()->GetCpuTime()    * MILLI_TO_RATE;
    f32 CalcTime = PerformMgr::GetInstance()->GetCalcTime()   * MILLI_TO_RATE;
    f32 DrawTime = PerformMgr::GetInstance()->GetDrawTime()   * MILLI_TO_RATE;
    f32 GpuTime  = PerformMgr::GetInstance()->GetGpuTime()    * MILLI_TO_RATE;
    f32 AllTime  = PerformMgr::GetInstance()->GetSystemTime() * MILLI_TO_RATE;

    s32 w = 5;
    draw2D.SetLineWidth(1);
    draw2D.DrawRectangle(w,5,320-w*2,14,nw::ut::Color8(~0));

    const nw::ut::Color8 calcColor = nw::ut::Color8(255,0,0,255);
    const nw::ut::Color8 drawColor = nw::ut::Color8(0,0,255,255);
    const nw::ut::Color8  gpuColor = nw::ut::Color8(0,255,0,255);
    const nw::ut::Color8  allColor = nw::ut::Color8(32,32,32,255);

    // calc bar
    f32 barW  = 305.0f;
    s32 calcW = barW * CalcTime / 2.0f;
    s32 drawW = barW * DrawTime / 2.0f;
    s32 gpuW  = barW *  GpuTime / 2.0f;
    s32 allW  = barW *  AllTime;

    // all bar
    draw2D.FillRectangle(w+3,8,allW,4,allColor);

    // calc bar
    draw2D.FillRectangle(w+3,8,calcW,4,calcColor);
    // draw bar
    draw2D.FillRectangle(w+3+calcW,8, drawW,4,drawColor);
    // gpu bar
    draw2D.FillRectangle(w+3+calcW+drawW,14,gpuW,4,gpuColor);

    draw2D.EndDraw();
}
void
PerformDrawer::DrawMeter()
{
    vw::system::FontDrawer& font = vw::system::FontDrawer::GetInstance();
    font.BeginDraw();

    const f32 MILLI_TO_RATE = 1.0f / (1000.0f / 60.0f) * 100.0f;
    font.DrawString(214,20,"CPU :%6.2f%%\n", PerformMgr::GetInstance()->GetMaxCpuTime() *MILLI_TO_RATE);
    font.DrawString(214,40,"Calc:%6.2f%%\n", PerformMgr::GetInstance()->GetMaxCalcTime()*MILLI_TO_RATE);
    font.DrawString(214,60,"Draw:%6.2f%%\n", PerformMgr::GetInstance()->GetMaxDrawTime()*MILLI_TO_RATE);
    font.DrawString(214,80,"GPU :%6.2f%%\n", PerformMgr::GetInstance()->GetMaxGpuTime() *MILLI_TO_RATE);

    font.EndDraw();
}




}   /* namespace system */
}   /* namespace vw     */


