/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include <viewer/common.h>
#include <version.h>
#include "../include/system_private.h"
#include "../include/LogUtil.h"

using namespace nw;

namespace vw     {
namespace system {

const u32 TITLE_LEN_MAX = 64;

//static s32 sFrame = 0;
static char sTitle[TITLE_LEN_MAX] = "";


/*---------------------------------------------------------------------------*
  Name:         Version_SetTitle

  Description:

  Arguments:    title

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
Version_SetTitle( const char* fmt, ... )
{
    std::va_list vlist;
    va_start( vlist, fmt );

    Version_VSetTitle( fmt, vlist );

    va_end( vlist );
}

/*---------------------------------------------------------------------------*
  Name:         Version_VSetTitle

  Description:

  Arguments:    fmt
                args

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
Version_VSetTitle( const char* fmt, std::va_list args )
{
    std::vsnprintf( sTitle, TITLE_LEN_MAX, fmt, args );
}


/*---------------------------------------------------------------------------*
  Name:         Version_BeginShow

  Description:  Trigger to start display of version information

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
Version_BeginShow( void )
{
#if 0
    const s32 SHOW_MAX_FRAME = 100;

    sFrame = SHOW_MAX_FRAME;
#endif
}

/*---------------------------------------------------------------------------*
  Name:         Version_Draw

  Description:  Version Information render process

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
Version_Draw( void )
{
    vw::system::Draw2DHelper& draw2D = vw::system::Draw2DHelper::GetInstance();
    draw2D.BeginDraw();

    s32 aw = 240;
    s32 ah = 25;
    s32 w = 320 - (aw + 5);
    s32 h = 29;
    draw2D.FillRectangle(aw,ah,w,h, nw::ut::Color8(64,64,96,255));
    draw2D.EndDraw();

    // draw version
    vw::system::FontDrawer& font = vw::system::FontDrawer::GetInstance();
    font.BeginDraw();
    font.SetFontSize(8,12);
    font.SetFixedWidth(5.5f);

    if ( sTitle[ 0 ] )
    {
#if defined( NN_PCGX )
        font.DrawString(aw + 10, ah + 7, "%s", sTitle, NULL );
#else
        font.DrawString(aw + 10, ah + 7, "%s", sTitle );
#endif
    }
    else
    {
        font.DrawString(aw + 10, ah + 7, NW4C_VIEWER_VERSION_FORMAT);
    }
    font.DrawString("\n");
    font.EndDraw();
}


/*---------------------------------------------------------------------------*
  Name:         Version_PrintVersion

  Description:  Outputs version information to text writer

  Arguments:    writer

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
Version_Print( vw::system::FontDrawer* writer /* = NULL */ )
{
    if ( writer )
    {
        if ( sTitle[ 0 ] )
        {
            writer->DrawString( sTitle );
        }
        else
        {
            writer->DrawString( NW4C_VIEWER_VERSION_FORMAT );
        }
    }
    else
    {
        if ( sTitle[ 0 ] )
        {
            NW_VIEWER_DEV_LOG( sTitle );
        }
        else
        {
            NW_VIEWER_DEV_LOG( NW4C_VIEWER_VERSION_FORMAT );
        }
    }
}

}   /* namespace system */
}   /* namespace viewer */

