/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nw/viewer/sys/Heap.h>
#include <viewer/system.h>

namespace nw     {
namespace viewer {
namespace sys    {



// Allocates memory of the specified size.
void*
Heap::Memory_Alloc( u32 size, u32 align )
{
    return vw::system::Memory_AllocSys(size, align);
}

// Allocates device memory of the specified size.
void*
Heap::Memory_AllocDevice( u32 size, u32 align )
{
    return vw::system::Memory_AllocSysDevice(size, align);
}

// Returns memory obtained from the heap.
void
Heap::Memory_Free( void* addr )
{
    vw::system::Memory_Free( addr );
}
void
Heap::Memory_Free( const void* addr )
{
    vw::system::Memory_Free( addr );
}

// Checks the amount of heap memory currently available.
u32
Heap::Memory_FreeSize( void )
{
    return vw::system::Memory_FreeSize();
}

// Gets a pointer to the allocator.
nw::os::IAllocator*
Heap::Memory_GetAllocator()
{
    return vw::system::Memory_GetSysAllocator();
}

// Gets a pointer to the allocator for device memory.
nw::os::IAllocator*
Heap::Memory_GetAllocatorDevice()
{
    return vw::system::Memory_GetSysAllocatorDevice();
}




/*---------------------------------------------------------------------------*
  Name:         Heap::Heap

  Description:  Constructor

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
/* ctor */
Heap::Heap()
{
}

///*---------------------------------------------------------------------------*
//  Name:         Heap::AllocMEM1
//
//  Description:  Allocate memory from MEM1
//
//  Arguments:    size    memory size to allocate
//                align   Alignment
//
//  Returns:      Returns a pointer to the allocated memory.
//                If unsuccessful, returns NULL
// *---------------------------------------------------------------------------*/
//void*
//Heap::AllocMEM1( u32 size, s32 align /* = HEAP_DEFAULT_ALIGN */ )
//{
//    return vw::system::Memory_TryAllocMEM1( vw::system::USER_ID, size, align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::AllocMEM2
//
//  Description:  Allocate memory from MEM2
//
//  Arguments:    size    memory size to allocate
//                align   Alignment
//
//  Returns:      Returns a pointer to the allocated memory.
//                If unsuccessful, returns NULL
// *---------------------------------------------------------------------------*/
//void*
//Heap::AllocMEM2( u32 size, s32 align /* = HEAP_DEFAULT_ALIGN */ )
//{
//    return vw::system::Memory_TryAllocMEM2( vw::system::USER_ID, size, align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::Alloc1
//
//  Description:  Allocate memory, giving priority to allocating from MEM1
//
//  Arguments:    size    memory size to allocate
//                align   Alignment
//
//  Returns:      Returns a pointer to the allocated memory.
//                If unsuccessful, returns NULL
// *---------------------------------------------------------------------------*/
//void*
//Heap::Alloc1( u32 size, s32 align /* = HEAP_DEFAULT_ALIGN */ )
//{
//    return vw::system::Memory_TryAlloc( vw::system::USER_ID, size, align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::Alloc2
//
//  Description:  Allocate memory, giving priority to allocating from MEM2
//
//  Arguments:    size    memory size to allocate
//                align   Alignment
//
//  Returns:      Returns a pointer to the allocated memory.
//                If unsuccessful, returns NULL
// *---------------------------------------------------------------------------*/
//void*
//Heap::Alloc2( u32 size, s32 align /* = HEAP_DEFAULT_ALIGN */ )
//{
//    return vw::system::Memory_TryAlloc2( vw::system::USER_ID, size, align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::Realloc
//
//  Description:  Reallocate memory
//
//  Arguments:    addr    Address of original data
//                size    memory size to allocate
//                align   Alignment
//
//  Returns:      Returns a pointer to the allocated memory.
//                If unsuccessful, returns NULL
// *---------------------------------------------------------------------------*/
//void*
//Heap::Realloc( void* addr, u32 size, s32 align /* = HEAP_DEFAULT_ALIGN */ )
//{
//    return vw::system::Memory_TryRealloc( addr, size, align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::ResizeMBlock
//
//  Description:  Resize allocated memory
//
//  Arguments:    addr    Pointer to data block
//                size    Data size to resize
//
//  Returns:      Returns the data size after resizing
//                Returns 0 if resizing failed
// *---------------------------------------------------------------------------*/
//u32
//Heap::ResizeMBlock( void* addr, u32 size )
//{
//    return vw::system::Memory_ResizeMBlock( addr, size );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::Free
//
//  Description:  Deallocate memory
//
//  Arguments:    addr    Memory address to be deallocated.
//
//  Returns:      None.
// *---------------------------------------------------------------------------*/
//void
//Heap::Free( void* addr )
//{
//    vw::system::Memory_Free( addr );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatorMEM1
//
//  Description:  Get an Allocator that allocates memory from MEM1
//
//  Arguments:    None.
//
//  Returns:      A reference to the allocator
// *---------------------------------------------------------------------------*/
//MEMAllocator&
//Heap::GetAllocatorMEM1()
//{
//    return *vw::system::Memory_GetTryAllocatorMEM1( vw::system::USER_ID );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatorMEM2
//
//  Description:  Get an Allocator that allocates memory from MEM2
//
//  Arguments:    None.
//
//  Returns:      A reference to the allocator
// *---------------------------------------------------------------------------*/
//MEMAllocator&
//Heap::GetAllocatorMEM2()
//{
//    return *vw::system::Memory_GetTryAllocatorMEM2( vw::system::USER_ID );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocator1
//
//  Description:  Get an Allocator that allocates memory, giving priority to allocating from MEM1
//
//  Arguments:    None.
//
//  Returns:      A reference to the allocator
// *---------------------------------------------------------------------------*/
//MEMAllocator&
//Heap::GetAllocator1()
//{
//    return *vw::system::Memory_GetTryAllocator( vw::system::USER_ID );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocator2
//
//  Description:  Get an Allocator that allocates memory, giving priority to allocating from MEM2
//
//  Arguments:    None.
//
//  Returns:      A reference to the allocator
// *---------------------------------------------------------------------------*/
//MEMAllocator&
//Heap::GetAllocator2()
//{
//    return *vw::system::Memory_GetTryAllocator2( vw::system::USER_ID );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatedSize
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetAllocatedSize() const
//{
//    return vw::system::Memory_AllocatedSize();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatedSizeMEM1
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetAllocatedSizeMEM1() const
//{
//    return vw::system::Memory_AllocatedSizeMEM1();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatedSizeMEM2
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetAllocatedSizeMEM2() const
//{
//    return vw::system::Memory_AllocatedSizeMEM2();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatableSize
//
//  Description:
//
//  Arguments:    HEAP_DEFAULT_ALIGN
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetAllocatableSize( s32 align /* = HEAP_DEFAULT_ALIGN */ ) const
//{
//    return vw::system::Memory_AllocatableSize( align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatableSizeMEM1
//
//  Description:
//
//  Arguments:    HEAP_DEFAULT_ALIGN
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetAllocatableSizeMEM1( s32 align /* = HEAP_DEFAULT_ALIGN */ ) const
//{
//    return vw::system::Memory_AllocatableSizeMEM1( align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetAllocatableSizeMEM2
//
//  Description:
//
//  Arguments:    HEAP_DEFAULT_ALIGN
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetAllocatableSizeMEM2( s32 align /* = HEAP_DEFAULT_ALIGN */ ) const
//{
//    return vw::system::Memory_AllocatableSizeMEM2( align );
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetFreeSize
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetFreeSize() const
//{
//    return vw::system::Memory_FreeSize();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetFreeSizeMEM1
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetFreeSizeMEM1() const
//{
//    return vw::system::Memory_FreeSizeMEM1();
//}
//
///*---------------------------------------------------------------------------*
//  Name:         Heap::GetFreeSizeMEM2
//
//  Description:
//
//  Arguments:    None.
//
//  Returns:
// *---------------------------------------------------------------------------*/
//u32
//Heap::GetFreeSizeMEM2() const
//{
//    return vw::system::Memory_FreeSizeMEM2();
//}
//

} /* namespace sys    */
} /* namespace viewer */
} /* namespace nw4r   */





