/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include <nw/viewer/sys/McsManager.h>

namespace nw     {
namespace viewer {
namespace sys    {

/*---------------------------------------------------------------------------*
  Name:         McsChannel::McsChannel

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
/* ctor */
McsChannel::McsChannel( u16 channel )
 : mChannel( channel )
{
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::~McsChannel

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
/* dtor */
McsChannel::~McsChannel()
{
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::Destroy

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
McsChannel::Destroy()
{
    if ( this )
    {
        vw::system::Memory_Free( this );
    }
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::Read

  Description:

  Arguments:    data
                size

  Returns:
 *---------------------------------------------------------------------------*/
u32
McsChannel::Read( void* data, u32 size )
{
    return vw::system::Stream_Read( mChannel, data, size );
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::Write

  Description:

  Arguments:    data
                size

  Returns:
 *---------------------------------------------------------------------------*/
u32
McsChannel::Write( const void* data, u32 size )
{
    return vw::system::Stream_Write( mChannel, data, size );
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::Peek

  Description:

  Arguments:    data
                size

  Returns:
 *---------------------------------------------------------------------------*/
u32
McsChannel::Peek( void* data, u32 size )
{
    return vw::system::Stream_Peek( mChannel, data, size );
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::Skip

  Description:

  Arguments:    size

  Returns:
 *---------------------------------------------------------------------------*/
u32
McsChannel::Skip( u32 size )
{
    return vw::system::Stream_Skip( mChannel, size );
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::GetReadableBytes

  Description:

  Arguments:    None.

  Returns:
 *---------------------------------------------------------------------------*/
u32
McsChannel::GetReadableBytes() const
{
    return vw::system::Stream_GetReadableBytes( mChannel );
}

/*---------------------------------------------------------------------------*
  Name:         McsChannel::GetWritableBytes

  Description:

  Arguments:    pWritableBytes

  Returns:
 *---------------------------------------------------------------------------*/
u32
McsChannel::GetWritableBytes( u32* pWritableBytes ) const
{
    return vw::system::Stream_GetWritableBytes( mChannel, pWritableBytes );
}


/*---------------------------------------------------------------------------*
  Name:         McsChannel::Polling

  Description:

  Arguments:

  Returns:
 *---------------------------------------------------------------------------*/
bool
McsChannel::Polling()
{
    return vw::system::Stream_Polling( mChannel );
}


/*---------------------------------------------------------------------------*
  Name:         McsManager::IsServerConnected

  Description:

  Arguments:    None.

  Returns:
 *---------------------------------------------------------------------------*/
bool
McsManager::IsServerConnected() const
{
    bool IsRunning = false;
    nn::Result ret = nn::hio::GetHioDaemonStatus( &IsRunning );

    return (ret.IsSuccess() && IsRunning == true);
}


/*---------------------------------------------------------------------------*
  Name:         McsManager::CreateChannel

  Description:   Registers MCS channels

  Arguments:    channel

  Returns:
 *---------------------------------------------------------------------------*/
McsChannel*
McsManager::CreateChannel( u16 channel, u32 bufSize )
{
    void* buf = vw::system::Memory_AllocSys( sizeof(McsChannel) );
    vw::system::Stream_CreateChannel( channel, bufSize );
    return new(buf) McsChannel( channel );
}


} /* namespace sys    */
} /* namespace viewer */
} /* namespace nw     */



