﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_PARAMSETTER_H_
#define NW4C_VIEWER_G3D_PARAMSETTER_H_

#include <version.h>
#include <viewer/g3d/Protocol4C.h>
#include <nw/viewer/sys/Application.h>
#include "ModuleG3dImpl.h"
#include "ResourceManager.h"
#include "SceneManager.h"
#include "SceneElemSelector.h"
#include "SceneRenderer.h"
#include "AnimManager.h"
#include "UserDataManager.h"
#include "../../system/include/ConvertCharEncoding.h"
#include "../../system/include/LogUtil.h"
#include "../../../include/viewer/g3d/CameraMgr.h"
#include "../../../include/viewer/g3d/LightMgr.h"
#include "ErrorCheck.h"

namespace vw {
namespace g3d {

#define FOREACH_SELECTED_G3D( valName ) \
    for (nw::ut::MoveArray<Protocol4C::EditSelectedObject>::iterator valName = m_SelectedObject.begin(), end_it = m_SelectedObject.end();\
        valName != end_it; ++valName)

#define GET_RES_OBJECT_G3D( name, id, index, obj ) \
    nw::gfx::res::Res##name obj = m_ScnMgr->FindRes##name( id, index ); \
    NW_NULL_ASSERT( obj.ptr() );

#define GET_SCENE_NODE_G3D( name, id, index, obj ) \
    nw::gfx::name* obj = m_ScnMgr->Find##name##Node( id, index ); \
    NW_NULL_ASSERT( obj );

#define GET_RESMODEL_G3D( id, index, obj ) GET_RES_OBJECT_G3D( Model, id, index, obj )
#define GET_RESTEXTURE_G3D( id, index, obj ) GET_RES_OBJECT_G3D( Texture, id, index, obj )
#define GET_RESSCENESETTING_G3D( id, index, obj ) GET_RES_OBJECT_G3D( SceneEnvironmentSetting, id, index, obj )
#define GET_RESCAMERA_G3D( id, index, obj ) GET_RES_OBJECT_G3D( Camera, id, index, obj )
#define GET_RESLIGHT_G3D( id, index, obj ) GET_RES_OBJECT_G3D( Light, id, index, obj )
#define GET_RESFOG_G3D( id, index, obj ) GET_RES_OBJECT_G3D( Fog, id, index, obj )

#define GET_MODEL_G3D( id, index, obj ) GET_SCENE_NODE_G3D( Model, id, index, obj )
#define GET_SCENESETTING_G3D( id, index, obj ) GET_SCENE_NODE_G3D( SceneEnvironmentSetting, id, index, obj )
#define GET_CAMERA_G3D( id, index, obj ) GET_SCENE_NODE_G3D( Camera, id, index, obj )
#define GET_LIGHT_G3D( id, index, obj ) GET_SCENE_NODE_G3D( Light, id, index, obj )
#define GET_FOG_G3D( id, index, obj ) GET_SCENE_NODE_G3D( Fog, id, index, obj )

#define GET_RESMATERIAL_ORIGINAL_G3D( index, obj ) \
    nw::gfx::res::ResMaterial obj = model->GetMaterial( index )->GetOriginal()

#define GET_RESMATERIAL_BUFFER_G3D( index, obj ) \
    nw::gfx::res::ResMaterial obj = model->GetMaterial( index )->GetBuffer()

#define SET_FLAGS( obj, flag, bitFlag )      \
    {                                        \
        u32 nFlags = obj.GetFlags();         \
        if ( flag ) { nFlags |= (bitFlag); } \
        else { nFlags &= ~(bitFlag); }       \
        obj.SetFlags( nFlags );              \
    }

#define GET_RES_ORIG_G3D( basename, name, id, index, obj )                               \
    nw::gfx::res::Res##name obj;                                                         \
    {                                                                                    \
        nw::gfx::basename* obj_##basename = m_ScnMgr->Find##basename##Node( id, index ); \
        if ( obj_##basename != NULL )                                                    \
        {                                                                                \
            u8* ptr = this->GetOriginalValue( obj_##basename->GetAnimGroup() );          \
            if ( ptr != NULL ) { obj = nw::gfx::res::Res##name( ptr ); }                 \
        }                                                                                \
    }

#define GET_RESLIGHT_ORIG_G3D( id, index, obj )         GET_RES_ORIG_G3D( Light, Light, id, index, obj )
#define GET_RESFRAGMENTLIGHT_ORIG_G3D( id, index, obj ) GET_RES_ORIG_G3D( Light, FragmentLight, id, index, obj )
#define GET_RESFOG_ORIG_G3D(id, index, obj )            GET_RES_ORIG_G3D( Fog, Fog, id, index, obj )


class ParamSetter
{
public:
    enum WRAP_TYPE
    {
        Wrap_S,
        Wrap_T
    };
    enum TEX_ST
    {
        TexS = 0x01,
        TexT = 0x02
    };
    enum AXIS_TYPE
    {
        AXIS_X,
        AXIS_Y,
        AXIS_Z
    };

    static ParamSetter& GetInstance();
    static void Initialize();
    static void Finalize();

    void
    Focus()
    {
        struct Inner
        {
            static nw::math::AABB MergeAABB(nw::math::AABB a, const nw::math::AABB& b)
            {
                if(a.Pmin.x>b.Pmin.x) a.Pmin.x=b.Pmin.x;
                if(a.Pmin.y>b.Pmin.y) a.Pmin.y=b.Pmin.y;
                if(a.Pmin.z>b.Pmin.z) a.Pmin.z=b.Pmin.z;
                if(a.Pmax.x<b.Pmax.x) a.Pmax.x=b.Pmax.x;
                if(a.Pmax.y<b.Pmax.y) a.Pmax.y=b.Pmax.y;
                if(a.Pmax.z<b.Pmax.z) a.Pmax.z=b.Pmax.z;
                return a;
            }
            static nw::math::AABB CalcAABB(const nw::ut::MoveArray<nw::math::VEC3>& vertexPositions)
            {
                nw::math::AABB aabb;
                aabb.Pmin = nw::math::VEC3(0.0f,0.0f,0.0f);
                aabb.Pmax = nw::math::VEC3(0.0f,0.0f,0.0f);
                if(vertexPositions.empty())
                    return aabb;
                aabb.Pmin=aabb.Pmax = vertexPositions[0];
                for(size_t i=1; i<vertexPositions.size(); ++i){
                    nw::math::VEC3 p = vertexPositions[i];
                    aabb.Pmin.x = nw::math::Min(aabb.Pmin.x, p.x);
                    aabb.Pmin.y = nw::math::Min(aabb.Pmin.y, p.y);
                    aabb.Pmin.z = nw::math::Min(aabb.Pmin.z, p.z);
                    aabb.Pmax.x = nw::math::Max(aabb.Pmax.x, p.x);
                    aabb.Pmax.y = nw::math::Max(aabb.Pmax.y, p.y);
                    aabb.Pmax.z = nw::math::Max(aabb.Pmax.z, p.z);
                }
                return aabb;
            }
            static nw::ut::MoveArray<nw::math::VEC3> GetOBBPosition(const nw::ut::ResOrientedBoundingBox& obb)
            {
                const nw::math::VEC3 obb_vertex[8] = {
                     nw::math::VEC3(-0.5f,-0.5f,-0.5f),
                     nw::math::VEC3(-0.5f,-0.5f,+0.5f),
                     nw::math::VEC3(-0.5f,+0.5f,-0.5f),
                     nw::math::VEC3(-0.5f,+0.5f,+0.5f),
                     nw::math::VEC3(+0.5f,-0.5f,-0.5f),
                     nw::math::VEC3(+0.5f,-0.5f,+0.5f),
                     nw::math::VEC3(+0.5f,+0.5f,-0.5f),
                     nw::math::VEC3(+0.5f,+0.5f,+0.5f)
                };
                nw::ut::MoveArray<nw::math::VEC3> pos( Memory_GetAllocator() );
                for(u32 i=0; i<8; ++i){
                    nw::math::VEC3 p = obb_vertex[i];
                    p *= obb.GetSize();
                    nw::math::MTX33 mat = obb.GetOrientationMatrix();

                    // transpose matrix
                    std::swap(mat._10,mat._01);
                    std::swap(mat._20,mat._02);
                    std::swap(mat._21,mat._12);

                    nw::math::VEC3Transform(&p,&mat,&p);
                    p += obb.GetCenterPosition();
                    pos.push_back(p);
                }
                return pos;
            }
            static nw::math::AABB MakeModelAABB(nw::gfx::Model* model,
                                                nw::gfx::SkeletalModel* skeletalModel,
                                                nw::gfx::res::ResMesh mesh,
                                                bool& isDefaultObb)
            {
                isDefaultObb = false;
                nw::ut::MoveArray<nw::math::VEC3> vertexPositions( Memory_GetAllocator() );
                nw::math::VEC3 deafultObbSize(1.0f, 1.0f, 1.0f);
                nw::math::MTX33 defualtObbOrientationMatrix;
                nw::math::MTX33Identity(&defualtObbOrientationMatrix);

                nw::gfx::res::ResShape shape = model->GetResModel().GetShapes(mesh.GetShapeIndex());
                nw::ut::ResOrientedBoundingBox obb = shape.GetOrientedBoundingBox();

                // Determines whether it is an initialization OBB.
                if ((obb.GetSize() == deafultObbSize) &&
                    (obb.GetOrientationMatrix() == defualtObbOrientationMatrix))
                {
                    isDefaultObb = true;
                }

                for(s32 j=0; j<shape.GetPrimitiveSetsCount(); ++j)
                {
                    nw::gfx::res::ResPrimitiveSet primitiveSet = shape.GetPrimitiveSets(j);
                    s32 boneIndexCount = primitiveSet.GetBoneIndexTableCount();

                    if (skeletalModel == NULL || boneIndexCount == 0)
                    {
                        // Finds the world coordinates of each OBB vertex.
                        nw::ut::MoveArray<nw::math::VEC3> obb_pos = GetOBBPosition(obb);
                        for(size_t k=0; k<obb_pos.size(); ++k)
                        {
                            nw::math::VEC3Transform(&obb_pos[k], &model->WorldMatrix(), &obb_pos[k]);
                            vertexPositions.push_back(obb_pos[k]);
                        }
                    }
                    else
                    {
                        bool noneSkinning = primitiveSet.GetSkinningMode() == nw::gfx::res::ResPrimitiveSet::SKINNING_MODE_NONE;

                        s32 matrixPalletCount = primitiveSet.GetBoneIndexTableCount();
                        nw::ut::MoveArray<nw::math::VEC3> obb_pos = GetOBBPosition(obb);

                        nw::math::MTX34 matrix;
                        for(size_t k=0; k<obb_pos.size(); ++k)
                        {
                            nw::math::VEC3 temp_pos(0.0f,0.0f,0.0f);

                            for (s32 count = 0; count < matrixPalletCount; ++count)
                            {
                                s32 boneIndex = primitiveSet.GetBoneIndexTable(count);

                                nw::gfx::Skeleton* skeleton = skeletalModel->GetSkeleton();
                                nw::gfx::res::ResSkeleton resSkeleton = skeleton->GetResSkeleton();
                                nw::gfx::res::ResBone bone = resSkeleton.GetBones(boneIndex);

                                if(noneSkinning)
                                {
                                    nw::gfx::Skeleton::MatrixPose& matrixPose = skeleton->WorldMatrixPose();
                                    matrix = *matrixPose.GetMatrix(boneIndex);
                                }
                                else
                                {
                                    nw::math::MTX34Mult(&matrix,
                                                        skeleton->WorldMatrixPose().GetMatrix(boneIndex),
                                                        &bone.GetInverseBaseMatrix());
                                }
                                nw::math::VEC3 p, pos=obb_pos[k];
                                nw::math::VEC3Transform(&p,&matrix,&pos);
                                temp_pos += p;
                            }

                            // Everything is handled as being equal weight value because there is no envelope weight information.
                            temp_pos /= matrixPalletCount;
                            vertexPositions.push_back(temp_pos);
                        }
                    }
                }
                nw::math::AABB aabb = CalcAABB(vertexPositions);
                return aabb;
            }
        };

        nw::math::AABB selectedModelsAabb;
        selectedModelsAabb.Pmin = nw::math::VEC3(0.0f,0.0f,0.0f);
        selectedModelsAabb.Pmax = nw::math::VEC3(0.0f,0.0f,0.0f);
        bool isDefaultObb=false, initAABB=false;

        nw::ut::MoveArray<SceneRenderer::BlinkAnimInfo>::iterator it = m_ScnRdr->GetBlinkAnimationsBegin();
        nw::ut::MoveArray<SceneRenderer::BlinkAnimInfo>::iterator end = m_ScnRdr->GetBlinkAnimationsEnd();
        for ( ; it != end; ++it )
        {
            if(it->model)
            {
                nw::math::AABB modelAabb = Inner::MakeModelAABB(
                    it->model,
                    nw::ut::DynamicCast<nw::gfx::SkeletalModel*>(it->model),
                    it->mesh,
                    isDefaultObb
                );

                if (!initAABB) {
                    initAABB = true;
                    selectedModelsAabb = modelAabb;
                } else {
                    selectedModelsAabb = Inner::MergeAABB(selectedModelsAabb, modelAabb);
                }
            }
        }

        nw::math::VEC3 center = (selectedModelsAabb.Pmax + selectedModelsAabb.Pmin) / 2.0f;
        nw::math::VEC3 size   = (selectedModelsAabb.Pmax - selectedModelsAabb.Pmin);
        if(nw::math::VEC3Len(&size) > 0.0f)
        {
            CameraMgr::GetInstance()->Focus(selectedModelsAabb);
        }
    }

    void
    SetAmbient(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetAmbient());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetAmbient(modColor.r, modColor.g, modColor.b);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetAmbient());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetAmbient(modColor.r, modColor.g, modColor.b);
            }
        }
    }

    void
    SetVertexColorScale(float scale)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();
                resMaterialColor.SetVertexColorScale(scale);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();
                resMaterialColor.SetVertexColorScale(scale);
            }
        }
    }

    void
    SetDiffuse(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetDiffuse());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetDiffuse(modColor.r, modColor.g, modColor.b, modColor.a);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetDiffuse());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetDiffuse(modColor.r, modColor.g, modColor.b, modColor.a);
            }
        }
    }

    void
    SetSpecular0(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetSpecular0());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetSpecular0(modColor.r, modColor.g, modColor.b);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetSpecular0());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetSpecular0(modColor.r, modColor.g, modColor.b);
            }
        }
    }

    void
    SetSpecular1(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetSpecular1());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetSpecular1(modColor.r, modColor.g, modColor.b);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetSpecular1());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetSpecular1(modColor.r, modColor.g, modColor.b);
            }
        }
    }

    void
    SetEmission(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetEmission());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetEmission(modColor.r, modColor.g, modColor.b);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetEmission());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], 1.0f);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetEmission(modColor.r, modColor.g, modColor.b);
            }
        }
    }

    void
    SetConstant0(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant0());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant0(modColor.r, modColor.g, modColor.b, modColor.a);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant0());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant0(modColor.r, modColor.g, modColor.b, modColor.a);
            }
        }
    }

    void
    SetConstant1(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant1());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant1(modColor.r, modColor.g, modColor.b, modColor.a);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant1());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant1(modColor.r, modColor.g, modColor.b, modColor.a);
            }
        }
    }

    void
    SetConstant2(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant2());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant2(modColor.r, modColor.g, modColor.b, modColor.a);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant2());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant2(modColor.r, modColor.g, modColor.b, modColor.a);
            }
        }
    }

    void
    SetConstant3(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant3());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant3(modColor.r, modColor.g, modColor.b, modColor.a);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant3());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant3(modColor.r, modColor.g, modColor.b, modColor.a);
            }
        }
    }

    void
    SetConstant4(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant4());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant4(modColor.r, modColor.g, modColor.b, modColor.a);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant4());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant4(modColor.r, modColor.g, modColor.b, modColor.a);
            }
        }
    }

    void
    SetConstant5(float *color, unsigned int modifiedPropertyElements)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant5());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant5(modColor.r, modColor.g, modColor.b, modColor.a);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResMaterialColor resMaterialColor = resMaterial.GetMaterialColor();

                nw::ut::FloatColor modColor(resMaterialColor.GetConstant5());
                const nw::ut::FloatColor transColor(color[0], color[1], color[2], color[3]);
                SelectColorElement(modColor, transColor, modifiedPropertyElements);

                resMaterialColor.SetConstant5(modColor.r, modColor.g, modColor.b, modColor.a);
            }
        }
    }

    // Axis rendering additional functions
    void
    AddDrawAxis()
    {
        // Temporary cancellation of model and bone axis rendering
        m_ScnRdr->DeleteDrawAxis(SceneRenderer::AxisModel);
        m_ScnRdr->DeleteDrawAxis(SceneRenderer::AxisBone);

        for ( int i = 0; i < m_SelectedObject.size(); ++i )
        {
            switch ( m_SelectedType )
            {
            case Protocol4C::EditTargetType_Model:
            case Protocol4C::EditTargetType_Mesh:
            case Protocol4C::EditTargetType_Material:
                {
                    nw::gfx::Model* model = m_ScnMgr->FindModelNode(m_SelectedObject[ i ].Id);
                    if (model)
                    {
                        SceneRenderer::DrawAxisInfo info;
                        info.axisType = SceneRenderer::AxisModel;
                        info.axisTransform = &(model->WorldTransform());
                        info.key1 = model;
                        info.key2 = NULL;
                        m_ScnRdr->AddDrawAxis(info);
                    }
                }
                break;
            case Protocol4C::EditTargetType_Texture:
                {
                    nw::gfx::res::ResTexture texture = m_ScnMgr->FindResTexture(m_SelectedObject[ i ].Id);

                    SceneManager::ModelList::iterator it = m_ScnMgr->GetModelsBegin();
                    SceneManager::ModelList::iterator end = m_ScnMgr->GetModelsEnd();
                    for ( ; it != end; ++it )
                    {
                        nw::gfx::Model* model = it->GetSceneNode();
                        if (!model) continue;

                        for ( int j = 0; j < model->GetMaterialCount(); ++j )
                        {
                            nw::gfx::res::ResMaterial material = model->GetMaterial( j )->GetBuffer();
                            for ( int k = 0; k < material.GetTextureMappersCount(); ++k )
                            {
                                nw::gfx::res::ResTextureMapper mapper = material.GetTextureMappers(k);
                                if (!mapper.IsValid()) continue;

                                if (mapper.GetTexture().Dereference().ptr() == texture.ptr())
                                {
                                    SceneRenderer::DrawAxisInfo info;
                                    info.axisType = SceneRenderer::AxisModel;
                                    info.axisTransform = &(model->WorldTransform());
                                    info.key1 = model;
                                    info.key2 = NULL;
                                    m_ScnRdr->AddDrawAxis(info);
                                }
                            }
                        }
                    }
                }
                break;
            case Protocol4C::EditTargetType_Bone:
                {
                    nw::gfx::SkeletalModel* model =
                        nw::ut::DynamicCast<nw::gfx::SkeletalModel*>(m_ScnMgr->FindModelNode(m_SelectedObject[ i ].Id));
                    if (model)
                    {
                        SceneRenderer::DrawAxisInfo info;
                        info.axisType = SceneRenderer::AxisBone;
                        info.axisTransform =
                            model->GetSkeleton()->WorldTransformPose().GetTransform(m_SelectedObject[ i ].Index);
                        info.key1 = model;
                        info.key2 = NULL;
                        m_ScnRdr->AddDrawAxis(info);
                    }
                }
                break;
            default:
                // Invalid Type
                break;
            }
        }
    }

    void
    SelectObject(
        Protocol4C::EditTargetType selectedType,
        nw::ut::MoveArray<Protocol4C::EditSelectedObject>& selectedObject)
    {
        m_SelectedType = selectedType;
        m_SelectedObject = selectedObject;

        // Additional axis rendering
        AddDrawAxis();

        // Blink animation processing
        {
            SceneRenderer::BlinkAnimTarget target;
            switch ( m_SelectedType )
            {
            case Protocol4C::EditTargetType_Model:
                target.type = SceneRenderer::BlinkAnimTarget::BLINK_MODEL;
                break;
            case Protocol4C::EditTargetType_Mesh:
                target.type = SceneRenderer::BlinkAnimTarget::BLINK_MESH;
                break;
            case Protocol4C::EditTargetType_Texture:
                target.type = SceneRenderer::BlinkAnimTarget::BLINK_TEXTURE;
                break;
            case Protocol4C::EditTargetType_Material:
                target.type = SceneRenderer::BlinkAnimTarget::BLINK_MATERIAL;
                break;
            default:
                // Invalid Type
                target.type = SceneRenderer::BlinkAnimTarget::BLINK_NUM;
                break;
            }

            nw::ut::MoveArray<SceneRenderer::BlinkAnimTarget> blinkTargets( Memory_GetAllocator() );
            for ( int i = 0; i < m_SelectedObject.size(); ++i )
            {
                target.hashID = m_SelectedObject[ i ].Id;
                target.index = m_SelectedObject[ i ].Index;
                blinkTargets.push_back( target );
            }

            m_ScnRdr->SetBlinkAnimation( blinkTargets );
        }
    }

    void
    SetTexture(Protocol4C::MESSAGEPACKET_Material_SetTexture *setTexture)
    {
        NW_VIEWER_LOG("[Id, Index] =  [%u, %u]\n", setTexture->Id, setTexture->Index);
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                GET_RESTEXTURE_G3D( setTexture->Id, 0, resTexture );
                resMaterial.GetTextureMappers(setTexture->Index).SetTexture(resTexture);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                GET_RESTEXTURE_G3D( setTexture->Id, 0, resTexture );
                resMaterial.GetTextureMappers(setTexture->Index).SetTexture(resTexture);
            }

            // ErrorCheck
            ErrorCheck::CheckModelTextureReference( model );
        }
    }

    void
    AddAnimationSet(const Protocol4C::Header& header, Protocol4C::MESSAGEPACKET_Animation_AddAnimationSet *body)
    {
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(body->AnimationSetName.Value, utf8Str, MAX_PATH_SIZE);

        NW_VIEWER_LOG("AnimSet name : %s ( ", utf8Str);
        nw::ut::MoveArray<AnimElem*> anims( Memory_GetAllocator() );
        for (int i=0; i<header.arraySize/Protocol4C::AnimationId::StructSize; ++i)
        {
            Protocol4C::AnimationId animId = *(Protocol4C::AnimationId*)
               ( (unsigned char*)body + Protocol4C::MESSAGEPACKET_Animation_AddAnimationSet::StructSize
                                      + Protocol4C::AnimationId::StructSize * i);
            SceneManager::AnimList::iterator it = m_ScnMgr->FindAnim( animId.Id, 0 );
            if (it != m_ScnMgr->GetAnimsEnd()) {
                NW_VIEWER_LOG("%s, ", it->GetResObject().GetName() );
                anims.push_back(&*it);
            }
        }
        NW_VIEWER_LOG(") \n");

        m_ScnMgr->FindModel( body->ModelId, 0 )->GetAnimLinker()->Group( utf8Str, anims );
    }

    void
    BindSceneAnimation(Protocol4C::MESSAGEPACKET_Animation_BindSceneAnimation *body)
    {
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(body->AnimationName.Value, utf8Str, MAX_PATH_SIZE);

        switch (body->AnimType)
        {
        case Protocol4C::SceneAnimationType_Camera:
            {
                m_ScnMgr->FindCamera( body->TargetID, body->TargetIndex )->GetAnimLinker()->ChangeTo( utf8Str );
            }
            break;
        case Protocol4C::SceneAnimationType_Light:
            {
                m_ScnMgr->FindLight( body->TargetID, body->TargetIndex )->GetAnimLinker()->ChangeTo( utf8Str );
            }
            break;
        case Protocol4C::SceneAnimationType_Fog:
            {
                m_ScnMgr->FindFog( body->TargetID, body->TargetIndex )->GetAnimLinker()->ChangeTo( utf8Str );
            }
            break;
        }
    }

    void
    ClearAnimationSet(Protocol4C::MESSAGEPACKET_Animation_ClearAnimationSet *body)
    {
        m_ScnMgr->FindModel( body->ModelId, 0 )->GetAnimLinker()->Clear();
    }

    void
    AddStrayAnimations(const Protocol4C::Header& header, Protocol4C::MESSAGEPACKET_Animation_AddStrayAnimations *body)
    {
        NW_VIEWER_LOG("Animations ( ");
        nw::ut::MoveArray<AnimElem*> anims( Memory_GetAllocator() );
        for (int i=0; i<header.arraySize/Protocol4C::AnimationId::StructSize; ++i)
        {
            Protocol4C::AnimationId animId = *(Protocol4C::AnimationId*)
               ( (unsigned char*)body + Protocol4C::MESSAGEPACKET_Animation_AddStrayAnimations::StructSize
                                      + Protocol4C::AnimationId::StructSize * i);
            SceneManager::AnimList::iterator it = m_ScnMgr->FindAnim( animId.Id, 0 );
            if (it != m_ScnMgr->GetAnimsEnd()) {
                NW_VIEWER_LOG("%s, ", it->GetResObject().GetName() );
                anims.push_back(&*it);
            }
        }
        NW_VIEWER_LOG(") \n");

        m_ScnMgr->FindModel( body->ModelId, 0 )->GetAnimLinker()->GroupByName( anims );
    }

    void
    ActivateAnimationSet(Protocol4C::MESSAGEPACKET_Animation_ActivateAnimationSet *body)
    {
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(body->AnimationSetName.Value, utf8Str, MAX_PATH_SIZE);
        m_ScnMgr->FindModel( body->ModelId, 0 )->GetAnimLinker()->ChangeTo( utf8Str );
    }

    void
    RenameAnimationSet(Protocol4C::MESSAGEPACKET_Animation_RenameAnimationSet *body)
    {
        const int MAX_PATH_SIZE = 260;
        char oldName[MAX_PATH_SIZE];
        char newName[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(body->OldAnimationSetName.Value, oldName, MAX_PATH_SIZE);
        vw::system::ConvertStrWtoUTF8(body->NewAnimationSetName.Value, newName, MAX_PATH_SIZE);
        m_ScnMgr->FindModel( body->ModelId, 0 )->GetAnimLinker()->RenameAnimSet( oldName, newName );
    }

    void
    DeactivateAnimationSet(Protocol4C::MESSAGEPACKET_Animation_DeActivateAnimationSet *body)
    {
        m_ScnMgr->FindModel( body->ModelId, 0 )->GetAnimLinker()->ChangeToFirstStrayAnimation();
    }

    void
    AddLightAnimation(Protocol4C::MESSAGEPACKET_Animation_AddLightAnimation *body)
    {
        SceneManager::AnimList::iterator it = m_ScnMgr->FindAnim(body->AnimationID, body->AnimationIndex);
        if (it != m_ScnMgr->GetAnimsEnd()) {
            m_ScnMgr->FindLight( body->LightID, body->LightIndex )->GetAnimLinker()->Group(&*it);
        }
    }

    void
    AddCameraAnimation(Protocol4C::MESSAGEPACKET_Animation_AddCameraAnimation *body)
    {
        SceneManager::AnimList::iterator it = m_ScnMgr->FindAnim(body->AnimationID, body->AnimationIndex);
        if (it != m_ScnMgr->GetAnimsEnd()) {
            m_ScnMgr->FindCamera( body->CameraID, body->CameraIndex )->GetAnimLinker()->Group(&*it);
        }
    }

    void
    AddFogAnimation(Protocol4C::MESSAGEPACKET_Animation_AddFogAnimation *body)
    {
        SceneManager::AnimList::iterator it = m_ScnMgr->FindAnim(body->AnimationID, body->AnimationIndex);
        if (it != m_ScnMgr->GetAnimsEnd()) {
            m_ScnMgr->FindFog( body->FogID, body->FogIndex )->GetAnimLinker()->Group(&*it);
        }
    }

    void
    StoreCurrentFrame(Protocol4C::MESSAGEPACKET_Animation_StoreCurrentFrame* body)
    {
        ResourceSet::BinaryFileType types[] = {
            ResourceSet::Type_Model,        // StoreTargetObjectType_Model = 0,         // model.
            ResourceSet::Type_Camera,       // StoreTargetObjectType_Camera = 1,        // camera.
            ResourceSet::Type_Light,        // StoreTargetObjectType_Light = 2,         // light.
            ResourceSet::Type_Fog           // StoreTargetObjectType_Fog = 3,           // fog.
        };

        AnimManager::GetInstance()->StoreCurrentAnimFrame(
            types[ body->TargetType ],
            body->TargetID,
            body->TargetIndex
        );
    }

    void
    RestoreCurrentFrame(Protocol4C::MESSAGEPACKET_Animation_RestoreCurrentFrame* body)
    {
        ResourceSet::BinaryFileType types[] = {
            ResourceSet::Type_Model,        // StoreTargetObjectType_Model = 0,         // model.
            ResourceSet::Type_Camera,       // StoreTargetObjectType_Camera = 1,        // camera.
            ResourceSet::Type_Light,        // StoreTargetObjectType_Light = 2,         // light.
            ResourceSet::Type_Fog           // StoreTargetObjectType_Fog = 3,           // fog.
        };

        AnimManager::GetInstance()->RestoreCurrentAnimFrame(
            types[ body->TargetType ],
            body->TargetID,
            body->TargetIndex
        );
    }

    // Setting buffer colors
    void
    SetBufferColor(float *color)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                resFragmentShader.SetBufferColor(color[0], color[1], color[2], color[3]);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                resFragmentShader.SetBufferColor(color[0], color[1], color[2], color[3]);
            }
        }
    }

    // Setting culling modes
    void
    SetCullingMode(Protocol4C::MaterialCullingMode mode)
    {
        const nw::gfx::res::ResRasterization::CullingMode cullMode[4] =
        {
            nw::gfx::res::ResRasterization::CULLINGMODE_FRONT,  //
            nw::gfx::res::ResRasterization::CULLINGMODE_BACK,   //
            nw::gfx::res::ResRasterization::CULLINGMODE_NONE    //
        };

        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResRasterization resRasterization = resMaterial.GetRasterization();
                resRasterization.SetCullingMode( cullMode[mode] );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResRasterization resRasterization = resMaterial.GetRasterization();
                resRasterization.SetCullingMode( cullMode[mode] );
            }
        }
    }

    // Setting display layers
    void
    SetTranslucencyKind(Protocol4C::TranslucencyKindCtr kind)
    {
        const nw::gfx::res::ResMaterial::TranslucencyKind kindlist[] = {
            nw::gfx::res::ResMaterial::TRANSLUCENCY_KIND_LAYER0,   //
            nw::gfx::res::ResMaterial::TRANSLUCENCY_KIND_LAYER1,   //
            nw::gfx::res::ResMaterial::TRANSLUCENCY_KIND_LAYER2,   //
            nw::gfx::res::ResMaterial::TRANSLUCENCY_KIND_LAYER3    //
        };

        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                resMaterial.SetTranslucencyKind(kindlist[kind]);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                resMaterial.SetTranslucencyKind(kindlist[kind]);
            }
        }
    }

    // Setting alpha tests
    void
    SetAlphaTest(nw::gfx::res::ResAlphaTest resAlphaTest, Protocol4C::MESSAGEPACKET_Material_SetAlphaTest *alphaTest)
    {
        const nw::gfx::res::ResAlphaTest::TestFunc testFuncList[] = {
            nw::gfx::res::ResAlphaTest::TESTFUNC_NEVER,      // TestFunctionCtr_Never = 0,           // No matter what the value, it will not pass through.
            nw::gfx::res::ResAlphaTest::TESTFUNC_ALWAYS,     // TestFunctionCtr_Always = 1,           // No matter what the value, it will pass through.
            nw::gfx::res::ResAlphaTest::TESTFUNC_LESS,       // TestFunctionCtr_Less = 2,           // If the value is smaller than the reference value, it will pass through.
            nw::gfx::res::ResAlphaTest::TESTFUNC_LEQUAL,     // TestFunctionCtr_Lequal = 3,           // If the value is at or below the reference value, it will pass through.
            nw::gfx::res::ResAlphaTest::TESTFUNC_EQUAL,      // TestFunctionCtr_Equal = 4,           // If the value is equal to the reference value, it will pass through.
            nw::gfx::res::ResAlphaTest::TESTFUNC_GEQUAL,     // TestFunctionCtr_Gequal = 5,           // If the value is at or below the reference value, it will pass through.
            nw::gfx::res::ResAlphaTest::TESTFUNC_GREATER,    // TestFunctionCtr_Greater = 6,           // If the value is larger than the reference value, it will pass through.
            nw::gfx::res::ResAlphaTest::TESTFUNC_NOTEQUAL    // TestFunctionCtr_Notequal = 7,           // If the value is equal to the reference value, it will pass through.
        };

        if( alphaTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetAlphaTest::PropertyFlag_IsTestEnabled )
        {
            resAlphaTest.SetTestEnabled( alphaTest->IsTestEnabled );
        }
        if( alphaTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetAlphaTest::PropertyFlag_TestFunction )
        {
            resAlphaTest.SetTestFunc( testFuncList[alphaTest->TestFunction] );
        }
        if( alphaTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetAlphaTest::PropertyFlag_TestReference )
        {
            resAlphaTest.SetTestReference( alphaTest->TestReference );
        }
    }

    void
    SetAlphaTest(Protocol4C::MESSAGEPACKET_Material_SetAlphaTest *alphaTest)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResAlphaTest resAlphaTest = resFragmentShader.GetAlphaTest();
                SetAlphaTest( resAlphaTest, alphaTest );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResAlphaTest resAlphaTest = resFragmentShader.GetAlphaTest();
                SetAlphaTest( resAlphaTest, alphaTest );
            }
        }
    }

    // Depth test settings
    void
    SetDepthTest(nw::gfx::res::ResDepthOperation resDepthOperation, Protocol4C::MESSAGEPACKET_Material_SetDepthTest *depthTest)
    {
        const nw::gfx::res::ResDepthOperation::TestFunc testFuncList[] = {
            nw::gfx::res::ResDepthOperation::TESTFUNC_NEVER,     // TestFunctionCtr_Never = 0,           // No matter what the value, it will not pass through.
            nw::gfx::res::ResDepthOperation::TESTFUNC_ALWAYS,    // TestFunctionCtr_Always = 1,           // No matter what the value, it will pass through.
            nw::gfx::res::ResDepthOperation::TESTFUNC_LESS,      // TestFunctionCtr_Less = 2,           // If the value is smaller than the reference value, it will pass through.
            nw::gfx::res::ResDepthOperation::TESTFUNC_LEQUAL,    // TestFunctionCtr_Lequal = 3,           // If the value is at or below the reference value, it will pass through.
            nw::gfx::res::ResDepthOperation::TESTFUNC_EQUAL,     // TestFunctionCtr_Equal = 4,           // If the value is equal to the reference value, it will pass through.
            nw::gfx::res::ResDepthOperation::TESTFUNC_GEQUAL,    // TestFunctionCtr_Gequal = 5,           // If the value is at or below the reference value, it will pass through.
            nw::gfx::res::ResDepthOperation::TESTFUNC_GREATER ,  // TestFunctionCtr_Greater = 6,           // If the value is larger than the reference value, it will pass through.
            nw::gfx::res::ResDepthOperation::TESTFUNC_NOTEQUAL,  // TestFunctionCtr_Notequal = 7,           // If the value is equal to the reference value, it will pass through.
        };

        if( depthTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetDepthTest::PropertyFlag_IsTestEnabled )
        {
            SET_FLAGS( resDepthOperation, depthTest->IsTestEnabled, nw::gfx::res::ResDepthOperationData::FLAG_TEST_ENABLED )
        }
        if( depthTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetDepthTest::PropertyFlag_TestFunction )
        {
            resDepthOperation.SetTestFunc(testFuncList[depthTest->TestFunction]);
        }
        if( depthTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetDepthTest::PropertyFlag_IsMaskEnabled )
        {
            SET_FLAGS( resDepthOperation, depthTest->IsMaskEnabled, nw::gfx::res::ResDepthOperationData::FLAG_MASK_ENABLED )
        }
    }

    void
    SetDepthTest(Protocol4C::MESSAGEPACKET_Material_SetDepthTest *depthTest)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentOperation resFragmentOperation = resMaterial.GetFragmentOperation();
                nw::gfx::res::ResDepthOperation resDepthOperation = resFragmentOperation.GetDepthOperation();
                SetDepthTest( resDepthOperation, depthTest );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentOperation resFragmentOperation = resMaterial.GetFragmentOperation();
                nw::gfx::res::ResDepthOperation resDepthOperation = resFragmentOperation.GetDepthOperation();
                SetDepthTest( resDepthOperation, depthTest );
            }
        }
    }

    // Stencil test settings
    void
    SetStencilTest(Protocol4C::MESSAGEPACKET_Material_SetStencilTest *stencilTest)
    {
        const nw::gfx::res::ResStencilOperation::StencilOp stencilOpList[] = {
            nw::gfx::res::ResStencilOperation::STENCILOP_KEEP,           // StencilTestOperationCtr_Keep = 0,           // Maintains current value.
            nw::gfx::res::ResStencilOperation::STENCILOP_ZERO,           // StencilTestOperationCtr_Zero = 1,           // Replaces with 0.
            nw::gfx::res::ResStencilOperation::STENCILOP_REPLACE,        // StencilTestOperationCtr_Replace = 2,           // Replaces with reference value.
            nw::gfx::res::ResStencilOperation::STENCILOP_INCREASE,       // StencilTestOperationCtr_Increase = 3,           // Increases. Clamping takes place if the result of the increase is exceeding the extreme value.
            nw::gfx::res::ResStencilOperation::STENCILOP_INCREASE_WRAP,  // StencilTestOperationCtr_IncreaseWrap = 4,           // Increases. Wrapping takes place if the result of the increase is exceeding the extreme value.
            nw::gfx::res::ResStencilOperation::STENCILOP_DECREASE,       // StencilTestOperationCtr_Decrease = 5,           // Decreases. Clamping takes place if the result of the decrease is exceeding the extreme value.
            nw::gfx::res::ResStencilOperation::STENCILOP_DECREASE_WRAP,  // StencilTestOperationCtr_DecreaseWrap = 6,           // Decrease. Wrapping takes place if the result of the decrease is exceeding the extreme value.
            nw::gfx::res::ResStencilOperation::STENCILOP_INVERT,         // StencilTestOperationCtr_Invert = 7,           // Inverts in bit units.
        };

        const nw::gfx::res::ResStencilOperation::TestFunc testFuncList[] = {
            nw::gfx::res::ResStencilOperation::TESTFUNC_NEVER,       // TestFunctionCtr_Never = 0,           // No matter what the value, it will not pass through.
            nw::gfx::res::ResStencilOperation::TESTFUNC_ALWAYS,      // TestFunctionCtr_Always = 1,           // No matter what the value, it will pass through.
            nw::gfx::res::ResStencilOperation::TESTFUNC_LESS,        // TestFunctionCtr_Less = 2,           // If the value is smaller than the reference value, it will pass through.
            nw::gfx::res::ResStencilOperation::TESTFUNC_LEQUAL,      // TestFunctionCtr_Lequal = 3,           // If the value is at or below the reference value, it will pass through.
            nw::gfx::res::ResStencilOperation::TESTFUNC_EQUAL,       // TestFunctionCtr_Equal = 4,           // If the value is equal to the reference value, it will pass through.
            nw::gfx::res::ResStencilOperation::TESTFUNC_GEQUAL,      // TestFunctionCtr_Gequal = 5,           // If the value is at or below the reference value, it will pass through.
            nw::gfx::res::ResStencilOperation::TESTFUNC_GREATER,     // TestFunctionCtr_Greater = 6,           // If the value is larger than the reference value, it will pass through.
            nw::gfx::res::ResStencilOperation::TESTFUNC_NOTEQUAL,    // TestFunctionCtr_Notequal = 7,           // If the value is equal to the reference value, it will pass through.
        };

        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentOperation resFragmentOperation = resMaterial.GetFragmentOperation();
                nw::gfx::res::ResStencilOperation resStencilOperation = resFragmentOperation.GetStencilOperation();

                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_IsTestEnabled )
                {
                    resStencilOperation.SetTestEnabled( stencilTest->IsTestEnabled );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_TestFunction )
                {
                    resStencilOperation.SetTestFunction( testFuncList[stencilTest->TestFunction] );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_TestReference )
                {
                    resStencilOperation.SetTestReference( stencilTest->TestReference );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_TestMask )
                {
                    resStencilOperation.SetTestMask( stencilTest->TestMask );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_FailOperation )
                {
                    resStencilOperation.SetFailOperation( stencilOpList[stencilTest->FailOperation] );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_ZFailOperation )
                {
                    resStencilOperation.SetZFailOperation( stencilOpList[stencilTest->ZFailOperation] );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_PassOperation )
                {
                    resStencilOperation.SetPassOperation( stencilOpList[stencilTest->PassOperation] );
                }
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentOperation resFragmentOperation = resMaterial.GetFragmentOperation();
                nw::gfx::res::ResStencilOperation resStencilOperation = resFragmentOperation.GetStencilOperation();

                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_IsTestEnabled )
                {
                    resStencilOperation.SetTestEnabled( stencilTest->IsTestEnabled );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_TestFunction )
                {
                    resStencilOperation.SetTestFunction( testFuncList[stencilTest->TestFunction] );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_TestReference )
                {
                    resStencilOperation.SetTestReference( stencilTest->TestReference );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_TestMask )
                {
                    resStencilOperation.SetTestMask( stencilTest->TestMask );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_FailOperation )
                {
                    resStencilOperation.SetFailOperation( stencilOpList[stencilTest->FailOperation] );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_ZFailOperation )
                {
                    resStencilOperation.SetZFailOperation( stencilOpList[stencilTest->ZFailOperation] );
                }
                if( stencilTest->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetStencilTest::PropertyFlag_PassOperation )
                {
                    resStencilOperation.SetPassOperation( stencilOpList[stencilTest->PassOperation] );
                }
            }
        }
    }

    void
    SetShaderProgramDescriptionIndex(Protocol4C::MESSAGEPACKET_Material_SetShaderProgramDescriptionIndex *body)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                resMaterial.SetShaderProgramDescriptionIndex( body->ShaderProgramDescriptionIndex );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                resMaterial.SetShaderProgramDescriptionIndex( body->ShaderProgramDescriptionIndex );
            }
        }
    }

    // Blend settings
    void
    SetBlend(nw::gfx::res::ResBlendOperation resBlendOperation, Protocol4C::MESSAGEPACKET_Material_SetBlend *blend)
    {
        const nw::gfx::res::ResBlendOperation::LogicOp logicList[] = {
            nw::gfx::res::ResBlendOperation::LOGICOP_CLEAR,          // LogicOperationCtr_Clear = 0,           // Sets all bit values to 0.
            nw::gfx::res::ResBlendOperation::LOGICOP_COPY,           // LogicOperationCtr_Copy = 1,           // Takes the value of the source color bits.
            nw::gfx::res::ResBlendOperation::LOGICOP_NOOP,           // LogicOperationCtr_NoOp = 2,           // Takes the value of the destination color bits.
            nw::gfx::res::ResBlendOperation::LOGICOP_SET,            // LogicOperationCtr_Set = 3,           // Sets all bit values to 1.
            nw::gfx::res::ResBlendOperation::LOGICOP_COPY_INVERTED,  // LogicOperationCtr_CopyInverted = 4,           // Takes the inverted bit value of the source color.
            nw::gfx::res::ResBlendOperation::LOGICOP_INVERT,         // LogicOperationCtr_Invert = 5,           // Takes the inverted bit value of the destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_AND_REVERSE,    // LogicOperationCtr_AndReverse = 6,           // Takes the bitwise AND of the inverted bit values of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_OR_REVERSE,     // LogicOperationCtr_OrReverse = 7,           // Takes the bitwise OR of the inverted bit values of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_AND,            // LogicOperationCtr_And = 8,           // Takes the bitwise AND of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_OR,             // LogicOperationCtr_Or = 9,           // Takes the bitwise OR of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_NAND,           // LogicOperationCtr_Nand = 10,           // Takes the inverted bit value of the bitwise AND of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_NOR,            // LogicOperationCtr_Nor = 11,           // Takes the inverted bit value of the bitwise OR of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_XOR,            // LogicOperationCtr_Xor = 12,           // Takes a bitwise XOR of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_EQUIV,          // LogicOperationCtr_Equiv = 13,           // Takes the inverted bit value of the bitwise XOR of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_AND_INVERTED,   // LogicOperationCtr_AndInverted = 14,           // Takes the bitwise AND of the inverted bit value of the source color and destination color.
            nw::gfx::res::ResBlendOperation::LOGICOP_OR_INVERTED     // LogicOperationCtr_OrInverted = 15,           // Takes the bitwise OR of the inverted bit value of the source color and destination color.
        };

        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_Mode )
        {
#if NW4C_VIEWER_VERSION >= NW4C_VIEWER_VERSION_NUMBER(2,1,0,0)
            switch (blend->Mode)
            {
            case Protocol4C::BlendModeCtr_NotUsed:
            case Protocol4C::BlendModeCtr_SeparateBlend:
                resBlendOperation.SetBlendMode(true);
                break;
            case Protocol4C::BlendModeCtr_Logic:
                resBlendOperation.SetBlendMode(false);
                break;
            case Protocol4C::BlendModeCtr_Blend:
            default:
                break;
            }
#else
            resBlendOperation.SetMode((nw::gfx::res::ResBlendOperation::Mode)blend->Mode);
#endif
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_LogicOperation )
        {
            resBlendOperation.SetLogicOperation(logicList[blend->LogicOperation]);
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_RgbFuncSource )
        {
            resBlendOperation.SetBlendFuncSrcRgb((nw::gfx::res::ResBlendOperation::BlendFactor)blend->RgbFuncSource);
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_RgbFuncDestination )
        {
            resBlendOperation.SetBlendFuncDstRgb((nw::gfx::res::ResBlendOperation::BlendFactor)blend->RgbFuncDestination);
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_RgbEquation )
        {
            resBlendOperation.SetBlendEquationRgb((nw::gfx::res::ResBlendOperation::BlendEquation)blend->RgbEquation);
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_AlphaFuncSource )
        {
            resBlendOperation.SetBlendFuncSrcAlpha((nw::gfx::res::ResBlendOperation::BlendFactor)blend->AlphaFuncSource);
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_AlphaFuncDestination )
        {
            resBlendOperation.SetBlendFuncDstAlpha((nw::gfx::res::ResBlendOperation::BlendFactor)blend->AlphaFuncDestination);
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_AlphaEquation )
        {
            resBlendOperation.SetBlendEquationAlpha((nw::gfx::res::ResBlendOperation::BlendEquation)blend->AlphaEquation);
        }
        if( blend->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetBlend::PropertyFlag_BlendColor )
        {
            resBlendOperation.SetBlendColor( blend->BlendColor.R, blend->BlendColor.G, blend->BlendColor.B, blend->BlendColor.A );
        }
    }

    void
    SetBlend(Protocol4C::MESSAGEPACKET_Material_SetBlend *blend)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentOperation resFragmentOperation = resMaterial.GetFragmentOperation();
                nw::gfx::res::ResBlendOperation resBlendOperation = resFragmentOperation.GetBlendOperation();
                SetBlend( resBlendOperation, blend );
            }

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentOperation resFragmentOperation = resMaterial.GetFragmentOperation();
                nw::gfx::res::ResBlendOperation resBlendOperation = resFragmentOperation.GetBlendOperation();
                SetBlend( resBlendOperation, blend );
            }
        }
    }

    // Texture combiner settings
    void
    SetTextureCombiner(
        nw::gfx::res::ResFragmentShader resFragmentShader,
        Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner *combiner
    )
    {
        const nw::gfx::res::ResTextureCombiner::Source sourceList[] = {
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE0,                   // TextureCombinerSourceKind_Texture0 = 0,           // Texture 0.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE1,                   // TextureCombinerSourceKind_Texture1 = 1,           // Texture 1.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE2,                   // TextureCombinerSourceKind_Texture2 = 2,           // Texture 2.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE3,                   // TextureCombinerSourceKind_Texture3 = 3,           // Texture 3.
            nw::gfx::res::ResTextureCombiner::SOURCE_CONSTANT,                   // TextureCombinerSourceKind_Constant = 4,           // Constant color.
            nw::gfx::res::ResTextureCombiner::SOURCE_PRIMARY_COLOR,              // TextureCombinerSourceKind_PrimaryColor = 5,           // This is the fragment color that enters texture combiner 0 before texture processing.
            nw::gfx::res::ResTextureCombiner::SOURCE_PREVIOUS,                   // TextureCombinerSourceKind_Previous = 6,           // This is the fragment color that enters from the previous texture combiner.
            nw::gfx::res::ResTextureCombiner::SOURCE_PREVIOUS_BUFFER,            // TextureCombinerSourceKind_PreviousBuffer = 7,           // This is the color that enters from the previous texture combiner buffer.
            nw::gfx::res::ResTextureCombiner::SOURCE_FRAGMENT_PRIMARY_COLOR,     // TextureCombinerSourceKind_FragmentPrimaryColor = 8,           // Fragment light primary color (representing diffused, ambient, and emissive).
            nw::gfx::res::ResTextureCombiner::SOURCE_FRAGMENT_SECONDARY_COLOR,   //TextureCombinerSourceKind_FragmentSecondaryColor = 9,           // Fragment light secondary color (representing specular).
        };
        const nw::gfx::res::ResTextureCombiner::OperandRgb operandRGBList[] = {
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_SRC_COLOR,          // TextureCombinerOperandKind_Color = 0,           // Source color.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_SRC_COLOR,// TextureCombinerOperandKind_OneMinusColor = 1,           // 1 color
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_SRC_ALPHA,          // TextureCombinerOperandKind_Alpha = 2,           // Source alpha value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_SRC_ALPHA,// TextureCombinerOperandKind_OneMinusAlpha = 3,           // 1 alpha.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_RED,                // TextureCombinerOperandKind_Red = 4,           // Source red value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_GREEN,              // TextureCombinerOperandKind_Green = 5,           // Source green value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_BLUE,               // TextureCombinerOperandKind_Blue = 6,           // Source blue value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_RED,      // TextureCombinerOperandKind_OneMinusRed = 7,           // 1 red.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_GREEN,    // TextureCombinerOperandKind_OneMinusGreen = 8,           // 1 green.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_BLUE,     // TextureCombinerOperandKind_OneMinusBlue = 9,           // 1 blue.
        };
        const nw::gfx::res::ResTextureCombiner::OperandAlpha operandAlphaBList[] = {
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ALPHA,            // TextureCombinerOperandKind_Color = 0,           // Source color.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ONE_MINUS_ALPHA,  // TextureCombinerOperandKind_OneMinusColor = 1,           // 1 color
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ALPHA,            // TextureCombinerOperandKind_Alpha = 2,           // Source alpha value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ONE_MINUS_ALPHA,  // TextureCombinerOperandKind_OneMinusAlpha = 3,           // 1 alpha.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_RED,                  // TextureCombinerOperandKind_Red = 4,           // Source red value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_GREEN,                // TextureCombinerOperandKind_Green = 5,           // Source green value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_BLUE,                 // TextureCombinerOperandKind_Blue = 6,           // Source blue value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_RED,        // TextureCombinerOperandKind_OneMinusRed = 7,           // 1 red.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_GREEN,      // TextureCombinerOperandKind_OneMinusGreen = 8,           // 1 green.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_BLUE,       // TextureCombinerOperandKind_OneMinusBlue = 9,           // 1 blue.
        };
        const nw::gfx::res::ResTextureCombiner::Combine combineList[] = {
            nw::gfx::res::ResTextureCombiner::COMBINE_REPLACE,       // TextureCombineMode_Replace = 0,           // Uses operation 0.
            nw::gfx::res::ResTextureCombiner::COMBINE_MODULATE,      // TextureCombineMode_Modulate = 1,           // Multiplies operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADD,           // TextureCombineMode_Add = 2,           // Adds operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADDSIGNED,     // TextureCombineMode_AddSigned = 3,           // Subtracts 0.5 from the addition of operation 0 and operation1.
            nw::gfx::res::ResTextureCombiner::COMBINE_INTERPOLATE,   // TextureCombineMode_Interpolate = 4,           // Interpolates by operation 0 and operation 1 for operation 2.
            nw::gfx::res::ResTextureCombiner::COMBINE_SUBTRACT,      // TextureCombineMode_Subtract = 5,           // Subtracts operation 1 from operation 0.
            nw::gfx::res::ResTextureCombiner::COMBINE_DOT3_RGB,      // TextureCombineMode_DotProduct3Rgb = 6,           // Replaces each of R, G, and B with the inner product of operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_DOT3_RGBA,     // TextureCombineMode_DotProduct3Rgba = 7,           // Replaces each of R, G, B, and A with the inner product of operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADD_MULT,      // TextureCombineMode_AddMult = 8,           // Multiplies the addition of operation 0 and operation 1 with operation 2.
            nw::gfx::res::ResTextureCombiner::COMBINE_MULT_ADD,      // TextureCombineMode_MultAdd = 9,           // Adds the multiplication of operation 0 and operation 1 with operation 2.
        };

        int nCombinerCount = resFragmentShader.GetTextureCombinersCount();
        if( nCombinerCount > combiner->Index )
        {
            nw::gfx::res::ResTextureCombiner resTextureCombiner = resFragmentShader.GetTextureCombiners( combiner->Index );

            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_CombineRgb )
            {
                resTextureCombiner.SetCombineRgb(combineList[combiner->CombineRgb]);
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_CombineAlpha )
            {
                resTextureCombiner.SetCombineAlpha(combineList[combiner->CombineAlpha]);
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_ScaleRgb )
            {
                resTextureCombiner.SetScaleRgb((nw::gfx::res::ResTextureCombiner::Scale)combiner->ScaleRgb);
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_ScaleAlpha )
            {
                resTextureCombiner.SetScaleAlpha((nw::gfx::res::ResTextureCombiner::Scale)combiner->ScaleAlpha);
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_Constant )
            {
                resTextureCombiner.SetConstant((nw::gfx::res::ResTextureCombiner::Constant)combiner->Constant);
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_BufferInputRgb )
            {
                if( combiner->Index >= 1 && combiner->Index <= 4 )
                {
                    if( combiner->BufferInputRgb == Protocol4C::TextureCombineBufferInput_Previous ){
                        resFragmentShader.SetBufferInputRgb( combiner->Index, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS );
                    }
                    else
                    {
                        resFragmentShader.SetBufferInputRgb( combiner->Index, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS_BUFFER );
                    }
                }
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_BufferInputAlpha )
            {
                if( combiner->Index >= 1 && combiner->Index <= 4 )
                {
                    if( combiner->BufferInputAlpha == Protocol4C::TextureCombineBufferInput_Previous ){
                        resFragmentShader.SetBufferInputAlpha( combiner->Index, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS );
                    }
                    else
                    {
                        resFragmentShader.SetBufferInputAlpha( combiner->Index, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS_BUFFER );
                    }
                }
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_OperandRgb0 )
            {
                resTextureCombiner.SetOperandRgb( 0, operandRGBList[combiner->OperandRgb0] );
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_OperandRgb1 )
            {
                resTextureCombiner.SetOperandRgb( 1, operandRGBList[combiner->OperandRgb1] );
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_OperandRgb2 )
            {
                resTextureCombiner.SetOperandRgb( 2, operandRGBList[combiner->OperandRgb2] );
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_OperandAlpha0 )
            {
                resTextureCombiner.SetOperandAlpha( 0, operandAlphaBList[combiner->OperandAlpha0] );
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_OperandAlpha1 )
            {
                resTextureCombiner.SetOperandAlpha( 1, operandAlphaBList[combiner->OperandAlpha1] );
            }
            if( combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_OperandAlpha2 )
            {
                resTextureCombiner.SetOperandAlpha( 2, operandAlphaBList[combiner->OperandAlpha2] );
            }

            // Simultaneously sets SourceRgb and SourceAlpha for ErrorCheck.
            if(
                combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_SourceRgb0 ||
                combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_SourceRgb1 ||
                combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_SourceRgb2 ||
                combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_SourceAlpha0 ||
                combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_SourceAlpha1 ||
                combiner->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner::PropertyFlag_SourceAlpha2
            ) {
                resTextureCombiner.SetSourceRgb( 0, sourceList[combiner->SourceRgb0] );
                resTextureCombiner.SetSourceRgb( 1, sourceList[combiner->SourceRgb1] );
                resTextureCombiner.SetSourceRgb( 2, sourceList[combiner->SourceRgb2] );
                resTextureCombiner.SetSourceAlpha( 0, sourceList[combiner->SourceAlpha0] );
                resTextureCombiner.SetSourceAlpha( 1, sourceList[combiner->SourceAlpha1] );
                resTextureCombiner.SetSourceAlpha( 2, sourceList[combiner->SourceAlpha2] );
            }
        }
    }

    void
    SetTextureCombiner(Protocol4C::MESSAGEPACKET_Material_SetTextureCombiner *combiner)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                SetTextureCombiner( resFragmentShader, combiner );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                SetTextureCombiner( resFragmentShader, combiner );
            }

            // ErrorCheck
            ErrorCheck::CheckModelTextureCombiner( model );
        }
    }

    // Texture combiner settings
    void
    SetTextureCombiner(
        int nIndex,
        nw::gfx::res::ResFragmentShader resFragmentShader,
        Protocol4C::TextureCombiner *combiner
    )
    {
        const nw::gfx::res::ResTextureCombiner::Source sourceList[] = {
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE0,                   // TextureCombinerSourceKind_Texture0 = 0,           // Texture 0.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE1,                   // TextureCombinerSourceKind_Texture1 = 1,           // Texture 1.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE2,                   // TextureCombinerSourceKind_Texture2 = 2,           // Texture 2.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE3,                   // TextureCombinerSourceKind_Texture3 = 3,           // Texture 3.
            nw::gfx::res::ResTextureCombiner::SOURCE_CONSTANT,                   // TextureCombinerSourceKind_Constant = 4,           // Constant color.
            nw::gfx::res::ResTextureCombiner::SOURCE_PRIMARY_COLOR,              // TextureCombinerSourceKind_PrimaryColor = 5,           // This is the fragment color that enters texture combiner 0 before texture processing.
            nw::gfx::res::ResTextureCombiner::SOURCE_PREVIOUS,                   // TextureCombinerSourceKind_Previous = 6,           // This is the fragment color that enters from the previous texture combiner.
            nw::gfx::res::ResTextureCombiner::SOURCE_PREVIOUS_BUFFER,            // TextureCombinerSourceKind_PreviousBuffer = 7,           // This is the color that enters from the previous texture combiner buffer.
            nw::gfx::res::ResTextureCombiner::SOURCE_FRAGMENT_PRIMARY_COLOR,     // TextureCombinerSourceKind_FragmentPrimaryColor = 8,           // Fragment light primary color (representing diffused, ambient, and emissive).
            nw::gfx::res::ResTextureCombiner::SOURCE_FRAGMENT_SECONDARY_COLOR,   //TextureCombinerSourceKind_FragmentSecondaryColor = 9,           // Fragment light secondary color (representing specular).
        };
        const nw::gfx::res::ResTextureCombiner::OperandRgb operandRGBList[] = {
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_SRC_COLOR,          // TextureCombinerOperandKind_Color = 0,           // Source color.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_SRC_COLOR,// TextureCombinerOperandKind_OneMinusColor = 1,           // 1 color
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_SRC_ALPHA,          // TextureCombinerOperandKind_Alpha = 2,           // Source alpha value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_SRC_ALPHA,// TextureCombinerOperandKind_OneMinusAlpha = 3,           // 1 alpha.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_RED,                // TextureCombinerOperandKind_Red = 4,           // Source red value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_GREEN,              // TextureCombinerOperandKind_Green = 5,           // Source green value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_BLUE,               // TextureCombinerOperandKind_Blue = 6,           // Source blue value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_RED,      // TextureCombinerOperandKind_OneMinusRed = 7,           // 1 red.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_GREEN,    // TextureCombinerOperandKind_OneMinusGreen = 8,           // 1 green.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_BLUE,     // TextureCombinerOperandKind_OneMinusBlue = 9,           // 1 blue.
        };
        const nw::gfx::res::ResTextureCombiner::OperandAlpha operandAlphaBList[] = {
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ALPHA,            // TextureCombinerOperandKind_Color = 0,           // Source color.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ONE_MINUS_ALPHA,  // TextureCombinerOperandKind_OneMinusColor = 1,           // 1 color
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ALPHA,            // TextureCombinerOperandKind_Alpha = 2,           // Source alpha value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ONE_MINUS_ALPHA,  // TextureCombinerOperandKind_OneMinusAlpha = 3,           // 1 alpha.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_RED,                  // TextureCombinerOperandKind_Red = 4,           // Source red value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_GREEN,                // TextureCombinerOperandKind_Green = 5,           // Source green value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_BLUE,                 // TextureCombinerOperandKind_Blue = 6,           // Source blue value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_RED,        // TextureCombinerOperandKind_OneMinusRed = 7,           // 1 red.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_GREEN,      // TextureCombinerOperandKind_OneMinusGreen = 8,           // 1 green.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_BLUE,       // TextureCombinerOperandKind_OneMinusBlue = 9,           // 1 blue.
        };
        const nw::gfx::res::ResTextureCombiner::Combine combineList[] = {
            nw::gfx::res::ResTextureCombiner::COMBINE_REPLACE,       // TextureCombineMode_Replace = 0,           // Uses operation 0.
            nw::gfx::res::ResTextureCombiner::COMBINE_MODULATE,      // TextureCombineMode_Modulate = 1,           // Multiplies operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADD,           // TextureCombineMode_Add = 2,           // Adds operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADDSIGNED,     // TextureCombineMode_AddSigned = 3,           // Subtracts 0.5 from the addition of operation 0 and operation1.
            nw::gfx::res::ResTextureCombiner::COMBINE_INTERPOLATE,   // TextureCombineMode_Interpolate = 4,           // Interpolates by operation 0 and operation 1 for operation 2.
            nw::gfx::res::ResTextureCombiner::COMBINE_SUBTRACT,      // TextureCombineMode_Subtract = 5,           // Subtracts operation 1 from operation 0.
            nw::gfx::res::ResTextureCombiner::COMBINE_DOT3_RGB,      // TextureCombineMode_DotProduct3Rgb = 6,           // Replaces each of R, G, and B with the inner product of operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_DOT3_RGBA,     // TextureCombineMode_DotProduct3Rgba = 7,           // Replaces each of R, G, B, and A with the inner product of operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADD_MULT,      // TextureCombineMode_AddMult = 8,           // Multiplies the addition of operation 0 and operation 1 with operation 2.
            nw::gfx::res::ResTextureCombiner::COMBINE_MULT_ADD,      // TextureCombineMode_MultAdd = 9,           // Adds the multiplication of operation 0 and operation 1 with operation 2.
        };

        int nCombinerCount = resFragmentShader.GetTextureCombinersCount();
        if( nCombinerCount > nIndex )
        {
            nw::gfx::res::ResTextureCombiner resTextureCombiner = resFragmentShader.GetTextureCombiners( nIndex );

            resTextureCombiner.SetCombineRgb(combineList[combiner->CombineRgb]);
            resTextureCombiner.SetCombineAlpha(combineList[combiner->CombineAlpha]);
            resTextureCombiner.SetScaleRgb((nw::gfx::res::ResTextureCombiner::Scale)combiner->ScaleRgb);
            resTextureCombiner.SetScaleAlpha((nw::gfx::res::ResTextureCombiner::Scale)combiner->ScaleAlpha);
            resTextureCombiner.SetConstant((nw::gfx::res::ResTextureCombiner::Constant)combiner->Constant);
            if( nIndex >= 1 && nIndex <= 4 ){   // Can set stages 1 to 4 only.
                if( combiner->BufferInputRgb == Protocol4C::TextureCombineBufferInput_Previous ){
                    resFragmentShader.SetBufferInputRgb( nIndex, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS );
                }
                else
                {
                    resFragmentShader.SetBufferInputRgb( nIndex, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS_BUFFER );
                }
                if( combiner->BufferInputAlpha == Protocol4C::TextureCombineBufferInput_Previous ){
                    resFragmentShader.SetBufferInputAlpha( nIndex, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS );
                }
                else
                {
                    resFragmentShader.SetBufferInputAlpha( nIndex, nw::gfx::res::ResFragmentShader::BUFFERINPUT_PREVIOUS_BUFFER );
                }
            }
            resTextureCombiner.SetSourceRgb( 0, sourceList[combiner->SourceRgb0] );
            resTextureCombiner.SetSourceRgb( 1, sourceList[combiner->SourceRgb1] );
            resTextureCombiner.SetSourceRgb( 2, sourceList[combiner->SourceRgb2] );
            resTextureCombiner.SetOperandRgb( 0, operandRGBList[combiner->OperandRgb0] );
            resTextureCombiner.SetOperandRgb( 1, operandRGBList[combiner->OperandRgb1] );
            resTextureCombiner.SetOperandRgb( 2, operandRGBList[combiner->OperandRgb2] );
            resTextureCombiner.SetSourceAlpha( 0, sourceList[combiner->SourceAlpha0] );
            resTextureCombiner.SetSourceAlpha( 1, sourceList[combiner->SourceAlpha1] );
            resTextureCombiner.SetSourceAlpha( 2, sourceList[combiner->SourceAlpha2] );
            resTextureCombiner.SetOperandAlpha( 0, operandAlphaBList[combiner->OperandAlpha0] );
            resTextureCombiner.SetOperandAlpha( 1, operandAlphaBList[combiner->OperandAlpha1] );
            resTextureCombiner.SetOperandAlpha( 2, operandAlphaBList[combiner->OperandAlpha2] );
        }
    }

    void
    SetTextureCombiner(int nIndex, Protocol4C::TextureCombiner *combiner)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                SetTextureCombiner( nIndex, resFragmentShader, combiner );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                SetTextureCombiner( nIndex, resFragmentShader, combiner );
            }

            // ErrorCheck
            ErrorCheck::CheckModelTextureCombiner( model );
        }
    }

    // Layer configuration settings
    void
    SetFShaderConfig(Protocol4C::FragmentShaderConfig config)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetLayerConfig((nw::gfx::res::ResFragmentLighting::Config)config);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetLayerConfig((nw::gfx::res::ResFragmentLighting::Config)config);
            }
        }
    }

    // Fog configuration
    void
    SetFog(nw::gfx::res::ResMaterial resMaterial, Protocol4C::MESSAGEPACKET_Material_SetFog *fog)
    {
        if( fog->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFog::PropertyFlag_Enable )
        {
            SET_FLAGS( resMaterial, fog->Enable, nw::gfx::res::ResMaterialData::FLAG_FOG_ENABLED )
        }
        if( fog->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFog::PropertyFlag_FogIndex )
        {
            resMaterial.SetFogIndex( fog->FogIndex );
        }
    }

    void
    SetFog(Protocol4C::MESSAGEPACKET_Material_SetFog *fog)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetFog( resMaterial, fog );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetFog( resMaterial, fog );
            }

            // ErrorCheck
            ErrorCheck::CheckModelFogIndexBoundary( model );
            ErrorCheck::CheckModelFogIndexReference( model );
        }
    }

    // light settings
    void
    SetLight(nw::gfx::res::ResMaterial resMaterial, Protocol4C::MESSAGEPACKET_Material_SetLight *light)
    {
        if( light->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetLight::PropertyFlag_IsFragmentLightEnabled )
        {
            SET_FLAGS( resMaterial, light->IsFragmentLightEnabled, nw::gfx::res::ResMaterialData::FLAG_FRAGMENTLIGHT_ENABLED )
        }
        if( light->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetLight::PropertyFlag_IsVertexLightEnabled )
        {
            SET_FLAGS( resMaterial, light->IsVertexLightEnabled, nw::gfx::res::ResMaterialData::FLAG_VERTEXLIGHT_ENABLED )
        }
        if( light->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetLight::PropertyFlag_IsHemiSphereLightEnabled )
        {
            SET_FLAGS( resMaterial, light->IsHemiSphereLightEnabled, nw::gfx::res::ResMaterialData::FLAG_HEMISPHERELIGHT_ENABLED )
        }
        if( light->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetLight::PropertyFlag_IsHemiSphereOcclusionEnabled )
        {
            SET_FLAGS( resMaterial, light->IsHemiSphereOcclusionEnabled, nw::gfx::res::ResMaterialData::FLAG_HEMISPHERE_OCCLUSION_ENABLED )
        }
        if( light->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetLight::PropertyFlag_LightSetIndex )
        {
            resMaterial.SetLightSetIndex( light->LightSetIndex );
        }
    }

    void
    SetLight(Protocol4C::MESSAGEPACKET_Material_SetLight *light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetLight( resMaterial, light );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetLight( resMaterial, light );
            }

            // ErrorCheck
            ErrorCheck::CheckModelLightSetIndexBoundary( model );
            ErrorCheck::CheckModelLightSetIndexReference( model );
        }
    }

    // Fragment lighting settings
    void
    SetFragmentLighting(
        nw::gfx::res::ResFragmentLighting resFragmentLighting,
        Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting *lighting
    )
    {
        const nw::gfx::res::ResFragmentLighting::FresnelConfig fresnelList[] = {
            nw::gfx::res::ResFragmentLighting::CONFIG_NO_FRESNEL,            // FresnelConfigCtr_No = 0,           // Not applied to either primary or secondary colors.
            nw::gfx::res::ResFragmentLighting::CONFIG_PRI_ALPHA_FRESNEL,     // FresnelConfigCtr_Pri = 1,           // Applies only to the alpha component of primary colors.
            nw::gfx::res::ResFragmentLighting::CONFIG_SEC_ALPHA_FRESNEL,     // FresnelConfigCtr_Sec = 2,           // Applies only to the alpha component of secondary colors.
            nw::gfx::res::ResFragmentLighting::CONFIG_PRI_SEC_ALPHA_FRESNEL, // FresnelConfigCtr_PriSec = 3,           // Applies to the alpha component of of both primary and secordary colors.
        };

        if( lighting->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting::PropertyFlag_FresnelConfig )
        {
            resFragmentLighting.SetFresnelConfig( fresnelList[lighting->FresnelConfig] );
        }
        if( lighting->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting::PropertyFlag_IsClampHighLight )
        {
            SET_FLAGS( resFragmentLighting, lighting->IsClampHighLight, nw::gfx::res::ResFragmentLightingData::FLAG_CLAMP_HIGH_LIGHT )
        }
        if( lighting->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting::PropertyFlag_IsDistribution0Enabled )
        {
            SET_FLAGS( resFragmentLighting, lighting->IsDistribution0Enabled, nw::gfx::res::ResFragmentLightingData::FLAG_DISTRIBUTION0_ENABLED )
        }
        if( lighting->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting::PropertyFlag_IsDistribution1Enabled )
        {
            SET_FLAGS( resFragmentLighting, lighting->IsDistribution1Enabled, nw::gfx::res::ResFragmentLightingData::FLAG_DISTRIBUTION1_ENABLED )
        }
        if( lighting->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting::PropertyFlag_IsGeometricFactor0Enabled )
        {
            SET_FLAGS( resFragmentLighting, lighting->IsGeometricFactor0Enabled, nw::gfx::res::ResFragmentLightingData::FLAG_GEOMETRIC_FACTOR0_ENABLED )
        }
        if( lighting->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting::PropertyFlag_IsGeometricFactor1Enabled )
        {
            SET_FLAGS( resFragmentLighting, lighting->IsGeometricFactor1Enabled, nw::gfx::res::ResFragmentLightingData::FLAG_GEOMETRIC_FACTOR1_ENABLED )
        }
        if( lighting->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting::PropertyFlag_IsReflectionEnabled )
        {
            SET_FLAGS( resFragmentLighting, lighting->IsReflectionEnabled, nw::gfx::res::ResFragmentLightingData::FLAG_REFLECTION_ENABLED )
        }
    }

    void
    SetFragmentLighting(Protocol4C::MESSAGEPACKET_Material_SetFragmentLighting *lighting)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                SetFragmentLighting( resFragmentLighting, lighting );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                SetFragmentLighting( resFragmentLighting, lighting );
            }

            // ErrorCheck
            ErrorCheck::CheckModelLookupTableReference( model );
        }
    }

    // Unit assignment settings of texture coordinates
    void
    SetTexCoordConfig(Protocol4C::TextureCoordinateConfigCtr texcoordConfig)
    {
        const nw::gfx::res::ResMaterial::TextureCoordinateConfig texcoordList[] = {
            nw::gfx::res::ResMaterial::CONFIG_0120,  // TextureCoordinateConfigCtr_Config0120 = 0,      
            nw::gfx::res::ResMaterial::CONFIG_0110,  // TextureCoordinateConfigCtr_Config0110 = 1,           // Assigns coordinate numbers 0, 1, 1, and 0 to the texture units.
            nw::gfx::res::ResMaterial::CONFIG_0111,  // TextureCoordinateConfigCtr_Config0111 = 2,           // Assigns coordinate numbers 0, 1, 1, and 1 to the texture units.
            nw::gfx::res::ResMaterial::CONFIG_0112,  // TextureCoordinateConfigCtr_Config0112 = 3,           // Assigns coordinate numbers 0, 1, 1, and 2 to the texture units.
            nw::gfx::res::ResMaterial::CONFIG_0121,  // TextureCoordinateConfigCtr_Config0121 = 4,           // Assigns coordinate numbers 0, 1, 2, and 1 to the texture units.
            nw::gfx::res::ResMaterial::CONFIG_0122,  // TextureCoordinateConfigCtr_Config0122 = 5,           // Assigns coordinate numbers 0, 1, 2, and 2 to the texture units.
        };

        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                resMaterial.SetTextureCoordinateConfig(texcoordList[texcoordConfig]);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                resMaterial.SetTextureCoordinateConfig(texcoordList[texcoordConfig]);
            }
        }
    }

    // Texture MinFilter settings
    void
    SetTextureMinFilter(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        Protocol4C::TextureSamplerMinFilter minFilter
    )
    {
        int nTexMapperCount = resMaterial.GetTextureMappersCount();
        if( nIndex < nTexMapperCount )
        {
            nw::gfx::res::ResPixelBasedTextureMapper resPixelTexMapper = resMaterial.GetTextureMappers( nIndex );
            nw::gfx::res::ResTextureSampler resTexSampler = resPixelTexMapper.GetSampler();
            resTexSampler.SetMinFilter( (nw::gfx::res::ResTextureSampler::MinFilter)minFilter );
        }
    }

    void
    SetTextureMinFilter( unsigned int nIndex, Protocol4C::TextureSamplerMinFilter minFilter )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTextureMinFilter( nIndex, resMaterial, minFilter );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTextureMinFilter( nIndex, resMaterial, minFilter );
            }
        }
    }

    // Texture MagFilter settings
    void
    SetTextureMagFilter(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        Protocol4C::TextureSamplerMagFilter magFilter
    )
    {
        int nTexMapperCount = resMaterial.GetTextureMappersCount();
        if( nIndex < nTexMapperCount )
        {
            nw::gfx::res::ResPixelBasedTextureMapper resPixelTexMapper = resMaterial.GetTextureMappers( nIndex );
            nw::gfx::res::ResTextureSampler resTexSampler = resPixelTexMapper.GetSampler();
            resTexSampler.SetMagFilter( (nw::gfx::res::ResTextureSampler::MagFilter)magFilter );
        }
    }

    void
    SetTextureMagFilter( unsigned int nIndex, Protocol4C::TextureSamplerMagFilter magFilter )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTextureMagFilter( nIndex, resMaterial, magFilter );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTextureMagFilter( nIndex, resMaterial, magFilter );
            }
        }
    }

    // Texture Wrap settings
    void
    SetTextureWrap(
        WRAP_TYPE type,
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        Protocol4C::TextureSamplerWrap wrap
    )
    {
        const nw::gfx::res::ResStandardTextureSampler::Wrap wrapList[] = {
            nw::gfx::res::ResStandardTextureSampler::WRAP_REPEAT,            // TextureSamplerWrap_Repeat = 0,           // Repeat
            nw::gfx::res::ResStandardTextureSampler::WRAP_MIRRORED_REPEAT,   // TextureSamplerWrap_MirroredRepeat = 1,           // MirroredRepeat
            nw::gfx::res::ResStandardTextureSampler::WRAP_CLAMP_TO_EDGE,     // TextureSamplerWrap_ClampToEdge = 2,           // ClampToEdge
            nw::gfx::res::ResStandardTextureSampler::WRAP_CLAMP_TO_BORDER,   // TextureSamplerWrap_ClampToBorder = 3,           // ClampToBorder
        };

        int nTexMapperCount = resMaterial.GetTextureMappersCount();
        if( nIndex < nTexMapperCount )
        {
            nw::gfx::res::ResPixelBasedTextureMapper resPixelTexMapper = resMaterial.GetTextureMappers( nIndex );
            nw::gfx::res::ResTextureSampler resTexSampler = resPixelTexMapper.GetSampler();
            nw::gfx::res::ResStandardTextureSampler resStandardSampler = nw::ut::ResDynamicCast<nw::gfx::res::ResStandardTextureSampler>(resTexSampler);

            if( resStandardSampler.IsValid() )
            {
                if( type == Wrap_S )
                {
                    resStandardSampler.SetWrapS( wrapList[wrap] );
                }
                else
                {
                    resStandardSampler.SetWrapT( wrapList[wrap] );
                }
            }
        }
    }

    void
    SetTextureWrap( WRAP_TYPE type, unsigned int nIndex, Protocol4C::TextureSamplerWrap wrap )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTextureWrap( type, nIndex, resMaterial, wrap );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTextureWrap( type, nIndex, resMaterial, wrap );
            }
        }
    }

    // Texture MinLod settings
    void
    SetTextureMinLod(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        unsigned int minLod
    )
    {
        int nTexMapperCount = resMaterial.GetTextureMappersCount();
        if( nIndex < nTexMapperCount )
        {
            nw::gfx::res::ResPixelBasedTextureMapper resPixelTexMapper = resMaterial.GetTextureMappers( nIndex );
            nw::gfx::res::ResTextureSampler resTexSampler = resPixelTexMapper.GetSampler();
            nw::gfx::res::ResStandardTextureSampler resStandardSampler = nw::ut::ResDynamicCast<nw::gfx::res::ResStandardTextureSampler>(resTexSampler);

            if( resStandardSampler.IsValid() )
            {
                resStandardSampler.SetMinLod( minLod );
            }
        }
    }

    void
    SetTextureMinLod( unsigned int nIndex, unsigned int minLod )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTextureMinLod( nIndex, resMaterial, minLod );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTextureMinLod( nIndex, resMaterial, minLod );
            }
        }
    }

    // Texture LodBias setings
    void
    SetTextureLodBias(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        float lodBias
    )
    {
        int nTexMapperCount = resMaterial.GetTextureMappersCount();
        if( nIndex < nTexMapperCount )
        {
            nw::gfx::res::ResPixelBasedTextureMapper resPixelTexMapper = resMaterial.GetTextureMappers( nIndex );
            nw::gfx::res::ResTextureSampler resTexSampler = resPixelTexMapper.GetSampler();
            nw::gfx::res::ResStandardTextureSampler resStandardSampler = nw::ut::ResDynamicCast<nw::gfx::res::ResStandardTextureSampler>(resTexSampler);

            if( resStandardSampler.IsValid() )
            {
                resStandardSampler.SetLodBias( lodBias );
            }
        }
    }

    void
    SetTextureLodBias( unsigned int nIndex, float lodBias )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTextureLodBias( nIndex, resMaterial, lodBias );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTextureLodBias( nIndex, resMaterial, lodBias );
            }
        }
    }

    // Texture border color settings
    void
    SetTextureBorderColor(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        float *color
    )
    {
        int nTexMapperCount = resMaterial.GetTextureMappersCount();
        if( nIndex < nTexMapperCount )
        {
            nw::gfx::res::ResPixelBasedTextureMapper resPixelTexMapper = resMaterial.GetTextureMappers( nIndex );
            nw::gfx::res::ResTextureSampler resTexSampler = resPixelTexMapper.GetSampler();
            nw::gfx::res::ResStandardTextureSampler resStandardSampler = nw::ut::ResDynamicCast<nw::gfx::res::ResStandardTextureSampler>(resTexSampler);

            if( resStandardSampler.IsValid() )
            {
                resStandardSampler.SetBorderColor(color[0],color[1],color[2],color[3]);
            }
        }
    }

    void
    SetTextureBorderColor( unsigned int nIndex, float *color )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTextureBorderColor( nIndex, resMaterial, color );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTextureBorderColor( nIndex, resMaterial, color );
            }
        }
    }

    // Texture SouceCoordinate settings
    void
    SetTexcoordSourceCoord(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        int sourceCoord
    )
    {
        int nCoordNum = resMaterial.GetTextureCoordinatorsCount();
        if( nIndex < nCoordNum )
        {
            nw::gfx::res::ResTextureCoordinator texCoord = resMaterial.GetTextureCoordinators(nIndex);
            texCoord.SetSourceCoordinate(sourceCoord);
            texCoord.SetDirty(true);
        }
    }

    void
    SetTexcoordSourceCoord( unsigned int nIndex, int sourceCoord )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTexcoordSourceCoord( nIndex, resMaterial, sourceCoord );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTexcoordSourceCoord( nIndex, resMaterial, sourceCoord );
            }
        }
    }

    // Texture MappingMethod settings
    void
    SetTexcoordMappingMethod(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        Protocol4C::TextureMappingMethodCtr mappingMethod
    )
    {
        const nw::gfx::res::ResTextureCoordinator::MappingMethod mappingList[] = {
            nw::gfx::res::ResTextureCoordinator::MAPPINGMETHOD_UV_COORDINATE,        // TextureMappingMethodCtr_UvCoordinateMap = 0,           // UvCoordinateMap
            nw::gfx::res::ResTextureCoordinator::MAPPINGMETHOD_CAMERA_CUBE_ENV,      // TextureMappingMethodCtr_CameraCubeEnvMap = 1,           // CameraCubeEnvMap
            nw::gfx::res::ResTextureCoordinator::MAPPINGMETHOD_CAMERA_SPHERE_ENV,    // TextureMappingMethodCtr_CameraSphereEnvMap = 2,           // CameraSphereEnvMap
            nw::gfx::res::ResTextureCoordinator::MAPPINGMETHOD_PROJECTION,           // TextureMappingMethodCtr_ProjectionMap = 3,           // ProjectionMap
        };
        const int indices[nw::gfx::res::ResMaterial::CONFIG_NUM][3] = {
            { 0, 1, 2 }, // CONFIG_0120
            { 0, 1, 1 }, // CONFIG_0110
            { 0, 1, 1 }, // CONFIG_0111
            { 0, 1, 1 }, // CONFIG_0112
            { 0, 1, 2 }, // CONFIG_0121
            { 0, 1, 2 }, // CONFIG_0122
        };
        const nw::gfx::res::ResTextureSampler::SamplerType samplerTypeList[] = {
            nw::gfx::res::ResTextureSampler::SAMPLERTYPE_TEXTURE_2D,
            nw::gfx::res::ResTextureSampler::SAMPLERTYPE_CUBE_MAP,
            nw::gfx::res::ResTextureSampler::SAMPLERTYPE_TEXTURE_2D,
            nw::gfx::res::ResTextureSampler::SAMPLERTYPE_PROJECTION,
        };

        int nCoordNum = resMaterial.GetTextureCoordinatorsCount();
        if( nIndex < nCoordNum )
        {
            nw::gfx::res::ResTextureCoordinator texCoord = resMaterial.GetTextureCoordinators(nIndex);
            texCoord.SetMappingMethod(mappingList[mappingMethod]);
            texCoord.SetDirty(true);

            nw::gfx::res::ResPixelBasedTextureMapper resPixelTexMapper =
                resMaterial.GetTextureMappers(indices[resMaterial.GetTextureCoordinateConfig()][nIndex]);
            nw::gfx::res::ResTextureSampler resTexSampler = resPixelTexMapper.GetSampler();
            resTexSampler.SetSamplerType(samplerTypeList[mappingMethod]);
        }
    }

    void
    SetTexcoordMappingMethod( unsigned int nIndex, Protocol4C::TextureMappingMethodCtr mappingMethod )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTexcoordMappingMethod( nIndex, resMaterial, mappingMethod );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTexcoordMappingMethod( nIndex, resMaterial, mappingMethod );
            }

            // ErrorCheck
            ErrorCheck::CheckModelCameraReferenceIndexBoundary( model );
            ErrorCheck::CheckModelCameraReferenceIndexReference( model );
        }
    }

    // Texture ReferenceCamera settings
    void
    SetTexcoordReferenceCamera(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        int referenceCamera
    )
    {
        int nCoordNum = resMaterial.GetTextureCoordinatorsCount();
        if( nIndex < nCoordNum )
        {
            nw::gfx::res::ResTextureCoordinator texCoord = resMaterial.GetTextureCoordinators(nIndex);
            texCoord.SetReferenceCamera( referenceCamera );
            texCoord.SetDirty(true);
        }
    }

    void
    SetTexcoordReferenceCamera( unsigned int nIndex, int referenceCamera )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTexcoordReferenceCamera( nIndex, resMaterial, referenceCamera );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTexcoordReferenceCamera( nIndex, resMaterial, referenceCamera );
            }

            // ErrorCheck
            ErrorCheck::CheckModelCameraReferenceIndexBoundary( model );
            ErrorCheck::CheckModelCameraReferenceIndexReference( model );
        }
    }

    // Texture MatrixMode settings
    void
    SetTexcoordMatrixMode(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        Protocol4C::TextureMatrixModeCtr matrixMode
    )
    {
        const nw::gfx::res::ResTextureCoordinator::MappingMatrixMode matrixModeList[] = {
            nw::gfx::res::ResTextureCoordinator::MAPPINGMATRIXMODE_MAYA,         // TextureMatrixModeCtr_DccMaya = 0,           // Maya mode
            nw::gfx::res::ResTextureCoordinator::MAPPINGMATRIXMODE_SOFTIMAGE,    // TextureMatrixModeCtr_DccSoftimage = 1,           // SoftImage mode
            nw::gfx::res::ResTextureCoordinator::MAPPINGMATRIXMODE_3DSMAX,       // TextureMatrixModeCtr_Dcc3dsMax = 2,           // 3dsMax mode
        };

        int nCoordNum = resMaterial.GetTextureCoordinatorsCount();
        if( nIndex < nCoordNum )
        {
            nw::gfx::res::ResTextureCoordinator texCoord = resMaterial.GetTextureCoordinators(nIndex);
            texCoord.SetMatrixMode( matrixModeList[matrixMode] );
            texCoord.SetDirty(true);
        }
    }

    void
    SetTexcoordMatrixMode( unsigned int nIndex, Protocol4C::TextureMatrixModeCtr matrixMode )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTexcoordMatrixMode( nIndex, resMaterial, matrixMode );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTexcoordMatrixMode( nIndex, resMaterial, matrixMode );
            }
        }
    }

    // Texture Scale settings
    void
    SetTexcoordScale(
        unsigned int nFlagST,
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        float scaleS, float scaleT
    )
    {
        int nCoordNum = resMaterial.GetTextureCoordinatorsCount();
        if( nIndex < nCoordNum )
        {
            nw::gfx::res::ResTextureCoordinator texCoord = resMaterial.GetTextureCoordinators(nIndex);
            nw::math::VEC2 scaleST = texCoord.GetScale();
            if( nFlagST & TexS )
            {
                scaleST.x = scaleS;
            }
            if( nFlagST & TexT )
            {
                scaleST.y = scaleT;
            }
            texCoord.SetScale( scaleST );
        }
    }

    void
    SetTexcoordScale( unsigned int nFlagST, unsigned int nIndex, float scaleS, float scaleT )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTexcoordScale( nFlagST, nIndex, resMaterial, scaleS, scaleT );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTexcoordScale( nFlagST, nIndex, resMaterial, scaleS, scaleT );
            }
        }
    }

    // Texture Rotate settings
    void
    SetTexcoordRotate(
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        float rotate
    )
    {
        int nCoordNum = resMaterial.GetTextureCoordinatorsCount();
        if( nIndex < nCoordNum )
        {
            nw::gfx::res::ResTextureCoordinator texCoord = resMaterial.GetTextureCoordinators(nIndex);
            texCoord.SetRotate(rotate);
        }
    }

    void
    SetTexcoordRotate( unsigned int nIndex, float rotate )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTexcoordRotate( nIndex, resMaterial, rotate );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTexcoordRotate( nIndex, resMaterial, rotate );
            }
        }
    }

    // Texture Translate settings
    void
    SetTexcoordTranslate(
        unsigned int nFlagST,
        unsigned int nIndex,
        nw::gfx::res::ResMaterial resMaterial,
        float translate
    )
    {
        int nCoordNum = resMaterial.GetTextureCoordinatorsCount();
        if( nIndex < nCoordNum )
        {
            nw::gfx::res::ResTextureCoordinator texCoord = resMaterial.GetTextureCoordinators(nIndex);
            nw::math::VEC2 transST = texCoord.GetTranslate();
            if( nFlagST & TexS )
            {
                transST.x = translate;
            }
            if( nFlagST & TexT )
            {
                transST.y = translate;
            }
            texCoord.SetTranslate( transST );
        }
    }

    void
    SetTexcoordTranslate( unsigned int nFlagST, unsigned int nIndex, float translate )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                SetTexcoordTranslate( nFlagST, nIndex, resMaterial, translate );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                SetTexcoordTranslate( nFlagST, nIndex, resMaterial, translate );
            }
        }
    }

    // Sets the texture used for bump map.
    void
    SetBumpTexIndex( Protocol4C::BumpTextureIndex bumpIndex )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetBumpTextureIndex((nw::gfx::res::ResFragmentLighting::BumpTexture)bumpIndex);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetBumpTextureIndex((nw::gfx::res::ResFragmentLighting::BumpTexture)bumpIndex);
            }
        }
    }

    // Sets the bump map mode.
    void
    SetBumpMode( Protocol4C::BumpMode bumpMode )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetBumpMode((nw::gfx::res::ResFragmentLighting::BumpMode)bumpMode);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetBumpMode((nw::gfx::res::ResFragmentLighting::BumpMode)bumpMode);
            }
        }
    }

    // Sets recalculation of the z component of bump map.
    void
    SetRenormalize( bool renormalize )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetBumpRenormalize(renormalize);
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
                resFragmentLighting.SetBumpRenormalize(renormalize);
            }
        }
    }

    // Sets enable and disable of polygon offset.
    void
    SetPolygonOffset( bool offset )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResRasterization resRasterization = resMaterial.GetRasterization();
                SET_FLAGS( resRasterization, offset, nw::gfx::res::ResRasterizationData::FLAG_POLYGON_OFFSET_ENABLED )
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResRasterization resRasterization = resMaterial.GetRasterization();
                SET_FLAGS( resRasterization, offset, nw::gfx::res::ResRasterizationData::FLAG_POLYGON_OFFSET_ENABLED )
            }
        }
    }

    // Sets the unit of polygon offset.
    void
    SetPolygonOffsetUnit( float unit )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResRasterization resRasterization = resMaterial.GetRasterization();
                resRasterization.SetPolygonOffsetUnit( unit );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResRasterization resRasterization = resMaterial.GetRasterization();
                resRasterization.SetPolygonOffsetUnit( unit );
            }
        }
    }

    // Auxiliary function of the look-up table setting
    void
    SetFragmentLightingTableUtil(
        nw::gfx::res::ResLightingLookupTable lookupTable,
        bool absValue,
        int inputValue,
        int scaleValue,
        int hashID,
        const wchar_t* setName,
        const wchar_t* tableName,
        bool isSetAbs,
        bool isSetInput,
        bool isSetScale,
        bool isSetTable
    ) {
        if ( !lookupTable.IsValid() ) return;

        const nw::gfx::res::ResLightingLookupTable::Input inputList[] =
        {
            nw::gfx::res::ResLightingLookupTable::INPUT_NH,   // LightingLookupTableInput_CosNormalHalf = 0,           // Cosine value of the angle formed by the normal and half vector.
            nw::gfx::res::ResLightingLookupTable::INPUT_VH,   // LightingLookupTableInput_CosViewHalf = 1,           // Cosine value of the angle formed by the line-of-sight vector and half vector.
            nw::gfx::res::ResLightingLookupTable::INPUT_NV,   // LightingLookupTableInput_CosNormalView = 2,           // Cosine value of the angle formed by the normal and line-of-sight vector.
            nw::gfx::res::ResLightingLookupTable::INPUT_LN,   // LightingLookupTableInput_CosLightNormal = 3,           // Cosine value of the angle formed by the light vector and normal.
            nw::gfx::res::ResLightingLookupTable::INPUT_SP,   // LightingLookupTableInput_CosLightSpot = 4,           // Cosine value of the angle formed by the light vector and spot direction vector.
            nw::gfx::res::ResLightingLookupTable::INPUT_CP,   // LightingLookupTableInput_CosPhi = 5,           // Cosine value of the angle formed by the tangent plane projection vector and the tangent line vector.
        };

        const nw::gfx::res::ResLightingLookupTable::Scale scaleList[] =
        {
            nw::gfx::res::ResLightingLookupTable::SCALE_0_25,        // LightingLookupTableScale_Quarter = 0,           // 0.25 times.
            nw::gfx::res::ResLightingLookupTable::SCALE_0_5,         // LightingLookupTableScale_Half = 1,           // 0.5 times.
            nw::gfx::res::ResLightingLookupTable::SCALE_1,           // LightingLookupTableScale_One = 2,           // 1 time.
            nw::gfx::res::ResLightingLookupTable::SCALE_2,           // LightingLookupTableScale_Two = 3,           // 2 times.
            nw::gfx::res::ResLightingLookupTable::SCALE_4,           // LightingLookupTableScale_Four = 4,           // 4 times.
            nw::gfx::res::ResLightingLookupTable::SCALE_8,           // LightingLookupTableScale_Eight = 5,           // 8 times.
        };

        if( isSetAbs )
        {
            nw::gfx::res::ResReferenceLookupTable refLut =
                nw::ut::ResStaticCast<nw::gfx::res::ResReferenceLookupTable>(lookupTable.GetSampler());
            if ( refLut.IsValid() && refLut.GetTargetLut().IsValid() ) refLut.Dereference().SetAbs(absValue);
        }
        if( isSetInput )
        {
            lookupTable.SetInput(inputList[inputValue]);
        }
        if( isSetScale )
        {
            lookupTable.SetScale(scaleList[scaleValue]);
        }
        if( isSetTable )
        {
            if ( !lookupTable.GetSampler().IsValid() ) return;

            if ( setName[0] == 0 || tableName[0] == 0 ){
                lookupTable.SetSampler( m_ResMgr->GetDefaultLookupTable() );
            } else {
                char szSetName[260], szTableName[260];
                vw::system::ConvertStrWtoUTF8( setName, szSetName, 260 );
                vw::system::ConvertStrWtoUTF8( tableName, szTableName, 260 );

#if defined( NN_PCGX )
                nw::gfx::res::ResGraphicsFile resGraphicsFile = m_ScnMgr->FindModel(hashID)->GetResSource()->GetResource();
                nw::gfx::res::ResLookupTableSet lookupTableSet = resGraphicsFile.GetLutSets( szSetName );
                if ( lookupTableSet.IsValid() )
                {
                    nw::gfx::res::ResLookupTable resLookupTable = GetResLookupTable( hashID, szSetName, szTableName );
                    lookupTable.SetSampler( resLookupTable );
                }
#else
                nw::gfx::res::ResLookupTable resLookupTable = GetResLookupTable( hashID, szSetName, szTableName );
                lookupTable.SetSampler( resLookupTable );
#endif
            }
        }
    }

    // Fragment light table settings
    void
    SetFragmentLightingTable(
        int    hashID,
        nw::gfx::res::ResFragmentShader resFragmentShader,
        Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable *lightingTable
    )
    {
        nw::gfx::res::ResFragmentLighting resFragmentLighting = resFragmentShader.GetFragmentLighting();
        nw::gfx::res::ResFragmentLightingTable resFragmentLightingTable = resFragmentShader.GetFragmentLightingTable();

        if( lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_LookupTableSetName )
        {
            if( lightingTable->LookupTableSetName.ToString()[0] == 0 )
            {
                // Disable if not used.
                u32 nFlags = resFragmentLighting.GetFlags();
                nFlags &= ~nw::gfx::res::ResFragmentLightingData::FLAG_DISTRIBUTION0_ENABLED;
                nFlags &= ~nw::gfx::res::ResFragmentLightingData::FLAG_DISTRIBUTION1_ENABLED;
                nFlags &= ~nw::gfx::res::ResFragmentLightingData::FLAG_REFLECTION_ENABLED;
                resFragmentLighting.SetFlags( nFlags );
                resFragmentLighting.SetFresnelConfig( nw::gfx::res::ResFragmentLighting::CONFIG_NO_FRESNEL );
            }
        }

        // GetReflectanceRSampler
        SetFragmentLightingTableUtil(
            resFragmentLightingTable.GetReflectanceRSampler(),
            lightingTable->RefSamplerRIsAbs,
            lightingTable->RefSamplerRInput,
            lightingTable->RefSamplerRScale,
            hashID,
            lightingTable->LookupTableSetName.ToString(),
            lightingTable->RefSamplerRLutName.ToString(),
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerRIsAbs,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerRInput,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerRScale,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerRLutName
        );

        // GetReflectanceGSampler
        SetFragmentLightingTableUtil(
            resFragmentLightingTable.GetReflectanceGSampler(),
            lightingTable->RefSamplerGIsAbs,
            lightingTable->RefSamplerGInput,
            lightingTable->RefSamplerGScale,
            hashID,
            lightingTable->LookupTableSetName.ToString(),
            lightingTable->RefSamplerGLutName.ToString(),
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerGIsAbs,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerGInput,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerGScale,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerGLutName
        );

        // GetReflectanceBSampler
        SetFragmentLightingTableUtil(
            resFragmentLightingTable.GetReflectanceBSampler(),
            lightingTable->RefSamplerBIsAbs,
            lightingTable->RefSamplerBInput,
            lightingTable->RefSamplerBScale,
            hashID,
            lightingTable->LookupTableSetName.ToString(),
            lightingTable->RefSamplerBLutName.ToString(),
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerBIsAbs,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerBInput,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerBScale,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_RefSamplerBLutName
        );

        // GetDistribution0Sampler
        SetFragmentLightingTableUtil(
            resFragmentLightingTable.GetDistribution0Sampler(),
            lightingTable->Dis0SamplerIsAbs,
            lightingTable->Dis0SamplerInput,
            lightingTable->Dis0SamplerScale,
            hashID,
            lightingTable->LookupTableSetName.ToString(),
            lightingTable->Dis0SamplerLutName.ToString(),
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis0SamplerIsAbs,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis0SamplerInput,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis0SamplerScale,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis0SamplerLutName
        );

        // GetDistribution1Sampler
        SetFragmentLightingTableUtil(
            resFragmentLightingTable.GetDistribution1Sampler(),
            lightingTable->Dis1SamplerIsAbs,
            lightingTable->Dis1SamplerInput,
            lightingTable->Dis1SamplerScale,
            hashID,
            lightingTable->LookupTableSetName.ToString(),
            lightingTable->Dis1SamplerLutName.ToString(),
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis1SamplerIsAbs,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis1SamplerInput,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis1SamplerScale,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_Dis1SamplerLutName
        );

        // GetFresnelSampler
        SetFragmentLightingTableUtil(
            resFragmentLightingTable.GetFresnelSampler(),
            lightingTable->FresnelSamplerIsAbs,
            lightingTable->FresnelSamplerInput,
            lightingTable->FresnelSamplerScale,
            hashID,
            lightingTable->LookupTableSetName.ToString(),
            lightingTable->FresnelSamplerLutName.ToString(),
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_FresnelSamplerIsAbs,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_FresnelSamplerInput,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_FresnelSamplerScale,
            lightingTable->ModifiedProperties & Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable::PropertyFlag_FresnelSamplerLutName
        );
    }

    void
    SetFragmentLightingTable( Protocol4C::MESSAGEPACKET_Material_SetFragmentLightingTable *lightingTable )
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_MODEL_G3D( selected->Id, 0, model );

            // Settings to MaterialOriginal
            {
                GET_RESMATERIAL_ORIGINAL_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                SetFragmentLightingTable( selected->Id, resFragmentShader, lightingTable );
            }

            // Settings to MaterialBuffer
            {
                GET_RESMATERIAL_BUFFER_G3D( selected->Index, resMaterial );
                nw::gfx::res::ResFragmentShader resFragmentShader = resMaterial.GetFragmentShader();
                SetFragmentLightingTable( selected->Id, resFragmentShader, lightingTable );
            }

            // ErrorCheck
            ErrorCheck::CheckModelLookupTableReference( model );
        }
    }

    // Model display settings
    void
    SetModelVisibility( unsigned int nID, bool bVisible )
    {
        GET_MODEL_G3D( nID, 0, pModel );
        if( pModel ){
            pModel->SetVisible( bVisible );
        }
    }

    // Model scale settings
    void
    SetModelScale( AXIS_TYPE axis, unsigned int nID, float scale )
    {
        GET_MODEL_G3D( nID, 0, pModel );
        if( pModel ){
            nw::gfx::CalculatedTransform& transform = pModel->Transform();
            nw::math::VEC3 vtScale = transform.Scale();
            switch( axis ){
                case AXIS_X:
                    vtScale.x = scale;
                    break;
                case AXIS_Y:
                    vtScale.y = scale;
                    break;
                case AXIS_Z:
                    vtScale.z = scale;
                    break;
            }

            transform.SetScale( vtScale );
        }
    }

    // Model rotation settings
    void
    SetModelRotate( AXIS_TYPE axis, unsigned int nID, float rotate )
    {
        GET_MODEL_G3D( nID, 0, pModel );
        if( pModel ){
            nw::gfx::res::ResModel resModel = pModel->GetResModel();
            nw::gfx::CalculatedTransform& transform = pModel->Transform();
            nw::math::Transform3& trans = resModel.GetTransform();
            switch( axis ){
                case AXIS_X:
                    trans.rotate.x = rotate;
                    break;
                case AXIS_Y:
                    trans.rotate.y = rotate;
                    break;
                case AXIS_Z:
                    trans.rotate.z = rotate;
                    break;
            }
            transform.SetRotateXYZ(trans.rotate.x,trans.rotate.y,trans.rotate.z);
        }
    }

    // Model position settings
    void
    SetModelTranslate( AXIS_TYPE axis, unsigned int nID, float translate )
    {
        GET_MODEL_G3D( nID, 0, pModel );
        if( pModel ){
            nw::gfx::CalculatedTransform& transform = pModel->Transform();
            nw::math::VEC3 vtTranslate = transform.GetTranslate();
            switch( axis ){
                case AXIS_X:
                    vtTranslate.x = translate;
                    break;
                case AXIS_Y:
                    vtTranslate.y = translate;
                    break;
                case AXIS_Z:
                    vtTranslate.z = translate;
                    break;
            }

            transform.SetTranslate( vtTranslate );
        }
    }

    // Preview model display settings
    void
    SetModelPreviewVisibility( unsigned int nID, bool bVisible )
    {
        GET_MODEL_G3D( nID, 0, pModel );
        if( pModel ){
            u32 nUserParam = 0;
            if( !bVisible ){
                nUserParam = 1;
            }
            pModel->SetUserParameter( nUserParam );
        }
    }

    // Preview model scale settings
    void
    SetModelPreviewScale( AXIS_TYPE axis, unsigned int nID, float scale )
    {
        GET_MODEL_G3D( nID, 0, pModel );
        if( pModel ){
            nw::gfx::SceneNode *pParent = pModel->GetParent();
            if( pParent ){
                nw::gfx::TransformNode *pTransformNode = reinterpret_cast<nw::gfx::TransformNode*>(pParent);
                nw::gfx::CalculatedTransform& transform = pTransformNode->Transform();
                nw::math::VEC3 vtScale = transform.Scale();
                switch( axis ){
                    case AXIS_X:
                        vtScale.x = scale;
                        break;
                    case AXIS_Y:
                        vtScale.y = scale;
                        break;
                    case AXIS_Z:
                        vtScale.z = scale;
                        break;
                }

                transform.SetScale( vtScale );
            }
        }
    }

    // Preview model rotation settings
    void
    SetModelPreviewRotate( AXIS_TYPE axis, unsigned int nID, float rotate )
    {
        SceneManager::ModelList::iterator it = m_ScnMgr->FindModel( nID );
        if( it != m_ScnMgr->GetModelsEnd() ){
            nw::gfx::Model* pModel = it->GetSceneNode();
            nw::gfx::SceneNode *pParent = pModel->GetParent();
            if( pParent ){
                nw::gfx::TransformNode *pTransformNode = reinterpret_cast<nw::gfx::TransformNode*>(pParent);
                nw::gfx::CalculatedTransform& transform = pTransformNode->Transform();
                float *previewRot = it->GetResSource()->GetUserData();
                switch( axis ){
                    case AXIS_X:
                        previewRot[0] = rotate;
                        break;
                    case AXIS_Y:
                        previewRot[1] = rotate;
                        break;
                    case AXIS_Z:
                        previewRot[2] = rotate;
                        break;
                }
                transform.SetRotateXYZ( previewRot[0],previewRot[1],previewRot[2] );
            }
        }
    }

    // Preview model position settings
    void
    SetModelPreviewTranslate( AXIS_TYPE axis, unsigned int nID, float translate )
    {
        GET_MODEL_G3D( nID, 0, pModel );
        if( pModel ){
            nw::gfx::SceneNode *pParent = pModel->GetParent();
            if( pParent ){
                nw::gfx::TransformNode *pTransformNode = reinterpret_cast<nw::gfx::TransformNode*>(pParent);
                nw::gfx::CalculatedTransform& transform = pTransformNode->Transform();
                nw::math::VEC3 vtTranslate = transform.GetTranslate();
                switch( axis ){
                    case AXIS_X:
                        vtTranslate.x = translate;
                        break;
                    case AXIS_Y:
                        vtTranslate.y = translate;
                        break;
                    case AXIS_Z:
                        vtTranslate.z = translate;
                        break;
                }

                transform.SetTranslate( vtTranslate );
            }
        }
    }

    // Model transform settings
    void
    SetModelTransform( Protocol4C::MESSAGEPACKET_Model_SetModelTransformValue* transformValue )
    {
        GET_MODEL_G3D( transformValue->ModelId, 0, pModel );
        if( pModel ){
        }
    }

    // Model preview transform settings
    void
    SetModelPreviewTransform( Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue* transformValue )
    {
        SceneManager::ModelList::iterator it = m_ScnMgr->FindModel( transformValue->ModelId );
        if( it != m_ScnMgr->GetModelsEnd() ) {
            nw::gfx::Model* pModel = it->GetSceneNode();
            nw::gfx::SceneNode *pParent = pModel->GetParent();
            if( pParent ){
                nw::gfx::TransformNode *pTransformNode = reinterpret_cast<nw::gfx::TransformNode*>(pParent);
                nw::gfx::CalculatedTransform& transform = pTransformNode->Transform();
                nw::math::VEC3 vtScale = transform.Scale();
                nw::math::VEC3 vtTranslate = transform.GetTranslate();
                float *previewRot = it->GetResSource()->GetUserData();

                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_IsVisible )
                {
                    u32 nUserParam = 0;
                    if( !transformValue->IsVisible ){
                        nUserParam = 1;
                    }
                    pModel->SetUserParameter( nUserParam );
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformScaleX )
                {
                    vtScale.x = transformValue->TranceformScaleX;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformScaleY )
                {
                    vtScale.y = transformValue->TranceformScaleY;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformScaleZ )
                {
                    vtScale.z = transformValue->TranceformScaleZ;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformTranslateX )
                {
                    vtTranslate.x = transformValue->TranceformTranslateX;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformTranslateY )
                {
                    vtTranslate.y = transformValue->TranceformTranslateY;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformTranslateZ )
                {
                    vtTranslate.z = transformValue->TranceformTranslateZ;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformRotateX )
                {
                    previewRot[ 0 ] = transformValue->TranceformRotateX;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformRotateY )
                {
                    previewRot[ 1 ] = transformValue->TranceformRotateY;
                }
                if( transformValue->ModifiedProperties & Protocol4C::MESSAGEPACKET_Model_SetModelPreviewTransformValue::PropertyFlag_TranceformRotateZ )
                {
                    previewRot[ 2 ] = transformValue->TranceformRotateZ;
                }

                transform.SetScale( vtScale );
                transform.SetTranslate( vtTranslate );
                transform.SetRotateXYZ( previewRot[0],previewRot[1],previewRot[2] );
            }
        }
    }

    // Constraint settings
    void
    SetModelPreviewBoneLink( Protocol4C::MESSAGEPACKET_Model_SetModelPreviewBoneLink* link )
    {
        vw::g3d::SceneManager::ModelList::iterator node = m_ScnMgr->FindModel( link->ModelId );
        if ( node == m_ScnMgr->GetModelsEnd() ) return;

        // Gets the sub-root node.
        nw::gfx::TransformNode* subRoot =
            nw::ut::DynamicCast<nw::gfx::TransformNode*>( node->GetSceneNode()->GetParent()->GetParent() );
        if ( subRoot == NULL ) return;

        // Cancels the target object link.
        vw::g3d::SceneManager::ModelList::iterator it = m_ScnMgr->GetModelsBegin();
        vw::g3d::SceneManager::ModelList::iterator end = m_ScnMgr->GetModelsEnd();
        for ( ; it != end; ++it ) {
            it->GetNodeConstraint().DeleteConstraint(subRoot);
        }

        // Converts character string to char.
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(link->BoneName.ToString(), utf8Str, MAX_PATH_SIZE);

        if ( utf8Str[0] != 0 )
        {
            it = m_ScnMgr->FindModel( link->ModelNameHashID );
            if ( it != m_ScnMgr->GetModelsEnd() ) {
                if ( it->GetResObject().GetTypeInfo() == nw::gfx::res::ResSkeletalModel::TYPE_INFO )
                {
                    vw::g3d::Constraint constraint;
                    constraint.m_ConstrainedObject = subRoot;
                    constraint.m_UseOffset[0] = false;
                    constraint.m_UseOffset[1] = false;
                    constraint.m_UseOffset[2] = false;
                    constraint.m_InheritTransform[0] = true;
                    constraint.m_InheritTransform[1] = true;
                    constraint.m_InheritTransform[2] = true;
                    it->GetNodeConstraint().AddConstraint( constraint, utf8Str );
                }
            }
        }
    }

    // Constraint settings
    void
    SetModelPreviewBoneUnLink( Protocol4C::MESSAGEPACKET_Model_SetModelPreviewBoneUnLink* link )
    {
        vw::g3d::SceneManager::ModelList::iterator node = m_ScnMgr->FindModel( link->ModelId );
        if ( node == m_ScnMgr->GetModelsEnd() ) return;

        // Gets the sub-root node.
        nw::gfx::TransformNode* subRoot =
            nw::ut::DynamicCast<nw::gfx::TransformNode*>( node->GetSceneNode()->GetParent()->GetParent() );
        if ( subRoot == NULL ) return;

        vw::g3d::SceneManager::ModelList::iterator it = m_ScnMgr->GetModelsBegin();
        vw::g3d::SceneManager::ModelList::iterator end = m_ScnMgr->GetModelsEnd();
        for ( ; it != end; ++it ) {
            it->GetNodeConstraint().DeleteConstraint(subRoot);
        }
    }

    // Mesh display settings
    void
    SetMeshVisibility( unsigned int nModelID, int nMeshID, bool bVisible )
    {
        GET_MODEL_G3D( nModelID, 0, pModel );
        if( pModel ){
            nw::gfx::res::ResMeshArray resMeshArray = pModel->GetResMeshes();
            resMeshArray[nMeshID].SetVisible(bVisible);
        }
    }

    // Mesh node display settings
    void
    SetMeshNodeVisibility( unsigned int nModelID, int nMeshID, bool bVisible )
    {
        GET_MODEL_G3D( nModelID, 0, pModel );
        if( pModel ){
            nw::gfx::res::ResMeshNodeVisibility resMeshNodeVisibility = pModel->GetResMeshNodeVisibilities(nMeshID);
            resMeshNodeVisibility.SetVisible(bVisible);
        }
    }

    // Mesh render priority settings
    void
    SetMeshRenderPriority( unsigned int nModelID, unsigned int nMeshID, unsigned char nPriority )
    {
        GET_MODEL_G3D( nModelID, 0, pModel );
        if( pModel ){
            nw::gfx::res::ResMeshArray resMeshArray = pModel->GetResMeshes();
            resMeshArray[nMeshID].SetRenderPriority(nPriority);
        }
    }

    // Bone billboard settings.
    void
    SetBoneBillboard( unsigned int nModelID, Protocol4C::String &boneName, Protocol4C::BoneBillboardMode billboardMode )
    {
        GET_MODEL_G3D( nModelID, 0, pModel );
        if( pModel ){
            if( nw::gfx::res::ResSkeletalModel::TYPE_INFO == pModel->GetResSceneNode().GetTypeInfo() )
            {
                nw::gfx::SkeletalModel *pSkeletalModel = reinterpret_cast<nw::gfx::SkeletalModel*>(pModel);
                nw::gfx::res::ResSkeleton resSkeleton = pSkeletalModel->GetSkeleton()->GetResSkeleton();

                char szBoneName[260];
                vw::system::ConvertStrWtoUTF8( boneName.ToString(),  szBoneName, 260 );
                int nIndex = resSkeleton.GetBonesIndex( szBoneName );
                nw::gfx::res::ResBone resBone = resSkeleton.GetBones(nIndex);

                resBone.SetBillboardMode((nw::gfx::res::ResBone::BillboardMode)billboardMode);
            }
        }
    }

    // Updates lookup table.
    void
    SetLUT(Protocol4C::MESSAGEPACKET_LUT_ModifyLookUpTable* lut)
    {
        s32 id=0, index=0;
        nw::gfx::res::ResGraphicsFile resource;
        if (!resource.IsValid()) {
            SceneManager::ModelList::iterator it = m_ScnMgr->FindModel(lut->ModelId);
            if ( it != m_ScnMgr->GetModelsEnd() ) {
                resource = it->GetResSource()->GetResource();
            }
        }
        if (!resource.IsValid()) {
            SceneManager::LightList::iterator it = m_ScnMgr->FindLight(lut->ModelId);
            if ( it != m_ScnMgr->GetLightsEnd() ) {
                resource = it->GetResSource()->GetResource();
            }
        }
        if (!resource.IsValid()) return;

        char szName[260];
        vw::system::ConvertStrWtoUTF8( lut->LutSetName.ToString(), szName, 260 );
        nw::gfx::res::ResLookupTableSet lutSet = resource.GetLutSets( szName );
        if (!lutSet.IsValid()) return;
        vw::system::ConvertStrWtoUTF8( lut->LutName.ToString(), szName, 260 );
        nw::gfx::res::ResImageLookupTable imgLUTSet = lutSet.GetSamplers( szName );
        if (!imgLUTSet.IsValid()) return;
        u32* commandCache = imgLUTSet.GetCommandCache();

        nw::os::IAllocator* pAllocator = Memory_GetAllocator();
        nw::os::IAllocator* pDeviceAllocator = Memory_GetAllocatorDevice();

        // Loads resource.
        nw::ut::MoveArray<u8> file(
            vw::system::Utility::LoadFileFromHost(pDeviceAllocator, lut->FilePath.ToString())
        );
        nw::gfx::res::ResGraphicsFile lut_resource(&(file.front()));
        lut_resource.Setup(pAllocator);
        u32* newCommandCache = lut_resource.GetLutSets(0).GetSamplers(0).GetCommandCache();

        memcpy( commandCache, newCommandCache, sizeof(u32) * 259 );
    }

    // Fog settings
    void
    SetFog(Protocol4C::MESSAGEPACKET_Fog_SetFog* fog)
    {
        nw::gfx::res::ResFogUpdater::FogUpdaterType fogTypeList[] = {
            nw::gfx::res::ResFogUpdater::FOG_UPDATER_TYPE_LINEAR,            // FogKindCtr_Linear = 0,           // Linear
            nw::gfx::res::ResFogUpdater::FOG_UPDATER_TYPE_EXPONENT,          // FogKindCtr_Exponent = 1,         // Exponent
            nw::gfx::res::ResFogUpdater::FOG_UPDATER_TYPE_EXPONENT_SQUARE,   // FogKindCtr_ExponentSquare = 2,   // ExponentSquare
            nw::gfx::res::ResFogUpdater::FOG_UPDATER_TYPE_NONE               // FogKindCtr_None = 3,             // None
        };

        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESFOG_G3D( selected->Id, selected->Index, resFog );
            nw::gfx::res::ResFogUpdater updater = resFog.GetFogUpdater();

            if( fog->ModifiedProperties & Protocol4C::MESSAGEPACKET_Fog_SetFog::PropertyFlag_FogKind )
            {
                updater.SetFogUpdaterType( fogTypeList[fog->FogKind] );
            }
            if( fog->ModifiedProperties & Protocol4C::MESSAGEPACKET_Fog_SetFog::PropertyFlag_MinFogDepth )
            {
                updater.SetMinFogDepth( fog->MinFogDepth );
            }
            if( fog->ModifiedProperties & Protocol4C::MESSAGEPACKET_Fog_SetFog::PropertyFlag_MaxFogDepth )
            {
                updater.SetMaxFogDepth( fog->MaxFogDepth );
            }
            if( fog->ModifiedProperties & Protocol4C::MESSAGEPACKET_Fog_SetFog::PropertyFlag_Density )
            {
                updater.SetDensity( fog->Density );
            }
        }
    }

    // Sets the enabling flag for fog z flip.
    void
    SetFogIsZFlip(Protocol4C::MESSAGEPACKET_Fog_SetFogIsZFlip* fog)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESFOG_G3D( selected->Id, selected->Index, resFog );
            SET_FLAGS( resFog, fog->IsZFlip, nw::gfx::res::ResFogData::FLAG_ZFLIP_ENABLED )
        }
    }

    // Fog color settings
    void
    SetFogColor(Protocol4C::MESSAGEPACKET_Fog_SetFogColor* fog)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESFOG_G3D( selected->Id, selected->Index, resFog );
            resFog.SetColor(fog->Color.R, fog->Color.G, fog->Color.B, fog->Color.A);

            // Edits OriginalValue with the transferred value.
#if 1 // Uses macros.
            GET_RESFOG_ORIG_G3D( selected->Id, selected->Index, resOrigFog );
            if ( resOrigFog.IsValid() )
                resOrigFog.SetColor(fog->Color.R, fog->Color.G, fog->Color.B, fog->Color.A);
#else // Method without using macros:
            if( nw::gfx::Fog* obj_Fog = m_ScnMgr->FindFogNode(selected->Id, selected->Index) )
            {
                if(u8* ptr = GetOriginalValue( obj_Fog->GetAnimGroup()))
                {
                    nw::gfx::res::ResFog resFogOriginal = nw::gfx::res::ResFog( ptr );
                    if( resFogOriginal.IsValid() )
                        resFogOriginal.SetColor(fog->Color.R, fog->Color.G, fog->Color.B, fog->Color.A);
                }
            }
#endif
        }
    }

    // Sets the depth value in a viewing coordinate system in which the fog is beginning to take effect.
    void
    SetMinFogDepth(Protocol4C::MESSAGEPACKET_Fog_SetMinFogDepth* fog)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESFOG_G3D( selected->Id, selected->Index, resFog );
            nw::gfx::res::ResFogUpdater updater = resFog.GetFogUpdater();
            updater.SetMinFogDepth(fog->MinFogDepth);
        }
    }

    // Sets the depth value in a viewing coordinate system in which the fog is densest.
    void
    SetMaxFogDepth(Protocol4C::MESSAGEPACKET_Fog_SetMaxFogDepth* fog)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESFOG_G3D( selected->Id, selected->Index, resFog );
            nw::gfx::res::ResFogUpdater updater = resFog.GetFogUpdater();
            updater.SetMaxFogDepth(fog->MaxFogDepth);
        }
    }

    // Fog density setting
    void
    SetFogDensity(Protocol4C::MESSAGEPACKET_Fog_SetFogDensity* fog)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESFOG_G3D( selected->Id, selected->Index, resFog );
            nw::gfx::res::ResFogUpdater updater = resFog.GetFogUpdater();
            updater.SetDensity(fog->Density);
        }
    }

    // Light enabled flag setting
    void
    SetLightEnable(Protocol4C::MESSAGEPACKET_Light_SetLightIsLightEnabled* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            resLight.SetLightEnabled( light->IsLightEnabled );

            // Edits OriginalValue with the transferred value.
            GET_RESLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigLight );
            if ( resOrigLight.IsValid() )
            { resOrigLight.SetLightEnabled( light->IsLightEnabled ); }
        }
    }

    // Light position (Transform(Translate)) settings
    void
    SetLightTranslate(Protocol4C::MESSAGEPACKET_Light_SetLightTranslate* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_LIGHT_G3D( selected->Id, selected->Index, lightObj );
            nw::gfx::CalculatedTransform& transform = lightObj->Transform();
            transform.SetTranslate( light->Translate.X, light->Translate.Y, light->Translate.Z );
        }
    }

    // Light ambient color settings
    void
    SetAmbientLightColor(Protocol4C::MESSAGEPACKET_Light_SetAmbientLightColor* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResAmbientLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResAmbientLight resAmbientLight = nw::ut::ResDynamicCast<nw::gfx::res::ResAmbientLight>(resLight);
                resAmbientLight.SetAmbient( light->Color.R, light->Color.G, light->Color.B );
            }
        }
    }

    // Hemisphere light ground color settings
    void
    SetHemiSphereGroundColor(Protocol4C::MESSAGEPACKET_Light_SetHemiSphereGroundColor* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResHemiSphereLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResHemiSphereLight resHemiSphereLight = nw::ut::ResDynamicCast<nw::gfx::res::ResHemiSphereLight>(resLight);
                resHemiSphereLight.SetGroundColor( light->GroundColor.R, light->GroundColor.G, light->GroundColor.B );
            }
        }
    }

    // Hemisphere light sky color settings
    void
    SetHemiSphereSkyColor(Protocol4C::MESSAGEPACKET_Light_SetHemiSphereSkyColor* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResHemiSphereLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResHemiSphereLight resHemiSphereLight = nw::ut::ResDynamicCast<nw::gfx::res::ResHemiSphereLight>(resLight);
                resHemiSphereLight.SetSkyColor( light->SkyColor.R, light->SkyColor.G, light->SkyColor.B );
            }
        }
    }

    // Hemisphere light direction settings
    void
    SetHemiSphereDirection(Protocol4C::MESSAGEPACKET_Light_SetHemiSphereDirection* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResHemiSphereLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResHemiSphereLight resHemiSphereLight = nw::ut::ResDynamicCast<nw::gfx::res::ResHemiSphereLight>(resLight);
                resHemiSphereLight.SetDirection( light->Direction.X, light->Direction.Y, light->Direction.Z );
            }
        }
    }

    // Hemisphere light linear interpolation system settings
    void
    SetHemiSphereLerpFactor(Protocol4C::MESSAGEPACKET_Light_SetHemiSphereLerpFactor* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResHemiSphereLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResHemiSphereLight resHemiSphereLight = nw::ut::ResDynamicCast<nw::gfx::res::ResHemiSphereLight>(resLight);
                resHemiSphereLight.SetLerpFactor( light->LerpFactor );
            }
        }
    }

    // Setting the rotation inheritance enabling flag of the hemisphere light parent node
    void
    SetHemiSphereIsInheritingDirectionRorate(Protocol4C::MESSAGEPACKET_Light_SetHemiSphereIsInheritingDirectionRotate* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResHemiSphereLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResHemiSphereLight resHemiSphereLight = nw::ut::ResDynamicCast<nw::gfx::res::ResHemiSphereLight>(resLight);
                SET_FLAGS(
                    resHemiSphereLight,
                    light->IsInheritingDirectionRotate,
                    nw::gfx::ResHemiSphereLightData::FLAG_IS_INHERITING_DIRECTION_ROTATE
                )
            }
        }
    }

    // Vertex light light source settings
    void
    SetVLLightKind(Protocol4C::MESSAGEPACKET_Light_SetVLLightKind* light)
    {
        nw::gfx::res::ResLight::Kind lightTypeList[] = {
            nw::gfx::res::ResLight::KIND_DIRECTIONAL,    // Parallel light sources
            nw::gfx::res::ResLight::KIND_POINT,          // Point lights
            nw::gfx::res::ResLight::KIND_SPOT,           // Spot light source
            nw::gfx::res::ResLight::KIND_UNUSED          // Does not use the light source type of Ambient and HemiSphere.
        };

        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                resVertexLight.SetLightKind( lightTypeList[light->LightKind] );

                // Resets light data with a type change.
                nw::ut::MoveArray<g3d::Light>* lights = LightMgr::GetInstance()->GetLightArray();
                for(nw::ut::MoveArray<g3d::Light>::iterator i=lights->begin(); i!=lights->end(); ++i)
                {
                    if (i->GetLight()->GetResSceneObject().ptr() == resLight.ptr())
                    {
                        i->SetupLight();
                        break;
                    }
                }
            }
        }
    }

    // Vertex light ambient color settings
    void
    SetVLAmbient(Protocol4C::MESSAGEPACKET_Light_SetVLAmbient* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                resVertexLight.SetAmbient( light->Ambient.R, light->Ambient.G, light->Ambient.B );
            }
        }
    }

    // Vertex light diffused color settings
    void
    SetVLDiffuse(Protocol4C::MESSAGEPACKET_Light_SetVLDiffuse* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                resVertexLight.SetDiffuse( light->Diffuse.R, light->Diffuse.G, light->Diffuse.B, light->Diffuse.A );
            }
        }
    }

    // Vertex light direction settings
    void
    SetVLDirection(Protocol4C::MESSAGEPACKET_Light_SetVLDirection* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                resVertexLight.SetDirection( light->Direction.X, light->Direction.Y, light->Direction.Z );
            }
        }
    }

    // Setting the rotation inheritance enabling flag of the vertex light parent node
    void
    SetVLInheritingDirectionRotate(Protocol4C::MESSAGEPACKET_Light_SetVLIsInheritingDirectionRotate* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                SET_FLAGS(
                    resVertexLight,
                    light->IsInheritingDirectionRotate,
                    nw::gfx::ResVertexLightData::FLAG_IS_INHERITING_DIRECTION_ROTATE
                )
            }
        }
    }

    // Setting the constant attenuation system of vertex light distance attenuation.
    void
    SetVLDistanceAttenuationConstant(Protocol4C::MESSAGEPACKET_Light_SetVLDistanceAttenuationConstant* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                const nw::math::VEC3 param = resVertexLight.GetDistanceAttenuation();

                resVertexLight.SetDistanceAttenuation( light->DistanceAttenuationConstant, param.y, param.z );
            }
        }
    }

    // Setting the linear attenuation system of vertex light distance attenuation.
    void
    SetVLDistanceAttenuationLinear(Protocol4C::MESSAGEPACKET_Light_SetVLDistanceAttenuationLinear* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                const nw::math::VEC3 param = resVertexLight.GetDistanceAttenuation();

                resVertexLight.SetDistanceAttenuation( param.x, light->DistanceAttenuationLinear, param.z );
            }
        }
    }

    // Setting the secondary attenuation system of vertex light distance attenuation.
    void
    SetVLDistanceAttenuationQuadratic(Protocol4C::MESSAGEPACKET_Light_SetVLDistanceAttenuationQuadratic* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                const nw::math::VEC3 param = resVertexLight.GetDistanceAttenuation();

                resVertexLight.SetDistanceAttenuation( param.x, param.y, light->DistanceAttenuationQuadratic );
            }
        }
    }

    // Setting the enabling flag of vertex light distance attenuation.
    void
    SetVLDistanceAttenuationEnable(Protocol4C::MESSAGEPACKET_Light_SetVLIsDistanceAttenuationEnabled* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                resVertexLight.SetDistanceAttenuationEnabled( light->IsDistanceAttenuationEnabled );
            }
        }
    }

    // Setting the angular attenuation system of vertex light spot light.
    void
    SetVLSpotExponet(Protocol4C::MESSAGEPACKET_Light_SetVLSpotExponent* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                const nw::math::VEC2 param = resVertexLight.GetSpotFactor();

                resVertexLight.SetSpotFactor( light->SpotExponent, param.y );
            }
        }
    }

    // Setting the cut-off angle of the vertex light spot light.
    void
    SetVLSpotCutoffAngle(Protocol4C::MESSAGEPACKET_Light_SetVLSpotCutoffAngle* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResVertexLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResVertexLight resVertexLight = nw::ut::ResDynamicCast<nw::gfx::res::ResVertexLight>(resLight);
                const nw::math::VEC2 param = resVertexLight.GetSpotFactor();

                resVertexLight.SetSpotFactor( param.x, light->SpotCutoffAngle );
            }
        }
    }

    // Fragment light type settings
    void
    SetFLKind(Protocol4C::MESSAGEPACKET_Light_SetFLKind* light)
    {
        nw::gfx::res::ResLight::Kind lightTypeList[] = {
            nw::gfx::res::ResLight::KIND_DIRECTIONAL,    // Parallel light sources
            nw::gfx::res::ResLight::KIND_POINT,          // Point lights
            nw::gfx::res::ResLight::KIND_SPOT,           // Spot light source
            nw::gfx::res::ResLight::KIND_UNUSED          // Does not use the light source type of Ambient and HemiSphere.
        };

        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);

                resFragmentLight.SetLightKind( lightTypeList[light->LightKind] );

                // Resets light data with a type change.
                nw::ut::MoveArray<g3d::Light>* lights = LightMgr::GetInstance()->GetLightArray();
                for(nw::ut::MoveArray<g3d::Light>::iterator i=lights->begin(); i!=lights->end(); ++i)
                {
                    if (i->GetLight()->GetResSceneObject().ptr() == resLight.ptr())
                    {
                        i->SetupLight();
                        break;
                    }
                }
            }
        }
    }

    // Fragment light ambient color settings
    void
    SetFLAmbient(Protocol4C::MESSAGEPACKET_Light_SetFLAmbient* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);

                resFragmentLight.SetAmbient( light->Ambient.R, light->Ambient.G, light->Ambient.B );

                // Edits OriginalValue with the transferred value.
                GET_RESFRAGMENTLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigFragmentLight );
                if ( resOrigFragmentLight.IsValid() )
                { resOrigFragmentLight.SetAmbient( light->Ambient.R, light->Ambient.G, light->Ambient.B ); }
            }
        }
    }

    // Fragment light diffused color settings
    void
    SetFLDiffuse(Protocol4C::MESSAGEPACKET_Light_SetFLDiffuse* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);

                resFragmentLight.SetDiffuse( light->Diffuse.R, light->Diffuse.G, light->Diffuse.B );

                // Edits OriginalValue with the transferred value.
                GET_RESFRAGMENTLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigFragmentLight );
                if ( resOrigFragmentLight.IsValid() )
                { resOrigFragmentLight.SetDiffuse( light->Diffuse.R, light->Diffuse.G, light->Diffuse.B, light->Diffuse.A ); }
            }
        }
    }

    // Sets the enabling flag of dual surface lighting for fragment light diffused colors.
    void
    SetFLIsTowSideDiffuse(Protocol4C::MESSAGEPACKET_Light_SetFLIsTwoSideDiffuse* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                SET_FLAGS(
                    resFragmentLight,
                    light->IsTwoSideDiffuse,
                    nw::gfx::ResFragmentLightData::FLAG_TWO_SIDE_DIFFUSE_ENABLED
                )
            }
        }
    }

    // Fragment light specular color settings
    void
    SetFLSpecular(int nIndex, Protocol4C::Rgba rgba)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                GET_RESFRAGMENTLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigFragmentLight );
                if( nIndex == 0 ){
                    resFragmentLight.SetSpecular0( rgba.R, rgba.G, rgba.B );

                    // Edits OriginalValue with the transferred value.
                    if ( resOrigFragmentLight.IsValid() )
                    { resOrigFragmentLight.SetSpecular0( rgba.R, rgba.G, rgba.B ); }
                }else{
                    resFragmentLight.SetSpecular1( rgba.R, rgba.G, rgba.B );

                    // Edits OriginalValue with the transferred value.
                    if ( resOrigFragmentLight.IsValid() )
                    { resOrigFragmentLight.SetSpecular1( rgba.R, rgba.G, rgba.B ); }
                }
            }
        }
    }

    // Setting the rotation inheritance enabling flag of the fragment light parent node
    void
    SetFLIsInheritingDirectionRotate(Protocol4C::MESSAGEPACKET_Light_SetFLIsInheritingDirectionRotate* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                SET_FLAGS(
                    resFragmentLight,
                    light->IsInheritingDirectionRotate,
                    nw::gfx::ResFragmentLightData::FLAG_IS_INHERITING_DIRECTION_ROTATE
                )
            }
        }
    }

    // Fragment light direction settings
    void
    SetFLDirection(Protocol4C::MESSAGEPACKET_Light_SetFLDirection* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);

                resFragmentLight.SetDirection( light->Direction.X, light->Direction.Y, light->Direction.Z );

                // Edits OriginalValue with the transferred value.
                GET_RESFRAGMENTLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigFragmentLight );
                if ( resOrigFragmentLight.IsValid() )
                { resOrigFragmentLight.SetDirection( light->Direction.X, light->Direction.Y, light->Direction.Z ); }
            }
        }
    }

    // Setting the look-up table for AngleSampler and DistanceSampler of fragment light.
    void
    SetFLSamplerLUTS(Protocol4C::MESSAGEPACKET_Light_SetFLSamplerLUTS* light)
    {
    }

    // Setting the enabling flag of fragment light distance attenuation.
    void
    SetFLDistanceAttenuationEnable(Protocol4C::MESSAGEPACKET_Light_SetFLIsDistanceAttenuationEnabled* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_LIGHT_G3D( selected->Id, selected->Index, objLight );
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                SET_FLAGS(
                    resFragmentLight,
                    light->IsDistanceAttenuationEnabled,
                    nw::gfx::ResFragmentLightData::FLAG_DISTANCE_ATTENUATION_ENABLED
                )

                // Edits OriginalValue with the transferred value.
                GET_RESFRAGMENTLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigFragmentLight );
                if ( resOrigFragmentLight.IsValid() )
                {
                    SET_FLAGS(
                        resOrigFragmentLight,
                        light->IsDistanceAttenuationEnabled,
                        nw::gfx::ResFragmentLightData::FLAG_DISTANCE_ATTENUATION_ENABLED
                    )
                }

                // ErrorCheck
                ErrorCheck::CheckLightLookupTableReference( objLight );
            }
        }
    }

    // Setting the distance that starts the brightness attenuation of fragment light.
    void
    SetFLDistanceAttenuationStart(Protocol4C::MESSAGEPACKET_Light_SetFLIsDistanceAttenuationStart* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                resFragmentLight.SetDistanceAttenuationStart( light->DistanceAttenuationStart );

                // Edits OriginalValue with the transferred value.
                GET_RESFRAGMENTLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigFragmentLight );
                if ( resOrigFragmentLight.IsValid() )
                { resOrigFragmentLight.SetDistanceAttenuationStart( light->DistanceAttenuationStart ); }
            }
        }
    }

    // Setting the distance that stops the brightness attenuation of fragment light.
    void
    SetFLDistanceAttenuationEnd(Protocol4C::MESSAGEPACKET_Light_SetFLIsDistanceAttenuationEnd* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                resFragmentLight.SetDistanceAttenuationEnd( light->DistanceAttenuationEnd );

                // Edits OriginalValue with the transferred value.
                GET_RESFRAGMENTLIGHT_ORIG_G3D( selected->Id, selected->Index, resOrigFragmentLight );
                if ( resOrigFragmentLight.IsValid() )
                { resOrigFragmentLight.SetDistanceAttenuationEnd( light->DistanceAttenuationEnd ); }
            }
        }
    }

    // Setting the sampler look-up table for fragment light distance.
    void
    SetFLDistanceSamplerLUT(Protocol4C::MESSAGEPACKET_Light_SetFLDistanceSamplerLUT* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_LIGHT_G3D( selected->Id, selected->Index, objLight );
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                const wchar_t *szSetString = light->LookupTableSetName.ToString();
                const wchar_t *szTableString = light->LookupTableName.ToString();
                if( !resFragmentLight.GetDistanceSampler().IsValid() ) continue;

                if( szSetString[0] == 0 || szTableString[0] == 0 ){     // Disabled
                    resFragmentLight.SetDistanceSampler( m_ResMgr->GetDefaultLookupTable() );
                }else{
                    char szSetName[260], szTableName[260];
                    vw::system::ConvertStrWtoUTF8( szSetString, szSetName, 260 );
                    vw::system::ConvertStrWtoUTF8( szTableString, szTableName, 260 );

                    nw::gfx::res::ResGraphicsFile resGraphicsFile = m_ScnMgr->FindLight( selected->Id )->GetResSource()->GetResource();
                    nw::gfx::res::ResLookupTableSet lookupTableSet = resGraphicsFile.GetLutSets( szSetName );
                    int nIndex = lookupTableSet.GetSamplersIndex( szTableName );
                    nw::gfx::res::ResLookupTable resLookupTable = lookupTableSet.GetSamplers(nIndex);

                    nw::gfx::res::ResImageLookupTable resImagelookupTable = resLookupTable.Dereference();
                    resFragmentLight.SetDistanceSampler( resImagelookupTable );
                }

                // ErrorCheck
                ErrorCheck::CheckLightLookupTableReference( objLight );
            }
        }
    }

    // Setting the sampler look-up table for the fragment light angle.
    void
    SetFLAngleSamplerLUT(Protocol4C::MESSAGEPACKET_Light_SetFLAngleSamplerLUT* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_LIGHT_G3D( selected->Id, selected->Index, objLight );
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                const wchar_t *szSetString = light->LookupTableSetName.ToString();
                const wchar_t *szTableString = light->LookupTableName.ToString();

                nw::gfx::res::ResLightingLookupTable lookupTable = resFragmentLight.GetAngleSampler();
                if( !lookupTable.IsValid() || !lookupTable.GetSampler().IsValid() ) continue;

                if( szSetString[0] == 0 || szTableString[0] == 0 ){
                    lookupTable.SetSampler( m_ResMgr->GetDefaultLookupTable() );
                }else{
                    char szSetName[260], szTableName[260];
                    vw::system::ConvertStrWtoUTF8( szSetString, szSetName, 260 );
                    vw::system::ConvertStrWtoUTF8( szTableString, szTableName, 260 );

                    nw::gfx::res::ResGraphicsFile resGraphicsFile = m_ScnMgr->FindLight( selected->Id )->GetResSource()->GetResource();
                    nw::gfx::res::ResLookupTableSet lookupTableSet = resGraphicsFile.GetLutSets( szSetName );
                    int nIndex = lookupTableSet.GetSamplersIndex( szTableName );
                    nw::gfx::res::ResLookupTable resLookupTable = lookupTableSet.GetSamplers(nIndex);

                    lookupTable.SetSampler( resLookupTable );
                }

                // ErrorCheck
                ErrorCheck::CheckLightLookupTableReference( objLight );
            }
        }
    }

    // Setting the input values for accessing the sampler look-up table for the fragment light angle.
    void
    SetFLAngleSamplerInput(Protocol4C::MESSAGEPACKET_Light_SetFLAngleSamplerInput* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                nw::gfx::res::ResLightingLookupTable resLightingLookupTable = resFragmentLight.GetAngleSampler();

                resLightingLookupTable.SetInput( (nw::gfx::res::ResLightingLookupTable::Input)light->Input );
            }
        }
    }

    // Setting the scalar value applied to output value drawn from the sampler look-up table for the fragment light angle.
    void
    SetFLAngleSamplerScale(Protocol4C::MESSAGEPACKET_Light_SetFLAngleSamplerScale* light)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESLIGHT_G3D( selected->Id, selected->Index, resLight );
            if( nw::gfx::res::ResFragmentLight::TYPE_INFO == resLight.GetTypeInfo() )
            {
                nw::gfx::res::ResFragmentLight resFragmentLight = nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);
                nw::gfx::res::ResLightingLookupTable resLightingLookupTable = resFragmentLight.GetAngleSampler();

                nw::gfx::res::ResLightingLookupTable::Scale scaleList[] =
                {
                    nw::gfx::res::ResLightingLookupTable::SCALE_0_25,        // LightingLookupTableScale_Quarter = 0,           // 0.25 times.
                    nw::gfx::res::ResLightingLookupTable::SCALE_0_5,         // LightingLookupTableScale_Half = 1,           // 0.5 times.
                    nw::gfx::res::ResLightingLookupTable::SCALE_1,           // LightingLookupTableScale_One = 2,           // 1 time.
                    nw::gfx::res::ResLightingLookupTable::SCALE_2,           // LightingLookupTableScale_Two = 3,           // 2 times.
                    nw::gfx::res::ResLightingLookupTable::SCALE_4,           // LightingLookupTableScale_Four = 4,           // 4 times.
                    nw::gfx::res::ResLightingLookupTable::SCALE_8,           // LightingLookupTableScale_Eight = 5,           // 8 times.
                };
                resLightingLookupTable.SetScale( scaleList[light->Scale] );
            }
        }
    }

    // Gets the fragment light information.
    void
    GetFragmentLightInfo(Protocol4C::MESSAGEPACKET_Light_GetFragmentLightInfo* light)
    {
        Protocol4C::LightKindCtr lightTypeList[] = {
            Protocol4C::LightKindCtr_DirectionalLight,    // Parallel light sources
            Protocol4C::LightKindCtr_PointLight,          // Point lights
            Protocol4C::LightKindCtr_SpotLight,           // Spot light source
            Protocol4C::LightKindCtr_Unused               // Does not use the light source type of Ambient and HemiSphere.
        };

        GET_RESLIGHT_G3D( light->LightID, light->LightIndex, resLight );
        GET_LIGHT_G3D( light->LightID, light->LightIndex, objLight );

        // Copying the location.
        nw::gfx::CalculatedTransform& transform = objLight->Transform();
        transform.GetTranslate( ( nw::math::VEC3*)( &light->Position ) );

        switch ( resLight.GetTypeInfo() )
        {
        case nw::gfx::res::ResFragmentLight::TYPE_INFO:
            {
                nw::gfx::res::ResFragmentLight resFragmentLight =
                    nw::ut::ResDynamicCast<nw::gfx::res::ResFragmentLight>(resLight);

                // Copying the light source type.
                light->LightKind = lightTypeList[resFragmentLight.GetLightKind()];

                // Copying the direction vectors.
                memcpy(
                    &( light->Direction ),
                    &( resFragmentLight.GetDirection() ),
                    sizeof( nw::math::VEC3 )
                );
            }
            break;
        }
    }

    // Camera view (aim) settings.
    void
    SetTargetViewAim(Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim* camera)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESCAMERA_G3D( selected->Id, selected->Index, resCamera );
            if( resCamera.GetViewType() == nw::gfx::res::ResCamera::VIEWTYPE_AIM )
            {
                GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );
                nw::gfx::res::ResAimTargetViewUpdater resAimTargetView = nw::ut::ResDynamicCast<nw::gfx::res::ResAimTargetViewUpdater>(resCamera.GetViewUpdater());

                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_IsInheritingTargetRotate )
                {
                    SET_FLAGS(
                        resAimTargetView,
                        camera->IsInheritingTargetRotate,
                        nw::gfx::ResAimTargetViewUpdaterData::FLAG_INHERITING_TARGET_ROTATE
                    )
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_IsInheritingTargetTranslate )
                {
                    u32 nFlags = resAimTargetView.GetFlags();
                    if( camera->IsInheritingTargetTranslate ){
                        nFlags |= nw::gfx::res::ResAimTargetViewUpdaterData::FLAG_INHERITING_TARGET_TRANSLATE;
                        camera->ModifiedProperties |= Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_TargetPosition;
                    }else{
                        nFlags &= ~nw::gfx::res::ResAimTargetViewUpdaterData::FLAG_INHERITING_TARGET_TRANSLATE;
                    }
                    resAimTargetView.SetFlags( nFlags );
                }

                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_TargetPosition )
                {
                    resAimTargetView.SetTargetPosition( camera->TargetPosition.X, camera->TargetPosition.Y, camera->TargetPosition.Z );
                    // Determines if camera position is the same.
                    ErrorCheck::CheckCameraPosition( objCamera );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_Twist )
                {
                    resAimTargetView.SetTwist( camera->Twist );
                }


                // Edits OriginalValue with the transferred value.
                u8* ptr = GetOriginalValue( objCamera->GetAnimGroup(), nw::anim::res::ResAnimGroupMember::OBJECT_TYPE_VIEW_UPDATER);
                if( !ptr ) continue;
                nw::gfx::res::ResAimTargetViewUpdater resOrigAimTargetView = nw::gfx::res::ResAimTargetViewUpdater( ptr );
                if( !resOrigAimTargetView.IsValid() ) continue;
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_IsInheritingTargetRotate )
                {
                    SET_FLAGS(resOrigAimTargetView, camera->IsInheritingTargetRotate, nw::gfx::ResAimTargetViewUpdaterData::FLAG_INHERITING_TARGET_ROTATE);
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_IsInheritingTargetTranslate )
                {
                    SET_FLAGS(resOrigAimTargetView, camera->IsInheritingTargetTranslate, nw::gfx::ResAimTargetViewUpdaterData::FLAG_INHERITING_TARGET_TRANSLATE);
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_TargetPosition )
                {
                    resOrigAimTargetView.SetTargetPosition( objCamera->GetTargetPosition().x, objCamera->GetTargetPosition().y, objCamera->GetTargetPosition().z );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewAim::PropertyFlag_Twist )
                {
                    resOrigAimTargetView.SetTwist( camera->Twist );
                }
            }
        }
    }

    // Camera view (LookAt) settings.
    void
    SetTargetViewLookAt(Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt* camera)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESCAMERA_G3D( selected->Id, selected->Index, resCamera );
            if( resCamera.GetViewType() == nw::gfx::res::ResCamera::VIEWTYPE_LOOKAT )
            {
                GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );

                nw::gfx::res::ResLookAtTargetViewUpdater resLookAtTargetView = nw::ut::ResDynamicCast<nw::gfx::res::ResLookAtTargetViewUpdater>(resCamera.GetViewUpdater());

                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_TargetPosition ){
                    resLookAtTargetView.SetTargetPosition( camera->TargetPosition.X, camera->TargetPosition.Y, camera->TargetPosition.Z );
                    // Determines if camera position is the same.
                    ErrorCheck::CheckCameraPosition( objCamera );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_UpwardVector ){
                    resLookAtTargetView.SetUpwardVector( camera->UpwardVector.X, camera->UpwardVector.Y, camera->UpwardVector.Z );
                    // Checks up vector
                    ErrorCheck::CheckCameraUpVectorZero( objCamera );
                    ErrorCheck::CheckCameraUpVectorParallel( objCamera );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_IsInheritingTargetRotate ){
                    SET_FLAGS(
                        resLookAtTargetView,
                        camera->IsInheritingTargetRotate,
                        nw::gfx::ResLookAtTargetViewUpdaterData::FLAG_INHERITING_TARGET_ROTATE
                    )
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_IsInheritingTargetTranslate ){
                    SET_FLAGS(
                        resLookAtTargetView,
                        camera->IsInheritingTargetTranslate,
                        nw::gfx::ResLookAtTargetViewUpdaterData::FLAG_INHERITING_TARGET_TRANSLATE
                    )
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_IsInheritingUpRotate ){
                    SET_FLAGS(
                        resLookAtTargetView,
                        camera->IsInheritingUpRotate,
                        nw::gfx::ResLookAtTargetViewUpdaterData::FLAG_INHERITING_UP_ROTATE
                    )
                }

                // Edits OriginalValue with the transferred value.
                u8* ptr = GetOriginalValue( objCamera->GetAnimGroup(), nw::anim::res::ResAnimGroupMember::OBJECT_TYPE_VIEW_UPDATER);
                if( !ptr ) continue;
                nw::gfx::res::ResLookAtTargetViewUpdater resOrigLookAtTargetView = nw::gfx::res::ResLookAtTargetViewUpdater( ptr );
                if( !resOrigLookAtTargetView.IsValid() ) continue;
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_TargetPosition )
                {
                    resOrigLookAtTargetView.SetTargetPosition( objCamera->GetTargetPosition().x, objCamera->GetTargetPosition().y, objCamera->GetTargetPosition().z );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_UpwardVector )
                {
                    resOrigLookAtTargetView.SetUpwardVector( objCamera->GetUpwardVector().x, objCamera->GetUpwardVector().y, objCamera->GetUpwardVector().z );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_IsInheritingTargetRotate )
                {
                    SET_FLAGS(resOrigLookAtTargetView, camera->IsInheritingTargetRotate, nw::gfx::ResLookAtTargetViewUpdaterData::FLAG_INHERITING_TARGET_ROTATE);
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_IsInheritingTargetTranslate )
                {
                    SET_FLAGS(resOrigLookAtTargetView, camera->IsInheritingTargetTranslate, nw::gfx::ResLookAtTargetViewUpdaterData::FLAG_INHERITING_TARGET_TRANSLATE);
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewLookAt::PropertyFlag_IsInheritingUpRotate )
                {
                    SET_FLAGS(resOrigLookAtTargetView, camera->IsInheritingUpRotate, nw::gfx::ResLookAtTargetViewUpdaterData::FLAG_INHERITING_UP_ROTATE);
                }
            }
        }
    }

    // Camera view (Rotate) settings.
    void
    SetTargetViewRotate(Protocol4C::MESSAGEPACKET_Camera_SetTargetViewRotate* camera)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESCAMERA_G3D( selected->Id, selected->Index, resCamera );
            if( resCamera.GetViewType() == nw::gfx::res::ResCamera::VIEWTYPE_ROTATE )
            {
                nw::gfx::res::ResRotateViewUpdater resRotateView = nw::ut::ResDynamicCast<nw::gfx::res::ResRotateViewUpdater>(resCamera.GetViewUpdater());

                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewRotate::PropertyFlag_ViewRotate ){
                    resRotateView.SetViewRotate( camera->ViewRotate.X, camera->ViewRotate.Y, camera->ViewRotate.Z );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewRotate::PropertyFlag_IsInheritingRotate ){
                    SET_FLAGS(
                        resRotateView,
                        camera->IsInheritingRotate,
                        nw::gfx::res::ResRotateViewUpdaterData::FLAG_INHERITING_ROTATE
                    )
                }

                GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );
                // Edits OriginalValue with the transferred value.
                u8* ptr = GetOriginalValue( objCamera->GetAnimGroup(), nw::anim::res::ResAnimGroupMember::OBJECT_TYPE_VIEW_UPDATER);
                if( !ptr ) continue;
                nw::gfx::res::ResRotateViewUpdater resOrigRotateView = nw::gfx::res::ResRotateViewUpdater( ptr );
                if( !resOrigRotateView.IsValid() ) continue;
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewRotate::PropertyFlag_ViewRotate )
                {
                    resOrigRotateView.SetViewRotate( camera->ViewRotate.X, camera->ViewRotate.Y, camera->ViewRotate.Z );
                }
                if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetTargetViewRotate::PropertyFlag_IsInheritingRotate )
                {
                    SET_FLAGS(resOrigRotateView, camera->IsInheritingRotate, nw::gfx::res::ResRotateViewUpdaterData::FLAG_INHERITING_ROTATE);
                }
            }
        }
    }

    // Sets the projection matrix using an orthogonal projection.
    void
    SetOrthoProjection(Protocol4C::MESSAGEPACKET_Camera_SetOrthoProjection* camera)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESCAMERA_G3D( selected->Id, selected->Index, resCamera );
            if( resCamera.GetProjectionType() != nw::gfx::res::ResCamera::PROJTYPE_ORTHO )
                continue;

            nw::gfx::res::ResOrthoProjectionUpdater resOPUpdater =  nw::ut::ResDynamicCast<nw::gfx::res::ResOrthoProjectionUpdater>(resCamera.GetProjectionUpdater());
            const f32 width = camera->Height * camera->AspectRatio;
            const f32 rect_l = camera->Center.X - width / 2.f;
            const f32 rect_r = camera->Center.X + width / 2.f;
            const f32 rect_b = camera->Center.Y + camera->Height / 2.f;
            const f32 rect_t = camera->Center.Y - camera->Height / 2.f;
            resOPUpdater.SetRectWithYFlip(rect_l, rect_r, rect_b, rect_t);

            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetOrthoProjection::PropertyFlag_Near )
            {
                resOPUpdater.SetNear( camera->Near );
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetOrthoProjection::PropertyFlag_Far )
            {
                resOPUpdater.SetFar( camera->Far );
            }

            GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );

            // Prevents Near from becoming 0.
            ErrorCheck::CheckCameraNear( objCamera );
            // Prevents Near and Far from becoming invalid.
            ErrorCheck::CheckCameraNearFar( objCamera );

            // Updates WScale.
            {
                f32 wScale = 1.0f / resOPUpdater.GetFar();
                resCamera.SetWScale(
                    resCamera.GetWScale() != 0.0f ? wScale : 0.0f
                );
                objCamera->SetWScale(
                    ( objCamera->GetWScale() != 0.0f ||
                      m_ScnMgr->GetForceUseCameraWScale() ) ?
                    wScale : 0.0f
                );
            }


            // Edits OriginalValue with the transferred value.
            u8* ptr = GetOriginalValue( objCamera->GetAnimGroup(), nw::anim::res::ResAnimGroupMember::OBJECT_TYPE_PROJECTION_UPDATER);
            if( !ptr )
                continue;

            nw::gfx::res::ResOrthoProjectionUpdater resOrigOPpdater = nw::gfx::res::ResOrthoProjectionUpdater( ptr );
            if( !resOrigOPpdater.IsValid() )
                continue;

            resOrigOPpdater.SetRectWithYFlip(rect_l, rect_r, rect_b, rect_t);

            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetOrthoProjection::PropertyFlag_Near )
            {
            //    resOrigOPpdater.SetNear( camera->Near );
                resOrigOPpdater.SetNear( objCamera->GetNear() );    // Makes use of camera values tentatively checked.
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetOrthoProjection::PropertyFlag_Far )
            {
            //    resOrigOPpdater.SetFar( camera->Far );
                resOrigOPpdater.SetFar( objCamera->GetFar() );        // Makes use of camera values tentatively checked.
            }

        }
    }

    // Sets the orthogonal projections using viewing angles.
    void
    SetPerspectiveProjection(Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection* camera)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESCAMERA_G3D( selected->Id, selected->Index, resCamera );
            if( resCamera.GetProjectionType() != nw::gfx::res::ResCamera::PROJTYPE_PERSPECTIVE )
                continue;

            nw::gfx::res::ResPerspectiveProjectionUpdater resPPUpdater =  nw::ut::ResDynamicCast<nw::gfx::res::ResPerspectiveProjectionUpdater>(resCamera.GetProjectionUpdater());

            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_Near )
            {
                resPPUpdater.SetNear( camera->Near );
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_Far )
            {
                resPPUpdater.SetFar( camera->Far );
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_Fovy )
            {
                resPPUpdater.SetFovy( camera->Fovy );
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_AspectRatio )
            {
                resPPUpdater.SetAspectRatio( camera->AspectRatio );
            }

            GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );

            // Prevents Near from becoming 0.
            ErrorCheck::CheckCameraNear( objCamera );
            // Prevents Near and Far from becoming invalid.
            ErrorCheck::CheckCameraNearFar( objCamera );

            // Updates WScale.
            {
                f32 wScale = 1.0f / resPPUpdater.GetFar();
                resCamera.SetWScale(
                    resCamera.GetWScale() != 0.0f ? wScale : 0.0f
                );
                objCamera->SetWScale(
                    ( objCamera->GetWScale() != 0.0f ||
                      m_ScnMgr->GetForceUseCameraWScale() ) ?
                    wScale : 0.0f
                );
            }

            // Edits OriginalValue with the transferred value.
            u8* ptr = GetOriginalValue( objCamera->GetAnimGroup(), nw::anim::res::ResAnimGroupMember::OBJECT_TYPE_PROJECTION_UPDATER);
            if( !ptr )
                continue;

            nw::gfx::res::ResPerspectiveProjectionUpdater resOrigPPUpdater = nw::gfx::res::ResPerspectiveProjectionUpdater( ptr );
            if( !resOrigPPUpdater.IsValid() )
                continue;

            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_Near )
            {
            //    resOrigPPUpdater.SetNear( camera->Near );
                resOrigPPUpdater.SetNear( objCamera->GetNear() );    // Makes use of camera values tentatively checked.
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_Far )
            {
            //    resOrigPPUpdater.SetFar( camera->Far );
                resOrigPPUpdater.SetFar( objCamera->GetFar() );        // Makes use of camera values tentatively checked.
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_Fovy )
            {
                resOrigPPUpdater.SetFovy( camera->Fovy );
            }
            if( camera->ModifiedProperties & Protocol4C::MESSAGEPACKET_Camera_SetPerspectiveProjection::PropertyFlag_AspectRatio )
            {
                resOrigPPUpdater.SetAspectRatio( camera->AspectRatio );
            }
        }
    }

    // Sets the camera transform.
    void
    SetCameraTransform(Protocol4C::MESSAGEPACKET_Camera_SetCameraTransform* camera )
    {
        FOREACH_SELECTED_G3D( selected )
        {

            GET_RESCAMERA_G3D( selected->Id, selected->Index, resCamera );
            resCamera.SetTransform(nn::math::Transform3(
                nn::math::VEC3( camera->Scale.X, camera->Scale.Y, camera->Scale.Z ),
                nn::math::VEC3( camera->Rotate.X, camera->Rotate.Y, camera->Rotate.Z ),
                nn::math::VEC3( camera->Translate.X, camera->Translate.Y, camera->Translate.Z )));

            GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );

            nw::gfx::CalculatedTransform& transform = objCamera->Transform();

            transform.SetTranslate( camera->Translate.X, camera->Translate.Y, camera->Translate.Z );
            transform.SetScale( camera->Scale.X, camera->Scale.Y, camera->Scale.Z );
            transform.SetRotateXYZ( camera->Rotate.X, camera->Rotate.Y, camera->Rotate.Z );

            // Determines if camera position is the same.
            ErrorCheck::CheckCameraPosition( objCamera );

            // Edits OriginalValue with the transferred value.
            if(u8* ptr = GetOriginalValue( objCamera->GetAnimGroup(), nw::anim::res::ResAnimGroupMember::OBJECT_TYPE_TRANSFORM) )
            {
                nw::gfx::res::ResCamera resOrigCamera = nw::gfx::res::ResCamera( ptr );
                if( resOrigCamera.IsValid() )
                {
                    resOrigCamera.GetTransform().translate.x = objCamera->Transform().GetTranslate().x;
                    resOrigCamera.GetTransform().translate.y = objCamera->Transform().GetTranslate().y;
                    resOrigCamera.GetTransform().translate.z = objCamera->Transform().GetTranslate().z;
                }
            }
        }
    }

    // Sets the camera's W buffer.
    void
    SetCameraIsWBuffer(Protocol4C::MESSAGEPACKET_Camera_SetCameraIsWBuffer* camera)
    {
        FOREACH_SELECTED_G3D( selected )
        {
            GET_RESCAMERA_G3D( selected->Id, selected->Index, resCamera );
            GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );
            f32 wScale = 0.0f;
            if( camera->IsWBuffer || m_ScnMgr->GetForceUseCameraWScale() ){
                f32 nFar = 1.0f;
                if( resCamera.GetProjectionType() == nw::gfx::res::ResCamera::PROJTYPE_PERSPECTIVE ){
                    nw::gfx::res::ResPerspectiveProjectionUpdater resPPUpdater =  nw::ut::ResDynamicCast<nw::gfx::res::ResPerspectiveProjectionUpdater>(resCamera.GetProjectionUpdater());
                    nFar = resPPUpdater.GetFar();
                }else if( resCamera.GetProjectionType() == nw::gfx::res::ResCamera::PROJTYPE_ORTHO ){
                    nw::gfx::res::ResOrthoProjectionUpdater resOPUpdater =  nw::ut::ResDynamicCast<nw::gfx::res::ResOrthoProjectionUpdater>(resCamera.GetProjectionUpdater());
                    nFar = resOPUpdater.GetFar();
                }
                wScale = 1.0f / nFar;
            }
            resCamera.SetWScale( camera->IsWBuffer ? wScale : 0.0f );
            objCamera->SetWScale( camera->IsWBuffer || m_ScnMgr->GetForceUseCameraWScale() ? wScale : 0.0f );
        }
    }

    // Gets camera information.
    void
    GetCameraInfo(Protocol4C::MESSAGEPACKET_Camera_GetCameraInfo* camera)
    {
        GET_RESCAMERA_G3D( camera->CameraID, camera->CameraIndex, resCamera );
        GET_CAMERA_G3D( camera->CameraID, camera->CameraIndex, objCamera );

        // Copying the location.
        nw::gfx::CalculatedTransform& transform = objCamera->Transform();
        transform.GetTranslate( ( nw::math::VEC3*)( &camera->Position ) );

        switch ( resCamera.GetViewType() )
        {
        case nw::gfx::res::ResCamera::VIEWTYPE_AIM:
            {
                nw::gfx::res::ResAimTargetViewUpdater resAimTargetView =
                    nw::ut::ResDynamicCast<nw::gfx::res::ResAimTargetViewUpdater>(resCamera.GetViewUpdater());

                // Sets camera type.
                camera->CameraType = Protocol4C::ViewMode_AimTarget;

                // Copying the gaze point.
                memcpy(
                    &( camera->TargetPosition ),
                    &( resAimTargetView.GetTargetPosition() ),
                    sizeof( nw::math::VEC3 )
                );

                // Copying the twist value.
                camera->Twist = resAimTargetView.GetTwist();
            }
            break;
        case nw::gfx::res::ResCamera::VIEWTYPE_LOOKAT:
            {
                nw::gfx::res::ResLookAtTargetViewUpdater resLookAtTargetView =
                    nw::ut::ResDynamicCast<nw::gfx::res::ResLookAtTargetViewUpdater>(resCamera.GetViewUpdater());

                // Sets camera type.
                camera->CameraType = Protocol4C::ViewMode_LookAtTarget;

                // Copying the gaze point.
                memcpy(
                    &( camera->TargetPosition ),
                    &( resLookAtTargetView.GetTargetPosition() ),
                    sizeof( nw::math::VEC3 )
                );

                // Copying UpVector.
                memcpy(
                    &( camera->UpwardVector ),
                    &( resLookAtTargetView.GetUpwardVector() ),
                    sizeof( nw::math::VEC3 )
                );
            }
            break;
        case nw::gfx::res::ResCamera::VIEWTYPE_ROTATE:
            {
                nw::gfx::res::ResRotateViewUpdater resRotateView =
                    nw::ut::ResDynamicCast<nw::gfx::res::ResRotateViewUpdater>(resCamera.GetViewUpdater());

                // Sets camera type.
                camera->CameraType = Protocol4C::ViewMode_Rotate;

                // Copying the rotation value.
                memcpy(
                    &( camera->ViewRotate ),
                    &( resRotateView.GetViewRotate() ),
                    sizeof( nw::math::VEC3 )
                );
            }
            break;
        }
    }

    // Making the active scene environment setting.
    void
    SetSceneActive(Protocol4C::MESSAGEPACKET_Scene_SetSceneActive* scene)
    {
        SceneManager::SceneEnvironmentSettingList::iterator it =
            m_ScnMgr->FindSceneEnvironmentSetting( scene->SceneId, scene->Index );
        if ( it != m_ScnMgr->GetSceneEnvironmentSettingsEnd() ) {
            m_ScnSlr->SetTargetSceneEnvironmentSetting( &(*it) );
        }
    }

    // User data setting function
    void
    SetUserData(Protocol4C::MESSAGEPACKET_UserData_SendUserData* data)
    {
        // Check arguments
        if ( data->ObjectCount == 0 ) return;

        // Executing data load
        ResourceManager::GetInstance()->LoadBinaryData(
            ResourceSet::Type_UserData,
            data->Id,
            ModuleG3dImpl::GetInstance()->Get3DEditorActiveChannel(),
            data->FilePath.ToString(),
            true
        );

        // Gets data
        ResourceManager::ResourceList::iterator it = ResourceManager::GetInstance()->Find(
            ResourceSet::Type_UserData,
            data->Id
        );

        // Error check
        if ( it == ResourceManager::GetInstance()->GetResourcesEnd() ) return;

        // Getting ResGraphicsFile.
        nw::gfx::res::ResGraphicsFile file = it->GetResource();

        // Error check
        if ( file.GetLutSetsCount() == 0 ) return;

        // Getting the object that is the replacement source.
        nw::gfx::res::ResLookupTableSet set = file.GetLutSets( 0 );

        // Error check
        if ( !set.IsValid() ) return;

        nw::ut::MoveArray<UserDataManager::LinkData> objects( Memory_GetAllocator() );
        for ( int i = 0; i < data->ObjectCount; ++i )
        {
            Protocol4C::EditUserDataObject* obj = (Protocol4C::EditUserDataObject*)(
                (unsigned char*)data +
                Protocol4C::MESSAGEPACKET_UserData_SendUserData::StructSize +
                i * Protocol4C::EditUserDataObject::StructSize
            );

            #define USER_DATA_MACRO( type, resType ) { \
                SceneManager::type##List::iterator iter = m_ScnMgr->Find##type( obj->Id, obj->Index ); \
                if ( iter != m_ScnMgr->Get##type##sEnd() ) \
                { \
                    UserDataManager::LinkData linkData; \
                    linkData.m_target = iter->GetResObject().ptr(); \
                    linkData.m_targetType = resType; \
                    linkData.m_resourceSet = iter->GetResSource(); \
                    objects.push_back( linkData ); \
                } }

            switch( obj->ObjectType )
            {
            case Protocol4C::UserDataTargetType_Model:
                USER_DATA_MACRO( Model, TARGET_SCENE_OBJECT )
                break;
            case Protocol4C::UserDataTargetType_Material:
                {
                    SceneManager::ModelList::iterator iter = m_ScnMgr->FindModel( obj->Id, 0 );
                    if ( iter != m_ScnMgr->GetModelsEnd() )
                    {
                        nw::gfx::Material* material = iter->GetSceneNode()->GetMaterial( obj->Index );

                        UserDataManager::LinkData linkData;
                        linkData.m_targetType = TARGET_SCENE_OBJECT;
                        linkData.m_resourceSet = iter->GetResSource();

                        linkData.m_target = material->GetOriginal().ptr();
                        objects.push_back( linkData );

                        linkData.m_target = material->GetBuffer().ptr();
                        objects.push_back( linkData );
                    }
                }
                break;
            case Protocol4C::UserDataTargetType_Mesh:
                {
                    SceneManager::ModelList::iterator iter = m_ScnMgr->FindModel( obj->Id, 0 );
                    if ( iter != m_ScnMgr->GetModelsEnd() )
                    {
                        nw::gfx::ResMeshArray meshes = iter->GetSceneNode()->GetResMeshes();
                        if ( meshes.size() > obj->Index )
                        {
                            UserDataManager::LinkData linkData;
                            linkData.m_target = meshes[ obj->Index ].ptr();
                            linkData.m_targetType = TARGET_SCENE_OBJECT;
                            linkData.m_resourceSet = iter->GetResSource();
                            objects.push_back( linkData );
                        }
                    }
                }
                break;
            case Protocol4C::UserDataTargetType_Bone:
                {
                    SceneManager::ModelList::iterator iter = m_ScnMgr->FindModel( obj->Id, 0 );
                    if ( iter != m_ScnMgr->GetModelsEnd() )
                    {
                        nw::gfx::ResSkeletalModel skeletalModel =
                            nw::ut::ResDynamicCast<nw::gfx::ResSkeletalModel>(iter->GetSceneNode()->GetResSceneObject());
                        if ( skeletalModel.IsValid() )
                        {
                            nw::gfx::ResSkeleton skeleton = skeletalModel.GetSkeleton();
                            if ( skeleton.GetBonesCount() > obj->Index )
                            {
                                UserDataManager::LinkData linkData;
                                linkData.m_target = skeleton.GetBones( obj->Index ).ptr();
                                linkData.m_targetType = TARGET_BONE;
                                linkData.m_resourceSet = iter->GetResSource();
                                objects.push_back( linkData );
                            }
                        }
                    }
                }
                break;
            case Protocol4C::UserDataTargetType_Texture:
                USER_DATA_MACRO( Texture, TARGET_SCENE_OBJECT )
                break;
            case Protocol4C::UserDataTargetType_Camera:
                USER_DATA_MACRO( Camera, TARGET_SCENE_OBJECT )
                break;
            case Protocol4C::UserDataTargetType_Fog:
                USER_DATA_MACRO( Fog, TARGET_SCENE_OBJECT )
                break;
            case Protocol4C::UserDataTargetType_Light:
                USER_DATA_MACRO( Light, TARGET_SCENE_OBJECT )
                break;
            case Protocol4C::UserDataTargetType_Scene:
                USER_DATA_MACRO( SceneEnvironmentSetting, TARGET_SCENE_OBJECT )
                break;
            case Protocol4C::UserDataTargetType_LUTS:
                USER_DATA_MACRO( LookupTableSet, TARGET_SCENE_OBJECT )
                break;
            case Protocol4C::UserDataTargetType_SkeletalAnim:
            case Protocol4C::UserDataTargetType_MaterialAnim:
            case Protocol4C::UserDataTargetType_ModelAnim:
            case Protocol4C::UserDataTargetType_CameraAnim:
            case Protocol4C::UserDataTargetType_FogAnim:
            case Protocol4C::UserDataTargetType_LightAnim:
                USER_DATA_MACRO( Anim, TARGET_ANIMATION )
                break;
            }
        }

        UserDataManager::GetInstance()->Link(
            objects,
            set,
            data->Id
        );
    }

    void
    CallModelCallBack( unsigned int nID )
    {
        // Gets the callback class.
        nw::viewer::g3d::IResourceListener* g3dListener =
            nw::viewer::sys::App::GetInstance().GetModule3D().GetResourceListener();

        // Gets the model.
        GET_MODEL_G3D( nID, 0, pModel );

        if (pModel && g3dListener) {
            g3dListener->OnUpdateModel( pModel );
        }
    }

    void
    CallMaterialCallBack()
    {
        // Gets the callback class.
        nw::viewer::g3d::IResourceListener* g3dListener =
            nw::viewer::sys::App::GetInstance().GetModule3D().GetResourceListener();

        if (g3dListener)
        {
            FOREACH_SELECTED_G3D( selected )
            {
                GET_MODEL_G3D( selected->Id, 0, objModel );
                if (objModel) {
                    nw::gfx::Material* objMaterial = objModel->GetMaterial( selected->Index );
                    if (objMaterial) {
                        g3dListener->OnUpdateMaterial( objModel, objMaterial );
                    }
                }
            }
        }
    }

    void
    CallCameraCallBack()
    {
        // Gets the callback class.
        nw::viewer::g3d::IResourceListener* g3dListener =
            nw::viewer::sys::App::GetInstance().GetModule3D().GetResourceListener();

        if (g3dListener)
        {
            FOREACH_SELECTED_G3D( selected )
            {
                GET_CAMERA_G3D( selected->Id, selected->Index, objCamera );
                if (objCamera) {
                    g3dListener->OnUpdateCamera( objCamera );
                }
            }
        }
    }

    void
    CallLightCallBack()
    {
        // Gets the callback class.
        nw::viewer::g3d::IResourceListener* g3dListener =
            nw::viewer::sys::App::GetInstance().GetModule3D().GetResourceListener();

        if (g3dListener)
        {
            FOREACH_SELECTED_G3D( selected )
            {
                GET_LIGHT_G3D( selected->Id, selected->Index, objLight );
                if (objLight) {
                    g3dListener->OnUpdateLight( objLight );
                }
            }
        }
    }

    void
    CallFogCallBack()
    {
        // Gets the callback class.
        nw::viewer::g3d::IResourceListener* g3dListener =
            nw::viewer::sys::App::GetInstance().GetModule3D().GetResourceListener();

        if (g3dListener)
        {
            FOREACH_SELECTED_G3D( selected )
            {
                GET_FOG_G3D( selected->Id, selected->Index, objFog );
                if (objFog) {
                    g3dListener->OnUpdateFog( objFog );
                }
            }
        }
    }

    void
    EnableDrawing()
    {
        ModuleG3dImpl::GetInstance()->SetCanDraw( true );
    }

    void
    DisableDrawing()
    {
        ModuleG3dImpl::GetInstance()->SetCanDraw( false );
    }

private:
    ResourceManager *m_ResMgr;
    SceneManager *m_ScnMgr;
    SceneElemSelector *m_ScnSlr;
    SceneRenderer *m_ScnRdr;
    Protocol4C::EditTargetType m_SelectedType;
    nw::ut::MoveArray<Protocol4C::EditSelectedObject> m_SelectedObject;
    static ParamSetter* m_pInstance;

    ParamSetter() : m_SelectedObject( Memory_GetAllocator() )
    {
        m_ResMgr = ResourceManager::GetInstance();
        m_ScnMgr = SceneManager::GetInstance();
        m_ScnSlr = SceneElemSelector::GetInstance();
        m_ScnRdr = SceneRenderer::GetInstance();
    }

    ~ParamSetter() {}

    void SelectColorElement(nw::ut::FloatColor& dst, const nw::ut::FloatColor& src , unsigned int modifiedPropertyElements)
    {
        if((modifiedPropertyElements & (1<<0)) != 0){   dst.r = src.r;  }
        if((modifiedPropertyElements & (1<<1)) != 0){   dst.g = src.g;  }
        if((modifiedPropertyElements & (1<<2)) != 0){   dst.b = src.b;  }
        if((modifiedPropertyElements & (1<<3)) != 0){   dst.a = src.a;  }
    }

    nw::gfx::res::ResLookupTable GetResLookupTable( int nID, const char *szSetName, const char *szTableName )
    {
        nw::gfx::res::ResGraphicsFile resGraphicsFile = m_ScnMgr->FindModel(nID)->GetResSource()->GetResource();
        nw::gfx::res::ResLookupTableSet lookupTableSet = resGraphicsFile.GetLutSets( szSetName );
        int nIndex = lookupTableSet.GetSamplersIndex( szTableName );
        return lookupTableSet.GetSamplers(nIndex);
    }



    ///Searches the content of @brief animGroup and gets the resource object pointer holding the parameter's original value from a member that is not GetObjectType() OBJECT_TYPE_TRANSFORM.
    // 
    ///return fail: null;  success: resource object pointer
    u8* GetOriginalValue( nw::gfx::AnimGroup* animGroup )
    {
        if ( animGroup == NULL ) { return NULL; }
        if ( !animGroup->HasOriginalValue() ) { return NULL; }

        int animMemberCount = animGroup->GetMemberCount();
        for( int idx = 0; idx < animMemberCount; ++idx )
        {
            nw::anim::res::ResAnimGroupMember member = animGroup->GetResAnimGroupMember( idx );

            if ( member.GetObjectType() != nw::anim::res::ResAnimGroupMember::OBJECT_TYPE_TRANSFORM )
            {
                if ( !animGroup->HasOriginalValue() ) { continue; }

                u8* origValue = reinterpret_cast< u8* >( const_cast< void* >( animGroup->GetOriginalValue( idx ) ) );
                u32 offset = member.GetMemberOffset();

                u8* ptr = reinterpret_cast< u8* >( origValue - offset );
                if ( ptr != NULL ) { return ptr; }
            }
        }
        return NULL;
    }


    ///Outputs all the member information of @brief animGroup to the debug log.
    ///@param[in]  animGroup
    void DebugAnimGroupContents(nw::gfx::AnimGroup* animGroup)
    {
        static char s_objectTypes[18][256] =
        {
            "OBJECT_TYPE_BONE",
            "OBJECT_TYPE_MATERIAL_COLOR",
            "OBJECT_TYPE_TEXTURE_SAMPLER",
            "OBJECT_TYPE_TEXTURE_MAPPER",
            "OBJECT_TYPE_BLEND_OPERATION",

            "OBJECT_TYPE_TEXTURE_COORDINATOR",
            "OBJECT_TYPE_MODEL",
            "OBJECT_TYPE_MESH",
            "OBJECT_TYPE_MESH_NODE_VISIBILITY",
            "OBJECT_TYPE_TRANSFORM",

            "OBJECT_TYPE_VIEW_UPDATER",
            "OBJECT_TYPE_PROJECTION_UPDATER",
            "OBJECT_TYPE_LIGHT",
            "OBJECT_TYPE_FRAGMENT_LIGHT",
            "OBJECT_TYPE_VERTEX_LIGHT",

            "OBJECT_TYPE_HEMISPHERE_LIGHT",
            "OBJECT_TYPE_AMBIENT_LIGHT",
            "OBJECT_TYPE_FOG"
        };

        // TODO
        static char s_memberTypes[18][6][256] =
        {
            {"MEMBER_TYPE_TRANSFORM"},    // ResBoneMember
            {"TODO!!!"},    // ResMaterialColorMember
            {"TODO!!!"},    // ResTextureSamplerMember
            {"TODO!!!"},    // ResTextureMapperMember
            {"TODO!!!"},    // ResBlendOperationMember

            {"MEMBER_TYPE_SCALE", "MEMBER_TYPE_ROTATE", "MEMBER_TYPE_TRANSLATE"},    // ResTextureCoordinatorMember
            {"MEMBER_TYPE_BRANCH_VISIBLE","MEMBER_TYPE_VISIBLE"},    // ResModelMember
            {"MEMBER_TYPE_VISIBLE",},        // ResMeshMember
            {"MEMBER_TYPE_VISIBLE",},        // ResMeshNodeVisibilityMember
            {"MEMBER_TYPE_TRANSFORM",},        // ResTransformMember

            {"MEMBER_TYPE_TARGET_POSITION",    "MEMBER_TYPE_TWIST",    "MEMBER_TYPE_UPWARD_VECTOR",    "MEMBER_TYPE_VIEW_ROTATE",    "MEMBER_TYPE_TRANSFORM"},                        // ResViewUpdaterMember
            {"MEMBER_TYPE_NEAR",            "MEMBER_TYPE_FAR",        "MEMBER_TYPE_FOVY",                "MEMBER_TYPE_ASPECT_RATIO",    "MEMBER_TYPE_HEIGHT",    "MEMBER_TYPE_CENTER"},    // ResProjectionUpdaterMember
            {"TODO!!!"},
            {"TODO!!!"},
            {"TODO!!!"},

            {"TODO!!!"},
            {"TODO!!!"},
            {"TODO!!!"},
        };

        NW_VIEWER_LOG("animGroup->GetName()==%s\n", animGroup->GetName() );

        int animMemberCount = animGroup->GetMemberCount();
        NW_VIEWER_LOG("animGroup->GetMemberCount()==%i\n", animMemberCount );

        for( int idx = 0; idx < animMemberCount; ++idx )
        {
            NW_VIEWER_LOG("\nanim member#%i\n", idx );

            nw::anim::res::ResAnimGroupMember member = animGroup->GetResAnimGroupMember( idx );

            NW_VIEWER_LOG("member.GetClassName()== %s\n", member.GetClassName() );
            NW_VIEWER_LOG("member.GetObjectType()== %i, %s \n", member.GetObjectType(), s_objectTypes[member.GetObjectType()]);
            NW_VIEWER_LOG("member.GetMemberType()== %i, %s \n", member.GetMemberType(), s_memberTypes[member.GetObjectType()][member.GetMemberType()] );
            NW_VIEWER_LOG("member.GetPath()== %s\n", member.GetPath() );
            NW_VIEWER_LOG("member.GetMemberOffset()== %i\n", member.GetMemberOffset() );
        }
    }





    ///Searches the content of @brief animGroup and gets the resource object pointer holding the parameter's original value from a member for which objectType is equal to GetObjectType.
    // 
    ///return fail: null;  success: resource object pointer
    u8* GetOriginalValue( nw::gfx::AnimGroup* animGroup, int objectType)
    {
        if ( animGroup == NULL )
        {
            NW_VIEWER_LOG("GetOriginalValue error : animGroup is NULL !!! \n" );
            return NULL;
        }

        if ( !animGroup->HasOriginalValue() )
        {
            NW_VIEWER_LOG("GetOriginalValue error : animGroup->HasOriginalValue() false !!! \n" );
            return NULL;
        }

        int animMemberCount = animGroup->GetMemberCount();

        for( int idx = 0; idx < animMemberCount; ++idx )
        {
            nw::anim::res::ResAnimGroupMember member = animGroup->GetResAnimGroupMember( idx );

            if( member.GetObjectType() == objectType )
            {
                const void* memberOriginalValue = animGroup->GetOriginalValue( idx );
                u8* origValue = reinterpret_cast< u8* >( const_cast< void* >( memberOriginalValue ) );
                u32 offset = member.GetMemberOffset();
                u8* ptr = reinterpret_cast< u8* >( origValue - offset );
                if ( ptr != NULL ) { return ptr; }
            }
        }
        return NULL;
    }

};

}
}
#endif
